<?php
/**
* 2014 KerAwen
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2014 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/

require_once (dirname(__FILE__).'/../defines.php');
require_once (_KERAWEN_DIR_.'/kerawen.php');
require_once (_KERAWEN_API_DIR_.'/request.php');
require_once (_KERAWEN_API_DIR_.'/handler.php');
require_once (_KERAWEN_TOOLS_DIR_.'/utils.php');
require_once (_KERAWEN_CLASS_DIR_.'/data.php');
require_once (_KERAWEN_CLASS_DIR_.'/context.php');
require_once (_KERAWEN_CLASS_DIR_.'/permissions.php');
require_once (_KERAWEN_CLASS_DIR_.'/cart.php');

$context = Context::getContext();
$request = new Request(json_decode(Tools::file_get_contents('php://input')), $context);
$kerawen = Module::getInstanceByName('kerawen');

$error = false;

if (!$error) {
	require_once(_KERAWEN_DIR_.'/install/audit.php');
	$audit = audit(null, true);
	$valid = true;
	foreach ($audit['warnings'] as $warn) $valid = $valid && $warn['valid'];
	if (!$valid) {
		$request->setError(+1, $kerawen->l('PrestaShop configuration is incorrect', pathinfo(__FILE__, PATHINFO_FILENAME)));
		$error = true;
	}
}

if (!$error && !$context->employee->isLoggedBack()) {
	$request->setError(+2, $kerawen->l('Operator is not logged in', pathinfo(__FILE__, PATHINFO_FILENAME)));
	$error = true;
}

if (!$error) {
	if (!isset($request->version) || $request->version != $kerawen->version) {
		$request->setError(+3, $kerawen->l('Module version has changed', pathinfo(__FILE__, PATHINFO_FILENAME)));
		$error = true;
	}
}

if (!$error) {
	if (isset($request->config) && $request->config != Configuration::get('KERAWEN_CONFIG')) {
		$request->setError(+4, $kerawen->l('Module configuration has changed', pathinfo(__FILE__, PATHINFO_FILENAME)));
		$error = true;
	}
}

if (!$error) {
	/* Override some configuration parameters during the request only */
	Configuration::set('PS_CATALOG_MODE', 0);

	/* Update context */
	$context->module = $kerawen;
	
    $cookie = $context->cookie;
	if ($cookie->id_employee_kerawen)
		$context->employee = new Employee($cookie->id_employee_kerawen);
	// Allow anything to application
	$context->employee->id_profile = _PS_ADMIN_PROFILE_;
	
	// Reload all shops
	Shop::cacheShops(true);
	if (isset($request->params->id_shop) && $request->params->id_shop) {
		$context->shop = new Shop($request->params->id_shop);
		Shop::setContext(Shop::CONTEXT_SHOP, $request->params->id_shop);
	}

	$context->cart = getCart(
	    isset($request->params->id_cart) ? $request->params->id_cart : null,
	    isset($request->params->id_cust) ? $request->params->id_cust : null,
	    $context, false);
	
	//TODO: Simplified Mode or Advanced Mode
	$permissions = new kerawenPermissions();
	$context->permissions = $permissions->getKerawenPermissionsByEmployee(($context->employee->id) ? $context->employee->id : 0);
	
	setCustomerContext(isset($request->params->id_cust) ? $request->params->id_cust : false);
	
	/* Extended context, TO BE COMPLETED */
	setExtendedContext('kerawen', true);
	setExtendedContext('id_shop', $context->shop ? $context->shop->id : false);
	if (isset($request->params->id_cashdrawer))
		setExtendedContext('id_cashdrawer', $request->params->id_cashdrawer);

	$method = $request->method;
	$handler = new Handler();
	try {
		$result = $handler->$method($request);
		/* TEMP backward compatibility */
		if ($result !== null)
			$request->result = $result;
		
		// Profiling
		if (_PS_DEBUG_PROFILING_) {
			require_once (_KERAWEN_TOOLS_DIR_.'/profiling.php');
			$request->result['profiling'] = getProfiling();
		}
	}
	catch (Exception $e) {
		unset($request->context);
		unset($request->result);
		echo '<pre>';
		echo $e;
		echo '<hr>';
		print_r($request);
		echo '</pre>';
		exit();
	}
}

/* Send response */
$request->finalize();
header('Content-Type: application/json');
echo json_encode($request);
exit();
