<?php
/**
* 2016 KerAwen
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2016 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/

/*
Source :
http://blog.belvg.com/pdf-in-prestashop.html
*/

//require to build pdf
require_once (_KERAWEN_CLASS_DIR_.'/data.php');
  
class kerawen_translation_HTMLTemplateGiftCardPdf {
	public function l($string) {		
		return Translate::getModuleTranslation('kerawen', $string, basename(__FILE__, '.php'));
	}		
}

class HTMLTemplateGiftCardPdf extends HTMLTemplate
{
	public $custom_model;
	public $smarty;
	public $file_name;
	public $module;
	public $id_lang;
	public $id_shop;
	
	public function __construct($custom_object, $smarty)
	{
			
		$this->custom_model = $custom_object;
		$this->smarty = $smarty;
		
		
		$context = Context::getContext();
		$this->id_shop = $context->shop->id;
				
		// Transalation
		//Can't use $this->l() -> already use by static parent class
		$this->module = new kerawen_translation_HTMLTemplateGiftCardPdf();
				
	}
	
	
	public function getStyleTemplate() {
	    //echo serialize(array(25,39,68));
	    $bgcolor = Configuration::get('KERAWEN_GIFCARD_BGCOLOR') ? unserialize(Configuration::get('KERAWEN_GIFCARD_BGCOLOR')) : false;
	    return array(
	        'bgimage' => $this->get_path_file('bg_gift_card.jpg'),
	        'bgcolor' => $bgcolor,
	    );
	}
	
	
	/**
	 * Returns the template's HTML content
	 * @return string HTML content
	 */
	public function getContent()
	{	    
	    $data = array(
	        'shop_name' => Configuration::get('PS_SHOP_NAME', null, null, $this->id_shop),
	        'img_card' => $this->get_path_file('img_card.jpg')
	    );
	    $this->smarty->assign(array_merge($this->custom_model, $data));
	    
	    
	    $tpls = array(
	        'style_tab' => false,
	    );
	    $this->smarty->assign($tpls);
        return $this->smarty->fetch( $this->get_path_file('giftcard.tpl') );
	}
 

	public function get_path_file($file) {

	    $file_dir = '';
	    
		//theme
		if (file_exists ( _PS_THEME_DIR_ . 'pdf/'  . $file )) {
   			$file_dir = _PS_THEME_DIR_ . 'pdf/'  . $file;
   		
   		//root
		} elseif (file_exists ( _PS_ROOT_DIR_ . '/pdf/'  . $file )) {
   			$file_dir =  _PS_ROOT_DIR_ . '/pdf/'  . $file;
   		
   		//kerawen module
		} elseif (file_exists ( _KERAWEN_DIR_. '/views/templates/front/pdf/'  . $file )) {
   			$file_dir =  _KERAWEN_DIR_. '/views/templates/front/pdf/' . $file;
		}      
		
		return $file_dir;
		
	}
   
   

	public function getPagination() {
	    return false;
	}


	public function getHeader()
	{
	    require_once(_PS_CLASS_DIR_ . 'Tools.php');
	    
	    $array_header = array(
	        'shop_name' => '',
	        'header' => '',
	        'title' => '',
	        'date' => date('Y-m-d H:i:s')
	    );
	    
	    if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '<=')) {
	        
	        $path_logo = $this->getLogo();
	        
	        $width = 0;
	        $height = 0;
	        if (!empty($path_logo)) {
	            list($width, $height) = getimagesize($path_logo);
	        }
	        
	        // Limit the height of the logo for the PDF render
	        $maximum_height = 100;
	        if ($height > $maximum_height) {
	            $ratio = $maximum_height / $height;
	            $height *= $ratio;
	            $width *= $ratio;
	        }
	        
	        
	        $array_header['logo_path'] = $path_logo;
	        $array_header['img_ps_dir'] = 'http://'.Tools::getMediaServer(_PS_IMG_)._PS_IMG_;
	        $array_header['img_update_time'] = Configuration::get('PS_IMG_UPDATE_TIME');
	        $array_header['width_logo'] = $width;
	        $array_header['height_logo'] = $height;
	        
	    } else {
	        $this->assignCommonHeaderData();
	    }

	    $this->smarty->assign($array_header);
	    return $this->smarty->fetch( $this->get_path_file('giftcard.header.tpl') );
	}


    /**
     * Compute layout elements size
     *
     * @param $params Array Layout elements
     *
     * @return Array Layout elements columns size
     */
    protected function computeLayout($params)
    {
        return array();
    }


	/**
	 * Returns the template filename
	 * @return string filename
	 */

	
	public function getFooter()
	{
	    
	    $array_data = array(
	        'shop_address' => $this->getShopAddressFormatted(' - '),
	    );

	    $shop_phone = Configuration::get('PS_SHOP_PHONE', null, null, $this->id_shop);
	    if ($shop_phone != '') {
	        $array_data['shop_phone'] = $shop_phone;
	    }
	    
	    $shop_fax = Configuration::get('PS_SHOP_FAX', null, null, $this->id_shop);
	    if ($shop_fax != '') {
	        $array_data['shop_fax'] = $shop_fax;
	    }
	    
	    $shop_details = Configuration::get('PS_SHOP_DETAILS', null, null, $this->id_shop);
	    if ($shop_details != '') {
	        $array_data['shop_details'] = $shop_details;
	    }
	    
	    
	    $link = new Link;
	    $shop_url = $link->getPageLink('index', true, null, null, false, $this->id_shop);
	    $array_data['free_text'] = $shop_url;
	    
	    $this->smarty->assign($array_data);
	    return $this->smarty->fetch( $this->get_path_file('giftcard.footer.tpl') );
	    
	}
	
	public function getShopAddressFormatted($sep = '<br />') {
	    
	    return
	    $this->newLine(Configuration::get('PS_SHOP_NAME', null, null, $this->id_shop), $sep)
	    . $this->newLine(Configuration::get('PS_SHOP_ADDR1', null, null, $this->id_shop), $sep)
	    . $this->newLine(Configuration::get('PS_SHOP_ADDR2', null, null, $this->id_shop), $sep)
	    . $this->newLine(Configuration::get('PS_SHOP_CODE', null, null, $this->id_shop) . ' ' . Configuration::get('PS_SHOP_CITY', null, null, $this->id_shop) , $sep)
	    . $this->newLine(Country::getNameById(Configuration::get('PS_LANG_DEFAULT', null, null, $this->id_shop), Configuration::get('PS_SHOP_COUNTRY_ID', null, null, $this->id_shop)), '');
	    
	}
	
	public function newLine($data, $sep) {
	    
	    if (trim($data) != '')
	        $data .= $sep;
	        
	        return $data;
	}
	
	/**
	 * Returns the template filename
	 * @return string filename
	 */
	public function getFilename()
	{
	    return 'giftcard-' . date('YmdHis') .'.pdf';
	}
 
	/**
	 * Returns the template filename when using bulk rendering
	 * @return string filename
	 */
	public function getBulkFilename()
	{
	    return 'giftcard-' . date('YmdHis') .'.pdf';
	}
}
