<?php
/**
* 2016 KerAwen
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2016 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/

/*
Source :
http://blog.belvg.com/pdf-in-prestashop.html
*/

class kerawen_translation_HTMLTemplatePurchaseOrderPdf {
    public function l($string) {
        return Translate::getModuleTranslation('kerawen', $string, basename(__FILE__, '.php'));
    }
}

class HTMLTemplatePurchaseOrderPdf extends HTMLTemplateInvoice
{
    
    public function __construct(OrderInvoice $order_invoice, $smarty, $bulk_mode = false)
    {
        $this->module = new kerawen_translation_HTMLTemplatePurchaseOrderPdf();
        parent::__construct($order_invoice, $smarty, $bulk_mode);
    }
    
    
    
    public function getHeader()
    {
        require_once(_PS_CLASS_DIR_ . 'Tools.php');
        $header = $this->module->l('Purchase order');
        $array_header = array(
            'shop_name' => $header,
            'header' => $header,
            'title' => '#' . $this->getPurchaseOrderNumber(),
            'date' => date('Y-m-d')
        );
        
        if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '<=')) {
            
            $path_logo = $this->getLogo();
            
            $width = 0;
            $height = 0;
            if (!empty($path_logo)) {
                list($width, $height) = getimagesize($path_logo);
            }
            
            // Limit the height of the logo for the PDF render
            $maximum_height = 100;
            if ($height > $maximum_height) {
                $ratio = $maximum_height / $height;
                $height *= $ratio;
                $width *= $ratio;
            }
            
            
            $array_header['logo_path'] = $path_logo;
            $array_header['img_ps_dir'] = 'http://'.Tools::getMediaServer(_PS_IMG_)._PS_IMG_;
            $array_header['img_update_time'] = Configuration::get('PS_IMG_UPDATE_TIME');
            $array_header['width_logo'] = $width;
            $array_header['height_logo'] = $height;
            
            
        } else {
            $this->assignCommonHeaderData();
            //$array_header['header'] = $height;
        }
        
        
        
        $this->smarty->assign($array_header);
        
        return $this->smarty->fetch( $this->get_path_file('purchase.header.tpl') );
        
    }

 
    public function getContent()
    {
        require_once(_KERAWEN_CLASS_DIR_ . '/order.php');
        $context = Context::getContext();
        $context->module = Module::getInstanceByName('kerawen');
        $context->cart = new Cart();
        
        $ret = parent::getContent();
        $db = Db::getInstance();
        
        $buf = $db->executeS('
          	SELECT
         		od.id_order_detail AS id,
         		odk.note AS note
         	FROM '._DB_PREFIX_.'order_detail od
         	LEFT JOIN '._DB_PREFIX_.'order_detail_kerawen odk ON odk.id_order_detail = od.id_order_detail
         	WHERE od.id_order = '.pSQL($this->order_invoice->id_order)
        );
        
        $notes = array();
        foreach ($buf as $row) {
            $notes[$row['id']] = $row['note'];
        }
        
        $params = new stdClass();
        $params->id_order = $this->order_invoice->id_order;
        $k_order = getOrder($context, $params);
        
        $this->smarty->assign(
            array(
                "notes" => $notes,
                "mode" => $k_order['mode'],
                "invoice_note" => $k_order['invoice_note'],
                "remain" => $k_order['remain'],
            )
        );

        $tpls = array(
            'addresses_tab' => $this->smarty->fetch( $this->get_path_file('purchase.addresses.tpl') ),
            'summary_tab' => $this->smarty->fetch( $this->get_path_file('purchase.summary.tpl') ),
            'payment_tab' => $this->smarty->fetch( $this->get_path_file('purchase.payment.tpl') ),
            'product_tab' => $this->smarty->fetch( $this->get_path_file('purchase.product.tpl') ),
            'total_tab' => $this->smarty->fetch( $this->get_path_file('purchase.total.tpl') ),
        );   
        
        $this->smarty->assign($tpls);
        return $this->smarty->fetch($this->getTemplateByCountry($context->language->iso_code));
    }
    
    

    public function get_path_file($file) {

        //theme
        if (file_exists ( _PS_THEME_DIR_ . 'pdf/'  . $file )) {
            $file_dir = _PS_THEME_DIR_ . 'pdf/'  . $file;
            
            //root
        } elseif (file_exists ( _PS_ROOT_DIR_ . '/pdf/'  . $file )) {
            $file_dir =  _PS_ROOT_DIR_ . '/pdf/'  . $file;
            
            //kerawen module
        } else {
            $file_dir =  _KERAWEN_DIR_. '/views/templates/front/pdf/' . $file;
        }
        
        return $file_dir;
        
    }

    
    public function getPurchaseOrderNumber() {
        return $this->module->l('P') . str_pad($this->order_invoice->id_order, 8, '0', STR_PAD_LEFT);
    }
 
    
    public function getFilename()
    {
        return $this->getPurchaseOrderNumber() . '.pdf';
    }
    
    public function getBulkFilename()
    {
        return $this->getPurchaseOrderNumber(). '.pdf';
    }
    
}

