<?php
/**
 * @author    KerAwen <contact@kerawen.com>
 * @copyright 2024 KerAwen
 */

class KerawenTax
{
    public static function GetVatMarginRule() {
        $id = (int)Configuration::get('KERAWEN_TAX_VATMARGIN');
        $tax = $id ? new TaxRulesGroup($id) : null;
        if (!Validate::isLoadedObject($tax)) {
            $tax = new TaxRulesGroup();
            $tax->name = Context::getContext()->module->l('VAT on margin', pathinfo(__FILE__, PATHINFO_FILENAME));
            $tax->active = true;
            $tax->save();
            Configuration::updateValue('KERAWEN_TAX_VATMARGIN', $tax->id);
        }
        
        //Try to build tax
        if ($tax) {
            $id_lang = Context::getContext()->language->id; //Configuration::get('PS_LANG_DEFAULT');
            $taxRules = TaxRule::getTaxRulesByGroupId($id_lang, $tax->id);
            if (empty($taxRules)) {
                $id_tax = (int) Db::getInstance()->getValue('SELECT id_tax FROM `' . _DB_PREFIX_ . 'tax_lang` WHERE id_lang = ' . (int) $id_lang . ' AND name = "TVA FR 20%"');
                if ($id_tax) {
                    $tr = new TaxRule();
                    $tr->id_tax_rules_group = $tax->id;
                    $tr->id_country = (int) Configuration::get('PS_COUNTRY_DEFAULT');
                    $tr->id_state = 0;
                    $tr->zipcode_from = 0;
                    $tr->zipcode_to = 0;
                    $tr->id_tax = $id_tax;
                    $tr->behavior = 0;
                    $tr->description = "";
                    $tr->save();
                }
            }
        }
        return $tax->id;
    }
}


class KerawenTaxManagerMargin extends TaxRulesTaxManagerCore
{
    protected $public_calculator;
    
    public function __construct(Address $address, $type) {
        parent::__construct($address, $type);
        $this->public_calculator = new KerawenTaxCalculatorMargin($this);
    }
    
    public function getTaxCalculator() {
        return $this->public_calculator;
    }
    
    public function getPrivateTaxCalculator() {
        return parent::getTaxCalculator();
    }
}


class KerawenTaxCalculatorMargin extends TaxCalculator
{
    // DO NOT apply taxes, but keep total rate
    protected $manager;
    
    public function __construct($manager) {
        $this->manager = $manager;
    }
    
    protected function getCalculator() {
        return $this->manager->getPrivateTaxCalculator();
    }
    
    public function addTaxes($price_te) { return $price_te; }
    public function removeTaxes($price_ti) { return $price_ti; }
    public function getTotalRate() { return $this->getCalculator()->getTotalRate(); }
    public function getTaxesName(){ return $this->getCalculator()->getTaxesName(); }
    public function getTaxesAmount($price_te) { return []; }
    public function getTaxesTotalAmount($price_te) { return 0; }
}

