<?php
/**
* 2014 KerAwen
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2014 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/

// Warning: false means OK...
function checkCart($id_cart, $version, $module)
{
	$db = Db::getInstance();
	if (!$db->getValue('
			SELECT id_cart FROM '._DB_PREFIX_.'cart
			WHERE id_cart = '. (int) $id_cart))
		return $module->l('Previous cart has been deleted', pathinfo(__FILE__, PATHINFO_FILENAME));
	if ($db->getValue('
			SELECT id_cart FROM '._DB_PREFIX_.'orders
			WHERE id_cart = '. (int) $id_cart))
		return $module->l('Previous cart has been checked out', pathinfo(__FILE__, PATHINFO_FILENAME));
	if ($version && !$db->getValue('
			SELECT id_cart FROM '._DB_PREFIX_.'cart_kerawen
			WHERE id_cart = '. (int) $id_cart .'
			AND version = '. (int) $version))
		return $module->l('Previous cart has been modified', pathinfo(__FILE__, PATHINFO_FILENAME));
	return false;
}

/*
 * Applies delivery options to cart
 * Automatically override carriers if the given one is not applicable
 * Returns list of applicable carriers for the given address
 */
function setDelivery($cart, $mode, $id_address = null, $carrier = null, $date = null)
{
	Context::getContext()->cart = $cart;
	$db = Db::getInstance();

	//initialize values
	$data_delivery_mode = " delivery_mode = 0";
	$data_id_address = " id_address = 0";
	$data_carrier = " carrier = NULL";
	$data_delivery_date = " delivery_date = NULL";
	
	if (isset($mode)) {
	    $data_delivery_mode = " delivery_mode = " . (int) $mode;
	    if ($id_address) $data_id_address = " id_address = " . (int) $id_address;
	    if ($carrier) $data_carrier = " carrier =  '" . pSQL($carrier) . "'";
	    if ($date) $data_delivery_date = " delivery_date = '" . pSQL($date) . "'";
	} else {
	    $ck = $db->getRow("SELECT * FROM " . _DB_PREFIX_ . "cart_kerawen WHERE id_cart = " . (int) $cart->id);
	    if ($ck) {
	        $data_delivery_mode = " delivery_mode = " . (int) $ck['delivery_mode'];
	        if ($ck["delivery_date"]) $data_delivery_date = " delivery_date = '" . pSQL($ck["delivery_date"]) . "'";
	        //clear data carrier
	    }
	}

	$q = "UPDATE " . _DB_PREFIX_ . "cart_kerawen SET " . $data_delivery_mode . ", " . $data_id_address . ", " . $data_carrier . ", " . $data_delivery_date . "  WHERE id_cart = " . (int) $cart->id;
	$db->execute($q);
}

function setDeliveryNote($cart, $private_note, $public_note)
{
    $db = Db::getInstance();

    $db->execute('
        INSERT INTO '._DB_PREFIX_.'cart_kerawen (id_cart, private_note, public_note)
        VALUES (
            ' . (int) $cart->id . ',
            "'.pSQL($private_note).'",
            "'.pSQL($public_note).'"
        ) ON DUPLICATE KEY UPDATE
            private_note = VALUES(private_note),
            public_note = VALUES(public_note)
    ');
}

    
function setDeliveryAddress($cart, $id_address)
{
	$cart->id_address_delivery = $id_address;
	Db::getInstance()->update('cart_product', array(
		'id_address_delivery' => (int) $cart->id_address_delivery,
	), 'id_cart = '. (int) $cart->id);
}

/* TO REFACTOR */
function resetCart($context, &$params, &$response)
{
    
    $db = Db::getInstance();
    
	$id_lang = $context->language->id;
	$id_curr = $context->currency->id;
	$id_empl = $context->employee->id;

	$id_cart = $params->id_cart;
	$suspend = isset($params->suspend) ? $params->suspend : false;
	
	$id_next = $params->id_next;
	$id_shop = $params->id_shop;
	$id_cust = $params->id_cust;

	if ($id_cart) {
	    if ($warning = checkCart($id_cart, $params->version, $context->module)) {
	        $response->addMessage('warning', $warning);
	    }
	    else {
	        if ($suspend) {
	            $db->update('cart_kerawen', array(
	                'suspended' => 1,
	            ), 'id_cart = '. (int) $id_cart);
	        } else {
	            //exclude quote
	            if (! (int) $db->getValue('SELECT 1 FROM '._DB_PREFIX_.'cart_kerawen WHERE quote = 1 AND id_cart = '. (int) $id_cart)) {
	                deleteCart(new Cart($id_cart));
	            }
	        }
	    }
	}
	
	$cart = null;
	if ($id_next) {
	    if ($warning = checkCart($id_next, false, $context->module)) {
	        $response->addMessage('warning', $warning);
	        $id_next = false;
	        require_once(_KERAWEN_CLASS_DIR_.'/quote.php');
	        $response->addResult('quotes', getQuotes($params));
	    } else {
	        setCartEmployee($id_next, $id_empl);
	        $cart = new Cart($id_next);
	        // Change customer to the cart owner
	        $params->id_cust = $id_cust = $cart->id_customer;
	        // New version of cart
	        $cart->kerawen_version = time();
	    }
	}
	
	if ($cart == null) {
	    $id_new = createCart($id_shop, $id_empl, $id_cust, $id_lang, $id_curr);
	    $cart = new Cart($id_new);
	}
	
	// Goto cart shop
	$context->shop = new Shop($cart->id_shop);
	Shop::setContext(Shop::CONTEXT_SHOP, $cart->id_shop);
	
	
	$cartAsArray = cartAsArray($cart);
	
	if ((int) $cartAsArray['quote']) {

	    $q = '
    	    SELECT
        	    CONCAT(id_product, "-", id_product_attribute, "-", quantity) AS pindex,
        	    id_product,
        	    id_product_attribute,
        	    quantity
    	    FROM '._DB_PREFIX_.'%s
    	    WHERE id_cart = ' . (int) $cartAsArray['id'];
	    
	    $kprods = indexArray($db->executeS(sprintf($q, 'cart_product_kerawen')) ,'pindex');
	    $pprods = indexArray($db->executeS(sprintf($q, 'cart_product')), 'pindex');
	   
	    $diff = array_merge(
	       array_diff_key($kprods, $pprods), 
	       array_diff_key($pprods, $kprods)
	    );

	    if (count($diff)) {
	        $response->addResult('quotediff', $diff);
	    }
	}
		
	$response->addResult('cart', $cartAsArray);
	$response->addResult('carts', getSuspendedCarts());
	
	require_once (dirname(__FILE__).'/customer.php');
	$response->addResult('cust', getCustomer($id_cust, $id_lang, 'all'));
	$response->addResult('referrals', getReferral($id_cust));
	
	require_once (dirname(__FILE__).'/cartrules.php');
	$response->addResult('rules', getCartRules($id_cust, $id_lang, $id_shop));

	//fixe cache issue PS 1.5.6.1 (le-chatel-des-vivaces)
	if ($id_next && Tools::version_compare(_PS_VERSION_, '1.6.0.0', '<=')) {
	    $response->addResult('cart', cartAsArray(new Cart($id_next)));
	}
	

}

function createCart($id_shop, $id_empl, $id_cust, $id_lang, $id_curr)
{
	$cart = new Cart();
	$cart->id_shop = $id_shop;
	$cart->id_customer = $id_cust;
	$cart->id_lang = $id_lang;
	$cart->id_currency = $id_curr;
	
	$cart->save();
	$id_cart = $cart->id;

	Db::getInstance()->insert('cart_kerawen', array(
		'id_cart' => $id_cart,
		'id_employee' => $id_empl,
	    'delivery_mode' => (int) Configuration::get('KERAWEN_DEFAULT_DELIVERY_MODE'),
		'version' => time(),
	));
	
	return $id_cart;
}

function selectShop($context, $params, &$response)
{
	$id_cart = $params->id_cart;
	$id_shop = $params->id_shop;

	// Update database references
	$db = Db::getInstance();

	$cart = new Cart($id_cart);
	$cart->id_shop = $id_shop;
	$cart->save();
	
	$sql = 'UPDATE `'._DB_PREFIX_.'cart_product` SET
			id_shop = '. (int) $id_shop .'
			WHERE id_cart = '. (int) $id_cart;
	$db->execute($sql);

	$response->addResult('cart', cartAsArray($cart));
}

function setCartEmployee($id_cart, $id_empl)
{
	$db = Db::getInstance();
	$db->execute('
		UPDATE `'._DB_PREFIX_.'cart_kerawen`
		SET id_employee = '. (int) $id_empl.'
		WHERE id_cart = '. (int) $id_cart);
}

function updateCart(&$cart, $version)
{
	$db = Db::getInstance();
	
	if (!$version) $version = $db->getValue('
			SELECT version
			FROM '._DB_PREFIX_.'cart_kerawen
			WHERE id_cart = '. (int) $cart['id']);
	$cart['version'] = $version; 

	$db->update('cart_kerawen', array(
		'suspended' => 0,
		'count' => $cart['count_cart'] + $cart['count_ret'],
		'total' => $cart['total_cart'] + $cart['total_ret'],
		'version' => $version,
	), 'id_cart = '.pSQL($cart['id']));
}

function getSuspendedCarts()
{
	$db = Db::getInstance();
	// Clean-up carts that have been ordered
	$db->execute('
		DELETE c
		FROM '._DB_PREFIX_.'cart_kerawen c
		LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_cart = c.id_cart
		WHERE o.id_order IS NOT NULL AND c.quote = 0');

	return $db->executeS('
		SELECT
			ca.id_cart AS id,
			ck.version AS version,
			ca.date_upd AS date,
			CONCAT(cu.firstname, " ", cu.lastname) AS cust,
			ca.id_shop AS id_shop,
			e.id_employee AS id_empl,
			CONCAT(e.firstname, " ", e.lastname) AS empl,
			ck.count AS count,
			ck.total AS total,
			ck.suspended AS suspended
		FROM '._DB_PREFIX_.'cart_kerawen AS ck
		JOIN '._DB_PREFIX_.'cart AS ca ON ca.id_cart = ck.id_cart
		LEFT JOIN '._DB_PREFIX_.'customer AS cu ON cu.id_customer = ca.id_customer
		LEFT JOIN '._DB_PREFIX_.'employee AS e ON e.id_employee = ck.id_employee
		WHERE ck.quote != 1
			AND (ck.suspended = 1 OR (ck.suspended = 0 AND ca.date_upd > NOW() - INTERVAL 7 DAY ))
		ORDER BY ck.version ASC');
}


function getCartQuantityByProduct($id_cart, $id_prod, $id_attr, $id_custom)
{
	if ($id_custom) {
		$q = '
			SELECT quantity FROM '._DB_PREFIX_.'customization
			WHERE id_cart = '. (int) $id_cart .'
			AND id_customization = '. (int) $id_custom;
	}
	else {
		$q = '
			SELECT quantity FROM '._DB_PREFIX_.'cart_product
			WHERE id_cart = '. (int) $id_cart.'
			AND id_product = '. (int) $id_prod.'
			AND id_product_attribute = '. (int) $id_attr;
	}
	return (int) Db::getInstance()->getValue($q);
}


function deleteCart($cart)
{
	if (!$cart->id) return;
	
	// Clean-up related elements
	$db = Db::getInstance();
	$db->delete('cart_kerawen', 'id_cart = '. (int) $cart->id);
	
	deletePaymentReceived($cart->id, 0);
	
	// Returns & virtual order details
	$db->execute('
		DELETE rk, od
		FROM '._DB_PREFIX_.'return_kerawen rk
		LEFT JOIN '._DB_PREFIX_.'order_detail od
			ON od.id_order_detail = rk.id_order_detail AND od.id_order = 0
		WHERE rk.id_cart = '. (int) $cart->id);


	// Specific combinations
	$buf = $db->executeS('
		SELECT `id_product_attribute` FROM `'._DB_PREFIX_.'product_attribute_kerawen`
		WHERE `id_cart` = '. (int) $cart->id);
	foreach ($buf as $comb) {
		$comb = new Combination($comb['id_product_attribute'], null, $cart->id_shop);
		$comb->delete();
	}
	$db->delete('product_attribute_kerawen', 'id_cart = '. (int) $cart->id);
	
	
	$db->delete('cart_product_kerawen', 'id_cart = '. (int) $cart->id);
	
	//Specific prices
	delItemPrice($cart->id);

	$cart->delete();
}


function adjustItem($context, $params, &$response)
{
    require_once(_KERAWEN_CLASS_DIR_.'/cartrules.php');
    
	$db = Db::getInstance();
	$cart = getCart($params->id_cart, $params->id_cust, $context);
	$context->cart = $cart;
	
	$is_quote = (int) $db->getValue('
		SELECT 1 FROM '._DB_PREFIX_.'cart_kerawen
		WHERE quote = 1 AND id_cart = '. (int) $cart->id
	);
	

	$id_prod = $params->id_prod;
	$id_lang = Configuration::get('PS_LANG_DEFAULT');
	$id_shop = $context->shop->id;
	
	// Load product with full context in order to activate same cache as Cart
	$prod = new Product($id_prod, false, $id_lang, $id_shop);

	$res = true;
	$applyQty = true;
	$code = false;
	
	$is_gift_card = false;
	if (isset($params->giftcode)) {
	    if (!($params->giftcode === false)) {
	        $is_gift_card = true;
	    }
	}
	
	if ($is_gift_card) {
	    if ($params->giftcode == '') {
	        $code = generateRuleCode();
	        //OR get code from existing list...
	    }
	    else {
	        $code = trim($params->giftcode);
	        $id_cart = (int) Db::getInstance()->getValue('SELECT id_cart FROM '._DB_PREFIX_.'cart_product_kerawen WHERE giftcode = "' . pSQL($params->giftcode) . '"');
	        if (CartRule::cartRuleExists($code) || $id_cart > 0 || $code == '') {
	            $applyQty = false;
	            $res = false;
	        }
	    }
	    if ($res && !isset($params->price)) {
            // Default price
            $comb = new Combination($params->id_attr, null, $id_shop);
            $params->price = $prod->price + ($comb->id_product ? $comb->price : 0);
        }
	}
	

	// Create attribute if needed
	if (isset($params->price)) {
	    $id_code = isset($params->id_code) ? $params->id_code : null;
	    $prod_type = isset($params->prod_type) ? $params->prod_type : '';
	    $id_attr = createProductPrice($cart, $prod, $id_code, $params->price, $code, $prod_type);
	}
	else {
	    $id_attr = (int)($params->id_attr ? $params->id_attr : $prod->getDefaultIdProductAttribute());
	}
	
	// Create customization if needed ?
	$id_custom = isset($params->id_custom) ? $params->id_custom : false;

	$initial_qty = getCartQuantityByProduct($params->id_cart, $params->id_prod, $id_attr, $id_custom);
	$minimal_qty = ($id_attr > 0) ? ((Tools::version_compare(_PS_VERSION_, '8', '>=')) ? ProductAttribute::getAttributeMinimalQty($id_attr) : Attribute::getAttributeMinimalQty($id_attr)) : $prod->minimal_quantity;
	
	if (!$minimal_qty) $minimal_qty = 1;

	$set = isset($params->set) ? $params->set : false;
	$qty = (int)$params->qty + ($set ? 0 : $initial_qty);
	
	if ($qty == 0) {
		if ($id_attr == -1) {
			// Remove any combination of product
			//TODO Improve
			$attrs = Db::getInstance()->executeS('
				SELECT id_product_attribute, quantity
				FROM '._DB_PREFIX_.'cart_product
				WHERE id_cart = '. (int) $cart->id.'
				AND id_product = '. (int) $prod->id);
			if (count($attrs)) {
				$shop = new Shop($cart->id_shop);
				foreach ($attrs as $attr)
					$cart->updateQty($attr['quantity'], $prod->id, $attr['id_product_attribute'], false, 'down', 0, $shop);
			}
		}
		else {
			$cart->deleteProduct($prod->id, $id_attr, $id_custom);
			$db->delete('cart_product_kerawen', 'id_cart = '. (int) $cart->id . ' AND id_product = ' . (int) $prod->id . ' AND id_product_attribute = ' . (int) $id_attr );
			delItemPrice($cart->id, $prod->id, $id_attr);
		}
		
	}
	else {
		$diff = $qty - $initial_qty;
		$op = $diff < 0 ? 'down' : 'up';
		$diff = round(abs($diff), 0);
		
		// Check product status and quantities
		$outofstock = null;
		if (isset($params->force) && $params->force) {
			$sa = new StockAvailable(StockAvailable::getStockAvailableIdByProductId($prod->id, $id_attr));
			$outofstock = $sa->out_of_stock;
			StockAvailable::setProductOutOfStock($prod->id, true, $cart->id_shop, $id_attr);
		}
		
		$available = null;
		if (isset($params->force) && $params->force) {
			$available = $prod->available_for_order;
			// Update cache for Cart
			$cache_id = 'objectmodel_Product_'.(int)$id_prod.'_'.(int)$id_shop.'_'.(int)$id_lang;
			$data = Cache::retrieve($cache_id);
			$data['available_for_order'] = true;
			Cache::store($cache_id, $data);
		}
		
		if ($diff) {
		    
		    if ($res) {
    			// Create custom if needed
    			if (!$id_custom && isset($params->custom) && $params->custom) {
    				$custom = new Customization();
    				$custom->id_cart = $cart->id;
    				$custom->id_product = $prod->id;
    				$custom->id_product_attribute = $id_attr;
    				$custom->id_address_delivery = $cart->id_address_delivery;
    				$custom->quantity = 0;
    				$custom->quantity_refunded = 0;
    				$custom->quantity_returned = 0;
    				$custom->in_cart = 1;
    				$custom->save();
    				$id_custom = $custom->id;
    				
    				foreach($params->custom as $field) {
    					if (isset($field->value)) {
    						$db->insert('customized_data', array(
    							'id_customization' => (int) $id_custom,
    							'index' => (int) $field->id_customization_field,
    							'type' => (int) $field->type,
    							'value' => pSQL($field->value),
    						));
    					}
    				}
    			}
    			

    			if (Tools::version_compare(_PS_VERSION_, '1.7.3', '<')) {
    				$res = $cart->updateQty($diff, $prod->id, $id_attr, $id_custom,
    					$op, 0, new Shop($cart->id_shop));
    			}
    			else {
    				$res = $cart->updateQty($diff, $prod->id, $id_attr, $id_custom,
    					$op, 0, new Shop($cart->id_shop), true, isset($params->force) && $params->force);
    			}
    			
    			//cart tags
    			if (Configuration::get('KERAWEN_CART_TAG')) {
    			    $id_tag = (int) $db->getValue('SELECT id_tag FROM '._DB_PREFIX_.'tag_kerawen WHERE tag_default = 1');
    			    $db->execute('
        			    INSERT INTO '._DB_PREFIX_.'cart_product_kerawen (id_cart, id_product, id_product_attribute, id_tag)
        			    VALUES (
        			        '. (int) $cart->id.',
        			        '. (int) $prod->id.',
        			        '. (int) $id_attr.',
        			        '. (int) $id_tag.'
                        ) ON DUPLICATE KEY UPDATE
        			        id_tag = IF(id_tag IS NULL, ' . (int) $id_tag . ', id_tag)
    			    ');
    			}
    			
		    }
		}
		
		
		//$r = $cart->containsProduct($prod->id, $id_attr);
		
		$r = $db->getRow('
            SELECT cp.`quantity` 
                FROM `'._DB_PREFIX_.'cart_product` cp
			WHERE cp.`id_product` = '.(int) $prod->id.'
			AND cp.`id_product_attribute` = '.(int) $id_attr.'
			AND cp.`id_cart` = '.(int)$cart->id);

		if ($r) {
			$sp = SpecificPrice::getIdsByProductId($params->id_prod, $id_attr, $params->id_cart);
			if (count($sp)) {
				$discountCart = -1;
				$calc = 0;
				$final_qty = (int) $r['quantity'];
				
				$id_specific_price = $sp[0]['id_specific_price'];
				$specificPrice = new SpecificPrice($id_specific_price);
				
				if ( $final_qty > 0) {
					$cpk = $db->getRow('SELECT specific_price_cart, specific_price_cart_calc FROM '._DB_PREFIX_.'cart_product_kerawen WHERE id_cart = ' . (int) $params->id_cart . ' AND id_product = ' . (int) $params->id_prod . ' AND id_product_attribute = ' . (int) $id_attr);
					if ($cpk) {
						if ((int) $cpk['specific_price_cart_calc']) {
						    $specificPrice->reduction = round($initial_qty / $final_qty * $specificPrice->reduction, 6);
						}
					}
				}

				//give more weight to priority specific price - keep it !!!!
				$specificPrice->from_quantity = $final_qty;
				$specificPrice->update();
			}
			
			if ($is_quote) {
			    set_cart_product_kerawen($params->id_cart, $params->id_prod, $id_attr, $r['quantity']);
			}
			
		}
		else {
			delItemPrice($params->id_cart, $params->id_prod, $id_attr);
			$db->delete('cart_product_kerawen', 'id_cart = '. (int) $params->id_cart . ' AND id_product = ' . (int) $params->id_prod . ' AND id_product_attribute = ' . (int) $id_attr);
		}
		
		// Restore product configuration
		if ($outofstock !== null)
			StockAvailable::setProductOutOfStock($prod->id, $outofstock, $cart->id_shop, $id_attr);
	}

	if ($res !== true) {
		// Determine error case
	    if (!$applyQty)
	        $response->addResult('giftCardExist', true);
		elseif ($res < 0)
			$response->addResult('minimal_qty', $minimal_qty);
		elseif (!$prod->available_for_order)
			$response->addResult('notavailable', true);
		else
			$response->addResult('outofstock', true);
	}
	else
	{
		$bean = cartAsArray($cart);
		$response->addResult('cart', $bean);
		$response->addResult('added', array(
			'id_prod' => $prod->id,
			'id_attr' => $id_attr,
			'id_custom' => $id_custom,
		));
	}
}

function set_cart_product_kerawen($id_cart, $id_product, $id_product_attribute, $quantity) {
    
    $db = Db::getInstance();
    $db->execute('
    	INSERT INTO '._DB_PREFIX_.'cart_product_kerawen (id_cart, id_product, id_product_attribute, quantity)
    	VALUES (
    		'. (int) $id_cart .',
    		'. (int) $id_product .',
    		'. (int) $id_product_attribute .',
    		'. (int) $quantity .')
    	ON DUPLICATE KEY UPDATE
    		quantity = VALUES(quantity)
    ');
    
}


function addRule($context, $params, &$response)
{
	$id_cart = $params->id_cart;
	$id_rule = $params->id_rule;
	$id_lang = $context->language->id;
	$id_shop = $context->shop->id;
	
	if ($id_cart && $id_rule)
	{
		$cart = new Cart($id_cart);
		$rule = new CartRule($id_rule);
		$id_anonymous = (int)Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER');
		
		if ($cart->id_customer && $cart->id_customer != $id_anonymous) {
			// Check it's the same customer
			if ($rule->id_customer && $rule->id_customer != $cart->id_customer) {
				$response->addMessage('error', $context->module->l('Different customer', pathinfo(__FILE__, PATHINFO_FILENAME)));
				return;
			}
		}
		else if ($rule->id_customer)
		{
			// Set the customer for cart
			if ($cart->id_customer != $rule->id_customer) {
				$cart->id_customer = $rule->id_customer;
				$cart->save();
				$response->addMessage('info', 'Customer has been selected');
			}

			require_once (dirname(__FILE__).'/customer.php');
			$response->addResult('cust', getCustomer($cart->id_customer, $id_lang));
		}

		$context->cart = $cart;
		$cart->addCartRule($id_rule);
		$response->addResult('cart', cartAsArray($cart));
		
		// Remove rule as a payment
		Db::getInstance()->execute('
            DELETE FROM '._DB_PREFIX_.'payment_received_kerawen
            WHERE id_rule = '.(int)$id_rule);
        
		// Update available cart rules
		require_once (dirname(__FILE__).'/cartrules.php');
		$response->addResult('rules', getCartRules($cart->id_customer, $id_lang, $id_shop));
	}
}

function remRule($context, $params, &$response)
{
	$id_cart = $params->id_cart;
	$id_rule = $params->id_rule;
	$id_lang = $context->language->id;
	$id_shop = $context->shop->id;
	
	if ($id_cart)
	{
		$cart = new Cart($id_cart);
		$cart->removeCartRule($id_rule);

		// Delete the rule if cart restricted
		$db = Db::getInstance();
		if ($db->getValue('
			SELECT id_cart FROM `'._DB_PREFIX_.'cart_rule_kerawen`
			WHERE type != \'' . _KERAWEN_CR_GIFT_CARD_ . '\' AND `id_cart_rule` = '. (int) $id_rule))
		{
			$rule = new CartRule($id_rule);
			$rule->delete();
			$db->delete('cart_rule_kerawen',
				'`id_cart_rule` = '. (int) $id_rule);
		}

		$response->addResult('cart', cartAsArray($cart));

		// Update available cart rules
		require_once (dirname(__FILE__).'/cartrules.php');
		$response->addResult('rules', getCartRules($cart->id_customer, $id_lang, $id_shop));
	}
}

/*
 *  Create a product combination with the given price
*/
function createProductPrice($cart, $prod, $id_code, $price, $code, $prod_type = '')
{
    
    require_once(_KERAWEN_API_DIR_.'/constants.php');
    
	$db = Db::getInstance();
	$context = Context::getContext();

	// Ensure combinations are active
	Configuration::updateGlobalValue('PS_COMBINATION_FEATURE_ACTIVE', '1');
	
	//Ensure product is not virtual (cause need combinations)
	if ($prod->is_virtual && !(Module::isInstalled('combinationsfile') && Module::isEnabled('combinationsfile')) ) {
	    $prod->is_virtual = 0;
	    $prod->save();
	}
	
	$cart_customer = new Customer($cart->id_customer);
	$id_group = (int) Configuration::get('KERAWEN_OVERRIDE_GROUP') > 0 ? (int) Configuration::get('KERAWEN_OVERRIDE_GROUP') : $cart_customer->id_default_group;
	$display_method = (int) Group::getPriceDisplayMethod($cart_customer->id_default_group);
	$rate_group = 1;

	if ($prod_type == '') {
	    $prod_wm = $db->getRow('SELECT * FROM '._DB_PREFIX_.'product_wm_kerawen WHERE id_product=' . (int) $prod->id);
	    if ($prod_wm) {
	        //history compatibiliy
	        if ($prod_wm['product_type'] == _KERAWEN_TYPE_GIFTCARD_ || (int) $prod_wm['is_gift_card'] == 1) {
	            $prod_type = _KERAWEN_TYPE_GIFTCARD_;
	        } else {
	            if ((int) $prod_wm['is_free_price'] == 1) {
	                $prod_type = _KERAWEN_TYPE_GENERIC_;
	            }
	            if ((int) $prod_wm['measured'] == 1) {
	                $prod_type = _KERAWEN_TYPE_MEASURED_;
	            }
	        }
	    }
	}
	
	if (
	   ((boolean) Configuration::get('KERAWEN_WM_GROUP_EXC') && $prod_type == _KERAWEN_TYPE_MEASURED_) 
	    || $prod_type == _KERAWEN_TYPE_GIFTCARD_ 
	    || $prod_type == _KERAWEN_TYPE_GENERIC_
	) {
	   $ReductionByIdGroup = Group::getReductionByIdGroup($id_group);
	   $rate_group = (1 - $ReductionByIdGroup/100);
	}
	

	$attr_wholesale_price = 0.0;
	$vat = 1.0 + $prod->getTaxesRate() / 100.0;
	if ($prod_type == _KERAWEN_TYPE_DEPOSIT_) {
	    $rate = 1;
	    $margin_free_price = (float) $db->getValue('SELECT margin_free_price FROM '._DB_PREFIX_.'product_wm_kerawen WHERE id_product=' . (int)$prod->id);
	    if ($margin_free_price > 0) {
	        $rate = $margin_free_price / 100 + 1;
	    } else {
    	    $cfg = json_decode(Configuration::get('KERAWEN_PROD_TYPES'), true);
    	    if (isset($cfg['DEPOSIT'])) {
    	        $cfg_deposit = $cfg['DEPOSIT'];
    	        $margin_rate = isset($cfg_deposit['margin_rate']) ? (int) $cfg_deposit['margin_rate'] : 0;
    	        $rate = $margin_rate / 100 + 1;
    	    }
	    }
	    if ($rate > 1) {
	        $attr_wholesale_price = $price / (($rate - 1) * $vat + 1);
	    }	    
	} else {
	    if (!$display_method || $id_code) {
	        $price = round($price / $vat, 6);
	    }
	}
	
	$DefaultIdProductAttribute = (int)Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('
		SELECT pa.`id_product_attribute`
		FROM `'._DB_PREFIX_.'product_attribute` pa
		INNER JOIN '._DB_PREFIX_.'product_attribute_shop product_attribute_shop ON (product_attribute_shop.id_product_attribute = pa.id_product_attribute)
		WHERE pa.`id_product` = '.(int)$prod->id.'
		AND product_attribute_shop.default_on = 1'
	);
	
	// Ensure default combination exists
	//if (!$prod->getDefaultIdProductAttribute()) {
	if ($DefaultIdProductAttribute == 0) {
		$id_attr_default = $prod->addCombinationEntity(0.0, 0.0, 0.0, 0.0, 0.0, 0,
				null, null, null, null, true,
				null, null, 1,
				Shop::getShops(false, null, true));
		Product::flushPriceCache();
	}
	
	// Create combination
	// Adjust price from product base price
	// Hopefully avoid scientific notation when validating Combination price
	$attr_price = Tools::ps_round(($price - $prod->price) / $rate_group , 6);
	$attr_wholesale_price = Tools::ps_round($attr_wholesale_price, 6);
	
	$id_attr = $prod->addCombinationEntity($attr_wholesale_price, $attr_price, 0.0, 0.0, 0.0, 0,
		null, null, null, null, false,
		null, null, 1, array($cart->id_shop));
	
	
	// Limit usage of combination to current cart
	$more = array(
		'id_product_attribute' => $id_attr,
		'id_cart' => $cart->id,
	);
	
	//default qty
	$def_qty = 2;
	
	// Weight or measure
	if ($id_code) {
		require_once(dirname(__FILE__).'/catalog.php');
		$ext = getProductExtension($prod->id, false);
		if ($ext && $ext['measured']) {
			$unit_price = $db->getValue('
				SELECT unit_price
				FROM '._DB_PREFIX_.'product_wm_code_kerawen
				WHERE id_code = '. (int) $id_code);
			$more = array_merge($more, array(
				'id_code' => $id_code,
				'measure' => $unit_price > 0 ? Tools::ps_round($price/$unit_price, (int)$ext['precision']) : null,
			));
		}
	}
	
	$db->insert('product_attribute_kerawen', $more); 
	
	//Giftcard code
	if ($code) {
	    $db->execute('
		INSERT INTO '._DB_PREFIX_.'cart_product_kerawen (id_cart, id_product, id_product_attribute, giftcode)
		VALUES (
			'. (int) $cart->id.',
			'. (int) $prod->id.',
			'. (int) $id_attr.',
			"'.pSQL($code).'")
		ON DUPLICATE KEY UPDATE
			giftcode = VALUES(giftcode)');
	}
	
	
	StockAvailable::setQuantity($prod->id, $id_attr, 3, $cart->id_shop);
	StockAvailable::setProductOutOfStock($prod->id, true, $cart->id_shop, $id_attr);
	
	// Flush prices to get new attribute into account
	Product::flushPriceCache();
	
	return $id_attr;
}


function setItemCartPrice($id_cart, $id_prod, $id_attr, $value = -1) {

	$db = Db::getInstance();
	
	$context = Context::getContext();
	$id_shop = $context->shop->id;
	$id_lang = $context->language->id;

	if (isset($context->group)) {
		$id_group = $context->group->id;
		$reduction_group = $context->group->reduction;
	} else {
		$id_group = 1;
		$reduction_group = Group::getReductionByIdGroup($id_group);
	}	

	$reduction_group_category = GroupReduction::getValueForProduct($id_prod, $id_group);
	$display_method = (int) Group::getPriceDisplayMethod($id_group);
	
	$init_value = $value;
	if ($value == '') { $value = -1; }
	
	if ( $value > -1 ) { 
		$prod = new Product($id_prod, true, $id_lang, $id_shop);
		$vat = $display_method ? 1 : (1.0 + $prod->getTaxesRate() / 100.0);
		
		$vat_offset = 0;
		$vat_margin = isset($prod->vat_margin) ? $prod->vat_margin : 0;
		if ($vat_margin && isset($prod->wholesale_price)) {
			$vat_offset = $prod->wholesale_price;
		}
		
		if ($reduction_group_category) {
			$spec_rate = 1 / ( 1 - $reduction_group_category );
		} else {
			$spec_rate = 1 / ( 1 - $reduction_group / 100 );
		}

		$value = round((($value - $vat_offset) / $vat + $vat_offset) * $spec_rate, 6);
	}
	
	$id_spec = false;
	$sp = SpecificPrice::getIdsByProductId($id_prod, $id_attr, $id_cart);
	if (count($sp)) {
		$id_spec = $sp[0]['id_specific_price'];
	}
	
	$specificPrice = new SpecificPrice($id_spec);
	$specificPrice->id_shop = $id_shop; //!important
	$specificPrice->id_cart = $id_cart;
	$specificPrice->id_product = $id_prod;
	$specificPrice->id_product_attribute = $id_attr;
	$specificPrice->reduction_tax = 1;
	
	$specificPrice->id_shop_group = 0;
	$specificPrice->id_currency = 0;
	$specificPrice->id_country = 0;
	$specificPrice->id_group = 0;
	$specificPrice->id_customer = 0;
	$specificPrice->from = '0000-00-00 00:00:00';
	$specificPrice->to = '0000-00-00 00:00:00';
	
	if (!$id_spec) {
	   $specificPrice->from_quantity = 1;
	   $specificPrice->reduction_type = 'amount';
	   $specificPrice->reduction = 0;
	}
	
	$specificPrice->price = $value;
	$specificPrice->save();

	$values_cpk = array();
	$values_cpk['specific_price_redefined'] = 1;
	$values_cpk['specific_price_cart_price'] = $init_value == '' ? null : (float) round($init_value, 6);
	$values_cpk['id_cart'] = (int) $id_cart;
	$values_cpk['id_product'] = (int) $id_prod;
	$values_cpk['id_product_attribute'] = (int) $id_attr;

	$where = ' id_cart = ' . (int) $id_cart . ' AND id_product = ' . (int) $id_prod . ' AND id_product_attribute = ' . (int) $id_attr;
	$cpk = (int) $db->getValue('SELECT 1 FROM '._DB_PREFIX_.'cart_product_kerawen WHERE ' . $where);
	if ($cpk) {
	    $db->update('cart_product_kerawen', $values_cpk, $where, 0, true);
	} else {
		$db->insert('cart_product_kerawen', $values_cpk, true);
	}
}


function delItemPrice($id_cart, $id_prod = false, $id_attr = false)
{
    SpecificPrice::deleteByIdCart($id_cart, $id_prod, $id_attr);
    
    //Set default values
    $values_cpk = array(
        'specific_price_cart' => 0,
        'specific_price_cart_calc' => 0,
        'specific_price_redefined' => 0,
        'specific_price_cart_value' => null,
        'specific_price_cart_price' => null,
    );
 
    Db::getInstance()->update(
        'cart_product_kerawen', 
        $values_cpk,
        'id_cart = ' . (int) $id_cart . ' AND id_product = ' . (int) $id_prod . ' AND id_product_attribute = ' . (int) $id_attr,
        0,
        true
    );
}

function setItemPrice($id_cart, $id_prod, $id_attr, $type, $value, $calc = 0)
{
	$db = Db::getInstance();
	
	$context = Context::getContext();
	$id_lang = $context->language->id;
	$id_customer = $context->customer->id;
	$id_shop = $context->shop->id;

	$id_currency = $context->currency->id;
	$id_country = $context->country->id;
	
	if (isset($context->group)) {
		$id_group = $context->group->id;
		$reduction_group = $context->group->reduction;		
	} else {
		$id_group = 1;
		$reduction_group = Group::getReductionByIdGroup($id_group);
	}

	$reduction_group_category = GroupReduction::getValueForProduct($id_prod, $id_group);
	//$display_method = (int) Group::getPriceDisplayMethod($id_group);
	

	//keep price with/without tax even if the defaut group is forced -> apply specific price (absolute price/discount) in display method context (with/without vat)
	$cart_customer = new Customer($id_customer);
	$display_method = (int) Group::getPriceDisplayMethod($cart_customer->id_default_group);

		
	$where = ' id_cart = ' . (int) $id_cart . ' AND id_product = ' . (int) $id_prod . ' AND id_product_attribute = ' . (int) $id_attr;
	
	$initial_value = $value;
	
	//Discount only one product or all products
	if ($calc) {		
		$quantity = getCartQuantityByProduct($id_cart, $id_prod, $id_attr, null);
		if ($quantity > 1) {
			$value = $value / $quantity;
		}
	}
	
	$cart = new Cart($id_cart);
	//$res = $cart->containsProduct($id_prod, $id_attr);
	
	$res = $db->getRow('
            SELECT cp.`quantity`
                FROM `'._DB_PREFIX_.'cart_product` cp
			WHERE cp.`id_product` = '.(int) $id_prod.'
			AND cp.`id_product_attribute` = '.(int) $id_attr.'
			AND cp.`id_cart` = '.(int) $id_cart);
	
	$quantity = $res ? $res['quantity'] : 1;

	$prod = new Product($id_prod, true, $id_lang, $id_shop);

	$vat = (1.0 + $prod->getTaxesRate() / 100.0);
	
	$value = ($display_method && $type == 'amount') ? ($value * $vat) : $value;
	

	//Important : Fix context (exclude current cart context)
	$current_cart = $context->cart;
	$specific_price_output = null;
	$context->cart = new Cart();
	
	$init_price = $prod->getPriceStatic($prod->id, true, $id_attr, 6, null, false, false, 1, false, $id_customer, null, null, $specific_price_output, true, true, $context, true);	
	$init_price_vat_excl = round($init_price / $vat, 6);
	
	//restore context cart
	$context->cart = $current_cart;
	
	$spec_rate = 1;
	$spec_price = -1;
	//$spec_price = $init_price_vat_excl;

	
	$specific = SpecificPrice::getSpecificPrice($prod->id, $id_shop, $id_currency, $id_country, $id_group, 1, $id_attr, $id_customer);
	
	// IMPORTANT: Keep full price resolution 
	$price = Product::priceCalculation(
			$id_shop, $prod->id, $id_attr, $id_country, null, null, $id_currency,
			$id_group, 1, true, 6, false, true, true, $specific, true, $id_customer, true, 0, 0);

		
	//fix bug cart by using absolute price
	//need to adjust PS version - (bibop-et-lula)
	if (Tools::version_compare(_PS_VERSION_, '1.6.1.20', '<')) {
		if ((int) $id_attr > 0) {
			$attribute_price = (float) $db->getValue('SELECT price FROM '._DB_PREFIX_.'product_attribute WHERE id_product_attribute = ' . (int) $id_attr);
			if ($attribute_price != 0) {
				$spec_price = $init_price_vat_excl;
			}
		}
	}

	//catalog
	if (isset($specific['price'])) {
		//cache issue can't use $specific['price']
		$price_no_cache = (float) $db->getValue('SELECT price FROM '._DB_PREFIX_.'specific_price WHERE id_specific_price = ' . (int) $specific['id_specific_price']);		
		if ( $price_no_cache !== -1.00) {
		//if ( (float) $specific['price'] !== -1.00) {
			$init_price = $price;
			$init_price_vat_excl = round($init_price / $vat, 6);	
			$spec_price = $init_price_vat_excl;
		}
	}	


	//fix bug PS 1.6.1.4 if default price = 0 !!!important	
	if ((float) $prod->price == 0.0) {
		$spec_price = $init_price_vat_excl;
	}

	//reverse reduction group amount
	if ($reduction_group_category) {
		$reduction_group_category = (float) $reduction_group_category;
		$spec_rate = 1 / ( 1 - $reduction_group_category );
		$spec_price = $init_price_vat_excl;
	} else {
		if ($reduction_group > 0) {
			$spec_rate = 1 / ( 1 - $reduction_group / 100 );
			$spec_price = $init_price_vat_excl;
		}
	}
	

	//Specific price cart exists
	$id_spec = false;
	//reduction_tax, 
	$sp = $db->getRow('SELECT id_specific_price, price, reduction_type FROM '._DB_PREFIX_.'specific_price WHERE id_cart = ' . (int) $id_cart . ' AND id_product = ' . (int) $id_prod . ' AND id_product_attribute = ' . (int) $id_attr);
	
	if ($sp) {
		$id_spec = $sp['id_specific_price'];
		if ($sp['price'] > -1) {

			//si prix redefini
			if ($db->getValue('SELECT specific_price_redefined FROM '._DB_PREFIX_.'cart_product_kerawen WHERE ' . $where)) {
				$init_price = $price = round($sp['price'] * $vat, 6);
			}
			
			$spec_price = $sp['price'];	
				
			//keep same absolute price
			//if ($reduction_group > 0) {
				$spec_price = $spec_price / $spec_rate;
			//}

		} else {
		    if ((float) $value <= 0) {
		      delItemPrice($id_cart, $id_prod, $id_attr);
		      return false;
		    }
		}
	}


	$specificPrice = new SpecificPrice($id_spec);	
	$specificPrice->id_shop = $id_shop; //!important
	$specificPrice->id_cart = $id_cart;
	$specificPrice->id_product = $id_prod;
	$specificPrice->id_product_attribute = $id_attr;
	$specificPrice->from_quantity = $quantity; //!important
	
	
	$specificPrice->id_shop_group = 0;
	$specificPrice->id_currency = 0;
	$specificPrice->id_country = 0;
	$specificPrice->id_group = 0;
	$specificPrice->id_customer = 0;
	$specificPrice->from = '0000-00-00 00:00:00';
	$specificPrice->to = '0000-00-00 00:00:00';
	
	
	//Change discount % to amount when 100% -> otherwise initial price is lost on order detail
	if ($type == 'percentage' && $value == '100') {
		$value = $price;
		$type = 'amount';
	}
	
	
	if ($type == 'percentage') {
		
		$reduction = ($init_price > 0 ) ? 1 - $price * (1 - $value/100) / $init_price : 0;
				
		$reducTotal = $init_price * $reduction;
		$reducCatalog = $init_price - $price;
		$reducCart = $reducTotal - $reducCatalog;

		//round important
		$specificPrice->price = round($spec_price * $spec_rate, 6);
		$specificPrice->reduction_type = 'percentage';
		$specificPrice->reduction = round($reduction, 6);

	}
	elseif ($type == 'amount') {
		
		$reduction = $value + $init_price - $price;
		
		$reducTotal = $reduction;
		$reducCatalog = $init_price - $price;
		$reducCart = $value / $spec_rate;

		//round important
		$specificPrice->price = round($spec_price * $spec_rate, 6);
		$specificPrice->reduction_type = 'amount';
		$specificPrice->reduction = round($reduction * $spec_rate, 6);

	}
	else {
		
		$reduction = 0;
		
		$reducTotal = 1;
		$reducCatalog = 0;
		$reducCart = 0;
		
		$specificPrice->price = $price;
		$specificPrice->reduction_type = 'amount';
		$specificPrice->reduction = 0;
	}
	

	$specificPrice->save();

	if ($reducTotal > 0) {
		$pc = round($reducCart / $reducTotal, 4);
	} else {
		$pc = 0;
	}
	
	//Save specific price status from cart
	$values_cpk = array();
	$values_cpk['specific_price_cart'] = $pc;
	$values_cpk['specific_price_cart_calc'] = $calc;
	$values_cpk['specific_price_cart_value'] = round($initial_value, 6);
	
	$cpk = (int) $db->getValue('SELECT 1 FROM '._DB_PREFIX_.'cart_product_kerawen WHERE ' . $where);	
	if ($cpk) {
		$db->update('cart_product_kerawen', $values_cpk, $where);
	} else {
		if ( $values_cpk['specific_price_cart'] > 0 ) {
			$values_cpk['id_cart'] = (int) $id_cart;
			$values_cpk['id_product'] = (int) $id_prod;
			$values_cpk['id_product_attribute'] = (int) $id_attr;
			$db->insert('cart_product_kerawen', $values_cpk);
		}
	}
}

function annotateCartItem($id_cart, $id_prod, $id_attr, $note)
{
	$db = Db::getInstance();
		
	$oldNote = (String) $db->getValue('SELECT note FROM '._DB_PREFIX_.'cart_product_kerawen WHERE id_cart= ' . (int) $id_cart . ' AND id_product = ' . (int) $id_prod . ' AND id_product_attribute = ' . (int) $id_attr);

	if ($oldNote != $note) {
    	$db->execute('
    		INSERT INTO '._DB_PREFIX_.'cart_product_kerawen (id_cart, id_product, id_product_attribute, note)
    		VALUES (
    			'. (int) $id_cart.',
    			'. (int) $id_prod.',
    			'. (int) $id_attr.',
    			"'.pSQL($note).'")
    		ON DUPLICATE KEY UPDATE
    			note = VALUES(note)');
	}
}

function tagCartItem($id_cart, $id_prod, $id_attr, $id_tag)
{
    $db = Db::getInstance();
    $db->execute('
		INSERT INTO '._DB_PREFIX_.'cart_product_kerawen (id_cart, id_product, id_product_attribute, id_tag)
		VALUES (
			'. (int) $id_cart.',
			'. (int) $id_prod.',
			'. (int) $id_attr.',
			'. (int) $id_tag.')
		ON DUPLICATE KEY UPDATE
			id_tag = VALUES(id_tag)');
}


/*
* ------------------------------------------------------------------
* Internals
*/

function ApplyPsRoundPriceMethod($price, $qty) {
	
	if ($qty == 0) {
		return 0;
	}
	
	$round_type = Configuration::get('PS_ROUND_TYPE');
	if (!$round_type) $round_type = _KERAWEN_RM_ITEM_;

	switch ($round_type) {
		case _KERAWEN_RM_TOTAL_:
			$price_qty = $price * (int)$qty;
			break;
		case _KERAWEN_RM_LINE_:
			$price_qty = Tools::ps_round($price * (int)$qty, _PS_PRICE_COMPUTE_PRECISION_);
			break;
		case _KERAWEN_RM_ITEM_:
		default:
			$price_qty = Tools::ps_round($price, _PS_PRICE_COMPUTE_PRECISION_) * (int)$qty;
			break;
	}

	return $price_qty;
	
}


function discountAsArray($cart, $no_payment = false) {

    $context = Context::getContext();
    $db = Db::getInstance();
    
    $price_display_method = isset($context->group) ? (int) $context->group->price_display_method : 0;
    
    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
    
    // Remember and recompute rules, invalid ones will be reinjected
    $rules_before = $cart->getCartRules();
    
    // Update cart rules
    CartRule::autoRemoveFromCart();
    CartRule::autoAddToCart();
    
    // Group restriction by override group exclusive
    $rules_after = $cart->getCartRules();
    $override_group = Configuration::get('KERAWEN_OVERRIDE_GROUP');
    if ($override_group) {
        foreach ($rules_after as $k => $rule) {
            if ($rule['group_restriction']) {
                if ((int) $db->getValue('SELECT id_group FROM '._DB_PREFIX_.'cart_rule_group WHERE id_group = ' . (int) $override_group . ' AND id_cart_rule = '. (int) $rule['id_cart_rule']) == 0) {
                    $cart->removeCartRule($rule['id_cart_rule']);
                    unset($rules_after[$k]);
                }
            }
        }
    }
    
    
    if (!(Module::isInstalled('quantitydiscountpro') && Module::isEnabled('quantitydiscountpro'))) {
        // Reinject manual rules even invalid
        $rules_after = indexArray($rules_after, 'id_cart_rule');
        foreach ($rules_before as $rule) {
            $id_rule = $rule['id_cart_rule'];
            if (!isset($rules_after[$id_rule]) && Tools::strlen($rule['code'])) {
                // Check if rule still exists
                if ($db->getValue('
    				SELECT id_cart_rule FROM '._DB_PREFIX_.'cart_rule
    				WHERE id_cart_rule = '. (int) $id_rule)) {
                    $cart->addCartRule($id_rule);
                }
            }
        }
    }
    
	$total_reduc_ti = 0;
	$total_reduc_te = 0;
	$reducs = array();
	$rules = $cart->getCartRules();
	
	if (Tools::version_compare(_PS_VERSION_, '1.7.6', '>=')) {
	    // Use calculator
	    // Available from PS 1.7.6 for cart rules
	    
	    $calculator = $cart->newCalculator($cart->getProducts(), $rules, $cart->id_carrier);
	    $calculator->processCalculation(_PS_PRICE_COMPUTE_PRECISION_);
	    $res = $calculator->getCartRulesData();
	    
	    foreach ($res as $data) {
	        $rule = $data->getCartRule();
	        
	        // Avoid shipping discount if no shipping!
	        // TODO CHECK IF STILL USEFULL
	        if ($cart->id_carrier == 0
	            && $rule->free_shipping == 1
	            && $rule->reduction_percent == 0
	            && $rule->reduction_amount == 0) continue;
	        if (!$cart->id_carrier) $rule->free_shipping = false;
	        
	        $id_rule = $rule->id;
	        $type = $db->getValue('SELECT type FROM '._DB_PREFIX_.'cart_rule_kerawen WHERE id_cart_rule = '.(int)$id_rule);
	        
	        if ($rule->checkValidity($context, true, false)) {
	            $discount = $data->getDiscountApplied();
	            $value_tax_incl = $discount->getTaxIncluded();
	            $value_tax_excl = $discount->getTaxExcluded();
	        }
	        else {
	            $value_tax_incl = 0;
	            $value_tax_excl = 0;
	        }
	        
	        $total_reduc_ti += $value_tax_incl;
	        $total_reduc_te += $value_tax_excl;
	        
    	    $reducs[] = array(
    	        'id' => $id_rule,
    	        'type' => $type,
    	        'code' => $rule->code,
    	        'name' => $rule->name,
    	        'mode' => $rule->reduction_percent > 0 ? 'percent' : 'amount',
    	        'tax' => $rule->reduction_tax,
    	        'value' => $rule->reduction_percent > 0 ? $rule->reduction_percent : $rule->reduction_amount, //($price_display_method ? $value_tax_excl : $value_tax_incl),
    	        'reduc' => $price_display_method ? $value_tax_excl : $value_tax_incl,
    	        'reduc_tax_excl' => $value_tax_excl,
    	        'reduc_tax_incl' => $value_tax_incl,
    	        'dispMethode' => $price_display_method,
    	        'partial_use' => $rule->partial_use,
	        );
	    }
	}
	else {
	    // Legacy
	    foreach ($rules as $rule) {
	        $id_rule = $rule['id_cart_rule'];
	        $reduc = new CartRule($id_rule);
	        
	        // Avoid shipping discount if no shipping!
	        if ($cart->id_carrier == 0
	            && $rule['free_shipping'] == 1
	            && $rule['reduction_percent'] == 0
	            && $rule['reduction_amount'] == 0) continue;
	        if (!$cart->id_carrier) $reduc->free_shipping = false;
	        
	        $type = $db->getValue('SELECT type FROM '._DB_PREFIX_.'cart_rule_kerawen WHERE id_cart_rule = '.(int)$id_rule);
	        
	        if ($reduc->checkValidity($context, true, false)) {
	            $value_tax_incl = $reduc->getContextualValue(true, $context, CartRule::FILTER_ACTION_ALL_NOCAP);
	            $value_tax_excl = $reduc->getContextualValue(false, $context, CartRule::FILTER_ACTION_ALL_NOCAP);
	        }
	        else {
	            $value_tax_incl = 0;
	            $value_tax_excl = 0;
	        }
	        
	        $total_reduc_ti += $value_tax_incl;
	        $total_reduc_te += $value_tax_excl;
	        
	        $reducs[] = array(
	            'id' => $id_rule,
	            'type' => $type,
	            'code' => $rule['code'],
	            'name' => $rule['name'],
	            'mode' => $reduc->reduction_percent > 0 ? 'percent' : 'amount',
	            'tax' => $reduc->reduction_tax,
	            'value' => $reduc->reduction_percent > 0 ? $reduc->reduction_percent : $reduc->reduction_amount, //($price_display_method ? $value_tax_excl : $value_tax_incl),
	            'reduc' => $price_display_method ? $value_tax_excl : $value_tax_incl,
	            'reduc_tax_excl' => $value_tax_excl,
	            'reduc_tax_incl' => $value_tax_incl,
	            'dispMethode' => $price_display_method,
	            'partial_use' => $rule['partial_use'],
	        );
	    }
	}
	
	return array(
		'reducs' => $reducs,
		'total_ti' => Tools::ps_round($total_reduc_ti, _PS_PRICE_COMPUTE_PRECISION_),
		'total_te' => $total_reduc_te,
	);
}


function cartAsArray($cart, $more_details = false, $is_quote = false)
{
	require_once (_KERAWEN_API_DIR_.'/bean/ListBean.php');
	require_once (_KERAWEN_API_DIR_.'/bean/ProductBean.php');
	require_once (_KERAWEN_API_DIR_.'/bean/VoucherBean.php');
	require_once (_KERAWEN_CLASS_DIR_.'/quote.php');
	require_once (_KERAWEN_CLASS_DIR_.'/catalog.php');
	
	$db = Db::getInstance();
	Product::flushPriceCache();
	
	$disc_cal = array('multi','once');
	
	$res = array();
	$res['id'] = $cart->id;
	$res['shop'] = $cart->id_shop;
	$res['id_lang'] = $cart->id_lang;
	
	//required for quotes
	$res['id_address_delivery'] = (int) $cart->id_address_delivery;
	$res['id_address_invoice'] = (int) $cart->id_address_invoice;
		
	// Customer (required in context)
	// Set customer before rules for correct computing
	$cust = new Customer($cart->id_customer);
	if (!$cust->id) {
		$cart->id_customer = (int) Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER');
		$cust = new Customer($cart->id_customer);
	}
	$is_anonymous = $cust->id == Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER');
	$res['cust'] = array(
		'id' => $is_anonymous ? null : $cust->id,
		'firstname' => $is_anonymous ? null : $cust->firstname,
		'lastname' => $is_anonymous ? null : $cust->lastname,
		'email' => $is_anonymous ? null : $cust->email
	);

	// Setup context for computations
	$context = Context::getContext();
	$context->cart = $cart;
	
	// Extended data
	$more = $db->getRow('
		SELECT cart_kerawen.*, 
		IFNULL(shop_url.id_shop, false) AS id_shop,
		IF(quote_title = "" OR quote_title IS NULL, LPAD(quote_number, 10, "#Q000000000"), quote_title) AS title,
		cart.id_lang,
        cart.id_customer
		FROM '._DB_PREFIX_.'cart_kerawen cart_kerawen 
		LEFT JOIN '._DB_PREFIX_.'cart cart ON cart_kerawen.id_cart = cart.id_cart 
		LEFT JOIN '._DB_PREFIX_.'shop_url shop_url ON cart.id_shop = shop_url.id_shop
		WHERE cart_kerawen.id_cart = '. (int) $cart->id);
	
	if ($more) {
		$res['quote'] = (int) $more['quote'];
		$res['quote_expiry'] = $more['quote_expiry'];
		$res['quote_title'] = $more['title'];
		$res['quote_pdf'] = getQuoteDownloadUri($cart->id, $more['id_shop'], $more['id_lang'], $more['id_customer']);
		$res['id_empl'] = (int) $more['id_employee'];
	}
		
	// Complete delivery data
	// FIXME Optimize by (re)transfering to setDelivery
	$mode = $more ? $more['delivery_mode'] : _KERAWEN_DM_IN_STORE_;
	$carriers = array();
	$carrier = null;
	if ($mode == _KERAWEN_DM_DELIVERY_) {
		$id_address = $more['id_address'];
		if (!$id_address && $cart->id_customer && !$is_anonymous)
			$id_address = $db->getValue('
				SELECT id_address FROM '._DB_PREFIX_.'address
				WHERE id_customer = '. (int) $cart->id_customer.'
				AND deleted = 0');
		if (!$id_address) $id_address = 0;

		setDeliveryAddress($cart, $id_address);
		$dol = $cart->getDeliveryOptionList(null, true);
		
		$best_grade = null;
		foreach ($dol as $id_address => $do)
			if ($id_address == $cart->id_address_delivery)
				foreach ($do as $key => $option)
				{
					$name = null;
					foreach ($option['carrier_list'] as /*$id_carrier =>*/ $carrier)
					{
						$name = $name ? ($name.', ') : '';
						$name .= $carrier['instance']->name;
					}
					
					// Keep order
					//$carriers[$option['position']] = array(
					$carriers[$key] = array(
						'id' => $key,
						'name' => $name,
						'price' => $option['total_price_with_tax'],
						'price_without_tax' => $option['total_price_without_tax'],
					    'date_required' => false,
					);
					
					//Date/time required
					if (Module::isInstalled('deliveree') && Module::isEnabled('deliveree')) {
					    $dc = Configuration::get('DELIVEREE_CARRIERS');
					    if ($key == $dc . ',') {
					        $carriers[$key]['date_required'] = true;
					    }
					}
					
					if ($more_details) {
    					$carriers[$key]['rate'] = (int) $db->getValue('
                            SELECT 
                                t.rate
    					   FROM '._DB_PREFIX_.'carrier_tax_rules_group_shop ctrgs
    					   LEFT JOIN '._DB_PREFIX_.'tax_rule tr ON ctrgs.id_tax_rules_group = tr.id_tax_rules_group AND id_country = (
                                SELECT id_country FROM '._DB_PREFIX_.'address WHERE id_address = ' . (int) $id_address . '
                            )
    					   LEFT JOIN '._DB_PREFIX_.'tax t ON tr.id_tax = t.id_tax
    					   WHERE ctrgs.id_carrier = ' . (int) $key . ' AND ctrgs.id_shop = ' . (int) $cart->id_shop
    					);
					}
					
					if ($option['is_best_grade']) $best_grade = $key;
				}
		$carrier = $more['carrier'];
		if (!$carrier || !isset($carriers[$carrier]))
			$carrier = $best_grade;
	}
	else
		setDeliveryAddress($cart, 0);

	$cart->setDeliveryOption($cart->id_address_delivery && $carrier
		? array($cart->id_address_delivery => $carrier) : null);
	
	// Update original cart
	// Complete if necessary to avoid validation exception
	if (!$cart->id_currency) $cart->id_currency = Configuration::get('PS_CURRENCY_DEFAULT');
	if (!$cart->id_lang) $cart->id_lang = Configuration::get('PS_LANG_DEFAULT');
	// And update context
	$id_lang = $cart->id_lang;
	if (Tools::version_compare(_PS_VERSION_, '1.6.1.0', '<')) $id_lang = null;
	$context->currency = new Currency($cart->id_currency, $id_lang, $cart->id_shop);
	// Finally upddate cart
	$cart->update();

	require_once(dirname(__FILE__).'/address.php');
	$delivery = array(
		'mode' => $mode,
		'date' => $more ? $more['delivery_date'] : null,
	    'note' => $more ? $more['delivery_note'] : null,
	    'private_note' => $more ? $more['private_note'] : null,
	    'public_note' => $more ? $more['public_note'] : null,
		'address' => getAddress((int)$cart->id_address_delivery),
		'carrier' => $carrier,
		'carriers' => $carriers,
	    'errormsg' => false,
	);
	$res['delivery'] = $delivery;
	
	// Compute total price (after delivery has been set)
	$res['count_cart'] = Cart::getNbProducts($cart->id);
	
	// Get additional data about items
	$items = $db->executeS('
		SELECT
			cp.id_product_attribute AS id_product_attribute,
			cp.date_add AS `date`,
			cpk.note AS note,
            cpk.giftcode,
            cpk.id_tag,
            cpk.specific_price_cart_value AS discount_value,
            cpk.specific_price_cart_calc AS discount_calc,
            cpk.specific_price_cart_price AS discount_price,
			p.*
		FROM `'._DB_PREFIX_.'cart_product` cp
		LEFT JOIN '._DB_PREFIX_.'cart_product_kerawen cpk
			ON cpk.id_cart = cp.id_cart AND cpk.id_product = cp.id_product AND cpk.id_product_attribute = cp.id_product_attribute
		LEFT JOIN `'._DB_PREFIX_.'product` p
			ON p.id_product = cp.id_product
		WHERE cp.id_cart = '. (int) $cart->id);
	$more = array();
	foreach ($items as &$i)
		$more[$i['id_product'].'_'.$i['id_product_attribute']] = $i;

	if (Configuration::get('PS_TAX_ADDRESS_TYPE') == 'id_address_invoice') {
		$address_id = (int)$cart->id_address_invoice;
	} else {
		$address_id = (int)$cart->id_address_delivery;
	}
		
	if (!Address::addressExists($address_id)) {
		$address_id = null;
	}

	//Cart with or without tax
	$group = new Group($cust->id_default_group);
	$price_display_method = $is_quote ? true : (int) $group->price_display_method;
	$res['disp_method'] = $price_display_method;
	
	// Clean up customizations
	$customs = $db->executeS('
		SELECT id_customization
		FROM '._DB_PREFIX_.'customization
		WHERE id_cart = '. (int) $cart->id.'
			AND quantity = 0');
	if ($customs) {
    	foreach($customs as $c) {
    		$db->delete('customization', 'id_customization = '. (int) $c['id_customization']);
    		$db->delete('customized_data', 'id_customization = '. (int) $c['id_customization']);
    	}
	}

	$products = array();
	//$pp = $cart->getProducts();
	//Fix hast.fr
	$pp = $cart->getProducts(true);
	$cart_shop_context = Context::getContext()->cloneContext();
	
	$total_wholesale_price = 0;
	$total_rest_voucher = 0;
	
	foreach ($pp as &$p)
	{
		$id_prod = (int)$p['id_product'];
		$index = $id_prod.'_'.$p['id_product_attribute'];

		$specific = SpecificPrice::getSpecificPrice(
			$id_prod,
			$cart->id_shop,
			$cart->id_currency,
			$context->country->id,
			$cust ? Customer::getDefaultGroupId($cust->id) : null,
			$p['cart_quantity'],
			$p['id_product_attribute'],
			$cust ? $cust->id : null,
			$cart->id,
			$p['cart_quantity']); // Should be prod qty vs attr qty ;
		
		$specific_hack = $specific;
			
		// Backward compatibility
		if (!(isset($p['price_without_reduction']))) {
			$p['price_without_reduction'] = Product::getPriceStatic(
				(int)$p['id_product'],
				true,
				isset($p['id_product_attribute']) ? (int)$p['id_product_attribute'] : null,
				6,
				null,
				false,
				false,
				$p['cart_quantity'],
				false,
				(int)$cart->id_customer,
				(int)$cart->id,
				$address_id,
				$specific,
				true,
				true,
				$cart_shop_context
			);
		}
			
		// Backward compatibility
		if (!(isset($p['price_with_reduction_without_tax']))) {
			$p['price_with_reduction_without_tax'] = $p['price'];
		}

		$img = getProductImage($id_prod, $p['id_product_attribute'], $p['link_rewrite']);
		
		$wm = $db->getRow('
			SELECT
				pak.measure AS measure,
				pk.unit as unit,
				pk.`precision` AS `precision`,
                pk.is_gift_card,
                pwck.unit_price
			FROM '._DB_PREFIX_.'product_attribute_kerawen pak
			JOIN '._DB_PREFIX_.'product_attribute pa
				ON pa.id_product_attribute = pak.id_product_attribute
			JOIN '._DB_PREFIX_.'product_wm_kerawen pk
				ON pk.id_product = pa.id_product
            JOIN '._DB_PREFIX_.'product_wm_code_kerawen pwck
				ON pak.id_code = pwck.id_code
			WHERE pak.id_product_attribute = '. (int) $p['id_product_attribute']);
		
		$is_gift_card = 0;
		$is_free_price = 0;
		$rest_voucher = 0;
		$no_tax = 0;
		
		$wmk = $db->getRow('
            SELECT
                is_gift_card,
                is_free_price,
                rest_voucher,
                product_type
            FROM '._DB_PREFIX_.'product_wm_kerawen
            WHERE id_product = '.(int)$p['id_product']);

		if ($wmk) {
		    $is_gift_card = (int)$wmk['is_gift_card'];
		    $is_free_price = (int)$wmk['is_free_price'];
		    $rest_voucher = (int)$wmk['rest_voucher'];
		    $no_tax = in_array($wmk['product_type'], array(
		        _KERAWEN_TYPE_PREPAID_,
		        _KERAWEN_TYPE_GIFTCARD_,
		        _KERAWEN_TYPE_REFUND_,
		    ));
		}

		//UNIT VAT EXCL
		$price_excl = (float)$p['total'];
		$price_excl_init = Product::getPriceStatic(
			(int)$p['id_product'],
			false,
			isset($p['id_product_attribute']) ? (int)$p['id_product_attribute'] : null,
			6,
			null,
			false,
			false,
			$p['cart_quantity'],
			false,
			(int)$cart->id_customer,
			(int)$cart->id,
			$address_id,
			$specific,
			true,
			true,
			$cart_shop_context 
		);
		
		
		//IMPORTANT!!!!!!
		//PS 1.6.15
		$specific = $specific_hack;
		
		//UNIT VAT INCL
		$price_incl = (float)$p['total_wt'];
		$price_incl_init = (float)$p['price_without_reduction'];

		$rate = is_numeric($p['rate']) ? $p['rate'] : 0;
		
		
        //Discount
		$discount_type = $specific ? $specific['reduction_type'] : 'amount'; //default value
		$coef = ($specific && $specific['reduction_type'] == 'percentage') ? 100 : 1;
		$discount_value = $specific ? ((float)$specific['reduction'] * $coef) : null;
		$discount_edit = false;

		if ($specific && (int) $specific['id_cart'] > 0) {
		    if (isset($res['quote']) && (int) $res['quote'] == 1) {
		        if ($specific['reduction'] > 0) {
		            $discount_edit = true;
		        }
		    } else {
		        $discount_edit = true;
		    }
		}
		
	    // Adapt amounts display TI vs TE
	    
		if ($price_display_method) {
			// Tax excluded
			$price = $price_excl;
			$price_init = $price_excl_init;
			if ($discount_type == 'amount' && isset($specific['reduction_tax']) && $specific['reduction_tax']) {
		        $discount_value = $discount_value / (1 + $rate/100);
		    }
		}
		else {
			// Tax included
			$price = (float)$p['total_wt'];
			$price_init = (float)$p['price_without_reduction'];
			if ($discount_type == 'amount' && isset($specific['reduction_tax']) && !$specific['reduction_tax']) {
		        $discount_value = $discount_value * (1 + $rate/100);
		    }
		}

		//FIX PS 1.6.0.6
		//PS compatibility
		if ($specific && !isset($specific['reduction_tax']) && $discount_type == 'amount' && $price_display_method) {
		    $discount_value = (ApplyPsRoundPriceMethod($price_init, $p['cart_quantity']) - $price) / $p['cart_quantity'];
		}

		$total_wholesale_price += (int)$p['cart_quantity'] * $p['wholesale_price'];
		if ($rest_voucher) {
		  $total_rest_voucher += $price;
		}
		
		//PS 1.7.x + 8.x
		if (method_exists('StockAvailable','getLocation')) {
		    $location = StockAvailable::getLocation($id_prod, $p['id_product_attribute']);
		} else {
		    //PS > 1.6.x
		    if ((int) $p['id_product_attribute']) {
		        $combination = new Combination($p['id_product_attribute']);
		        $location = isset($combination->location) ? $combination->location : '';
		    } else {
		        $prod = new Product($id_prod, false);
		        $location = isset($prod->location) ? $prod->location : '';
		    }
		}

		$tmp_product = array(
			'prod' => $id_prod,
			'attr' => $p['id_product_attribute'],
			'name' => $p['name'],
			'version' => isset($p['attributes_small']) ? $p['attributes_small'] : null,
			'reference' => $p['reference'],
		    'location' => $location,
			'qty' => (int)$p['cart_quantity'],
			'unit' => (float)$p['price_wt'],
			'price' => $price,
			'unit_init' => (float)$p['price_without_reduction'],
			'price_init' => ApplyPsRoundPriceMethod($price_init, $p['cart_quantity']),			
			'img' => $img,
		    'date' => isset($more[$index]['date']) ? $more[$index]['date']: '' ,
		    'note' => isset($more[$index]['note']) ? $more[$index]['note'] : '',
		    'id_tag' => isset($more[$index]['id_tag']) ? $more[$index]['id_tag'] : '',
		    'discount_type' => $discount_type,
		    'discount_value' => $discount_value,
		    'discount_init_value' => isset($more[$index]['discount_value']) ? $more[$index]['discount_value'] : '' ,
		    'discount_calc' => isset($more[$index]['discount_calc']) ? $disc_cal[$more[$index]['discount_calc']] : $disc_cal[0],
		    'discount_price' => isset($more[$index]['discount_price']) ? $more[$index]['discount_price'] : '',
		    'discount_edit' => $discount_edit, //TODO: improve
			'specific' => $specific,
			'display_method' => $price_display_method,
			'wm' => $wm && $wm['measure'] ? array(
				'measure' => (float)$wm['measure'],
				'precision' => (int)$wm['precision'],
				'unit' => $wm['unit'],
			    'unit_price' => $wm['unit_price'],
			) : null,
		    'is_gift_card' => $is_gift_card,
		    'is_free_price' => $is_free_price,
		    'rest_voucher' => $rest_voucher,
		    'no_tax' => $no_tax,
		    'giftcode' => isset($more[$index]['giftcode']) ? $more[$index]['giftcode'] : '',
		    'rate' => $rate,
			'unit_price_init' => $price_init,
			'minimal_quantity' => $p['minimal_quantity'],
		    'ean13' => $p['ean13'],
		);

		//Quotation display
		if ($more_details) {
			//unit by product
			$tmp_product['unit_init_tax_incl'] = $price_incl_init;
			$tmp_product['unit_init_tax_excl'] = $price_excl_init;
			
			//total by product
			$tmp_product['price_init_tax_incl'] = ApplyPsRoundPriceMethod($price_incl_init, $p['cart_quantity']);
			$tmp_product['price_init_tax_excl'] = ApplyPsRoundPriceMethod($price_excl_init, $p['cart_quantity']);
			$tmp_product['price_tax_incl'] = $price_incl;
			$tmp_product['price_tax_excl'] = $price_excl;
		
			$tmp_product['unit_ecotax'] = $p['ecotax'];
			$tmp_product['total_ecotax'] = (float) $p['ecotax'] * (float)$p['cart_quantity'];
			$tmp_product['tax_name'] = $p['tax_name'];

			if (isset($cover) && $cover) {
			    $tmp_product['image_id'] = $cover['id_image'];
			    $tmp_product['image_path'] = $img;
			}
		} 
		$tmp_product['vat_margin'] = isset($more[$index]['vat_margin']) ? $more[$index]['vat_margin'] : 0;
		$tmp_product['wholesale_price'] = (float)$p['wholesale_price'];

		// Customizations
		$custom = $db->executeS('
			SELECT id_customization, quantity
			FROM '._DB_PREFIX_.'customization
			WHERE id_cart='. (int) $cart->id.'
			AND id_product='. (int) $id_prod.'
			AND id_product_attribute='. (int) $p['id_product_attribute'].'
			AND in_cart > 0
			ORDER BY id_customization');
		foreach($custom as &$c) {
			$c['fields'] = $db->executeS('
				SELECT d.index, f.name, d.value
				FROM '._DB_PREFIX_.'customized_data d
				JOIN '._DB_PREFIX_.'customization_field_lang f
					ON f.id_customization_field = d.index
					AND f.id_lang = '. (int) $cart->id_lang.'
					AND f.id_shop = '. (int) $cart->id_shop.'
				WHERE d.id_customization='. (int) $c['id_customization'].'
				ORDER BY d.index');
		}
		$tmp_product['custom'] = $custom;
		
		$products[] = $tmp_product;
	}
	
	usort(
		$products, 
		function($a,$b) { 
			return strtotime($a['date']) - strtotime($b['date']);
		} 
	);
	
	$res['products'] = $products;

	$res['returns'] = getReturns($cart->id, $price_display_method);
	$count_ret = 0;
	$total_ret = 0;
	$total_ret_te = 0;
	foreach ($res['returns'] as &$value) {
		$count_ret += $value['qty'];
		$total_ret += $value['price_tax_incl'];
		$total_ret_te += $value['price_tax_excl'];
	}
	$res['count_ret'] = $count_ret;
	$res['total_ret'] = $total_ret;
	$res['total_ret_te'] = $total_ret_te;
	
	$total_ti = $cart->getOrderTotal(true, Cart::ONLY_PRODUCTS) + $cart->getOrderTotal(true, Cart::ONLY_WRAPPING);
	$total_te = $cart->getOrderTotal(false, Cart::ONLY_PRODUCTS) + $cart->getOrderTotal(false, Cart::ONLY_WRAPPING);
	if ($mode == _KERAWEN_DM_DELIVERY_ && $delivery['carrier']) {
		$total_ti += $cart->getOrderTotal(true, Cart::ONLY_SHIPPING);
		$total_te += $cart->getOrderTotal(false, Cart::ONLY_SHIPPING);
	}
	
	$reducs = discountAsArray($cart);
	$res['reducs'] = $reducs['reducs'];
	$total_ti -= $reducs['total_ti'];
	$total_te -= $reducs['total_te'];
	if ($total_ti < 0) $total_ti = 0;
	if ($total_te < 0) $total_te = 0;
	
	$res['total_cart'] = $total_ti;
	$res['total_cart_te']= $total_te;
	$res['total_wholesale_price'] = $total_wholesale_price;
	$res['total_rest_voucher'] = $total_rest_voucher;
	
	// Restore customer
	$cart->id_customer = $cust->id;
	$cart->save();
	
	updateCart($res, isset($cart->kerawen_version) ? $cart->kerawen_version : null);
	return $res;
}

function computeCartPrices($cart) {
	$db = Db::getInstance();
	$context = Context::getContext();
	
	// Remove cart discounts first (avoid cache issue)
	$db->delete('specific_price', 'id_cart = '. (int) $cart->id);
	Product::flushPriceCache();
	
	require_once(_KERAWEN_CLASS_DIR_.'/data.php');
	$address_delivery = $cart->id_address_delivery ? new Address ($cart->id_address_delivery) : getDefaultDeliveryAddress(true);
	
	$discounted = $db->executeS('
		SELECT
			cpk.*,
			cp.quantity
		FROM '._DB_PREFIX_.'cart_product_kerawen cpk
		JOIN '._DB_PREFIX_.'cart_product cp ON
			cp.id_cart = cpk.id_cart
			AND cp.id_product = cpk.id_product
			AND cp.id_product_attribute = cpk.id_product_attribute
		WHERE cpk.id_cart = '. (int) $cart->id.'
		AND cpk.price_type IS NOT NULL');
	
	$id_group = $context->group->id;
	foreach($discounted as $discount) {
		$tax_manager = TaxManagerFactory::getManager($address_delivery, Product::getIdTaxRulesGroupByIdProduct((int)$discount['id_product'], $context));
		$tax_calculator = $tax_manager->getTaxCalculator();
	
		$specific = false;
		$init = Product::getPriceStatic(
			$discount['id_product'],
			false,
			$discount['id_product_attribute'],
			6,
			null,
			false,
			true,
			1,
			false,
			$cart->id_customer,
			null,
			$cart->id_address_delivery,
			$specific,
			true,
			true,
			null,
			true);
	
		if ($discount['price_type'] == _KERAWEN_PA_REPLACE_) {
			$price = $tax_calculator->removeTaxes((float)$discount['price_value']);
		}
		else if ($discount['price_type'] == _KERAWEN_PA_AMOUNT_) {
			$price = $init - $tax_calculator->removeTaxes((float)$discount['price_value']);
			if ($price < 0) $price = 0;
		}
		else if ($discount['price_type'] == _KERAWEN_PA_PERCENT_) {
			$price = $init*(1 - (float)$discount['price_value']/100);
		}
		else {
			$price = $init;
		}
	
		if ($discount['price_qty']) {
			// Apply to one item only
			$qty = (float)$discount['quantity'];
			$price = ($init*($qty - 1) + $price)/$qty;
		}
	
		$group_reduction = GroupReduction::getValueForProduct($discount['id_product'], $id_group);
		if ($group_reduction === false)
			$group_reduction = Group::getReductionByIdGroup($id_group)/100;
		$price = $price/(1 - (float)$group_reduction);
	
		$specificPrice = new SpecificPrice();
		$specificPrice->id_shop = $cart->id_shop;
		$specificPrice->id_cart = $cart->id;
		$specificPrice->id_product = $discount['id_product'];
		$specificPrice->id_product_attribute = $discount['id_product_attribute'];
		$specificPrice->from_quantity = $discount['quantity'];
		$specificPrice->id_shop_group = 0;
		$specificPrice->id_currency = 0;
		$specificPrice->id_country = 0;
		$specificPrice->id_group = 0;
		$specificPrice->id_customer = 0;
		$specificPrice->from = '0000-00-00 00:00:00';
		$specificPrice->to = '0000-00-00 00:00:00';
		$specificPrice->price = round($price, 6);
		$specificPrice->reduction_type = 'amount';
		$specificPrice->reduction = 0;
		$specificPrice->save();
	}
}


function newCart($context, $id_cust)
{
	$c = new Cart();
	$c->id_shop = $context->shop->id;
	$c->id_customer = $id_cust;
	$c->id_lang = $context->language->id;
	$c->id_currency = $context->currency->id;
	$c->save();

	return $c;
}

function getCart($id_cart, $id_cust, $context, $save = true)
{
    $c = new Cart($id_cart);
    if (!$id_cart) {
        $c->id_shop = $context->shop->id;
        $c->id_customer = $id_cust;
        $c->id_lang = $context->language->id;
        $c->id_currency = $context->currency->id;
        if ($save) $c->save();
    }
    
    require_once(_KERAWEN_CLASS_DIR_.'/data.php');
    $context->customer = $id_cust ? new Customer($id_cust) : getAnonymousCustomer();
	
    return $c;
}

function getProductsByDate($c, $id_lang)
{
	// KPOS Cart specific context
	$address_id = null;

	$sql = 'SELECT
					cp.`id_shop`,
					cp.`id_product`,
					cp.`id_product_attribute`,
					cp.`quantity` AS `cart_quantity`,
					cp.`date_add`,
					pl.`name`
				FROM
					'._DB_PREFIX_.'cart_product cp,
					'._DB_PREFIX_.'product  p,
					'._DB_PREFIX_.'product_lang pl
				WHERE
					cp.`id_cart` = '.(int)$c->id.'
					AND p.`id_product` = cp.`id_product`
					AND pl.`id_product` = p.`id_product`
					AND pl.`id_lang` = '.(int)$id_lang.'
				ORDER BY
					cp.date_add ASC';

	$result = Db::getInstance()->executeS($sql);

	$cart_shop_context = Context::getContext()->cloneContext();
	foreach ($result as &$row)
	{
		if ($cart_shop_context->shop->id != $row['id_shop'])
			$cart_shop_context->shop = new Shop((int)$row['id_shop']);

		//Fix bug "Fatal error: Only variables can be passed by reference" php7
		$null = null;	
			
		$row['price_wt'] = Product::getPriceStatic((int)$row['id_product'],
				true,
				(int)$row['id_product_attribute'],
				2,
				null,
				false,
				true,
				$row['cart_quantity'],
				false,
				$c->id_customer ? $c->id_customer : null,
				$c->id,
				$address_id,
				$null,
				true,
				true,
				$cart_shop_context);
		$row['price_wt'] = Tools::ps_round($row['price_wt'], _PS_PRICE_COMPUTE_PRECISION_);
		$row['total_wt'] = $row['price_wt'] * (int)$row['cart_quantity'];
	}

	return $result;
}


function getReturns($id_cart, $price_display_method = 0)
{
	if (!$id_cart) return array();
	
	$refund_tax = ($price_display_method) ? 'excl' : 'incl';
	$db = Db::getInstance();
	$sql = 'SELECT
				rk.`id_return` AS id,
				rk.`id_cart` AS id_cart,
				rk.`id_order` AS id_order,
				rk.`id_order_detail` AS id_order_detail,
				rk.`quantity` AS qty,
				-rk.`refund_tax_' . $refund_tax . '` AS price,
				-rk.`refund_tax_excl` AS price_tax_excl,
				-rk.`refund_tax_incl` AS price_tax_incl,
				od.product_name AS name
			FROM `'._DB_PREFIX_.'return_kerawen` rk
			LEFT JOIN `'._DB_PREFIX_.'order_detail` od ON rk.id_order_detail = od.id_order_detail
			WHERE `id_cart` = '. (int) $id_cart;
	return $db->executeS($sql);
}


function addPaymentReceived($params) {
    $db = Db::getInstance();
    $payments = $params->payments;
    foreach($payments as &$pay) {
        $prc = array(
            'id_cart' => (int)$params->id_cart,
            'mode' => (int)$pay->mode,
            'label' => pSQL($pay->label),
            'paid' =>  pSQL($pay->paid),
            'credit' => (int)$pay->credit,
            'id_rule' => isset($pay->id_rule) ? (int)$pay->id_rule : null,
            'date' => isset($pay->date) ? pSQL($pay->date) : null,
            'remain' => isset($pay->remain) ? 1 : null,
            'id_receipt'  => isset($pay->id_receipt) ? (int) $pay->id_receipt : null,
        );
        $db->insert('payment_received_kerawen', $prc, true);
    }
}

function deletePaymentReceived($id_cart, $id_prc) {
    $db = Db::getInstance();
    
    if((int) $id_prc > 0) {
        $condition = 'id_prk = '. (int) $id_prc;
    } else {
        $condition = 'id_cart = '. (int) $id_cart;
    }
    
    $db->delete('payment_received_kerawen', $condition);
}

function getPaymentReceived($id_cart) {
    $db = Db::getInstance();
    return $db->executeS('
		SELECT *
		FROM '._DB_PREFIX_.'payment_received_kerawen
		WHERE id_cart = ' . (int) $id_cart);
}

function newTpeReceipt($params) {
    $data = array(
        'id_cart' => (int) $params->id_cart,
        'type' =>  pSQL($params->type),
        'status' => pSQL($params->status),
        'device' => pSQL($params->device)
    );
    $db = Db::getInstance();
    $db->insert('tpe_receipt_kerawen', $data);
    return (int) $db->Insert_ID();
}

function saveTpeReceipt($params) {

    $data = array();
    $data['message'] = pSQL($params->message);
    if (isset($params->status) && $params->status != '') {
        $data['status'] = pSQL($params->status);
    }

    $db = Db::getInstance();
    $db->update('tpe_receipt_kerawen', $data, 'id_receipt = ' . (int) $params->id_receipt);
    return (int) $params->id_receipt;
}

function getTpeReceipt($id_receipts) {
    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
    return Db::getInstance()->executeS('
        SELECT 
            IFNULL(message, "") AS message
        FROM '._DB_PREFIX_.'tpe_receipt_kerawen 
        WHERE id_receipt IN (' . dataListToIntList($id_receipts) . ')'
    );
}



