<?php
/**
* 2014 KerAwen
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2014 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/

function generateRuleCode()
{
	do
		$code = Tools::strtoupper(Tools::passwdGen(8, 'ALPHANUMERIC'));
	while (CartRule::cartRuleExists($code));
	return $code;
}

function getReceiptCartRules($context, $params, &$response) {

    require_once(_KERAWEN_CLASS_DIR_.'/data.php');
    
    $id_shop = $context->shop->id;
    $id_lang = $context->language->id;

    $cartRules = Db::getInstance()->executeS('
        SELECT
            crl.name AS title,
            cr.code, 
            cr.id_cart_rule, 
            cr.date_from, 
            cr.date_to, 
            cr.minimum_amount, 
            cr.reduction_amount, 
            cr.reduction_currency, 
            cr.reduction_percent, 
            cr.reduction_tax,
            cr.free_shipping,
            cr.description
        FROM '._DB_PREFIX_.'cart_rule cr
        LEFT JOIN '._DB_PREFIX_.'cart_rule_lang crl ON cr.id_cart_rule = crl.id_cart_rule  AND id_lang = '.(int)$id_lang.'
        LEFT JOIN '._DB_PREFIX_.'cart_rule_kerawen crk ON cr.id_cart_rule = crk.id_cart_rule
        WHERE cr.active = 1 AND cr.code != "" AND crk.is_voucher = 1
        AND ( 
               ( NOW() BETWEEN crk.display_from AND crk.display_to ) 
            OR ( ISNULL(crk.display_from) AND ISNULL(crk.display_to) )
            OR ( ISNULL(crk.display_from) AND NOW() < crk.display_to )
            OR ( crk.display_from < NOW() AND ISNULL(crk.display_to) )
        )
        ORDER BY cr.id_cart_rule ASC
    ');
    

    $response->addResult('receiptCartRules', $cartRules);
}


function getReceiptRewards($ref) {
    
    require_once(_KERAWEN_CLASS_DIR_.'/data.php');
    $context = Context::getContext();

    return Db::getInstance()->executeS('
        SELECT
            crl.name AS title,
            cr.code,
            cr.id_cart_rule,
            cr.date_from,
            cr.date_to,
            cr.minimum_amount,
            cr.reduction_amount,
            cr.reduction_currency,
            cr.reduction_percent,
            cr.reduction_tax,
            cr.description

        FROM '._DB_PREFIX_.'cart_rule_kerawen crk
        INNER JOIN '._DB_PREFIX_.'cart_rule cr ON cr.id_cart_rule = crk.id_cart_rule
        LEFT JOIN '._DB_PREFIX_.'cart_rule_lang crl ON cr.id_cart_rule = crl.id_cart_rule AND crl.id_lang = ' . (int) $context->language->id . '

        WHERE crk.order_ref_src = "' . pSQL($ref) . '"
        AND crk.type = "' . pSQL(_KERAWEN_CR_REWARD_) . '"
        AND cr.active = 1 
        AND cr.quantity = 1 /* Maybe not */
        AND cr.code != ""
    ');
}


function getCartRules($id_cust, $id_lang, $id_shop)
{
    $now = date('Y-m-d H:i:s');
    if (!$id_cust) $id_cust = (int)Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER');
    $anonymous = $id_cust == (int)Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER');
    $groups = Customer::getGroupsStatic($id_cust);
    if (empty($groups)) $groups = array(0);
    $generic = (bool) Configuration::get('KERAWEN_CARTRULE_INCL_GENERIC');
    
	// Loyalty
	require_once(dirname(__FILE__).'/loyalty.php');
	if (!$anonymous && ($loyalty = KerawenLoyalty::getPlugin()))
		$loyalty = $loyalty->getRules($id_cust);
	else
		$loyalty = array(
			'loyalty' => array(),
			'referral' => array(),
			'sponsor' => array(),
		);
	
	// Get active rules for customer or everybody	
	$additional_filter = Configuration::get('KERAWEN_CARTRULE_FILTER');
	if (!$additional_filter) $additional_filter = 'TRUE';
		

	$order_by = (bool) Configuration::get('KERAWEN_CARTRULE_ORDER_BY_DATE') ? 'cr.date_upd DESC' : 'cr.priority DESC, crl.name';
	
	$buf = Db::getInstance()->executeS('
        SELECT
            cr.id_cart_rule AS id,
            cr.code,
            crl.name,
			cr.date_from AS `from`,
			cr.date_to AS `to`,
		    cr.id_customer,
            cr.group_restriction,
            cr.quantity_per_user - LEAST(cr.quantity_per_user, IFNULL(cru.used, 0)) AS qty,
            cr.reduction_percent,
            cr.reduction_amount,
            IF(cr.reduction_percent > 0, "percent", "amount") AS mode,
            IF(cr.reduction_percent > 0, cr.reduction_percent, cr.reduction_amount) AS value,
            cr.reduction_tax AS tax,
            crk.type,
			0 AS points
            
        FROM '._DB_PREFIX_.'cart_rule cr
        LEFT JOIN '._DB_PREFIX_.'cart_rule_lang crl ON crl.id_cart_rule = cr.id_cart_rule AND crl.id_lang = '. (int) $id_lang.'
        LEFT JOIN '._DB_PREFIX_.'cart_rule_kerawen crk ON crk.id_cart_rule = cr.id_cart_rule

        LEFT JOIN (
            SELECT
                ocr.id_cart_rule,
                COUNT(o.id_order) AS used
            FROM '._DB_PREFIX_.'order_cart_rule ocr
            JOIN '._DB_PREFIX_.'orders o ON o.id_order = ocr.id_order AND o.id_customer = '. (int) $id_cust.'
            GROUP BY ocr.id_cart_rule
        ) cru ON cru.id_cart_rule = cr.id_cart_rule

        LEFT JOIN (
            SELECT DISTINCT id_cart_rule
            FROM '._DB_PREFIX_.'cart_rule_group
            WHERE id_group IN ('.implode(',', array_map('intval', $groups)).')
        ) crg ON crg.id_cart_rule = cr.id_cart_rule

        LEFT JOIN (
            SELECT DISTINCT id_cart_rule
            FROM '._DB_PREFIX_.'cart_rule_shop
            WHERE id_shop = '.(int)$id_shop.'
        ) crs ON crs.id_cart_rule = cr.id_cart_rule

        WHERE
            '.$additional_filter.'
            AND cr.active = 1
            AND cr.quantity > 0
            AND CHAR_LENGTH(cr.code) > 0
            AND (crk.id_cart IS NULL OR crk.id_cart = 0)
            AND (crk.type IS NULL OR crk.type != "' . pSQL(_KERAWEN_CR_GIFT_CARD_) . '" OR cr.id_customer > 0)
            AND (crk.type IS NULL OR crk.type != "' . pSQL(_KERAWEN_CR_REWARD_) . '" OR cr.id_customer > 0)
            AND cr.date_from <= "'.pSQL($now).'"
            AND cr.date_to >= "'.pSQL($now).'"
            AND (cr.group_restriction = 0 OR crg.id_cart_rule IS NOT NULL)
            AND (cr.shop_restriction = 0 OR crs.id_cart_rule IS NOT NULL)
            AND (
                '.(!$anonymous ? 'cr.id_customer = '. (int) $id_cust : 'FALSE').'
                OR '.($generic ? 'cr.id_customer = 0' : 'FALSE').')
        HAVING qty > 0
        ORDER BY '. $order_by);
	
	$rules = array();
	foreach ($buf as &$rule) {
	    $id_rule = $rule['id'];
	    
		// Recompute type
		$type = '';
		if (in_array($id_rule, $loyalty['loyalty'])) {
			$type = _KERAWEN_CR_LOYALTY_;
		}
		else if (in_array($id_rule, $loyalty['referral'])) {
			$type = _KERAWEN_CR_REFERRAL_;
		}
		else if (in_array($id_rule, $loyalty['sponsor'])) {
			$type = _KERAWEN_CR_REFERRAL_;
		}
		else if ($rule['type'] == _KERAWEN_CR_CREDIT_) {
			$type = _KERAWEN_CR_CREDIT_;
		}
		else if ($rule['group_restriction']) {
			$type = _KERAWEN_CR_GROUP_;
		}
		
		// Exclude anonymous individual & special rules
		if ($rule['id_customer'] == 0 && $type != '' && $type != _KERAWEN_CR_GROUP_) {
		    continue;
		}
		
		$rule['type'] = $type;
		if ($type == _KERAWEN_CR_LOYALTY_) {
			if ((float)Configuration::get('PS_LOYALTY_POINT_VALUE') > 0) {
			    $rule['points'] = intval((float)$rule['reduction_amount']/Configuration::get('PS_LOYALTY_POINT_VALUE'));
			}
		}
		$rules[] = $rule;
	}
	return $rules;
}


function getKerawenCartRule($id_cart_rule) {
    
    $context = Context::getContext();
    
    $rule = new CartRule((int) $id_cart_rule);
    if (!$rule) {
        return false;
    }
    
    return array(
        'id' => $id_cart_rule,
        'code' => $rule->code,
        'name' => $rule->name[$context->language->id],
        'qty' => $rule->quantity_per_user,
        'mode' => $rule->reduction_amount > 0 ? 'amount' : 'percent',
        'value' => $rule->reduction_amount > 0 ? $rule->reduction_amount : $rule->reduction_percent,
        'reduction_tax' => (int) $rule->reduction_tax,
        'from' => $rule->date_from,
        'to' => $rule->date_to,
        'cust' => null,
    );

}



function setKerawenCartRule($id, $code, $id_cust, $reduction_currency, $reduction_percent, $reduction_amount, $reduction_tax, $partial_use, $highlight, $priority, $quantity, $quantity_per_user, $active, $date_from, $date_to, $name, $description, $cart_rule_type, $id_cart, $id_product, $id_attribute, $id_order, $is_voucher, $display_from, $display_to, $minimum_amount, $minimum_amount_tax) {

    $db = Db::getInstance();
    
    $context = Context::getContext();
    if ($id_cust == Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER')) $id_cust = null;
    
    $rule = new CartRule($id);
    if ($code) {
        $rule->code = $code;
    }
    $rule->id_customer = $id_cust;
    $rule->reduction_currency = $reduction_currency;
    $rule->reduction_percent = $reduction_percent;
    $rule->reduction_amount = $reduction_amount;
    $rule->reduction_tax = $reduction_tax;
    $rule->partial_use = $partial_use;
    $rule->highlight = $highlight;
    $rule->priority = $priority;
    $rule->quantity = $quantity;
    $rule->quantity_per_user = $quantity_per_user;
    $rule->active = $active;
    $rule->date_from = $date_from;
    $rule->date_to = $date_to;
    $rule->name = getForLanguages($name);
    $rule->description = $description;
    
    $rule->minimum_amount = $minimum_amount;
    $rule->minimum_amount_tax = $minimum_amount_tax;
    
    $rule->save();
    

    //Kerawen spec
    $sent = $cart_rule_type == _KERAWEN_CR_GIFT_CARD_ ? '0' : 'NULL';
    $sql = '
        INSERT INTO `'._DB_PREFIX_.'cart_rule_kerawen`
		(`id_cart_rule`, `type`,`id_cart`,`id_product`,`id_attribute`,`id_order`,`is_voucher`,`display_from`,`display_to`,`sent`)
		VALUES (
            '. (int) $rule->id.',
            \''. pSQL($cart_rule_type) .'\',
            ' . (int) $id_cart . ',
            ' . (int) $id_product . ',
            ' . (int) $id_attribute . ',
            ' . (int) $id_order . ',
            ' . (int) $is_voucher . ',
            ' . $display_from . ',
            ' . $display_to . ', 
            ' . $sent . ' 
        ) ON DUPLICATE KEY UPDATE
            `display_from` = VALUES(`display_from`),
            `display_to` = VALUES(`display_to`)
        ';    
    $db->execute($sql);

    return $rule;
}


function createCredit($value, $id_cust, $cart_rule_type = _KERAWEN_CR_CREDIT_, $id_cart = 'null', $id_product = 'null', $id_attribute = 'null', $id_order = 'null', $label = '', $code = false, $type = 0, $duration = 0)
{
	$db = Db::getInstance();
	
	$context = Context::getContext();

	if ($id_cust == Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER')) $id_cust = null;

	switch($cart_rule_type) {
	    case _KERAWEN_CR_GIFT_CARD_:
	        $name = $context->module->l('Gift card', pathinfo(__FILE__, PATHINFO_FILENAME));
	        $partial_use = true;
	        $active = 0;
	        $period = (int) Configuration::get('KERAWEN_GIFT_CARD_DURATION');
	        break;
	    case _KERAWEN_CR_VOUCHER_:
	        $name = Configuration::get('KERAWEN_VOUCHER_LABEL');;
	        $partial_use = true;
	        $active = 1;
	        $period = (int) Configuration::get('KERAWEN_VOUCHER_DURATION');
	        break;
	    //case _KERAWEN_CR_CREDIT_:
	    default :
	        $name = ($label == '') ? $context->module->l('Credit', pathinfo(__FILE__, PATHINFO_FILENAME)) : $label;
	        $partial_use = (boolean) Configuration::get('KERAWEN_CREDIT_PARTIAL_USE');
	        $active = 1;
	        $period = (int) Configuration::get('KERAWEN_DISCOUNT_DURATION');
	        break;
	}

	$period = ($period > 0) ? $period : 365;
	$period = ((int) $duration > 0) ? (int) $duration : $period;
	
	$date_from = date('Y-m-d H:i:s', time() - 1);
	$date_to = date('Y-m-d 23:59:59', strtotime($date_from.' +' . $period . ' days'));
	

	$highlight = $id_cust != null;
	$priority = Configuration::get('KERAWEN_CREDIT_PRIORITY');

	
	if ($type) {
	    $reduction_percent = $value;
	    $reduction_amount = 0;
	} else {
	    $reduction_percent = 0;
	    $reduction_amount = $value;
	}


	$rule = setKerawenCartRule(
	    null, 
	    ($code === false || $code == '') ? generateRuleCode() : $code,
	    $id_cust,
	    $context->currency->id,
	    $reduction_percent,
	    $reduction_amount,
	    true, 
	    $partial_use,
	    $highlight, 
	    $priority, 
	    1, 
	    1, 
	    $active, 
	    $date_from, 
	    $date_to, 
	    $name,
	    '',
	    $cart_rule_type, 
	    $id_cart, 
	    $id_product, 
	    $id_attribute, 
	    $id_order, 
	    0, 
	    'NULL', 
	    'NULL',
	    0,
	    1
	);
	$id_rule = $rule->id;

	// Return the new rule...
	$cust = $id_cust ? new Customer($id_cust) : null;
	return array(
		'id' => $id_rule,
		'code' => $rule->code,
		'name' => $rule->name[$context->language->id],
		'qty' => $rule->quantity_per_user,
	    'mode' => $rule->reduction_amount > 0 ? 'amount' : 'percent',
	    'value' => $rule->reduction_amount > 0 ? $rule->reduction_amount : $rule->reduction_percent,
		'reduction_tax' => (int) $rule->reduction_tax,
		'from' => $rule->date_from,
		'to' => $rule->date_to,
		'cust' => $id_cust ? array(
			'id' => $cust->id,
			'firstname' => $cust->firstname,
			'lastname' => $cust->lastname,
		) : null,
	);
}

function applySpecialOffer($context, $params, &$response)
{
	$all_res = array();

	for ($i = 1; $i <= $params->nb ; $i++) {
    
	// Ensure rules are active
	Configuration::updateGlobalValue('PS_CART_RULE_FEATURE_ACTIVE', '1');
	
	$value      = $params->value ;
	$percent    = $params->percent ;
	$id_product = $params->id_product ;
	$cust    = $params->cust ;

	$rule = new CartRule() ;
	$rule->code = generateRuleCode() ;
	if ($percent)
		$rule->reduction_percent = $value;
	else
	{
		$rule->reduction_currency = $context->currency->id;
		$rule->reduction_amount = $value;
	}
	$rule->partial_use = false ;
	$rule->reduction_tax = true ;
	$rule->quantity = 1 ;
	$rule->quantity_per_user = 1 ;
	$rule->product_restriction = 1 ;	  

	//$rule->date_from = date('Y-m-d H:i:s', time() - 1);
	$rule->date_from = date($params->from.' H:i:s', time() - 1);
	 
	//$rule->date_to = date('Y-m-d H:i:s', strtotime($rule->date_from.' +1 year'));
	$rule->date_to = date($params->to.' H:i:s', time() );



	$languages = Language::getLanguages(true);
	foreach ($languages as $l)
		$rule->name[(int)$l['id_lang']] = $context->module->l('Special Offer', pathinfo(__FILE__, PATHINFO_FILENAME));

	$rule->add();
	
	$id_rule = $rule->id;
	$db = Db::getInstance();
		
	$sql = 'INSERT INTO `'._DB_PREFIX_.'cart_rule_product_rule_group`
				(`id_product_rule_group`, `id_cart_rule`, `quantity`)
				VALUES ("",'. (int) $id_rule .',1)';
	$db->execute($sql);

	$id_product_rule_group = $db->Insert_ID();
	 
	$sql = 'INSERT INTO `'._DB_PREFIX_.'cart_rule_product_rule`
				(`id_product_rule`, `id_product_rule_group`, `type`)
				VALUES ("",'. (int) $id_product_rule_group .',"products")';
	$db->execute($sql);

	$id_product_rule = $db->Insert_ID();

	$sql = 'INSERT INTO `'._DB_PREFIX_.'cart_rule_product_rule_value`
				(`id_product_rule`, `id_item`)
				VALUES ('. (int) $id_product_rule .','. (int) $id_product .')';
	$db->execute($sql);

// Return the new rule...
	//  $cust = $id_cust ? new Customer($id_cust) : null;
	$product = $id_product ? new Product($id_product) : null;
 	 

	array_push($all_res, array(
		'id' => $id_rule,
		'code' => $rule->code,
		'name' => $rule->name[$context->language->id],
		'qty' => $rule->quantity_per_user,
		'mode' => 'amount',
		'value' => $rule->reduction_amount,
		'from' => $rule->date_from,
		'to' => $rule->date_to,
		'cust' => $cust ? array(
			'id' => $cust->id,
			'firstname' => $cust->firstname,
			'lastname' => $cust->lastname,
		) : null ,
		'product' => $product->name[1] 
	));
} // fin for...
	return $all_res;
}



function setGiftCard($id_order, $action)
{
    require_once(_KERAWEN_CLASS_DIR_.'/giftcard.php');
    require_once(_KERAWEN_CLASS_DIR_.'/data.php');
    
	$db = Db::getInstance();
	$q = '
		SELECT
			orders.id_cart,
			order_kerawen.gift_card_flag,
			order_kerawen.id_till,
			order_kerawen.id_employee,
			orders.id_shop,
			order_kerawen.is_paid,
            
            orders.id_order, 
            orders.id_lang,
            orders.id_customer,
            c.firstname, 
            c.lastname, 
            c.email

		FROM '._DB_PREFIX_.'orders orders
		INNER JOIN '._DB_PREFIX_.'order_kerawen order_kerawen 
            ON orders.id_order = order_kerawen.id_order

        INNER JOIN '._DB_PREFIX_.'customer c 
            ON c.id_customer = orders.id_customer

		WHERE orders.id_order = ' . (int) $id_order;

	if ($order = $db->getRow($q)) {
	    $id_cart = (int) $order['id_cart'];
	    $gift_card_flag = (int) $order['gift_card_flag'];
	    $id_till = (int) $order['id_till'];
	    $id_employee = (int) $order['id_employee'];
	    $id_shop = (int) $order['id_shop'];
	    $is_paid = (int) $order['is_paid'];
		
		if (!$gift_card_flag && $action == 'createCredit') {

			$q = '
			SELECT
				order_detail.product_id,
				order_detail.product_attribute_id,
				order_detail.unit_price_tax_incl,
				order_detail.product_quantity
			FROM '._DB_PREFIX_.'order_detail order_detail
			INNER JOIN '._DB_PREFIX_.'product_wm_kerawen product_wm_kerawen 
				ON order_detail.product_id = product_wm_kerawen.id_product
				AND product_wm_kerawen.is_gift_card = 1
				AND id_order = ' . (int) $id_order;
			
			$buf = $db->executeS($q);
			if ($buf) {
				foreach ($buf as $row) {
					for ($i = 1; $i <= intval($row['product_quantity']); $i++) {
						//get giftcode
						$code = false;
					    if ($row['product_quantity'] == 1) {
							$code = $db->getValue('SELECT giftcode FROM '._DB_PREFIX_.'cart_product_kerawen WHERE id_cart = ' . (int) $id_cart . ' AND id_product = ' . (int) $row['product_id'] . ' AND id_product_attribute = ' . (int) $row['product_attribute_id'] . ' AND giftcode IS NOT NULL');
					    }

						$data_rules = createCredit(
							(float) $row['unit_price_tax_incl'], 
							0, 
							_KERAWEN_CR_GIFT_CARD_, 
							(int) $id_cart, 
							(int) $row['product_id'], 
							(int) $row['product_attribute_id'], 
							(int) $id_order,
						    '', 
						    $code
						);

					}
				}
			}

			//Change flag status (create gift card only once)
			$db->update('order_kerawen', array('gift_card_flag' => 1), 'id_order = ' . (int) $id_order);
		}
	}
}

function updateGiftCard($order_reference) {

    require_once(_KERAWEN_CLASS_DIR_.'/data.php');
	$db = Db::getInstance();

	// Get payment status
	$due = $db->getValue('
		SELECT SUM(total_paid_tax_incl)
		FROM '._DB_PREFIX_.'orders
		WHERE reference = "'.pSQL($order_reference).'"');
	$paid = $db->getValue('
		SELECT SUM(amount)
		FROM '._DB_PREFIX_.'order_payment
		WHERE order_reference = "'.pSQL($order_reference).'"');
	
	// Update gift cards
	$active = $paid >= $due ? 1 : 0;
	$db->execute('
		UPDATE '._DB_PREFIX_.'cart_rule cr
		JOIN '._DB_PREFIX_.'cart_rule_kerawen crk ON crk.id_cart_rule = cr.id_cart_rule AND crk.type ="'._KERAWEN_CR_GIFT_CARD_.'"
		JOIN '._DB_PREFIX_.'orders o ON o.id_order = crk.id_order AND o.reference = "'.pSQL($order_reference).'"
		SET cr.active = '. (int) $active);

	if ($active) {
	   require_once(_KERAWEN_CLASS_DIR_.'/giftcard.php');
	   sendEmailGiftCard($order_reference);
	}
	
	//Rewards
	$order = $db->getRow('
	    SELECT
	       o.id_order,
           o.id_shop,
	       o.id_customer,
           o.reference,
	       IFNULL(ok.id_till, 0) AS id_till,
           c.id_default_group
	    FROM '._DB_PREFIX_.'orders o
	    LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
        LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
	    WHERE o.reference = "'.pSQL($order_reference).'"'
	);
	
	//Fully paid only -> $active == 1
	//Till only -> $order['id_till'] > 0
	if ($order && (int) $order['id_till'] > 0) {

	    $order['due'] = $due;
	    $order['paid'] = $paid;
	    $order['done'] = $active;
	    $order['isAnonymous'] = isAnonymousCustomer($order['id_customer']);
	    
	    $cust = new Customer($order['id_customer']);
	    $order['cust_group'] = $cust->id_default_group;
	    
	    
	    $nb_rewards = (int) $db->getValue('SELECT COUNT(id_cart_rule) AS nb FROM '._DB_PREFIX_.'cart_rule_kerawen WHERE `type` = "REWARD" AND id_reward > 0 AND order_ref_src = "'.pSQL($order_reference).'"');
	    if (!$nb_rewards) {
	        applyRewardRules($order, getAvailableRewardRules($order));
	    }
	}
}


function applyRewardRules($order, $rewardRules) {
    
    $db = Db::getInstance();
    $context = Context::getContext();

     //Ensure rules are active
     Configuration::updateGlobalValue('PS_CART_RULE_FEATURE_ACTIVE', '1');
     
     foreach ($rewardRules as $row) {
     
         $rule = new CartRule();
         $rule->code = generateRuleCode();
         
         if ($row['cr_reduction_percent'] > 0) {
             $rule->reduction_percent = $row['cr_reduction_percent'];
         } else {
             $rule->reduction_amount = $row['cr_reduction_amount'];
             $rule->reduction_currency = $context->currency->id; //always ?
         }
         
         $rule->partial_use = (int) $row['cr_partial_use'];
         $rule->reduction_tax = (int) $row['cr_reduction_tax'];
         $rule->id_customer = $order['isAnonymous'] ? 0 : $order['id_customer'];
         $rule->quantity = 1;
         $rule->quantity_per_user = 1;         
         $rule->active = 1;

         $start_date = $row['rm_date_start'] ? date('Y-m-d', strtotime($row['rm_date_start'])) : date('Y-m-d');
         $rule->date_from = $start_date . ' 00:00:00';
         $rule->date_to = date('Y-m-d', strtotime($start_date . ' +' . ((int) $row['rm_duration']-1) . ' days')) . ' 23:59:59';

         $languages = Language::getLanguages(true);
         foreach ($languages as $l) {
             $rule->name[(int)$l['id_lang']] = $row['cr_name'];
         }
         $rule->description = $row['cr_description'];
         
         $rule->add();

         if ((int) $row['cr_shop_restriction'] > -1) {
             $id_shop = ((int) $row['cr_shop_restriction'] > 0) ? (int) $row['cr_shop_restriction'] : (int) $context->shop->id;
             $db->execute('
                INSERT INTO `'._DB_PREFIX_.'cart_rule_shop`
				    (`id_cart_rule`, `id_shop`)
				VALUES 
                    (' . (int) $rule->id . ', '. $id_shop .')'
             );
         }
         
         $db->execute('
            INSERT INTO `'._DB_PREFIX_.'cart_rule_kerawen`
                (`id_cart_rule`, `type`, `id_reward`, `order_ref_src`)
            VALUES
                (' . (int) $rule->id . ', "' . pSQL(_KERAWEN_CR_REWARD_) . '",' . (int) $row['id_reward'] . ',"' . pSQL($order['reference']) . '")'
         );
     }
}


function getAvailableRewardRules($order) {

    $db = Db::getInstance();

    $buf = $db->executeS('
        SELECT * 
        FROM '._DB_PREFIX_.'reward_marketing_kerawen
        WHERE rm_active = 1
        AND rm_minimum_amount <= ' . (float) $order['paid'] . '
        AND ( NOW() BETWEEN IFNULL(rm_date_from, "1900-01-01 00:00:00") AND IFNULL(rm_date_to, NOW()) )
        AND (rm_customer_group = "" OR CONCAT(",",rm_customer_group,",") LIKE "%,' . $order['cust_group'] . ',%")
        AND (cr_shop_restriction = "" OR CONCAT(",",cr_shop_restriction,",") LIKE "%,' . $order['id_shop'] . ',%")
        AND (rm_customer = 1 OR (rm_customer = 0 AND 0 = ' . (int) $order['isAnonymous'] . '))
        ORDER BY rm_cumulative DESC, rm_priority DESC
    ');


    //Check cumulative
    $rmk = array();
    foreach ($buf as $row) {
        if ($row['rm_cumulative']) {
            $rmk[] = $row;
        } else {
            if (count($rmk) === 0) {
                $rmk[] = $row;
                break;
            }
        }
    }

    return $rmk;
}



function slipGiftCard($id_slip, $amount) {

	if (!empty($id_slip)) {

		//$id_slip = 	$slip->id;
	
		$db = Db::getInstance();

		$q = "
		SELECT 
		SUBSTRING_INDEX(GROUP_CONCAT(cart_rule_kerawen.id_cart_rule SEPARATOR ','), ',', order_slip_detail.product_quantity) AS id_cart_rule
		FROM " . _DB_PREFIX_ . "order_slip order_slip
		LEFT JOIN " . _DB_PREFIX_ . "order_slip_detail order_slip_detail ON order_slip.id_order_slip = order_slip_detail.id_order_slip
		LEFT JOIN " . _DB_PREFIX_ . "order_detail order_detail ON order_slip_detail.id_order_detail = order_detail.id_order_detail
		LEFT JOIN " . _DB_PREFIX_ . "cart_rule_kerawen cart_rule_kerawen ON order_slip.id_order = cart_rule_kerawen.id_order AND order_detail.product_id = cart_rule_kerawen.id_product AND order_detail.product_attribute_id = cart_rule_kerawen.id_attribute AND cart_rule_kerawen.type = '" . _KERAWEN_CR_GIFT_CARD_ . "' 
		WHERE id_cart_rule IS NOT NULL AND order_slip.id_order_slip = " .  (int) $id_slip . " 
		GROUP BY order_slip_detail.id_order_detail";

		if ($buf = $db->executeS($q)) {
			foreach ($buf as $row) {
				/*
				$db->delete('cart_rule', 'id_cart_rule IN(' . $row['id_cart_rule'] . ')');
				$db->delete('cart_rule_kerawen', 'id_cart_rule IN(' . $row['id_cart_rule'] . ')');
				*/
				//OR SET cart_rule.active=0 OR cart_rule.quantity=0 ?
				$db->update('cart_rule', array('active' => 0), 'id_cart_rule IN(' . $row['id_cart_rule'] . ')');				
			}
			
			
			/*
			//temporary disable flow -> TODO LATER
			$q = "
			SELECT csk.id_cashdrawer_sale, csk.id_cashdrawer_op, csk.id_order, cfk.amount*(-1) AS amount
			FROM " . _DB_PREFIX_ . "cashdrawer_sale_kerawen  csk
			LEFT JOIN " . _DB_PREFIX_ . "cashdrawer_flow_kerawen cfk ON csk.id_cashdrawer_op = cfk.id_cashdrawer_op
			WHERE csk.id_order_slip = " . (int) $id_slip . " AND csk.oper = 'SLIP'
			";

			if ($row = $db->getRow($q)) {	
				//$row['amount']; //-> not ready yet
				$db->insert('cashdrawer_flow_kerawen', array(
					'id_cashdrawer_op' => $row['id_cashdrawer_op'],
					'id_order' => $row['id_order'],
					'id_payment_mode' => _KERAWEN_PM_GIFT_CARD_,
					'amount' => (float) $amount,
				), true);				
			}
			*/

		}

	}

}


function getLoyaltyForTicket($cust) {
    
    require_once(dirname(__FILE__).'/loyalty.php');
    
    $rules = array();

    if ($cust && Configuration::get('KERAWEN_TICKET_LOYALTY_PRINT')) {
        $loyalty = KerawenLoyalty::getPlugin();
        if ($loyalty) {
    
            $loyalty = $loyalty->getRules($cust["id"]);
            $now = date('Y-m-d H:i:s', time());
            $cr = Db::getInstance()->executeS("
        		SELECT
                    cart_rule.id_cart_rule AS id,
                    cart_rule.code,
                    cart_rule.reduction_amount AS value,
                    cart_rule.reduction_currency AS currency,
                    cart_rule.reduction_tax AS tax,
                    cart_rule.date_to AS `to`
        		FROM " . _DB_PREFIX_ . "cart_rule cart_rule
        		WHERE
                    cart_rule.active = 1 
                    AND cart_rule.quantity > 0
                    AND cart_rule.id_customer = " . (int) $cust["id"] . "
                    AND ('" . pSQL($now) . "' BETWEEN cart_rule.date_from AND cart_rule.date_to)"
            );
    
            foreach ($cr as &$rule) {
                if (in_array($rule['id'], $loyalty['loyalty'])) {
                    $rule['name'] = "Loyalty";
                    $rule['mode'] = 'amount';
                    $rules[] = $rule;
                }
            }
        }
       
    }
    return $rules;
    
}


function getGiftCardByTicket($id_order, $active = 1) {

    require_once (_KERAWEN_TOOLS_DIR_.'/utils.php');
    $now = date('Y-m-d H:i:s', time());
    
    $cards = Db::getInstance()->executeS("
		SELECT 
            cart_rule.id_cart_rule,
            cart_rule.code, 
            cart_rule.reduction_amount AS value, 
            cart_rule.reduction_currency AS currency, 
            cart_rule.reduction_tax AS tax, 
            cart_rule.date_to AS `to`,
            IF(cart_rule.quantity = 1 AND cart_rule.active = 1 AND ('" . pSQL($now) . "' BETWEEN cart_rule.date_from AND cart_rule.date_to), 1,0) AS active
		FROM " . _DB_PREFIX_ . "cart_rule_kerawen cart_rule_kerawen
		INNER JOIN " . _DB_PREFIX_ . "cart_rule cart_rule 
            ON cart_rule_kerawen.id_cart_rule = cart_rule.id_cart_rule 
		WHERE 
            (cart_rule.active = 1 OR cart_rule.active = " . (int) $active . ") 
            AND cart_rule.quantity = 1 AND cart_rule_kerawen.type= '" . _KERAWEN_CR_GIFT_CARD_ . "' 
            AND ('" . pSQL($now) . "' BETWEEN cart_rule.date_from AND cart_rule.date_to) 
            AND cart_rule_kerawen.id_order = " . (int) $id_order
	);
	
	foreach($cards as $k => $card) {
	    $cards[$k]['link'] = getKerawenLink('giftcardnext', array('code'=>$card['code']));
	}
	
	return $cards;
	
}


function applyDiscount($context, $params, &$response)
{
	// Ensure rules are active
	Configuration::updateGlobalValue('PS_CART_RULE_FEATURE_ACTIVE', '1');
	
	$id_cart = $params->id_cart;
	$value = $params->value;
	$percent = $params->percent;

	$rule = new CartRule();
	$rule->code = generateRuleCode();
	if ($percent)
		$rule->reduction_percent = $value;
	else
	{
		$rule->reduction_currency = $context->currency->id;
		$rule->reduction_amount = $value;
	}
	$rule->partial_use = 0;
	$rule->reduction_tax = !((int) $context->group->price_display_method);
	$rule->quantity = 1;
	$rule->quantity_per_user = 1;

	$rule->date_from = date('Y-m-d H:i:s', time() - 1);
	$rule->date_to = date('Y-m-d H:i:s', strtotime($rule->date_from.' +1 year'));

	$languages = Language::getLanguages(true);
	foreach ($languages as $l)
		$rule->name[(int)$l['id_lang']] = $context->module->l('Cart discount', pathinfo(__FILE__, PATHINFO_FILENAME));

	$rule->add();
	$id_rule = $rule->id;

	// Limit rule to the current cart
	$db = Db::getInstance();
	$sql = 'INSERT INTO `'._DB_PREFIX_.'cart_rule_kerawen`
				(`id_cart_rule`, `type`, `id_cart`)
				VALUES ('.pSQL($id_rule).',\''._KERAWEN_CR_DISCOUNT_.'\','.pSQL($id_cart).')';
	$db->execute($sql);

	// Use rule immediatly
	require_once (dirname(__FILE__).'/cart.php');
	$params->id_rule = $id_rule;
	addRule($context, $params, $response);
}

function getCartRulesVouchers($id_lang, $id_cart_rule) {
    return Db::getInstance()->executeS('
        SELECT 
            cr.id_cart_rule AS id, 
            crl.name AS `name`,
            cr.description,
            cr.code,
            cr.priority,
            DATE_FORMAT(cr.date_from, "%d/%m/%Y") AS date_from,
            DATE_FORMAT(cr.date_to, "%d/%m/%Y") AS date_to,
            cr.quantity,
            cr.minimum_amount,
            cr.reduction_percent,
            cr.reduction_amount,
            cr.active,
            DATE_FORMAT(crk.display_from, "%d/%m/%Y") AS display_from,
            DATE_FORMAT(crk.display_to, "%d/%m/%Y") AS display_to,
            crk.display_counter
        FROM '._DB_PREFIX_.'cart_rule cr
        LEFT JOIN '._DB_PREFIX_.'cart_rule_lang crl ON cr.id_cart_rule = crl.id_cart_rule AND id_lang = ' . (int)$id_lang . '
        LEFT JOIN '._DB_PREFIX_.'cart_rule_kerawen crk ON cr.id_cart_rule = crk.id_cart_rule
        WHERE 
            crk.is_voucher = 1
            AND (0 = ' . (int) $id_cart_rule . ' OR cr.id_cart_rule = ' . (int) $id_cart_rule . ')
        ORDER BY cr.id_cart_rule DESC
	');
}


function getCartRulesRewards($id_reward = 0) {
    return Db::getInstance()->executeS('
        SELECT 
            rmk.id_reward AS id,
            rmk.rm_active,
            DATE_FORMAT(rmk.rm_date_from, "%d/%m/%Y") AS rm_date_from,
            DATE_FORMAT(rmk.rm_date_to, "%d/%m/%Y") AS rm_date_to,
            DATE_FORMAT(rmk.rm_date_start, "%d/%m/%Y") AS rm_date_start,
            rmk.rm_duration,
            rmk.rm_priority,
            rmk.rm_cumulative,
            rmk.rm_minimum_amount,
            rmk.rm_customer,
            rmk.rm_customer_group,
            rmk.cr_name,
            rmk.cr_description,
            rmk.cr_partial_use,
            rmk.cr_shop_restriction,
            rmk.cr_reduction_percent,
            rmk.cr_reduction_amount,
            rmk.cr_reduction_tax
        FROM '._DB_PREFIX_.'reward_marketing_kerawen rmk
        WHERE (0 = ' . (int) $id_reward . ' OR rmk.id_reward = ' . (int) $id_reward . ')
        ORDER BY rmk.id_reward DESC
	');
}


function deleteCartRuleReward($id_reward) {
    
    $db = Db::getInstance();

    $db->execute('
		DELETE cr
		FROM '._DB_PREFIX_.'cart_rule cr
		LEFT JOIN '._DB_PREFIX_.'cart_rule_kerawen crk ON crk.id_cart_rule = cr.id_cart_rule
		WHERE crk.id_reward = ' . (int) $id_reward
    );
    
    $db->delete('cart_rule_kerawen', 'id_reward = '. (int) $id_reward);
    $db->delete('reward_marketing_kerawen', 'id_reward = '. (int) $id_reward);

}


function applyCartRuleReward($data) {
    
    $db = Db::getInstance();

    $values = array();

    
    $values['cr_name'] = '"' . pSQL($data->cr_name) . '"';
    $values['cr_description'] = '"' . pSQL($data->cr_description) . '"';
    $values['cr_shop_restriction'] = '"' . pSQL($data->cr_shop_restriction) . '"';
    $values['rm_customer_group'] = '"' . pSQL($data->rm_customer_group) . '"';
    $values['rm_customer'] = (int) $data->rm_customer;
    
    
    $values['rm_date_from'] = $data->rm_date_from ? '"' . pSQL(implode('-', array_reverse(explode('/', $data->rm_date_from))) . ' 00:00:00') . '"' : 'NULL';
    $values['rm_date_to'] = $data->rm_date_to ? '"' . pSQL(implode('-', array_reverse(explode('/', $data->rm_date_to))) . ' 23:59:59') . '"' : 'NULL';
    $values['rm_minimum_amount'] = '"' . (float) $data->rm_minimum_amount . '"';
    if ($data->discount_type == 'amount') {
        $values['cr_reduction_percent'] = '"0.00"';
        $values['cr_reduction_amount'] = '"' . (float) $data->discount_value . '"';
    } else {
        $values['cr_reduction_percent'] = '"' . (float) $data->discount_value . '"';
        $values['cr_reduction_amount'] = '"0.00"';
    }
    $values['cr_reduction_tax'] = (int) $data->cr_reduction_tax;
    $values['rm_cumulative'] = (int) $data->rm_cumulative;
    $values['rm_priority'] = (int) $data->rm_priority;
    $values['rm_active'] = (int) $data->rm_active;
    
    
    $values['rm_date_start'] = $data->rm_date_start ? '"' . pSQL(implode('-', array_reverse(explode('/', $data->rm_date_start))) . ' 23:59:59') . '"' : 'NULL';
    $values['rm_duration'] = (int) $data->rm_duration;
    $values['cr_minimum_amount'] = (int) $data->cr_minimum_amount;
    $values['cr_partial_use'] = (int) $data->cr_partial_use;


    $values['rm_date_upd'] = '"' . date("Y-m-d H:i:s", time()) . '"';

    if ((int) $data->id_reward === 0) {
        $values['rm_date_add'] = $values['rm_date_upd'];
        $q = 'INSERT INTO `'._DB_PREFIX_.'reward_marketing_kerawen` (' . implode(',', array_keys($values)) . ') VALUES (' . implode(',', $values) . ')';
    } else {
        $q = 'UPDATE `'._DB_PREFIX_.'reward_marketing_kerawen` SET ';
        $sep = '';
        foreach($values as $key => $value) {
            $q .= $sep . $key . ' = ' . $value;
            $sep = ',';
        }
        $q .= ' WHERE id_reward = '. (int) $data->id_reward;
    }
    $db->execute($q);

}



function applyCartRuleVoucher($data) {

    $context = Context::getContext();

    if ($data->id > 0) {
       $id_cart_rule = $data->id;
       $code = null;
       $quantity = (int) $data->quantity;
       $quantity_per_user = (int) $data->quantity;
    } else {
       $id_cart_rule = null;
       $code = generateRuleCode();
       $quantity = ((int) $data->quantity == 0) ? 1000 : (int) $data->quantity;
       $quantity_per_user = $quantity;
    }

    if ($data->discount_type == 'percent') {
        $reduction_percent = $data->discount_value;
        $reduction_amount = 0;
    } else {
        $reduction_percent = 0;
        $reduction_amount = $data->discount_value;
    }

    $date_from = implode("-", array_reverse(explode("/", $data->date_from)));
    $date_to = implode("-", array_reverse(explode("/", $data->date_to)));
    $display_from = $data->display_from ? "'" . implode("-", array_reverse(explode("/", $data->display_from))) ." 00:00:00'" : 'NULL';
    $display_to = $data->display_to ? "'" . implode("-", array_reverse(explode("/", $data->display_to))). " 23:59:59'" : 'NULL';
   

    $rule = setKerawenCartRule(
        $id_cart_rule, 
        $code, 
        0,
        $context->currency->id,
        $reduction_percent,
        $reduction_amount,
        1, 
        1, 
        0,
        $data->priority, 
        $quantity, 
        $quantity_per_user, 
        $data->active, 
        $date_from, 
        $date_to, 
        $data->name,
        $data->description,
        _KERAWEN_CR_VOUCHER_,
        'NULL', 
        'NULL', 
        'NULL', 
        'NULL', 
        1, 
        $display_from,
        $display_to,
        $data->minimum_amount,
        1
    ); 

}


function deleteCartRuleVoucher($id_cart_rule) {
    $db = Db::getInstance();
    $rule = new CartRule($id_cart_rule);
    $rule->delete();
    $db->delete('cart_rule_kerawen', 'id_cart_rule = '. (int) $id_cart_rule);
}

