<?php
/**
* 2014 KerAwen
*
* NOTICE OF LICENSE
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2014 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/

function getCategories($context)
{
	// Get all categories, even inactive ones
	$all = Category::getCategories($context->language->id, false);
	$id_root = Category::getRootCategory()->id;
	return getSubCategories($id_root, $all);
}

function getSubCategories($id_cat, &$all)
{
	$res = array();
	if (isset($all[$id_cat]))
	{
		$children = $all[$id_cat];
		foreach ($children as $id_child => $cat)
		{
			$res[] = array(
				'id' => $id_child,
				'name' => $cat['infos']['name'],
			);
			$res = array_merge($res, getSubCategories($id_child, $all));
		}
	}
	return $res;
}


function getProducts($context, $cat, $id_shop, $id_lang, $id_customer = 0, $id_group = 1, $from = 0, $count = 0)
{
	$prods = array();
	$total = 0;
	
	if ($id_shop)
	{
		$db = Db::getInstance();
		
		$active_filter = Configuration::get('KERAWEN_CATALOG_INACTIVE') ? '' : 'AND ps.active = 1';
		
		$order_list = array('cp.position ASC', 'pl.name ASC', 'pl.name DESC');
		$order_by = isset($order_list[(int) Configuration::get('KERAWEN_CATALOG_PRODUCT_ORDER')]) ? $order_list[(int) Configuration::get('KERAWEN_CATALOG_PRODUCT_ORDER')] : $order_list[0];
		
		$pp = $db->executeS('
			SELECT SQL_CALC_FOUND_ROWS
				cp.id_product AS id
			FROM '._DB_PREFIX_.'category_product cp
			JOIN '._DB_PREFIX_.'product_shop ps
				ON ps.id_product = cp.id_product
            LEFT JOIN '._DB_PREFIX_.'product_lang pl
                ON pl.id_product = ps.id_product 
                AND pl.id_lang = ' . (int) $id_lang . ' 
                AND pl.id_shop = ' . (int) $id_shop . '
			WHERE
				cp.id_category = '. (int) $cat->id .'
				AND ps.id_shop = '. (int) $id_shop .'
				'.$active_filter.'
			ORDER BY ' . $order_by . '
			'.($count ? 'LIMIT '. (int) $from .','. (int) $count : ''));
		$total = $db->getValue('SELECT FOUND_ROWS()');
				
		foreach ($pp as &$p)
		{
			$id_prod = $p['id'];
			$prod = new Product($id_prod, true, $id_lang, $id_shop);
			if ($prod->id) {
				$detail = detailProduct($prod, $id_lang, false, $id_customer, $id_group);
				if ($detail) $prods[] = $detail;
			}
		}
	}
	return array(
		'prods' => $prods,
		'total' => $total,
	);
}



function getProductQtyDetail($request) {

    if (Tools::version_compare(_PS_VERSION_, '1.7', '<')) {
        
        $qr = array();
        
        /*
        //Only for test
        $db = Db::getInstance();
        $qr = $db->executeS('
            SELECT
                (od.product_quantity - od.product_quantity_refunded) AS res_qty,
                o.id_order,
                o.reference,
                o.current_state AS state
            FROM '._DB_PREFIX_.'orders o
            INNER JOIN '._DB_PREFIX_.'order_detail od ON od.id_order = o.id_order
            INNER JOIN '._DB_PREFIX_.'order_state os ON os.id_order_state = o.current_state
            WHERE o.id_shop = ' . (int) $request->params->id_prod_shop . '
            AND od.product_id = ' . (int) $request->params->id_prod . '
            AND od.product_attribute_id = ' . (int) $request->params->id_prod_attr . '
            LIMIT 10'
        );
        */
        
    } else {
        //src/adapter/stockManager.php -> updateReservedProductQuantity(
        $db = Db::getInstance();
        $qr = $db->executeS('
            SELECT
                (od.product_quantity - od.product_quantity_refunded) AS rqty,
                o.id_order,
                o.reference,
                o.current_state AS state
            FROM '._DB_PREFIX_.'orders o
            INNER JOIN '._DB_PREFIX_.'order_detail od ON od.id_order = o.id_order
            INNER JOIN '._DB_PREFIX_.'order_state os ON os.id_order_state = o.current_state
            WHERE o.id_shop = ' . (int) $request->params->id_prod_shop . '
            AND os.shipped != 1
            AND (
                o.valid = 1 OR (
                    os.id_order_state != ' . (int) Configuration::get('PS_OS_CANCELED') . ' 
                    AND os.id_order_state != ' . (int) Configuration::get('PS_OS_ERROR') . '
                )
            )
            AND od.product_id = ' . (int) $request->params->id_prod . '
            AND od.product_attribute_id = ' . (int) $request->params->id_prod_attr
        );
    }
    return $qr;
}


function actionProduct($request) {

    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
    require_once(_KERAWEN_API_DIR_.'/constants.php');
    require_once(_KERAWEN_CLASS_DIR_.'/data.php');
    require_once(_KERAWEN_CLASS_DIR_.'/KerawenTax.php');
    
    $db = Db::getInstance();
    
    $virtual_shop = 0;
    if (Module::isInstalled('kerawen_ocs') && Module::isEnabled('kerawen_ocs')) {
        $virtual_shop = (int) $db->getValue('SELECT id_shop FROM '._DB_PREFIX_.'kerawen_cnc_shop_type WHERE type = "virtual"');
    }
    
    $prod = new Product($request->params->id_prod, true);
    $tax = TaxManagerFactory::getManager(getDefaultDeliveryAddress(true), $prod->id_tax_rules_group);

    if (Module::isInstalled('vatmargin') && Module::isEnabled('vatmargin') && isset($prod->vat_margin)) {
        $with_vat_margin = 1;
        $vat_margin = (int) $prod->vat_margin;
    } else {
        $with_vat_margin = 0;
        $vat_margin = 0;
    }

    $id_lang = Context::getContext()->language->id;
    $margin_calc = 0;
    
    if ($prod->id) {

       $dp = detailProduct(
            $prod,
            $request->context->language->id,
            true,
            $request->context->customer->id,
            $request->context->group->id,
            getProductBackEndUrl($prod),
            getProductFrontEndUrl($prod),
            true
        );

       $dp['width'] = $prod->width;
       $dp['height'] = $prod->height;
       $dp['depth'] = $prod->depth;
       $dp['weight'] = $prod->weight;

       $prod_type = _KERAWEN_TYPE_STANDARD_;
       
       if ($dp['wm']) {
           if ( $dp['wm']['codes']) {
               foreach($dp['wm']['codes'] as &$code) {
                   $code['product_attribute'] = ((int) $code['id_product_attribute'] > 0) ? getProdAttrComByIdProdAttr($code['id_product_attribute']) : '';
               }
           }
           if ((String) $dp['wm']['margin_calc'] !== '') {
               $margin_calc = $dp['wm']['margin_calc'];
           }
           
           if ((String) $dp['wm']['product_type'] !== '') {
               $prod_type = $dp['wm']['product_type'];
           } else {
               if ($vat_margin) {
                   $prod_type = '_KERAWEN_TYPE_DEALER_';
               }
           }
           
           if ((int) $dp['wm']['is_gift_card']) {
               $prod_type = _KERAWEN_TYPE_GIFTCARD_;
           }
           if ((int) $dp['wm']['is_account_credit']) {
               $prod_type = _KERAWEN_TYPE_PREPAID_;
           }
           if ((int) $dp['wm']['measured']) {
               $prod_type = _KERAWEN_TYPE_MEASURED_;
           }
           if ((int) $dp['wm']['id_customer']) {
               $prod_type = _KERAWEN_TYPE_DEPOSIT_;
           }
       }
       
        $shops = array();
        $ss = $db->executeS('
    		SELECT id_shop
    		FROM '._DB_PREFIX_.'shop
            WHERE deleted = 0 AND active = 1 AND id_shop != ' . $virtual_shop . '
            ORDER BY id_shop ASC
            '
         );
    
        $attrs = array();
        $stocks = array();
        
        if (count($ss) > 0) {
    
            $attrs = $db->executeS('
                SELECT
                    pa.id_product_attribute,
                    GROUP_CONCAT(al.name ORDER BY ag.position ASC, a.position ASC) AS name
                FROM '._DB_PREFIX_.'product_attribute pa
                INNER JOIN '._DB_PREFIX_.'product_attribute_combination pac ON pa.id_product_attribute = pac.id_product_attribute AND pa.id_product = ' . (int) $request->params->id_prod . '
                LEFT JOIN '._DB_PREFIX_.'attribute_lang al ON al.id_attribute = pac.id_attribute AND al.id_lang = ' . (int) $request->context->language->id . '
                LEFT JOIN '._DB_PREFIX_.'attribute a ON a.id_attribute = pac.id_attribute
                LEFT JOIN '._DB_PREFIX_.'attribute_group ag ON a.id_attribute_group = ag.id_attribute_group
                GROUP BY pa.id_product, pa.id_product_attribute
                ORDER BY ag.position ASC
            ');
    
 
            if (count($attrs) == 0) {
                $as_attr = false;
                $attrs[0] = array(
                    'id_product_attribute' => 0,
                    'name' => $prod->name[$id_lang],
                );
            } else {
                $as_attr = true;
            }
    
            foreach($attrs as $key => $attr) {
                $attrs[$key]['item'] = array();
                foreach($ss as $s) {
                    
                    if (Tools::version_compare(_PS_VERSION_, '1.7', '<')) {
                        
                        $quantity = (int) StockAvailable::getQuantityAvailableByProduct((int) $request->params->id_prod, (int) $attr['id_product_attribute'], (int) $s['id_shop']);
                        $physical_quantity = $quantity;
                        $reserved_quantity = 0;
                        
                    } else {
                        
                        $qty = $db->getRow('SELECT quantity, physical_quantity, reserved_quantity FROM '._DB_PREFIX_.'stock_available WHERE id_product = ' . (int) $request->params->id_prod . ' AND id_product_attribute = ' . (int) $attr['id_product_attribute'] .' AND id_shop = ' . (int) $s['id_shop']);
                        if ($qty) {
                            $quantity = (int) $qty['quantity'];
                            $physical_quantity = (int) $qty['physical_quantity'];
                            $reserved_quantity = (int) $qty['reserved_quantity'];
                        } else {
                            $quantity = 0;
                            $physical_quantity = 0;
                            $reserved_quantity = 0;
                        }

                    }
                    
                    $attrs[$key]['item'][$s['id_shop']] = array();
                    $attrs[$key]['item'][$s['id_shop']]['qty'] = $quantity;
                    $attrs[$key]['item'][$s['id_shop']]['pqty'] = $physical_quantity;
                    $attrs[$key]['item'][$s['id_shop']]['rqty'] = $physical_quantity - $quantity;
                    $attrs[$key]['item'][$s['id_shop']]['id_prod'] = (int) $request->params->id_prod;
                    $attrs[$key]['item'][$s['id_shop']]['id_prod_attr'] = (int) $attr['id_product_attribute'];
                    $attrs[$key]['item'][$s['id_shop']]['id_prod_shop'] = (int) $s['id_shop'];

                }
            }

            //Change order stocks
            if ($as_attr) {
                $attrs = indexArray($attrs, 'id_product_attribute');
                foreach($dp['versions'] as $version) {
                    if (isset($attrs[$version['id']])) {
                        $stocks[(int)$version['pos']] = $attrs[$version['id']];
                    }
                }
            } else {
                $stocks = $attrs;
            }
            
        }

        $location = Tools::version_compare(_PS_VERSION_, '1.7.5', '>=') ? 'sa.location' : 'pa.location';

        $attrs_detail = $db->executeS('
            SELECT

                GROUP_CONCAT(pac.id_attribute ORDER BY ag.position ASC) AS id,
                GROUP_CONCAT(ag.id_attribute_group ORDER BY ag.position ASC) AS ag,
                pas.id_product_attribute,
                pas.ecotax,
                pas.weight,
                pas.unit_price_impact,
                pas.wholesale_price,
                pas.price,             
                pas.minimal_quantity,
                IF(pas.default_on = 1, 1, 0) AS default_on,
                pas.low_stock_threshold AS low_stock,
                DATE_FORMAT(pas.available_date, "%Y-%m-%d") AS available_date,
                pas.unit_price_impact,

                pa.reference,
                pa.ean13,
                pa.upc,
                pa.weight,

                pak.packing,
                pak.target,

                ' . $location . ' AS location,
                1 AS active

            FROM '._DB_PREFIX_.'product_attribute_shop pas
	        INNER JOIN '._DB_PREFIX_.'product_attribute pa ON pa.id_product_attribute = pas.id_product_attribute
            INNER JOIN '._DB_PREFIX_.'product_attribute_combination pac ON pas.id_product_attribute = pac.id_product_attribute
            LEFT JOIN '._DB_PREFIX_.'attribute a ON pac.id_attribute = a.id_attribute
            LEFT JOIN '._DB_PREFIX_.'attribute_group ag ON ag.id_attribute_group = a.id_attribute_group
            LEFT JOIN '._DB_PREFIX_.'product_attr_kerawen pak ON pas.id_product_attribute = pak.id_product_attribute AND pak.id_shop = ' . (int) $request->context->shop->id . '
            LEFT JOIN '._DB_PREFIX_.'stock_available sa ON pas.id_product_attribute = sa.id_product_attribute AND sa.id_shop = ' . (int) $request->context->shop->id . '
            WHERE pas.id_product = ' . (int) $prod->id . ' AND pas.id_shop = ' . (int) $request->context->shop->id . '
            GROUP BY pas.id_product_attribute
        ');
        
        foreach($attrs_detail as &$attr) {
            $attr['price_ti'] = $tax->getTaxCalculator()->addTaxes($attr['price']);
        }

        $attrs_detail = indexArray($attrs_detail, 'id');

        $edit_pr = $db->getRow('SELECT price, wholesale_price FROM ' . _DB_PREFIX_ . 'product_shop WHERE id_product = ' . (int) $prod->id . ' AND id_shop = ' . (int) $request->context->shop->id);

        $output = array_merge($dp, array(
            'prod_type' => $prod_type,
            'title' => $dp['name'][$id_lang],
            'stocks' => $stocks,
            'attrs_detail' => $attrs_detail,
            'edit_price_te' => '',
            'edit_price' => $tax->getTaxCalculator()->addTaxes($edit_pr['price']),
            'edit_wholesale_price' => $edit_pr['wholesale_price'],
            'vat_margin' => (int)($vat_margin || $prod->id_tax_rules_group == KerawenTax::GetVatMarginRule()),
        ));

    } else {

        
        $langs = Language::getLanguages(false, false, false);
        $languages = array();
        foreach ($langs as $lang) { $languages[$lang['id_lang']] = ''; }
        
        $output = array(
            'prod_type' => _KERAWEN_TYPE_STANDARD_,
            'id' => 0,
            'title' => null,
            'visibility' => "none",
            'active' => true,
            'name' => $languages,
            'description' => $languages,
            'description_short' => $languages,
            'ref' => "",
            'stock' => 0,
            'low_stock' => null,
            'packing' => "",
            'target' => "",
            'edit_price_te' => "",
            'edit_price' => "",
            'margin_rate' => 0,
            'edit_wholesale_price' => Configuration::get('KERAWEN_NEW_PROD_WHOLESALE_PRICE') ? '0' : '',
            'price' => "",
            'price_te' => "",
            'init_price' => "",
            'init_price_te' => "",
            'specific_type' => false,
            'specific' => false,
            'img' => "",
            'attrs' => false,
            'options' => array(),
            'versions' => array(),
            'def' => 0,
            'wm' => false,
            'custom' => false,
            'backend_url' => "",
            'frontend_url' => "",
            'id_tax_rules_group' => 0,
            'id_manufacturer' => 0,
            'id_supplier' => 0,
            'wholesale_price' => "",
            'rate' => 0,
            'display_method' => 0,
            'dos' => false,
            'entrepots' => array(),
            'ean' => isset($request->params->code) ? $request->params->code : "",
            'stocks' => array(),
            'attrs_detail' => array(),
            'vat_margin' => 0,
            'width' => '',
            'height' => '',
            'depth' => '',
            'weight' => '',
        );
    }
 
    $output['with_vat_margin'] = $with_vat_margin;
    $output['margin_calc'] = $margin_calc;
    $output['default_qty'] = 0;
    $output['ags'] = getKerawenAttributes();
    
    //Need to improve/crapy
    $output['multishops'] = "0";
    $output['tab'] = "";
    $output['isnew'] = ((int) $prod->id) ? 0 : 1;
    $output['id_lang'] = $request->context->language->id;
    $output['firstload'] = !(isset($request->params->product));
    
    if (isset($request->params->product->multishops)) {
        $output['multishops'] = $request->params->product->multishops;
    }
    if (isset($request->params->multishops)) {
        $output['multishops'] = $request->params->multishops;
    }

    if (isset($request->params->product->tab)) {
        $output['tab'] = $request->params->product->tab;
    }
    if (isset($request->params->tab)) {
        $output['tab'] = $request->params->tab;
    }
    
    if (isset($request->params->product->isnew)) {
        $output['isnew'] = $request->params->product->isnew;
    }
    if (isset($request->params->isnew)) {
        $output['isnew'] = $request->params->isnew;
    }
    
    if (isset($request->params->product->id_lang)) {
        $output['id_lang'] = $request->params->product->id_lang;
    }
    if (isset($request->params->id_lang)) {
        $output['id_lang'] = $request->params->id_lang;
    }
    
    return $output;
    
}


function applyProdIndex($k1, $k2, $i) {
    return $i ? $k1 : $k2;
}

function formatOrder($number, $len) {
    return sprintf('%0' . $len . 'd', $number);
}


function detailProduct($prod, $id_lang, $with_attributes = true, $id_customer = 0, $id_group = 1, $backend_url = '', $frontend_url = '', $indexed = false)
{
    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
    require_once(_KERAWEN_API_DIR_.'/constants.php');
    
	static $totswitchattribute = null;
	if ($totswitchattribute === null) {
		$totswitchattribute = Module::isEnabled('totswitchattribute');
	}
	
	if(!isset($prod->id)) {
		return false;
	}

	$context = Context::getContext();
	$id_shop = $context->shop->id;
	$id_currency = $context->currency->id;
	$id_country = $context->country->id;
	
	$db = Db::getInstance();
	
	// KerAwen extension of product
	$ext = getProductExtension($prod->id, true);
	
	$image_type = Configuration::get('KERAWEN_IMAGE_PRODUCT');

	static $link = null;
	if (!$link) $link = new Link();
	
	$manufacturers = getKerawenManufacturers();
	
	if ($id_customer === 0) {
		
		$id_customer = (int) Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER');
		$anonymous_customer = new Customer($id_customer);
		if ($anonymous_customer) {
			$id_group = $anonymous_customer->id_default_group;
		}
		
	}
	
	$display_method = Group::getPriceDisplayMethod($id_group);
	$ReductionByIdGroup = Group::getReductionByIdGroup($id_group);
	$rate_group = 1 - $ReductionByIdGroup/100;
	$rate_group_wm = ((boolean) Configuration::get('KERAWEN_WM_GROUP_EXC')) ? 1 : $rate_group;
	$rate_group_dec = $ext ? (($ext['product_type'] == _KERAWEN_TYPE_GIFTCARD_ || (int) $ext['is_gift_card'] == 1) ? $rate_group : 1) : 1;
		
	$attrs = (boolean)$db->getValue('SELECT EXISTS(
		SELECT 1 FROM '._DB_PREFIX_.'product_attribute_shop pas
        LEFT JOIN  '._DB_PREFIX_.'product_attribute pa ON pa.id_product_attribute = pas.id_product_attribute
		WHERE pa.id_product = '. (int) $prod->id .' AND pas.id_shop = '. (int) $id_shop .')');
	$attr_groups = $prod->getAttributesGroups($id_lang);

	$extker = $db->executeS('
			SELECT * FROM '._DB_PREFIX_.'product_attr_kerawen
			WHERE id_shop = ' . (int) $id_shop . ' AND id_product = '. (int) $prod->id
	);
	$extker = indexArray($extker, 'id_product_attribute');

	
	$id_attr_default = $prod->getDefaultIdProductAttribute();
	if ($attrs && !$id_attr_default) {
		// Force one default combination
		$id_attr_default = (int)$db->getValue('
				SELECT pas.id_product_attribute
				FROM '._DB_PREFIX_.'product_attribute_shop pas
                LEFT JOIN  '._DB_PREFIX_.'product_attribute pa ON pa.id_product_attribute = pas.id_product_attribute
				WHERE pa.id_product = '. (int) $prod->id .' AND pas.id_shop = '. (int) $id_shop);
	}
	
	if ($totswitchattribute 
	    && $attrs
	    && !count($attr_groups)
	    && (!$ext || (!$ext['is_free_price'] && !$ext['measured']))) {
		// All combinations are deactivated
		return false;
	}
	

	$options = array();
	$versions = array();
	
	$prefix = $context->currency->prefix; 
    $suffix = $context->currency->suffix;
	require_once(_KERAWEN_CLASS_DIR_.'/price.php');


    if ($attrs && $with_attributes)
    {
        
        $list_options = array();
        $list_vals = array();
        
        $default = $prod->getDefaultIdProductAttribute();

        $pos = -1;
        $id_opt = 0;
        $id_attr = 0;
        $attr_length = strlen(count($attr_groups));
        
        foreach ($attr_groups as &$group)
        {
            
            if ((int) $id_opt !== (int) $group['id_attribute_group']) { 
                $pos = -1;
                $id_attr = 0;
            }
            if ((int) $id_attr !== (int) $group['id_attribute']) { $pos++; }
            
            $id_attr = (int) $group['id_attribute'];
            $id_opt = $group['id_attribute_group'];
            $key = array_search($id_opt, $list_options);
            if ($key === false) {
                $list_options[] = $id_opt;
                $key = count($list_options) - 1;
            }
            
            if (!isset($options[applyProdIndex($id_opt, $key, $indexed)])) {
                $options[applyProdIndex($id_opt, $key, $indexed)] = array(
                    'id' => applyProdIndex($id_opt, $key, $indexed),
                    'name' => $group['group_name'],
                    'color' => ($group['is_color_group'] != 0),
                    'vals' => array(),
                );
            }

            $id_val = $group['id_attribute'];
            $k = array_search($id_val, $list_vals);
            if ($k === false) {
                $list_vals[] = $id_val;
                $k = count($list_vals) - 1;
            }
            
            $options[applyProdIndex($id_opt, $key, $indexed)]['vals'][applyProdIndex($id_val, $k, $indexed)] = array(
                'id' => applyProdIndex($id_val, $k, $indexed),
                'name' => $group['attribute_name'],
                'color' => $group['attribute_color'],
            );
            

            $id_ver = $group['id_product_attribute'];
            if (!isset($versions[$id_ver])) {
                
                $specific = SpecificPrice::getSpecificPrice($prod->id, $id_shop, $id_currency, $id_country, $id_group, 1, $id_ver, $id_customer);
                $init_price_te = $prod->getPriceStatic($prod->id, false, $id_ver, 6, null, false, false, 1, false, $id_customer);
                
                //standard stock - advanced stock ?
                $wholesale_price = $prod->wholesale_price;
                $combination = new Combination($id_ver);
                if ($combination && $combination->wholesale_price != '0.000000') {
                    $wholesale_price = $combination->wholesale_price;
                }
                
                //PS Compatibility
                if (isset($combination->low_stock_threshold)) {
                    $low_stock_threshold = $combination->low_stock_threshold;
                } else {
                    $low_stock_threshold = $db->getValue('
                       SELECT low_stock_threshold
                       FROM '._DB_PREFIX_.'product_attribute_shop
                       WHERE id_shop = '. (int) $id_shop .' AND id_product_attribute = ' . (int) $id_ver
                    );
                }
                
                $warehouses = array();
                if ($prod->depends_on_stock) {
                    $warehouses = Warehouse::getProductWarehouseList($prod->id, $id_ver);
                    foreach ($warehouses as &$value) {
                        $manager = StockManagerFactory::getManager() ;
                        $value['quantity'] = $manager->getProductRealQuantities( $prod->id , $id_ver, $value['id_warehouse'], true);
                        unset($value['name']);
                    }
                }
                
                $ean = $db->getValue('
    				SELECT ean13
    				FROM '._DB_PREFIX_.'product_attribute pa
    				JOIN '._DB_PREFIX_.'product_attribute_shop pas
    					ON pas.id_product_attribute = pa.id_product_attribute AND pas.id_shop = '. (int) $id_shop .'
    				WHERE pa.id_product = '. (int) $prod->id.'
    				AND pa.id_product_attribute = '. (int) $id_ver);
                
                $versions[$id_ver] = array(
                    'id' => $id_ver,
                    'name' => '',
                    'ref' => $group['reference'],
                    'stock' => Product::getQuantity($prod->id, $id_ver),
                    'location' => (isset($combination->location)) ? $combination->location : '',
                    'low_stock' => $low_stock_threshold,
                    'packing' => isset($extker[$id_ver]) ? $extker[$id_ver]['packing'] : '',
                    'target' => isset($extker[$id_ver]) ? $extker[$id_ver]['target'] : '',
                    'price' => round($prod->getPrice(true, $id_ver, 6) / $rate_group_dec, 6),
                    'price_te' => round($prod->getPrice(false, $id_ver, 6) / $rate_group_dec, 6),
                    'init_price' => round($prod->getPriceStatic($prod->id, true, $id_ver, 6, null, false, false, 1, false, $id_customer) / $rate_group_dec, 6),
                    'init_price_te' => round($init_price_te / $rate_group_dec, 6),
                    'specific_type' => specificType($specific),
                    'vals' => array(),
                    'margin' =>	($init_price_te > 0) ? ($init_price_te - $wholesale_price) : '',
                    'wholesale_price' => $wholesale_price,
                    'entrepots' => $warehouses,
                    'ean' => $ean,
                    'img' => getProductImage($prod->id, $id_ver, $prod->link_rewrite),
                    'pos' => '1',
                );
            }
            
            $versions[$id_ver]['vals'][applyProdIndex($id_opt, $key, $indexed)] = applyProdIndex($id_val, $k, $indexed);
            $versions[$id_ver]['name'] .= ($versions[$id_ver]['name'] ? ', ' : '').$group['attribute_name'];
            $versions[$id_ver]['pos'] .= formatOrder($pos, $attr_length);
                
        }
        
        $tmp_versions = array();
        foreach($versions as $item) {
            $tmp_versions[(int) $item['pos']] = $item;
            if ($item['id'] == $default) { $def = $item['pos']; }
        }
        $versions = $tmp_versions;            
        

        // Ensure attributes have been found
        if (!count($versions)) $attrs = false;
    }
	        

	$custom = $prod->getCustomizationFields($id_lang, $id_shop);
	
	//extend custom fields to kerawen
	if ($custom && count($custom)) {
    	$icf = array();
    	$custom_kerawen = array();
    	foreach ($custom as $customItem) {
    	    $icf[] = (int) $customItem['id_customization_field'];
    	}
    	if (count($icf)) {
    	   $custom_kerawen = $db->executeS('SELECT * FROM ' . _DB_PREFIX_ . 'customization_field_kerawen WHERE id_customization_field IN (' . implode(',', $icf) . ')');
    	   if ($custom_kerawen) {
    	       $custom_kerawen = indexArray($custom_kerawen, 'id_customization_field');
    	   }
    	}
    	foreach ($custom as &$customItem) {
    	    $customItem['field_type'] = '';
    	    if (isset($custom_kerawen[$customItem['id_customization_field']])) {
    	        $customItem['field_type'] = $custom_kerawen[$customItem['id_customization_field']]['field_type'];
    	    }
    	}
    }

	// Prices computation: cart independent
	$price_ti = Product::priceCalculation(
	$id_shop, $prod->id, $id_attr_default, $id_country, null, null, $id_currency,
	    $id_group, 1, true, 6, false, true, true, $specific, true, $id_customer, true, 0, 0) / $rate_group_dec;
	$price_te = Product::priceCalculation(
		$id_shop, $prod->id, $id_attr_default, $id_country, null, null, $id_currency,
	    $id_group, 1, false, 6, false, true, true, $specific, true, $id_customer, true, 0, 0) / $rate_group_dec;

	$init_price_te = $prod->getPriceStatic($prod->id, false, null, 6, null, false, false, 1, false, $id_customer) / $rate_group_dec;
	$init_price = $prod->getPriceStatic($prod->id, true,  null, 6, null, false, false, 1, false, $id_customer) / $rate_group_dec;

	$specificType = specificType($specific);
	$rate = is_numeric($prod->tax_rate) ? (float) $prod->tax_rate : 0;


	if ($ext) {
	    if (isset($ext['codes'])) {
	        if (isset($ext['codes'][0])) {
	            $init_price = ($ext['codes'][0]['unit_price']) * $rate_group_wm;
	            $init_price_te = ($ext['codes'][0]['unit_price_te']) * $rate_group_wm;
	            
	            $price_ti = $init_price;
	            $price_te = $init_price_te;
	            
	            if ($specificType) {
	                if ($specificType == 'PERCENT') {
	                    $price_ti *= (1 - $specific['reduction']);
	                    $price_te *= (1 - $specific['reduction']);
	                } else {
	                    if ($specific['reduction_tax']) {
	                        $price_ti -= $specific['reduction'] * $rate_group_wm;
	                        $price_te = $price_ti / ($rate/100 +1 );
	                    } else {
	                        $price_te -= $specific['reduction'] * $rate_group_wm;
	                        $price_ti = $price_te * ($rate/100 + 1);
	                    }
	                }
	            }
	        }
	    }
	}
	
	$warehouses = array();
	$ean = '';
	$description = '';
	$description_short = '';
	
	if ($with_attributes) {
		$ean = $prod->ean13;
		$description = $prod->description;
		$description_short = $prod->description_short;
		if ($prod->depends_on_stock && !$attrs) {
			$warehouses = Warehouse::getProductWarehouseList($prod->id, 0);
			foreach ($warehouses as &$value) {
		    	$manager = StockManagerFactory::getManager() ;
		    	$value['quantity'] = $manager->getProductRealQuantities( $prod->id , 0, $value['id_warehouse'] , true );
				unset($value['name']);
			}
		}
	}


	if (isset($prod->low_stock_threshold)) {
	    $low_stock_threshold = $prod->low_stock_threshold;
	} else {
	    $low_stock_threshold = 	$db->getValue('
	       SELECT low_stock_threshold
	       FROM '._DB_PREFIX_.'product_shop
           WHERE id_shop = '. (int) $id_shop .' AND id_product = '. (int) $prod->id
	    );
	}

	//$prod->quantity -> deprecated since 1.7.8
	$stock = (Tools::version_compare(_PS_VERSION_, '1.7.8', '>=')) ? StockAvailable::getQuantityAvailableByProduct($prod->id, 0) : $prod->quantity;
	
	$allstock = '';
	if (Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE')) {
	    $allstock = allShopsQuantity($prod->id, 0);
	}
	
	return array(
		'id' => $prod->id,
		'active' => (boolean)$prod->active,
		'name' => $prod->name,
	    'visibility' => $prod->visibility,
		'ref' => $prod->reference,
	    'stock' => $stock,
	    'allstock' => $allstock,
	    'location' => isset($prod->location) ? $prod->location : '',
	    'low_stock' => $low_stock_threshold,
	    'packing'=> isset($extker[0]) ? $extker[0]['packing'] : '',
	    'target'=> isset($extker[0]) ? $extker[0]['target'] : '',
		'price' => $price_ti,
		'price_te' => $price_te,
	    'init_price' => $init_price,
		'init_price_te' => $init_price_te,
	    'specific_type' => $specificType,
		'specific' => $specific,
	    'img' => getProductImage($prod->id, 0, $prod->link_rewrite),
		'attrs' => $attrs,
		'options' => $options,
		'versions' => $versions,
		'def' => isset($def) ? $def : null,
	    'wm' => $ext,
		'custom' => $custom,
		'backend_url' => $backend_url,
		'frontend_url' => $frontend_url,
		'margin' =>	($init_price_te > 0) ? ($init_price_te - $prod->wholesale_price) : '',
		'wholesale_price' => $prod->wholesale_price,
	    'rate' => $rate,
	    'id_tax_rules_group' => $prod->id_tax_rules_group,
		'display_method' => (int) $display_method,
		'dos' => $prod->depends_on_stock,
		'entrepots' => $warehouses,
		'ean' => $ean,
	    'upc' => $prod->upc,
	    'id_manufacturer' => $prod->id_manufacturer,
	    'manufacturer' => isset($manufacturers[(int) $prod->id_manufacturer]) ? $manufacturers[(int) $prod->id_manufacturer]['name'] : '',
	    'id_supplier' => $prod->id_supplier,
		'description' => $description,
		'description_short' => $description_short,
	);
}


function specificType($specific) {

 	$specific_type = false;
	
	if ($specific) {
  
  		if ($specific['reduction_type'] == 'amount') {
			$specific_type = 'AMOUNT';
		} else {
			$specific_type = 'PERCENT';
		}
  
	}

	return $specific_type;

}



/* TO REFACTOR */
function browseCategory($context, $params)
{
	static $link = null;
	if (!$link) $link = new Link();

	$id_cat = $params->id_cat;
	$id_shop = $params->id_shop;
	$id_lang = $context->language->id;

	$id_group = $context->group->id;
	$id_customer = (int) $context->customer->id;

	if (!$id_cat) $id_cat = Category::getRootCategory($id_lang, $context->shop)->id;
	$cat = new Category($id_cat, $id_lang);

	$path = array();
	$buf = $cat;
	while (!$buf->is_root_category && $buf->id_parent) {
		array_unshift($path, array(
			'id' => $buf->id,
			'name' => $buf->name,
		));
		$buf = new Category($buf->id_parent, $id_lang);
	}
	
	// Set shop context before recovering children
	Shop::setContext(Shop::CONTEXT_SHOP, $id_shop);

	$image_type = Configuration::get('KERAWEN_IMAGE_CATEGORY');
	$cc = Category::getChildren($id_cat, $id_lang, (boolean) Configuration::get('KERAWEN_CATEGORY_ACTIVE'), $id_shop);
	
	$categories = array();
	foreach ($cc as &$c)
	{
		$cat_img = $image_type ? '//'.$link->getCatImageLink($c['link_rewrite'], $c['id_category'], $image_type) : false;
		$categories[] = array(
			'id' => $c['id_category'],
			'name' => $c['name'],
			'img' => $cat_img,
		);
	}

	$shortcuts = array();
	$ids = Configuration::get('KERAWEN_SHORTCUT_CATEGORIES');
	if ($ids) $shortcuts = Db::getInstance()->executeS('
		SELECT
			cl.id_category AS id,
			cl.name AS name
		FROM '._DB_PREFIX_.'category_lang cl
		JOIN '._DB_PREFIX_.'category_shop cs
			ON cs.id_category = cl.id_category AND cs.id_shop = cl.id_shop
		WHERE
			cl.id_category IN ('.dataListToIntList($ids).')
			AND cs.id_shop = '. (int) $id_shop .'
			AND cl.id_lang = '. (int) $id_lang);
	
	return array(
		'shorts' => $shortcuts,
		'path' => $path,
		'cats' => $categories,
		'prods' => getProducts($context, $cat, $id_shop, $id_lang, $id_customer, $id_group, $params->from, $params->count),
	);
}


function getCatalogToken() {
    $context = Context::getContext();
    $v = 'AdminProducts';
    $token_key = Tools::getAdminToken($v.(int)(Tab::getIdFromClassName($v)).(int)($context->employee->id));
    return Dispatcher::getInstance()->createUrl($v, $context->language->id, array('token' => $token_key), false);
}


function getProductFrontEndUrl($prod) {
    $context = Context::getContext();
    $id_lang = $context->language->id;
    return $context->link->getPageLink('product', null, $id_lang, array('id_product' => $prod->id), false, $prod->id_shop_default, false);
}

function getProductBackEndUrl($prod) {
    return getCatalogToken() . '&updateproduct&id_product=' . $prod->id;
}


function advSearchProduct($context, $params) {
 
    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
    //require_once(_KERAWEN_CLASS_DIR_.'/KerawenTax.php');
    
    $db = Db::getInstance();
    
    $from = $params->from;
    $count = $params->count;
    
    $id_lang = $context->language->id;
    $id_shop = $context->shop->id;
    $id_customer = (int) $context->customer->id;
    $id_country = (int) $context->country->id;


    if ($id_customer === 0) {
        $id_customer = (int) Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER');
    }
    $cust = new Customer($id_customer);
    $id_group = $cust->id_default_group;
    $display_method = Group::getPriceDisplayMethod($id_group);

    //$taxrule_vatmargin = (int) KerawenTax::GetVATMarginRule();

    $dataSearch = $params->dataSearch;

    $whereTerm = '';
    if (isset($dataSearch->term)) {
        $list_term = array();
        foreach (preg_split('/[\s]+/', str_replace('"', '\\"', $dataSearch->term[0]->value)) as $str) {
            if (Tools::strlen($str)) $list_term[] = $str;
        }
        
        $list_fft = array(
            'pl.name',
            'p.reference',
            'p.ean13',
            'p.upc',
            'pa.reference',
            'pa.ean13',
            'pa.upc'
        );
        $sep1 = '';
        $whereTerm .= ' AND (';
        foreach($list_fft as $fft) {
            $whereTerm .= $sep1 . '(';
            $sep2 = '';
            foreach($list_term as $str) {
                $whereTerm .= $sep2 . $fft . ' LIKE "%' . pSQL($str) . '%"';
                $sep2 = ' AND ';
            }
            $whereTerm .= ')';
            $sep1 = ' OR ';
        }
        $whereTerm .= ')';
    }

    $whereManufacturer = '';
    if (isset($dataSearch->manufacturer)) {
        $whereManufacturer .= 'AND (p.id_manufacturer=' . (int) $dataSearch->manufacturer[0][0]->value . ')';
    }

    $whereSupplier = '';
    $joinSupplier = '';
    if (isset($dataSearch->supplier)) {
        $joinSupplier = 'INNER JOIN '._DB_PREFIX_.'product_supplier psup ON p.id_product=psup.id_product AND psup.id_product_attribute=IFNULL(pas.id_product_attribute, 0)';
        //$whereSupplier .= ' AND (psup.id_supplier = ' . (int) $dataSearch->supplier[0][0]->value . ')';
        $joinSupplier .= ' AND (psup.id_supplier = ' . (int) $dataSearch->supplier[0][0]->value . ')';
    }
    
    $whereStock = '';
    $joinStock = '';
    if (isset($dataSearch->stock)) {
        $list_compare = array('greater' => '>','equal' => '=','less' => '<');
        $joinStock = 'INNER JOIN '._DB_PREFIX_.'stock_available sa ON p.id_product=sa.id_product AND sa.id_product_attribute=IFNULL(pas.id_product_attribute, 0) AND ( ' . getKerawenStockRestriction() . ')';
        //$whereStock = ' AND (sa.quantity' . $list_compare[$dataSearch->stock[0][0]->key] . (int) $dataSearch->stock[0][0]->value . ')';
        $joinStock .= ' AND (sa.quantity' . $list_compare[$dataSearch->stock[0][0]->key] . (int) $dataSearch->stock[0][0]->value . ')';
    }
 
    $whereAttribute = '';
    $joinAttribute = '';
    if (isset($dataSearch->options)) {
        foreach ($dataSearch->options as $key => $attributeGroup) {
            $ids_attr = array();
            foreach ($attributeGroup as $id_attribute) {
                $ids_attr[] = $id_attribute->value;
            }
            $pac = 'pac' . $key;
            $joinAttribute .= '
                INNER JOIN '._DB_PREFIX_.'product_attribute_combination ' . $pac . ' ON pas.id_product_attribute=' . $pac . '.id_product_attribute
                    AND ' . $pac . '.id_attribute IN (' . implode(',', $ids_attr) . ')
                ';
        }
    }
 
    $whereFeature = '';
    $joinFeature = '';
    if (isset($dataSearch->features)) {
        foreach ($dataSearch->features as $key => $feature) {
            $ids_feature = array();
            foreach ($feature as $id_feature) {
                $ids_feature[] = $id_feature->value;
            }
            $fea = 'fea' . $key;
            $joinFeature .= '
                INNER JOIN '._DB_PREFIX_.'feature_product ' . $fea . ' ON ps.id_product=' . $fea . '.id_product
                    AND ' . $fea . '.id_feature_value IN (' . implode(',', $ids_feature) . ')
                ';
        }
    }

    $whereAmount = '';
    $joinAmount = '';
    if (isset($dataSearch->amount)) {
        $amounts = array();
        $amount = $dataSearch->amount[0][0];
        
        //TODO: vatmargin
        $price = '(IFNULL(ps.price, 0) + IFNULL(pas.price, 0))';
        $price .= ($display_method) ? '' : ' * (IFNULL(t.rate, 0)/100 + 1)';

        if (is_numeric($amount->min)) {
            $amounts[] = '(' . $price . ' >= ' . (float)$amount->min . ')';
        }
        if (is_numeric($amount->max)) {
            $amounts[] = '(' . $price . ' <= ' . (float)$amount->max . ')';
        }
        
        if (count($amounts)) {
            $joinAmount = '
                LEFT JOIN ' . _DB_PREFIX_ . 'tax_rule tr ON ps.id_tax_rules_group = tr.id_tax_rules_group 
                    AND tr.id_country = ' . (int) $id_country . '
                LEFT JOIN ' . _DB_PREFIX_ . 'tax t ON tr.id_tax = t.id_tax
            ';
            $whereAmount = ' AND (' . implode(' AND ', $amounts) . ')';
        }
    }
    
    $active_filter = Configuration::get('KERAWEN_CATALOG_INACTIVE') ? '' : 'AND ps.active = 1';
    $q = '
        SELECT
            SQL_CALC_FOUND_ROWS DISTINCT
            p.id_product
        FROM '._DB_PREFIX_.'product p
        INNER JOIN '._DB_PREFIX_.'product_shop ps ON ps.id_product = p.id_product AND ps.id_shop = '.(int)$id_shop.'
        INNER JOIN '._DB_PREFIX_.'product_lang pl ON pl.id_product = p.id_product AND pl.id_lang = '.(int)$id_lang.' AND pl.id_shop = '.(int)$id_shop.'
        LEFT JOIN '._DB_PREFIX_.'product_attribute pa ON pa.id_product = p.id_product
        LEFT JOIN '._DB_PREFIX_.'product_attribute_shop pas ON pas.id_product_attribute = pa.id_product_attribute AND pas.id_shop = '.(int)$id_shop.'
        ' . $joinStock . '
        ' . $joinSupplier . '
        ' . $joinAttribute . '
        ' . $joinFeature . '
        ' . $joinAmount . '
        WHERE TRUE
        '.$active_filter.'
        ' . $whereTerm . '
        ' . $whereManufacturer . '
        ' . $whereStock . '
        ' . $whereSupplier . '
        ' . $whereAttribute . '
        ' . $whereAmount
    ;
    if ((int) $count > 0) {
        $q .= 'LIMIT ' . (int) $from . ', ' . (int) $count;
    }
    
/*
    echo '<pre>';
    print_r($q);
    echo '</pre>';
*/
   
    $pp = $db->executeS($q);
    $total = $db->getValue('SELECT FOUND_ROWS()');
    $products = array();
    if ($pp) {
        foreach ($pp as &$p)
        {
            $prod = new Product($p['id_product'], true, $id_lang, $id_shop);
            if ($prod->active) {
                $detail = detailProduct($prod, $id_lang, false, $id_customer, $id_group);
                //$detail['stockadv'] = advSearchQuantity($context, $detail, $dataSearch);
                if ($detail) $products[] = $detail;
            }
        }
    }

    return array(
        'shorts' => array(),
        'path' => array(),
        'cats' => array(),
        'prods' => array(
            'prods' => $products,
            'total' => $total
        )
    );
    
}


function searchProduct($context, $params)
{    
    $term = $params->dataSearch->term[0]->value;
    $from = $params->from;
    $count = $params->count;
    $id_lang = $context->language->id;
    $id_shop = $context->shop->id;
    
    $id_group = (isset($context->group)) ? $context->group->id : 1;
    $id_customer = (int) $context->customer->id;
    
    // Alternative 1: BO search
    // 	$pp = Product::searchByName($id_lang, $term, $context);
    
    // Alternative 2: FO search
    // 	$term = Tools::replaceAccentedChars($term);
    // 	$pp = Search::find($id_lang, $term, 1, 100, 'position', 'desc', true);
    
    // Alternative 3: Dedicated search
    $db = Db::getInstance();
    
    $where = array();
    foreach (preg_split('/[\s]+/', str_replace('"', '\\"', $term)) as $str)
        if (Tools::strlen($str)) $where[] = 'data LIKE "%'.$str.'%"';
        
        $active_filter = Configuration::get('KERAWEN_CATALOG_INACTIVE') ? '' : 'AND active = 1';
        $pp = $db->executeS('
    		SELECT SQL_CALC_FOUND_ROWS DISTINCT id_product
    		FROM '._DB_PREFIX_.'product_search_kerawen
    		WHERE id_shop = '. (int) $id_shop .'
                '.$active_filter.'
    			AND id_lang = '. (int) $id_lang .'
    			AND '.implode(' AND ', $where).'
    		'.($count ? 'LIMIT '. (int) $from.','. (int) $count : ''));
        $total = $db->getValue('SELECT FOUND_ROWS()');
        
        //$base = $shop->domain . $shop->getBaseURI();
        
        $products = array();
        if ($pp)
            foreach ($pp as &$p)
            {
                $prod = new Product($p['id_product'], true, $id_lang, $id_shop);
                $detail = detailProduct($prod, $id_lang, false, $id_customer, $id_group);
                if ($detail) $products[] = $detail;
            }
        return array(
            'prods' => array(
                'prods' => $products,
                'total' => $total
            )
        );
}


function allShopsQuantity($id_product, $id_product_attribute = 0) {
    $db = Db::getInstance();
    
    $filter = 'TRUE';
    if (Module::isInstalled('kerawen_ocs') && Module::isEnabled('kerawen_ocs')) {
        $virtual_shop = (int)$db->getValue('SELECT id_shop FROM '._DB_PREFIX_.'kerawen_cnc_shop_type WHERE type = "virtual"');
        if ($virtual_shop) $filter = 'sa.id_shop != '.$virtual_shop;
    }
    $q = '
        SELECT SUM(sa.quantity) AS quantity
        FROM '._DB_PREFIX_.'stock_available sa
        WHERE sa.id_product = '.(int)$id_product.'
            AND sa.id_product_attribute = '.(int)$id_product_attribute.'
            AND '.$filter;
    
    return (int) $db->getValue($q);
}


function advSearchQuantity($context, $detail, $dataSearch) {
    
    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
    
    $db = Db::getInstance();
    
    $stock = $detail['stock'];
    
    if ($detail['attrs'] && isset($dataSearch->options)) {
 
        $joinAttribute = '';
        foreach ($dataSearch->options as $key => $attributeGroup) {
            $ids_attr = array();
            foreach ($attributeGroup as $id_attribute) {
                $ids_attr[] = $id_attribute->value;
            }
            $pac = 'pac' . $key;
            $joinAttribute .= '
                INNER JOIN '._DB_PREFIX_.'product_attribute_combination ' . $pac . ' ON sa.id_product_attribute=' . $pac . '.id_product_attribute
                    AND ' . $pac . '.id_attribute IN (' . implode(',', $ids_attr) . ')
                ';
        }

        $q = 'SELECT 
                SUM(sa.quantity) AS quantity
              FROM '._DB_PREFIX_.'stock_available sa
              ' . $joinAttribute . '
              WHERE sa.id_product = ' . (int) $detail['id'] . ' AND  (' . getKerawenStockRestriction() . ')';
        
        return (int) $db->getValue($q);
    }
    
    return $stock;
}




function saveTillSettings($params) {
    if ($params->name != '') {
        Configuration::updateGlobalValue($params->name, $params->settings);
    }
}


function getProdTxtAlt($vals, $alt) {
    foreach ($vals as &$val) { $val = (trim($val) == '') ? $alt : $val; }
    return $vals;
}


function saveProduct($params)
{

    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
    require_once(_KERAWEN_API_DIR_.'/constants.php');
    $db = Db::getInstance();
    
    $context = Context::getContext();
    $id_lang = (int) $context->language->id;
    
    $data = $params->product;
    $params->product->is_new = ((int) $data->id == 0);

    $id_customer_init = (int) $db->getValue('SELECT id_customer FROM '._DB_PREFIX_.'product_wm_kerawen WHERE id_product = ' . (int) $data->id);
    
    $id_tax_rules_group = ((int) $data->vat_margin) ? (int) Configuration::get('KERAWEN_TAX_VATMARGIN') : (int) $data->id_taxrule;
    
    require_once(_KERAWEN_CLASS_DIR_.'/data.php');
    require_once(_KERAWEN_CLASS_DIR_.'/KerawenTax.php');
    $params->tax = TaxManagerFactory::getManager(getDefaultDeliveryAddress(true), $id_tax_rules_group);
    $price = round($params->tax->getTaxCalculator()->removeTaxes($data->price), 6);
    
    $wholesale_price = $data->wholesale_price;
    
    $current_shop = new shop($params->id_shop);
    
    // Backup shop context
    $backup = backupShopContext();
    
    if ((int) $data->multishops) {
        $shops = $db->executeS('SELECT id_shop FROM '._DB_PREFIX_.'shop WHERE active = 1');
        Shop::setContext(Shop::CONTEXT_ALL);
    } else {
        $shops = $db->executeS('SELECT id_shop FROM '._DB_PREFIX_.'shop WHERE id_shop = ' . (int) $current_shop->id);
        Shop::setContext(Shop::CONTEXT_SHOP, $current_shop->id);
    }

    $prod = new Product($data->id, false, null, null);
    $prod->ean13 = $data->code;
    $prod->upc = $data->upc;
    $prod->reference = $data->reference;
    $prod->id_tax_rules_group = $id_tax_rules_group;
    $prod->price = $price;
    $prod->wholesale_price = round($wholesale_price, 6);
    $prod->id_supplier = (int) $data->supplier; //last supplier selected ?
    $prod->date_upd = date('Y-m-d H:i:s');
    $prod->visibility = $data->visibility;
    $prod->minimal_quantity = max(1, (int) $data->minimal_quantity);

    $prod->width = $data->width == '' ? '' : round($data->width, 6);
    $prod->height = $data->height == '' ? '' : round($data->height, 6);
    $prod->depth = $data->depth == '' ? '' : round($data->depth, 6);
    $prod->weight = $data->weight == '' ? '' : round($data->weight, 6);
    
    $data->name = getKerawenObjectToArray($data->name);
    $data->description_short = getKerawenObjectToArray($data->description_short);
    $data->description = getKerawenObjectToArray($data->description);
    $data->id_lang = (int) $data->id_lang;
    
    $default_name = isset($data->name[$data->id_lang]) ? $data->name[$data->id_lang] : '';
    $default_description_short = isset($data->description_short[$data->id_lang]) ? $data->description_short[$data->id_lang] : '';
    $default_description = isset($data->description[$data->id_lang]) ? $data->description[$data->id_lang] : '';
    //if $data->id > 0 allow empty for description_short and description?

    $prod->name = getProdTxtAlt($data->name, $default_name);
    $prod->description_short = getProdTxtAlt($data->description_short, $default_description_short);
    $prod->description = getProdTxtAlt($data->description, $default_description);
    $prod->link_rewrite = array_map('Tools::link_rewrite', $prod->name);

    if ((int) $data->id === 0) {
        $prod->id_category_default = (isset($data->id_cat) && (int) $data->id_cat > 0) ? $data->id_cat : $current_shop->getCategory();
        $prod->date_add = date('Y-m-d H:i:s');
        if (Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT') && Configuration::get('PS_FORCE_ASM_NEW_PRODUCT')) {
            $prod->advanced_stock_management = 1;
            $prod->depends_on_stock = 1;
        }
    }
    
    //manufacturer
    if ((int) $data->manufacturer) {
        $prod->id_manufacturer = $data->manufacturer;
    }
    

    $prod->save();
    $id_prod = $prod->id;
    
    //supplier
    if ((int) $data->supplier) {
        $prod->addSupplierReference((int)$data->supplier, 0);
    }

    foreach ($shops as &$s)
    {
        $id_shop = $s['id_shop'];
        $prod->associateTo((int)$id_shop);
        
        $shop = new Shop((int)$id_shop);
        Shop::setContext(Shop::CONTEXT_SHOP, $id_shop);
        
        $ps = new Product($id_prod, false, null, $id_shop);

        if ((int) $data->id === 0) {
            //Apply category to each shop
            $id_cat = $shop->getCategory();
            if (isset($data->id_cat) && (int) $data->id_cat > 0) {
                //category available for this shop ?
                if ( (int) $db->getValue('SELECT `id_category` FROM `'._DB_PREFIX_.'category_shop` WHERE `id_shop` = ' . (int) $id_shop . ' AND  `id_category` = ' . (int) $data->id_cat) ) {
                    $id_cat = $data->id_cat;
                }
            }
            $ps->category_default = $id_cat;
            
            
            StockAvailable::setProductDependsOnStock($id_prod, $prod->depends_on_stock, $id_shop);
            
            try
            {
                $db->insert('category_product', array(
                    'id_category' => (int) $id_cat,
                    'id_product' => (int) $id_prod,
                    'position' => (int) $db->getValue('
    					SELECT MAX(`position`)
    					FROM `'._DB_PREFIX_.'category_product`
    					WHERE `id_category` = '. (int) $id_cat),
                ));
            }
            catch (Exception $e)
            {
                //I'm laid back
            }
        }
        
        $ps->id_tax_rules_group = $id_tax_rules_group;
        $ps->price = $price;
        $ps->wholesale_price = round($wholesale_price, 6);
        $ps->visibility = $data->visibility;
        $ps->date_upd = date('Y-m-d H:i:s');
        $ps->date_add = ($ps->date_add == '0000-00-00 00:00:00') ? date('Y-m-d H:i:s') : $ps->date_add;
        $ps->minimal_quantity = max(1, (int) $data->minimal_quantity);
        $ps->save();
        
        saveLowStock($ps->id, 0, $data->low_stock, $data->packing, $data->target, $data->location, array($id_shop));
        
    }

    $params->product->id = $prod->id;
    $prdAttr = saveProductAttrs($params);
    
    // Restore shop context
    restoreShopContext($backup);
    $id_shop = $current_shop->id;

    if ($data->is_new && count((array)$data->attrs) == 0) {
        saveProductStock($prod, 0, $data->qty, $id_shop);
    }
    
    $values = array(
        'id_customer' => (int) $data->wm->id_customer,
        'prod_note' => pSQL(trim($data->wm->prod_note)),
        'is_gift_card' => (int) $data->wm->is_gift_card,
        'is_account_credit' => (int) $data->wm->is_account_credit,
        'rest_voucher' => (int) $data->wm->rest_voucher,
        'is_free_price' => (int) $data->wm->is_free_price,
        'measured' => (int) $data->wm->measured,
        'unit' => ((int) $data->wm->measured) ? pSQL($data->wm->unit) : '',
        'precision' => ((int) $data->wm->measured) ? (int) $data->wm->precision : 0,
        'margin_calc' => (int) $data->margin_calc,
        'product_type' => pSQL(trim($data->wm->product_type)),
        'margin_free_price' => $data->wm->margin_free_price == '' ? null : (float) $data->wm->margin_free_price,
    );


    $clear_wm = false;
    if (   $values['id_customer'] == 0 
        && $values['prod_note'] == ''
        && $values['is_gift_card'] == 0
        && $values['is_account_credit'] == 0
        && $values['rest_voucher'] == 0
        && $values['is_free_price'] == 0
        && $values['measured'] == 0
        && $values['margin_calc'] == (int) Configuration::get('KERAWEN_MARGIN_CALC')
        && $values['product_type'] == _KERAWEN_TYPE_STANDARD_
    ) {
        $clear_wm = true;
    }

    if ($clear_wm) {
        $db->delete('product_wm_kerawen', 'id_product = ' . (int) $id_prod);
    } else {
        $wm_exists = $db->getValue('SELECT 1 FROM '._DB_PREFIX_.'product_wm_kerawen WHERE id_product = ' . (int) $id_prod);
        if ($wm_exists) {
            $db->update('product_wm_kerawen', $values, 'id_product = ' . (int) $id_prod, 0, true);
        } else {
            $values['id_product'] = (int) $id_prod;
            $db->insert('product_wm_kerawen', $values, true);
        }
    }

    $list_code = array(0);
    if ((int) $data->wm->measured && !$clear_wm) {
        foreach($data->wm->codes as $item) {

            $idpa = getIdProdAttrByAttr($prod->id, $item->product_attribute, $prod->id_shop_default);

            $values = array(
                'code' => pSQL($item->code),
                'unit_price' => (float) $item->unit_price_te,
                'id_product_attribute' => $idpa ? $idpa : -1,
                'id_product' => (int) $id_prod,
            );
            if ((int) $item->id_code) {
                $db->update('product_wm_code_kerawen', $values, 'id_code = ' . (int) $item->id_code);
                $list_code[] = (int) $item->id_code;
            } else {
                $db->insert('product_wm_code_kerawen', $values);
                $list_code[] = (int) $db->Insert_ID();
            }
        }
    }
    
    $db->delete('product_wm_code_kerawen', 'id_product = ' . (int) $id_prod . ' AND id_code NOT IN (' . implode(',',$list_code) . ')');

    
    if ((int) $data->wm->id_customer > 0 && $id_customer_init != (int) $data->wm->id_customer) {
        sendEmailToReseller($prod, $data->wm->id_customer, 'added');
    }
    

    return $prod;
}

function sendEmailToReseller($prod, $id_customer, $action) {

    $cust = getProductCustomerExtension($id_customer);
   
    if ($cust && (int) $cust['fakemail'] == 0) {

        //$context = Context::getContext();
        //$id_lang = $context->language->id;
        //$id_shop = $context->shop->id;
        
        $id_shop = (int) Configuration::get('PS_SHOP_DEFAULT');
        $id_lang = (int) Configuration::get('PS_LANG_DEFAULT', null, null, $id_shop);
        
        $link = new Link;
        $shop_url = $link->getPageLink('index', true, $id_lang, null, false, $id_shop);
        $shop_name = Configuration::get('PS_SHOP_NAME', null, null, $id_shop);
    
        $logo = '';
        if (Configuration::get('PS_LOGO_MAIL') !== false && file_exists(_PS_IMG_DIR_.Configuration::get('PS_LOGO_MAIL', null, null, $id_shop))) {
            $logo = _PS_IMG_DIR_ . Configuration::get('PS_LOGO_MAIL', null, null, $id_shop);
        } else {
            if (file_exists(_PS_IMG_DIR_ . Configuration::get('PS_LOGO', null, null, $id_shop))) {
                $logo = _PS_IMG_DIR_ . Configuration::get('PS_LOGO', null, null, $id_shop);
            }
        }
        
        $shop_address =
        Configuration::get('PS_SHOP_NAME', null, null, $id_shop) . ' - ' .
        Configuration::get('PS_SHOP_ADDR1', null, null, $id_shop) . ' - ';
        if (Configuration::get('PS_SHOP_ADDR2', null, null, $id_shop) != '') {
            $shop_address .= Configuration::get('PS_SHOP_ADDR2', null, null, $id_shop) . ' - ';
        }
        $shop_address .=
        Configuration::get('PS_SHOP_CODE', null, null, $id_shop) . ' ' . Configuration::get('PS_SHOP_CITY', null, null, $id_shop) . ' - ' .
        Country::getNameById($id_lang, Configuration::get('PS_SHOP_COUNTRY_ID', null, null, $id_shop));

        $product = detailProduct($prod, $id_lang, false);
        $prod_name = isset($product['name'][$id_lang]) ? $product['name'][$id_lang] : '';
        $prod_wholesale_price = $product['wholesale_price'];
        $prod_price_te = $product['price_te'];
        $prod_price_ti = $product['price'];

        if ($action == 'added') {
            $mailTitle = 'Votre produit a été ajouté au catalogue';
        } else {
            $mailTitle = 'Votre produit a été vendu';
        }

        $dataMail = array(
            '{shop_name}' => $shop_name,
            '{shop_url}' => $shop_url,
            '{shop_email}' => Configuration::get('PS_SHOP_EMAIL', null, null, $id_shop),
            '{shop_logo}' => $logo,
            '{name_to}' => $cust['firstname'] . ' ' . $cust['lastname'],
            '{shop_link}' => '',
            '{shop_address}' => $shop_address,
            '{prod_name}' => $prod_name,
            '{prod_wholesale_price}' => $prod_wholesale_price,
            '{prod_price_te}' =>  $prod_price_te,
            '{prod_price_ti}' =>  $prod_price_ti,
            '{mail_title}' =>  $mailTitle,
        );
        
        $context = Context::getContext();
        $ps_mail_method = (isset($context->kerawen->PS_MAIL_METHOD) && (int) $context->kerawen->PS_MAIL_METHOD > 0) ? $context->kerawen->PS_MAIL_METHOD : Configuration::get('PS_MAIL_METHOD');
        $ps_mail_method_active = Configuration::get('PS_MAIL_METHOD');
        //Used original method
        Configuration::set('PS_MAIL_METHOD', $ps_mail_method);
        
        Mail::Send(
            $id_lang,
            'newreseller',
            ' - ' . $mailTitle,
            $dataMail,
            $cust['email'],
            NULL,
            Configuration::get('PS_SHOP_EMAIL', null, null, $id_shop),
            Configuration::get('PS_SHOP_NAME', null, null, $id_shop),
            false,
            NULL,
            _KERAWEN_DIR_ . '/mails/'
        );
        
        //Restore
        Configuration::set('PS_MAIL_METHOD', $ps_mail_method_active);
    
    }
}


function saveProductStock($prod, $attr, $qty, $id_shop) {
    if ((int) $qty > 0) {
        $id_warehouse = 0;
        if ($prod->advanced_stock_management) {
            $id_warehouse = (int)Configuration::get('PS_DEFAULT_WAREHOUSE_NEW_PRODUCT');
            
            if (!$id_warehouse) {
                //Select first warehouse available
                $id_warehouse = (int) $db->getValue('
                    SELECT id_warehouse FROM '._DB_PREFIX_.'warehouse_shop
                    WHERE id_shop = ' . (int) $id_shop . '
                    ORDER BY id_warehouse
                ');
            }
            
            if ($id_warehouse) {
                $loc = new WarehouseProductLocation();
                $loc->id_product = $id_prod;
                $loc->id_product_attribute = $attr;
                $loc->id_warehouse = $id_warehouse;
                $loc->location = "";
                $loc->save();
            }
        }
        
        // Set initial quantity
        if ($id_warehouse) {
            $stock = new Stock();
            $stock->id_warehouse = $id_warehouse;
            $stock->id_product = $prod->id;
            $stock->id_product_attribute = $attr;
            $stock->reference = $prod->reference;
            $stock->ean13 = $prod->ean13;
            $stock->upc = $prod->upc;
            $stock->physical_quantity = 0;
            $stock->usable_quantity = 0;
            $stock->price_te = 0.001;
            $stock->save();
            
            require_once(_KERAWEN_CLASS_DIR_.'/stock.php');
            $id_reason = Configuration::get('PS_STOCK_MVT_INC_REASON_DEFAULT');
            injectStock($prod->id, $attr, $id_warehouse, $id_shop, $id_reason, $qty, 0);
        }
        else {
            StockAvailable::setQuantity($prod->id, $attr, $qty, $id_shop);
        }
    }
}


function saveProductAttrs($params) {
    
    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
    $db = Db::getInstance();
    
    /*
    echo '<pre style="text-align:left">';
    print_r($params);
    echo '</pre>';
    */
    
    $data = $params->product;
    
    $context = Context::getContext();
    $id_lang = $context->language->id;
    
    $current_shop = new shop($params->id_shop);
        
    if ((int) $data->multishops) {
        $shops = array();
        $res = $db->executeS('SELECT id_shop FROM '._DB_PREFIX_.'shop WHERE deleted = 0 AND active = 1');
        foreach ($res as $s) {
            $shops[] = $s['id_shop'];
        }
    } else {
        $shops = array($current_shop->id);
    }
    
    $db->execute('UPDATE ' . _DB_PREFIX_ . 'product_attribute SET default_on = NULL WHERE default_on = 1 AND id_product = ' . (int) $data->id);
    $db->execute('UPDATE ' . _DB_PREFIX_ . 'product_attribute_shop SET default_on = NULL WHERE default_on = 1 AND id_product = ' . (int) $data->id . ' AND id_shop IN(' . dataListToIntList($shops) . ')');
    
    $prod = new Product($data->id, false, null, null);
    
    $ids_pa = array();
    
    $idpa_default_on = array();
    
    foreach ($data->attrs as $attr) {
        
        if ((int) $attr->id_product_attribute) {
            
            $ids_pa[] = $attr->id_product_attribute;
            
            $list_id_prd_attr = array();
            $list_id_prd_attr[$attr->id_product_attribute] = array();
            array_push($list_id_prd_attr[$attr->id_product_attribute], $current_shop->id);
            if ((int) $attr->default_on) {
                $idpa_default_on[$current_shop->id] = $attr->id_product_attribute;
            }
            
            if (count($shops) > 1) {
                $ids_attr = $db->getValue('SELECT GROUP_CONCAT(id_attribute) FROM '._DB_PREFIX_.'product_attribute_combination WHERE id_product_attribute = ' . (int) $attr->id_product_attribute);
                if ($ids_attr) {
                    foreach ($shops as $id_shop) {
                        if ((int) $current_shop->id != (int) $id_shop) {
                            $ipa = getIdProdAttrByAttr($prod->id, $ids_attr, $id_shop);
                            if ($ipa) {
                                //Update
                                if (!isset($list_id_prd_attr[$ipa])) { $list_id_prd_attr[$ipa] = array();}
                                array_push($list_id_prd_attr[$ipa], $id_shop);
                                if ((int) $attr->default_on) {
                                    $idpa_default_on[$id_shop] = $attr->id_product_attribute;
                                }
                            } else {
                                //Insert if dosesn't exist
                                $ipa = addIdProductAttr($prod, $attr, $ids_attr, array($id_shop), $params->tax);
                            }
                            
                            if ($data->is_new && (int) $ipa) {
                                saveProductStock($prod, $ipa, $data->qty, $id_shop);
                            }
                        }
                    }
                }
            }
            
            foreach ($list_id_prd_attr as $id_product_attribute => $id_shop_list) {
                updateIdProductAttr($prod, $attr, $id_product_attribute, $id_shop_list, $params->tax);
            }
            
        } else {
            
            $insertAttrShop = array();
            
            //Should be single
            foreach ($shops as $id_shop) {
                $ipa = (count($shops) > 1) ? getIdProdAttrByAttr($prod->id, $attr->id, $id_shop) : 0;
                if ($ipa) {
                    //Could be different with same combination for each shop
                    updateIdProductAttr($prod, $attr, $ipa, array($id_shop), $params->tax);
                } else {
                    array_push($insertAttrShop, $id_shop);
                }
            }
            
            if (count($insertAttrShop)) {
                $ipa = addIdProductAttr($prod, $attr, $attr->id, $insertAttrShop, $params->tax);
                $ids_pa[] = $ipa;
                
                if ($data->is_new && (int) $ipa) {
                    saveProductStock($prod, $ipa, $data->qty, $shops[0]);
                }
            }
            
        }
        
        $ids_pa = array_unique($ids_pa);
        
    }
    
    
    //default attribute //only for update ?
    foreach ($idpa_default_on as $id_shop => $idpa) {
        $db->execute('UPDATE ' . _DB_PREFIX_ . 'product_attribute SET default_on = 1 WHERE id_product_attribute = ' . (int) $idpa);
        $db->execute('UPDATE ' . _DB_PREFIX_ . 'product_attribute_shop SET default_on = 1 WHERE id_product_attribute = ' . (int) $idpa . ' AND id_shop = ' . (int) $id_shop);
        $prod->setDefaultAttribute($idpa);
        Product::updateDefaultAttribute($prod->id);
    }
    
    
    //Items to delete
    //Combinations could be the same between shops but with a different id_product_attribute !!!
    //Do not used as bellow:
    //$combination = new Combination($id_product_attribute);
    //$combination->deleteAssociations();
    
    $not_in = count($ids_pa) ? ' AND id_product_attribute NOT IN (' . dataListToIntList($ids_pa) . ') ' : '';
    
    $res = $db->executeS('
        SELECT
            id_product_attribute
        FROM '._DB_PREFIX_.'product_attribute_shop
        WHERE id_shop = ' . (int) $current_shop->id . '
        AND id_product = ' . (int) $prod->id . '
        ' . $not_in . '
    ');
    
    foreach ($res as $item) {
        deleteProdAttr($prod->id, $item['id_product_attribute'], $current_shop->id);
    }
    
    
    //Clean for other shops
    if (count($shops) > 1) {
        
        $q = '
            SELECT
                pas.id_product_attribute,
                (
                    SELECT
                        GROUP_CONCAT(pac.id_attribute ORDER BY pac.id_attribute ASC)
                    FROM '._DB_PREFIX_.'product_attribute_combination pac
                    WHERE pac.id_product_attribute = pas.id_product_attribute
                ) AS attrs
                        
            FROM '._DB_PREFIX_.'product_attribute_shop pas
            WHERE pas.id_product = ' . (int) $prod->id
        ;
            
        $res = $db->executeS($q .' AND pas.id_shop = ' . (int) $current_shop->id);
        $attrs = array();
        foreach ($res as $item) {
            $attrs[] = $item['attrs'];
        }
            
        foreach ($shops as $id_shop) {
            $res = $db->executeS($q .' AND pas.id_shop = ' . (int) $id_shop);
            foreach ($res as $item) {
                if (!in_array($item['attrs'], $attrs)) {
                    deleteProdAttr($prod->id, $item['id_product_attribute'], (int) $id_shop);
                }
            }
        }
    }
    
    
    if (count((array) $data->attrs) == 0) {
        $db->execute('UPDATE ' . _DB_PREFIX_ . 'product SET cache_default_attribute = 0 WHERE id_product = ' . (int) $prod->id);
        foreach ($shops as $id_shop) {
            $db->execute('UPDATE ' . _DB_PREFIX_ . 'product_shop SET cache_default_attribute = 0 WHERE id_product = ' . (int) $prod->id . ' AND id_shop = ' . (int) $id_shop);
        }
    }
    
    return $prod;
}



function updateAltValue($v, $alt = 0) {
    return ($v == '') ? $alt : $v;
}

function updateIdProductAttr($prod, $attr, $id_product_attribute, $shops, $tax) {
    
    $db = Db::getInstance();

    $prod->updateAttribute(
        $id_product_attribute, //$id_product_attribute
        updateAltValue($attr->wholesale_price), //$wholesale_price
        updateAltValue(round($tax->getTaxCalculator()->removeTaxes((float) $attr->price_ti), 6)), //$price
        updateAltValue($attr->weight), //$weight
        updateAltValue($attr->unit_price_impact), //unit_price_impact
        updateAltValue($attr->ecotax), //$ecotax
        null, //$id_images
        $attr->reference, //$reference
        $attr->ean13, //$ean13
        (int) $attr->default_on ? 1 : null,
        $attr->location, //$location
        $attr->upc, //$upc
        updateAltValue($attr->minimal_quantity, 1), //$minimal_quantity
        updateAltValue($attr->available_date, '0000-00-00'), //$available_date
        false, //$update_all_fields
        $shops //$id_shop_list = array()
    );
    

    //FIX: table ps_product_attribute otherwise get 0 to id_product when multishops, bug PS ?
    $db->update('product_attribute', array('id_product' => $prod->id), 'id_product = 0 AND id_product_attribute = ' . (int) $id_product_attribute);
    
    saveLowStock($prod->id, $id_product_attribute, $attr->low_stock, $attr->packing, $attr->target, $attr->location, $shops);
}

function addIdProductAttr($prod, $attr, $attrs, $shops, $tax) {
    
    $id_product_attribute = (int) $prod->addCombinationEntity(
        updateAltValue($attr->wholesale_price), //$wholesale_price
        updateAltValue(round($tax->getTaxCalculator()->removeTaxes($attr->price_ti), 6)), //$price
        updateAltValue($attr->weight), //$weight
        updateAltValue($attr->unit_price_impact), //$unit_price_impact
        updateAltValue($attr->ecotax), //$ecotax
        1, //$quantity
        null, //$id_images
        $attr->reference, //$reference,
        null, //$id_supplier,
        $attr->ean13, //$ean13,
        (int) $attr->default_on ? 1 : null, //$default,
        $attr->location, //$location
        $attr->upc, //$upc
        updateAltValue($attr->minimal_quantity, 1), //$minimal_quantity
        $shops, //array $id_shop_list = array(),
        updateAltValue($attr->available_date, '0000-00-00')
    );
        
    $combination = new Combination($id_product_attribute);
    $combination->setAttributes(explode(",", $attrs));
    
    saveLowStock($prod->id, $id_product_attribute, $attr->low_stock, $attr->packing, $attr->target, $attr->location, $shops);
    
    return $id_product_attribute;
    
}


function saveLowStock($id_prod, $id_product_attribute, $low_stock, $packing, $target, $location, $shops) {
 
    $db = Db::getInstance();
    
    if ((int) $id_product_attribute) {
        $table = 'product_attribute';
        $where = 'id_product_attribute = ' . (int) $id_product_attribute;
    } else {
        $table = 'product';
        $where = 'id_product = ' . (int) $id_prod;
    }
    
    $ps_location = (Tools::version_compare(_PS_VERSION_, '8', '>=')) ? '' : ', location = "' . pSQL($location) . '"';
        
    
    
    $low_stock = is_numeric($low_stock) && (int) $low_stock >= 0 ? $low_stock : 'NULL';

    $db->execute('
        UPDATE '._DB_PREFIX_.$table.'_shop
        SET low_stock_threshold = ' . $low_stock . '
        WHERE id_shop IN(' . dataListToIntList($shops) . ')' . ' AND ' . $where
    );


    $db->execute('
        UPDATE '._DB_PREFIX_.$table.'
        SET low_stock_threshold = ' . $low_stock . ' ' . $ps_location .  '
        WHERE ' . $where
    );

    if (trim($packing) == '' && trim($target == '')) {
        $db->delete('product_attr_kerawen', 'id_product= ' . (int) $id_prod . ' AND id_product_attribute = ' . (int) $id_product_attribute . ' AND id_shop IN(' . dataListToIntList($shops) . ')');
        
    } else {
    
        foreach ($shops as $id_shop) {
            $db->execute('
                INSERT INTO '._DB_PREFIX_.'product_attr_kerawen (id_shop, id_product, id_product_attribute, packing, target)
                VALUES (
                    ' . (int) $id_shop . ',
                    ' . (int) $id_prod . ',
                    '. (int) $id_product_attribute .',
                    "'. pSQL($packing) .'",
                    "'. pSQL($target) .'"
                ) ON DUPLICATE KEY UPDATE
                	packing = VALUES(packing),
                    target = VALUES(target)
            ');
        }

    }
            

    //Location PS 1.7 -> should be standard stock
    if (Tools::version_compare(_PS_VERSION_, '1.7', '>=')) {
        foreach ($shops as $id_shop) { 
            $out_of_stock = Configuration::get('PS_ORDER_OUT_OF_STOCK');
            
            //Should be a problem if shared stocks
            $id_shop_group = 0;

            $db->execute('
                INSERT INTO '._DB_PREFIX_.'stock_available (id_shop, id_product, id_product_attribute, id_shop_group, out_of_stock, location)
                VALUES (
                    ' . (int) $id_shop . ',
                    ' . (int) $id_prod . ',
                    ' . (int) $id_product_attribute .',
                    ' . (int) $id_shop_group . ',
                    ' . (int) $out_of_stock . ',
                    "' . pSQL($location) . '"
                ) ON DUPLICATE KEY UPDATE
                	location = VALUES(location)
            ');    
        }
    }

}

/*
function getIdProdAttrByAttrName($id_prod, $attrs_name, $id_shop) {
    
    $context = Context::getContext();
    $id_lang = $context->language->id;

    $cnt_attr = count(explode(',', $attrs_name));
 
    $q = '
        SELECT 
            pas.id_product_attribute
        FROM '._DB_PREFIX_.'product_attribute_shop pas
        INNER JOIN '._DB_PREFIX_.'product_attribute_combination pac on pac.id_product_attribute =  pas.id_product_attribute
        INNER JOIN '._DB_PREFIX_.'attribute_lang al on al.id_attribute = pac.id_attribute and al.id_lang = ' . $id_lang . ' AND al.name IN (' . $attrs_name . ')
        WHERE pas.id_shop = ' . (int) $id_shop . ' AND pas.id_product = ' . (int) $id_prod . '
        GROUP BY pas.id_product_attribute
        HAVING COUNT(pas.id_product_attribute) = ' . $cnt_attr;
    
    return (int) Db::getInstance()->getValue($q);

}
*/

function getProdAttrComByIdProdAttr($id_product_attribute) {
    $q = '
        SELECT 
            GROUP_CONCAT(pac.id_attribute ORDER BY ag.position ASC) AS product_attribute
        FROM '._DB_PREFIX_.'product_attribute_combination pac
        INNER JOIN '._DB_PREFIX_.'attribute a on a.id_attribute = pac.id_attribute
        INNER JOIN '._DB_PREFIX_.'attribute_group ag on ag.id_attribute_group = a.id_attribute_group
        WHERE pac.id_product_attribute = ' . (int) $id_product_attribute . '
        GROUP BY pac.id_product_attribute
    ';
    
    return (String) Db::getInstance()->getValue($q);
}

function getIdProdAttrByAttr($id_prod, $ids_attr, $id_shop) {
    $db = Db::getInstance();
    
    if ($ids_attr == "") return 0;
    
    $cnt_attr = count(explode(',', $ids_attr));

    $q = '
        SELECT
            pas.id_product_attribute
        FROM '._DB_PREFIX_.'product_attribute_shop pas
        INNER JOIN '._DB_PREFIX_.'product_attribute_combination pac ON pac.id_product_attribute = pas.id_product_attribute AND id_attribute IN (' . $ids_attr . ')
        WHERE pas.id_shop = ' . (int) $id_shop . ' AND pas.id_product = ' . (int) $id_prod . '
        GROUP BY id_product_attribute
        HAVING COUNT(pas.id_product_attribute) = ' . $cnt_attr;

    return (int) Db::getInstance()->getValue($q);
}


function deleteProdAttr($id_product, $id_product_attribute, $id_shop) {
    $db = Db::getInstance();

    if ((int) $id_shop === 0 || (int) $db->getValue('SELECT id_product_attribute FROM '._DB_PREFIX_.'product_attribute_shop WHERE id_shop != ' . (int) $id_shop . ' AND id_product_attribute = ' . (int) $id_product_attribute) === 0) {
        $db->delete('product_attribute', 'id_product_attribute = '.  (int) $id_product_attribute);
        $db->delete('product_attribute_combination', 'id_product_attribute = '.  (int) $id_product_attribute);
        $db->delete('product_attribute_image', 'id_product_attribute = '.  (int) $id_product_attribute);
    }
    
    $where  = 'id_product = ' . (int) $id_product . ' AND id_product_attribute = ' . (int) $id_product_attribute;
    $where .=  (int) $id_shop === 0 ? '' : ' AND id_shop = ' . (int) $id_shop;

    $db->delete('product_attribute_shop', $where);
    $db->delete('cart_product', $where);
    
    $db->delete('product_attr_kerawen', $where);
    
    $shop = ((int) $id_shop) ? new Shop((int) $id_shop) : null;
    StockAvailable::removeProductFromStockAvailable($id_product, $id_product_attribute, $shop);
}


function createProduct($data)
{
    $db = Db::getInstance();
    $context = Context::getContext();
    $id_lang = $context->language->id;
    $current_shop = $context->shop;
    
    // Backup shop context
    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
    $backup = backupShopContext();
    
    // For all shops
    Shop::setContext(Shop::CONTEXT_ALL);
    
    $prod = new Product(null, false, null, null);
    
    $prod->name = getForLanguages($data->name);
    $prod->reference = 'NEW';
    $prod->visibility = 'none';
    $prod->ean13 = $data->code;
    $prod->reference = $data->reference;
    $prod->id_tax_rules_group = $data->id_taxrule;
    $prod->price = round($data->price / (1.0 + $prod->getTaxesRate() / 100.0), 6);
    $prod->wholesale_price = $data->wholesale_price;
    $prod->link_rewrite = getForLanguages(Tools::link_rewrite($data->name));
    $prod->id_category_default = isset($data->id_cat) ? $data->id_cat : $current_shop->getCategory();
    $prod->date_add = date('Y-m-d H:i:s');
    $prod->date_upd = date('Y-m-d H:i:s');
    if (Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT') && Configuration::get('PS_FORCE_ASM_NEW_PRODUCT')) {
        $prod->advanced_stock_management = 1;
        $prod->depends_on_stock = 1;
    }

    //manufacturer
    if ($data->manufacturer) {
        $prod->id_manufacturer = $data->manufacturer;
    }
    
    $prod->save();
    $id_prod = $prod->id;
    
    //supplier    
    if ($data->supplier) {
        $prod->addSupplierReference((int)$data->supplier, 0);
    }

    // Init stock
    $id_warehouse = 0;
    if ($prod->advanced_stock_management) {
        $id_warehouse = (int)Configuration::get('PS_DEFAULT_WAREHOUSE_NEW_PRODUCT');
        
        if (!$id_warehouse) {
            //Select first warehouse available
            $id_warehouse = (int) $db->getValue('
                SELECT id_warehouse FROM '._DB_PREFIX_.'warehouse_shop
                WHERE id_shop = ' . (int) $current_shop->id . '
                ORDER BY id_warehouse
            ');
        }
        
        if ($id_warehouse) {
            $loc = new WarehouseProductLocation();
            $loc->id_product = $id_prod;
            $loc->id_product_attribute = 0;
            $loc->id_warehouse = $id_warehouse;
            $loc->location = pSQL('');
            $loc->save();
        }
    }
    
    // For each shop
    $shops = Db::getInstance()->executeS('
		SELECT id_shop FROM '._DB_PREFIX_.'shop WHERE active = 1');
    foreach ($shops as &$s)
    {
        $id_shop = $s['id_shop'];
        $prod->associateTo((int)$id_shop);
        
        $shop = new Shop((int)$id_shop);
        Shop::setContext(Shop::CONTEXT_SHOP, $id_shop);
        
        //Apply category to each shop
        $id_cat = $shop->getCategory();
         if (isset($data->id_cat)) {
            //category available for this shop ?
            if ( (int) $db->getValue('SELECT `id_category` FROM `'._DB_PREFIX_.'category_shop` WHERE `id_shop` = ' . (int) $id_shop . ' AND  `id_category` = ' . (int) $data->id_cat) ) {
                 $id_cat = $data->id_cat;
            }
         }

        $ps = new Product($id_prod, false, null, $id_shop);
        $ps->category_default = $id_cat;
        $ps->save();
        
        StockAvailable::setProductDependsOnStock($id_prod, $prod->depends_on_stock, $id_shop);
        
        try
        {
            $db->insert('category_product', array(
                'id_category' => (int) $id_cat,
                'id_product' => (int) $id_prod,
                'position' => (int) $db->getValue('
					SELECT MAX(`position`)
					FROM `'._DB_PREFIX_.'category_product`
					WHERE `id_category` = '. (int) $id_cat),
            ));
        }
        catch (Exception $e)
        {
            //I'm laid back
        }
    }
    
    // Restore shop context
    restoreShopContext($backup);
    $id_shop = $current_shop->id;
    
    // Set initial quantity
    if ($id_warehouse) {
        $stock = new Stock();
        $stock->id_warehouse = $id_warehouse;
        $stock->id_product = $id_prod;
        $stock->id_product_attribute = 0;
        $stock->reference = $prod->reference;
        $stock->ean13 = $prod->ean13;
        $stock->upc = $prod->upc;
        $stock->physical_quantity = 0;
        $stock->usable_quantity = 0;
        $stock->price_te = 0.001;
        $stock->save();
        
        require_once(_KERAWEN_CLASS_DIR_.'/stock.php');
        $id_reason = Configuration::get('PS_STOCK_MVT_INC_REASON_DEFAULT');
        injectStock($id_prod, 0, $id_warehouse, $id_shop, $id_reason, $data->qty, 0);
    }
    else {
        StockAvailable::setQuantity($id_prod, 0, $data->qty, $id_shop);
    }
    
    if (isset($data->product_type)) {
        $values['id_product'] = (int) $id_prod;
        $values['product_type'] = pSQL(_KERAWEN_TYPE_REFUND_);
        $db->insert('product_wm_kerawen', $values);
    }

    return $prod;
}



function createProduct2($context, $params, &$response)
{
	$prod = $params->product;
	$id_lang = $context->language->id;
	$id_shop = null;
	$context = null;
	//Variation Mofidication
	if (isset($prod->id_pa) && $prod->id_pa != 0 && isset($prod->id) && $prod->id != 0)
	{
		//Combination modification only (not creation)

		$c = new Combination($prod->id_pa, null, $id_shop);//EVOL add id shop;
		$p = new Product($prod->id, false, $id_lang, $id_shop, $context);//not fully loaded

		$c->price = $prod->base - $p->price;
		if ($prod->ecotax == $p->ecotax)
			$c->ecotax = 0;
		else
		{
			if ($prod->ecotax == 0 && $p->ecotax != 0)
				$c->ecotax = '0.000001';//forcing the combination ecotax to 0
			else
				$c->ecotax = $prod->ecotax;
			$c->price += Tools::ps_round(($prod->ecotax - $p->ecotax) * (1 + Tax::getProductEcotaxRate() / 100) / (1 + $p->getTaxesRate() / 100), 6);
		}
		$c->ean13 = $prod->code;
		$c->reference = $prod->ref;
		$c->wholesale_price = Tools::ps_round(isset($prod->wholesale) ? $prod->wholesale - $p->wholesale_price :0, 6);
		$c->save();
		// 		var_dump($c);
		// 		die();
		//EVOL add price
	}
	//Product
	else
	{
		$p = new Product(isset($prod->id)?$prod->id:null, false, $id_lang);
		$p->name = $prod->name;
		$p->reference = $prod->ref;
		$p->visibility = isset($prod->visibility) ? $prod->visibility : 'none';
		$p->ean13 = $prod->code;
		$p->id_tax_rules_group = $prod->id_taxrule;
		$p->ecotax = Tools::ps_round(isset($prod->ecotax) ? $prod->ecotax :0, 6);
		$p->wholesale_price = Tools::ps_round(isset($prod->wholesale) ? $prod->wholesale :0, 6);
		//EVOL do it this way (for register as well)
		$p->price = Tools::ps_round($prod->base, 6);
		// in the form ecotax is included in the price
		//$p->price = Tools::ps_round(+$prod->price / (1.0 + $p->getTaxesRate() / 100.0 - (+$p->ecotax)), 6)  ;
		$p->link_rewrite = Tools::link_rewrite($p->name);
		// 		var_dump($p);
		// 		die();
		$p->save();
		
		//sets quantity only in the current shop (context)
		if (isset($prod->qty))
			StockAvailable::setQuantity($p->id, 0, $prod->qty);
		//StockAvailable::setProductOutOfStock($p->id, true);

		// Attach to root category at last position (if new product)
		if (!isset($prod->id))
		{
			$id_cat = Category::getRootCategory()->id;
			$db = Db::getInstance();
			$db->insert('category_product', array(
					'id_category' => (int) $id_cat,
					'id_product' => (int) $p->id,
					'position' => (int) $db->getValue('
			SELECT MAX(`position`)
			FROM `'._DB_PREFIX_.'category_product`
			WHERE `id_category` = '. (int) $id_cat),
			));
		}
		$response->addResult('prod', detailProduct($p, $id_lang, true));
	}
}

function getProductCustomerExtension($id_customer) {
    $cust = null;
    if ((int) $id_customer > 0) {
        $res = Db::getInstance()->getRow('
            SELECT 
                c.id_customer AS id, 
                c.firstname, 
                c.lastname, 
                c.email,
                IFNULL(ck.fakemail, 0) AS fakemail
            FROM '._DB_PREFIX_.'customer c
            LEFT JOIN '._DB_PREFIX_.'customer_kerawen ck ON c.id_customer = ck.id_customer
            WHERE c.id_customer = ' . (int) $id_customer
        );
        if ($res) {
            $cust = $res;
        }
    }
    
    return $cust;
}

function getProductExtension($id_prod, $with_tax = false)
{
	if ($id_prod){
		$db = Db::getInstance();
		$ext = $db->getRow('
			SELECT * FROM '._DB_PREFIX_.'product_wm_kerawen
			WHERE id_product = '. (int) $id_prod);
		
		if ($ext)
		{
			$ext['codes'] = $db->executeS('
				SELECT *, id_code AS id FROM '._DB_PREFIX_.'product_wm_code_kerawen
				WHERE id_product = '. (int) $id_prod
			);
			
			$ext['cust'] = getProductCustomerExtension($ext['id_customer']);

			if ($with_tax) {
				$context = Context::getContext();
				require_once(dirname(__FILE__).'/data.php');
				$address = getDefaultDeliveryAddress(true);
				$tax_manager = TaxManagerFactory::getManager($address, Product::getIdTaxRulesGroupByIdProduct((int)$id_prod, $context));
				$tax_calculator = $tax_manager->getTaxCalculator();
				
				foreach($ext['codes'] as &$code) {
					$code['unit_price_te'] = $code['unit_price'];
					$code['unit_price'] = $tax_calculator->addTaxes((float)$code['unit_price_te']);
				}
			}
		}
		
		return $ext;
	}
	else return array();
}


function createBareCode($mask = '20xxxxxxxxxx') {

	$db = Db::getInstance();
	
	$barcode = 'xxxxxxxxxxxx';
	for ($k = 0; $k < 12; $k++) {
		if ($mask[$k] == 'x')
			$barcode[$k] = mt_rand(0, 9);
		else
			$barcode[$k] = $mask[$k];
	}
	
	//check digit
	$barcode .= ean_checkdigit($barcode);
	
	$q = "
		SELECT
			product.id_product
		FROM " . _DB_PREFIX_ . "product product
		LEFT JOIN " . _DB_PREFIX_ . "product_attribute product_attribute ON product.id_product = product_attribute.id_product
		WHERE
			product.ean13 = '" . pSQL($barcode) . "'
		OR
			product_attribute.ean13 = '" . pSQL($barcode) . "'
		";

	$res = $db->executeS($q);	
	if ($res) {
		return createBareCode($mask);
	}
	
	return $barcode;
}


function ean_checkdigit($code) {
	$sum = 0;
	$weightflag = true;
	for ($i = strlen($code) - 1; $i >= 0; $i--) {
	    $sum += (int)$code[$i] * ($weightflag?3:1);
	    $weightflag = !$weightflag;
	}
	
	if (($sum % 10) == 0)  { return 0; } 
	    
	$vis = (int)($sum / 10);
	$sup = ($vis + 1) * 10;
	return ($sup - $sum); 
}

function getProductFullName($id_prod, $id_attr, $id_lang) {
	$db = DB::getInstance();
	
	$name = $db->getValue('
			SELECT name FROM '._DB_PREFIX_.'product_lang
			WHERE id_product = '. (int) $id_prod .' AND id_lang = '. (int) $id_lang);
	
	if ($id_attr) {
		$attrs = $db->executeS('
			SELECT
				agl.name AS name, al.name AS value
			FROM '._DB_PREFIX_.'product_attribute_combination pac
			LEFT JOIN '._DB_PREFIX_.'attribute a ON a.id_attribute = pac.id_attribute
			LEFT JOIN '._DB_PREFIX_.'attribute_lang al ON al.id_attribute = a.id_attribute AND al.id_lang = '. (int) $id_lang . '
			LEFT JOIN '._DB_PREFIX_.'attribute_group ag ON ag.id_attribute_group = a.id_attribute_group
			LEFT JOIN '._DB_PREFIX_.'attribute_group_lang agl ON agl.id_attribute_group = ag.id_attribute_group AND agl.id_lang = '. (int) $id_lang .'
			WHERE pac.id_product_attribute = '. (int) $id_attr .'
			ORDER BY ag.position ASC, a.position ASC');
		for ($i = 0; $i < count($attrs); $i++) {
			$name = $name.($i ? ', ' : ' - ').$attrs[$i]['name'].': '.$attrs[$i]['value'];
		}
	}
	
	return $name;
}

function getProductImage($id_prod, $id_attr, $rewrite) {
    
    $context = Context::getContext();
    $id_lang = (int) $context->language->id;
    
    $link = new Link();
    $image_type = Configuration::get('KERAWEN_IMAGE_PRODUCT');
    $img = null;
    
    if (!$img && $id_attr) {
        $img = Db::getInstance()->getValue('
			SELECT id_image FROM '._DB_PREFIX_.'product_attribute_image
			WHERE id_product_attribute = '. (int) $id_attr);
    }
    
    if (!$img) {
        $cover = Product::getCover($id_prod);
        if (isset($cover['id_image'])) {
            $img = $cover['id_image'];
       }
    }

    $rewrite = is_array($rewrite) ? (isset($rewrite[$id_lang]) ? $rewrite[$id_lang] : reset($rewrite)) : $rewrite;
        
    $img = $img && $image_type ? '//'.$link->getImageLink($rewrite, $id_prod.'-'.$img, $image_type) : null;
    $img = str_replace('//http', 'http', $img);
    return $img;
}