<?php
/**
 * 2015 KerAwen
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@kerawen.com so we can send you a copy immediately.
 *
 * @author    KerAwen <contact@kerawen.com>
 * @copyright 2015 KerAwen
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
 */

function getKerawenSecureKey()
{
	$key = Configuration::get('KERAWEN_SECURE_KEY');
	if (!$key)
	{
		$key = md5(uniqid(rand(), true));
		Configuration::updateValue('KERAWEN_SECURE_KEY', $key);
	}
	return $key;
}

function getDefaultDeliveryAddress($full = false)
{
	$address = Configuration::get('KERAWEN_DEFAULT_ADDRESS');
	if (!Address::addressExists($address)) {
	    $context = Context::getContext();
	    $customer = getAnonymousCustomer();
	    
		$addr = new Address($address);
		$addr->id_customer = $customer->id;
		$addr->alias = $context->module->l('Shop takeaway');
		$addr->firstname = '-';
		$addr->lastname = '-';
		$addr->company = '-';
		$addr->address1 = $context->module->l('Shop takeaway');
		$addr->city = '-';
		$addr->id_country = $context->country->id;
		$addr->phone = '0100000000';
		$addr->phone_mobile = '0600000000';
		$addr->date_add = Db::getInstance()->getValue('SELECT NOW()');
		$addr->postcode = '0';
		$addr->save();
		
		$address = $addr->id;
		Configuration::updateValue('KERAWEN_DEFAULT_ADDRESS', $address);
	}
	return $full ? new Address($address) : $address;
}

function getAnonymousCustomer()
{
	$id = Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER');
	$customer = $id ? new Customer($id) : null;
	if (!Validate::isLoadedObject($customer)) {
		$customer = new Customer();
		$customer->firstname = '-';
		$customer->lastname = '-';
		$customer->optin = 0;
		$customer->email = Configuration::get('PS_SHOP_EMAIL');
		$customer->passwd = Tools::encrypt(Tools::passwdGen(MIN_PASSWD_LENGTH));
		$customer->save();
		
		Configuration::updateValue('KERAWEN_ANONYMOUS_CUSTOMER', $customer->id);
		
		Db::getInstance()->execute('
			INSERT INTO '._DB_PREFIX_.'customer_kerawen (id_customer, fakemail)
			VALUES ('. (int) $customer->id .', 1)
			ON DUPLICATE KEY UPDATE fakemail = 1'
		);
	}
	return $customer;
}

function isAnonymousCustomer($customer) {
		
	if ( is_numeric($customer) ) {
		$id_customer = $customer;
	} else {
		$id_customer = isset($customer->id) ? $customer->id : 0;
	}
	
	return ($id_customer == Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER'));
}

function getInShopCarrier() {
    
    require_once (_KERAWEN_TOOLS_DIR_.'/utils.php');
    
    $context = Context::getContext();
    $kdc = Configuration::get('KERAWEN_DEFAULT_CARRIER');
    
    $kdc_array = false;
    if(!is_numeric($kdc)) {
        //search carrier by till
        $kdc_array = unserialize($kdc);
        if (is_array($kdc_array)) {
            $id_till = (int) getExtendedContext('id_cashdrawer', 0);
            if (isset($kdc_array[$id_till])) {
                $kdc = $kdc_array[$id_till];
            }
        }
    }
    
    $carrier = new Carrier($kdc);
    if (!Validate::isLoadedObject($carrier)) {
        
        $carrier->name = $context->module->l('In shop', pathinfo(__FILE__, PATHINFO_FILENAME));
        $carrier->delay = getForLanguages('-');
        $carrier->is_free = true;
        $carrier->shipping_handling = false;
        $carrier->is_module = true;
        $carrier->external_module_name = $context->module->name;
        $carrier->active = false;
        $carrier->save();
        
        if (is_array($kdc_array)) {
            $kdc_array[0] = (int) $carrier->id;
            $data_carrier = serialize($kdc_array);
        }
        else {
            $data_carrier = $carrier->id;
        }
        Configuration::updateGlobalValue('KERAWEN_DEFAULT_CARRIER', $data_carrier);
        
        // Update legacy orders
        Db::getInstance()->execute('
			UPDATE '._DB_PREFIX_.'orders
			SET id_carrier = '. (int) $carrier->id .'
			WHERE id_carrier = 0');
        Db::getInstance()->execute('
			UPDATE '._DB_PREFIX_.'order_carrier
			SET id_carrier = '. (int) $carrier->id .'
			WHERE id_carrier = 0');
    }
    return $carrier->id;
}

