<?php
/**
* 2014 KerAwen
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2014 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/

require_once (_KERAWEN_API_DIR_.'/constants.php');
require_once (_KERAWEN_TOOLS_DIR_.'/utils.php');

/**
 * Counts legacy PrestaShop orders
 * Optionnaly references these orders in KerAwen orders table
 */
function legacyOrders($doit = false)
{
	$db = Db::getInstance();
	$buf = $db->executeS('
		SELECT `id_order` FROM `'._DB_PREFIX_.'orders`
		WHERE `id_order` NOT IN (SELECT `id_order` FROM `'._DB_PREFIX_.'order_kerawen`)');
	$count = count($buf);
	if ($doit)
	{
		$count = 0;
		foreach ($buf as $prev)
		{
			$order = new Order($prev['id_order']);
			$db->insert('order_kerawen', array(
				'id_order' => (int) $order->id,
			));
			$count++;
		}
	}
	return $count;
}

function addReturnedProduct($id_cart, $id_order, $id_order_detail, $id_prod, $id_attr,
	$quantity, $refund_ti, $back_to_stock)
{
	require_once(_KERAWEN_CLASS_DIR_.'/data.php');
	$db = Db::getInstance();
	
	$context = Context::getContext();
	$cart = new Cart($id_cart);
	$order = $id_order ? new Order($id_order) : false;
	$address = $order ? new Address($order->id_address_delivery) : getDefaultDeliveryAddress(true);
	$detail = $id_order_detail ? new OrderDetail($id_order_detail) : false;
	
	// Compute tax according to product/carrier configuration
	// TODO adapt vat_margin module with specific tax calculator
	$refund_te = $refund_ti;
	if ($id_prod) {
		$id_tax_rules_group = Product::getIdTaxRulesGroupByIdProduct($id_prod, $context);
		$tax_manager = TaxManagerFactory::getManager($address, $id_tax_rules_group);
		$tax_calculator = $tax_manager->getTaxCalculator();
		$refund_te = $tax_calculator->removeTaxes($refund_ti);
		
		require_once(_KERAWEN_CLASS_DIR_.'/catalog.php');

		// Create a virtual order detail
		$detail = new OrderDetail();
		$detail->id_order = 0;
		$detail->id_shop = $cart->id_shop;
		$detail->id_warehouse = 0;
		$detail->product_id = $id_prod;
		$detail->product_attribute_id = $id_attr;
		$detail->product_name = getProductFullName($id_prod, $id_attr, $context->language->id);
		$detail->product_quantity = $quantity;
		$detail->product_price = round($refund_te/$quantity, 6);
		$detail->original_product_price = round($refund_te/$quantity,6);
		$detail->total_price_tax_incl = round($refund_ti, 6);
		$detail->total_price_tax_excl = round($refund_te, 6);
		$detail->unit_price_tax_incl = round($refund_ti/$quantity, 6);
		$detail->unit_price_tax_excl = round($refund_te/$quantity, 6);
		$detail->id_tax_rules_group = $id_tax_rules_group;
		
		// Additional info
		$isbn = Tools::version_compare(_PS_VERSION_, '1.7', '>=');
		$mpn = Tools::version_compare(_PS_VERSION_, '1.7.7', '>=');
		
		$more = $db->getRow('
            SELECT
                IF(pa.reference IS NULL OR pa.reference = "", p.reference, pa.reference) AS reference,
                IF(pa.supplier_reference IS NULL OR pa.supplier_reference = "", p.supplier_reference, pa.supplier_reference) AS supplier_reference,
                IF(pa.ean13 IS NULL OR pa.ean13 = "", p.ean13, pa.ean13) AS ean13,
                IF(pa.upc IS NULL OR pa.upc = "", p.upc, pa.upc) AS upc,
                '.($isbn ? 'IF(pa.isbn IS NULL OR pa.isbn = "", p.isbn, pa.isbn) AS isbn,' : '').'
                '.($mpn ? 'IF(pa.mpn IS NULL OR pa.mpn = "", p.mpn, pa.mpn) AS mpn,' : '').'
                IFNULL(pa.weight, p.weight) AS weight,
                IF(pas.wholesale_price IS NULL OR pas.wholesale_price = 0, ps.wholesale_price, pas.wholesale_price) AS wholesale_price
            FROM '._DB_PREFIX_.'product p
            LEFT JOIN '._DB_PREFIX_.'product_shop ps ON ps.id_product = p.id_product AND ps.id_shop = '. (int) $cart->id_shop . '
            LEFT JOIN '._DB_PREFIX_.'product_attribute pa ON pa.id_product = p.id_product
            LEFT JOIN '._DB_PREFIX_.'product_attribute_shop pas ON pas.id_product_attribute = pa.id_product_attribute AND pas.id_shop = '. (int) $cart->id_shop .'
            WHERE p.id_product = '. (int) $id_prod .'
            AND pa.id_product_attribute'.($id_attr ? ' = '. (int) $id_attr : ' IS NULL'));

		$detail->product_reference = $more['reference'];
		$detail->product_supplier_reference = $more['supplier_reference'];
		$detail->product_ean13 = $more['ean13'];
		$detail->product_upc = $more['upc'];
		if($isbn) $detail->product_isbn = $more['isbn'];
		if($mpn) $detail->product_mpn = $more['mpn'];
		$detail->product_weight = $more['weight'];
		$detail->purchase_supplier_price = $more['wholesale_price'];
		
		$detail->add();
		$id_order_detail = $detail->id;
		$id_order = 0;
		$order = null;
	}
	else if ($detail) {
		$max_returns = getMaxReturns($id_order_detail);
		if ($quantity > $max_returns) {
			$e = new PrestaShopException('Invalid quantity ('.$quantity.') The maximum number of returns is : '.$max_returns);
			$e->quantity = $quantity;
			$e->max_returns = $max_returns;
			throw $e;
		}
		
		$tax_manager = TaxManagerFactory::getManager($address, Product::getIdTaxRulesGroupByIdProduct($detail->product_id, $context));
		$tax_calculator = $tax_manager->getTaxCalculator();
		$refund_te = $tax_calculator->removeTaxes($refund_ti);
	}
	else if ($order) {
		$refund_te = $refund_ti/(1 + $order->carrier_tax_rate/100);
	}
	
	$db->insert('return_kerawen', array(
		'id_cart' => (int) $id_cart,
		'id_order' => (int) $id_order,
		'id_order_detail' => (int) $id_order_detail,
		'quantity' => (int) $quantity,
		'refund_tax_incl'=> (float) $refund_ti,
        'refund_tax_excl' => (float) $refund_te,
		'back_to_stock'=> (int) $back_to_stock,
		'id_order_after' => 0,
		'order_after_reference' => 'NOREF',
	));
	
	// Check customers
	$id_cust_anonymous = (int)Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER');
	
	$id_cust_cart = $cart->id_customer;
	if ($id_cust_cart == $id_cust_anonymous) $id_cust_cart = 0;
	
	$id_cust_order = $order ? $order->id_customer : 0;
	if ($id_cust_order == $id_cust_anonymous) $id_cust_order = 0;
    
	return $id_cust_cart != $id_cust_order ? ($id_cust_cart && $id_cust_order ? -1 : $id_cust_order) : 0;
}

function eraseReturn($id_return)
{
	Db::getInstance()->execute('
		DELETE rk, od
		FROM '._DB_PREFIX_.'return_kerawen rk
		LEFT JOIN '._DB_PREFIX_.'order_detail od
			ON od.id_order_detail = rk.id_order_detail AND od.id_order = 0
		WHERE rk.id_return ='. (int) $id_return);
}

/* returns the number of possible returns of a specific order detail */
function getMaxReturns($id_order_detail)
{
	$db = Db::getInstance();
	//number of items
	$max = $db->getValue('
			SELECT SUM(`product_quantity`)
			FROM `'._DB_PREFIX_.'order_detail`
			WHERE `id_order_detail` = '. (int) $id_order_detail);
	//number of items that have already been returned
	$ps_qties = Db::getInstance()->getRow('
			SELECT `product_quantity_return`, `product_quantity_refunded`
			FROM `'._DB_PREFIX_.'order_detail`
			WHERE `id_order_detail` = '. (int) $id_order_detail);
	$max -= ($ps_qties['product_quantity_return'] + $ps_qties['product_quantity_refunded']);
	//number of items that are going to be returned
	$max -= $db->getValue('
			SELECT SUM(`quantity`) FROM `'._DB_PREFIX_.'return_kerawen`
			WHERE `id_order_detail` = '. (int) $id_order_detail.'
			AND `took_effect` = 0');
	return $max;
}



function searchOrders($params) {

    $db = Db::getInstance();
    $context = Context::getContext();
    
    $permissions = $context->permissions;
    
    $dataSearch = $params->dataSearch;
    
    $withJoinProduct = '
        INNER JOIN '._DB_PREFIX_.'order_detail od ON o.id_order = od.id_order
    ';
    $joinProduct = '';
 
    //Filter + Permissions
    $whereShop = '';
    $shops = array();
    if (isset($dataSearch->shops)) {
        //Permissions already apply from JS  (shops < 2 means no filter displayed)
        foreach ($dataSearch->shops[0] as $item) {
            $shops[] = 'o.id_shop = ' . (int) $item->value;
        }
        $whereShop = ' AND (' . implode(' OR ', $shops) . ')';
    } else {
        //Without filter
        $whereShop = ' AND FALSE ';
        if (isset($permissions->ordersDisplay)) {
            if (!empty($permissions->ordersDisplay->id_shop)) {
                $list_shops = explode(',', $permissions->ordersDisplay->id_shop);
                foreach ($list_shops as $item) {
                    $shops[] = 'o.id_shop = ' . (int) $item;
                }
                $whereShop = ' AND (' . implode(' OR ', $shops) . ')';
            } else {
                if (!empty($permissions->ordersDisplay->id_employee)) {
                    $whereShop = ' AND (ok.id_employee = ' . (int) $permissions->ordersDisplay->id_employee . ')';
                }
            }
        }
    }


    $whereEmployee = '';
    $employees = array();
    if (isset($dataSearch->employees)) {
        foreach ($dataSearch->employees[0] as $item) {
            $employees[] = 'ok.id_employee = ' . (int) $item->value;
        }
        $whereEmployee = ' AND (' . implode(' OR ', $employees) . ')';
    }
    
    $whereTerm = '';
    if (isset($dataSearch->term)) {
        $whereTermOrder = '';
        $whereTermProduct = '';
        $list_term = array();
        foreach (preg_split('/[\s]+/', str_replace('"', '\\"', $dataSearch->term[0]->value)) as $str) {
            if (Tools::strlen($str)) $list_term[] = $str;
        }
        
        //--------------------order--------------------
        $list_fft = array(
            'o.id_order' => '=',
            'o.reference' => '=',
        );
        
        if (isset($dataSearch->note_cust)) {
            $list_fft['o.note'] = 'like';
        }
        
        if (isset($dataSearch->note_priv)) {
            $list_fft['ok.private_note'] = 'like';
        }
        
        if (isset($dataSearch->note_pub)) {
            $list_fft['ok.invoice_note'] = 'like';
        }
        
        $sep1 = '';
        $whereTermOrder .= ' (';
        foreach($list_fft as $field=>$fft) {
            $whereTermOrder .= $sep1 . '(';
            $sep2 = '';
            foreach($list_term as $str) {
                $whereTermOrder .= $sep2 . $field;
                $whereTermOrder .= ($fft == '=') ? ' = "' . pSQL($str) . '"' : ' LIKE "%' . pSQL($str) . '%"';
                $sep2 = ' AND ';
            }
            $whereTermOrder .= ')';
            $sep1 = ' OR ';
        }
        $whereTermOrder .= ')';
        
        
        //--------------------order detail--------------------
        $list_ffp = array(
            'od.product_name',
            'od.product_ean13',
            'od.product_reference'
        );

        $whereTermProduct = '';
        $joinProduct = $withJoinProduct;
        
        $sep1 = '';
        $whereTermProduct .= ' (';
        foreach($list_ffp as $ffp) {
            $whereTermProduct .= $sep1 . '(';
            $sep2 = '';
            foreach($list_term as $str) {
                $whereTermProduct .= $sep2 . $ffp . ' LIKE "%' . pSQL($str) . '%"';
                $sep2 = ' AND ';
            }
            $whereTermProduct .= ')';
            $sep1 = ' OR ';
        }
        $whereTermProduct .= ')';
        
        $whereTerm = ' AND (' . $whereTermOrder . ' OR ' . $whereTermProduct . ')';
    }
    
    $whereCustomer = '';
    if (isset($dataSearch->cust)) {
        $whereCustomer = ' AND o.id_customer = ' . (int) $dataSearch->cust[0][0]->value;
    }

    $whereOrigine = '';
/*
    if (isset($dataSearch->origine)) {
        $q_origine = array();
        foreach ($dataSearch->origine[0] as $item) {
            $q_origine[] = ((int) $item->value == 1) ? '(IFNULL(ok.id_till,0) > 0)' : '(IFNULL(ok.id_till, 0) = 0)';
        }
        if (count($q_origine) == 1) {
            $whereOrigine = ' AND (' . implode(' OR ', $q_origine) . ')';
        }
    }
*/


    $whereTill = '';
    $tills = array();
    if (isset($dataSearch->tills)) {
        foreach ($dataSearch->tills[0] as $item) {
            if ((int) $item->value > 0) {
                $tills[] = 'ok.id_till = ' . (int) $item->value;
            } else {
                $tills[] = ((int) $item->value == -1) ? '(IFNULL(ok.id_till,0) > 0)' : '(IFNULL(ok.id_till, 0) = 0)';
            }
        }
        if (count($tills) > 0) {
            $whereTill = ' AND (' . implode(' OR ', $tills) . ')';
        }
    }
    
    $joinCarriers = '';
    if (isset($dataSearch->carriers)) {
        $carriers = array();
        foreach ($dataSearch->carriers[0] as $item) {
            $carriers[] = (int) $item->value;
        }
        $joinCarriers = '
            INNER JOIN ' . _DB_PREFIX_.'order_carrier oca 
                ON oca.id_order = o.id_order
                AND oca.id_carrier IN (
                    SELECT id_carrier FROM '._DB_PREFIX_.'carrier WHERE id_reference IN (' . implode(',', $carriers) . ')
                )
        ';
    }

    $joinTaxrules = '';
    if (isset($dataSearch->taxrules)) {
        $taxrules = array();
        foreach ($dataSearch->taxrules[0] as $item) {
            $taxrules[] = '(odt.id_tax_rules_group = ' . (int) $item->value . ')';
        }
        $joinTaxrules = '
            INNER JOIN '._DB_PREFIX_.'order_detail odt ON odt.id_order = o.id_order
                 AND (' . implode(' OR ', $taxrules) . ')
        ';
    }

    $joinType = '';
    if (isset($dataSearch->prod_types)) {
        $joinProduct = $withJoinProduct;
        $types = array();
        foreach ($dataSearch->prod_types[0] as $item) {
            $types[] = '(odk.product_type = "' . pSQL($item->key) . '")';
            if ($item->key == "STANDARD") {
                $types[] = '(ISNULL(odk.product_type))';
            }
        }
        $joinType = '
            INNER JOIN '._DB_PREFIX_.'order_detail_kerawen odk ON odk.id_order_detail = od.id_order_detail
                 AND (' . implode(' OR ', $types) . ')
        ';
    }

    $whereMode = '';
    if (isset($dataSearch->modes)) {
        $modes = array();
        foreach ($dataSearch->modes[0] as $item) {
            $modes[] = '(IFNULL(ok.delivery_mode, 2) = ' . $item->value . ')';
        }
        $whereMode = ' AND (' . implode(' OR ', $modes) . ')';
    }
    
    $wherePaid = '';
    if (isset($dataSearch->paid)) {
        $paids = array();
        foreach ($dataSearch->paid[0] as $item) {
            $paids[] = 'ok.is_paid = ' . (int) $item->value;
        }
        if (count($paids) == 1) {
            $wherePaid = ' AND (' . implode(' OR ', $paids) . ')';
        }
    }
    
    $whereAccounted = '';
    if (isset($dataSearch->accounted)) {
        $accounteds = array();
        foreach ($dataSearch->accounted[0] as $item) {
            $accounteds[] = 'ok.accounted = ' . (int) $item->value;
        }
        if (count($accounteds) == 1) {
            $whereAccounted = ' AND (' . implode(' OR ', $accounteds) . ')';
        }
    }
    
    $whereInvoice = '';
    if (isset($dataSearch->invoice)) {
        $invoices = array();
        foreach ($dataSearch->invoice[0] as $item) {
            $invoices[] = ((int) $item->value == 1) ? '(o.invoice_number != 0)' : '(o.invoice_number = 0)';
        }
        if (count($invoices) == 1) {
            $whereInvoice = ' AND (' . implode(' OR ', $invoices) . ')';
        }
    }
    
    $whereAmount = '';
    if (isset($dataSearch->amount)) {
        $amounts = array();
        $amount = $dataSearch->amount[0][0];
        if (is_numeric($amount->min)) {
            $amounts[] = 'o.total_paid >= ' . (float)$amount->min;
        }
        if (is_numeric($amount->max)) {
            $amounts[] = 'o.total_paid <= ' . (float)$amount->max;
        }
        $whereAmount = ' AND (' . implode(' AND ', $amounts) . ')';
    }

    $joinQuantity = '';
    if (isset($dataSearch->quantity)) {
        $qties = array();
        $qty = $dataSearch->quantity[0][0];
        if (is_numeric($qty->min)) {
            $qties[] = 'qty >= ' . (int) $qty->min;
        }
        if (is_numeric($qty->max)) {
            $qties[] = 'qty <= ' . (int) $qty->max;
        }
        
        if (count($qties)) {
            $joinQuantity = '
                INNER JOIN (SELECT 
                    id_order, 
                    SUM(product_quantity) AS qty 
                FROM '._DB_PREFIX_.'order_detail 
                GROUP BY id_order 
                HAVING (' . implode(' AND ', $qties) . ')) qty ON qty.id_order=o.id_order
            ';
        }
    }
    
 
    $joinReference = '';
    if (isset($dataSearch->reference)) {
        $refs = array();
        $ref = $dataSearch->reference[0][0];
        if (is_numeric($ref->min)) {
            $refs[] = 'qty >= ' . (int) $ref->min;
        }
        if (is_numeric($ref->max)) {
            $refs[] = 'qty <= ' . (int) $ref->max;
        }
        
        if (count($refs)) {
            $joinReference = '
                INNER JOIN (SELECT
                    id_order,
                    count(id_order_detail) AS qty
                FROM '._DB_PREFIX_.'order_detail
                GROUP BY id_order
                HAVING (' . implode(' AND ', $refs) . ')) ref ON ref.id_order=o.id_order
            ';
        }
    }
    
    
    $joinPayment = '';
    $payments = array();
    if (isset($dataSearch->payments)) {
        foreach ($dataSearch->payments[0] as $key => $payment) {
            $payments[] = '"' . pSQL($payment) . '"';
        }
        $joinPayment .= '
            INNER JOIN '._DB_PREFIX_.'order_payment op ON op.order_reference = o.reference
                AND op.payment_method IN (' . implode(',', $payments) . ')
        ';
    }

    $wherePeriod = '';
    if (isset($dataSearch->period)) {
        $dates = array();
        $period = $dataSearch->period[0][0];
        $match = "/[0-9]{2}\/[0-9]{2}\/[0-9]{4}/"; //heuristique
        if ($period->from !== '' && preg_match($match, $period->from)) {
            $dates[] = 'o.date_add >= "' . implode("-", array_reverse(explode("/", $period->from))) . ' 00:00:00"';
        }
        if ($period->to !== '' && preg_match($match, $period->to)) {
            $dates[] = 'o.date_add <= "' . implode("-", array_reverse(explode("/", $period->to))) . ' 23:59:59"' ;
        }
        if (count($dates)) {
            $wherePeriod = ' AND (' . implode(' AND ', $dates) . ')';
        }
    }

    $whereStat = '';
    if (isset($dataSearch->order_states)) {
        $stats = array();
        foreach ($dataSearch->order_states[0] as $item) {
            $stats[] = 'o.current_state = ' . (int) $item->value;
        }
        $whereStat = ' AND (' . implode(' OR ', $stats) . ')';
    }

    $joinAttribute = '';
    if (isset($dataSearch->options)) {
        $joinProduct = $withJoinProduct;
        foreach ($dataSearch->options as $key => $attributeGroup) {
            $ids_attr = array();
            foreach ($attributeGroup as $id_attribute) {
                $ids_attr[] = $id_attribute->value;
            }
            $pac = 'pac' . $key;
            $joinAttribute .= '
                INNER JOIN '._DB_PREFIX_.'product_attribute_combination ' . $pac . ' ON od.product_attribute_id = ' . $pac . '.id_product_attribute
                    AND ' . $pac . '.id_attribute IN (' . implode(',', $ids_attr) . ')
                ';
        }
    }

    $joinFeature = '';
    if (isset($dataSearch->features)) {
        $joinProduct = $withJoinProduct;
        foreach ($dataSearch->features as $key => $feature) {
            $ids_feature = array();
            foreach ($feature as $id_feature) {
                $ids_feature[] = $id_feature->value;
            }
            $fea = 'fea' . $key;
            $joinFeature .= '
                INNER JOIN '._DB_PREFIX_.'feature_product ' . $fea . ' ON od.product_id=' . $fea . '.id_product
                    AND ' . $fea . '.id_feature_value IN (' . implode(',', $ids_feature) . ')
                ';
        }
    }

    $joinSupplier = '';
    if (isset($dataSearch->supplier)) {
        $joinProduct = $withJoinProduct;
        $joinSupplier = '
            INNER JOIN '._DB_PREFIX_.'product_supplier psup ON psup.id_product=od.product_id AND psup.id_product_attribute=IFNULL(od.product_attribute_id, 0)
                AND (psup.id_supplier = ' . (int) $dataSearch->supplier[0][0]->value . ')
        ';
    }
    
    $joinManufacturer = '';
    if (isset($dataSearch->manufacturer)) {
        $joinProduct = $withJoinProduct;
        $joinManufacturer = '
            INNER JOIN '._DB_PREFIX_.'product p ON p.id_product=od.product_id
                AND (p.id_manufacturer=' . (int) $dataSearch->manufacturer[0][0]->value . ')
        ';
    }
 

    $joinReturn = '';
    if (isset($dataSearch->returns)) {
        $returns = array();
        foreach ($dataSearch->returns[0] as $item) {
            $returns[] = ((int) $item->value == 1) ? '(ret > 0)' : '(ret = 0)';
        }
        if (count($returns) == 1) {
            $joinReturn = '
                INNER JOIN (SELECT
                    id_order,
                    SUM(product_quantity_refunded) AS ret
                FROM '._DB_PREFIX_.'order_detail
                GROUP BY id_order
                HAVING (' . implode(' OR ', $returns) . ')) odr ON odr.id_order=o.id_order
            ';
        }
    }

    $joinProdDisc = '';
    if (isset($dataSearch->prod_disc)) {
        $discs = array();
        foreach ($dataSearch->prod_disc[0] as $item) {
            $discs[] = ((int) $item->value == 1) ? '(reduc > 0)' : '(reduc = 0)';
        }
        if (count($discs) == 1) {
            $joinProdDisc = '
                INNER JOIN (SELECT
                    id_order,
                    SUM(reduction_amount + reduction_percent) AS reduc
                FROM '._DB_PREFIX_.'order_detail
                GROUP BY id_order
                HAVING (' . implode(' OR ', $discs) . ')) odpd ON odpd.id_order=o.id_order
            ';
        }
    }
    
    $whereCartDisc = '';
    if (isset($dataSearch->cart_disc)) {
        $discs = array();
        foreach ($dataSearch->cart_disc[0] as $item) {
            $discs[] = ((int) $item->value == 1) ? '(o.total_discounts > 0)' : '(o.total_discounts = 0)';
        }
        if (count($discs) == 1) {
            $whereCartDisc = ' AND (' . implode(' OR ', $discs) . ')';
        }
    }

    $joinThread = '';
    if (isset($dataSearch->thread_state)) {
        $stats = array();
        foreach ($dataSearch->thread_state[0] as $item) {
            $stats[] = (int) $item->value;
        }
        $joinThread = '
            INNER JOIN '._DB_PREFIX_.'customer_thread ct ON ct.id_order=o.id_order
            INNER JOIN '._DB_PREFIX_.'customer_thread_kerawen ctk ON ct.id_customer_thread=ctk.id_customer_thread
                AND ctk.id_thread_state IN (' . implode(',', $stats) . ')
        ';
    }
    

    $q= '
        SELECT
            SQL_CALC_FOUND_ROWS DISTINCT
            o.id_order
        FROM '._DB_PREFIX_.'orders o
        LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
        ' . $joinProduct . '
        ' . $joinAttribute . '
        ' . $joinFeature . '
        ' . $joinSupplier . '
        ' . $joinManufacturer . '
        ' . $joinPayment . '
        ' . $joinReturn . '
        ' . $joinTaxrules . '
        ' . $joinQuantity . '
        ' . $joinReference . '
        ' . $joinType . '
        ' . $joinProdDisc . '
        ' . $joinThread . '
        ' . $joinCarriers . '
        WHERE TRUE
        ' . $whereTerm . '
        ' . $whereCustomer . '
        ' . $whereOrigine . '
        ' . $whereMode . '
        ' . $wherePaid . '
        ' . $wherePeriod . '
        ' . $whereStat . '
        ' . $whereShop . '
        ' . $whereTill . '
        ' . $whereEmployee . '
        ' . $whereAccounted . '
        ' . $whereInvoice . '
        ' . $whereAmount . '
        ' . $whereCartDisc . '
        ORDER BY o.id_order DESC
        LIMIT ' . ((int) $params->page->num * (int) $params->page->size) . ', ' . (int) $params->page->size
    ;

    $pp = $db->executeS($q);
    $total = (int) $db->getValue('SELECT FOUND_ROWS()');

    $orders = array();
    if ($pp) {
        foreach ($pp as $p) {
            $detail = detailOrder($p['id_order']);
            if ($detail) { $orders[]  = $detail; }
        }
    }
    

    return array(
        'page' => array(
            'num' => (int) $params->page->num,
            'count' => (int) ceil($total/$params->page->size),
            'total' => $total,
        ),
        'data' => $orders,
    );
}


function detailOrder($id_order) {
    
    $join_ocs = '';
    $pickup_shop = 'o.id_shop AS shop,';
    $pickup_mode = 'ok.delivery_mode AS mode,';

    //TODO: OCS spec
    
    return Db::getInstance()->getRow('
		SELECT
			o.id_order AS id,
			' . $pickup_shop . '
            o.date_add,
            o.module,
            o.payment,
			ok.id_till AS till,
			' . $pickup_mode . '
			o.current_state AS state,
			CONCAT_WS(" ", cu.firstname, cu.lastname) AS cust,
            ok.id_employee,
			cu.company AS company,
			ca.name AS carrier,
			ok.delivery_date AS deliv,
			ok.display_date AS date,
			(SELECT SUM(od.product_quantity)
				FROM '._DB_PREFIX_.'order_detail od
				WHERE od.id_order = o.id_order) AS items,
			o.total_paid AS total,
			ok.is_paid AS is_paid,
			ok.id_till,
			ok.accounted AS is_accounted
		FROM '._DB_PREFIX_.'orders o
		LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
		LEFT JOIN '._DB_PREFIX_.'customer cu ON cu.id_customer = o.id_customer
		LEFT JOIN '._DB_PREFIX_.'carrier ca ON ca.id_carrier = o.id_carrier
		LEFT JOIN '._DB_PREFIX_.'customer_kerawen ck ON ck.id_customer = o.id_customer
        ' . $join_ocs . '
        WHERE o.id_order = ' . (int) $id_order
     );
}


function validateCart($context, $params, &$response)
{
	require_once(dirname(__FILE__).'/data.php');
	require_once(_KERAWEN_CLASS_DIR_.'/customer.php');
	
	
	// Override tax based address while logging order
	// Only delivery is known at this moment
	Configuration::set('PS_TAX_ADDRESS_TYPE', 'id_address_delivery');
	
	// Avoid mail alerts if required
	if (Configuration::get('KERAWEN_AVOID_NEW_ORDER_ALERT')) {
	    Configuration::set('MA_MERCHANT_ORDER', 0);
	}

	$db = Db::getInstance();

	$id_cart = $params->id_cart;
	$id_cashdrawer = $params->id_cashdrawer;
	$id_employee = $context->employee->id;
	$payments = $params->payment;
	
	$cart = new Cart($id_cart);
	$customer = $cart->id_customer ? new Customer($cart->id_customer) : getAnonymousCustomer();
	$currency = new Currency($cart->id_currency);
	
	// Setup context for computations
	$context = Context::getContext();
	$context->cart = $cart;
	$context->shop = new Shop($cart->id_shop);

	$cartExtended = $db->getRow('SELECT * FROM '._DB_PREFIX_.'cart_kerawen WHERE id_cart=' . (int) $id_cart);
	if (!$cartExtended) {
		$cartExtended['quote'] = 0;
	}
	
	// Complete cart addresses because some modules require both
	if (!$cart->id_address_delivery) $cart->id_address_delivery = getDefaultDeliveryAddress(false);
	
	$addresses = isAnonymousCustomer($customer)  ? array() : $customer->getAddresses($cart->id_lang);
	if (!$cart->id_address_invoice) $cart->id_address_invoice = count($addresses) ? $addresses[0]['id_address'] : $cart->id_address_delivery;
	
	// Get rid of invalid rules
	// IMPORTANT: DO IT AFTER CONTEXT COMPLETION !!!!
	// otherwise group reductions are cancelled in PS 1.6.0.9
	CartRule::autoRemoveFromCart();

	$order = null;
	$credit = null;
	if (Cart::getNbProducts($id_cart)) {
		// Create the order
		$order = new Order();
		require_once (dirname(__FILE__).'/data.php');

		$order->reference = Order::generateReference();
		$order->id_customer = $customer->id;
		$order->id_address_invoice = $cart->id_address_invoice;
		$order->id_address_delivery = $cart->id_address_delivery;
		$order->id_currency = $cart->id_currency;
		$order->id_lang = $cart->id_lang;
		$order->id_cart = $cart->id;
		$order->id_shop = $context->shop->id;
		$order->id_shop_group = $context->shop->id_shop_group;

		$order->secure_key = getKerawenSecureKey();
		$order->payment = $context->module->displayName;
		$order->module = $context->module->name;

		$order->recyclable = $cart->recyclable;
		$order->gift = $cart->gift;
		$order->gift_message = $cart->gift_message;
		$order->mobile_theme = $cart->mobile_theme;
		$order->conversion_rate = $currency->conversion_rate;

		$order->invoice_date = date('Y-m-d H:i:s', time());
		$order->delivery_date = '0000-00-00 00:00:00';
		
		$order->total_products_wt = $cart->getOrderTotal(true, Cart::ONLY_PRODUCTS);
		$order->total_products = $cart->getOrderTotal(false, Cart::ONLY_PRODUCTS);

		$order->total_wrapping_tax_excl = $cart->getOrderTotal(false, Cart::ONLY_WRAPPING);
		$order->total_wrapping_tax_incl = $cart->getOrderTotal(true, Cart::ONLY_WRAPPING);
		$order->total_wrapping = $order->total_wrapping_tax_incl;

		if ($cart->id_carrier) {
			$order->total_shipping_tax_excl = $cart->getOrderTotal(false, Cart::ONLY_SHIPPING);
			$order->total_shipping_tax_incl = $cart->getOrderTotal(true, Cart::ONLY_SHIPPING);
			
			$carrier = new Carrier($cart->id_carrier);
			if (!is_null($carrier) && Validate::isLoadedObject($carrier))
				$order->carrier_tax_rate = $carrier->getTaxesRate(new Address($order->{Configuration::get('PS_TAX_ADDRESS_TYPE')}));
		}
		else {
			$order->total_shipping_tax_excl = 0;
			$order->total_shipping_tax_incl = 0;
		}
		$order->total_shipping = $order->total_shipping_tax_incl;
		$order->id_carrier = $cart->id_carrier ? $cart->id_carrier : getInShopCarrier();
		
		/*
		 $order->total_discounts_tax_excl = (float)abs($cart->getOrderTotal(false, Cart::ONLY_DISCOUNTS));
		 $order->total_discounts_tax_incl = (float)abs($cart->getOrderTotal(true, Cart::ONLY_DISCOUNTS));
		*/
		
		require_once(_KERAWEN_CLASS_DIR_.'/cart.php');
		$reducs = discountAsArray($cart);

		// Warning: discounts may be greater than due (e.g. credit)
		$discount_te = Tools::ps_round($reducs['total_te'], _PS_PRICE_COMPUTE_PRECISION_);
		$discount_ti = Tools::ps_round($reducs['total_ti'], _PS_PRICE_COMPUTE_PRECISION_);
		$total_te = $order->total_products + $order->total_wrapping_tax_excl + $order->total_shipping_tax_excl;
		$total_ti = $order->total_products_wt + $order->total_wrapping_tax_incl + $order->total_shipping_tax_incl;
		if ($discount_te > $total_te) $discount_te = $total_te;
		if ($discount_ti > $total_ti) $discount_ti = $total_ti;
		
		$order->total_discounts_tax_excl = $discount_te;
		$order->total_discounts_tax_incl = $discount_ti;
		$order->total_discounts = $order->total_discounts_tax_incl;

		$order->total_paid_tax_excl = Tools::ps_round(
			$order->total_products - $order->total_discounts_tax_excl + $order->total_wrapping_tax_excl + $order->total_shipping_tax_excl,
			_PS_PRICE_COMPUTE_PRECISION_);
		$order->total_paid_tax_incl = Tools::ps_round(
			$order->total_products_wt - $order->total_discounts_tax_incl + $order->total_wrapping_tax_incl + $order->total_shipping_tax_incl,
			_PS_PRICE_COMPUTE_PRECISION_);
		$order->total_paid = $order->total_paid_tax_incl;
		$order->total_paid_real = 0.00;

		// Create order
		if (!$order->add())
			throw new PrestaShopException('Can\'t save order');

		// Detail order
		$od = new OrderDetail();
		$packagess = $cart->getPackageList();

		foreach($packagess as $packages)
			foreach($packages as $package)
				$od->createList($order, $cart, Configuration::get('KERAWEN_OS_RECEIVED'),
					$package['product_list'], 0, true, $package['id_warehouse']);
		
		// Carrier info
		if ($order->id_carrier) {
			$order_carrier = new OrderCarrier();
			$order_carrier->id_order = (int)$order->id;
			$order_carrier->id_carrier = (int)$order->id_carrier;
			$order_carrier->weight = $order->getTotalWeight();
			$order_carrier->shipping_cost_tax_excl = $order->total_shipping_tax_excl;
			$order_carrier->shipping_cost_tax_incl = $order->total_shipping_tax_incl;
			$order_carrier->add();
		}
		
		// Apply rules
		// EVOL Sort by expiration date
		$total_tax_incl = $order->total_paid_tax_incl + $order->total_discounts_tax_incl;
		$total_tax_excl = $order->total_paid_tax_excl + $order->total_discounts_tax_excl;
		$discount_tax_incl = 0;
		$discount_tax_excl = 0;
		foreach ($reducs['reducs'] as &$reduc) {
			$remain = $total_tax_incl - $discount_tax_incl;
			if ($remain <= 0) break;
            
			$value_tax_incl = $reduc['reduc_tax_incl'];
			$value_tax_excl = $reduc['reduc_tax_excl'];
			
			if ($value_tax_incl > $remain) {
				$value_tax_incl = $remain;
				$value_tax_excl = $total_tax_excl - $discount_tax_excl;
			}
			
			$discount_tax_incl += $value_tax_incl;
			$discount_tax_excl += $value_tax_excl;

			// (Re)load in multi-language mode to avoid errors when saving
			$rule = new CartRule($reduc['id']);
			
			// Register rule
			$order->addCartRule($rule->id, $rule->name[$order->id_lang], array(
				'tax_incl' => $value_tax_incl,
				'tax_excl' => $value_tax_excl,
			));

			// Consume rule
			$rule->quantity = max(0, $rule->quantity - 1);

			$cartExtended = $db->getRow('SELECT * FROM '._DB_PREFIX_.'cart_kerawen WHERE id_cart=' . (int) $id_cart);
			if (!$cartExtended) {
			    $cartExtended['quote'] = 0;
			}

			$crk = $db->getRow('
				SELECT 
                    `type`, 
                    `id_parent_cart_rule` 
                FROM `'._DB_PREFIX_.'cart_rule_kerawen`
				WHERE `id_cart_rule` = '. (int) $rule->id
			);
			
			$type = isset($crk['type']) ? $crk['type'] : '';
			$id_parent_cart_rule = isset($crk['id_parent_cart_rule']) ? $crk['id_parent_cart_rule'] : '';

			//Hast
			if ($type == _KERAWEN_CR_CREDIT_ && !Configuration::get('KERAWEN_CREDIT_PARTIAL_USE')) {
			    $rule->partial_use = 1;
			}
			
			$rule->save();

			if ($rule->partial_use && ($rule->reduction_amount > $value_tax_incl)) {

				if (Module::isInstalled('loyalty') && Module::isEnabled('loyalty')) {
					$id_loyalty = (int)$db->getValue('
						SELECT id_loyalty FROM `'._DB_PREFIX_.'loyalty`
						WHERE `id_cart_rule` = '. (int) $rule->id);
					if ($id_loyalty) $type = _KERAWEN_CR_LOYALTY_;
				}

				
				//TODO: refactoring -> cartrules.php : createCredit()
				//TODO: refactoring -> share code with front-office ?
				// Recreate one with remaining
				require_once (dirname(__FILE__).'/cartrules.php');
				$id_parent_cart_rule = $id_parent_cart_rule ? $id_parent_cart_rule : $rule->id;
				
				//rule code heritage or not
				//OR like front-office ? (xxxxx-2)
				$code_heritage = 1;
				if ($code_heritage) {
					$code = $rule->code;
					$rule->code = generateRuleCode();
					$rule->save();
				} else {
					$code = generateRuleCode();
				}
				
				unset($rule->id);
				$rule->code = $code;
				$rule->id_customer = $order->id_customer;
				$rule->quantity = 1;
				$rule->quantity_per_user = 1;
				$rule->free_shipping = 0;
				$rule->reduction_amount -= $value_tax_incl;


				if ($type == _KERAWEN_CR_GIFT_CARD_ && isAnonymousCustomer($order->id_customer)) {
				    $rule->id_customer = 0;
				}
				
				if ($type == _KERAWEN_CR_CREDIT_ && !Configuration::get('KERAWEN_CREDIT_PARTIAL_USE')) {
				    $rule->partial_use = 0;
				}
				
				$rule->add();

				if ($type == _KERAWEN_CR_CREDIT_ || $type == _KERAWEN_CR_GIFT_CARD_)
				{
					$data = array(
						'id_cart_rule' => (int) $rule->id,
						'type' => pSQL($type),
						'id_parent_cart_rule' => (int) $id_parent_cart_rule,
					);
					if (!empty($order->id)) {
						$data['id_order'] = $order->id;
					}
					$db->insert('cart_rule_kerawen', $data);
				}
				else if ($type == _KERAWEN_CR_LOYALTY_) {
					// Keep it as loyalty
					$db->update('loyalty', array(
						'id_cart_rule' => (int) $rule->id,
						'id_loyalty_state' => 4),
					'id_loyalty = '. (int) $id_loyalty);
				}
				
				if ($type != _KERAWEN_CR_GIFT_CARD_) {
					$credit = array(
						'id' => $rule->id,
						'code' => $rule->code,
						'type' => $type,
						'name' => $rule->name[$context->language->id],
						'qty' => $rule->quantity_per_user,
						'mode' => 'amount',
						'value' => $rule->reduction_amount,
						'from' => $rule->date_from,
						'to' => $rule->date_to,
						'cust' => $customer ? array(
							'id' => $customer->id,
							'firstname' => $customer->firstname,
							'lastname' => $customer->lastname,
						) : null,
					);
				}
			}
		}

		// Set state before saving to trigger stock movement
		$order->current_state = Configuration::get('KERAWEN_OS_RECEIVED');
		$order->save();
		Hook::exec('actionValidateOrder', array(
			'cart' => $cart,
			'order' => $order,
			'customer' => $customer,
			'currency' => $currency,
			'orderStatus' => new OrderState(Configuration::get('KERAWEN_OS_RECEIVED'), $order->id_lang),
			'kerawen' => true,
		));
	}

	// Manage returns
	$id_slip_order = false;
	$slips = array();
	$total_refund_tax_incl = 0;
	$total_refund_tax_excl = 0;
	
	if ($returns = $db->executeS('
		SELECT * FROM '._DB_PREFIX_.'return_kerawen
		WHERE id_cart='. (int) $id_cart))
	{
		if ($cart->id_customer) {
			$customer = (object) array_merge( (array) $customer, array( 'addrs' => getAddresses($customer, (int) $cart->id_lang)) );
		}
	
		foreach ($returns as $return) {
			$id_slip_order = (int)$return['id_order'];
			$refund_ti = 0;
			$refund_te = 0;
				
			require_once(dirname(__FILE__).'/stock.php');
			$id_reason = getStockReturnReason();
	
			if (!isset($slips[$id_slip_order])) {
				// Create a new order slip for original order
				$slip = new OrderSlip();
				$slip->order = new Order($id_slip_order);
				$slip->id_order = $id_slip_order;
				$slip->id_customer = $cart->id_customer;
	
				$slip->total_products_tax_excl = 0;
				$slip->total_products_tax_incl = 0;
				$slip->total_shipping_tax_incl = 0;
				$slip->total_shipping_tax_excl = 0;
				$slip->conversion_rate = $currency->conversion_rate;
				$slip->partial = 1;
	
				// Prepare slip details
				$slip->details = array();
	
				$slips[$id_slip_order] = $slip;
			}
			$slip = $slips[$id_slip_order];
				
			if ($return['id_order_detail']) {
				$order_detail = new OrderDetail($return['id_order_detail']);
				$tax_rate = $order_detail->unit_price_tax_excl != 0 ? $order_detail->unit_price_tax_incl / $order_detail->unit_price_tax_excl : 1;
				
				// Return details
				$quantity = (int)$return['quantity'];
				$refund_ti = Tools::ps_round((float)$return['refund_tax_incl'], _PS_PRICE_COMPUTE_PRECISION_);
				$refund_te = Tools::ps_round((float)($tax_rate != 0 ? $refund_ti / $tax_rate : $refund_ti), _PS_PRICE_COMPUTE_PRECISION_);
				
				$wholesale_value = (Tools::version_compare(_PS_VERSION_, '1.6.1.1', '>=')) ? $order_detail->original_wholesale_price : $order_detail->purchase_supplier_price;
				$refund_wholesale = Tools::ps_round(((float)$wholesale_value * $quantity), _PS_PRICE_COMPUTE_PRECISION_);
				
				// Mark as refunded
				$order_detail->product_quantity_refunded += $quantity;
				if (Tools::version_compare(_PS_VERSION_, '1.7.7.0', '>=')) {
				    $order_detail->total_refunded_tax_excl += $refund_te;
				    $order_detail->total_refunded_tax_incl += $refund_ti;
				}
				$order_detail->save();
	
				// SPECIAL PRODUCTS
				$ext = $db->getRow('
                    SELECT 
                        pk.is_account_credit,
                        pk.product_type AS product_type,
                        pk.id_customer AS id_customer
                    FROM '._DB_PREFIX_.'order_detail od
                    JOIN '._DB_PREFIX_.'product_wm_kerawen pk ON pk.id_product = od.product_id
                    WHERE od.id_order_detail = '. (int) $order_detail->id);

				if ($ext) {
				    if ($ext['product_type'] == _KERAWEN_TYPE_PREPAID_ || $ext['is_account_credit']) {
    				    require_once(_KERAWEN_CLASS_DIR_.'/customer.php');
    				    adjustCustomerAccount($customer->id, -$refund_ti, _KERAWEN_CA_REFUND_CREDIT_, $order_detail->id, $order_detail->id_order);
    				}
    				if ($ext['product_type'] == _KERAWEN_TYPE_DEPOSIT_ && (int) $ext['id_customer']) {
    				    require_once(_KERAWEN_CLASS_DIR_.'/customer.php');
    				    adjustResellerAccount($ext['id_customer'], -$refund_wholesale, _KERAWEN_CA_EARN_CREDIT_, $order_detail->id, $order_detail->id_order);
    				}
    				if ($ext['product_type'] == _KERAWEN_TYPE_REFUND_) {
    				    require_once(_KERAWEN_CLASS_DIR_.'/customer.php');
    				    adjustResellerAccount($customer->id, -$refund_ti, _KERAWEN_CA_REFUND_CREDIT_, $order_detail->id, $order_detail->id_order);
    				}
				}
    				
				// Trigger hook
				// IMPORTANT: requires returned quantity as POST/GET parameter (paypal module)
				$cq = array();
				$cq[$order_detail->id] = $quantity;
				$_POST['cancelQuantity'] = $cq;
				Hook::exec('actionProductCancel', array(
					'order' => $slip->order,
					'id_order_detail' => $return['id_order_detail'],
					'product_quantity' => $quantity,
				), null, false, true, false, $slip->order->id_shop);
	
				if (!isset($slip->details[$order_detail->id])) {
					$slip->details[$order_detail->id] = array(
						// SET LATER ON 'id_order_slip' => $slip->id,
						'id_order_detail' => $order_detail->id,
						'product_quantity' => 0,
						'amount_tax_excl' => 0,
						'amount_tax_incl' => 0,
					);
				}
				$slip_detail = &$slip->details[$order_detail->id];
				$slip_detail['product_quantity'] += $quantity;
				$slip_detail['amount_tax_excl'] += $refund_te;
				$slip_detail['amount_tax_incl'] += $refund_ti;
	
				$slip->total_products_tax_excl += $refund_te;
				$slip->total_products_tax_incl += $refund_ti;
	
				// Back to stock
				if ($return['back_to_stock']) {
					// Check if combination is still available
					if (!$order_detail->product_attribute_id ||
						$db->getValue('
								SELECT 1 FROM '._DB_PREFIX_.'product_attribute
								WHERE id_product = '. (int) $order_detail->product_id .'
								AND id_product_attribute = '. (int) $order_detail->product_attribute_id)) {
						// Quite easy to return in original shop/warehouse, less in the current one
						if ($order_detail->id_warehouse) {
							injectStock(
							$order_detail->product_id,
							$order_detail->product_attribute_id,
							$order_detail->id_warehouse,
							$order_detail->id_shop,
							$id_reason,
							$quantity,
							0);
						}
						else {
							StockAvailable::updateQuantity(
							$order_detail->product_id, $order_detail->product_attribute_id,
							$quantity, $cart->id_shop,
							// Log stock movement from PS 1.7
							true, array(
								'id_order' => $id_slip_order,
								'id_stock_mvt_reason' => Configuration::get('PS_STOCK_CUSTOMER_RETURN_REASON')
							));
						}
					}
				}
			}
			else {
				// Shipping refunding
				$refund_ti = (float)$return['refund_tax_incl'];
				$refund_te = $refund_ti / (1.0 + $slip->order->carrier_tax_rate / 100.0);
	
				$slip->total_shipping_tax_excl += $refund_te;
				$slip->total_shipping_tax_incl += $refund_ti;
			}
				
			// Complete slip for backward compatibility
			$slip->amount = $slip->total_products_tax_incl + $slip->total_shipping_tax_incl;
			$slip->shipping_cost_amount = $slip->total_shipping_tax_incl;
			$slip->shipping_cost = $slip->total_shipping_tax_incl > 0 ? 1 : 0;
	
			// Increment global refund
			$total_refund_tax_incl += $refund_ti;
			$total_refund_tax_excl += $refund_te;
		}
	
		// Save slips and details
		$productList = array();
		$qtyList = array();
		$buf = array();
	
		foreach ($slips as $slip) {
			$slip->save();
				
			foreach ($slip->details as &$slip_detail) {
				$slip_detail['id_order_slip'] = $slip->id;
				$qtyList[$slip_detail['id_order_detail']] = $slip_detail['product_quantity'];
	
				if (Tools::version_compare(_PS_VERSION_, '1.6.0.10', '>=')) {
					$slip_detail = array_merge($slip_detail, array(
						'unit_price_tax_excl' => (float) ($slip_detail['amount_tax_excl'] / $slip_detail['product_quantity']),
					    'unit_price_tax_incl' => (float) ($slip_detail['amount_tax_incl'] / $slip_detail['product_quantity']),
					    'total_price_tax_excl' => (float) $slip_detail['amount_tax_excl'],
					    'total_price_tax_incl' => (float) $slip_detail['amount_tax_incl'],
					));
					$productList[] = array(
						'amount' => $slip_detail['amount_tax_incl'],
					);
					//TODO: MORE DATA REQUIRED ???
					//MINIMUM REQUIRED FOR FIDELISA
				}
				else {
					$productList[] = $slip_detail['id_order_detail'];
				}
					
				$db->insert('order_slip_detail', $slip_detail);
				$db->delete('return_kerawen', 'id_cart = '. (int) $id_cart .'
					AND id_order_detail = '. (int) $slip_detail['id_order_detail']);
			}
				
			$invoice_number = (int) $db->getValue('
				SELECT invoice_number FROM '._DB_PREFIX_.'orders
				WHERE id_order = '. (int) $slip->id_order);
				
			//TODO Report slip details as order
			require_once (dirname(__FILE__).'/shop.php');
			$buf[] = array(
				'number' => sprintf('%06d', $slip->id),
				'amount' => -$slip->amount,
				'url_invoice' => getAdminLink('AdminPdf', array(
					'submitAction' => 'generateOrderSlipPDF',
					'id_order_slip' => $slip->id,
				)),
				'id' => $slip->id,
				'id_order' => $slip->id_order,
				'cust' => isAnonymousCustomer($customer) ? null : $customer,
				'language' => isset($customer->id_lang) ? $customer->id_lang : 1,
				'prods' => $slip->details,
				'flagAddress' => ($invoice_number > 0),
			);
	
			// Trigger hook
			Hook::exec('actionOrderSlipAdd', array(
				'order' => $slip->order,
				'productList' => $productList,
				'qtyList' => $qtyList,
				'order_slip' => $slip,
			));
				
			$refund = new stdClass();
			$refund->return = $slip->order;
			$refund->mode = _KERAWEN_PM_RETURN_;
			$refund->label = $context->module->l('Exchanged products', pathinfo(__FILE__, PATHINFO_FILENAME));
			$refund->paid = -$slip->amount;
			array_unshift($payments, $refund);
		}
		$response->addResult('slips', $buf);
	
		// Register refunding as payment
		if ($total_refund_tax_incl > 0) {
			$refund = new stdClass();
			$refund->return = true;
			$refund->mode = _KERAWEN_PM_RETURN_;
			$refund->label = $context->module->l('Exchanged products', pathinfo(__FILE__, PATHINFO_FILENAME));
			$refund->paid = $total_refund_tax_incl;
			array_unshift($payments, $refund);
		}
	}

	$credits = array();
	if ($credit) $credits[] = $credit;
	
	$id_os = null;
	if ($order) {
		// Set order state depending on mode
		// TODO make it configurable
		$more = $db->getRow('SELECT * FROM `'._DB_PREFIX_.'order_kerawen` WHERE id_order = '. (int) $order->id);
		$mode = $more ? $more['delivery_mode'] : _KERAWEN_DM_DELIVERY_;
		
		$paid = 0.0;
		foreach ($payments as &$p) $paid += $p->paid;

		setPaymentLabel($order, $payments);
		$id_os = getPaymentStatus(($paid + $total_refund_tax_incl + $order->total_paid_real), $order->total_paid, $mode, $order->id);
		
		//$total_refund_tax_incl
	}
	
	// Update advanced stock if necessary
	if (Module::isEnabled('advancedstock')) {
		$os = new OrderState($id_os);
		if ($os->shipped) {
			require_once(_PS_MODULE_DIR_.'advancedstock/classes/Model/AdvancedStockExtendedOrderDetail.php');
			require_once(_PS_MODULE_DIR_.'advancedstock/classes/Model/AdvancedStockStockMovements.php');
			
			$ods = $order->getOrderDetailList();
			foreach ($ods as $od) {
				$eod = AdvancedStockExtendedOrderDetail::getEodFromOdId($od['id_order_detail']);
				if ($eod->eod_reserved_qty < $eod->eod_qty_to_ship) {
					AdvancedStockStockMovements::create(
						$od['product_id'], $od['product_attribute_id'],
						0, $eod->eod_warehouse_id,
						$eod->eod_qty_to_ship - $eod->eod_reserved_qty,
						AdvancedStockStockMovements::TYPE_ADJUSTMENT,
						'Stock adjustment from KerAwen',
						$id_employee);
					$eod->eod_reserved_qty = $eod->eod_qty_to_ship;
					$eod->save();
				}
			}
		}
	}
	
	changeOrderState($order, $id_os, $payments, $credits);
	$response->addResult('credits', $credits);
	
	$buf = $db->executeS('SELECT id_receipt FROM `'._DB_PREFIX_.'tpe_receipt_kerawen` WHERE id_cart = ' . (int) $id_cart);
	$tpeReceipt = array();
	foreach ($buf as $receipt) {
	    $tpeReceipt[] = $receipt['id_receipt'];
	}
	$response->addResult('tpeReceipt', $tpeReceipt);
	
	
	if ($order) {
		// Do not return order at the moment
		// NF525 logging still to occur
		$response->addResult('id_order', $order->id);
	}
	
	// Delete cart
	if (!$cartExtended['quote']) {
	    require_once (dirname(__FILE__).'/cart.php');
	    deleteCart($cart);
	}
}

function getOrderReference($id_customer)
{
	// TEMP
	return Order::generateReference();
	
	$ref = false;
	$db = Db::getInstance();
	$info = $db->getRow('
		SELECT account, reference
		FROM '._DB_PREFIX_.'customer_kerawen
		WHERE id_customer = '. (int) $id_customer);
	if ($info && $info['account']) $ref = $info['reference'];
	if (!$ref) $ref = Order::generateReference();
	if ($info && $info['account']) {
		$db->update('customer_kerawen', array(
			'reference' => pSQL($ref)
		), 'id_customer = '. (int) $id_customer);
	}
	return $ref;
}

function handleOrderCreation($order, $cart, $state)
{
	$db = Db::getInstance();

	/* DO NOT CHANGE REFERENCE ANY MORE ?
	// Update Reference for accounted customer
	$order->reference = getOrderReference($order->id_customer);
	$order->save();
	*/
	
	// Get additional data from cart
	$more = $db->getRow('SELECT * FROM `'._DB_PREFIX_.'cart_kerawen` WHERE id_cart = '. (int) $cart->id);

	if ($more) {
	    $mode = $more['delivery_mode'];
	} else {
	    $mode = _KERAWEN_DM_DELIVERY_;
	    //mode by carrier
	    if (Configuration::get('KERAWEN_CARRIER_MODE') && (int) $order->id_carrier) {
	        $carrier_mode = unserialize(Configuration::get('KERAWEN_CARRIER_MODE'));
	        $id_reference = (int) $db->getValue('SELECT id_reference FROM '._DB_PREFIX_.'carrier WHERE id_carrier = ' . (int) $order->id_carrier);
	        if ($id_reference && is_array($carrier_mode) && isset($carrier_mode[$id_reference])) {
	            $mode = $carrier_mode[$id_reference];
	        }
	    }
	}

	$date = $more ? $more['delivery_date'] : null;
	$private_note = $more ? $more['private_note'] : null;
	$invoice_note = $more ? $more['public_note'] : null;
	
	
	if ($mode == _KERAWEN_DM_IN_STORE_) $date = date('Y-m-d H:i:s');
	
	//print in kitchen
	//TODO : add setting to include delay, and use delivery_date instead for modes 1,2
	$print_date = date('Y-m-d H:i:s');
	

	// Transform credits from rules to payments
	applyOrderCredits($order);
	
	// Register global discount by cart rules
	$discount = 0;
	$free_shipping = 0;
	$crs = $order->getCartRules();
	foreach ($crs as $cr)
	{
		$discount += $cr['value_tax_excl'];
		if ($cr['free_shipping'])
		{
			$free_shipping = 1;
			$discount -= $order->total_shipping_tax_excl;
		}
	}
	
	$id_till = getExtendedContext('id_cashdrawer', 0);
	$id_employee = $more ? $more['id_employee'] : 0;
	$product_global_discount = ($order->total_products == 0) ? 1 : ($discount / $order->total_products);
	$preparation_status = Configuration::get('KERAWEN_OS_RECEIVED');
	$round_type = Configuration::get('PS_ROUND_TYPE');
	$accounted = 0;

	$q = '
        INSERT INTO '._DB_PREFIX_.'order_kerawen
		(
            id_order,
            id_till,
            id_employee,
            delivery_mode,
            delivery_date,
            private_note,
            invoice_note,
            product_global_discount,
            free_shipping,
            preparation_status,
            round_type,
            accounted,
            print_date
        ) VALUES ('
            . (int) $order->id . ','
            . (int) $id_till . ','
            . (int) $id_employee . ','
            . (int) $mode . ','
            . '"' . pSQL($date) . '",'
            . '"' . pSQL($private_note) . '",'
            . '"' . pSQL($invoice_note) . '",'
            . $product_global_discount . ','
            . (int) $free_shipping . ','
            . (int) $preparation_status . ','
            . (int) $round_type . ','
            . (int) $accounted . ','
            . '"' . pSQL($print_date) .'"
        )
		ON DUPLICATE KEY
        UPDATE
            id_till = VALUES(id_till),
            id_employee = VALUES(id_employee),
            delivery_mode = IF(delivery_mode IS NULL, VALUES(delivery_mode), delivery_mode),
            delivery_date = IF(delivery_date IS NULL, VALUES(delivery_date), delivery_date),
            private_note = IF(private_note IS NULL, VALUES(private_note), private_note),
            invoice_note = IF(invoice_note IS NULL, VALUES(invoice_note), invoice_note),
            product_global_discount = VALUES(product_global_discount),
            free_shipping = VALUES(free_shipping),
            preparation_status = VALUES(preparation_status),
            round_type = VALUES(round_type),
            accounted = VALUES(accounted),
            print_date = IF(print_date IS NULL, VALUES(print_date), print_date)
    ';

	$db->execute($q);
	
                               
	// Register weights and measures
	completeOrderDetail($order);

	// Register ecotaxes
	registerEcotax($order);
	
	// Register state
	handleOrderState($order->id, $state, true);
	
	// Kitchen print
	handleOrderReceipt($order);
	
}


function handleOrderReceipt($order) {

    if (Configuration::get('KERAWEN_PRINT_KITCHEN')) {
    
        $db = Db::getInstance();
        
        $id_till = (int) getExtendedContext('id_cashdrawer', 0);
        if ($id_till == 0) {
            //Multishop + shop/till activated
            if (Configuration::get('KERAWEN_TILL_SHOP')) {
                $q = '
                    SELECT DISTINCT 
                        cdk.printer_remote AS mac
                    FROM '._DB_PREFIX_.'cashdrawer_shop_kerawen csk
                    INNER JOIN '._DB_PREFIX_.'cash_drawer_kerawen cdk ON cdk.id_cash_drawer = csk.id_cash_drawer
                    WHERE COALESCE( cdk.printer_remote, "") != "" AND csk.id_shop = ' . (int) $order->id_shop;
            } else {
                //First found
                $q = '
                    SELECT DISTINCT
                        cdk.printer_remote AS mac
                    FROM '._DB_PREFIX_.'cash_drawer_kerawen cdk
                    WHERE COALESCE( cdk.printer_remote, "") != ""
                    ORDER BY cdk.id_cash_drawer
                    LIMIT 0,1';
            } 
            
            //Need more specific options
            
            
        } else {
            $q = '
                SELECT 
                    cdk.printer_remote AS mac 
                FROM ps_cash_drawer_kerawen cdk 
                WHERE COALESCE( cdk.printer_remote, "") != "" AND cdk.id_cash_drawer = ' . (int) $id_till;
        }
        
        $printers = $db->executeS($q);
    
        $data = array(
            'jsonrpc' => '2.0',
            'method' => 'printfeed',
            'params' => array(
                'doc' => $order->id
            ),
        );
    
        if ($printers) {
            foreach ($printers as $printer) {
                //need curl_multi_init() ?
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, _KERAWEN_SERVER_NODE_PRINTER_RECEIPT_ . "/" . $printer['mac']);
                curl_setopt($ch, CURLOPT_POST, 1);
                curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
                curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 1);
                curl_setopt($ch, CURLOPT_TIMEOUT, 1);
                
                $server_output = curl_exec($ch);
                curl_close($ch);
            }
        }
    }
}

/**
* Change current order status
*
* @param int $id_order
* @param int $id_state
*/
function changeOrderState($order, $id_state, $payments, &$credits)
{
    $db = Db::getInstance();
    
	if ($order && $id_state) {
		$os = new OrderState($id_state, $order->id_lang);

		// Override standard behaviour wrt order state
		
		// Do not generate invoice under some conditions
		if (!Configuration::get('KERAWEN_INVOICE_AUTO')) $os->invoice = false;
		if (!$order->id_address_invoice) $os->invoice = false;
		if ($order->id_address_invoice == getDefaultDeliveryAddress(false)) $os->invoice = false;
		
		// Do not save but force caching
		$cache_id = 'objectmodel_'.get_class($os).'_'.(int)$os->id.'_'.(int)null.'_'.(int)$order->id_lang;
		Cache::store($cache_id, $os);

		// Additional data for order state hook
		setExtendedContext('payments', $payments);
		setExtendedContext('credits', $credits);
		
		//ZEROSIX -> hookactionOrderStatusUpdate()
		$context = Context::getContext();
		$context->cart = new Cart($order->id_cart);

		$oh = new OrderHistory();
		$oh->id_employee = isset($context->employee) ? $context->employee->id : null;
		$oh->id_order = $order->id;
		$oh->changeIdOrderState($id_state, $order, true);

		$fakemail = 0;
		$fakemail = (int)$db->getValue('
			SELECT ck.fakemail FROM '._DB_PREFIX_.'customer_kerawen AS ck WHERE ck.id_customer = '. (int) $order->id_customer); 
		if ((Configuration::get('KERAWEN_SEND_EMAIL')) && ($fakemail != 1)) {
			$oh->addWithemail();
		}
		else {
			$oh->add();
		}
		
		// Get results back
		$credits = getExtendedContext('credits', array());
	
		// Stock to be synchronized after state change
		// Override stock movement reason
		$id_reason = Configuration::get('KERAWEN_MR_SALE');
		if ($id_reason) Configuration::set('PS_STOCK_CUSTOMER_ORDER_REASON', $id_reason);
	
		if (Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT')) {
			$prods = $order->getProducts();
			foreach ($prods as $product)
			if (StockAvailable::dependsOnStock($product['product_id']))
				StockAvailable::synchronize($product['product_id'], $order->id_shop);
		}
	}
	elseif ($payments) {
	    setPaymentLabel($order, $payments);
		registerPayments($order, $payments, $credits);
		
		// DO NOT Adjust payment state if necessary
		/*
		require_once(_KERAWEN_CLASS_DIR_.'/order_state.php');
		if ($order && isOrderStateValid($order->current_state)) {
		    $more = $db->getRow('SELECT * FROM `'._DB_PREFIX_.'order_kerawen` WHERE id_order = '. (int) $order->id);
		    $mode = $more ? $more['delivery_mode'] : _KERAWEN_DM_DELIVERY_;
		    
		    //exchance product missing in total paid real
		    $total_paid = (float) $db->getValue('SELECT SUM(op.amount) AS paid FROM '._DB_PREFIX_.'order_payment op WHERE op.order_reference = "' . pSQL($order->reference) . '"');
		    
		    $id_state = getPaymentStatus($total_paid, $order->total_paid, $mode, $order->id);
			
			if ($id_state != $order->current_state) {
				$context = Context::getContext();
			
				$oh = new OrderHistory();
				$oh->id_employee = isset($context->employee) ? $context->employee->id : null;
				$oh->id_order = $order->id;
				$oh->changeIdOrderState($id_state, $order, true);
				$oh->add();
			}
		}
		*/
	}
}

function changeOrderMode($id_order, $id_order_payment, $id_mode, $mode, $date_deferred = 0) 
{
	$db = Db::getInstance();
	
	$flow_data = array(
	    'id_payment_mode' => (int) $id_mode
	);
	if ($date_deferred) {
	    $flow_data['date_deferred'] = $db->getValue('SELECT NOW()');
	}
	
	$db->update('cashdrawer_flow_kerawen', $flow_data, 'id_order_payment = '. (int) $id_order_payment);
	$db->update('order_payment_kerawen', array('id_mode'=>  (int) $id_mode), 'id_order_payment = '. (int) $id_order_payment);

	setExtendedContext('id_payment_mode', $id_mode);
	$op = new OrderPayment($id_order_payment);
	$op->payment_method = $mode;
	$op->save();
	setExtendedContext('id_payment_mode', null);

	$order = new order($id_order);
	setPaymentLabel($order, array());
}


function changeOrderCustomer($id_order, $id_customer)
{
	$db = Db::getInstance();
	
	//PS
	$op = new Order($id_order);
	$op->id_customer = $id_customer;
	$op->save();
	
	//525
	$id_sale = $db->getValue('SELECT id_sale FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order WHERE ps_order = '. (int) $id_order);
	$customer_name = $db->getValue('SELECT CONCAT(firstname, " ", lastname) FROM '._DB_PREFIX_.'customer WHERE id_customer = '. (int) $id_customer);
	if ($id_sale) {
		$db->execute('UPDATE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale SET id_customer = '. (int) $id_customer .', customer_name = "'.pSQL($customer_name).'" WHERE id_sale = '. (int) $id_sale);
	}
}

function changeOrderNote($id_order, $private_note, $invoice_note)
{
    Db::getInstance()->update('order_kerawen', array('delivery_note' => '', 'private_note' => pSQL($private_note), 'invoice_note' => pSQL($invoice_note)), 'id_order = '. (int) $id_order);
}

function changeOrderShop($id_order, $id_shop)
{
    $db = Db::getInstance();
    
    $ps_order = new Order($id_order);
    $init_shop = $ps_order->id_shop;    
    $ps_order->id_shop = (int) $id_shop;
    $ps_order->save();
    
    //525
    $id_operation = (int) $db->getValue('
        SELECT 
            s.id_operation 
        FROM ' . _DB_PREFIX_._KERAWEN_525_PREFIX_ . 'order o
        INNER JOIN ' . _DB_PREFIX_._KERAWEN_525_PREFIX_ . 'sale s ON o.id_sale = s.id_sale
        WHERE o.ps_order = '. (int) $id_order
    );
    
    if ($id_operation) {
        $ps_shop_name = Configuration::get('PS_SHOP_NAME', null, null, (int) $id_shop);
        $db->execute('UPDATE ' . _DB_PREFIX_._KERAWEN_525_PREFIX_ . 'operation SET id_shop = ' . (int) $id_shop .', PS_SHOP_NAME = "' . pSQL($ps_shop_name) . '" WHERE id_operation = '. (int) $id_operation);
    }
    
}


function changeOrderEmployee($id_order, $id_empl)
{
    $db = Db::getInstance();

    $id_operation = (int) $db->getValue('
        SELECT
            s.id_operation
        FROM ' . _DB_PREFIX_._KERAWEN_525_PREFIX_ . 'order o
        INNER JOIN ' . _DB_PREFIX_._KERAWEN_525_PREFIX_ . 'sale s ON o.id_sale = s.id_sale
        WHERE o.ps_order = '. (int) $id_order
    );
    
    if ($id_operation) {
        $db->execute('UPDATE ' . _DB_PREFIX_ . 'order_kerawen SET id_employee = ' . (int) $id_empl .' WHERE id_order = '. (int) $id_order);
        
        $operator_name = $db->getValue('SELECT CONCAT(firstname, " ", lastname) AS operator_name FROM ' . _DB_PREFIX_ . 'employee WHERE id_employee =' . (int) $id_empl);
        $db->execute('UPDATE ' . _DB_PREFIX_._KERAWEN_525_PREFIX_ . 'operation SET id_operator = ' . (int) $id_empl .', operator_name = "' . pSQL($operator_name) . '" WHERE id_operation = '. (int) $id_operation);
    }
}


function changeDelivery($id_order, $delivery_date, $delivery_mode)
{
    $db = Db::getInstance();
    $q = '
        INSERT INTO '._DB_PREFIX_.'order_kerawen
		(
           id_order,
           delivery_date,
           delivery_mode
        ) VALUES ('
           . (int) $id_order . ','
           . '"' . pSQL($delivery_date) . '",'
           . (int) $delivery_mode . '
        )
		ON DUPLICATE KEY
        UPDATE
           delivery_date = VALUES(delivery_date),
           delivery_mode = VALUES(delivery_mode)
    ';
    $db->execute($q);
    
    $db->update('orders', array('delivery_date' => pSQL($delivery_date)), 'id_order = '. (int) $id_order);
}


function completeResellerAccount($order, $is_valid) {
    
    $context = Context::getContext();
    
    foreach ($order->getOrderDetailList() as $od) {
        
        // Specific products
        $ext = Db::getInstance()->getRow('
			SELECT
                pk.product_type AS product_type,
                pk.id_customer AS id_customer
			FROM '._DB_PREFIX_.'product_wm_kerawen pk
			JOIN '._DB_PREFIX_.'product p ON p.id_product = pk.id_product
			WHERE
				pk.id_product = '. (int) $od['product_id'].'
                AND pk.product_type = "' . _KERAWEN_TYPE_DEPOSIT_ . '"
                AND IFNULL(pk.id_customer, 0) > 0
         ');
        
        if ($ext) {
                       
            require_once(_KERAWEN_CLASS_DIR_.'/customer.php');
            require_once(_KERAWEN_CLASS_DIR_.'/catalog.php');
            
            $field_wholesale_price = (Tools::version_compare(_PS_VERSION_, '1.6.1.1', '>=')) ? 'original_wholesale_price' : 'purchase_supplier_price';
            $sign = ($is_valid) ? 1 : -1;
            $value =  $sign * $od[$field_wholesale_price] * ($od['product_quantity'] - $od['product_quantity_refunded'] - $od['product_quantity_return']);

            adjustResellerAccount($ext['id_customer'], $value, _KERAWEN_CA_EARN_CREDIT_, $od['id_order_detail'], $order->id);
            
            $pmode = 0;
            addPayment(false, false, $pmode, $context->module->l('Depositor account', pathinfo(__FILE__, PATHINFO_FILENAME)), -$value);
            

            if ($is_valid) {
                $prod = new Product($od['product_id'], false);
                sendEmailToReseller($prod, $ext['id_customer'], 'sold');
            }

        }
    }
}


function handleOrderState($id_order, $os_new, $create)
{
	$db = Db::getInstance();
	
	$order = new Order($id_order);
	$os_curr = new OrderState($order->current_state);
	
	// Do not register payment automatically
	$context = Context::getContext();
	if (isset($context->kerawen) && isset($context->kerawen->kerawen)) {
	    $os_new->paid = 0;
	}
	$cache_id = 'objectmodel_'.get_class($os_new).'_'.(int)$os_new->id.'_'.(int)null.'_'.(int)$order->id_lang;
	Cache::store($cache_id, $os_new);
	
	require_once (_KERAWEN_CLASS_DIR_.'/order_state.php');
	require_once (_KERAWEN_CLASS_DIR_.'/drawer.php');
	
	$new = $create || !$order->current_state;
	$previously_valid = !$create && isOrderStateValid($os_curr->id);
	$currently_valid = isOrderStateValid($os_new->id);
	$changed = false;
	
	if (!$previously_valid && $currently_valid)
	{
		// Order has been (re)validated
		logOrderState($order->reference, $order->id, _KERAWEN_CDSO_ORDER_, $new, $order->total_paid_tax_incl, $new ? 0 : $order->total_paid_real, $order->payment);
		$changed = true;
	}
	if ($previously_valid && !$currently_valid)
	{
		// Order has been canceled
		logOrderState($order->reference, $order->id, _KERAWEN_CDSO_CANCEL_, $new, $order->total_paid_tax_incl, $order->total_paid_real, null);
		$changed = true;
		
	}

	if ($changed)
	{
		$payments = getExtendedContext('payments', array());
		if (!$payments) $payments = array();
		setExtendedContext('payments', $payments);
		setExtendedContext('regul', false);
		
		completeResellerAccount($order, $currently_valid);
		
	}
	
	// Check and correct stock if necessary
	if (!$os_curr->shipped && $os_new->shipped) {
		require_once(dirname(__FILE__).'/stock.php');
		$id_reason = getStockShippingReason();
		
		foreach ($order->getProductsDetail() as $detail) {
			if ((int)$detail['id_warehouse']) {
				$available = (int)$db->getValue('
					SELECT SUM(usable_quantity)
					FROM '._DB_PREFIX_.'stock
					WHERE id_warehouse = '. (int) $detail['id_warehouse'].'
					AND id_product = '. (int) $detail['product_id'].'
					AND id_product_attribute = '. (int) $detail['product_attribute_id']);
				$missing = (int)$detail['product_quantity'] - $available;

				if ($missing > 0) {
					injectStock(
						$detail['product_id'],
						$detail['product_attribute_id'],
						$detail['id_warehouse'],
						$detail['id_shop'],
						$id_reason,
						$missing,
						0);
				}
			}
		}
	}
}

function handlePostOrderState($id_order, $os_new)
{
	updateOrderStatus($id_order, isPreparationStatus($os_new->id) ? $os_new->id : null);

	// Register payments if any
	$payments = getExtendedContext('payments', false);
	if ($payments !== false)
	{
		$order = new Order($id_order);
		$credits = getExtendedContext('credits', array());
		registerPayments($order, $payments, $credits);
		
		// Avoid second registering
		setExtendedContext('payments', false);
		setExtendedContext('credits', $credits);
	}
	
	// Update prepared quantities (AFTER stock has been updated)
	if ($os_new->id == (int)Configuration::get('PS_OS_PREPARATION'))
	{
		$db = Db::getInstance();
		$db->execute('
			INSERT INTO '._DB_PREFIX_.'order_detail_kerawen (id_order_detail)
			SELECT od.id_order_detail FROM '._DB_PREFIX_.'order_detail od
				WHERE od.id_order = '. (int) $id_order .'
			ON DUPLICATE KEY UPDATE id_order_detail = od.id_order_detail');
		
		$db->execute('
			UPDATE '._DB_PREFIX_.'order_detail_kerawen odk
			INNER JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = odk.id_order_detail
			SET odk.quantity_ordered = od.product_quantity,
				odk.measure_ordered = odk.measure
			WHERE od.id_order = '. (int) $id_order .'
				AND odk.quantity_ordered IS NULL');
	}
}


function handleOrderSlip($slip)
{
	require_once (_KERAWEN_CLASS_DIR_.'/drawer.php');
	logOrderSlip(null /*$slip->order->reference*/, $slip->id_order, $slip->id);
	
	if (!getExtendedContext('kerawen', false)) {
		$amount = $slip->amount;
	}
}

function registerPayments($order, $payments, &$credits)
{
	$context = Context::getContext();
	$db = Db::getInstance();
	
	foreach ($payments as $p)
	{
		if (!isset($p->order)) $p->order = $order;
		if (!isset($p->return)) $p->return = false;
		$id_customer = isset($order->id_customer) ? $order->id_customer : $context->customer->id;
		
		$credit = false;
		if (isset($p->credit) && $p->credit) {
			
			if ($p->paid < 0) {
				// Generate credit
				require_once (dirname(__FILE__).'/cartrules.php');
				$credit = createCredit(-$p->paid, $id_customer);
				$credits[] = $credit;
			}
			
			if ($p->paid > 0 && isset($p->id_rule)) {
				// Consume credit
				$db->execute('
					UPDATE '._DB_PREFIX_.'cart_rule
					SET quantity = quantity - 1
					WHERE id_cart_rule = '. (int) $p->id_rule);
			}
		}
		addPayment($p->order, $p->return, $p->mode, $p->label, $p->paid, $credit ? $credit['id'] : null, isset($p->date) ? $p->date : null, (isset($p->remain) && (int)$p->remain == 1) ? 1 : null, isset($p->id_receipt) ? (int) $p->id_receipt : null);
		
		if($p->mode == _KERAWEN_PM_ACCOUNT_) {
			require_once(dirname(__FILE__).'/customer.php');
			adjustCustomerAccount($id_customer, -$p->paid, _KERAWEN_CA_USE_CREDIT_, 0, $order->id);
		}
		if($p->mode == _KERAWEN_PM_DEPOSIT_) {
		    require_once(dirname(__FILE__).'/customer.php');
		    adjustResellerAccount($id_customer, -$p->paid, _KERAWEN_CA_USE_CREDIT_, 0, $order->id);
		}

	}
	if ($order) $order->save();
}

function addPayment($order, $return, $mode, $label, $value, $id_credit = null, $date = null, $remain = null, $id_receipt = null)
{
	$context = Context::getContext();
	$db = Db::getInstance();
	$value = (float)Tools::ps_round($value, _PS_PRICE_COMPUTE_PRECISION_);

	$main_ref = $other_ref = null;
	if (!$return) {
		$main_ref = $order ? $order->reference : null;
	}
	else if ($return === true) {
		$main_ref = $order ? $order->reference : null;
		$other_ref = '--';
	}
	else {
		$other_ref = $return->reference;
	}
	
	$op = new OrderPayment();
	$op->order_reference = $main_ref;
	$op->id_currency = $order ? $order->id_currency : $context->currency->id;
	$op->conversion_rate = 1;
	$op->payment_method = $label;
	$op->amount = $value;
	$op->transaction_id = $id_credit;
	
	// Additional data for payment hook
	setExtendedContext('id_payment_mode', $mode);
	setExtendedContext('id_credit', $id_credit);
	setExtendedContext('date_deferred', $date);
	
	if (!$op->add())
		throw new PrestaShopException('Can\'t save payment');

	$db->insert('order_payment_kerawen', array(
		'id_order_payment' => (int) $op->id,
	    'reference' => pSQL($other_ref),
	    'id_mode' => (int) $mode,
	    'id_credit' => $id_credit,
	    'deferred' => pSQL($date),
	    'remain' => pSQL($remain),
	    'id_receipt' => (int) $id_receipt > 0 ? (int) $id_receipt : null,
	), true);
	
	// Cancel data for payment hook
	setExtendedContext('id_payment_mode', null);
	setExtendedContext('id_credit', null);
	setExtendedContext('date_deferred', null);
	
	if ($order) {
		// Link payment to invoice(s)
		$invoices = $order->getInvoicesCollection();
		foreach ($invoices as $invoice) {
			$db->insert('order_invoice_payment', array(
				'id_order_invoice' => (int)$invoice->id,
				'id_order_payment' => (int)$op->id,
				'id_order' => (int)$order->id,
			), true);
		}
	
		$order->total_paid_real += $value;
		// FIXME Maybe the payment has been registered on another order
		if ($order->total_paid_real < 0) $order->total_paid_real = 0;
	}
}

function handleOrderPayment($payment, $delete)
{
	require_once (_KERAWEN_CLASS_DIR_.'/drawer.php');
	
	$mode = getExtendedContext('id_payment_mode', null);
	if ($mode !== false)
	{
		if ($payment->amount == 0) {
			// Do not keep payment with no amount
			Db::getInstance()->delete('order_payment', 'id_order_payment = '. (int) $payment->id);
		}
		else {
			// Get back additionnal data
			$id_order = Db::getInstance()->getValue('
				SELECT id_order FROM '._DB_PREFIX_.'orders
				WHERE reference = "'.pSQL($payment->order_reference).'"');
			if (!$id_order) $id_order = null;
			$date = getExtendedContext('date_deferred', null);
			$id_credit = getExtendedContext('id_credit', null);
			
			logPayment($payment->order_reference, $id_order, null, $payment->id, $mode, $payment->amount, $id_credit, $date, null, $delete);
		}
	}
	
	// Update gift cards status according to paid status
	require_once(_KERAWEN_CLASS_DIR_.'/cartrules.php');
	updateGiftCard($payment->order_reference, false);
}

/**
 * Transfers credits from reductions to payments
 * in order to compute taxes correctly
 */
function applyOrderCredits($order)
{
    require_once(dirname(__FILE__).'/data.php');
    
	$db = Db::getInstance();
	$credit_tax_excl = 0.0;
	$credit_tax_incl = 0.0;
	$orules = $order->getCartRules();

	$pmodes = array(
		_KERAWEN_CR_CREDIT_ => _KERAWEN_PM_CREDIT_,
		_KERAWEN_CR_GIFT_CARD_ => _KERAWEN_PM_GIFT_CARD_,
	);
	
	foreach ($orules as $orule)
	{
		// Check if rule is a credit or prepaid
		$type = $db->getValue('
			SELECT `type`
			FROM '._DB_PREFIX_.'cart_rule_kerawen
			WHERE id_cart_rule = '. (int) $orule['id_cart_rule']);
		
		if (isset($pmodes[$type])) {
			$pmode = $pmodes[$type];
			
			// Remove from rules
			$db->delete('order_cart_rule',
				'id_order_cart_rule = '. (int) $orule['id_order_cart_rule']);
			$credit_tax_excl += $orule['value_tax_excl'];
			$credit_tax_incl += $orule['value'];

			// Find the corresponding cart_rule
			$row = $db->getRow('
				SELECT `reduction_amount`, `code`, `partial_use` FROM `'._DB_PREFIX_.'cart_rule`
				WHERE `id_cart_rule`='. (int) $orule['id_cart_rule']);
			$amount_init = $row['reduction_amount'];
			$amount = $orule['value'];
			$partial_use = $row['partial_use'];
			$code = $row['code'];
			
			// Guess if the rule has been split
			// FIXME
			$eps = 0.009;
			if ($amount_init >= ($amount - $eps) && $amount_init <= ($amount + $eps))
				addPayment($order, false, $pmode, $orule['name'], $amount, $orule['id_cart_rule']);
			else
			{
				addPayment($order, false, $pmode, $orule['name'], $amount_init, $orule['id_cart_rule']);
				addPayment($order, false, $pmode, $orule['name'], $amount - $amount_init, $orule['id_cart_rule']);
				try
				{
				    if ($partial_use && !getExtendedContext('id_cashdrawer', 0)) {

				        if (preg_match('/(.*)\-([0-9]{1,2})$/', $code, $matches) && $matches[1] && $matches[2]) {
				            $next = (int) $matches[2] + 1;
				            $code = $matches[1] . '-' . $next;
				        } else {
				            $code = $code . '-2';
				        }
				        
    					$id_ps_rule = (int) $db->getValue('
    						SELECT `id_cart_rule` FROM `'._DB_PREFIX_.'cart_rule`
    						WHERE `code` = "' . pSQL($code) . '"');

    					if ($id_ps_rule)
    					{
    						$id_parent_cart_rule = (int) $db->getValue('
    						  SELECT id_parent_cart_rule 
                                FROM `'._DB_PREFIX_.'cart_rule_kerawen`
    						  WHERE `id_cart_rule` = ' . (int) $orule['id_cart_rule']
    						);
    						
    						$id_parent_cart_rule = $id_parent_cart_rule > 0 ? $id_parent_cart_rule : (int) $orule['id_cart_rule'];
    						
    						//set as kerawen credit/prepaid
    						$db->execute('
    							INSERT INTO '._DB_PREFIX_.'cart_rule_kerawen
    							(id_cart_rule, `type`, id_order, id_parent_cart_rule)
    							VALUES ('. (int) $id_ps_rule .', "'.pSQL($type).'", ' . (int) $order->id . ', ' . (int) $id_parent_cart_rule . ')');
    						
    						//Anonymous ? shouldn't be
    						if ($type == _KERAWEN_CR_GIFT_CARD_ && isAnonymousCustomer($order->id_customer)) {
    						    $rule = new CartRule($id_ps_rule);
    						    $rule->id_customer = 0;
    						    $rule->save();
    						}
    					}    					
				    }
				}
				catch (Exception $e)
				{
					// order from the cash register
					die('Exception caught '.$e);
				}
			}
		}
	}

	// Reset order totals
	$order->total_discounts_tax_excl = Tools::ps_round($order->total_discounts_tax_excl - $credit_tax_excl, _PS_PRICE_COMPUTE_PRECISION_);
	$order->total_discounts_tax_incl = Tools::ps_round($order->total_discounts_tax_incl - $credit_tax_incl, _PS_PRICE_COMPUTE_PRECISION_);
	
	// TODO Fix rounding issue
	if ($order->total_discounts_tax_excl < 0.0) {
		$credit_tax_excl += $order->total_discounts_tax_excl;
		$order->total_discounts_tax_excl = 0.0;
	}
	if ($order->total_discounts_tax_incl < 0.0) {
		$credit_tax_incl += $order->total_discounts_tax_incl;
		$order->total_discounts_tax_incl = 0.0;
	}
	
	$order->total_discounts = $order->total_discounts_tax_incl;
	$order->total_paid_tax_excl = Tools::ps_round($order->total_paid_tax_excl + $credit_tax_excl, _PS_PRICE_COMPUTE_PRECISION_);
	$order->total_paid_tax_incl = Tools::ps_round($order->total_paid_tax_incl + $credit_tax_incl, _PS_PRICE_COMPUTE_PRECISION_);
	$order->total_paid = $order->total_paid_tax_incl;
	
	// Update order detail taxes
	if (Tools::version_compare(_PS_VERSION_, '1.6.1', '<')) {
		foreach ($order->getOrderDetailList() as $row) {
			$detail = new OrderDetail($row['id_order_detail']);
			$detail->updateTaxAmount($order);
		}
	}
	else {
		$order->updateOrderDetailTax();
	}
	
	if (!$order->save())
		throw new PrestaShopException('Can\'t save order');
}

function completeOrderDetail($order)
{
	// Generate gift card (disabled, waiting for payment)
	require_once(_KERAWEN_CLASS_DIR_.'/cartrules.php');
	setGiftCard($order->id, 'createCredit');
	// Activate it immediately if order is paid
	updateGiftCard($order->reference);
	
	$db = Db::getInstance();
	$tax_address = new Address((int)$order->{Configuration::get('PS_TAX_ADDRESS_TYPE')});
		
	foreach ($order->getOrderDetailList() as $od)
	{
		$more = $db->getRow('
			SELECT
				ps.id_tax_rules_group
			FROM '._DB_PREFIX_.'product_shop ps
			WHERE ps.id_product = '. (int) $od['product_id'].'
			AND ps.id_shop = '. (int) $od['id_shop']);
		
		$more['product_type'] = _KERAWEN_TYPE_STANDARD_;


		// Specific products
		$ext = $db->getRow('
			SELECT
				pk.is_account_credit,
				pk.unit AS unit,
				pk.`precision` AS `precision`,
                pk.rest_voucher AS rest_voucher,
                IFNULL(pk.product_type, "' . _KERAWEN_TYPE_STANDARD_ . '") AS product_type
			FROM '._DB_PREFIX_.'product_wm_kerawen pk
			JOIN '._DB_PREFIX_.'product p ON p.id_product = pk.id_product
			WHERE
				pk.id_product = '. (int) $od['product_id']);
		
		if ($ext) {

		    $more['product_type'] = $ext['product_type'];
		    
		    if ((int) $ext['rest_voucher'] == 1) {
		        $more['rest_voucher'] = 1; 
		    }
			
			if ($ext['product_type'] == _KERAWEN_TYPE_PREPAID_ || $ext['is_account_credit']) {
			    adjustCustomerAccount($order->id_customer, $od['total_price_tax_incl'], _KERAWEN_CA_BUY_CREDIT_, $od['id_order_detail'], $order->id);
			}
			if ($ext['product_type'] == _KERAWEN_TYPE_REFUND_) {
			    // Should not occur ?
			    adjustResellerAccount($order->id_customer, $od['total_price_tax_incl'], _KERAWEN_CA_REFUND_CREDIT_, $od['id_order_detail'], $order->id);
			}


    		$ext_attr = $db->getRow('
    			SELECT
                    pak.measure AS measure,
    				ck.unit_price AS unit_price,
    				ck.id_product_attribute AS id_product_attribute
    			FROM '._DB_PREFIX_.'product_wm_kerawen pk
    			JOIN '._DB_PREFIX_.'product p ON p.id_product = pk.id_product
    			LEFT JOIN '._DB_PREFIX_.'product_attribute pa ON pa.id_product = p.id_product
    			LEFT JOIN '._DB_PREFIX_.'product_attribute_kerawen pak ON pak.id_product_attribute = pa.id_product_attribute
    			LEFT JOIN '._DB_PREFIX_.'product_wm_code_kerawen ck ON ck.id_code = pak.id_code
    			WHERE
    				pk.id_product = '. (int) $od['product_id'].'
    				AND IFNULL(pa.id_product_attribute, 0) = '. (int) $od['product_attribute_id']);
    		
    		if ($ext_attr) {
        		if ($ext_attr['measure'] && $ext_attr['unit_price'] > 0) {
        		    $tax_manager = TaxManagerFactory::getManager($tax_address, $more['id_tax_rules_group']);
        		    $tax_calculator = $tax_manager->getTaxCalculator();
        		    $reduction = 1 - $od['group_reduction'] / 100;
        		    
        		    $more = array_merge($more, array(
        		        'measure' => $ext_attr['measure'],
        		        'unit' => $ext['unit'],
        		        'precision' => $ext['precision'],
        		        'unit_price_tax_excl' => $ext_attr['unit_price']*$reduction,
        		        'unit_price_tax_incl' => $tax_calculator->addTaxes($ext_attr['unit_price'])*$reduction,
        		    ));
        		    
        		    if ((int) $ext_attr['id_product_attribute'] > 0) {
        		        // Decrement related standard combination
        		        $db->update('order_detail', array(
        		            'product_attribute_id' => (int) $ext_attr['id_product_attribute'],
        		        ), 'id_order_detail = '. (int) $od['id_order_detail']);
        		    }
        		    
        		    // Update wholesale data
        		    $update = '
        					purchase_supplier_price = purchase_supplier_price*'.pSQL($ext_attr['measure']);
        		    if (Tools::version_compare(_PS_VERSION_, '1.6.1.1', '>=')) $update .= ',
        					original_wholesale_price = original_wholesale_price*'.pSQL($ext_attr['measure']);
        		    
        		    $db->execute('
        					UPDATE '._DB_PREFIX_.'order_detail SET '.$update.'
        					WHERE id_order_detail = '. (int) $od['id_order_detail']);
        		}
    		}
		}
		
		// Notes, specific price from cart, id_tag
		$result = $db->getRow('
			SELECT
				note, specific_price_cart, id_tag
			FROM '._DB_PREFIX_.'cart_product_kerawen
			WHERE id_cart = '.pSQL($order->id_cart).'
				AND id_product = '. (int) $od['product_id'].'
				AND id_product_attribute = '. (int) $od['product_attribute_id']);
		if ($result) {
			if ($result['note'] != '') {
				$more['note'] = pSQL($result['note']);
			}
			$more['specific_price_cart'] = $result['specific_price_cart'];

			if ($result['id_tag'] != '') {
			    $more['id_tag'] = (int) $result['id_tag'];
			}
		}

		// VAT margin module
		if (Module::isInstalled('vatmargin') && Module::isEnabled('vatmargin')) {
			$is_vat_margin = (int) $db->getValue('
				SELECT vat_margin
				FROM '._DB_PREFIX_.'product
				WHERE id_product = '. (int) $od['product_id']
			);
			if ($is_vat_margin) {
				$more['margin_vat'] = 1;
			}
		}

		if (Configuration::get('KERAWEN_NOTE_SHORT_DESC')) {
		    
		    $note =  $db->getValue('
                 SELECT
                    prod_note
                 FROM '._DB_PREFIX_.'product_wm_kerawen
                 WHERE id_product = ' . (int) $od['product_id']
		    );

		    if ($note) {
		      $more['note'] = pSQL($note, true);
		    }
		}
		
		if (count($more)) {
			$more['id_order_detail'] = $od['id_order_detail'];
			$db->insert('order_detail_kerawen', $more);
		}
	}
}

/**
 * Register ecotax for each product in order
 */
function registerEcotax($order)
{
	$db = Db::getInstance();

	$buf = $db->executeS('
		SELECT od.id_order_detail, p.ecotax AS ecotax_prod, pa.ecotax AS ecotax_attr
		FROM '._DB_PREFIX_.'order_detail od
		JOIN '._DB_PREFIX_.'product_shop p
			ON p.id_product = od.product_id AND p.id_shop = od.id_shop
		LEFT JOIN '._DB_PREFIX_.'product_attribute_shop pa 
			ON pa.id_product_attribute = od.product_attribute_id AND pa.id_shop = od.id_shop
		WHERE od.id_order = '. (int) $order->id);
}

/**
* Returns order status
*
* @param int $id_order
*/
function getCurrentOrderStatus($params)
{
	$id_order = $params->id_order;

	$order = new Order($id_order);
	return $order->getCurrentState();
}

/**
* Returns a list of existing order status
*
* @param int $id_order
*/
function getExistingOrderStatus($id_lang)
{
	static $order_status = array();

	if (! isset($order_status[$id_lang]))
	{

		$order_states = OrderState::getOrderStates($id_lang);
		$order_status[$id_lang] = array();
		foreach ($order_states as $os)
		{
			$order_status[$id_lang][$os['id_order_state']] = array(
				'id' => $os['id_order_state'],
				'text' => $os['name'],
				'color' => $os['color'],
			    'pay' => $os['id_order_state'] == Configuration::get('PS_OS_DELIVERED'),
			    'cancel' => $os['id_order_state'] == Configuration::get('PS_OS_CANCELED'),
			);
		}
	}

	return $order_status[$id_lang];
}

/**
* Returns the textual value of an order status id
*
* @param type $id_status
* @return string
*/
function getTextualStatus($id_lang, $id_status)
{
	$status_list = getExistingOrderStatus($id_lang);
	if (isset($status_list[$id_status]))
		return $status_list[$id_status]['text'];

	return '';
}

/**
* Returns the color value of an order status id
*
* @param type $id_status
* @return string
*/
function getColorStatus($id_lang, $id_status)
{
	$status_list = getExistingOrderStatus($id_lang);
	if (isset($status_list[$id_status]))
		return $status_list[$id_status]['color'];

	return '';
}


function getPayStatus($id_lang, $id_status)
{
    $status_list = getExistingOrderStatus($id_lang);
    if (isset($status_list[$id_status]))
        return $status_list[$id_status]['pay'];
        
    return false;
}

function getCancelStatus($id_lang, $id_status)
{
    $status_list = getExistingOrderStatus($id_lang);
    if (isset($status_list[$id_status]))
        return $status_list[$id_status]['cancel'];
        
    return false;
}

function getTextualStatusList($id_lang, $params)
{
	$status_list = $params->status_list;

	$return_list = array();
	foreach ($status_list as $id_status)
	{
		$return_list[] = array(
			'id' => $id_status,
			'text' => getTextualStatus($id_lang, $id_status),
			'color' => getColorStatus($id_lang, $id_status)
		);
	}
	return $return_list;
}

function getPreparationStatus($id_lang, $params)
{
	$oh = getOrderHistory($id_lang, $params->id_order);
	$params->status_list = getRegisterValue('statusPreparation');
	$params->default_status = getRegisterValue('statusPreparationDefault');
	$params->order_history = $oh;
	return getFirstStatusHistory($id_lang, $params);
}

function getPaymentStatus($paid, $due, $mode, $id_order)
{
    $db = Db::getInstance();
    require_once(_KERAWEN_CLASS_DIR_.'/KerawenPayment.php');
    
    $paid = round($paid, 2);
    $due = round($due, 2);
    $mode = (int) $mode;
    
    $kmp = KerawenPayment::getModesPayments();
    $kmp_active = $kmp[0];
    if (count($kmp) > 1) {
        $id_till = (int) $db->getValue('SELECT id_till FROM `'._DB_PREFIX_.'order_kerawen` WHERE id_order = '. (int) $id_order);
        if (isset($kmp[$id_till])) {
            if (isset($kmp[$id_till][$mode])) {
                $kmp_active = $kmp[$id_till];
            }
        }
    }
    $os = 0;
    if (isset($kmp_active[$mode])) {
        if ($paid == $due) $os = $kmp_active[$mode]['FP'];
        elseif ($paid == 0.0)  $os = $kmp_active[$mode]['NP'];
        elseif ($paid < $due)  $os = $kmp_active[$mode]['PP'];
        elseif ($paid > $due)  $os = $kmp_active[$mode]['TP'];
    }
    
    return $os;
}


function setPaymentLabel(&$order, $payments) {

    /*
    echo '<pre>';
    var_dump($payments);
    echo '</pre>';
    */
   
    if (Configuration::get('KERAWEN_PAYMENT_MODE') && $order) {
        $context = Context::getContext();
        $db = Db::getInstance();
        
        //getPayment History
        $all_payments = array();
        $history_payments = $db->executeS('
    		      SELECT DISTINCT
    			     op.payment_method AS label,
    			     SUM(op.amount) AS paid
    		      FROM '._DB_PREFIX_.'order_payment op
    		      WHERE op.order_reference = "'.pSQL($order->reference).'"
                  GROUP BY label');
        
        foreach ($history_payments as $payment) {
            if (!isset($all_payments[$payment['label']])) {
                $all_payments[$payment['label']] = 0;
            }
            $all_payments[$payment['label']] += $payment['paid'];
        }
        
        
        foreach($payments as $payment) {
            $is_mode = $payment->mode;
            
            //Remove condition below if "Exchange product" is not a payment mode
            if (isset($payment->return)) {
                $is_mode = gettype($payment->return) == 'boolean';
            }
            
            if ($is_mode) {
                if (!isset($all_payments[$payment->label])) {
                    $all_payments[$payment->label] = 0;
                }
                $all_payments[$payment->label] += $payment->paid;
            }
        }
    
        
        foreach($all_payments as $key=>$value) {
            if ($value == 0) {
                unset($all_payments[$key]);
            }
        }
 
        $nbPayment = count($all_payments);
        if ($nbPayment == 0) {
            $paymentLabel =  $context->module->l('None', pathinfo(__FILE__, PATHINFO_FILENAME));
        } else if ($nbPayment == 1) {
            foreach($all_payments as $key=>$value) {
                $paymentLabel = $key;
            }
        } else {
            $paymentLabel = $context->module->l('Multi', pathinfo(__FILE__, PATHINFO_FILENAME));
            //TODO: split items ?
        }
        
        $paymentLabel = ($paymentLabel == '') ? $context->module->l('Undefined', pathinfo(__FILE__, PATHINFO_FILENAME)) : $paymentLabel;
        
        $order->payment = $paymentLabel;
        $order->save();
    
    }
 
}


/**
*
* @param Context $context
* @param stdClass $params
*/
function getOrderHistory($id_lang, $id_order)
{
	$o = new Order($id_order);
	return $o->getHistory($id_lang, false, true /* No cache*/);
}

/**
 * Update order status in kerawen table
 *
 * @param int $id_order
 * @param int $preparation_status
 */
function updateOrderStatus($id_order, $preparation_status = null)
{
	$db = Db::getInstance();
	if ($preparation_status)
	{
		$db->execute('INSERT INTO `'._DB_PREFIX_.'order_kerawen`
					(`id_order`, `preparation_status`)
				VALUES ('. (int) $id_order .', '. (int) $preparation_status.')
				ON DUPLICATE KEY UPDATE
					`preparation_status` = VALUES(`preparation_status`);');

		$date = date('Y-m-d H:i:s');
		if ($preparation_status != Configuration::get('PS_OS_DELIVERED'))
			$date = $db->getValue('SELECT delivery_date
				FROM `'._DB_PREFIX_.'order_kerawen`
				WHERE id_order = '. (int) $id_order);
		$db->update('order_kerawen', array(
			'display_date' => pSQL($date),
		), 'id_order = '. (int) $id_order);
	}
}

function isPreparationStatus($status_id)
{
	$status = getRegisterValue('statusPreparation');
	return in_array($status_id, $status);
}

/**
* internal
*/
function getFirstStatusHistory($id_lang, $params)
{
	$status_mapping = $params->status_list;
	$oh = $params->order_history;
	$default_status = $params->default_status;

	foreach ($oh as &$o)
	{
		if (in_array($o['id_order_state'], $status_mapping))
		{
			//$color = $o['color'];
			// Compatibility with PrestaShop 1.5
			$os = new OrderState($o['id_order_state']);
			$color = $os->color;

			return array(
				'id' => $o['id_order_state'],
				'text' => $o['ostate_name'],
				'color' => $color,
			    'pay' => $o['id_order_state'] == Configuration::get('PS_OS_DELIVERED'),
			    'cancel' => $o['id_order_state'] == Configuration::get('PS_OS_CANCELED'),
			);
		}
	}
	unset($o);
	$params->id_status = $default_status;
	return array(
		'id' => $default_status,
		'text' => getTextualStatus($id_lang, $default_status),
		'color' => getColorStatus($id_lang, $default_status),
	    'pay' => $default_status == Configuration::get('PS_OS_DELIVERED'),
	    'cancel' => $default_status == Configuration::get('PS_OS_CANCELED'),
	);
}
function getRegisterValue($key)
{
	static $mapping = null;

	if (null === $mapping)
	{
		// EVOL mapping in config file or parameters
		$mapping = array(
			'statusPreparation' => array(
				Configuration::get('KERAWEN_OS_RECEIVED'),
				Configuration::get('PS_OS_ERROR'),
				Configuration::get('PS_OS_OUTOFSTOCK'),
				Configuration::get('PS_OS_PREPARATION'),
				Configuration::get('KERAWEN_OS_READY'),
				Configuration::get('PS_OS_SHIPPING'),
				Configuration::get('PS_OS_DELIVERED'),
				Configuration::get('PS_OS_CANCELED'),
			),
			'statusPreparationDefault' => Configuration::get('KERAWEN_OS_RECEIVED'),

			'statusPayment' => array(
				Configuration::get('KERAWEN_OS_NOT_PAID'),
				Configuration::get('KERAWEN_OS_PAID_PARTIALLY'),
				Configuration::get('PS_OS_CHEQUE'),
				Configuration::get('PS_OS_BANKWIRE'),
				Configuration::get('PS_OS_PAYPAL'),
				Configuration::get('PS_OS_WS_PAYMENT'),
				Configuration::get('PS_OS_PAYMENT'),
				Configuration::get('PS_OS_REFUND'),
			),
			'statusPaymentDefault' => Configuration::get('KERAWEN_OS_NOT_PAID'),
		);
	}
	return $mapping[$key];
}

/**
*
* @param Context $context
* @param stdClass $params
* @return \OrderBean
*/
function getOrder($context, $params)
{
	require_once(dirname(__FILE__).'/order_state.php');
	require_once(dirname(__FILE__).'/cartrules.php');
	require_once(dirname(__FILE__).'/shop.php');
	require_once(dirname(__FILE__).'/catalog.php');
	require_once(dirname(__FILE__).'/customer.php');
	
	$id_order = $params->id_order;
	$id_lang = $context->language->id;
	$db = Db::getInstance();

	$order = new Order($id_order);
	// Customer
	$cust = null;
	if ($order->id_customer && ($order->id_customer != Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER'))) {
		require_once(_KERAWEN_CLASS_DIR_.'/customer.php');
		$cust = getCustomer($order->id_customer, $id_lang, '');
	}

	// Products
	$prods = array();
	$nb_prod = 0;

	$pp = array();
	$products = $order->getProductsDetail();
	foreach ($products as $row) {	
		// Backward compatibility 1.4 -> 1.5
		$order->setProductPrices($row);
		$pp[(int)$row['id_order_detail']] = $row;
	}
	
	foreach ($pp as &$p) {
	    
		// Backward compatibility PrestaShop 1.5
		if (!isset($p['id_product'])) $p['id_product'] = $p['product_id'];
		if (!isset($p['id_product_attribute'])) $p['id_product_attribute'] = $p['product_attribute_id'];
		
		$img = null;
		if ($p['id_product']) {
			// Additional data for image
			$link_rewrite = $db->getValue('
				SELECT link_rewrite FROM `'._DB_PREFIX_.'product_lang`
				WHERE id_product = '. (int) $p['id_product'].'
				AND id_lang = '. (int) $id_lang);
			$img = getProductImage($p['id_product'], $p['id_product_attribute'], $link_rewrite);
		}

		$id_order_detail = $p['id_order_detail'];
		$returned_quantity = $db->getValue('
			SELECT SUM(`quantity`) FROM `'._DB_PREFIX_.'return_kerawen`
			WHERE `id_order_detail` = '. (int) $id_order_detail.' AND `took_effect` = 0');


		//PS 1.7.x + 8.x
		if (method_exists('StockAvailable','getLocation')) {
		    $stock_location = StockAvailable::getLocation($p['id_product'], $p['id_product_attribute'], $order->id_shop);
		} else {
		    //PS > 1.6.x
		    if ((int) $p['id_product_attribute']) {
		        $combination = new Combination($p['id_product_attribute']);
		        $stock_location = isset($combination->location) ? $combination->location : '';
		    } else {
		        $prod = new Product($p['id_product'], false);
		        $stock_location = isset($prod->location) ? $prod->location : '';
		    }
		}
		
		
		$ps_qties = $db->getRow('
			SELECT `product_quantity_return`, `product_quantity_refunded`
			FROM `'._DB_PREFIX_.'order_detail` WHERE `id_order_detail` = '. (int) $id_order_detail);
		$returned_quantity += $ps_qties['product_quantity_return'] + $ps_qties['product_quantity_refunded'];

		$more = $db->getRow('
			SELECT *
			FROM '._DB_PREFIX_.'order_detail_kerawen
			WHERE id_order_detail = '. (int) $id_order_detail);

		$reduc_amount = (float) $p['reduction_amount'];		
		$reduc_percent = 1 - ( (float) $p['reduction_percent'] / 100);

		$sale_detail = $db->getRow('
            SELECT
                item_name, 
                discount_rate
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
		    WHERE id_order_detail = '. (int) $p['id_order_detail']
		);
		if ($sale_detail) {
		    $short_name = $sale_detail['item_name'];
		    $discount_rate = (float) $sale_detail['discount_rate'];
		} else {
		    $short_name = false;
		    $discount_rate = 0;
		}
		
		$prod = array(
			'id' => $id_order_detail,
			'prod' => $p['id_product'],
			'attr' => $p['id_product_attribute'],
			'name' => $p['product_name'],
		    'short_name' => $short_name ? $short_name : $p['product_name'],
		    'disc_rate' => $discount_rate,
			'ref' => $p['product_reference'],
			'location' => $stock_location,
			'qty' => (int)$p['product_quantity'],
			'ret' => $returned_quantity,
			'qord' => isset($more['quantity_ordered']) ? (int) $more['quantity_ordered'] : null,
			'mord' => isset($more['measure_ordered']) ? (float) $more['measure_ordered'] : null,
			'prepared' => isset($more['prepared']) ? (boolean) $more['prepared'] : false,
			'ecotax' => $p['ecotax'] * (1 + $p['ecotax_tax_rate'] / 100),
			'unit' => (float) $p['product_price_wt'],
			'total' => (float) $p['total_wt'],
			'tax_rate' => (float) $p['tax_rate'],
			'img' => $img,
			'reduction_percent' => (float) $p['reduction_percent'],
			'reduction_amount' => (float) $p['reduction_amount'],
			'unit_init' => (float) ($reduc_percent == 0) ? 0 : $p['product_price_wt'] / $reduc_percent + $reduc_amount,
			'total_init' => (float) ($reduc_percent == 0) ? 0 : $p['total_wt'] / $reduc_percent + ( $reduc_amount * (int) $p['product_quantity'] ),
		    'id_tag' => isset($more['id_tag']) ? $more['id_tag'] : null,
		);
		
		if ($more) {
			$prod['note'] = $more['note'];
			
			if (isset($more['measure'])) {
				$prod['wm'] = array(
					'measure' => (float)$more['measure'],
					'precision' => (int)$more['precision'],
					'unit' => $more['unit'],
					'price' => (float)$more['unit_price_tax_incl'],
				);
			}
		}
		
		$prods[] = $prod;
		$nb_prod += $p['product_quantity'];
	}

	// Vouchers
	$vouchers = array();
	$cart_rules = $order->getCartRules();
	foreach ($cart_rules as &$r)
	{
		$voucher = array(
			'id_reduc' => $r['id_cart_rule'],
			'name' => $r['name'],
			'reduc' => -(float)$r['value'],
		);
		$vouchers[] = $voucher;
	}

	// Payments
	$payments = $db->executeS('
		SELECT DISTINCT
			op.id_order_payment AS id,
			op.payment_method AS mode,
			op.amount AS amount,
			op.date_add AS date,
			cf.date_deferred AS deferred,
			cf.id_payment_mode AS id_mode,
            opk.id_receipt AS id_receipt
		FROM '._DB_PREFIX_.'order_payment op
		LEFT JOIN '._DB_PREFIX_.'order_payment_kerawen opk ON opk.id_order_payment = op.id_order_payment
		LEFT JOIN '._DB_PREFIX_.'cashdrawer_flow_kerawen cf ON cf.id_order_payment = op.id_order_payment
		WHERE op.order_reference = "'.pSQL($order->reference).'"
			OR opk.reference = "'.pSQL($order->reference).'"
		ORDER BY op.id_order_payment ASC');

	$payment_mode = $order->payment;
	if ($payment_mode == $context->module->displayName && !count($payments))
		$payment_mode = $context->module->l('Deferred payment', pathinfo(__FILE__, PATHINFO_FILENAME));
	
	// Associated orders
	$default_status = Configuration::get('KERAWEN_OS_RECEIVED');
	$link_orders = $db->executeS('
		SELECT
			o.id_order AS id,
			o.total_paid_tax_incl AS total,
			IFNULL(ok.preparation_status, '.pSQL($default_status).') AS status
		FROM '._DB_PREFIX_.'orders o
		LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
		WHERE o.reference = "'.pSQL($order->reference).'"');
	if ($link_orders) {
    	foreach ($link_orders as &$o) {
    		$o['canceled'] = !isOrderStateValid($o['status']);
    	}
    	$link_orders = indexArray($link_orders, 'id');
	}
	
	// Status
	$preparation_status = getPreparationStatus($context->language->id, $params);
	$s = array(
		'id' => $preparation_status['id'],
		'text' => $preparation_status['text'],
		'color' => $preparation_status['color'],
	    'pay' => $preparation_status['pay'],
	    'cancel' => $preparation_status['cancel'],
	);
	
	$canceled = !isOrderStateValid($preparation_status['id']);
	// TODO on application side + order states mapping
	$action = getOrderAction($preparation_status['id']);
	
	$params->status_list = getRegisterValue('statusPreparation');
	$statuses = array();
	foreach (getTextualStatusList($context->language->id, $params) as $status)
	{
		$res_date = $db->getRow('
			SELECT oh.`date_add`
			FROM `'._DB_PREFIX_.'order_history` oh
			WHERE oh.`id_order` = '.(int)$id_order.' AND oh.`id_order_state` = '.(int)$status['id'].'
			ORDER BY oh.`date_add` DESC, oh.`id_order_history` DESC');
		
		$status_date = $res_date ? $res_date['date_add'] : null;
		if (!$status_date && (
				$status['id'] == $preparation_status['id'] ||
				$status['id'] == Configuration::get('KERAWEN_OS_RECEIVED')))
			$status_date = $order->date_add;

		$statuses[] = array(
			'id' => $status['id'],
			'text' => $status['text'],
			'color' => $status['color'],
			'date' => $status_date,
		);
	}


	$statusHistory = array();
	$history = array_reverse($order->getHistory($context->language->id));
	foreach ($history as $status) {
	    $statusHistory[] = array(
	        'color' => $status['color'],
	        'date' => $status['date_add'],
	        'text' => $status['ostate_name'],
	        //'id_order_history' => (int) $status['id_order_history'],
	        'id' => (int) $status['id_order_state'],
	    );
	}
	
	
	// Extended data
	$extended = $db->getRow('
		SELECT * FROM `'._DB_PREFIX_.'order_kerawen`
		WHERE `id_order` = '.$order->id);
	
	// Employee
	$id_employee = $extended ? $extended['id_employee'] : 0;
	
	// Delivery
	$delivery_mode = $extended ? $extended['delivery_mode'] : _KERAWEN_DM_DELIVERY_;
	
	$delivery_address = null;
	if ($delivery_mode == _KERAWEN_DM_DELIVERY_)
	{
		$delivery_address = $db->getRow('
			SELECT
				a.firstname AS firstname,
				a.lastname AS lastname,
				a.company AS company,
				a.vat_number AS vat_number,
				a.address1 AS address1,
				a.address2 AS address2,
				a.postcode AS postcode,
				a.city AS city,
				cl.name AS country,
				a.phone AS phone,
				a.phone_mobile AS mobile
			FROM '._DB_PREFIX_.'address a
			LEFT JOIN '._DB_PREFIX_.'country_lang cl
				ON cl.id_country = a.id_country AND cl.id_lang = '. (int) $id_lang.'
			WHERE a.id_address = '. (int) $order->id_address_delivery);
	}
	
	// Shop
	$shop = new Shop($order->id_shop);

	// Returns
	$returns = array();
	$rr = $db->executeS('
		SELECT s.id_order_slip
        FROM '._DB_PREFIX_.'order_slip s
        JOIN '._DB_PREFIX_.'orders o ON o.id_order = s.id_order
		WHERE o.reference = "'.pSQL($order->reference).'"');
	if ($rr) {
    	// Backward compatibility
    	$return_price = Tools::version_compare(_PS_VERSION_, '1.6.0.9', '<=') ? 'amount' : 'total_price';
    	
    	foreach ($rr as $r) {
    		$slip = new OrderSlip($r['id_order_slip']);
    
    		// Backward compatibility
    		if (Tools::version_compare(_PS_VERSION_, '1.6.1.0', '<=')) {
    			$total_te = $slip->shipping_cost_amount/(1 + $order->carrier_tax_rate/100);
    			$total_ti = $slip->shipping_cost_amount;
    			$shipping_ti = $slip->shipping_cost_amount;
    		}
    		else {
    			$total_te = $slip->total_shipping_tax_excl;
    			$total_ti = $slip->total_shipping_tax_incl;
    			$shipping_ti = $slip->total_shipping_tax_incl;
    		}
    		
    		$ret_prods = $db->executeS(
    			'SELECT
    				od.product_id AS id_prod,
    				od.product_attribute_id AS id_attr,
    				od.product_name AS name,
    				od.product_reference AS ref,
    				osd.product_quantity AS qty,
    				-osd.'.$return_price.'_tax_incl AS total,
    				-osd.'.$return_price.'_tax_excl AS total_te
    			FROM '._DB_PREFIX_.'order_slip_detail osd
    			JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = osd.id_order_detail
    			WHERE osd.id_order_slip = '. (int) $slip->id);
    		
    		foreach ($ret_prods as &$rp) {
    		    $link_rewrite = $db->getValue('
    				SELECT link_rewrite FROM `'._DB_PREFIX_.'product_lang`
    				WHERE id_product = '. (int) $rp['id_prod'].'
    				AND id_lang = '. (int) $id_lang);
    		    $rp['img'] = getProductImage($rp['id_prod'], $rp['id_attr'], $link_rewrite);
    			$total_te -= $rp['total_te'];
    			$total_ti -= $rp['total'];
    		}
    
    		// Backoffice links
    		$url_invoice = getAdminLink('AdminPdf', array(
    			'submitAction' => 'generateOrderSlipPDF',
    			'id_order_slip' => $slip->id,
    		));
    
    		$return = array(
    		    'id' => $slip->id,
    		    'id_order' => $slip->id_order,
    		    'date' => $slip->date_add,
    		    'total' => -$total_ti,
    		    'tax' => -($total_ti - $total_te),
    		    'url_invoice' => $url_invoice,
    		    'products' => $ret_prods,
    		    'shipping' => -$shipping_ti,
    		);
    		$returns[$slip->id] = $return;
    	}
	}
		
	// Wrapping
	$wrapping = null;
	if ($order->gift) {
		$wrapping = array(
			'price' => $order->total_wrapping_tax_incl,
		);
	}
	
	// Shipping
	$shipping = null;
	$ship_taxes = array();
	if ($order->id_carrier)
	{
		$carrier = new Carrier($order->id_carrier);
		
		$refunded = $db->getValue('
			SELECT SUM(refund_tax_incl) FROM '._DB_PREFIX_.'return_kerawen
			WHERE id_order = '. (int) $order->id .' AND id_order_detail = 0');
		if (!$refunded) $refunded = 0;
		foreach ($returns as $r)
			$refunded += -$r['shipping'];

		$shipping = array(
			'carrier' => $carrier->name,
			'price' => $order->total_shipping_tax_incl,
			'refunded' => $refunded,
		);

		$ctx = new Context();
		$ctx->shop = $shop;
		$calc = $carrier->getTaxCalculator(new Address($order->{Configuration::get('PS_TAX_ADDRESS_TYPE')}));
		$ship_taxes = $calc->getTaxesAmount($order->total_shipping_tax_excl);
	}
	
    $url_bo = getAdminLink('AdminOrders', array(
        'vieworder' => true,
        'id_order' => $order->id,
    ));
	
	$url_invoice = getAdminLink('AdminPdf', array(
		'submitAction' => 'generateInvoicePDF',
		'id_order' => $order->id,
		'id_lang' => $order->id_lang,
	));

	$url_delivery = getAdminLink('AdminPdf', array(
	    'submitAction' => 'generateDeliverySlipPDF',
	    'id_order' => $order->id,
	    'id_lang' => $order->id_lang,
	));
	

	$url_purchase = getAdminLink('KerawenPurchaseOrder', array(
	    'submitAction' => 'generatePurchaseOrder',
	    'id_order' => $order->id,
	    'id_lang' => $order->id_lang,
	    'act' => 'download'
	));
	
	// Taxes detail
	$taxes = array();
	$buf = $db->executeS('
		SELECT
			odt.`id_tax` AS id,
			SUM(odt.`total_amount`) AS tax,
			SUM(od.`total_price_tax_excl`*(1 - IFNULL(ok.product_global_discount,0))) AS base
		FROM '._DB_PREFIX_.'order_detail_tax odt
		JOIN '._DB_PREFIX_.'order_detail od
			ON odt.id_order_detail = od.id_order_detail
		JOIN '._DB_PREFIX_.'order_kerawen ok
			ON ok.id_order = od.id_order
		WHERE od.id_order = '. (int) $order->id.'
		GROUP BY odt.`id_tax`');
	foreach ($buf as $t) $taxes[$t['id']] = $t;

	foreach ($ship_taxes as $id_tax => $amount)
	{
		if (!isset($taxes[$id_tax])) $taxes[$id_tax] = array(
			'id' => $id_tax,
			'tax' => 0,
			'base' => 0,
		);
		$taxes[$id_tax]['tax'] += $amount;
		$taxes[$id_tax]['base'] += $order->total_shipping_tax_excl;
	}
	
	
    $threads = $db->executeS('
        SELECT
            ct.*,
            ct.id_customer_thread AS id,
            IFNULL(ctk.sav, 0) AS sav, 
            IFNULL(ctk.id_thread_state, 0) AS id_thread_state,
            ctsk.thread_state_label AS state_label,
            ctsk.thread_state_color AS state_color,
            c.id_customer,
            c.firstname, 
            c.lastname, 
            c.email
    	FROM '._DB_PREFIX_.'customer_thread ct
        LEFT JOIN '._DB_PREFIX_.'customer_thread_kerawen ctk ON ct.id_customer_thread = ctk.id_customer_thread
    	LEFT JOIN '._DB_PREFIX_.'customer_thread_state_kerawen ctsk ON ctk.id_thread_state = ctsk.id_thread_state
        LEFT JOIN '._DB_PREFIX_.'orders o ON ct.id_order = o.id_order
        LEFT JOIN '._DB_PREFIX_.'customer c ON o.id_customer = c.id_customer
        /*
        LEFT JOIN
        (
            SELECT 
                id_customer_thread, `date_add` 
            FROM  ps_customer_message
            ORDER BY `date_add` DESC
            LIMIT 1
        ) cm ON cm.id_customer_thread = ct.id_customer_thread 
        */
        WHERE ct.id_order = '. (int) $order->id .'
        /*ORDER BY cm.`date_add` DESC*/
        ORDER BY ct.`date_upd` DESC
    ');

    foreach ($threads as &$thread) {
        $threads_msgs = $db->executeS('
            SELECT 
            	cm.*,
                cm.id_customer_message AS id,
            	IFNULL(cmk.id_thread_state, 0) AS id_thread_state,
            	ctsk.thread_state_label AS state_label,
            	ctsk.thread_state_color AS state_color 
            FROM '._DB_PREFIX_.'customer_message cm
            LEFT JOIN '._DB_PREFIX_.'customer_message_kerawen cmk ON cm.id_customer_message = cmk.id_customer_message
            LEFT JOIN '._DB_PREFIX_.'customer_thread_state_kerawen ctsk ON IFNULL(cmk.id_thread_state, 0) = ctsk.id_thread_state
            WHERE cm.id_customer_thread = ' . (int) $thread['id'] . '
            ORDER BY cm.`date_add` DESC
        ');
        $thread['messages'] = $threads_msgs;
    }

	$total_due = 0;
	$total_paid = 0;
	foreach($link_orders as $o) {
	    if ($o['id'] == $order->id) 
	        $total_due += $s['cancel'] ? 0 : $o['total'];
	    else 
	        $total_due += $o['canceled'] ? 0 : $o['total'];
	}
	foreach($returns as $r) {
	    $o = $link_orders[$r['id_order']];
	    if ($o['id'] == $order->id) 
	        $total_due += $s['cancel'] ? 0 : $r['total'];
	    else 
	        $total_due += $o['canceled'] ? 0 : $r['total'];
	}
	foreach($payments as $p) {
	    $total_paid += $p['amount'];
	}
	$remain = round(($total_due - $total_paid), 2);
	
	$shop_id = $order->id_shop;
	$store = $shop->name;
	
	$rewards = getReceiptRewards($order->reference);
	foreach($rewards as &$reward) {
	    $reward['name'] = $cust ? $cust['firstname'] . ' ' . $cust['lastname'] : '';
	}

	if (Module::isInstalled('kerawen_ocs') && Module::isEnabled('kerawen_ocs')) {
	    $ocs_shop_id = $db->getValue('SELECT id_shop_pickupatstore FROM '._DB_PREFIX_.'kerawen_cnc_pickupatstore_orders WHERE id_order = ' . (int) $order->id);
	    if ($ocs_shop_id) {
	        $delivery_mode = 1;
	        $ocs_shop = new Shop($ocs_shop_id);
	        $store = '(@) ' . $ocs_shop->name;
	    }
	}

	// Gather
	$ob = array(
		'id' => $order->id,
	    'id_shop' => $shop_id,
		'id_cart' => $order->id_cart,
		'language' => $order->id_lang,
		'id_empl' => $id_employee,
		'ref' => $order->reference,
	    'shipping_num' => Tools::version_compare(_PS_VERSION_, '8', '<') ? $order->shipping_number : $order->getShippingNumber(),
		'delivery_date' => ($order->delivery_date == '0000-00-00 00:00:00') ? '' : $order->delivery_date,
		'delivery_num' => $order->delivery_number,
	    'store' => $store,
		'date' => $order->date_upd,
		'mode' => $delivery_mode,
		'delivery_address' => $delivery_address,
		'dateInvoice' => $order->invoice_date,
		'dateDelivery' => $extended ? $extended['delivery_date'] : null,
	    'notePrivate' => $extended ? $extended['private_note'] : null,
	    'notePublic' => $extended ? ((((String) $extended['delivery_note'] == '') ? '' : $extended['delivery_note'] . "\n") . $extended['invoice_note']) : '',
		'dateDisplay' => $extended ? $extended['display_date'] : null,
		'cust' => $cust,
		'web' => $order->payment != $context->module->displayName,
		'prods' => $prods,
		'vouchers' => $vouchers,
		'wrapping' => $wrapping,
	    'gift' => $order->gift,
	    'recyclable' => $order->recyclable,
		'shipping' => $shipping,
		'nbProds' => $nb_prod,
		'state' => getKerawenOrderState($s['id']),
		'status' => $s,
		'canceled' => $canceled,
		'action' => $action,
		'statusAvailable' => $statuses,
	    'statusHistory' => $statusHistory,
		'total' => $order->total_paid_tax_incl,
		'base' => $order->total_paid_tax_excl,
		'tax' => $order->total_paid_tax_incl - $order->total_paid_tax_excl,
		'payment_mode' => $payment_mode,
		'payment' => $payments,
		'paid' => $order->total_paid_real,
		'is_paid' => $extended['is_paid'],
		'accounted' => $extended['accounted'],
	    'url_bo' => $url_bo,
		'url_invoice' => $url_invoice,
	    'url_delivery' => $url_delivery,
	    'url_purchase' => $url_purchase,
		'taxes' => $taxes,
	    'threads' => $threads,
		'links' => $link_orders,
		'gift_card' => getGiftCardByTicket($id_order, 0),
	    'loyalty' => getLoyaltyForTicket($cust),
		'returns' => $returns,
		'flagAddress' => ((int) $order->invoice_number > 0),
	    'flagPurchaseOrder' => ((int) $extended['purchase_number'] > 0),
	    'module' => $order->module,
	    'remain' => $remain,
	    'rewards' => $rewards,
	);
	
	return $ob;
}

function setInvoiceAddress($order, $address, $id_lang = false, $is_invoice = true)
{
    $db = Db::getInstance();
	$order->id_address_invoice = $address->id;
	if ($id_lang) { $order->id_lang = $id_lang; }
	if ($is_invoice) { 
	    $order->setInvoice(true); 
	} else {
	    $order->save();
	    $db->update('order_kerawen', array('purchase_number' => (int) $order->id), 'id_order = '. (int) $order->id);
	}
}

function adjustOrderQty($order, $id_detail, $qty)
{
	$db = Db::getInstance();

	$od = new OrderDetail($id_detail);
	$prev_te = $od->total_price_tax_excl;
	$prev_ti = $od->total_price_tax_incl;
	
	// Update order detail
	$od->product_quantity = $qty;
	$od->total_price_tax_excl = $qty * $od->unit_price_tax_excl;
	$od->total_price_tax_incl = $qty * $od->unit_price_tax_incl;
	$od->save();
	$od->updateTaxAmount($order);
	
	// Update order total
	$delta_te = $od->total_price_tax_excl - $prev_te;
	$delta_ti = $od->total_price_tax_incl - $prev_ti;
	$order->total_products += $delta_te;
	$order->total_products_wt += $delta_ti;
	$order->total_paid_tax_excl += $delta_te;
	$order->total_paid_tax_incl += $delta_ti;
	$order->total_paid = $order->total_paid_tax_incl;
	$order->save();
	
	// Mark as prepared
	$db->update('order_detail_kerawen', array(
		'prepared' => true,
	), 'id_order_detail = '.pSQL($id_detail));
}

function adjustOrderNote($params)
{
    $context = Context::getContext();
    
    if ($params->invoice_note == true) {
        setInvoiceNote((int) $params->id_order, $params->note);
    } else {
        
        $db = Db::getInstance();
        
        //product note
        if ($params->id_detail !== false) {
            $db->execute('
				INSERT INTO '._DB_PREFIX_.'order_detail_kerawen
				(id_order_detail, note) VALUES (' . (int) $params->id_detail . ', "' . pSQL($params->note) . '")
				ON DUPLICATE KEY UPDATE note = "' . pSQL($params->note) . '"
			');
        }
        
        //order note
        if ($params->id_customer_message !== false) {
            
            $ip = (int)ip2long($_SERVER['REMOTE_ADDR']);

            $ct = new CustomerThread((int) $params->id_customer_thread);
            
            if ((int) $params->id_customer_thread === 0) {
                $ct->token = Tools::passwdGen(12);
                $ct->id_shop = (int)$context->shop->id;
                $ct->id_lang = (int)$context->language->id;
                $ct->id_contact = 0; //from employee ?
                
                $customer = ((int) $params->id_customer_order > 0) ? new Customer((int) $params->id_customer_order) : getAnonymousCustomer();
                $ct->id_customer = $customer->id;
                $ct->email = $customer->email;
                $ct->id_order = (int) $params->id_order;
                
            } else {
                $customer = new Customer((int) $ct->id_customer);
            }

            if (isset($params->id_product)) {
                $ct->id_product = (int) $params->id_product;
            }
            $ct->status = $params->status;
            
            $ct->save();
            
            //ps_customer_thread_kerawen sav/id_thread_state
            $sav = (int) $params->id_thread_state ? '1' : '0';
            
            $cm = new CustomerMessage((int) $params->id_customer_message);
            $cm->id_customer_thread = (int) $ct->id;
            $cm->private = (int) $params->private;
            $cm->message = $params->note . ' '; //fixe bug empty message
            $cm->ip_address = $ip;
            $cm->id_employee = (int) $params->id_empl;
            $cm->save();
            
            $db->execute('
				INSERT INTO '._DB_PREFIX_.'customer_message_kerawen
				(id_customer_message, id_thread_state) VALUES (' . (int) $cm->id . ', ' . (int) $params->id_thread_state . ')
				ON DUPLICATE KEY UPDATE id_thread_state = ' . (int) $params->id_thread_state
			);
            
            //Update main state only -> with most recent message
            $id_thread_state = $db->getValue('
                SELECT 
                    cmk.id_thread_state
                FROM '._DB_PREFIX_.'customer_message cm
                LEFT JOIN '._DB_PREFIX_.'customer_message_kerawen cmk ON cm.id_customer_message = cmk.id_customer_message
                WHERE cm.id_customer_thread = ' . (int) $ct->id . '
                ORDER BY cm.id_customer_message DESC
            ');
            
            $db->execute('
				INSERT INTO '._DB_PREFIX_.'customer_thread_kerawen
				(id_customer_thread, sav, id_thread_state) VALUES (' . (int) $ct->id . ', ' . (int) $sav . ', ' . (int) $params->id_thread_state . ')
				ON DUPLICATE KEY UPDATE sav = ' . (int) $sav . ', id_thread_state = ' . (int) $id_thread_state
            );
            

            //TODO: mail to client
            if (!((int) $params->private)) {
                
                //TODO: if multishops with only one shop visible ?
                
                $contact = new Contact((int)$ct->id_contact, (int)$ct->id_lang);
                if (Validate::isLoadedObject($contact)) {
                    $from_name = $contact->name;
                    $from_email = $contact->email;
                } else {
                    $from_name = null;
                    $from_email = null;
                }
                
                $link = new Link;
                $params = array(
                    '{reply}' => Tools::nl2br($params->note),
                    '{link}' => Tools::url($link->getPageLink('contact', true, (int)$ct->id_lang, null, false, $ct->id_shop), 'id_customer_thread='.(int)$ct->id.'&token='.$ct->token),
                    '{firstname}' => $customer->firstname,
                    '{lastname}' => $customer->lastname
                );

                Mail::Send(
                    (int) $ct->id_lang,
                    'reply_msg',
                    sprintf($context->module->l('An answer to your message is available #ct%1$s #tc%2$s', pathinfo(__FILE__, PATHINFO_FILENAME)), $ct->id, $ct->token),
                    $params,
                    $customer->email,
                    null,
                    $from_email,
                    $from_name,
                    null,
                    null,
                    _PS_MAIL_DIR_,
                    false,
                    $ct->id_shop
                );
                
            }
            
        }
        
    }
    
}



function adjustOrderTag($params)
{
    $db = Db::getInstance();
    if ($params->id_detail !== false) {
        $db->execute('
			INSERT INTO '._DB_PREFIX_.'order_detail_kerawen
			(id_order_detail, id_tag) VALUES (' . (int) $params->id_detail . ', "' . pSQL($params->id_tag) . '")
			ON DUPLICATE KEY UPDATE id_tag = "' . pSQL($params->id_tag) . '"
		');
    }
}


function setInvoiceNote($id_order, $note)
{
	$db = Db::getInstance();

	$db->execute('
		INSERT INTO '._DB_PREFIX_.'order_kerawen
		(id_order, invoice_note) VALUES (' . (int) $id_order . ', "' . pSQL($note) . '")
		ON DUPLICATE KEY UPDATE invoice_note = "' . pSQL($note) . '"
	');	
}


//first time only
function addOrderNote($id_order, $note, $id_employee, $id_customer)
{
	$db = Db::getInstance();
	
	$ct = getCustomerThread($id_order, $id_customer);
		
		//Private message already exists ?
	$id_customer_message = (int)Db::getInstance()->getValue('
		SELECT cm.`id_customer_message` AS id_customer_message
		FROM `'._DB_PREFIX_.'customer_message` cm
		LEFT JOIN `'._DB_PREFIX_.'customer_thread` ct
		ON ct.`id_customer_thread` = cm.`id_customer_thread`
		WHERE ct.id_order = ' . (int) $id_order . ' AND cm.`private` = 1
		GROUP BY cm.`id_customer_thread`, cm.`id_customer_message`
		ORDER BY cm.`id_customer_message` DESC
	');
		
	if ($id_customer_message) {
	
		$cm = new CustomerMessage($id_customer_message);
		//fixe bug empty message
		$cm->message = $note . ' ';
		$cm->ip_address = (int)ip2long($_SERVER['REMOTE_ADDR']);
		$cm->id_employee = $id_employee;
		$cm->update();
	
	} else {
	
		$cm = new CustomerMessage();
		$cm->id_customer_thread = $ct->id;
		//fixe bug empty message
		$cm->message = $note . ' ';
		$cm->ip_address = (int)ip2long($_SERVER['REMOTE_ADDR']);
		$cm->id_employee = $id_employee;
		$cm->private = 1;
		$cm->add();
			
	}
		
}


function getCustomerThread($id_order, $id_customer) {

	$db = Db::getInstance();
	$context = Context::getContext();
	
	if ($id_customer === false) {
		$customer = getAnonymousCustomer();
		$id_customer = $customer->id;
	} else {
		$customer = new Customer($id_customer);
	}
		

	//thread
	$id_customer_thread = CustomerThread::getIdCustomerThreadByEmailAndIdOrder($customer->email, $id_order);
	
	if (!$id_customer_thread) {
	
		$ct = new CustomerThread();
		$ct->id_contact = 0;
		$ct->id_customer = (int)$customer->id;
		$ct->id_shop = (int)$context->shop->id;
	
		$ct->id_order = (int)$id_order;
		$ct->id_lang = (int)$context->language->id;
		$ct->email = $customer->email;
		$ct->status = 'closed';
		$ct->token = Tools::passwdGen(12);
		$ct->add();
	
	} else {
	
		$ct = new CustomerThread((int)$id_customer_thread);
		$ct->status = 'closed';
		$ct->update();
	
	}
	
	return $ct;
}


