<?php
/**
* 2014 KerAwen
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2014 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/

require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');

function getConfig($context, $params)
{	
	require_once (_KERAWEN_CLASS_DIR_.'/stats.php');
	require_once (_KERAWEN_CLASS_DIR_.'/data.php');
	require_once (_KERAWEN_CLASS_DIR_.'/log.php');
	
	$id_lang = $context->language->id;
	$db = Db::getInstance();

	// Get employees
	$employees = array();
	$active_employee = array();
	$token = array();

	$langkey = pathinfo(__FILE__, PATHINFO_FILENAME);	

	$virtual_shop = 0;
	if (Module::isInstalled('kerawen_ocs') && Module::isEnabled('kerawen_ocs')) {
	    $virtual_shop = (int) $db->getValue('SELECT id_shop FROM '._DB_PREFIX_.'kerawen_cnc_shop_type WHERE type = "virtual"');
	}

	$buf = Employee::getEmployees();
	foreach ($buf as $e)
	{
	    if (getEmployeeTillActive($e['id_employee'])) {
	    
    		$employee = new Employee($e['id_employee']);
    		$employees[$employee->id] = array(
    			'id' => $e['id_employee'],
    			'name' => $e['firstname'].' '.$e['lastname'],
    			'firstname' => $e['firstname'],
    			'lastname' => $e['lastname'],
    			'shops' => getEmployeeShops($employee),
    			'tills' => getEmployeeTills($employee),
    			'id_profile' => $employee->id_profile,
    			'passwordstatus' => getEmployeePasswordStatus($e['id_employee']),
    		);
	    }
	    
    	//Active employee
    	if ($context->employee->id == $e['id_employee']) {
    		$active_employee = getEmployee($employee->id);
    	}

	}

	//Get profiles & permissions
	$profiles = array();
	$profilesclasses = array();
	$tabs = $context->module->getTabsDescription();
	$buf = Profile::getProfiles($id_lang);
	foreach ($buf as $p) {
		$id_profile = $p['id_profile'];
		
		$profiles[$id_profile] = array(
			'id' => $id_profile,
			'name' => $p['name'],
		);
		
		$profilesclasses[$id_profile] = array();
		foreach ($tabs as $app => $t) {
			$id_tab = Tab::getIdFromClassName($t['class']);
			$access = Profile::getProfileAccess($id_profile, $id_tab);
			$profilesclasses[$id_profile][$app] = $access['edit'];
		}
	}


	// Get shops
	// do not use Shop::getShops(); for it returns only the shops allowed to the employee
	$ss = $db->executeS('
		SELECT s.name, s.id_shop
		FROM '._DB_PREFIX_.'shop s
        WHERE s.deleted = 0 AND s.active = 1
        ORDER BY s.id_shop ASC');
	
	$shops = array();
	foreach ($ss as &$s)
	{
		$id_shop = $s['id_shop'];
		$shop = new Shop($id_shop);
		$shops[$id_shop] = array(
			'id' => $id_shop,
			'name' => $s['name'],
		    'label' => Configuration::get('PS_SHOP_NAME', null, null, $id_shop) ? Configuration::get('PS_SHOP_NAME', null, null, $id_shop) : $s['name'],
			'addr1' => Configuration::get('PS_SHOP_ADDR1', null, null, $id_shop),
			'addr2' => Configuration::get('PS_SHOP_ADDR2', null, null, $id_shop),
			'postcode' => Configuration::get('PS_SHOP_CODE', null, null, $id_shop),
			'city' => Configuration::get('PS_SHOP_CITY', null, null, $id_shop),
			'country' => Configuration::get('PS_SHOP_COUNTRY', null, null, $id_shop),
			'country_id' => Configuration::get('PS_SHOP_COUNTRY_ID', null, null, $id_shop),
			'phone' => Configuration::get('PS_SHOP_PHONE', null, null, $id_shop),
			'url' => $shop->domain.$shop->physical_uri.$shop->virtual_uri,
			'details' => Configuration::get('PS_SHOP_DETAILS', null, null, $id_shop),
			'employees' => array(),
			'warehouses' => Warehouse::getWarehouses(false, $id_shop),
		);
		foreach ($employees as $e)
		{
			if (in_array($id_shop, $e['shops']))
				$shops[$id_shop]['employees'][] = $e['id'];
		}
	}

	require_once(dirname(__FILE__).'/drawer.php');
	$tills = indexArray(getTills(), 'id');
	

	// Get tax rules
	require_once(_KERAWEN_CLASS_DIR_.'/KerawenTax.php');
	$taxrules = array();
	$rules = TaxRulesGroup::getTaxRulesGroupsForOptions();
	$rates = TaxRulesGroup::getAssociatedTaxRatesByIdCountry(Configuration::get('PS_COUNTRY_DEFAULT'));
	foreach ($rules as $r)
	{
		$id = $r['id_tax_rules_group'];
		$rate = isset($rates[$id]) ? $rates[$id]: 0.0;
		$taxrules[$id] = array(
			'id' => (string)$id,
			'name' => $r['name'],
			'rate' => $rate,
		);
	}

	$suppliers = array();
	$sups = Supplier::getSuppliers(false, 0, false);
		
	foreach ($sups as $sup)
	{
	    $suppliers[] = array(
	        'id' => (int)$sup['id_supplier'],
	        'name' => $sup['name'],
	    );
	}
	$columns = array_column($suppliers, 'name');
	array_multisort($columns, SORT_ASC, $suppliers);
	
	$manufacturers = getKerawenManufacturers();
	$columns = array_column($manufacturers, 'name');
	array_multisort($columns, SORT_ASC, $manufacturers);
	
	// Get order states
	$order_states = array();
	$states = OrderState::getOrderStates($id_lang);
	$status_till = json_decode(Configuration::get('KERAWEN_STATUS_TILL'), true);
	
	foreach ($states as $id_os)
	{
	    $order_states[] = array(
	        'id' => (int)$id_os['id_order_state'],
	        'name' => $id_os['name'],
	        'color' => $id_os['color'],
	        'pay' => (int) $id_os['id_order_state'] == (int) Configuration::get('PS_OS_DELIVERED'),
	        'cancel' => (int) $id_os['id_order_state'] == (int) Configuration::get('PS_OS_CANCELED'),
	        'invoice' => $id_os['invoice'],
	        'send_email' => $id_os['send_email'],
	        'selectable' => true,
	        'active' => ($status_till && isset($status_till[(int) $id_os['id_order_state']])) ? (int) $status_till[$id_os['id_order_state']]['active'] : 1,
	    );
	}
	

	// Countries
	$countries = array();
	$buf = Country::getCountries($id_lang, true);
	foreach ($buf as $c)
		$countries[$c['id_country']] = array(
			'id' => $c['id_country'],
			'name' => $c['name'],
		);
	
	// Currency
	if (Tools::version_compare(_PS_VERSION_, '1.7', '<')) {
		$curr = array(
			'pref' => $context->currency->prefix,
			'suff' => $context->currency->suffix,
		    'iso_code' => $context->currency->iso_code_num,
		);
	}
	else {
		$curr = array(
			'pref' => '',
			'suff' => ' '.$context->currency->sign,
		    'iso_code' => $context->currency->iso_code_num,
		);
	}

	//Catolog with or without vat -> get anonymous customer default group
	$anonymousCust = getAnonymousCustomer();
	$id_default_group = $anonymousCust->id_default_group;
	$catalog_price_ti = true;
	
	
	// Customer groups
	$cust_groups = array();
	$buf = Group::getGroups($id_lang);
	
	$groups_amount = unserialize(Configuration::get('KERAWEN_GROUP_AMOUNT'));
	
	foreach ($buf as $g) {
		if ($id_default_group == $g['id_group']) {
			$catalog_price_ti = !$g['price_display_method'];
		}
		
		$amount = '0';
		if (is_array($groups_amount) && isset($groups_amount[$g['id_group']])) {
		    $amount = is_numeric($groups_amount[$g['id_group']]) ? $groups_amount[$g['id_group']] : '0';
		}
		
		$cust_groups[$g['id_group']] = array(
			'id' => $g['id_group'],
			'name' => $g['name'],
			'price_ti' => (!$g['price_display_method']),
		    'amount' => (float) $amount,
		);
	}
	// Categories
	$cats = Category::getCategories( (int)($id_lang), false, false);

	$cat_groups = array();
	foreach ($cats as $cat) {
		$cat_groups[$cat['id_category']] = array(
			'id_category' => $cat['id_category'], 
			'name' => $cat['name'],
		);
	}

	//token
	$urlWithToken = (int)Configuration::get('KERAWEN_URL_WITH_TOKEN');
	$adminHome = Tools::version_compare(_PS_VERSION_, '1.6', '<') ? 'AdminHome' : 'AdminDashboard';
	foreach(array($adminHome,'AdminProducts','KerawenRegister','KerawenReport','KerawenLabel','KerawenHome', 'KerawenExport','KerawenCertif','KerawenTills','KerawenMarketing','KerawenPicking','KerawenAftersale') as $v) {
		$token[$v]['key'] = Tools::getAdminToken($v.(int)(Tab::getIdFromClassName($v)).(int)($context->employee->id));
		$token[$v]['url'] = Dispatcher::getInstance()->createUrl($v, $context->language->id, $urlWithToken ? array('token'=>$token[$v]['key']) : array(), false);
	}

	//Payments
	$payments = json_decode(Configuration::get('KERAWEN_PAYMENTS'), true);
	$payments[] = array(
		'id' => _KERAWEN_PM_GIFT_CARD_,
		'label' => $context->module->l('Gift card', $langkey),
		'payment' => 0,
		'refund' => 0,
	);

	$payments_extended = json_decode(Configuration::get('KERAWEN_PAYMENTS_EXTENDED'), true);

	/*
	$payments_used = $db->executeS('
        SELECT 
            payment_method AS `name`
        FROM '._DB_PREFIX_.'order_payment
        WHERE COALESCE(order_reference, "") != ""
        GROUP BY payment_method
        ORDER BY count(payment_method) DESC
	');
	foreach ($payments_used as $key => &$value) {
	    $value['id'] = ($key+1);
	}
	*/
	
	//id as integer required!!!
	$db->query('set @N = 0;');
	$payments_used = $db->executeS('
        SELECT
            @N := @N +1 AS id,
            payment_method AS `name`
        FROM '._DB_PREFIX_.'order_payment
        WHERE COALESCE(order_reference, "") != ""
        GROUP BY payment_method
        ORDER BY count(payment_method) DESC
	');
	
	
	// Plugins loyalty
	require_once(dirname(__FILE__).'/loyalty.php');
	$loyalty = KerawenLoyalty::getPlugin();
	$loyaltyTransform = false;
	$loyaltyCheckBox = false;
	$loyaltySponsor = false;
	if ($loyalty) {
	    $loyaltyTransform = $loyalty->doActions();
	    $loyaltyCheckBox = $loyalty->checkBox();
	    $loyaltySponsor = $loyalty->sponsor();
	}
		
	// Plugins sponsor
	require_once(dirname(__FILE__).'/sponsor.php');
	$sponsor = KerawenSponsor::getPlugin();
	$sponsorAddReferer = false;
	$sponsorAddSponsor = false;
	$sponsorActive = false;
	if ($sponsor) {
	    $sponsorAddReferer = $sponsor->addReferer();
	    $sponsorAddSponsor = $sponsor->addSponsor();
	    $sponsorActive = $sponsor->isActive();
	}
	
	//Languages
	$langs = Language::getLanguages(false, false, false);
	$languages = array();
	foreach ($langs as $lang) {
		$languages[$lang['id_lang']] = array(
			'id' => $lang['id_lang'],
			'name' => $lang['name'], 
			'active' => $lang['active'], 
			'iso_code' =>$lang['iso_code'],
			'language_code' =>$lang['language_code'],
		);
	}
	
	// Carriers
	$carriers = array();
	$buf = $db->executeS('
		SELECT
			c.id_reference AS id,
			c.name AS name,
            c.active
		FROM '._DB_PREFIX_.'carrier c
        ORDER BY c.id_carrier ASC
    ');
	// Keep name from last carrier version
	foreach ($buf as $c) {
		$carriers[$c['id']] = $c;
	}

	//delivery modes
	$delivery_modes = array(
	    array('id' => '0', 'name' => Configuration::get('KERAWEN_LABEL_IN_STORE') ? Configuration::get('KERAWEN_LABEL_IN_STORE') : $context->module->l('In store', $langkey), 'anonymous'=> (int)Configuration::get('KERAWEN_MODE_0_ANONYMOUS'), 'inputs' => null),
	    array('id' => '1', 'name' => Configuration::get('KERAWEN_LABEL_TAKEAWAY') ? Configuration::get('KERAWEN_LABEL_TAKEAWAY') : $context->module->l('Takeaway', $langkey), 'anonymous'=> (int)Configuration::get('KERAWEN_MODE_1_ANONYMOUS'), 'inputs' => 'date'),
	    array('id' => '2', 'name' => Configuration::get('KERAWEN_LABEL_DELIVERY') ? Configuration::get('KERAWEN_LABEL_DELIVERY')  : $context->module->l('Delivery', $langkey), 'anonymous'=> (int)Configuration::get('KERAWEN_MODE_2_ANONYMOUS'), 'inputs' => 'address carrier'),
	);

	//Customer account required for deliveree module
	if (Module::isInstalled('deliveree') && Module::isEnabled('deliveree')) {
	    $delivery_modes[2]['anonymous'] = 0;
	}
	
	//labels printers settings
	$log = new stdClass();
	$log->require = array('labelSettings');
	$labels = getLogMvt($log);
	$labels['barcode_type'] = Configuration::get('KERAWEN_LABEL_BARCODE_TYPE');
	$labels['barcode'] = Configuration::get('KERAWEN_LABEL_BARCODE');
	

	$tags = $db->executeS('
		SELECT 
            id_tag AS id, 
            tag_name, 
            tag_value, 
            tag_default
		FROM '._DB_PREFIX_.'tag_kerawen
    ');
	$tags = indexArray($tags, 'id');

	
	$cust_def = Customer::$definition['fields'];
	$custdef = array(
	    'firstname' => $cust_def['firstname'],
	    'lastname' => $cust_def['lastname'],
	    'email' => $cust_def['email']
	);

	$prodVisibilities = array(
	    array('id' => 'both', 'name' => 'Everywhere'),
	    array('id' => 'catalog', 'name' => 'Catalog only'),
	    array('id' => 'search', 'name' => 'Search only'),
	    array('id' => 'none', 'name' => 'Till only (Nowhere)')
	);
	
	$thread_status = array(
	    'open' => array('id' => 'open', 'name' => 'Open', 'color' => '#3c763d'),
	    'pending1' => array('id' => 'pending1', 'name' => 'Pending 1', 'color' => '#8a6d3b'),
	    'pending2' => array('id' => 'pending2', 'name' => 'Pending 2', 'color' => '#8a6d3b'),
	    'closed' => array('id' => 'closed', 'name' => 'Closed', 'color' => '#a94442'),
	);
	
	$data = array(
		'config' => (int)Configuration::get('KERAWEN_CONFIG'),
		'timezone' => (int)date('Z'),
		'curr' => $curr,
		'shops' => $shops,
		'tills' => $tills,
		'labels' => $labels,
		'employees' => $employees,
		//'taxes' => $taxes,
		'taxrules' => $taxrules,
	    'taxrule_vatmargin' => KerawenTax::GetVATMarginRule(),
	    'suppliers' => $suppliers,
	    'manufacturers' => $manufacturers,
	    'features' => getKerawenFeatures(), //required only for advanced search
	    'attributes' => getKerawenAttributes(), //required only for advanced search
		'default_taxrules' => (int)Configuration::get('KERAWEN_DEFAULT_VAT'),
	    'vat_warning' => (int)Configuration::get('KERAWEN_VAT_WARNING'),
		'countries' => $countries,
		'order_states' => $order_states,
		'groups' => $cust_groups,
		'default_group' => (int)Configuration::get('KERAWEN_DEFAULT_GROUP'),
		'id_default_group' => $id_default_group,
	    'virtual_shop' => $virtual_shop,
	    'force_virtual_shop' => (int)Configuration::get('KERAWEN_FORCE_VIRTUAL_SHOP'),
		'modules' => array(
			'loyalty' => $loyalty != null,
		    'sponsor' => $sponsor != null,
			'blockwishlist' => Module::isInstalled('blockwishlist') && Module::isEnabled('blockwishlist'),
			'favoriteproducts' => Module::isInstalled('favoriteproducts') && Module::isEnabled('favoriteproducts'),
		    'ocs' => $virtual_shop ? true : false,
		    'vatmargin' => true, //Module::isInstalled('vatmargin') && Module::isEnabled('vatmargin'),
		),
		'notif' => (int)Configuration::get('KERAWEN_NOTIF_PERIOD'),
		'actions' => array(
			'switchEmployee' => (boolean)Configuration::get('KERAWEN_SWITCH_CASHIER'),
			'switchShop' => (boolean)Configuration::get('KERAWEN_SWITCH_SHOP'),
			'discountCart' => (boolean)Configuration::get('KERAWEN_DISCOUNT_CART'),
			'selectMode' => (boolean)Configuration::get('KERAWEN_SELECT_DELIVERY'),
			'showAmounts' => (boolean)Configuration::get('KERAWEN_SHOW_AMOUNTS'),
			'quotation' => (boolean)Configuration::get('KERAWEN_QUOTE_ACTIVE'),
			'switchEmployeePassword' => (boolean)Configuration::get('KERAWEN_EMPLOYEE_PASSWORD'),
			'excpEmployeePassword' => (boolean)Configuration::get('KERAWEN_EMPLOYEE_PASSWORD_EXCP'),
		    'employeeAlwaysSelect' => (boolean)Configuration::get('KERAWEN_EMPLOYEE_ALWAYS_SELECT'),
			'loyaltyTransform' => $loyaltyTransform,
		    'loyaltyCheckBox' => $loyaltyCheckBox,
		    'loyaltySponsor' => $loyaltySponsor,
		    'sponsorAddReferer' => $sponsorAddReferer,
		    'sponsorAddSponsor' => $sponsorAddSponsor,
		    'sponsorActive' => $sponsorActive,
			'orderQuickEnd' => (boolean)Configuration::get('KERAWEN_ORDER_QUICK_END'),
			'pulse' => (boolean)Configuration::get('KERAWEN_ORDER_PULSE'),
			'play_sound' => (boolean)Configuration::get('KERAWEN_PLAY_SOUND'),
			'scan_not_found' => (boolean)Configuration::get('KERAWEN_SCAN_NOT_FOUND'),
		    'cust_print' => (boolean)Configuration::get('KERAWEN_CUST_PRINT'),
		    'cust_optin' => (boolean)Configuration::get('KERAWEN_CUST_OPTIN'),
		    'restrictive_mode' => (boolean)Configuration::get('KERAWEN_RESTRICTIVE_MODE'),
		    'alert_prod_cart' => (boolean)Configuration::get('KERAWEN_ALERT_PROD_CART'),
		    'till_shop' => (boolean)Configuration::get('KERAWEN_TILL_SHOP'),
		    'flag_msg_public' => (boolean)Configuration::get('KERAWEN_FLAG_MSG_PUBLIC'),
		    'flag_msg_private' => (boolean)Configuration::get('KERAWEN_FLAG_MSG_PRIVATE'),
		    'deposit_with_free_price' => (boolean)Configuration::get('KERAWEN_DEPOSIT_WITH_FREE_PRICE'),
		),
	    'order_picking' => array(
	        'KERAWEN_DELIVERY_DATE_ORDER' => (boolean)Configuration::get('KERAWEN_DELIVERY_DATE_ORDER'),
	        'KERAWEN_ORDER_FILTER_TILL' => (boolean)Configuration::get('KERAWEN_ORDER_FILTER_TILL'),
	        'KERAWEN_ORDER_FILTER_WEB' => (boolean)Configuration::get('KERAWEN_ORDER_FILTER_WEB'),
	        'KERAWEN_ORDER_FILTER_CHECKOUT' => (boolean)Configuration::get('KERAWEN_ORDER_FILTER_CHECKOUT'),
	        'KERAWEN_ORDER_FILTER_TAKEAWAY' => (boolean)Configuration::get('KERAWEN_ORDER_FILTER_TAKEAWAY'),
	        'KERAWEN_ORDER_FILTER_DELIVERY' => (boolean)Configuration::get('KERAWEN_ORDER_FILTER_DELIVERY'),
	        'KERAWEN_ORDER_FILTER_BOTH' => (boolean)Configuration::get('KERAWEN_ORDER_FILTER_BOTH'),
	    ),
		'display' => array(
			'catalog_page_size' => Configuration::get('KERAWEN_CATALOG_PAGE_SIZE'),
			'catalog_full_names' => Configuration::get('KERAWEN_CATALOG_FULL_NAMES'),
			'catalog_refs' => Configuration::get('KERAWEN_CATALOG_REFERENCES'),
		    'cart_loc' => Configuration::get('KERAWEN_CART_LOCATION'),
			'catalog_price_ti' => $catalog_price_ti,
			'cart_full_names' => Configuration::get('KERAWEN_CART_FULL_NAMES'),
		    'cart_ref' => Configuration::get('KERAWEN_CART_REF'),
		    'cart_ean' => Configuration::get('KERAWEN_CART_EAN'),
			'orders_list_column_shop' => (int) Configuration::get('KERAWEN_ORDERS_LIST_COLUMN_SHOP'),
		    'orders_list_column_empl' => (int) Configuration::get('KERAWEN_ORDERS_LIST_COLUMN_EMPL'),
			'orders_list_column_carrier' => (int) Configuration::get('KERAWEN_ORDERS_LIST_COLUMN_CARRI'),
			'orders_list_column_till' => (int) Configuration::get('KERAWEN_ORDERS_LIST_COLUMN_TILL'),
			'orders_list_column_comp' => (int) Configuration::get('KERAWEN_ORDERS_LIST_COLUMN_COMP'),
		    'orders_list_column_add' => (int) Configuration::get('KERAWEN_ORDERS_LIST_COLUMN_ADD'),
		    'orders_list_column_pay' => (int) Configuration::get('KERAWEN_ORDERS_LIST_COLUMN_PAY'),
			'orders_list_items_by_page' => ((int) Configuration::get('KERAWEN_ORDERS_LIST_ITEMS_BY_PAG') > 0) ? Configuration::get('KERAWEN_ORDERS_LIST_ITEMS_BY_PAG') : 15,
			'decimal_separator' => Configuration::get('KERAWEN_DECIMAL_SEPARATOR'),
			'offer_period' => Configuration::get('KERAWEN_OFFER_PERIOD'),
			'postcode_required' => (boolean) Configuration::get('KERAWEN_POSTCODE_REQUIRED'),
			'cust_account_addr' => (boolean) Configuration::get('KERAWEN_CUST_ACCOUNT_ADDR'),
		    'address1_required' => (boolean) Configuration::get('KERAWEN_ADDRESS1_REQUIRED'),
		    'city_required' => (boolean) Configuration::get('KERAWEN_CITY_REQUIRED'),
		    'phone_required' => (boolean) Configuration::get('KERAWEN_PHONE_REQUIRED'),
		    'mobile_required' => ($loyalty && get_class($loyalty) == 'KerawenZerosixapi') ? true : (boolean) Configuration::get('KERAWEN_MOBILE_REQUIRED'),
		    'birthdate_required' => (boolean) Configuration::get('KERAWEN_BIRTHDATE_REQUIRED'),
		    'regex_mobile' => is_null(Configuration::get('KERAWEN_REGEX_MOBILE')) ? "" : Configuration::get('KERAWEN_REGEX_MOBILE'), //^(0|(00|\+)[-\s\.]?[0-9]{2,3}[-\s\.]?)[67][0-9]{8}$
		    'regex_phone' => is_null(Configuration::get('KERAWEN_REGEX_PHONE')) ? "" : Configuration::get('KERAWEN_REGEX_PHONE'),
		    'customer_password' => (boolean) Configuration::get('KERAWEN_CUSTOMER_PASSWORD'),
		    'daily_total' => (boolean) Configuration::get('KERAWEN_DAILY_TOTAL'),
		    'old_reporting' => (boolean) Configuration::get('KERAWEN_OLD_REPORTING'),
		    'cust_card' => ($loyalty && get_class($loyalty) == 'KerawenZerosixapi') ? false : (boolean) Configuration::get('KERAWEN_CUST_CARD'),
		    'account' => (int) Configuration::get('KERAWEN_ACCOUNT'),
		    'deposit_account' => (int) Configuration::get('KERAWEN_DEPOSIT_ACCOUNT'),
		    'account_cust' => (int) Configuration::get('KERAWEN_GROUP_AMOUNT_CUST'),
		    'mode_0_note' => (boolean) Configuration::get('KERAWEN_MODE_0_NOTE'),
		    'cart_tag' => (boolean) Configuration::get('KERAWEN_CART_TAG'),
		    'low_perf' => (boolean) Configuration::get('KERAWEN_LOW_PERF'),
		    'prod_table_setting' => Configuration::get('KERAWEN_PROD_TABLE_SETTING'),
		    'catalog_manu' => (boolean) Configuration::get('KERAWEN_CATALOG_MANUFACTURER'),
		    'new_msg_as_private' => (int) Configuration::get('KERAWEN_NEW_MSG_AS_PRIVATE'),
		    'display_stocks_res' => (int) Configuration::get('KERAWEN_DISPLAY_STOCKS_RES'),
		    'prod_types' => json_decode(Configuration::get('KERAWEN_PROD_TYPES')),
		    'discount_button' => (int) Configuration::get('KERAWEN_DISCOUNT_BUTTON'),
		    'advanced_search' => (int) Configuration::get('KERAWEN_ADVANCED_SEARCH'),
		    'reporting_buysale' => (int) Configuration::get('KERAWEN_REPORTING_BUYSALE'),
		),
	    'payments' => $payments,
	    'payments_extended' => $payments_extended,
	    'payments_used' => $payments_used,
	    'custdisplay' => array(
			'cpl' => Configuration::get('KERAWEN_DISPLAY_CPL'),
			'msg_start' => Configuration::get('KERAWEN_DISPLAY_MSG_START'),
			'msg_end' => Configuration::get('KERAWEN_DISPLAY_MSG_END'),
		),
	    'max_rest_voucher' => Configuration::get('KERAWEN_MAX_REST_VOUCHER'),
		'ticket' => getReceiptSettings(),
		'gather_measures' => (boolean)Configuration::get('KERAWEN_GATHER_MEASURES'),
		'profilesclasses' => $profilesclasses,
		'profiles' => $profiles,
		'employee' => $active_employee,
		'categories' => $cat_groups,
		'params' => array(
			'advanced_stock' => (int)Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT'),
			'mvt_stock' => (int)Tools::version_compare(_PS_VERSION_, '1.7', '>=')
		),
		'token' => $token,
		'carriers' => $carriers,
		'languages' => $languages,
	    'default_lang' => (int)Configuration::get('PS_LANG_DEFAULT'),
		'delivery_modes' => $delivery_modes,
	    'tpe_auto' => (int) Configuration::get('KERAWEN_TPE_AUTO'),
	    'tags' => $tags,
	    'thread_state' => getKerawenThreadStats(),
	    'thread_status' => $thread_status,
	    'thread_status_active' => (int) Configuration::get('KERAWEN_THREAD_STATUS_ACTIVE'),
	    'custdef' => $custdef,
	    'prodVisibilities' => $prodVisibilities,
	    'ps_version' => _PS_VERSION_,
	    'discount_duration' => ((int) Configuration::get('KERAWEN_DISCOUNT_DURATION') > 0) ? (int) Configuration::get('KERAWEN_DISCOUNT_DURATION') : 365,
	    'voucher_duration' => ((int) Configuration::get('KERAWEN_VOUCHER_DURATION') > 0) ? (int) Configuration::get('KERAWEN_VOUCHER_DURATION') : 30,
	);

		
	//Define specific data by app
	if (isset($params->appli)) {
	    switch($params->appli) {
	        case 'tills':
	            $data['node'] = array(
	               'maxUploadSize' => (4 * 1000 * 1024),
	            );
	        break;
	        case 'picking':
	            $log->require = array('getOrderPickingSettings');
	            $dataLog = getLogMvt($log);
	            $data['picking'] = $dataLog['getOrderPickingSettings'];
	        break;
	        case 'report':
	            $manufacturer = array();
	            foreach(Manufacturer::getManufacturers(false) as $item)
	                $manufacturer[$item['id_manufacturer']] = array('id' => $item['id_manufacturer'], 'name' => $item['name']);
    
	            $columns = array_column($manufacturer, 'name');
	            array_multisort($columns, SORT_ASC, $manufacturer);
	            $data['manufacturer'] = $manufacturer;
	            
	            $supplier = array();
	            foreach(Supplier::getSuppliers(false) as $item)
	                $supplier[$item['id_supplier']] = array('id' => $item['id_supplier'], 'name' => $item['name']);
	            
	            $columns = array_column($supplier, 'name');
	            array_multisort($columns, SORT_ASC, $supplier);
	            $data['supplier'] = $supplier;
	            
	            $data['withdeposit'] = withDeposit();
	            
	            
	        break;
	    }
	}

	return $data;
	
}

function withDeposit() {
    $db = Db::getInstance();
    $id_prod_deposit = (int) $db->getValue('
        SELECT p.id_product FROM ' . _DB_PREFIX_ . 'product p
        INNER JOIN ' . _DB_PREFIX_ . 'product_wm_kerawen pwk ON pwk.id_product = p.id_product AND pwk.product_type = "' . pSQL(_KERAWEN_TYPE_DEPOSIT_) . '"
    ');
    $id_account_deposit = (int) $db->getValue('
        SELECT id_account FROM '._DB_PREFIX_.'customer_kerawen_account WHERE type = "' . pSQL(_KERAWEN_TYPE_DEPOSIT_) . '"
    ');
    return (int) ($id_prod_deposit > 0 || $id_account_deposit > 0);
}


function selectPeriod() {
	$db = Db::getInstance();
	$refDate = $db->getValue('SELECT DATE_FORMAT(NOW(), "%Y/%m/%d")');
	return array(
		'from' => $refDate,
		'to' => $refDate,
	);
}


function getReceiptSettings() {
	
	$context = Context::getContext();
	$id_lang = $context->language->id;
	
	return array(
		'cpl' => (int)Configuration::get('KERAWEN_TICKET_CPL'),
		'image' => Configuration::get('KERAWEN_TICKET_IMAGE') ? _PS_IMG_ . Configuration::get('KERAWEN_TICKET_IMAGE') : "",
		'message' => Configuration::get('KERAWEN_TICKET_MESSAGE'),
		'gift_card_message' => Configuration::get('KERAWEN_GIFT_CARD_TICKET_MESSAGE'),
		'shop_name' => (boolean)Configuration::get('KERAWEN_TICKET_SHOP_NAME'),
		'shop_address' => (boolean)Configuration::get('KERAWEN_TICKET_SHOP_ADDRESS'),
		'shop_url' => (boolean)Configuration::get('KERAWEN_TICKET_SHOP_URL'),
		'shop_details' => (boolean)Configuration::get('KERAWEN_TICKET_SHOP_DETAILS'),
	    'comments_top' => (boolean)Configuration::get('KERAWEN_TICKET_COMMENTS_TOP'),
		'comments' => (boolean)Configuration::get('KERAWEN_TICKET_COMMENTS'),
		'prod_note' => (boolean)Configuration::get('KERAWEN_TICKET_PRODUCT_NOTE'),
		'full_names' => (boolean)Configuration::get('KERAWEN_TICKET_FULL_NAMES'),
	    'ref_name' => (boolean)Configuration::get('KERAWEN_TICKET_REF_NAME'),
		'taxes' => (boolean)Configuration::get('KERAWEN_TICKET_TAXES'),
		'detail_taxes' => (boolean)Configuration::get('KERAWEN_TICKET_DETAIL_TAXES'),
		'mode' => (boolean)Configuration::get('KERAWEN_TICKET_MODE'),
		'customer' => (boolean)Configuration::get('KERAWEN_TICKET_CUSTOMER'),
		'loyalty' => (int)Configuration::get('KERAWEN_TICKET_LOYALTY'),
		'barcode' => (boolean)Configuration::get('KERAWEN_TICKET_BARCODE'),
		'order_num' => (boolean)Configuration::get('KERAWEN_TICKET_ORDER_NUMBER'),
		'detail_discount' => (boolean)Configuration::get('KERAWEN_TICKET_DISCOUNT'),
		'reference' => (boolean)Configuration::get('KERAWEN_TICKET_REF'),	
	    'subtotal_discount' => (boolean)Configuration::get('KERAWEN_TICKET_SUBTOTAL_DISCOUNT'),
		'shop_email' => (boolean)Configuration::get('KERAWEN_TICKET_SHOP_EMAIL'),
	    'shop_phone' => (boolean)Configuration::get('KERAWEN_TICKET_SHOP_PHONE'),
		'shop_country' => (boolean)Configuration::get('KERAWEN_TICKET_SHOP_COUNTRY'),
		'employee_name' => (boolean)Configuration::get('KERAWEN_TICKET_EMPLOYEE_NAME'),
		'invoice_tax' => (boolean)Configuration::get('KERAWEN_INVOICE_TAX'),
		'print_open_close' => (boolean)Configuration::get('KERAWEN_TICKET_PRINT_OPEN_CLOSE'),
		'print_auto' => (boolean)Configuration::get('KERAWEN_TICKET_PRINT_AUTO'),
		'invoice_num_order' => (boolean) Configuration::get('KERAWEN_INVOICE_NUM_ORDER'),
		'invoice_num_cart' => (boolean) Configuration::get('KERAWEN_INVOICE_NUM_CART'),	
		'invoice_disp_tax' => (boolean) Configuration::get('KERAWEN_INVOICE_DISP_TAX'),
		'invoice_disp_shipping' => (boolean) Configuration::get('KERAWEN_INVOICE_DISP_SHIPPING'),
		'invoice_disp_unit_vat' => (boolean) Configuration::get('KERAWEN_INVOICE_DISP_UNIT_VAT'),
		'invoice_disp_total_vat' => (boolean) Configuration::get('KERAWEN_INVOICE_DISP_TOTAL_VAT'),	
		'invoice_disp_barcode' => (boolean) Configuration::get('KERAWEN_INVOICE_DISP_BARCODE'),
		'invoice_ref_col' => (boolean) Configuration::get('KERAWEN_INVOICE_REF_COL'),
		'invoice_free_text' => Configuration::get('KERAWEN_INVOICE_FREE_TEXT', $id_lang) ? Configuration::get('KERAWEN_INVOICE_FREE_TEXT', $id_lang) : Configuration::get('KERAWEN_INVOICE_FREE_TEXT'),
		'print_min_amount' => (float) Configuration::get('KERAWEN_TICKET_PRINT_MIN_AMOUNT'),
		'phone' => (boolean) Configuration::get('KERAWEN_TICKET_CUSTOMER_PHONE'),
		'header_date' => (boolean) Configuration::get('KERAWEN_INVOICE_HEADER_DATE'),
	    'msg_discount' => Configuration::get('KERAWEN_TICKET_MSG_DISCOUNT'),
	    'cust_header_msg' => Configuration::get('KERAWEN_CUST_HEADER_MESSAGE'),
	    'cust_footer_msg' => Configuration::get('KERAWEN_CUST_FOOTER_MESSAGE'),
	    'disp_shop_name' => (boolean) Configuration::get('KERAWEN_TICKET_PRINT_SHOP_NAME'),
	    'disp_qty_left' => (boolean) Configuration::get('KERAWEN_TICKET_QTY_LEFT'),
	    'disp_count_sale' => (boolean) Configuration::get('KERAWEN_TICKET_COUNT_SALE'),
	    'prepaid' => (boolean) Configuration::get('KERAWEN_TICKET_PRINT_PREPAID'),
	    'xz_employee' => (boolean) Configuration::get('KERAWEN_TICKET_XZ_EMPLOYEE'),
	    'exchange_header' => (boolean) Configuration::get('KERAWEN_EXCHANGE_HEADER'),
	    'exchange_employee' => (boolean) Configuration::get('KERAWEN_EXCHANGE_EMPLOYEE'),
	    'exchange_prod_name' => (boolean) Configuration::get('KERAWEN_EXCHANGE_PROD_NAME'),
	    'exchange_prod_ref' => (boolean) Configuration::get('KERAWEN_EXCHANGE_PROD_REF'),
	    'exchange_prod_note' => (boolean) Configuration::get('KERAWEN_EXCHANGE_PROD_NOTE'),
	    'exchange_modes' => (boolean) Configuration::get('KERAWEN_EXCHANGE_MODES'),
	    'exchange_footer' => (boolean) Configuration::get('KERAWEN_EXCHANGE_FOOTER'),
	    'exchange_prod_full_name' => (boolean) Configuration::get('KERAWEN_EXCHANGE_PROD_FULL_NAME'),
	    'exchange_threads' => (boolean) Configuration::get('KERAWEN_EXCHANGE_THREADS'),
	    'exchange_delivery' => (boolean) Configuration::get('KERAWEN_EXCHANGE_DELIVERY'),
	    'exchange_num' => (boolean) Configuration::get('KERAWEN_EXCHANGE_NUM'),
	    'exchange_noteDelivery' => (boolean) Configuration::get('KERAWEN_EXCHANGE_NOTEDELIVERY'),
	);
}

function selectShops($id_shop = 0) {

	$db = Db::getInstance();
	
	$shops = array();
	$ss = $db->executeS('
		SELECT s.name, s.id_shop
		FROM '._DB_PREFIX_.'shop s
		WHERE 0 = ' . (int) $id_shop . ' OR s.id_shop = ' . (int) $id_shop
	);
	
	foreach ($ss as $s)
	{
		$id_shop = $s['id_shop'];
		
		$tempArray = array(
			'id' => $id_shop,
			'name' => $s['name'],
			'label' => Configuration::get('PS_SHOP_NAME', null, null, $id_shop),
			'email' => Configuration::get('PS_SHOP_EMAIL', null, null, $id_shop),
			'addr1' => Configuration::get('PS_SHOP_ADDR1', null, null, $id_shop),
			'addr2' => Configuration::get('PS_SHOP_ADDR2', null, null, $id_shop),
			'postcode' => Configuration::get('PS_SHOP_CODE', null, null, $id_shop),
			'city' => Configuration::get('PS_SHOP_CITY', null, null, $id_shop),
			'country' => Configuration::get('PS_SHOP_COUNTRY', null, null, $id_shop),
			'country_id' => Configuration::get('PS_SHOP_COUNTRY_ID', null, null, $id_shop),
			'phone' => Configuration::get('PS_SHOP_PHONE', null, null, $id_shop),
			'details' => Configuration::get('PS_SHOP_DETAILS', null, null, $id_shop),

			'web' => Configuration::get('KERAWEN_SHOP_URL', null, null, $id_shop),
			'siret' => Configuration::get('KERAWEN_SHOP_SIRET', null, null, $id_shop),
			'naf' => Configuration::get('KERAWEN_SHOP_NAF', null, null, $id_shop),
			'tva_intra' => Configuration::get('KERAWEN_SHOP_TVA_INTRA', null, null, $id_shop),
		);
		
		array_walk($tempArray, function(&$val, $key) { $val = ($val === false) ? '' : $val; });
		
		$tempArray['valid'] = checkShop($tempArray);
		
		$shops[$id_shop] = $tempArray;
		
	}
	
	return $shops;

}


function checkShop($tempArray) {
	
	$required_fields = array(
		'email',
		'addr1',
		'postcode',
		'city',
		'country',
		'phone',
		'siret',
		'naf',
		'tva_intra',
	);
	
	$flag = true;
	foreach($required_fields as $v) {
		if (empty($tempArray[$v])) {
			$flag = false;
		}	
	}
	return $flag;
}


function saveShop($params) {
	
    require_once (_KERAWEN_CLASS_DIR_.'/data.php');
    $anon_cust = getAnonymousCustomer();
    
	$db = Db::getInstance();
	
	$id_shop = (int) $params->id_shop;
	$kda = 'KERAWEN_DEFAULT_ADDRESS';
	
	
	$shop_fields = array(
	    'PS_SHOP_NAME' => 'PS_SHOP_NAME',
		'PS_SHOP_EMAIL' => 'PS_SHOP_EMAIL',
		'PS_SHOP_ADDR1' => 'PS_SHOP_ADDR1',
		'PS_SHOP_ADDR2' => 'PS_SHOP_ADDR2',
		'PS_SHOP_CODE' => 'PS_SHOP_CODE',
		'PS_SHOP_CITY' => 'PS_SHOP_CITY',
		'PS_SHOP_COUNTRY_ID' => 'PS_SHOP_COUNTRY_ID',
		'PS_SHOP_PHONE' => 'PS_SHOP_PHONE',
		'KERAWEN_SHOP_SIRET' => 'KERAWEN_SHOP_SIRET',
		'KERAWEN_SHOP_NAF' => 'KERAWEN_SHOP_NAF',
		'KERAWEN_SHOP_TVA_INTRA' => 'KERAWEN_SHOP_TVA_INTRA',
		'KERAWEN_SHOP_URL' => 'KERAWEN_SHOP_URL',
	);

	$country = new Country((int)$params->data->PS_SHOP_COUNTRY_ID);
	if ($country) {
		if (is_array($country->name)) {
			Configuration::updateValue('PS_SHOP_COUNTRY', reset($country->name), false, false, $id_shop);
		}
	}


	foreach ($shop_fields as $name => $field) {
		Configuration::updateValue($name, $params->data->{$field}, false, false, $id_shop);
	}
	
	//multi
	if (Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE')) {
	    $id_address = (int) $db->getValue('SELECT value FROM ' . _DB_PREFIX_ . 'configuration WHERE id_shop = ' . $id_shop . ' AND name = "' . $kda . '"');
	    $addr = saveShopWithAddress($id_address, $anon_cust, $params);
	    Configuration::updateValue($kda, $addr->id, false, false, $id_shop);

	//mono  
	} else {
	    $id_address = Configuration::getGlobalValue($kda);
	    $addr = saveShopWithAddress($id_address, $anon_cust, $params);
	    Configuration::updateGlobalValue($kda, $addr->id);
	}
}


function saveShopWithAddress($id_address, $anon_cust, $params) {

    $addr = new Address($id_address);
    $addr->alias = 'Shop takeaway';
    $addr->id_customer = $anon_cust->id;
    $addr->firstname = ' ';
    $addr->lastname = ' ';
    $addr->company = $params->data->PS_SHOP_NAME;
    $addr->address1 = $params->data->PS_SHOP_ADDR1;
    $addr->address2 = $params->data->PS_SHOP_ADDR2;
    $addr->city = $params->data->PS_SHOP_CITY;
    $addr->id_country = $params->data->PS_SHOP_COUNTRY_ID;
    $addr->phone = $params->data->PS_SHOP_PHONE;
    $addr->phone_mobile = ($addr->phone_mobile == '0600000000' || $addr->phone_mobile == '' || $addr->phone_mobile == null) ? $params->data->PS_SHOP_PHONE : $addr->phone_mobile;
    $addr->date_add = Db::getInstance()->getValue('SELECT NOW()');
    $addr->postcode = $params->data->PS_SHOP_CODE;
    $addr->save();
    
    return $addr;
}


/* TO REFACTOR */
function startRegister($context, $params, &$response)
{
	$db = Db::getInstance();
	
	// Cart to restore
	$id_cart = null;
	if (isset($params->cart)) {
		$restore = $params->cart;
		if (isset($restore->id) && isset($restore->version)) {
			// Check cart is still available with right version
			if ($db->getValue('
				SELECT c.id_cart FROM '._DB_PREFIX_.'cart c
				JOIN '._DB_PREFIX_.'cart_kerawen ck ON ck.id_cart = c.id_cart
				WHERE c.id_cart = '. (int) $restore->id .'
				AND ck.version = '. (int) $restore->version)) {
				$id_cart = $restore->id;
			}
		}
	}
	
	require_once (dirname(__FILE__).'/cart.php');
	$params->id_next = $id_cart;
	$params->id_cust = null;
	$params->id_cart = null;
	$params->suspend = false;
	resetCart($context, $params, $response);

	require_once (dirname(__FILE__).'/notif.php');
	$response->addResult('notif', getNotif($context));
}

function stopRegister($context)
{
	$context->employee->logout();
}

function searchCode($code, $id_lang, $id_cart)
{
	$db = Db::getInstance();
	$res = array(
		'prods' => array(),
	);

	if (Tools::strlen($code) > 0)
	{
		$found = false;

		if (! $found)
		{
			// Loyalty number
			$id_cust = $db->getValue('
				SELECT id_customer
				FROM '._DB_PREFIX_.'customer_kerawen
				WHERE loyalty_number = "'.pSQL($code).'"');
			if ($id_cust)
			{
				$found = true;
				$res ['id_cust'] = $id_cust;
			}
		}
		
		if (! $found)
		{
		    $orders = array();

		    if (is_numeric($code) && class_exists('PrestaShopCollection') && (int) substr($code, 0, 1) > 0) {
    			$orderSearch = new PrestaShopCollection('Order');
    			$orderSearch->where("id_order", '=', $code);
    			$orders = $orderSearch->getResults();
			}
			else {
				// Order reference ?
				$orders = Order::getByReference($code)->getResults();
			}
			
			//Search by invoice reference #FAxxxx
			if (!count($orders)) {
			    $id_order = $db->getValue('
    			    SELECT 
                        o.ps_order 
                    FROM '._DB_PREFIX_.'kerawen_525_invoice ki
    			    INNER JOIN '._DB_PREFIX_.'kerawen_525_order o ON o.id_order = ki.id_order
    			    WHERE ki.invoice_label = "' . pSQL($code) . '" OR ki.invoice_label = "#' . pSQL($code) . '"
                ');

			    if ($id_order)
			    {
			        $order = new stdClass;
			        $order->id = $id_order;
			        $orders[0] = $order;
			    }
			}
			
			if (count($orders) > 0)
			{
				$found = true;
				$id_order = null;
				foreach ($orders as &$order)
				{
					// EVOL multiple order case
					// $ids[] = $o->id;
					$id_order = $order->id;
				}
				$res['id_order'] = $id_order;
			}
		}

		if (!$found)
		{
			if (Validate::isEan13($code))
			{
				$prefix = Configuration::get('KERAWEN_SCALE_PREFIX');
				$len = Tools::strlen($prefix);
				if ($len)
				{
					$start = 0;
					if (Tools::substr($code, $start, $len) == $prefix)
					{
						$start += $len;
						$len = Configuration::get('KERAWEN_SCALE_PRODUCT_LENGTH');
						$ref = Tools::substr($code, $start, $len);
						$buf = $db->getRow('
							SELECT id_product, id_code
							FROM '._DB_PREFIX_.'product_wm_code_kerawen
							WHERE code = "'.pSQL($ref) .'"');
						if ($buf)
						{
							$start += $len;
							$len = Configuration::get('KERAWEN_SCALE_PRICE_LENGTH');
							$price = (float)Tools::substr($code, $start, $len);
							$mult = (float)Configuration::get('KERAWEN_SCALE_PRICE_MULTIPLIER');
							
							$found = true;
							$res['prods'][] = array(
								'id_prod' => $buf['id_product'],
								'id_code' => $buf['id_code'],
								'price' => $price * $mult,
							);
						}
					}
				}
			}
		}
		
		if (!$found)
		{
			require_once (dirname(__FILE__).'/catalog.php');
			$context = Context::getContext();
			$id_shop = $context->shop->id;
			
			// Search for products
			if (Configuration::get('KERAWEN_LABEL_BARCODE') == 'ean') {
				$filter = ' p.ean13 LIKE "%'.pSQL(substr($code, 0, Configuration::get('KERAWEN_EAN13_SEARCH_LENGTH'))).'%" ';
			
    			if (Configuration::get('KERAWEN_UPC_SEARCH')) {
    			    $filter .= ' OR p.upc LIKE "%'.pSQL(substr($code, 0, Configuration::get('KERAWEN_UPC_SEARCH_LENGTH'))).'%" ';
    			}
    			
			} else {
				$filter = ' p.reference = "'.pSQL($code).'" ';
			}
			$active_filter = Configuration::get('KERAWEN_CATALOG_INACTIVE') ? '' : 'AND ps.active = 1';
			$buf = $db->executeS('
				SELECT p.id_product
				FROM '._DB_PREFIX_.'product p
                JOIN '._DB_PREFIX_.'product_shop ps
                    ON ps.id_product = p.id_product
                    AND ps.id_shop = '. (int) $id_shop .'
                    '.$active_filter.'
				WHERE '.$filter);

			foreach($buf as $p) {
				$id_prod = $p['id_product'];
				$detail = detailProduct(new Product($id_prod, true, $id_lang), $id_lang, true);
				if ($detail) {
					$res['prods'][] = array(
						'id_prod' => $id_prod,
						'detail' => $detail,
					);
					$found = true;
				}
			}
					
			$buf = $db->executeS('
				SELECT p.id_product, p.id_product_attribute
				FROM '._DB_PREFIX_.'product_attribute p
                JOIN '._DB_PREFIX_.'product_attribute_shop pas
                    ON pas.id_product_attribute = p.id_product_attribute
                    AND pas.id_shop = '. (int) $id_shop .'
                JOIN '._DB_PREFIX_.'product_shop ps
                    ON ps.id_product = p.id_product
                    AND ps.id_shop = pas.id_shop
                    AND ps.active = 1
				WHERE '.$filter);

			foreach($buf as $pa) {
				$id_prod = $pa['id_product'];
				$detail = detailProduct(new Product($id_prod, true, $id_lang), $id_lang, true);
				if ($detail) {
					$res['prods'][] = array(
						'id_prod' => $id_prod,
						'id_attr' => $pa['id_product_attribute'],
						'detail' => $detail,
					);
					$found = true;
				}
			}
		}
		
		//Module quantitydiscountpro
		if (! $found)
		{
		    if ($id_cart && Module::isInstalled('quantitydiscountpro') && Module::isEnabled('quantitydiscountpro')) {
		        include_once(_PS_MODULE_DIR_.'quantitydiscountpro/classes/QuantityDiscountRule.php');
		        include_once(_PS_MODULE_DIR_.'quantitydiscountpro/classes/QuantityDiscountRuleFamily.php');
		        include_once(_PS_MODULE_DIR_.'quantitydiscountpro/classes/QuantityDiscountDatabase.php');
		        include_once(_PS_MODULE_DIR_.'quantitydiscountpro/classes/QuantityDiscountRuleCondition.php');
		        include_once(_PS_MODULE_DIR_.'quantitydiscountpro/classes/QuantityDiscountRuleGroup.php');
		        include_once(_PS_MODULE_DIR_.'quantitydiscountpro/classes/QuantityDiscountRuleAction.php');
		        include_once(_PS_MODULE_DIR_.'quantitydiscountpro/classes/QuantityDiscountRuleMessage.php');
		        if (($quantityDiscount = new quantityDiscountRule(QuantityDiscountRule::getQuantityDiscountRuleByCode($code))) && Validate::isLoadedObject($quantityDiscount)) {
		            if ($quantityDiscount->createAndRemoveRules($code, Context::getContext(), false)) {
		                $cr = $db->getRow('
                            SELECT
                                id_cart_rule,
                                id_quantity_discount_rule
                            FROM `'._DB_PREFIX_.'quantity_discount_rule_cart`
                            WHERE `id_cart` = '. (int) $id_cart
		                    );
		                
		                //only one required to refresh cart
		                if ($cr) {
		                    $found = true;
		                    $res['id_reduc'] = $cr["id_cart_rule"];
		                }
		            }
		        }
		    }
		}

		if (!$found)
		{
			// Reduction code ?
			$id = CartRule::getIdByCode($code);
			if ($id)
			{
				$found = true;
				$rule = new CartRule($id);
				if ($rule->quantity
					&& $rule->active
					&& time() >= strtotime($rule->date_from)
					&& time() <= strtotime($rule->date_to)) {
					$res['id_reduc'] = $id;
				}
				else
					$res ['ERROR'] = 'Cart rule no more available';
			}
		}

		// Not found: determine code type
		if (! $found)
		{
			if (Validate::isEan13($code))
			{
				$found = true;
				$res['type'] = 'product';
			}
		}
		if (! $found)
		{
			if (preg_match('/CLI[0-9]+/', $code))
			{
				$found = true;
				$res['type'] = 'customer';
			}
		}
		
	}
	return $res;
}

/* Avoid context employee usage which may have been overriden */
function getAdminLink($controller, $params = null)
{
	$context = Context::getContext();
	if (!$params) $params = array();
	$params['token'] = Tools::getAdminToken($controller.(int)Tab::getIdFromClassName($controller).(int)$context->cookie->id_employee);
	return Dispatcher::getInstance()->createUrl($controller, $context->language->id, $params, false);
}

