<?php
/**
* 2014 KerAwen
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2014 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/

class KerawenSponsor
{
	protected static $plugin = null;
	
	public static function getPlugin() {


		static $modules = array(
			'referralbyphone_cpd' => 'KerawenReferralbyphoneCdp',
		    'allinone_rewards' => 'KerawenReferralAllInOneRewards',
		);

		if (self::$plugin == null) {
			self::$plugin = false;
			foreach ($modules as $name => $class) {
				if (Module::isInstalled($name) && Module::isEnabled($name)) {
					self::$plugin = new $class();
					break;
				}
			}
		}

		return self::$plugin;
	}
}


abstract class KerawenSponsorPlugin
{
	public abstract function getInfo($id_cust);
	public abstract function add($id_cust, $email);
	public abstract function getCustomerInfo($id_cust);
	public abstract function addReferer();
	public abstract function addSponsor();
	public abstract function createSponsorship($params, $spec);
	public abstract function isActive();
}



class KerawenReferralbyphoneCdp
{
    
    public function __construct() {
    }

    public function isActive() {
        return true;
    }
    
    public function addReferer() {
        return true;
    }

    public function addSponsor() {
        return false;
    }
    
    public function createSponsorship($params, $spec) {
        return false;
    }
    
    public function getInfo($id_cust) {
        
        require_once(_PS_MODULE_DIR_.'referralbyphone_cpd/ReferralByPhoneModule.php');
        $referral = ReferralByPhoneModule::getSponsorFriend((int)$id_cust);
     
        return $referral;
    }
 
    public function add($id_cust, $email) {
        
        require_once(_PS_MODULE_DIR_.'referralbyphone_cpd/ReferralByPhoneModule.php');
        require_once(_PS_MODULE_DIR_.'referralbyphone_cpd/referralbyphone_cpd.php');
        return ReferralByPhoneModule::inviteNewReferee(new Customer($id_cust), array($email));
        
        //return $email;
    }

    public function getCustomerInfo($id_cust) {
        return false;
    }

}


class KerawenReferralAllInOneRewards
{
    
    private $aior_instance = null;
    
    public function __construct() {
        $this->aior_instance = Module::getInstanceByName('allinone_rewards');
    }
 
    public function isActive() {
        return (boolean) Configuration::get('RSPONSORSHIP_ACTIVE');
    }
    
    public function addReferer() {
        return false;
    }
    
    public function addSponsor() {
        
        //TODO: could be false -> Depend on customer group ?
        
        return $this->isActive();
    }

    public function createSponsorship($params, $spec) {
        
        $customer = $params['customer'];
        
        $q = '
            SELECT 
                id_sponsorship 
            FROM '._DB_PREFIX_.'rewards_sponsorship
            WHERE id_customer = ' . (int) $customer->id;
        
        //Only once
        if ((int) Db::getInstance()->getValue($q) == 0) {
        
            $context = Context::getContext();
            $context->cookie->rewards_sponsor_id = $spec->id;
            
            $params['newCustomer'] = $customer;
            
            $this->aior_instance->hookActionCustomerAccountAdd($params);
        }

    }
    
    
    public function getInfo($id_cust) {
           
        $q = '
			SELECT
                rs.id_sponsorship AS id,
                rs.date_add,
                rs.date_upd,
                rs.id_customer,
                rs.id_cart_rule AS id_cart_rule_sponsor,
                IF(rs.id_customer = 0, rs.email, c.email) AS email
			FROM '._DB_PREFIX_.'rewards_sponsorship rs
            LEFT JOIN ' . _DB_PREFIX_ . 'customer c ON c.id_customer = rs.id_customer
			WHERE rs.id_sponsor = ' . (int) $id_cust;
        
        return Db::getInstance()->ExecuteS($q);
       
    }
    
    
    public function add($id_cust, $email) {
        return false;
    }
 
    public function getCustomerInfo($customer) {
        
        require_once(_PS_MODULE_DIR_.'allinone_rewards/models/RewardsSponsorshipModel.php');
        require_once(_PS_MODULE_DIR_.'allinone_rewards/models/RewardsSponsorshipCodeModel.php');

        $code = '';
        $name = '';
        $id = 0;
        
        if ($customer) {
            $code = RewardsSponsorshipModel::getSponsorshipCode($customer, true);
            
            $q = '
                SELECT 
                    IF(ISNULL(c.id_customer), "", CONCAT(c.firstname, " ", c.lastname)) AS name,
                    id_sponsor AS id
                FROM ' . _DB_PREFIX_ . 'rewards_sponsorship rs
                INNER JOIN ' . _DB_PREFIX_ . 'customer c ON c.id_customer = rs.id_sponsor
                WHERE rs.id_customer = ' . (int) $customer->id
            ;
            $sp = Db::getInstance()->getRow($q);
            if ($sp) {
                $name = $sp['name'];
                $id = (int) $sp['id'];
            }
        }
        
        return array(
            'code' => $code,
            'name' => $name,
            'id' => $id
        );
    }
    
}


