<?php
/**
 * 2015 KerAwen
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@kerawen.com so we can send you a copy immediately.
 *
 * @author    KerAwen <contact@kerawen.com>
 * @copyright 2014 KerAwen
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
 */

require_once (dirname(__FILE__).'/KerawenAdminController.php');
require_once (_KERAWEN_CLASS_DIR_.'/KerawenPayment.php');
require_once (_KERAWEN_TOOLS_DIR_.'/utils.php');
require_once (_KERAWEN_API_DIR_.'/constants.php');

class KerawenConfigController extends KerawenAdminController
{
    
    private $_confLanguages;
    var $cnt = 0;
    
    var $listPaymentStats = array('NP','PP','FP','TP');
    var $modes = array(0, 1, 2);
    
    public function __construct()
    {
        $this->display = 'edit';
        parent::__construct();
        $this->multishop_context = Shop::CONTEXT_ALL;
        $this->toolbar_title = $this->l('KerAwen module configuration');
        
        $this->_confLanguages = Language::getLanguages(true);
        

    }
    
    
    protected function getProdTypes() {
        return array(
            _KERAWEN_TYPE_STANDARD_  => $this->l('Standard'),
            _KERAWEN_TYPE_DEALER_ => $this->l('Buying/reselling'),
            _KERAWEN_TYPE_DEPOSIT_ => $this->l('Deposit sale'),
           // _KERAWEN_TYPE_GIFTCARD_ => $this->l('Gift card'),
           // _KERAWEN_TYPE_PREPAID_ => $this->l('Customer account recharge'),
           // _KERAWEN_TYPE_MEASURED_ => $this->l('Weighed or measured'),
           // _KERAWEN_TYPE_GENERIC_ => $this->l('Free price'),
        );
        
    }
    
    
    protected function renderWarnings()
    {
        return $this->renderContent();
    }
    
    protected function renderContent()
    {
        
        $forms = array(
            
            'payment' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Payment settings') .'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-des-moyens-de-paiements-c1-24-11-2023'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => $this->renderPaymentBlock(),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'mode' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Mode/Status settings'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => $this->renderModeBlock(),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'status' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Status display'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => $this->renderStatusBlock(),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'receipt' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Receipt settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-du-ticket-de-caisse-c2-16-10-2023'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        array(
                            'name' => 'KERAWEN_TICKET_CPL',
                            'label' => $this->l('Default characters per line'),
                            'type' => 'text',
                            //'desc' =>  $this->externalLink('https://support.kerawen.com/space/MC/149618795/Configuration+des+moyens+de+paiements+(%23C1)#test-5'),
                        ),
                        array(
                            'name' => 'KERAWEN_TICKET_IMAGE',
                            'label' => $this->l('Header image'),
                            'type' => 'free',
                            'content' => '
								<img id ="kerawen-ticket-image"
									'.(Configuration::get('KERAWEN_TICKET_IMAGE') ? 'src="'._PS_IMG_.Configuration::get('KERAWEN_TICKET_IMAGE').'"' : '')
                            .'/>',
                        ),
                        array(
                            'name' => 'KERAWEN_TICKET_IMAGE_ACTIONS',
                            'type' => 'free',
                            'content' => '
								<div class="btn btn-default" onclick="
									document.getElementById(\'kerawen-ticket-image\').style.display=\'none\';
									document.getElementById(\'KERAWEN_TICKET_IMAGE_DELETE\').value=1;">
									'.$this->l('Delete').'
								</div>
								<div class="btn btn-default" onclick="
									document.getElementById(\'kerawen-ticket-image\').style.display=\'block\';
									document.getElementById(\'KERAWEN_TICKET_IMAGE_DELETE\').value=0;">
									'.$this->l('Restore').'
								</div>',
                        ),
                        array(
                            'name' => 'KERAWEN_TICKET_IMAGE_DELETE',
                            'type' => 'hidden',
                        ),
                        array(
                            'name' => 'KERAWEN_TICKET_IMAGE_UPLOAD',
                            'type' => 'file',
                            'desc' => $this->l('Resolution: 72pdi, Max width: 450px, Format: jpg, gif, png (no transparency)'),
                        ),
                        array(
                            'name' => 'KERAWEN_TICKET_MESSAGE',
                            'label' => $this->l('Footer message'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 3,
                        ),
                        $this->renderSwitch('KERAWEN_TICKET_PRINT_SHOP_NAME', $this->l('Print shop name'), $this->l('Display shop name is required on receipt')),
                        $this->renderSwitch('KERAWEN_TICKET_SHOP_COUNTRY', $this->l('Print shop country')),
                        $this->renderSwitch('KERAWEN_TICKET_SHOP_URL', $this->l('Print shop URL')),
                        $this->renderSwitch('KERAWEN_TICKET_SHOP_EMAIL', $this->l('Print shop Email')),
                        $this->renderSwitch('KERAWEN_TICKET_SHOP_PHONE', $this->l('Print shop phone')),
                        $this->renderSwitch('KERAWEN_TICKET_COMMENTS_TOP', $this->l('Print order comments on header')),
                        $this->renderSwitch('KERAWEN_TICKET_COMMENTS', $this->l('Print order comments on footer')),
                        $this->renderSwitch('KERAWEN_TICKET_PRODUCT_NOTE', $this->l('Print product note')),
                        $this->renderSwitch('KERAWEN_TICKET_FULL_NAMES', $this->l('Print full product names')),
                        $this->renderSwitch('KERAWEN_TICKET_QTY_LEFT', $this->l('Print qty on left hand side')),
                        $this->renderSwitch('KERAWEN_TICKET_TAXES', $this->l('Print total taxes')),
                        $this->renderSwitch('KERAWEN_TICKET_DETAIL_TAXES', $this->l('Print taxes detail')),
                        $this->renderSwitch('KERAWEN_TICKET_CUSTOMER', $this->l('Print customer name')),
                        $this->renderSwitch('KERAWEN_TICKET_CUSTOMER_PHONE', $this->l('Print customer phone')),
                        $this->renderSwitch('KERAWEN_TICKET_REF', $this->l('Print reference')),
                        $this->renderSwitch('KERAWEN_TICKET_SUBTOTAL_DISCOUNT', $this->l('Print subtotal discounted')),
                        $this->renderSwitch('KERAWEN_TICKET_EMPLOYEE_NAME', $this->l('Print employee name')),
                        array(
                            'name' => 'KERAWEN_TICKET_LOYALTY',
                            'label' => $this->l('Print loyalty data'),
                            'type' => 'radio',
                            'class' => 't',
                            'values' => array(
                                array(
                                    'id' => 'loyalty_none',
                                    'value' => 0,
                                    'label' => $this->l('No'),
                                ),
                                array(
                                    'id' => 'loyalty_points',
                                    'value' => 1,
                                    'label' => $this->l('Points'),
                                ),
                                array(
                                    'id' => 'loyalty_amount',
                                    'value' => 2,
                                    'label' => $this->l('Amount'),
                                ),
                                array(
                                    'id' => 'loyalty_both',
                                    'value' => 3,
                                    'label' => $this->l('Both'),
                                ),
                            ),
                        ),
                        $this->renderSwitch('KERAWEN_TICKET_LOYALTY_PRINT', $this->l('Print loyalty vouchers')),
                        $this->renderSwitch('KERAWEN_TICKET_COUNT_SALE', $this->l('Print customer total sales')),
                        $this->renderSwitch('KERAWEN_TICKET_MODE', $this->l('Print mode')),
                        $this->renderSwitch('KERAWEN_TICKET_SHOP_DETAILS', $this->l('Print shop details')),
                        $this->renderSwitch('KERAWEN_TICKET_BARCODE', $this->l('Print ticket barecode')),
                        $this->renderSwitch('KERAWEN_TICKET_ORDER_NUMBER', $this->l('Print incremental order number')),
                        
                        array(
                            'name' => 'KERAWEN_TICKET_MSG_DISCOUNT',
                            'label' => $this->l('Discount message'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 3,
                        ),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            
            'invoice' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Invoice settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-des-factures-c3'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        array(
                            'name' => 'KERAWEN_INVOICE_FREE_TEXT',
                            'label' => $this->l('Invoice free text'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 3,
                            'lang' => true,
                        ),
                        $this->renderSwitch('KERAWEN_INVOICE_TAX', $this->l('Display taxes')),
                        $this->renderSwitch('KERAWEN_INVOICE_NUM_ORDER', $this->l('Display order number')),
                        $this->renderSwitch('KERAWEN_INVOICE_NUM_CART', $this->l('Display cart number')),
                        $this->renderSwitch('KERAWEN_INVOICE_DISP_TAX', $this->l('Display VAT on product row')),
                        $this->renderSwitch('KERAWEN_INVOICE_DISP_SHIPPING', $this->l('Display shipping as product')),
                        $this->renderSwitch('KERAWEN_INVOICE_DISP_UNIT_VAT', $this->l('Display unit price VAT include')),
                        $this->renderSwitch('KERAWEN_INVOICE_DISP_TOTAL_VAT', $this->l('Display total price VAT include')),
                        $this->renderSwitch('KERAWEN_INVOICE_DISP_BARCODE', $this->l('Display barcode')),
                        $this->renderSwitch('KERAWEN_INVOICE_REF_COL', $this->l('Display reference in individual column')),
                        array(
                            'name' => 'KERAWEN_INVOICE_HEADER_DATE',
                            'label' => $this->l('Invoice header date'),
                            'type' => 'radio',
                            'class' => 't',
                            'values' => array(
                                array(
                                    'id' => 'date_order',
                                    'value' => 0,
                                    'label' => $this->l('Order'),
                                ),
                                array(
                                    'id' => 'date_invoice',
                                    'value' => 1,
                                    'label' => $this->l('Invoice'),
                                ),
                            ),
                        ),
                        $this->renderSwitch('KERAWEN_INVOICE_DISP_ONLY_COMP', $this->l('Display only company name (not customer name)')),
                        $this->renderSwitch('KERAWEN_NOTE_SHORT_DESC', $this->l('Display additional product note')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),

            'quote' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Quotation settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-des-devis-c4'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_QUOTE_ACTIVE', $this->l('Display quotes on Front-Office')),
                        $this->renderSwitch('KERAWEN_QUOTE_PRODUCT_NOTE', $this->l('Display product note')),
                        $this->renderSwitch('KERAWEN_QUOTE_TAX', $this->l('Display taxes')),
                        $this->renderSwitch('KERAWEN_QUOTE_DISP_TAX', $this->l('Display VAT on product row')),
                        $this->renderSwitch('KERAWEN_QUOTE_DISP_UNIT_VAT', $this->l('Display unit price VAT include')),
                        $this->renderSwitch('KERAWEN_QUOTE_DISP_TOTAL_VAT', $this->l('Display total price VAT include')),
                        $this->renderSwitch('KERAWEN_QUOTE_REF_COL', $this->l('Display reference in individual column')),
                        $this->renderSwitch('KERAWEN_QUOTE_IMG', $this->l('Display product image')),
                        array(
                            'name' => 'KERAWEN_QUOTE_DURATION',
                            'label' => $this->l('Validity period'),
                            'type' => 'text',
                            'suffix' => $this->l('Days'),
                        ),
                        array(
                            'name' => 'KERAWEN_QUOTE_MESSAGE',
                            'label' => $this->l('Thanks'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 3,
                            'lang' => true,
                        ),
                        array(
                            'name' => 'KERAWEN_QUOTE_MESSAGE_2',
                            'label' => $this->l('Signature'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 3,
                            'lang' => true,
                        ),
                        array(
                            'name' => 'KERAWEN_QUOTE_MESSAGE_3',
                            'label' => $this->l('Buy online'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 3,
                            'lang' => true,
                        ),
                        array(
                            'name' => 'KERAWEN_QUOTE_MESSAGE_4',
                            'label' => $this->l('Termes and conditions'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 3,
                            'lang' => true,
                        ),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'credit' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Credit settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-des-avoirs-c5'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        array(
                            'name' => 'KERAWEN_DISCOUNT_DURATION',
                            'label' => $this->l('Discount validity period'),
                            'type' => 'text',
                            'suffix' => $this->l('days')
                        ),
                        $this->renderSwitch('KERAWEN_CREDIT_CODE_DISPLAY', $this->l('Display credit code in reporting')),
                        $this->renderSwitch('KERAWEN_CREDIT_PARTIAL_USE', $this->l('Credit partial use on web')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            'voucher' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Voucher settings'),
                        'icon' => $this->getIcon(),
                    ),

                    'input' => array(
                        array(
                            'name' => 'KERAWEN_VOUCHER_LABEL',
                            'label' => $this->l('Voucher name'),
                            'type' => 'text',
                        ),
                        array(
                            'name' => 'KERAWEN_VOUCHER_DURATION',
                            'label' => $this->l('Voucher validity period'),
                            'type' => 'text',
                            'suffix' => $this->l('days')
                        ),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            'giftcard' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Gift card settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-des-cartes-cadeaux-c6'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_GIFT_CARD_CODE_DISPLAY', $this->l('Display gift card code in reporting')),
                        $this->renderSwitch('KERAWEN_CUST_SEND_GIFTCARD', $this->l('Send gift card by email to buyer')),
                        array(
                            'name' => 'KERAWEN_GIFT_CARD_DURATION',
                            'label' => $this->l('Validity period'),
                            'type' => 'text',
                            'suffix' => $this->l('Days'),
                        ),
                        array(
                            'name' => 'KERAWEN_GIFT_CARD_TICKET_MESSAGE',
                            'label' => $this->l('Message'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 3,
                        ),
                        $this->renderSwitch('KERAWEN_GIFT_CARD_JS', $this->l('Optimize front-office gift card form with javascript (datepicker)')),
                        array(
                            'name' => 'KERAWEN_GIFT_CARD_CRON_URL',
                            'label' => $this->l('Cron URL'),
                            'type' => 'free',
                            'content' => '<label class="control-label">' . $this->getGiftCardCronUrl() . '</label>',
                        ),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            'OrderFeed' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Orders Feed settings'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        array(
                            'name' => 'KERAWEN_ORDER_FEED_URL',
                            'label' => $this->l('Orders feed URL'),
                            'type' => 'free',
                            'content' => '<label class="control-label"><a href="' . $this->getOrderFeedUrl() . '" target="kerawen_feed">' . $this->getOrderFeedUrl() . '</a></label>',
                        ),
                    )
                ),
            ),
            'employee' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Employee settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-de-l-employ%C3%A9-c7'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_SWITCH_CASHIER', $this->l('Allow to switch cashier')),
                        $this->renderSwitch('KERAWEN_EMPLOYEE_PASSWORD', $this->l('Password required to switch cashier')),
                        $this->renderSwitch('KERAWEN_EMPLOYEE_PASSWORD_EXCP', $this->l('Allow login when password is not defined')),
                        $this->renderSwitch('KERAWEN_EMPLOYEE_ALWAYS_SELECT', $this->l('Select cashier for each new order')),
                        $this->renderSwitch('KERAWEN_SWITCH_SHOP', $this->l('Allow to switch shop')),
                        $this->renderSwitch('KERAWEN_SELECT_DELIVERY', $this->l('Allow to select delivery mode')),
                        array(
                            'name' => 'KERAWEN_DEFAULT_DELIVERY_MODE',
                            'label' => $this->l('Delivery mode by default'),
                            'type'  => 'select',
                            'options'  => array(
                                'query' => array(
                                    array(
                                        'id' => '0',
                                        'name' => $this->l('In store'),
                                    ),
                                    array(
                                        'id' => '1',
                                        'name' => $this->l('Takeaway'),
                                    ),
                                    array(
                                        'id' => '2',
                                        'name' => $this->l('Delivery'),
                                    ),
                                ),
                                'id' => 'id',
                                'value' => 'id',
                                'name' => 'name',
                            )
                        ),
                        $this->renderSwitch('KERAWEN_DAILY_TOTAL', $this->l('Display daily total')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            
            'customer' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Customer settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-du-client-c8'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        array(
                            'name' => 'KERAWEN_DEFAULT_GROUP',
                            'label' => $this->l('New customer default group'),
                            'type'  => 'select',
                            'options'  => array(
                                'query' => Group::getGroups($this->context->language->id),
                                'id' => 'id_group',
                                'value' => 'id_group',
                                'name' => 'name',
                            )
                        ),
                        array(
                            'name' => 'KERAWEN_NEW_CUST_EMAIL_TPL',
                            'label' => $this->l('Email template for new customer'),
                            'type'  => 'select',
                            'options'  => array(
                                'query' => array(
                                    array(
                                        'tpl' => 'account',
                                        'name' => $this->l('Account') . ' (account)',
                                    ),
                                    array(
                                        'tpl' => 'password_query',
                                        'name' => $this->l('Initalize password') . ' (password_query)',
                                    ),
                                ),
                                'id' => 'tpl',
                                'value' => 'tpl',
                                'name' => 'name',
                            )
                        ),
                        $this->renderSwitch('KERAWEN_CUST_ACCOUNT_ADDR', $this->l('New customer account with address form')),
                        $this->renderSwitch('KERAWEN_CUST_OPTIN', $this->l('Display customer optin checkbox')),
                        $this->renderSwitch('KERAWEN_CUSTOMER_PASSWORD', $this->l('Display customer password form')),
                        $this->renderSwitch('KERAWEN_POSTCODE_REQUIRED', $this->l('Post code field is required')),
                        $this->renderSwitch('KERAWEN_ADDRESS1_REQUIRED', $this->l('Address field is required')),
                        $this->renderSwitch('KERAWEN_CITY_REQUIRED', $this->l('City field is required')),
                        $this->renderSwitch('KERAWEN_PHONE_REQUIRED', $this->l('Phone field is required')),
                        $this->renderSwitch('KERAWEN_MOBILE_REQUIRED', $this->l('Mobile field is required')),
                        $this->renderSwitch('KERAWEN_BIRTHDATE_REQUIRED', $this->l('Birthdate field is required')),
                        array(
                            'name' => 'spacer',
                            'label' => $this->l('Dematerialized customer card'),
                            'type' => 'free',
                            'content' => '<hr />',
                        ),
                        $this->renderSwitch('KERAWEN_CUST_CARD', $this->l('Activate customer card')),
                        $this->renderSwitch('KERAWEN_CUST_PRINT', $this->l('Print customer detail')),
                        array(
                            'name' => 'KERAWEN_CUST_HEADER_MESSAGE',
                            'label' => $this->l('Header message'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 3,
                        ),
                        array(
                            'name' => 'KERAWEN_CUST_FOOTER_MESSAGE',
                            'label' => $this->l('Footer message'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 3,
                        ),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'email' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Customer email settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-des-emails-client-c9'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_SEND_EMAIL', $this->l('Send email when changing order state (according to order configuration)')),
                        $this->renderSwitch('KERAWEN_OVERRIDE_EMAIL', $this->l('Override order confirmation emails (include certified invoice)')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'opening_closing' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Opening/closing settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-ouverture-cl%C3%B4ture-de-caisse-c10'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_TICKET_PRINT_OPEN_CLOSE', $this->l('Print ticket on open/close/status')),
                        $this->renderSwitch('KERAWEN_SHOW_AMOUNTS', $this->l('Show expected amounts when opening and closing a till')),
                        $this->renderSwitch('KERAWEN_TICKET_XZ_EMPLOYEE', $this->l('Print turnover by employee')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'display_cart' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Display settings - cart').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-affichage-caisse-panier-c11'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_CART_FULL_NAMES', $this->l('Display full product names in cart')),
                        $this->renderSwitch('KERAWEN_CART_REF', $this->l('Display reference in cart')),
                        $this->renderSwitch('KERAWEN_CART_EAN', $this->l('Display EAN13 code in cart')),
                        $this->renderSwitch('KERAWEN_CART_LOCATION', $this->l('Display location in cart')),
                        $this->renderSwitch('KERAWEN_ALERT_PROD_CART', $this->l('Alert when the product is out of stock')),
                        $this->renderSwitch('KERAWEN_VAT_WARNING', $this->l('Alert if the VAT is equal to 0')),
                        $this->renderSwitch('KERAWEN_CART_TAG', $this->l('Display products tags')),
                        $this->renderSwitch('KERAWEN_MODE_0_ANONYMOUS', $this->l('Allow in store mode for anonymous customer')),
                        $this->renderSwitch('KERAWEN_MODE_1_ANONYMOUS', $this->l('Allow takeway mode for anonymous customer')),
                        $this->renderSwitch('KERAWEN_MODE_2_ANONYMOUS', $this->l('Allow delivery mode for anonymous customer')),
                        $this->renderSwitch('KERAWEN_MODE_0_NOTE', $this->l('Allow delivery note for in store mode')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'display_order' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Display settings - orders').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-affichage-caisse-commandes-c12-16-10-2023'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_ORDERS_LIST_COLUMN_SHOP', $this->l('Display shop name on orders list')),
                        $this->renderSwitch('KERAWEN_ORDERS_LIST_COLUMN_EMPL', $this->l('Display employee name on orders list')),
                        $this->renderSwitch('KERAWEN_ORDERS_LIST_COLUMN_CARRI', $this->l('Display carrier name on orders list')),
                        $this->renderSwitch('KERAWEN_ORDERS_LIST_COLUMN_TILL', $this->l('Display till name on orders list')),
                        $this->renderSwitch('KERAWEN_ORDERS_LIST_COLUMN_COMP', $this->l('Display company name on orders list')),
                        $this->renderSwitch('KERAWEN_ORDERS_LIST_COLUMN_ADD', $this->l('Display date added on orders list')),
                        $this->renderSwitch('KERAWEN_ORDERS_LIST_COLUMN_PAY', $this->l('Display payment mode/module name on orders list')),
                        array(
                            'name' => 'KERAWEN_ORDERS_LIST_ITEMS_BY_PAG',
                            'label' => $this->l('Number of orders per page'),
                            'type' => 'text'
                        ),
                        array(
                            'name' => 'KERAWEN_NOTIF_PERIOD',
                            'label' => $this->l('Notification update period'),
                            'type' => 'text',
                            'suffix' => $this->l('seconds'),
                            'desc' => $this->l('0 means that the cash register will not be notified (e.g. when a new order is received).')
                        ),
                        $this->renderSwitch('KERAWEN_ORDER_QUICK_END', $this->l('Automatically back to catalog when order is ended')),
                        $this->renderSwitch('KERAWEN_AVOID_NEW_ORDER_ALERT', $this->l('Block mail alerts in case of new order from cash register')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            'display_catalog' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Display settings - catalog').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-affichage-caisse-catalogue-c13'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderShortcuts(),
                        $this->renderSwitch('KERAWEN_CATEGORY_ACTIVE', $this->l('Display only active categories')),
                        $this->renderSwitch('KERAWEN_CATALOG_INACTIVE', $this->l('Display inactive products')),
                        array(
                            'name' => 'KERAWEN_CATALOG_PRODUCT_ORDER',
                            'label' => $this->l('Display products order'),
                            'type'  => 'select',
                            'options'  => array(
                                'query' => array(
                                    array(
                                        'id' => '0',
                                        'name' => $this->l('Relevancy (position)'),
                                    ),
                                    array(
                                        'id' => '1',
                                        'name' => $this->l('Alphabetical'),
                                    ),
                                    array(
                                        'id' => '2',
                                        'name' => $this->l('Reverse alphabetical'),
                                    ),
                                ),
                                'id' => 'id',
                                'value' => 'id',
                                'name' => 'name',
                            )
                        ),
                        
                        array(
                            'name' => 'KERAWEN_CATALOG_PAGE_SIZE',
                            'label' => $this->l('Number of products per page'),
                            'type' => 'text',
                            'desc' => $this->l('0 means that all the category products are displayed at once.')
                        ),
                        $this->renderSwitch('KERAWEN_CATALOG_FULL_NAMES', $this->l('Display full product names in catalog as list')),
                        $this->renderSwitch('KERAWEN_CATALOG_REFERENCES', $this->l('Display product references in catalog as list')),
                        $this->renderSwitch('KERAWEN_CATALOG_MANUFACTURER', $this->l('Display product manufacturer')),
                        $this->renderImageTypes('KERAWEN_IMAGE_PRODUCT', $this->l('Image type for products'), 'products'),
                        $this->renderImageTypes('KERAWEN_IMAGE_CATEGORY', $this->l('Image type for categories'), 'categories'),
                        array(
                            'name' => 'KERAWEN_OVERRIDE_GROUP',
                            'label' => $this->l('Override customer group'),
                            'type'  => 'select',
                            'options'  => array(
                                'query' => array_merge(
                                    array(
                                        array(
                                            'id_group' => 0,
                                            'name' => $this->l('None'),
                                        ),
                                    ),
                                    Group::getGroups($this->context->language->id)
                                ),
                                'id' => 'id_group',
                                'value' => 'id_group',
                                'name' => 'name',
                            )
                        ),
                        $this->renderSwitch('KERAWEN_WM_GROUP_EXC', $this->l('Exclude customer group reduction for weight and measures products')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'display_product' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Display settings - products').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-affichage-caisse-produits-c14'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => $this->renderProdTypeBlock(),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),

            'display_after_sale' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Display settings - after sale service'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_NEW_MSG_AS_PRIVATE', $this->l('New message after-sales private')),
                        $this->renderSwitch('KERAWEN_THREAD_STATUS_ACTIVE', $this->l('Display Pretashop status')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'display_cartrules' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Display settings - cart rules').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-affichage-caisse-produits-c14'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_CARTRULE_INCL_GENERIC', $this->l('Display generic cart rules')),
                        $this->renderSwitch('KERAWEN_CARTRULE_ORDER_BY_DATE', $this->l('Ordering by date')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),

            'product_search' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Product search'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_UPC_SEARCH', $this->l('Include UPC code in product search')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'equiment_cashdrawer' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Equiment settings - cash drawer').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-mat%C3%A9riel-tiroir-caisse-c15'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_ORDER_PULSE', $this->l('Automatically open cash drawer when order is ended')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'equipment_scanner' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Equiment settings - scanner').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-mat%C3%A9riel-scanner-c16'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_PLAY_SOUND', $this->l('Play sound when product is scanned')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'equipment_printer' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Equiment settings - printer receipt').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-mat%C3%A9riel-impression-ticket-c17'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        
                        $this->renderSwitch('KERAWEN_TICKET_PRINT_AUTO', $this->l('Automatically print receipt')),
                        array(
                            'name' => 'KERAWEN_TICKET_PRINT_MIN_AMOUNT',
                            'label' => $this->l('Minimum amount for automatically print receipt'),
                            'type' => 'text',
                            'desc' => $this->l('Receipt is legaly required from € 25, leave blank to disable')
                        )
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),

            'equipment_tpe' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Equiment settings - TPE'),
                        'icon' => $this->getIcon(),
                    ),

                    'input' => array(
                        $this->renderSwitch('KERAWEN_TPE_AUTO', $this->l('Send automatically the amount to the TPE')),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'equipment_customer_display' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Equiment settings - customer display').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-mat%C3%A9riel-afficheur-client-c18'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        array(
                            'name' => 'KERAWEN_DISPLAY_CPL',
                            'label' => $this->l('Default characters per line'),
                            'type' => 'text',
                        ),
                        array(
                            'name' => 'KERAWEN_DISPLAY_MSG_START',
                            'label' => $this->l('Start message'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 2,
                        ),
                        array(
                            'name' => 'KERAWEN_DISPLAY_MSG_END',
                            'label' => $this->l('End message'),
                            'type' => 'textarea',
                            'cols' => 48,
                            'rows' => 2,
                        ),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'scale' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Scale settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-de-la-balance-c19'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        $this->renderSwitch('KERAWEN_GATHER_MEASURES', $this->l('Gather the different measures of same product')),
                        array(
                            'name' => 'KERAWEN_SCALE_PREFIX',
                            'label' => $this->l('Barcode prefix'),
                            'type' => 'text',
                        ),
                        array(
                            'name' => 'KERAWEN_SCALE_PRODUCT_LENGTH',
                            'label' => $this->l('Product code length'),
                            'type' => 'text',
                        ),
                        array(
                            'name' => 'KERAWEN_SCALE_PRICE_LENGTH',
                            'label' => $this->l('Price code length'),
                            'type' => 'text',
                        ),
                        array(
                            'name' => 'KERAWEN_SCALE_PRICE_MULTIPLIER',
                            'label' => $this->l('Price code multiplier'),
                            'type' => 'text',
                            'prefix' => $this->context->currency->prefix.$this->context->currency->suffix,
                            'desc' => $this->l('Multiplier applied to price code to obtain selling price tax incl.')
                        ),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'label' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Label settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-des-%C3%A9tiquettes-c20'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        array(
                            'name' => 'KERAWEN_LABEL_ITEMS_BY_PAGE',
                            'label' => $this->l('Items by page'),
                            'type' => 'text',
                        ),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
            
            'reporting' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Reporting settings').'  '. $this->externalLink('https://support.kerawen.com/portal/fr/kb/articles/configuration-du-pilotage-c21'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => array(
                        array(
                            'name' => 'KERAWEN_DECIMAL_SEPARATOR',
                            'label' => $this->l('Export decimal separator'),
                            'type'  => 'select',
                            'options'  => array(
                                'query' => array(
                                    array('char' => '.', 'name' => $this->l('dot')),
                                    array('char' => ',', 'name' => $this->l('comma')),
                                ),
                                'id' => 'char',
                                'value' => 'char',
                                'name' => 'name',
                            )
                        ),
                    ),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),

            'carrier' => array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Front-office carrier/mode settings'),
                        'icon' => $this->getIcon(),
                    ),
                    'input' => $this->getCarrierMode(),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            ),
        );
        
        if (Tools::getValue('advanced')) {

            $input_1 = array(
                array(
                    'name' => 'KERAWEN_THEME',
                    'label' => $this->l('Theme'),
                    'type'  => 'select',
                    'options'  => array(
                        'query' => array(
                            array('value' => 'theme-dark.css', 'name' => $this->l('Dark')),
                            array('value' => 'theme-light.css', 'name' => $this->l('Light')),
                        ),
                        'id' => 'value',
                        'value' => 'value',
                        'name' => 'name',
                    )
                ),
                $this->renderSwitch('KERAWEN_RESTRICTIVE_MODE', $this->l('Restrictive start')),
                $this->renderSwitch('KERAWEN_KEEP_VIEWS', $this->l('Keep search views when the module is updated')),
                $this->renderSwitch('KERAWEN_SEARCH_PROD_BY_VIEW', $this->l('Use view for the quick search')),
                $this->renderSwitch('KERAWEN_OLD_REPORTING', $this->l('Display old reporting')),
                $this->renderSwitch('KERAWEN_TILL_SHOP', $this->l('Link tills to shops')),
                $this->renderSwitch('KERAWEN_MANAGE_PO', $this->l('Manage purchase orders')),
                $this->renderSwitch('KERAWEN_INVOICE_AUTO', $this->l('Automatically generate invoices')),
                $this->renderSwitch('KERAWEN_FORCE_VIRTUAL_SHOP', $this->l('Allow to select virtual shop on register (OCS)')),
                $this->renderSwitch('KERAWEN_TICKET_REF_NAME', $this->l('Replace product name by reference on receipt')),
                $this->renderSwitch('KERAWEN_DISPLAY_STOCKS_RES', $this->l('Display reserved stocks')),
                array(
                    'name' => 'KERAWEN_MVT_BUY',
                    'label' => $this->l('Buying stock mouvement type'),
                    'type' => 'text',
                    'desc' => ' (id_stock_mvt) ' . $this->l('Separated by comma - ex : 11,17')
                ),
                $this->renderSwitch('KERAWEN_REPORTING_BUYSALE', $this->l('Reporting: display report Sales/Buy (PS 1.7)')),
                $this->renderSwitch('KERAWEN_LOW_PERF', $this->l('Reporting: add validate button')),
                $this->renderSwitch('KERAWEN_DEPOSIT_ACCOUNT', $this->l('Activate depositor account')),
                $this->renderSwitch('KERAWEN_ACCOUNT', $this->l('Activate customer prepaid account')),
                $this->renderSwitch('KERAWEN_ACCOUNT_FRONT', $this->l('Activate front-office payment with customer account')),
                $this->renderSwitch('KERAWEN_TICKET_PRINT_PREPAID', $this->l('Print prepaid amount on receipt')),
                array(
                    'name' => 'spacer',
                    'label' => $this->l('Limit amount'),
                    'type' => 'free',
                    'content' => '<hr />',
                ),
            );                        
            $input_2 = $this->getGroupCustAccount();
            $input_3 = array(
                $this->renderSwitch('KERAWEN_GROUP_AMOUNT_CUST', $this->l('Allow ajdustment by account')),
                array(
                    'name' => 'spacer2',
                    'type' => 'free',
                    'content' => '<br />',
                ),
                $this->renderSwitch('KERAWEN_UACA', $this->l('Update amount limit for all accounts')),
            );

            $forms['advanced'] = array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->getAnchor() . $this->l('Advanced settings'),
                        'icon' => 'icon-wrench',
                    ),
                    'input' => array_merge($input_1, $input_2, $input_3),
                    'submit' => $this->renderSubmit('submitConfig', $this->l('Save')),
                ),
            );
        }
        
        // Get the current parameter values
        $values = array();
        foreach ($forms as &$set)
            foreach ($set['form']['input'] as &$param)
            {
                $name = $param['name'];
                if ($param['type'] == 'free')
                {
                    $values[$name] = $param['content'];
                }
                else switch ($name)
                {
                    case 'KERAWEN_SHORTCUT_CATEGORIES':
                        break;
                    case 'KERAWEN_PREPAID_RELOAD':
                        $values[$name] = 'TEST';
                        break;
                        
                    default:
                        if (isset($param['lang'])) {
                            foreach ($this->_confLanguages as $language) {
                                $txt = Configuration::get($name, $language['id_lang'], 0, 0);
                                if (!$txt) {
                                    $txt = Configuration::get($name, 0, 0, 0);
                                }
                                $values[$name][$language['id_lang']] = $txt;
                            }
                        } else {
                            $values[$name] = Configuration::get($name, 0, 0, 0);
                        }
                }
            }
        
        // Render forms
        $this->setHelperDisplay(new HelperForm());
        $this->helper->tpl_vars = array(
            'fields_value' => $values,
            'id_language' => $this->context->language->id
        );
        
        return $this->helper->generateForm($forms);
    }
    
    protected function renderSubmit($name, $title)
    {
        $submit = array(
            'name' => $name,
            'title' => $title,
        );
        
        if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '<')) $submit['class'] = 'button';
        
        return $submit;
    }
    
    protected function externalLink($link, $txt = '') {
        $txt = $txt == '' ? $this->l('Online help') : $txt;
        return ' - <a href="' . $link . '" target="_blank"><i style="" class="' . $this->getIcon('icon-question-circle') . '"> </i> ' . $txt .  '</a>';
    }
    
    protected function getIcon($class = 'icon-wrench') {
        return $class;
    }
    
    protected function getAnchor() {
        return '';
        
        $this->$cnt++;
        return '<span id="C' . $this->$cnt . '"> #C' . $this->$cnt . '</span> - ';
    }
    
    protected function renderSwitch($name, $label, $desc = false)
    {
        
        if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '>='))
        {
            return array(
                'name' => $name,
                'label' => $label,
                'type' => 'switch',
                'is_bool' => true,
                'desc' => $desc,
                'values' => array(
                    array(
                        'id' => 'on',
                        'value' => 1,
                        'label' => $this->l('Yes'),
                    ),
                    array(
                        'id' => 'off',
                        'value' => 0,
                        'label' => $this->l('No'),
                    ),
                ),
            );
        }
        else
        {
            return array(
                'name' => $name,
                'label' => $label,
                'type' => 'radio',
                'class' => 't',
                'is_bool' => true,
                'desc' => $desc,
                'values' => array(
                    array(
                        'id' => $name.'_on',
                        'value' => 1,
                        'label' => $this->l('Enabled'),
                    ),
                    array(
                        'id' => $name.'_off',
                        'value' => 0,
                        'label' => $this->l('Disabled'),
                    ),
                ),
            );
        }
    }
    
    protected function renderShortcuts()
    {
        if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '>='))
        {
            return array(
                'name' => 'KERAWEN_SHORTCUT_CATEGORIES',
                'label' => $this->l('Quick view categories'),
                'type'  => 'categories',
                'tree'  => array(
                    'id' => 'KERAWEN_SHORTCUT_CATEGORIES',
                    'use_checkbox' => true,
                    'selected_categories' => explode(',', Configuration::get('KERAWEN_SHORTCUT_CATEGORIES')),
                ),
            );
        }
        else
        {
            $root_cat = Category::getTopCategory();
            
            return array(
                'name' => 'KERAWEN_SHORTCUT_CATEGORIES',
                'label' => $this->l('Quick view categories'),
                'type'  => 'categories',
                'values' => array(
                    'trads' => array(
                        'Root' => array(
                            'id_category' => $root_cat->id_category,
                            'name' => $root_cat->name,
                        ),
                        'selected' => $this->l('Selected'),
                        'Collapse All' => $this->l('Collapse All'),
                        'Expand All' => $this->l('Expand All'),
                        'Check All' => $this->l('Check All'),
                        'Uncheck All' => $this->l('Uncheck All'),
                    ),
                    'use_radio' => false,
                    'use_search' => false,
                    'input_name' => 'KERAWEN_SHORTCUT_CATEGORIES[]',
                    'selected_cat' => explode(',', Configuration::get('KERAWEN_SHORTCUT_CATEGORIES')),
                    'disabled_categories' => array($root_cat->id_category),
                    'top_category' => Category::getTopCategory(),
                ),
            );
        }
    }
    
    protected function getValueLanguages($key) {
        $list = array();
        foreach ($this->_confLanguages as $language) {
            $list[$language['id_lang']] = Tools::getValue($key . '_' . $language['id_lang']);
        }
        return $list;
    }
    
    public function postProcess()
    {
        $access = Profile::getProfileAccess($this->context->employee->id_profile, Tab::getIdFromClassName('KerawenConfig'));
        if ($access['edit']) {
            $backup = backupShopContext();
            Shop::setContext(Shop::CONTEXT_ALL);
            
            $infos = array();
            $errors = array();
            
            
            if (Tools::isSubmit('submitConfig')) {
                $this->processConfig($infos, $errors);
            }
            
            //KerawenFixesController.php
            if (Tools::isSubmit('submitWarnings')) {
                Configuration::updateValue('KERAWEN_IGNORE_WARNINGS', (int)Tools::getValue('KERAWEN_IGNORE_WARNINGS'));
            }
            
            
            // Report
            if (count($errors)) {
                $this->errors[] = implode('<br/>', $errors);
            }
            if (count($infos)) {
                $this->confirmations[] = implode('<br/>', $infos);
            }
            
            
            restoreShopContext($backup);
        }
        $this->display = 'view';
    }
    
    
    
    protected function processConfig(&$infos)
    {
        Configuration::updateValue('KERAWEN_NOTIF_PERIOD', (int)Tools::getValue('KERAWEN_NOTIF_PERIOD'));
        Configuration::updateValue('KERAWEN_DISCOUNT_DURATION', (int)Tools::getValue('KERAWEN_DISCOUNT_DURATION'));
        Configuration::updateValue('KERAWEN_VOUCHER_DURATION', (int)Tools::getValue('KERAWEN_VOUCHER_DURATION'));
        Configuration::updateValue('KERAWEN_SWITCH_CASHIER', (int)Tools::getValue('KERAWEN_SWITCH_CASHIER'));
        Configuration::updateValue('KERAWEN_VOUCHER_LABEL', Tools::getValue('KERAWEN_VOUCHER_LABEL'));
        Configuration::updateValue('KERAWEN_EMPLOYEE_PASSWORD', (int)Tools::getValue('KERAWEN_EMPLOYEE_PASSWORD'));
        Configuration::updateValue('KERAWEN_EMPLOYEE_PASSWORD_EXCP', (int)Tools::getValue('KERAWEN_EMPLOYEE_PASSWORD_EXCP'));
        Configuration::updateValue('KERAWEN_EMPLOYEE_ALWAYS_SELECT', (int)Tools::getValue('KERAWEN_EMPLOYEE_ALWAYS_SELECT'));
        Configuration::updateValue('KERAWEN_SWITCH_SHOP', (int)Tools::getValue('KERAWEN_SWITCH_SHOP'));
        Configuration::updateValue('KERAWEN_SELECT_DELIVERY', (int)Tools::getValue('KERAWEN_SELECT_DELIVERY'));
        Configuration::updateValue('KERAWEN_DEFAULT_DELIVERY_MODE', (int)Tools::getValue('KERAWEN_DEFAULT_DELIVERY_MODE'));
        Configuration::updateValue('KERAWEN_GATHER_MEASURES', (int)Tools::getValue('KERAWEN_GATHER_MEASURES'));
        Configuration::updateValue('KERAWEN_DEFAULT_GROUP', (int)Tools::getValue('KERAWEN_DEFAULT_GROUP'));
        Configuration::updateValue('KERAWEN_OVERRIDE_GROUP', (int)Tools::getValue('KERAWEN_OVERRIDE_GROUP'));
        Configuration::updateValue('KERAWEN_SHOW_AMOUNTS', (int)Tools::getValue('KERAWEN_SHOW_AMOUNTS'));
        Configuration::updateValue('KERAWEN_TICKET_XZ_EMPLOYEE', (int)Tools::getValue('KERAWEN_TICKET_XZ_EMPLOYEE'));
        Configuration::updateValue('KERAWEN_SEND_EMAIL', (int)Tools::getValue('KERAWEN_SEND_EMAIL'));
        Configuration::updateValue('KERAWEN_OVERRIDE_EMAIL', (int)Tools::getValue('KERAWEN_OVERRIDE_EMAIL'));
        Configuration::updateValue('KERAWEN_NEW_CUST_EMAIL_TPL', Tools::getValue('KERAWEN_NEW_CUST_EMAIL_TPL'));
        Configuration::updateValue('KERAWEN_CUSTOMER_PASSWORD', (int)Tools::getValue('KERAWEN_CUSTOMER_PASSWORD'));
        Configuration::updateValue('KERAWEN_CUST_OPTIN', (int)Tools::getValue('KERAWEN_CUST_OPTIN'));
        Configuration::updateValue('KERAWEN_DISPLAY_MARGIN', (int)Tools::getValue('KERAWEN_DISPLAY_MARGIN'));
        Configuration::updateValue('KERAWEN_ORDER_QUICK_END', (int)Tools::getValue('KERAWEN_ORDER_QUICK_END'));
        Configuration::updateValue('KERAWEN_AVOID_NEW_ORDER_ALERT', (int)Tools::getValue('KERAWEN_AVOID_NEW_ORDER_ALERT'));
        Configuration::updateValue('KERAWEN_NEW_MSG_AS_PRIVATE', (int)Tools::getValue('KERAWEN_NEW_MSG_AS_PRIVATE'));
        Configuration::updateValue('KERAWEN_THREAD_STATUS_ACTIVE', (int)Tools::getValue('KERAWEN_THREAD_STATUS_ACTIVE'));
        Configuration::updateValue('KERAWEN_CREDIT_PARTIAL_USE', (int)Tools::getValue('KERAWEN_CREDIT_PARTIAL_USE'));
        Configuration::updateValue('KERAWEN_CREDIT_CODE_DISPLAY', (int)Tools::getValue('KERAWEN_CREDIT_CODE_DISPLAY'));
        Configuration::updateValue('KERAWEN_ORDER_PULSE', (int)Tools::getValue('KERAWEN_ORDER_PULSE'));
        Configuration::updateValue('KERAWEN_PLAY_SOUND', (int)Tools::getValue('KERAWEN_PLAY_SOUND'));
        Configuration::updateValue('KERAWEN_SCAN_NOT_FOUND', (int)Tools::getValue('KERAWEN_SCAN_NOT_FOUND'));
        Configuration::updateValue('KERAWEN_PACK_SPLIT', (int)Tools::getValue('KERAWEN_PACK_SPLIT'));
        Configuration::updateValue('KERAWEN_NEW_PROD_REF_REQUIRED', (int)Tools::getValue('KERAWEN_NEW_PROD_REF_REQUIRED'));
        Configuration::updateValue('KERAWEN_NEW_PROD_WHOLESALE_PRICE', (int)Tools::getValue('KERAWEN_NEW_PROD_WHOLESALE_PRICE'));
        Configuration::updateValue('KERAWEN_DEPOSIT_WITH_FREE_PRICE', (int)Tools::getValue('KERAWEN_DEPOSIT_WITH_FREE_PRICE'));

        $this->processTypes();
        
        Configuration::updateValue('KERAWEN_ALERT_PROD_CART', (int)Tools::getValue('KERAWEN_ALERT_PROD_CART'));
        Configuration::updateValue('KERAWEN_VAT_WARNING', (int)Tools::getValue('KERAWEN_VAT_WARNING'));
        Configuration::updateValue('KERAWEN_DEFAULT_VAT', (int)Tools::getValue('KERAWEN_DEFAULT_VAT'));
        Configuration::updateValue('KERAWEN_CART_TAG', (int)Tools::getValue('KERAWEN_CART_TAG'));
        Configuration::updateValue('KERAWEN_MODE_0_ANONYMOUS', (int)Tools::getValue('KERAWEN_MODE_0_ANONYMOUS'));
        Configuration::updateValue('KERAWEN_MODE_1_ANONYMOUS', (int)Tools::getValue('KERAWEN_MODE_1_ANONYMOUS'));
        Configuration::updateValue('KERAWEN_MODE_2_ANONYMOUS', (int)Tools::getValue('KERAWEN_MODE_2_ANONYMOUS'));  
        if (Tools::getIsset('KERAWEN_ACCOUNT')) {
            Configuration::updateValue('KERAWEN_ACCOUNT', (int)Tools::getValue('KERAWEN_ACCOUNT'));
        }
        if (Tools::getIsset('KERAWEN_DEPOSIT_ACCOUNT')) {
            Configuration::updateValue('KERAWEN_DEPOSIT_ACCOUNT', (int)Tools::getValue('KERAWEN_DEPOSIT_ACCOUNT'));
        }
        if (Tools::getIsset('KERAWEN_ACCOUNT_FRONT')) {
            Configuration::updateValue('KERAWEN_ACCOUNT_FRONT', (int)Tools::getValue('KERAWEN_ACCOUNT_FRONT'));
        }
        if (Tools::getIsset('KERAWEN_TICKET_PRINT_PREPAID')) {
            Configuration::updateValue('KERAWEN_TICKET_PRINT_PREPAID', (int)Tools::getValue('KERAWEN_TICKET_PRINT_PREPAID'));
        }
        Configuration::updateValue('KERAWEN_MODE_0_NOTE', (int)Tools::getValue('KERAWEN_MODE_0_NOTE'));

        $shortcuts = array();
        $buf = Tools::getValue('KERAWEN_SHORTCUT_CATEGORIES');
        if (is_array($buf)) {
            foreach ($buf as $id_cat) {
                if ($id_cat) $shortcuts[] = $id_cat;
            }
        }
        Configuration::updateValue('KERAWEN_SHORTCUT_CATEGORIES', implode(',', $shortcuts));
        
        Configuration::updateValue('KERAWEN_CATALOG_PAGE_SIZE', (int)Tools::getValue('KERAWEN_CATALOG_PAGE_SIZE'));
        Configuration::updateValue('KERAWEN_ORDERS_LIST_ITEMS_BY_PAG', (int)Tools::getValue('KERAWEN_ORDERS_LIST_ITEMS_BY_PAG'));
        Configuration::updateValue('KERAWEN_CATALOG_FULL_NAMES', (int)Tools::getValue('KERAWEN_CATALOG_FULL_NAMES'));
        Configuration::updateValue('KERAWEN_WM_GROUP_EXC', (int)Tools::getValue('KERAWEN_WM_GROUP_EXC'));
        Configuration::updateValue('KERAWEN_CATEGORY_ACTIVE', (int)Tools::getValue('KERAWEN_CATEGORY_ACTIVE'));
        Configuration::updateValue('KERAWEN_CATALOG_REFERENCES', (int)Tools::getValue('KERAWEN_CATALOG_REFERENCES'));
        Configuration::updateValue('KERAWEN_CART_LOCATION', (int)Tools::getValue('KERAWEN_CART_LOCATION'));
        Configuration::updateValue('KERAWEN_CATALOG_MANUFACTURER', (int)Tools::getValue('KERAWEN_CATALOG_MANUFACTURER'));
        Configuration::updateValue('KERAWEN_CATALOG_INACTIVE', (int)Tools::getValue('KERAWEN_CATALOG_INACTIVE'));
        Configuration::updateValue('KERAWEN_CATALOG_PRODUCT_ORDER', (int)Tools::getValue('KERAWEN_CATALOG_PRODUCT_ORDER'));
        Configuration::updateValue('KERAWEN_CART_FULL_NAMES', (int)Tools::getValue('KERAWEN_CART_FULL_NAMES'));
        Configuration::updateValue('KERAWEN_CART_REF', (int)Tools::getValue('KERAWEN_CART_REF'));
        Configuration::updateValue('KERAWEN_CART_EAN', (int)Tools::getValue('KERAWEN_CART_EAN'));
        Configuration::updateValue('KERAWEN_ORDERS_LIST_COLUMN_SHOP', (int)Tools::getValue('KERAWEN_ORDERS_LIST_COLUMN_SHOP'));
        Configuration::updateValue('KERAWEN_ORDERS_LIST_COLUMN_EMPL', (int)Tools::getValue('KERAWEN_ORDERS_LIST_COLUMN_EMPL'));
        Configuration::updateValue('KERAWEN_ORDERS_LIST_COLUMN_CARRI', (int)Tools::getValue('KERAWEN_ORDERS_LIST_COLUMN_CARRI'));
        Configuration::updateValue('KERAWEN_ORDERS_LIST_COLUMN_TILL', (int)Tools::getValue('KERAWEN_ORDERS_LIST_COLUMN_TILL'));
        Configuration::updateValue('KERAWEN_ORDERS_LIST_COLUMN_COMP', (int)Tools::getValue('KERAWEN_ORDERS_LIST_COLUMN_COMP'));
        Configuration::updateValue('KERAWEN_ORDERS_LIST_COLUMN_ADD', (int)Tools::getValue('KERAWEN_ORDERS_LIST_COLUMN_ADD'));
        Configuration::updateValue('KERAWEN_ORDERS_LIST_COLUMN_PAY', (int)Tools::getValue('KERAWEN_ORDERS_LIST_COLUMN_PAY'));
        Configuration::updateValue('KERAWEN_CUST_ACCOUNT_ADDR', (int)Tools::getValue('KERAWEN_CUST_ACCOUNT_ADDR'));
        Configuration::updateValue('KERAWEN_IMAGE_PRODUCT', Tools::getValue('KERAWEN_IMAGE_PRODUCT'));
        Configuration::updateValue('KERAWEN_IMAGE_CATEGORY', Tools::getValue('KERAWEN_IMAGE_CATEGORY'));
        Configuration::updateValue('KERAWEN_CARTRULE_INCL_GENERIC', (int)Tools::getValue('KERAWEN_CARTRULE_INCL_GENERIC'));
        Configuration::updateValue('KERAWEN_CARTRULE_ORDER_BY_DATE', (int)Tools::getValue('KERAWEN_CARTRULE_ORDER_BY_DATE'));
        Configuration::updateValue('KERAWEN_UPC_SEARCH', (int)Tools::getValue('KERAWEN_UPC_SEARCH'));
        
        
        
        $cpl = (int)Tools::getValue('KERAWEN_DISPLAY_CPL');
        if ($cpl > 0) {
            Configuration::updateValue('KERAWEN_DISPLAY_CPL', $cpl);
        } else {
            $errors[] = $this->l('Invalid characters per line');
        }
        Configuration::updateValue('KERAWEN_DISPLAY_MSG_START', Tools::getValue('KERAWEN_DISPLAY_MSG_START'));
        Configuration::updateValue('KERAWEN_DISPLAY_MSG_END', Tools::getValue('KERAWEN_DISPLAY_MSG_END'));
        
        
        $to_unlink = null;
        
        $cpl = (int)Tools::getValue('KERAWEN_TICKET_CPL');
        if ($cpl > 0) {
            Configuration::updateValue('KERAWEN_TICKET_CPL', $cpl);
        }
        else {
            $errors[] = $this->l('Invalid characters per line');
        }
            
        if ($_FILES['KERAWEN_TICKET_IMAGE_UPLOAD'] && $_FILES['KERAWEN_TICKET_IMAGE_UPLOAD']['tmp_name']) {
            $file = $_FILES['KERAWEN_TICKET_IMAGE_UPLOAD'];
            $res = ImageManager::validateUpload($file);
            if (!$res) {
                // It is valid!
                // Use temp name to avoid browser cache
                $tmp = tempnam(_PS_IMG_DIR_, 'tic');
                $img = pathinfo($tmp, PATHINFO_FILENAME).'.'.pathinfo($file['name'], PATHINFO_EXTENSION);
                if (move_uploaded_file($file['tmp_name'], _PS_IMG_DIR_.$img)) {
                    $to_unlink = Configuration::get('KERAWEN_TICKET_IMAGE');
                    Configuration::updateValue('KERAWEN_TICKET_IMAGE', $img);
                }
                else {
                    $errors[] = $this->l('Cannot upload image');
                }
                unlink($tmp);
            }
            else {
                $errors[] = $res;
            }
        }
        else if (Tools::getValue('KERAWEN_TICKET_IMAGE_DELETE')) {
            $to_unlink = Configuration::get('KERAWEN_TICKET_IMAGE');
            Configuration::updateValue('KERAWEN_TICKET_IMAGE', null);
        }
        if ($to_unlink) unlink(_PS_IMG_DIR_.$to_unlink);
                
        Configuration::updateValue('KERAWEN_TICKET_SHOP_COUNTRY', (int)Tools::getValue('KERAWEN_TICKET_SHOP_COUNTRY'));
        Configuration::updateValue('KERAWEN_TICKET_SHOP_URL', (int)Tools::getValue('KERAWEN_TICKET_SHOP_URL'));
        Configuration::updateValue('KERAWEN_TICKET_SHOP_EMAIL', (int)Tools::getValue('KERAWEN_TICKET_SHOP_EMAIL'));
        Configuration::updateValue('KERAWEN_TICKET_SHOP_PHONE', (int)Tools::getValue('KERAWEN_TICKET_SHOP_PHONE'));
        Configuration::updateValue('KERAWEN_TICKET_SHOP_DETAILS', (int)Tools::getValue('KERAWEN_TICKET_SHOP_DETAILS'));
        Configuration::updateValue('KERAWEN_TICKET_COMMENTS', (int)Tools::getValue('KERAWEN_TICKET_COMMENTS'));
        Configuration::updateValue('KERAWEN_TICKET_COMMENTS_TOP', (int)Tools::getValue('KERAWEN_TICKET_COMMENTS_TOP'));
        Configuration::updateValue('KERAWEN_TICKET_PRODUCT_NOTE', (int)Tools::getValue('KERAWEN_TICKET_PRODUCT_NOTE'));
        Configuration::updateValue('KERAWEN_TICKET_FULL_NAMES', (int)Tools::getValue('KERAWEN_TICKET_FULL_NAMES'));
        Configuration::updateValue('KERAWEN_TICKET_TAXES', (int)Tools::getValue('KERAWEN_TICKET_TAXES'));
        Configuration::updateValue('KERAWEN_TICKET_DETAIL_TAXES', (int)Tools::getValue('KERAWEN_TICKET_DETAIL_TAXES'));
        Configuration::updateValue('KERAWEN_TICKET_MODE', (int)Tools::getValue('KERAWEN_TICKET_MODE'));
        Configuration::updateValue('KERAWEN_TICKET_CUSTOMER', (int)Tools::getValue('KERAWEN_TICKET_CUSTOMER'));
        Configuration::updateValue('KERAWEN_TICKET_CUSTOMER_PHONE', (int)Tools::getValue('KERAWEN_TICKET_CUSTOMER_PHONE'));
        Configuration::updateValue('KERAWEN_TICKET_REF', (int)Tools::getValue('KERAWEN_TICKET_REF'));
        Configuration::updateValue('KERAWEN_TICKET_SUBTOTAL_DISCOUNT', (int)Tools::getValue('KERAWEN_TICKET_SUBTOTAL_DISCOUNT'));
        Configuration::updateValue('KERAWEN_TICKET_EMPLOYEE_NAME', (int)Tools::getValue('KERAWEN_TICKET_EMPLOYEE_NAME'));
        Configuration::updateValue('KERAWEN_TICKET_LOYALTY', (int)Tools::getValue('KERAWEN_TICKET_LOYALTY'));
        Configuration::updateValue('KERAWEN_TICKET_MESSAGE', Tools::getValue('KERAWEN_TICKET_MESSAGE'));
        Configuration::updateValue('KERAWEN_TICKET_BARCODE', (int)Tools::getValue('KERAWEN_TICKET_BARCODE'));
        Configuration::updateValue('KERAWEN_TICKET_ORDER_NUMBER', (int)Tools::getValue('KERAWEN_TICKET_ORDER_NUMBER'));
        Configuration::updateValue('KERAWEN_TICKET_PRINT_OPEN_CLOSE', (int)Tools::getValue('KERAWEN_TICKET_PRINT_OPEN_CLOSE'));
        Configuration::updateValue('KERAWEN_TICKET_PRINT_MIN_AMOUNT', Tools::getValue('KERAWEN_TICKET_PRINT_MIN_AMOUNT') == '' ? '' : (float) Tools::getValue('KERAWEN_TICKET_PRINT_MIN_AMOUNT'));
        Configuration::updateValue('KERAWEN_TPE_AUTO', (int)Tools::getValue('KERAWEN_TPE_AUTO'));
        Configuration::updateValue('KERAWEN_TICKET_PRINT_AUTO', (int)Tools::getValue('KERAWEN_TICKET_PRINT_AUTO'));
        Configuration::updateValue('KERAWEN_TICKET_MSG_DISCOUNT', Tools::getValue('KERAWEN_TICKET_MSG_DISCOUNT'));
        Configuration::updateValue('KERAWEN_TICKET_PRINT_SHOP_NAME', (int)Tools::getValue('KERAWEN_TICKET_PRINT_SHOP_NAME'));
        Configuration::updateValue('KERAWEN_TICKET_QTY_LEFT', (int)Tools::getValue('KERAWEN_TICKET_QTY_LEFT'));
        Configuration::updateValue('KERAWEN_TICKET_COUNT_SALE', (int)Tools::getValue('KERAWEN_TICKET_COUNT_SALE'));
        Configuration::updateValue('KERAWEN_TICKET_LOYALTY_PRINT', (int)Tools::getValue('KERAWEN_TICKET_LOYALTY_PRINT'));
        
        Configuration::updateValue('KERAWEN_SCALE_PREFIX', Tools::getValue('KERAWEN_SCALE_PREFIX'));
        Configuration::updateValue('KERAWEN_SCALE_PRODUCT_LENGTH', (int)Tools::getValue('KERAWEN_SCALE_PRODUCT_LENGTH'));
        Configuration::updateValue('KERAWEN_SCALE_PRICE_LENGTH', (int)Tools::getValue('KERAWEN_SCALE_PRICE_LENGTH'));
        Configuration::updateValue('KERAWEN_SCALE_PRICE_MULTIPLIER', (float)Tools::getValue('KERAWEN_SCALE_PRICE_MULTIPLIER'));
        
        Configuration::updateValue('KERAWEN_QUOTE_ACTIVE', Tools::getValue('KERAWEN_QUOTE_ACTIVE'));
        Configuration::updateValue('KERAWEN_QUOTE_PRODUCT_NOTE', Tools::getValue('KERAWEN_QUOTE_PRODUCT_NOTE'));
        Configuration::updateValue('KERAWEN_QUOTE_DURATION', (int)Tools::getValue('KERAWEN_QUOTE_DURATION'));
        Configuration::updateValue('KERAWEN_QUOTE_MESSAGE',   $this->getValueLanguages('KERAWEN_QUOTE_MESSAGE'));
        Configuration::updateValue('KERAWEN_QUOTE_MESSAGE_2', $this->getValueLanguages('KERAWEN_QUOTE_MESSAGE_2'));
        Configuration::updateValue('KERAWEN_QUOTE_MESSAGE_3', $this->getValueLanguages('KERAWEN_QUOTE_MESSAGE_3'));
        Configuration::updateValue('KERAWEN_QUOTE_MESSAGE_4', $this->getValueLanguages('KERAWEN_QUOTE_MESSAGE_4'));
        Configuration::updateValue('KERAWEN_QUOTE_TAX', (int)Tools::getValue('KERAWEN_QUOTE_TAX'));
        Configuration::updateValue('KERAWEN_QUOTE_DISP_TAX', (int)Tools::getValue('KERAWEN_QUOTE_DISP_TAX'));
        Configuration::updateValue('KERAWEN_QUOTE_DISP_UNIT_VAT', (int)Tools::getValue('KERAWEN_QUOTE_DISP_UNIT_VAT'));
        Configuration::updateValue('KERAWEN_QUOTE_DISP_TOTAL_VAT', (int)Tools::getValue('KERAWEN_QUOTE_DISP_TOTAL_VAT'));
        Configuration::updateValue('KERAWEN_QUOTE_REF_COL', (int)Tools::getValue('KERAWEN_QUOTE_REF_COL'));
        Configuration::updateValue('KERAWEN_QUOTE_IMG', (int)Tools::getValue('KERAWEN_QUOTE_IMG'));
        
        Configuration::updateValue('KERAWEN_LABEL_ITEMS_BY_PAGE', (int)Tools::getValue('KERAWEN_LABEL_ITEMS_BY_PAGE') );
        
        Configuration::updateValue('KERAWEN_DECIMAL_SEPARATOR', Tools::getValue('KERAWEN_DECIMAL_SEPARATOR') );
        
        Configuration::updateValue('KERAWEN_CUST_PRINT', (int)Tools::getValue('KERAWEN_CUST_PRINT') );
        Configuration::updateValue('KERAWEN_CUST_CARD', (int)Tools::getValue('KERAWEN_CUST_CARD') );
        Configuration::updateValue('KERAWEN_CUST_HEADER_MESSAGE', Tools::getValue('KERAWEN_CUST_HEADER_MESSAGE') );
        Configuration::updateValue('KERAWEN_CUST_FOOTER_MESSAGE', Tools::getValue('KERAWEN_CUST_FOOTER_MESSAGE') );
        
        Configuration::updateValue('KERAWEN_ADDRESS1_REQUIRED', (int)Tools::getValue('KERAWEN_ADDRESS1_REQUIRED') );
        Configuration::updateValue('KERAWEN_CITY_REQUIRED', (int)Tools::getValue('KERAWEN_CITY_REQUIRED') );
        Configuration::updateValue('KERAWEN_POSTCODE_REQUIRED', (int)Tools::getValue('KERAWEN_POSTCODE_REQUIRED') );
        Configuration::updateValue('KERAWEN_INVOICE_FREE_TEXT', $this->getValueLanguages('KERAWEN_INVOICE_FREE_TEXT'), true );
        Configuration::updateValue('KERAWEN_INVOICE_TAX', (int)Tools::getValue('KERAWEN_INVOICE_TAX') );
        Configuration::updateValue('KERAWEN_INVOICE_NUM_ORDER', (int)Tools::getValue('KERAWEN_INVOICE_NUM_ORDER') );
        Configuration::updateValue('KERAWEN_INVOICE_NUM_CART', (int)Tools::getValue('KERAWEN_INVOICE_NUM_CART') );
        Configuration::updateValue('KERAWEN_INVOICE_DISP_TAX', (int)Tools::getValue('KERAWEN_INVOICE_DISP_TAX') );
        Configuration::updateValue('KERAWEN_INVOICE_DISP_SHIPPING', (int)Tools::getValue('KERAWEN_INVOICE_DISP_SHIPPING') );
        Configuration::updateValue('KERAWEN_INVOICE_DISP_UNIT_VAT', (int)Tools::getValue('KERAWEN_INVOICE_DISP_UNIT_VAT') );
        Configuration::updateValue('KERAWEN_INVOICE_DISP_TOTAL_VAT', (int)Tools::getValue('KERAWEN_INVOICE_DISP_TOTAL_VAT') );
        Configuration::updateValue('KERAWEN_INVOICE_DISP_BARCODE', (int)Tools::getValue('KERAWEN_INVOICE_DISP_BARCODE') );
        Configuration::updateValue('KERAWEN_INVOICE_REF_COL', (int)Tools::getValue('KERAWEN_INVOICE_REF_COL') );
        Configuration::updateValue('KERAWEN_INVOICE_DISP_ONLY_COMP', (int)Tools::getValue('KERAWEN_INVOICE_DISP_ONLY_COMP') );
        Configuration::updateValue('KERAWEN_INVOICE_HEADER_DATE', (int)Tools::getValue('KERAWEN_INVOICE_HEADER_DATE') );
        Configuration::updateValue('KERAWEN_POSTCODE_REQUIRED', (int)Tools::getValue('KERAWEN_POSTCODE_REQUIRED') );
        Configuration::updateValue('KERAWEN_ADDRESS1_REQUIRED', (int)Tools::getValue('KERAWEN_ADDRESS1_REQUIRED') );
        Configuration::updateValue('KERAWEN_CITY_REQUIRED', (int)Tools::getValue('KERAWEN_CITY_REQUIRED') );
        Configuration::updateValue('KERAWEN_PHONE_REQUIRED', (int)Tools::getValue('KERAWEN_PHONE_REQUIRED') );
        Configuration::updateValue('KERAWEN_MOBILE_REQUIRED', (int)Tools::getValue('KERAWEN_MOBILE_REQUIRED') );
        Configuration::updateValue('KERAWEN_BIRTHDATE_REQUIRED', (int)Tools::getValue('KERAWEN_BIRTHDATE_REQUIRED') );
                
        Configuration::updateValue('KERAWEN_GIFT_CARD_DURATION', (int)Tools::getValue('KERAWEN_GIFT_CARD_DURATION') );
        Configuration::updateValue('KERAWEN_GIFT_CARD_CODE_DISPLAY', (int)Tools::getValue('KERAWEN_GIFT_CARD_CODE_DISPLAY') );
        Configuration::updateValue('KERAWEN_CUST_SEND_GIFTCARD', (int)Tools::getValue('KERAWEN_CUST_SEND_GIFTCARD') );
        
        Configuration::updateValue('KERAWEN_GIFT_CARD_TICKET_MESSAGE', Tools::getValue('KERAWEN_GIFT_CARD_TICKET_MESSAGE') );
        Configuration::updateValue('KERAWEN_GIFT_CARD_JS', Tools::getValue('KERAWEN_GIFT_CARD_JS') );
        
        Configuration::updateValue('KERAWEN_LABEL_IN_STORE', Tools::getValue('KERAWEN_LABEL_IN_STORE'));
        Configuration::updateValue('KERAWEN_LABEL_TAKEAWAY', Tools::getValue('KERAWEN_LABEL_TAKEAWAY'));
        Configuration::updateValue('KERAWEN_LABEL_DELIVERY', Tools::getValue('KERAWEN_LABEL_DELIVERY'));
        
        Configuration::updateValue('KERAWEN_DAILY_TOTAL', Tools::getValue('KERAWEN_DAILY_TOTAL'));
        
        Configuration::updateValue('KERAWEN_NOTE_SHORT_DESC', Tools::getValue('KERAWEN_NOTE_SHORT_DESC'));
        
        $this->processPayment();
        Configuration::updateValue('KERAWEN_MAX_REST_VOUCHER', Tools::getValue('KERAWEN_MAX_REST_VOUCHER'));
        
        $this->processMode();
        
        $this->processTillStatus();
        
        Configuration::updateValue('KERAWEN_PAYMENT_MODE', (int)Tools::getValue('KERAWEN_PAYMENT_MODE'));

        $this->processCarrierMode();
        
        if (Tools::getIsset('KERAWEN_THEME')) {
            Configuration::updateValue('KERAWEN_THEME', Tools::getValue('KERAWEN_THEME'));
        }
        
        if (Tools::getIsset('KERAWEN_MANAGE_PO')) {
            Configuration::updateValue('KERAWEN_MANAGE_PO', (int)Tools::getValue('KERAWEN_MANAGE_PO') );
        }
        
        if (Tools::getIsset('KERAWEN_INVOICE_AUTO')) {
            Configuration::updateValue('KERAWEN_INVOICE_AUTO', (int)Tools::getValue('KERAWEN_INVOICE_AUTO') );
        }
        
        if (Tools::getIsset('KERAWEN_FORCE_VIRTUAL_SHOP')) {
            Configuration::updateValue('KERAWEN_FORCE_VIRTUAL_SHOP', (int)Tools::getValue('KERAWEN_FORCE_VIRTUAL_SHOP') );
        }

        if (Tools::getIsset('KERAWEN_TICKET_REF_NAME')) {
            Configuration::updateValue('KERAWEN_TICKET_REF_NAME', (int)Tools::getValue('KERAWEN_TICKET_REF_NAME') );
        }

        if (Tools::getIsset('KERAWEN_DISPLAY_STOCKS_RES')) {
            Configuration::updateValue('KERAWEN_DISPLAY_STOCKS_RES', (int)Tools::getValue('KERAWEN_DISPLAY_STOCKS_RES') );
        }
        
        if (Tools::getIsset('KERAWEN_LOW_PERF')) {
            Configuration::updateValue('KERAWEN_LOW_PERF', (int)Tools::getValue('KERAWEN_LOW_PERF') );
        }
        
        if (Tools::getIsset('KERAWEN_REPORTING_BUYSALE')) {
            Configuration::updateValue('KERAWEN_REPORTING_BUYSALE', (int)Tools::getValue('KERAWEN_REPORTING_BUYSALE') );
        }
        
        if (Tools::getIsset('KERAWEN_MVT_BUY')) {
            Configuration::updateValue('KERAWEN_MVT_BUY', Tools::getValue('KERAWEN_MVT_BUY') );
        }

        if (Tools::getIsset('KERAWEN_RESTRICTIVE_MODE')) {
            Configuration::updateValue('KERAWEN_RESTRICTIVE_MODE', Tools::getValue('KERAWEN_RESTRICTIVE_MODE'));
        }

        if (Tools::getIsset('KERAWEN_KEEP_VIEWS')) {
            Configuration::updateValue('KERAWEN_KEEP_VIEWS', Tools::getValue('KERAWEN_KEEP_VIEWS'));
        }
        
        if (Tools::getIsset('KERAWEN_SEARCH_PROD_BY_VIEW')) {
            Configuration::updateValue('KERAWEN_SEARCH_PROD_BY_VIEW', Tools::getValue('KERAWEN_SEARCH_PROD_BY_VIEW'));
        }

        if (Tools::getIsset('KERAWEN_OLD_REPORTING')) {
            Configuration::updateValue('KERAWEN_OLD_REPORTING', Tools::getValue('KERAWEN_OLD_REPORTING'));
        }
        
        if (Tools::getIsset('KERAWEN_TILL_SHOP')) {
            Configuration::updateValue('KERAWEN_TILL_SHOP', Tools::getValue('KERAWEN_TILL_SHOP'));
        }
             
        $this->processGroupsAmount();

        // Register this configuration as the latest one
        Configuration::updateValue('KERAWEN_CONFIG', time());
        $infos[] = $this->l('Settings updated') . '<br /><b>' . $this->l('Please restart cash register') . '</b>';
    }

    
    protected function getStatus($name, $sel) {

        $checked = 'checked="checked"';
        
        if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '>='))
        {
            $content = '
        		<div class="col-lg-3">
				    <span class="switch prestashop-switch fixed-width-lg">
				        <input type="radio" name="'.$name.'" id="'.$name.'_on" value="1"'.($sel ? $checked : '').'>
				        <label for="'.$name.'_on">'.$this->l('Yes').'</label>
				        <input type="radio" name="'.$name.'" id="'.$name.'_off" value="0"'.($sel ? '' : $checked).'>
				        <label for="'.$name.'_off">'.$this->l('No').'</label>
				        <a class="slide-button btn"></a>
                    </span>
                </div>';

        } else {
            $content = '
                <div class="kerawen-config-payment-block">
    				<input type="radio" name="'.$name.'" id="'.$name.'_on" value="1"'.($sel ? $checked : '').'>
    				<label class="t" for="'.$name.'_on">
    					<img src="../img/admin/enabled.gif" alt="Activé" title="Activé">
    				</label>
    				<input type="radio" name="'.$name.'" id="'.$name.'_off" value="0"'.($sel ? '' : $checked).'>
    				<label class="t" for="'.$name.'_off">
    					<img src="../img/admin/disabled.gif" alt="Désactivé" title="Désactivé">
    				</label>
                </div>
                ';
        }
        
        return $content;
    }
    

    protected function renderStatusBlock() {

        $status_till = json_decode(Configuration::get('KERAWEN_STATUS_TILL'), true);
        $notif_till = json_decode(Configuration::get('KERAWEN_NOTIF_TILL'), true);       
        $status_list = OrderState::getOrderStates($this->context->language->id);

        $input = array();
        
        if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '>='))
        {
            $header = array(
                'name' => '_KERAWEN_S_LIST_',
                'type' => 'free',
                'content' => '
					<div class="col-lg-3">
						<div style="text-align:center">'.$this->l('Till available status').'</div>
					</div>
					<div class="col-lg-3">
						<div style="text-align:center">'.$this->l('Till notification').'</div>
					</div>',
            );
        }
        else
        {
            $header = array(
                'name' => '_KERAWEN_S_LIST_',
                'type' => 'free',
                'content' => '
					<div class="kerawen-config-payment-block">
						<span>'.$this->l('Till available status').'</span>
					</div>
					<div class="kerawen-config-payment-block">
						<span>'.$this->l('Till notification').'</span>
					</div>',
            );
        }
        $input[] = $header;
        

        $key = 'KERAWEN_STATUS_TILL_';
        $key_2 = 'KERAWEN_NOTIF_TILL_';
        
        foreach($status_list as $item) {

            $sel = 1;
            $sel_2 = 0;
            if ($status_till) {
                if (isset($status_till[$item['id_order_state']])) {
                    $sel = (int) $status_till[$item['id_order_state']]['active'];
                }
            }
            if ($notif_till) {
                if (isset($notif_till[$item['id_order_state']])) {
                    $sel_2 = (int) $notif_till[$item['id_order_state']]['active'];
                }
            }

            $input[] = array(
                'name' => $key . $item['id_order_state'],
                'label' => $item['name'],
                'type' => 'free',
                'content' => $this->getStatus($key . $item['id_order_state'], $sel) . $this->getStatus($key_2 . $item['id_order_state'], $sel_2),
            );
        }
        return $input;
    }
 

    protected function getModes($name, $val, $list) {
        
        $select = '<select name="' . $name . '" id="' . $name . '">';
        foreach($list as $item) {
            $sel = $item['id_mode'] == (int) $val ? 'selected = "selected" ':'';
            $select .= '<option ' . $sel .' value="' . $item['id_mode'] . '">' . $item['name'] . '</option>';
        }
        $select .= '</select>';

        return $this->renderSelect($select);
    }
    
    protected function getCarrierMode()
    {
        $context = Context::getContext();
        
        $list = array(
            array(
                'id_mode' => 0,
                'name' => Configuration::get('KERAWEN_LABEL_IN_STORE') ? Configuration::get('KERAWEN_LABEL_IN_STORE') : $this->l('In store'),
            ),
            array(
                'id_mode' => 1,
                'name' => Configuration::get('KERAWEN_LABEL_TAKEAWAY') ? Configuration::get('KERAWEN_LABEL_TAKEAWAY') : $this->l('Takeaway'),
            ),
            array(
                'id_mode' => 2,
                'name' => Configuration::get('KERAWEN_LABEL_DELIVERY') ? Configuration::get('KERAWEN_LABEL_DELIVERY') : $this->l('Delivery'),
            ),
        );
        
        $carrier_mode = unserialize(Configuration::get('KERAWEN_CARRIER_MODE'));
        
        $carriers = array();
        $buf = Carrier::getCarriers($context->language->id);
        foreach ($buf as $c) {
            $mode = 2;
            if (is_array($carrier_mode) && isset($carrier_mode[$c['id_reference']])) {
                $mode = $carrier_mode[$c['id_reference']];
            }

            $carriers[$c['id_reference']] = array(
                'name' => 'KERAWEN_CARRIER_MODE_' . $c['id_reference'],
                'label' => $c['name'],
                'type' => 'free',
                'content' => $this->getModes('KERAWEN_CARRIER_MODE[' . $c['id_reference'] . ']', $mode, $list),
            );
        }

        return  $carriers;
    }
  
    protected function getGroupCustAccount()
    {
        $context = Context::getContext();

        $groups_amount = unserialize(Configuration::get('KERAWEN_GROUP_AMOUNT'));
        
        $groups = array();
        $buf = Group::getGroups($this->context->language->id);
        foreach ($buf as $g) {
            $amount = '0';
            if (is_array($groups_amount) && isset($groups_amount[$g['id_group']])) {
                $amount = is_numeric($groups_amount[$g['id_group']]) ? $groups_amount[$g['id_group']] : '0';
            }
            $groups[] = array(
                'name' => 'KERAWEN_GROUP_AMOUNT_' . $g['id_group'],
                'label' => $g['name'],
                'type' => 'free',
                'content' => '<input style="width:150px;" type="text" name="KERAWEN_GROUP_AMOUNT[' . $g['id_group'] . ']" placeholder="0" value="' . $amount . '" />',
            );
        }
        return $groups;
    }
    
    protected function getGiftCardCronUrl()
    {
        $id_lang = null;
        $id_shop = null;
        $link = new Link;
        return $link->getPageLink('cron', null, $id_lang, 'fc=module&module=kerawen&action=giftcard&key=' . Configuration::get('KERAWEN_CRON_KEY'), false, $id_shop);
    }
    
    protected function getOrderFeedUrl()
    {
        $id_lang = null;
        $id_shop = null;
        $link = new Link;
        return $link->getPageLink('feed', null, $id_lang, 'fc=module&module=kerawen&action=feed&key=' . Configuration::get('KERAWEN_ORDER_FEED_KEY'), false, $id_shop);
    }
    
    protected function getCategories()
    {
        $context = Context::getContext();
        return Db::getInstance()->executeS('
			SELECT cl.id_category AS id, cl.name AS name
			FROM '._DB_PREFIX_.'category_lang cl
			WHERE id_lang = '.pSQL($context->language->id));
    }
    
    

    protected function renderPaymentBlock()
    {
        $input = array();

        $input[] = array(
            'name' => 'spacer3',
            'label' => '',
            'type' => 'free',
            'content' => '<strong>' . $this->l('Changes will be applied to all employee profiles') . '</strong>',
        );

        
        if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '>='))
        {
            $header = array(
                'name' => '_KERAWEN_PM_LIST_',
                'type' => 'free',
                'content' => '
					<div class="col-lg-3">
						<div style="text-align:center">'.$this->l('Payment').'</div>
					</div>
					<div class="col-lg-3">
						<div style="text-align:center">'.$this->l('Refunding').'</div>
					</div>
                    <div class="col-lg-3">
                        <div style="text-align:center">'.$this->l('Max amount').'</div>
                    </div>',
            );
        }
        else
        {
            $header = array(
                'name' => '_KERAWEN_PM_LIST_',
                'type' => 'free',
                'content' => '
					<div class="kerawen-config-payment-block">
						<span>'.$this->l('Payment').'</span>
					</div>
					<div class="kerawen-config-payment-block">
						<span>'.$this->l('Refunding').'</span>
					</div>
					<div class="kerawen-config-payment-block">
						<span>'.$this->l('Max amount').'</span>
					</div>',
            );
        }
        $input[] = $header;
        
        $cfg = json_decode(Configuration::get('KERAWEN_PAYMENTS'), true);
        $modes = KerawenPayment::getModes($this->module);
        $checked = ' checked="checked"';
        foreach ($modes as $mode)
        {
            $id = (int)$mode['id'];
            $name = 'KERAWEN_PM_'.$id;
            $label = isset($cfg[$id]['label']) ? $cfg[$id]['label'] : $mode['label'];
            $payment = isset($cfg[$id]['payment']) ? $cfg[$id]['payment'] : $mode['payment'];
            $refund = isset($cfg[$id]['refund']) ? $cfg[$id]['refund'] : $mode['refund'];
            
            $amount_limit = '';
            if ($id == 25) {
                $amount_limit = '<div class="input-group"><input type="text" class="" name="KERAWEN_MAX_REST_VOUCHER" value="'.Configuration::get('KERAWEN_MAX_REST_VOUCHER').'" /><span class="input-group-addon">' . $this->context->currency->prefix.$this->context->currency->suffix . '</span></div>';
            } 
            
            if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '>='))
            {
                $content = '
					<div class="col-lg-3">
						<span class="switch prestashop-switch fixed-width-lg">
							<input type="radio" name="'.$name.'_PAYMENT" id="'.$name.'_PAYMENT_on" value="1"'.($payment ? $checked : '').'>
							<label for="'.$name.'_PAYMENT_on">'.$this->l('Yes').'</label>
							<input type="radio" name="'.$name.'_PAYMENT" id="'.$name.'_PAYMENT_off" value="0"'.($payment ? '' : $checked).'>
							<label for="'.$name.'_PAYMENT_off">'.$this->l('No').'</label>
							<a class="slide-button btn"></a>
						</span>
					</div>
					<div class="col-lg-3">
						<span class="switch prestashop-switch fixed-width-lg">
							<input type="radio" name="'.$name.'_REFUND" id="'.$name.'_REFUND_on" value="1"'.($refund ? $checked : '').'>
							<label for="'.$name.'_REFUND_on">'.$this->l('Yes').'</label>
							<input type="radio" name="'.$name.'_REFUND" id="'.$name.'_REFUND_off" value="0"'.($refund ? '' : $checked).'>
							<label for="'.$name.'_REFUND_off">'.$this->l('No').'</label>
							<a class="slide-button btn"></a>
						</span>
					</div>
                    <div class="col-lg-2">
                        ' . $amount_limit . '
                    </div>';
            }
            else
            {
                $content = '
					<div class="kerawen-config-payment-block">
						<input type="radio" name="'.$name.'_PAYMENT" id="'.$name.'_PAYMENT_on" value="1"'.($payment ? $checked : '').'>
						<label class="t" for="'.$name.'_PAYMENT_on">
							<img src="../img/admin/enabled.gif" alt="Activé" title="Activé">
						</label>
						<input type="radio" name="'.$name.'_PAYMENT" id="'.$name.'_PAYMENT_off" value="0"'.($payment ? '' : $checked).'>
						<label class="t" for="'.$name.'_PAYMENT_off">
							<img src="../img/admin/disabled.gif" alt="Désactivé" title="Désactivé">
						</label>
					</div>
					<div class="kerawen-config-payment-block">
						<input type="radio" name="'.$name.'_REFUND" id="'.$name.'_REFUND_on" value="1"'.($refund ? $checked : '').'>
						<label class="t" for="'.$name.'_REFUND_on">
							<img src="../img/admin/enabled.gif" alt="Activé" title="Activé">
						</label>
						<input type="radio" name="'.$name.'_REFUND" id="'.$name.'_REFUND_off" value="0"'.($refund ? '' : $checked).'>
						<label class="t" for="'.$name.'_REFUND_off">
							<img src="../img/admin/disabled.gif" alt="Désactivé" title="Désactivé">
						</label>
					</div>
                    <div class="kerawen-config-payment-block">
                        ' . $amount_limit . '
                    </div>';
            }
            
            if ($id >= _KERAWEN_PM_OTHER1_ && $id <= _KERAWEN_PM_OTHER11_)
                $label = '<input type="text" class="kerawen-config-payment-input" name="'.$name.'_LABEL" value="'.$label.'"/>';
                
            $row = array(
                'name' => $name,
                'label' => $label,
                'type' => 'free',
                'content' => $content,
            );
            if ($id >= _KERAWEN_PM_OTHER1_ && $id <= _KERAWEN_PM_OTHER11_) $row['class'] = 'label-editable';
                
                $input[] = $row;
        }

        return $input;
    }

    
    
    
    
    
    protected function renderProdTypeBlock()
    {
        $input = array();
        
        $input[] = $this->renderSwitch('KERAWEN_NEW_PROD_REF_REQUIRED', $this->l('New product: Field reference required'));
        $input[] = $this->renderSwitch('KERAWEN_NEW_PROD_WHOLESALE_PRICE', $this->l('New product: set wholesale price to 0'));
        $input[] = array(
            'name' => 'KERAWEN_DEFAULT_VAT',
            'label' => $this->l('New product default VAT'),
            'type'  => 'select',
            'options'  => array(
                'query' => Db::getInstance()->executeS('
					SELECT id_tax_rules_group, name
					FROM '._DB_PREFIX_.'tax_rules_group
					WHERE active = 1'),
                'id' => 'id_tax_rules_group',
                'value' => 'id_tax_rules_group',
                'name' => 'name',
            )
        );

        $input[] = array(
            'name' => 'spacer',
            'label' => $this->l('Product types'),
            'type' => 'free',
            'content' => '<hr />',
        );
        
        if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '>='))
        {
            $header = array(
                'name' => '_KERAWEN_PT_LIST_',
                'type' => 'free',
                'content' => '
					<div class="col-lg-3">
						<div style="text-align:center">'.$this->l('Default Qty').'</div>
					</div>
					<div class="col-lg-3">
						<div style="text-align:center">'.$this->l('Margin rate').'</div>
					</div>
                    <div class="col-lg-3">
                        <div style="text-align:center">'.$this->l('Active margin rate').'</div>
                    </div>',
            );
        }
        else
        {
            $header = array(
                'name' => '_KERAWEN_PT_LIST_',
                'type' => 'free',
                'content' => '
					<div class="kerawen-config-payment-block">
						<span>'.$this->l('Default Qty').'</span>
					</div>
					<div class="kerawen-config-payment-block">
						<span>'.$this->l('Margin rate').'</span>
					</div>
					<div class="kerawen-config-payment-block">
						<span>'.$this->l('Active margin rate').'</span>
					</div>',
            );
        }
        $input[] = $header;
        
        $cfg = json_decode(Configuration::get('KERAWEN_PROD_TYPES'), true);
        $checked = ' checked="checked"';
        foreach ($this->getProdTypes() as  $id => $label)
        {
 
            $qty = 0;
            $margin = '';
            $active_magin = 0;
            
            if ($cfg && isset($cfg[$id])) {
                $qty = (int) $cfg[$id]['qty'];
                $margin = $cfg[$id]['margin_rate'];
                $active_magin = (int) $cfg[$id]['margin_calc'];
            }
            
            $name = 'KERAWEN_TP_ACTIVE_'.$id;
            $input_qty = '<input style="width:100px;" type="text" name="KERAWEN_TP_QTY_' . $id . '" placeholder="0" value="' . $qty  . '" />';
            $input_margin = '<input style="width:100px;" type="text" name="KERAWEN_TP_MARGIN_' . $id . '" value="' . $margin  . '" />';
            
            if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '>='))
            {
                $content = '
					<div class="col-lg-3">
                        ' . $input_qty  . '
					</div>
                    <div class="col-lg-3">
                        ' . $input_margin . '
                    </div>
					<div class="col-lg-2">
						<span class="switch prestashop-switch fixed-width-lg">
							<input type="radio" name="'.$name.'" id="'.$name.'on" value="1"'.($active_magin ? $checked : '').'>
							<label for="'.$name.'on">'.$this->l('Yes').'</label>
							<input type="radio" name="'.$name.'" id="'.$name.'off" value="0"'.($active_magin ? '' : $checked).'>
							<label for="'.$name.'off">'.$this->l('No').'</label>
							<a class="slide-button btn"></a>
						</span>
					</div>
                ';
            }
            else
            {
                $content = '
					<div class="kerawen-config-payment-block">
                        ' . $input_qty  . '
					</div>
                    <div class="kerawen-config-payment-block">
                        ' . $input_margin . '
                    </div>
					<div class="kerawen-config-payment-block">
						<input type="radio" name="'.$name.'" id="'.$name.'on" value="1"'.($active_magin ? $checked : '').'>
						<label class="t" for="'.$name.'on">
							<img src="../img/admin/enabled.gif" alt="Activé" title="Activé">
						</label>
						<input type="radio" name="'.$name.'" id="'.$name.'off" value="0"'.($active_magin ? '' : $checked).'>
						<label class="t" for="'.$name.'off">
							<img src="../img/admin/disabled.gif" alt="Désactivé" title="Désactivé">
						</label>
					</div>
                ';
            }
            
            $row = array(
                'name' => $name,
                'label' => $label,
                'type' => 'free',
                'content' => $content,
            );

            $input[] = $row;
        }
        
        $input[] = $this->renderSwitch('KERAWEN_DEPOSIT_WITH_FREE_PRICE', $this->l('Allow free price for product type Deposit'));
        $input[] = $this->renderSwitch('KERAWEN_SCAN_NOT_FOUND', $this->l('Scan result: suggest product creation if no product found'));
        $input[] = $this->renderSwitch('KERAWEN_PACK_SPLIT', $this->l('Split pack product'));
        
        
        return $input;
    }
    

    protected function renderSelect($select) {
        
        if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '>=')) {
            $content = '
    					<div class="col-lg-2">
    						<span class="switch prestashop-switch fixed-width-lg">
                                '. $select . '
    						</span>
    					</div>';
            
        } else {
            $content = '
    					<div class="kerawen-config-payment-block">
                            '. $select . '
    					</div>';
        }
        
        return $content;
    }

    protected function renderModeBlock()
    {
        require_once(_KERAWEN_CLASS_DIR_.'/KerawenPayment.php');
        
        $input = array();
        
        $listStats = OrderState::getOrderStates($this->context->language->id);
        $listPaymentStatsLabels = array($this->l('Not paid'), $this->l('Partially paid'), $this->l('Fully paid'), $this->l('Too much paid'));
        
        $kmp = KerawenPayment::getModesPayments();
        $kmp = $kmp[0];
        $modes_id = array('KERAWEN_LABEL_IN_STORE', 'KERAWEN_LABEL_TAKEAWAY','KERAWEN_LABEL_DELIVERY');
        $modes_po = array($this->l('In store'), $this->l('Takeaway'), $this->l('Delivery'));

        $content = '';
        foreach ($this->listPaymentStats as $key => $paymentStats) {
            $label = $listPaymentStatsLabels[$key];
            if (Tools::version_compare(_PS_VERSION_, '1.6.0.0', '>=')) {
                $content .= '
					<div class="col-lg-2">
						<div style="text-align:center">'.$label.'</div>
					</div>';

            } else {
                $content .= '
					<div class="kerawen-config-payment-block">
						<span>'.$label.'</span>
					</div>';
            }
        }

        $header = array(
            'name' => '_KERAWEN_MODE_LIST_',
            'type' => 'free',
            'content' => $content
        );
        
        $input[] = $header;
                
        foreach ($this->modes as $mode)
        {
            $content = '';
            foreach ($this->listPaymentStats as $paymentStats) {
                $val = 0;
                if ($kmp) {
                    if(isset($kmp[$mode][$paymentStats])) {
                        $val = (int) $kmp[$mode][$paymentStats];
                    }
                }
                $select = $this->renderStatusList($listStats, 'KERAWEN_MODE_' . $mode . '_' . $paymentStats,  $val);
                $content .= $this->renderSelect($select);
            }
            
            $row = array(
                'name' => 'KERAWEN_MODES_' . $mode,
                'label' => '<input type="text" class="kerawen-config-payment-input" name="'. $modes_id[$mode] .'" placeholder="' . $modes_po[$mode]  . '" value="'. Configuration::get($modes_id[$mode]). '" />',
                'type' => 'free',
                'content' => $content,
            );
            $input[] = $row;
        }
        
        $input[] = $this->renderSwitch('KERAWEN_PAYMENT_MODE', $this->l('Display cashdrawer payment mode on BO'));
        
        return $input;
    }
    
    protected function processMode()
    {
        require_once(_KERAWEN_CLASS_DIR_.'/KerawenPayment.php');
        $modePayment = array();
        foreach($this->modes as $mode) {
            $modePayment[] = array();
            foreach($this->listPaymentStats as $PaymentStats) {
                $modePayment[$mode][$PaymentStats] = (int)Tools::getValue('KERAWEN_MODE_' . $mode . '_' . $PaymentStats);
            }
        }
        $kmp = KerawenPayment::getModesPayments();
        $kmp[0] = $modePayment;
        Configuration::updateValue('KERAWEN_MODES_PAYMENTS', json_encode($kmp));        
    }


    protected function processTillStatus()
    {
         $status_list = OrderState::getOrderStates($this->context->language->id);
         
         $status = array();
         $notif = array();
         
         foreach ($status_list as $item) {
             $status[$item['id_order_state']] = array(
                 'id' => $item['id_order_state'],
                 'active' => (int)Tools::getValue('KERAWEN_STATUS_TILL_'.$item['id_order_state']),
             );
             $notif[$item['id_order_state']] = array(
                 'id' => $item['id_order_state'],
                 'active' => (int)Tools::getValue('KERAWEN_NOTIF_TILL_'.$item['id_order_state']),
             );
         }
         
         Configuration::updateValue('KERAWEN_STATUS_TILL', (string) json_encode($status));
         Configuration::updateValue('KERAWEN_NOTIF_TILL', (string) json_encode($notif));
    }

    
    protected function processTypes() {
        $types = array();
        foreach ($this->getProdTypes() as $id => $label) {
            $types[$id] = array(
                'qty' => (int) Tools::getValue('KERAWEN_TP_QTY_'.$id),
                'margin_rate' => Tools::getValue('KERAWEN_TP_MARGIN_'.$id),
                'margin_calc' => (int) Tools::getValue('KERAWEN_TP_ACTIVE_'.$id)
            );
        }
        Configuration::updateValue('KERAWEN_PROD_TYPES', json_encode($types));
    }
    
    
    protected function processPayment()
    {
        $payments = array();
        $modes = KerawenPayment::getModes($this->module);
        foreach ($modes as $mode)
        {
            $id = $mode['id'];
            $label = Tools::getValue('KERAWEN_PM_'.$id.'_LABEL');
            
            $payments[$id] = array(
                'id' => $id,
                'label' => $label ? $label : $mode['label'],
                'payment' => (int)Tools::getValue('KERAWEN_PM_'.$id.'_PAYMENT'),
                'refund' => (int)Tools::getValue('KERAWEN_PM_'.$id.'_REFUND'),
            );
        }

        $old_p = json_decode(Configuration::get('KERAWEN_PAYMENTS'), true);
        $new_p = $payments;
        
        Configuration::updateValue('KERAWEN_PAYMENTS', json_encode($payments));

        $kp = Configuration::get('KERAWEN_PERMISSIONS');
        $profiles = Profile::getProfiles(Context::getContext()->language->id);
        if ($kp) {
            $kp = unserialize($kp);
            $with_change = false;
            foreach($profiles as $profile) {
                foreach($new_p as $k => $p) {

                    if (isset($old_p[$k]) && isset($new_p[$k]) && serialize($old_p[$k]) !== serialize($new_p[$k])) {

                        $list_mode = array();
                        $with_change = true;
                        if ($new_p[$k]['payment']) { $list_mode[] = 'payment'; }
                        if ($new_p[$k]['refund']) { $list_mode[] = 'refund'; }
                        
                        if (isset($kp[$profile['id_profile']])) {
                            if (isset($kp[$profile['id_profile']]['mode_' . $k])) {
                                $kp[$profile['id_profile']]['mode_' . $k] = implode(',', $list_mode);
                            }
                        }
                        
                    }    
                }
            }
            
            if ($with_change) {
                Configuration::updateValue('KERAWEN_PERMISSIONS', serialize($kp), null, 0, 0);
            }
        }
 
    }
    
    protected function processCarrierMode() {        
        Configuration::updateValue('KERAWEN_CARRIER_MODE', (string)serialize(Tools::getValue('KERAWEN_CARRIER_MODE')));
    }
    
    protected function processGroupsAmount() {
        
        if (Tools::getIsset('KERAWEN_GROUP_AMOUNT_CUST')) {            
            Configuration::updateValue('KERAWEN_GROUP_AMOUNT_CUST', (int)Tools::getValue('KERAWEN_GROUP_AMOUNT_CUST'));
        }

        if (Tools::getIsset('KERAWEN_GROUP_AMOUNT')) {
            Configuration::updateValue('KERAWEN_GROUP_AMOUNT', (string)serialize(Tools::getValue('KERAWEN_GROUP_AMOUNT')));
            
            if(Tools::getValue('KERAWEN_UACA')) {
                
                //Required ? -> create missing joins
                Db::getInstance()->executeS('
                    INSERT IGNORE INTO '._DB_PREFIX_.'customer_kerawen (id_customer)
                    SELECT id_customer FROM '._DB_PREFIX_.'customer
                ');
                
                //Update by group
                foreach(Tools::getValue('KERAWEN_GROUP_AMOUNT') as $group => $amount) {
                    $value =  is_numeric($amount) ? $amount : 0;
                    
                    Db::getInstance()->executeS('
                        UPDATE '._DB_PREFIX_.'customer_kerawen ck
                        INNER JOIN '._DB_PREFIX_.'customer c ON ck.id_customer = c.id_customer AND c.id_default_group = ' . (int) $group . '
                        SET ck.limit = "' . pSQL($value) . '"'
                    );
                }
            }
        }
    }
    
    
    protected function renderImageTypes($name, $label, $for) {
        return array(
            'name' => $name,
            'label' => $label,
            'type'  => 'select',
            'options'  => array(
                'query' => array_merge(
                    array(
                        array(
                            'value' => false,
                            'name' => $this->l('No image')
                        )),
                    Db::getInstance()->executeS('
						SELECT name AS value, name
						FROM '._DB_PREFIX_.'image_type
						WHERE '.$for.' = 1')),
                'id' => 'value',
                'value' => 'value',
                'name' => 'name',
            )
        );
    }
    
    
    protected function renderStatusList($list, $name, $val) {
        $select = '<select name="' . $name . '" id="' . $name . '">';
        foreach($list as $item) {
            $sel = $item['id_order_state'] == $val ? 'selected = "selected" ':'';
            $select .= '<option ' . $sel .' value="' . $item['id_order_state'] . '">' . $item['name'] . '</option>';
        }
        $select .= '</select>';
        return $select;
    }
    
    
    protected function renderStatus($name, $label) {
        return array(
            'name' => $name,
            'label' => $label,
            'type'  => 'select',
            'options'  => array(
                'query' => OrderState::getOrderStates($this->context->language->id),
                'id' => 'id_order_state',
                'value' => 'id_order_state',
                'name' => 'name',
            )
        );
    }
    
}
