<?php
/**
* 2014 KerAwen
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2014 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/

require_once(dirname(__FILE__).'/../../defines.php');

class KerawenMobileAppController extends ModuleAdminController
{
    public $redirect_after = '';
    public $redirect_to = 'KerawenHome';
    public function __construct()
    {
        parent::__construct();
        $this->bootstrap = true;

        if (Tools::getValue('verify_user')) {
            // verfy the user login details and permissions
            if (
                isset($_REQUEST['username']) &&
                !empty($_REQUEST['username']) &&
                isset($_REQUEST['password']) &&
                !empty($_REQUEST['password'])
            ) {
                $user_info = array( "username" => $_REQUEST['username'], "password" => $_REQUEST['password'], "ps_base_url" => $_REQUEST['ps_base_url'], "ps_bo_url" => $_REQUEST['ps_bo_url']);
            
                $employee = $this->gpIsEmployeeCredentialsValid($_REQUEST['username'], $_REQUEST['password']);
                if (!empty($employee)) {
                    $permissions_info = Profile::getProfileAccesses($employee->id_profile, "class_name");
            
                    if (!empty($permissions_info) &&
                        isset($permissions_info['KerawenHome']) &&
                        $permissions_info['KerawenHome']['view'] &&
                        $permissions_info['KerawenHome']['add'] &&
                        $permissions_info['KerawenHome']['edit']
                    ) {
                        echo json_encode(array("status" => "success","user" => $user_info));
                        die();
                    } else {
                        echo json_encode(array("status" => "failed", "info" => "Permissions non conformes"));
                        die();
                    }
                } else {
                    echo json_encode(array("status" => "failed", "info" => "Mauvais identifiants de connexion"));
                    die();
                }
            } else {
                echo json_encode(array("status" => "failed", "info" => "Champs obligatoires invalides"));
                die();
            }
        } else {
            $this->processLogin();
            if (empty($this->errors)) {
                Tools::redirectAdmin($this->redirect_after);
                exit;
            }
            foreach ($this->errors as $error) {
                print_r($error);
                echo "</br>";
            }
        }

        echo json_encode(array("status" => "failed", "info" => "Champs obligatoires invalides"));
        die();
    }


    public function gpIsEmployeeCredentialsValid($email, $passwd)
    {
        $passwd = trim($passwd);
        $email = trim($email);

        // Find employee
        $employee = new Employee();
        $is_employee_loaded = $employee->getByEmail($email, $passwd);
        $employee_associated_shop = $employee->getAssociatedShops();
        if (!$is_employee_loaded) {
            return 0;
        } elseif (empty($employee_associated_shop) && !$employee->isSuperAdmin()) {
            return 0;
        } else {
            return $employee;
        }
    }


    public function processLogin()
    {
        /* Check fields validity */
        $passwd = trim(Tools::getValue('passwd'));
        $email = trim(Tools::getValue('email'));
        if (empty($email)) {
            if (version_compare(_PS_VERSION_, '8.0.1', '>=') === true){
                $this->errors[] = $this->trans('Le champ Email est vide', array(), 'Admin.Notifications.Error');
            } else {
                $this->errors[] = $this->l('Le champ Email est vide', array(), 'Admin.Notifications.Error');
            }
        } elseif (!Validate::isEmail($email)) {
            if (version_compare(_PS_VERSION_, '8.0.1', '>=') === true){
                $this->errors[] = $this->trans('Email invalide'. $email, array(), 'Admin.Notifications.Error');
            }
            else{
                $this->errors[] = $this->l('Email invalide'. $email, array(), 'Admin.Notifications.Error');
            }            
        }

        if (version_compare(_PS_VERSION_, '8.0.1', '>=') === true){
            if (empty($passwd)) {
                $this->errors[] = $this->trans('Le champ Mot de passe est vide', array(), 'Admin.Notifications.Error');
            } elseif (!Validate::isPasswdAdmin($passwd)) { //isPasswd() depricated from PS version > 8.0
                $this->errors[] = $this->trans('Mauvais mot de passe', array(), 'Admin.Notifications.Error');
            }
        } else {
            if (empty($passwd)) {                
                $this->errors[] = $this->l('Le champ Mot de passe est vide', array(), 'Admin.Notifications.Error');
            } elseif (!Validate::isPasswd($passwd)) { //isPasswd() depricated from PS version > 8.0
                $this->errors[] = $this->l('Mauvais mot de passe', array(), 'Admin.Notifications.Error');
            }
        }

        if (!count($this->errors)) {
            // Find employee
            $this->context->employee = new Employee();
            $is_employee_loaded = $this->context->employee->getByEmail($email, $passwd);
            $employee_associated_shop = $this->context->employee->getAssociatedShops();
            if (!$is_employee_loaded) {
                if (version_compare(_PS_VERSION_, '8.0.1', '>=') === true){
                    $this->errors[] = $this->trans('L\'employé n\'existe pas, ou le mot de passe donné n\'est pas le bon', array(), 'Admin.Login.Notification');
                }
                else{
                    $this->errors[] = $this->l('L\'employé n\'existe pas, ou le mot de passe donné n\'est pas le bon', array(), 'Admin.Login.Notification');  
                }
                $this->context->employee->logout();
            } elseif (empty($employee_associated_shop) && !$this->context->employee->isSuperAdmin()) {
                if (version_compare(_PS_VERSION_, '8.0.1', '>=') === true){
                    $this->errors[] = $this->trans('L\'employé ne gère plus cette boutique, ou ses permissions ont été supprimées', array(), 'Admin.Login.Notification');
                }
                else {
                    $this->errors[] = $this->l('L\'employé ne gère plus cette boutique, ou ses permissions ont été supprimées', array(), 'Admin.Login.Notification');
                }                
                $this->context->employee->logout();
            } else {

                $permissions_info = Profile::getProfileAccesses($this->context->employee->id_profile, "class_name");
                if (!empty($permissions_info) &&
                    isset($permissions_info['KerawenHome']) &&
                    $permissions_info['KerawenHome']['view'] &&
                    $permissions_info['KerawenHome']['add'] &&
                    $permissions_info['KerawenHome']['edit']
                ) {
                    // all required permissions are ok
                }else{
                    // all required permissions are not ok
                    if (version_compare(_PS_VERSION_, '8.0.1', '>=') === true){
                        $this->errors[] = $this->trans('L\'employé n\a pas les bonnes permissions pour accéder aux applications Kerawen', array(), 'Admin.Login.Notification');
                    }
                    else {
                        $this->errors[] = $this->l('L\'employé n\a pas les bonnes permissions pour accéder aux applications Kerawen', array(), 'Admin.Login.Notification');   
                    }
                    return;
                }

                if (version_compare(_PS_VERSION_, '8.0.1', '>=') === true){
                    PrestaShopLogger::addLog($this->trans('Back office connection from %ip% - from mobile app', array('%ip%' => Tools::getRemoteAddr()), 'Admin.Advparameters.Feature'), 1, null, '', 0, true, (int) $this->context->employee->id);
                } else {
                    PrestaShopLogger::addLog($this->l('Back office connection from %ip% - from mobile app', array('%ip%' => Tools::getRemoteAddr()), 'Admin.Advparameters.Feature'), 1, null, '', 0, true, (int) $this->context->employee->id);
                }

                $this->context->employee->remote_addr = (int) ip2long(Tools::getRemoteAddr());
                // Update cookie
                $cookie = Context::getContext()->cookie;
                $cookie->id_employee = $this->context->employee->id;
                $cookie->email = $this->context->employee->email;
                $cookie->profile = $this->context->employee->id_profile;
                $cookie->passwd = $this->context->employee->passwd;
                $cookie->remote_addr = $this->context->employee->remote_addr;
                if (version_compare(_PS_VERSION_, '1.7.6.6', '>=') === true){
                    $cookie->registerSession(new EmployeeSession());
                } else {
                    $cookie->isLoggedBack($cookie->id_employee);
                }
                $cookie->write();


                // If there is a valid controller name submitted, redirect to it
                if (Validate::isControllerName($this->redirect_to)) {
                    $url = $this->context->link->getAdminLink($this->redirect_to);
                } else {
                    $tab = new Tab((int) $this->context->employee->default_tab);
                    $url = $this->context->link->getAdminLink($tab->class_name);
                }

                if (Tools::isSubmit('ajax')) {
                    die(json_encode(array('hasErrors' => false, 'redirect' => $url)));
                } else {
                    $this->redirect_after = $url;
                }
            }
        }
        if (Tools::isSubmit('ajax')) {
            die(json_encode(array('hasErrors' => true, 'errors' => $this->errors)));
        }
    }
}
