<?php
/**
* 2014 KerAwen
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/AFL-3.0
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to contact@kerawen.com so we can send you a copy immediately.
*
* @author    KerAwen <contact@kerawen.com>
* @copyright 2014 KerAwen
* @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
*/


require_once(_KERAWEN_525_CLASS_);

class KerawenFeedModuleFrontController extends ModuleFrontController
{

    var $context;
    var $db;
    
 	public function __construct() {
 		parent::__construct();

 		if (!(Tools::getValue('key') == Configuration::get('KERAWEN_ORDER_FEED_KEY') && Configuration::get('KERAWEN_ORDER_FEED_KEY')!= false)) {
 		    exit('Access denied - please check your settings');
 		}

 		$this->db = Db::getInstance();
 		$this->context = Context::getContext();
 		
 		header("Content-Type:text/xml; charset=utf-8");
 		$content = $this->getRssHeader()
 		         . $this->getRssContent()
 		         . $this->getRssFooter();
 		
 		exit($content);
 
 	}

 
 	function getRssHeader() {
 	    
 	    $db = Db::getInstance();
 	    
 	    return "<rss version='2.0' xmlns:atom='http://www.w3.org/2005/Atom'>\n"
 	          ."<channel>\n"
         	  ."<title>RSS Feed Orders</title>\n"
              ."<description>RSS Feed Orders</description>\n"
              ."<date>" . date('Y-m-d H:i:s') . "</date>\n";
 	}
 	

 	function getRssFooter() {
 	    return "</channel>\n"
 	          ."</rss>\n";
 	}
 	
 
 	function getRssContent() {
 	    
 	    $db = Db::getInstance();
 	    $proc = Kerawen525::getInstance();
 	    
 	    
 	    $where = array();
 	    $where[] = 'o.current_state != 6'; //exclude cancelled orders
 	    
 	    
 	    if (Tools::getIsset("id_shop")) {
 	        $where[] = 'o.id_shop IN (' . implode(',', array_map('intval', explode(',', Tools::getValue('id_shop')))) . ')';
 	    }

 	    if (Tools::getIsset("last_id")) {
 	        $where[] = 'o.id_order > ' . (int) Tools::getValue('last_id');
 	    }
 	    
 	    if (Tools::getIsset("id_order")) {
 	        $where[] = 'o.id_order = ' . (int) Tools::getValue('id_order');
 	    }
 	    
 	    if (Tools::getIsset("last_dt")) {
 	        $now = date('Y-m-d H:i:s');
 	        $where[] = '(ok.print_date > "' . pSQL(Tools::getValue('last_dt')) . '" AND ok.print_date <= "' . pSQL($now) . '")';
 	    }
 	    
 	    $where =  count($where) ? ' WHERE ' . implode(' AND ', $where) . ' ' : '';


 	    //categories filter
 	    $join_categories = '';
 	    if (Tools::getIsset("cat")) {
 	        $cat = Tools::getValue("cat");
 	        $join_categories = '
        	    INNER JOIN(
                    SELECT
                        od.id_order
        	        FROM ps_order_detail od
        	        INNER JOIN '._DB_PREFIX_.'category_product cp ON od.product_id = cp.id_product AND cp.id_category IN (' . implode(',', array_map('intval', explode(',', $cat))) . ')
        	        GROUP BY od.id_order
        	        ORDER BY od.id_order_detail DESC
                ) AS cat ON cat.id_order = o.id_order
            ';
 	        //exclude returned products ?
 	    }
 	    
 	    
 	    
 	    $orderby = ' ORDER BY o.id_order DESC ';
 	    if (Tools::getIsset("orderby")) {
 	        if (Tools::getValue("orderby") == "id_order") {
 	            //same as default
 	            $orderby = ' ORDER BY o.id_order DESC ';
 	        }
 	        if (Tools::getValue("orderby") == "print_date") {
 	            $orderby = ' ORDER BY ok.print_date DESC ';
 	        }
 	    }
 	    
 	    $limit = ' LIMIT 20 ';
 	    
 	    
 	    $q = '
    		SELECT
    			o.id_order,
                o.date_add,
    			ok.delivery_date AS delivery_date,
                ok.print_date,
                o.current_state AS status,
                IFNULL(ok.delivery_mode, 2) AS delivery_mode,
                ok.delivery_note,
                o.id_address_delivery,
                o.id_customer,
                o.id_shop
    		FROM '._DB_PREFIX_.'orders o
    		LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
            ' . $join_categories . '
		    ' . $where . '
		    ' . $orderby . '
    		' . $limit . '
        ';
 	    
 	    $orders = $db->executeS($q);
 	    
 	    $content = '';
 	    foreach ($orders as $o) {
 	        
 	        $orderlist = $proc->getReceiptData(array($o["id_order"]), array(), false, false, false);

 	        if (isset($orderlist[0])) {
 	            
 	            $order = $orderlist[0];
 	            
 	            /*
 	            echo '<pre>';
 	            print_r($order);
 	            echo '</pre>';
 	            die();
                */
 	            
 	            $content .= "<order>\n";
 	            
 	            $content .= $this->getKeyValue("reference", $order["reference"])
     	            . $this->getKeyValue("id_order", $order["ps_order"])
     	            . $this->getKeyValue("id_shop", $o["id_shop"])
     	            . $this->getKeyValue("id_till", $order["id_till"])
     	            . $this->getKeyValue("till_name", $order["till_name"])
     	            . $this->getKeyValue("shop_name", $order["PS_SHOP_NAME"])
     	            . $this->getKeyValue("delivery_mode", $o["delivery_mode"])
     	            . $this->getKeyValue("status", $o["status"])
     	            . $this->getKeyValue("date_add", $o["date_add"])
     	            . $this->getKeyValue("delivery_date", $o["delivery_date"])
     	            . $this->getKeyValue("print_date", $o["print_date"])
     	            . $this->getKeyValue("id_customer", $order["id_customer"])
     	            . $this->getKeyValue("customer", $order["customer_name"])
     	            . $this->getKeyValue("delivery_note", $o["delivery_note"])
     	            . $this->getKeyValue("id_operator", $order["id_operator"])
     	            . $this->getKeyValue("operator_name", $order["operator_name"])
     	            
     	            . $this->getKeyValue("order_total_ti", $this->amountPrecision($order["order_total_ti"]))
     	            . $this->getKeyValue("order_total_te", $this->amountPrecision($order["order_total_te"]))
     	            . $this->getKeyValue("slip_total_ti", $this->amountPrecision($order["slip_total_ti"]))
     	            . $this->getKeyValue("slip_total_te", $this->amountPrecision($order["slip_total_te"]))
     	            . $this->getKeyValue("carrier_total_ti", $this->amountPrecision($order["carrier_total_ti"]))
     	            . $this->getKeyValue("carrier_total_te", $this->amountPrecision($order["carrier_total_te"]))
     	            
     	            . $this->getKeyValue("total_paid", $this->amountPrecision($order["total_paid"]))
     	            . $this->getKeyValue("remain", $this->amountPrecision($order["remain"]))
     	            . $this->getKeyValue("nb_lines", $order["nb_lines"])
 	            ;



     	        $delivery_address = $this->getAddressDetail($o["id_address_delivery"]);
     	        if ($delivery_address) {
     	              $content .= "<delivery_address>\n"
         	              . $this->getKeyValue("firstname", $delivery_address["firstname"])
         	              . $this->getKeyValue("lastname", $delivery_address["lastname"])
         	              . $this->getKeyValue("company", $delivery_address["company"])
         	              . $this->getKeyValue("vat_number", $delivery_address["vat_number"])
         	              . $this->getKeyValue("address1", $delivery_address["address1"])
         	              . $this->getKeyValue("address2", $delivery_address["address2"])
         	              . $this->getKeyValue("postcode", $delivery_address["postcode"])
         	              . $this->getKeyValue("city", $delivery_address["city"])
         	              . $this->getKeyValue("country", $delivery_address["country"])
         	              . $this->getKeyValue("phone", $delivery_address["phone"])
         	              . $this->getKeyValue("mobile", $delivery_address["mobile"])
         	              . "</delivery_address>\n";
     	         }
     	        
     	             
     	        $msgs = CustomerThread::getCustomerMessages($o["id_customer"], null, $o["id_order"]);
     	        foreach ($msgs as $msg) {
     	            $content .= "<message>\n";
     	            
     	            $content .= $this->getKeyValue("private", (int) $msg['private'])
     	            . $this->getKeyValue("id_employee", (int) $msg['id_employee'])
     	            . $this->getKeyValue("text", $msg['message'])
     	            . $this->getKeyValue("id", $msg['id_customer_message'])
     	            ;
     	            
     	            $content .= "</message>\n";
     	        }
     	        

 	            foreach ($order["detail"] as $product) {
 	                $content .= "<product>\n"
 	                    . $this->productDetail($product)
     	                . "</product>\n";
 	            }

 	            foreach ($order["slip"] as $product) {
 	                $content .= "<slip>\n"
 	                    . $this->productDetail($product)
 	                    . "</slip>\n";
 	            }

 	            foreach ($order["carrier"] as $carrier) {
 	                $content .= "<carrier>\n"
 	                    . $this->productDetail($product)
 	                    ."</carrier>\n";
 	            }
 	            
 	            foreach ($order["cart_rule"] as $cartrule) {
 	                $content .= "<cartrule>\n"
 	                    . $this->getKeyValue("item_name", $cartrule["item_name"])
 	                    . $this->getKeyValue("discount_percent", $this->amountPrecision($cartrule["discount_percent"]))
 	                    . $this->getKeyValue("total_ti", $this->amountPrecision($cartrule["total_ti"]))
 	                    . $this->getKeyValue("total_te", $this->amountPrecision($cartrule["total_te"]))
 	                    . $this->getKeyValue("summary", $this->getCartRuleSummary($cartrule["id_cart_rule"]))
 	                    ."</cartrule>\n";  
 	            }
 	            
 	            foreach ($order["taxes"] as $taxe) {
 	                
 	                $content .= "<taxe>\n"
 	                    . $this->getKeyValue("tax_name", $taxe["tax_name"])
 	                    . $this->getKeyValue("tax_rate", $this->amountPrecision($taxe["tax_rate"]))
 	                    . $this->getKeyValue("total_te", $this->amountPrecision($taxe["total_te"]))
 	                    . $this->getKeyValue("total_ti", $this->amountPrecision($taxe["total_ti"]))
 	                    . $this->getKeyValue("tax_amount", $this->amountPrecision($taxe["tax_amount"]))
 	                    . $this->getKeyValue("id_tax", $taxe["id_tax"])
 	                    ."</taxe>\n";
 	            }
 	            
 	            foreach ($order["payments"] as $payment) {
 	                
 	                $content .= "<payment>\n"
 	                    . $this->getKeyValue("payment_method", $payment["payment_method"])
 	                    . $this->getKeyValue("mode", $payment["mode"])
 	                    . $this->getKeyValue("amount", $this->amountPrecision($payment["amount"]))
 	                    . $this->getKeyValue("date_add", $payment["date_add"])
 	                    ."</payment>\n";
 	            }
 	            
 	            $content .= "</order>\n";
 	         
 	        }

 	    }
 	    
 	    return $content;

 	}
 	
 	
 	function amountPrecision($v) {
 	    return is_numeric($v) ? round($v, 2) : $v;
 	}
 	
 	
 	function productDetail($product) {
 	    return $this->getKeyValue("id_order_detail", $product["id_order_detail"])
 	    . $this->getKeyValue("qty", $product["quantity"])
 	    . $this->getKeyValue("name", $product["item_name"])
 	    . $this->getKeyValue("version", $product["version_name"])
 	    . $this->getKeyValue("unit_te", $product["unit_te"])
 	    . $this->getKeyValue("unit_ti", $product["unit_ti"])
 	    . $this->getKeyValue("total_te", $this->amountPrecision($product["total_te"]))
 	    . $this->getKeyValue("total_ti", $this->amountPrecision($product["total_ti"]))
 	    . $this->getKeyValue("ref", $product["ref"])
 	    . $this->getKeyValue("note", $product["note"])
 	    . $this->getKeyValue("id_tax", $product["id_tax"])
 	    . $this->getKeyValue("discount_percent", $this->amountPrecision($product["discount_percent"]))
 	    . $this->getKeyValue("discount_te", $this->amountPrecision($product["discount_te"]))
 	    . $this->getKeyValue("discount_ti", $this->amountPrecision($product["discount_ti"]))
 	    . $this->getKeyValue("id_carrier", $product["id_carrier"])
 	    . $this->getKeyValue("ean13", $product["ean13"])
 	    . $this->getKeyValue("upc", $product["upc"])
 	    ;
 	}
 	
 	
 	
 	function getAddressDetail($iad) {
 
 	    return $this->db->getRow('
			SELECT
				a.firstname AS firstname,
				a.lastname AS lastname,
				a.company AS company,
				a.vat_number AS vat_number,
				a.address1 AS address1,
				a.address2 AS address2,
				a.postcode AS postcode,
				a.city AS city,
				cl.name AS country,
				a.phone AS phone,
				a.phone_mobile AS mobile
			FROM '._DB_PREFIX_.'address a
			LEFT JOIN '._DB_PREFIX_.'country_lang cl
				ON cl.id_country = a.id_country AND cl.id_lang = ' . (int) $this->context->language->id . '
			WHERE a.id_address = '. (int) $iad);
 	}
 	
 	function getCartRuleSummary($id_cart_rule) {
 	    return $this->db->getValue('
			SELECT
				description
			FROM '._DB_PREFIX_.'cart_rule
			WHERE id_cart_rule = '. (int) $id_cart_rule);
 	}
 	
 
 	function getKeyValue($key, $value) {
 	    //clean value
 	    $value = str_replace('&nbsp;', ' ', $value); //!!!important
 	    $value = str_replace('&', '_', $value); //!!!important
 	    $value = strip_tags($value); //!!!important
 	    return "<" . $key . ">" . $value . "</" . $key . ">" . "\n";
 	}
 	
}