<?php
/**
 * 2017 KerAwen
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@kerawen.com so we can send you a copy immediately.
 *
 * @author    KerAwen <contact@kerawen.com>
 * @copyright 2017 KerAwen
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
 */

function audit($controller, $ignore_warnings) {
	$class = pathinfo(__FILE__, PATHINFO_FILENAME);
	require_once(_KERAWEN_CLASS_DIR_.'/shop.php');
	
	$module = $controller ? $controller->module : null;
	$context = Context::getContext();
	$db = Db::getInstance();
	
	$warnings = array();
	$fixes = array();
	
	// ------------
	// ERRORS
	
	// Multishop installation
	$nb_shops = $db->getValue('
		SELECT COUNT(*) FROM '._DB_PREFIX_.'shop s
		WHERE s.active = 1 AND s.deleted = 0');
	$nb_actives = $db->getValue('
		SELECT COUNT(*) FROM '._DB_PREFIX_.'shop s
		JOIN '._DB_PREFIX_.'module_shop ms ON ms.id_shop = s.id_shop
		WHERE s.active = 1 AND s.deleted = 0 AND ms.id_module = '. (int) Module::getModuleIdByName('kerawen'));
	$warnings[] = array(
		'id' => 'ACTIVE',
		'title' => $module ? $module->l('KerAwen module activation', $class) : null,
		'desc' => $module ? $module->l('KerAwen module shall be activated in all shops.', $class) : null,
		'link' => $controller ? $controller->getAdminLink('AdminModules', array(
				'setShopContext' => '',
				'module_name' => $module->name,
			)) : null,
		'valid' => (int)$nb_shops == (int)$nb_actives,
		'error' => true,
		'correct' => false,
	);
	
	/*
	// Cart numbering
	$buf = $db->executeS('SHOW TABLE STATUS FROM `'._DB_NAME_.'` LIKE "'._DB_PREFIX_.'cart"');
	$id_cart_ai = ($buf && count($buf) && $buf[0]['Auto_increment']) ? $buf[0]['Auto_increment'] : false;
	$id_cart_order = $db->getValue('SELECT MAX(id_cart) FROM '._DB_PREFIX_.'orders');
	$warnings[] = array(
	    'id' => 'CART_AI',
	    'title' => $module ? $module->l('Cart numbering', $class) : null,
	    'desc' => $module ? $module->l('Ordered cart numbers shall not be reused.', $class) : null,
	    'link' => null,
	    'valid' => $id_cart_ai === false ? true : (int)$id_cart_ai > (int)$id_cart_order,
	    'error' => true,
	    'correct' => true,
	);
	*/
	// ------------
	// WARNINGS
	if (!$ignore_warnings) {
		
		// Check overrides for fixes
		require_once(_KERAWEN_FIX_DIR_.'/fixes.php');
		$fixes = getFixes($module);
		$fix_required = false;
		foreach($fixes as $fix) {
			if ($fix['compliant']) {
				$fix_required = true;
				break;
			}
		}
		
		$warnings[] = array(
		    'id' => 'CACHE',
		    'title' => $module ? $module->l('PrestaShop cache', $class) : null,
		    'desc' => $module ? $module->l('PrestaShop cache shall not be enabled in order KerAwen works properly.', $class) : null,
		    'link' => $controller ? $controller->getAdminLink('AdminPerformance') : null,
		    'valid' => !_PS_CACHE_ENABLED_,
		    'error' => true,
		    'correct' => false,
		);
		
		$warnings[] = array(
			'id' => 'OVERRIDES',
			'title' => $module->l('Overrides', $class),
			'desc' => $module->l('Overrides should not be disabled in order PrestaShop fixes are applied.', $class),
			'link' => $controller->getAdminLink('AdminPerformance'),
			'valid' => (!$fix_required || !Configuration::get('PS_DISABLE_OVERRIDES')),
			'error' => false,
			'correct' => true,
		);
	
		// Check fidelisa settings
		if (Module::isInstalled('fidelisa') && Module::isEnabled('fidelisa')) {
			
			//New order
			$warnings[] = array(
				'id' => 'FIDELISA_ORDER_SEND_STATE',
				'title' => $module->l('Fidelisa settings', $class),
				'desc' => $module->l('Order stat required to add customer points', $class),
				'link' => $controller->getAdminLink('AdminModules', array(
					'configure' => 'fidelisa',
					'tab_module' => 'other',
					'module_name' => 'fidelisa',
				)),
				'valid' => (Configuration::get('FIDELISA_ORDER_SEND_STATE') == 5),
			);
			
			//Cancel order
			$warnings[] = array(
				'id' => 'FIDELISA_ORDER_CANCEL_STATE',
				'title' => $module->l('Fidelisa settings', $class),
				'desc' => $module->l('Order stat required to remove customer points', $class),
				'link' => $controller->getAdminLink('AdminModules', array(
					'configure' => 'fidelisa',
					'tab_module' => 'other',
					'module_name' => 'fidelisa',
				)),
				'valid' => (Configuration::get('FIDELISA_ORDER_CANCEL_STATE') == 6),
			);
			
			//Slip product
			
		}
	}	
	
	return array(
		'warnings' => $warnings,
		'fixes' => $fixes,
	);;
}

function correct($issue, $module) {
    $db = Db::getInstance();
    
	switch ($issue) {
	
    case 'CART_AI':
        $id_cart_ai = (int)$db->getValue('SELECT MAX(id_cart) FROM '._DB_PREFIX_.'orders') + 1;
        $db->execute('ALTER TABLE '._DB_PREFIX_.'cart AUTO_INCREMENT = '.(int)$id_cart_ai);
        // Clean-up
        $cart_tables = array(
            'cart', 'cart_cart_rule', 'cart_product',
            'cart_kerawen', 'cart_product_kerawen',
            );
        foreach($cart_tables AS $table) {
            $db->execute('DELETE FROM '._DB_PREFIX_.$table.' WHERE id_cart >= '.(int)$id_cart_ai);
        }
        break;
        
    case 'CACHE':
		Configuration::updateValue('PS_DISABLE_OVERRIDES', false);
		break;

	case 'OVERRIDES':
		Configuration::updateValue('PS_DISABLE_OVERRIDES', false);
		break;

	case 'FIXES':
		require_once(_KERAWEN_FIX_DIR_.'/fixes.php');
		installAllFixes($module);
		break;

	case 'FIDELISA_ORDER_SEND_STATE':
		Configuration::updateValue('FIDELISA_ORDER_SEND_STATE', 5);
		break;	

	case 'FIDELISA_ORDER_CANCEL_STATE':
		Configuration::updateValue('FIDELISA_ORDER_CANCEL_STATE', 6);
		break;		
		
	}
} 