<?php
/**
* @author    KerAwen <contact@kerawen.com>
* @copyright 2017-2018 KerAwen
*/

require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
require_once(_KERAWEN_API_DIR_.'/constants.php');

class KerawenLog {
	
	var $db;
	var $filter;
	var $lang;
	var $init_filter;
	var $init_from;
	var $init_to;
	var $accounted;
	var $from2;
	var $to2;
	var $stats_filter;
	var $group_by;
	var $vars;
	
	var $off;
	var $dd;
	var $total_sale_te;
	var $total_sale_ti;
	var $total_ret_te;
	var $total_ret_ti;
	var $stat_join;
	var $id_shops;

	public function __construct($filter) {
		$this->db = Db::getInstance();
		$this->id_lang = Context::getContext()->language->id;
		
		$this->off = 'IF(sad.gift_card OR sad.account_credit, 0, 1)';
		$this->dd = 'IF(sad.product_type = "'._KERAWEN_TYPE_DEPOSIT_.'", 1, 0)';
		$this->total_sale_te = 'IFNULL(SUM((sad.total_te - sad.purchase_te*sad.quantity*'.$this->dd.')*'.$this->off.'), 0)';
		$this->total_sale_ti = 'IFNULL(SUM((sad.total_ti - sad.purchase_te*sad.quantity*'.$this->dd.')*'.$this->off.'), 0)';
		$this->total_ret_te = 'IFNULL(SUM((sad.total_te - sad.purchase_te*sad.quantity*'.$this->dd.')*'.$this->off.'*IF(sad.total_te < 0, 1, 0)), 0)';
		$this->total_ret_ti = 'IFNULL(SUM((sad.total_ti - sad.purchase_te*sad.quantity*'.$this->dd.')*'.$this->off.'*IF(sad.total_ti < 0, 1, 0)), 0)';

		$this->stat_join = '
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
            JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON op.id_operation = sa.id_operation
            JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad ON sad.id_sale = sa.id_sale
        ';
		
		if (is_array($filter)) {
		    
		    $this->id_shops = $filter['id_shop'];
		    $this->vars = isset($filter['vars']) ? $filter['vars'] : array();
		    
		    
		    if (isset($filter['id_order']) && $filter['id_order']) {
			    $ops = $this->opsbyorder($filter['id_order']);
			    if ($ops) {
			        $this->opfilter = 'op.id_operation IN (' . dataListToIntList($ops) . ')';
			    } else {
			        $this->opfilter = " FALSE ";
			    }
			    $this->safilter = "";
			    
			} else {
			    $this->opfilter = '
    				op.date BETWEEN "'.pSQL($filter['from']).'" AND "'.pSQL($filter['to']).'"
    				'.$this->getFilter('op.id_till', $filter['id_till']).'
    				'.$this->getFilter('op.id_operator', $filter['id_operator']).'
    				'.$this->getFilter('op.id_shop', $filter['id_shop']);
			    
			    $this->safilter = "";
			    $this->safilter .= $this->getFilter('sa.accounted', $filter['accounted'], true);
			    $this->safilter .= $this->getFilter('sa.id_group', $filter['id_group']);
			}
			
			$this->opfilter .= ' AND op.canceled IS NULL';
			
			$this->stats_filter = '
				'.$this->getFilter('op.id_till', $filter['id_till']).'
				'.$this->getFilter('op.id_operator', $filter['id_operator']).'
				'.$this->getFilter('op.id_shop', $filter['id_shop']).'
                '.$this->getFilter('sa.accounted', $filter['accounted']).'
				AND op.canceled IS NULL';

			$this->init_filter = '
				op.date BETWEEN "'.pSQL($filter['init_from']).'" AND "'.pSQL($filter['init_to']).'"
				'.$this->getFilter('op.id_till', $filter['id_till']).'
				'.$this->getFilter('op.id_operator', $filter['id_operator']).'
				'.$this->getFilter('op.id_shop', $filter['id_shop']).'
				'.$this->getFilter('sa.accounted', $filter['accounted']).'
				AND op.canceled IS NULL';
			    $this->init_filter .= " " . $this->getFilter('sa.id_group', $filter['id_group']);
			
			$this->init_from = $filter['init_from'];
			$this->init_to = $filter['init_to'];
			$this->from2 = isset($filter['from2']) ? $filter['from2'] : false;
			$this->to2 = isset($filter['to2']) ? $filter['to2']: false;

			$this->accounted = $filter['accounted'];
			$this->group_by = isset($filter['group_by']) ? $filter['group_by'] : '';

			if (isset($this->vars->by)) $this->group_by = $this->vars->by;
		}
		else {
			$this->opfilter = $filter;
			$this->safilter = ' AND TRUE';
		}
	}
	
	protected function getFilter($col, $val, $force = false) {
		if ($val === '') return $force ? 'AND TRUE' : '';
		if ($val == _KERAWEN_CD_ALL_) return $force ? 'AND TRUE' : '';
		return ' AND '.$col.' IN ('. dataListToIntList($val) .')';
	}

	public function opsbyorder($ps_order) {
	    return $this->db->getValue('
            SELECT
                GROUP_CONCAT(s.id_operation) AS ops
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order o
    		LEFT JOIN  '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s ON o.id_sale = s.id_sale
    		WHERE o.ps_order = ' . (int) $ps_order
	    );
	}
	
	public function ops() {
		return $this->db->executeS('
			SELECT
				op.id_operation AS id_op,
				op.id_till AS id_till,
				op.id_operator AS id_empl,
				op.id_shop AS id_shop,
				op.date AS date,
				op.type AS oper
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
			WHERE '.$this->opfilter);
	}

	public function sales() {
		$off = 'IF(sad.gift_card OR sad.account_credit, 0, 1)';
		$dd = 'IF(sad.product_type = "'._KERAWEN_TYPE_DEPOSIT_.'", 1, 0)';
		$old = 'IFNULL(sad.margin_vat, 0)';
		$new = 'IF('. $old .'=0, 1, 0)';
		
		// Ignore special products
		return $this->db->executeS('
			SELECT
				op.id_operation AS id_op,
				sa.accounted AS accounted,
				(CASE
					WHEN ord.ps_slip != 0 THEN "SLIP"
					WHEN ord.total_ti < 0 THEN "CANCEL"
					ELSE "ORDER"
				END) AS oper,
				ord.id_order AS id_sale,
				op.id_shop AS id_shop,
                op.operator_name AS empl,
                op.id_operator AS id_empl,
				ord.ps_order AS id_order,
				ord.ps_slip AS id_slip,
				o.reference AS ref,
				-- i.invoice_label AS id_invoice,
				IF(o.invoice_number > 0, o.invoice_number, "") AS id_invoice,
				SUM((sad.total_te - sad.purchase_te*sad.quantity*'.$dd.')*'.$off.') AS tax_excl,
				SUM((sad.total_ti - sad.purchase_te*sad.quantity*'.$dd.')*'.$off.') AS tax_incl,
				SUM((sad.total_ti - sad.purchase_te*sad.quantity*'.$dd.')*'.$off.'*'.$old.') AS tax_incl_o,
				SUM((sad.total_ti - sad.purchase_te*sad.quantity*'.$dd.')*'.$off.'*'.$new.') AS tax_incl_n,
				SUM(sad.unit_te*sad.measure*sad.quantity - sad.total_te) AS disc_te,
				SUM(sad.unit_ti*sad.measure*sad.quantity - sad.total_ti) AS disc_ti,
				sa.customer_name AS cust,
				c.company,
				gl.name AS `group`,
				SUM(sad.quantity) AS qty,
				SUM((sad.total_te - sad.purchase_te*sad.quantity)*'.$off.') AS profit,
                IF(ord.ps_slip > 0, "", a.vat_number) AS vat_num
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order ord
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON ord.id_sale = sa.id_sale
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON sa.id_operation = op.id_operation
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad ON sad.id_order = ord.id_order
			-- LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'invoice i ON i.id_order = ord.id_order
			LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = ord.ps_order
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = sa.id_customer
            LEFT JOIN '._DB_PREFIX_.'address a ON o.id_address_invoice = a.id_address
			LEFT JOIN '._DB_PREFIX_.'group_lang gl ON gl.id_group = c.id_default_group AND gl.id_lang = '. (int) $this->id_lang .'
			WHERE sad.gift_card = 0 AND sad.account_credit = 0
				AND '.$this->opfilter.' '.$this->safilter.'
			GROUP BY ord.id_order');
		}

	public function taxes() {
		// Exclude gift cards
		$off = 'IF(sad.gift_card OR sad.account_credit, 0, 1)';
		$dd = 'IF(sad.product_type = "'._KERAWEN_TYPE_DEPOSIT_.'", 1, 0)';
		
		return $this->db->executeS('
			SELECT
				op.id_operation AS id_op,
				sa.accounted AS accounted,
				(CASE
					WHEN ord.ps_slip != 0 THEN "SLIP"
					WHEN ord.total_ti < 0 THEN "CANCEL"
					ELSE "ORDER"
				END) AS oper,
				sa.id_sale AS id_sale,
				ord.ps_order AS id_order,
				ord.ps_slip AS id_slip,
				o.reference AS ref,
				op.id_shop AS id_shop,
				(CASE
					WHEN sad.wrapping IS NOT NULL THEN "WRAP"
					WHEN sad.id_carrier IS NOT NULL THEN "SHIP"
					ELSE "PROD"
				END) AS object,
				sad.id_tax AS id_taxrule,
				SUM((sad.unit_te - sad.purchase_te*'.$dd.')*IFNULL(sad.measure,1)*sad.quantity*'.$off.') AS gross_te,
				SUM((sad.total_te - sad.purchase_te*sad.quantity*'.$dd.')*'.$off.') AS tax_excl,
				SUM((sad.total_ti - sad.total_te)*'.$off.') AS tax,
				CONCAT(c.firstname, " ", c.lastname) AS customer,
				c.company,
				-- i.invoice_label AS inv_num,
				IF(o.invoice_number > 0, o.invoice_number, "") AS inv_num
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order ord ON ord.id_order = sad.id_order
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_sale = sad.id_sale
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = sa.id_operation
			LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = ord.ps_order
			-- LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'invoice i ON sad.id_order = i.id_order
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = sa.id_customer
			WHERE '.$this->opfilter.' '.$this->safilter.'
			GROUP BY op.id_operation, ord.id_order, sad.id_tax');
	}
	
	public function flows() {
		$module = Module::getInstanceByName('kerawen');
		$deferred = $module->l('Deferred', pathinfo(__FILE__, PATHINFO_FILENAME));
		$gcard = 'Carte cadeau';
		$acredit = 'Recharge compte client';
		
		$dd = 'IF(sad.product_type = "'._KERAWEN_TYPE_DEPOSIT_.'", 1, 0)';
		
		return array_merge(
		    
		    // Standard payments
			$this->db->executeS('
				SELECT DISTINCT
					op.id_operation AS id_op,
					pa.id_payment AS id_flow,
					pa.ps_orders AS id_order,
					0 AS id_slip,
					pa.id_order_payment AS id_payment,
					0 AS id_mode,
					pa.mode AS mode,
					pa.amount AS amount,
					1 AS count,
					null AS deferred,
					null AS note,
                    pa.remain
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment pa
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = pa.id_operation
                LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON op.id_operation = sa.id_operation
				WHERE '.$this->opfilter.' '.$this->safilter),
			
		    // Special products as payment (compatibility with previous versions)
		    $this->db->executeS('
				SELECT
					op.id_operation AS id_op,
					0 AS id_flow,
					o.ps_order AS id_order,
					0 AS id_slip,
					0 AS id_payment,
					0 AS id_mode,
					IF(sad.gift_card, "'.$gcard.'", "'.$acredit.'") AS mode,
					-sad.total_ti AS amount,
					1 AS count,
					null AS deferred,
					null AS note,
                    null AS remain
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order o ON o.id_order = sad.id_order
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_sale = o.id_sale
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = sa.id_operation
				WHERE (sad.gift_card = 1 OR sad.account_credit = 1)
					AND '.$this->opfilter.' '.$this->safilter),
		    
		    // Deferred
		    $this->db->executeS('
				SELECT
					op.id_operation AS id_op,
					0 AS id_flow,
					"" AS id_order,
					0 AS id_slip,
					0 AS id_payment,
					0 AS id_mode,
					"'.$deferred.'" AS mode,
					IFNULL(due.due, 0) - IFNULL(paid.paid, 0) AS amount,
					1 AS count,
					null AS deferred,
					null AS note,
                    null AS remain
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				LEFT JOIN (
					SELECT
						sa.id_operation AS id_operation,
						SUM(sad.total_ti - sad.purchase_te*sad.quantity*'.$dd.') AS due
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad ON sad.id_sale = sa.id_sale
					GROUP BY sa.id_operation) due ON due.id_operation = op.id_operation
				LEFT JOIN (
					SELECT
						pa.id_operation AS id_operation,
						SUM(pa.amount) AS paid
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment pa
					GROUP BY pa.id_operation) paid ON paid.id_operation = op.id_operation
				WHERE '.$this->opfilter.'
				HAVING amount != 0'),
			
		    // Drawer flows
		    $this->db->executeS('
				SELECT
					op.id_operation AS id_op,
					tf.id_tillflow AS id_flow,
					"" AS id_order,
					0 AS id_slip,
					0 AS id_payment,
					tf.id_mode AS id_mode,
					tf.mode AS mode,
					tf.amount AS amount,
					tf.count AS count,
					null AS deferred,
					comments AS note,
                    null AS remain
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow tf
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = tf.id_operation
				WHERE '.$this->opfilter)
		);
	}
	
	public function checks() {
		return $this->db->executeS('
			SELECT
				op.id_operation AS id_op,
				tc.id_tillcheck AS id_check,
				tc.id_mode AS id_mode,
				tc.checked AS checked,
				tc.error AS error
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_check tc
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = tc.id_operation
			WHERE '.$this->opfilter);
	}
	
	public function prods() {
	    
	    $field_stock = '';
	    $join_stock = '';

	    if (Module::isInstalled('kerawen_ocs') && Module::isEnabled('kerawen_ocs')) {
	        $exclude_shop = (int) $this->db->getValue('SELECT id_shop FROM '._DB_PREFIX_.'kerawen_cnc_shop_type WHERE type = "virtual"');
	    } else {
	        //SEVENS
	        $exclude_shop = (int) Configuration::get('KERAWEN_STOCK_SHOP_EXCLUDE');
	    }

	    $shops = array_map('intval', explode(',', $this->id_shops));
	    if ($exclude_shop > 0 && count($shops) > 1) {
	       if (($key = array_search((int) $exclude_shop, $shops)) !== false) { unset($shops[$key]); }
	    }

	    if (isset($this->vars->widthStock)) {
            $field_stock = 'psa.quantity AS stock,';
            $join_stock = '
               LEFT JOIN (
                 	SELECT
                 		SUM(quantity) AS quantity, 
                        id_product, 
                        id_product_attribute
                 	FROM '._DB_PREFIX_.'stock_available
                    WHERE id_shop IN (' . implode(',', $shops) . ')
                    GROUP BY id_product, id_product_attribute
                ) psa ON psa.id_product = od.product_id AND psa.id_product_attribute = od.product_attribute_id
            ';
	    }

		return $this->db->executeS('
			SELECT
				op.id_operation AS id_op,
				sa.accounted AS accounted,
				(CASE
					WHEN ps_slip = 0 THEN "ORDER"
					ELSE "SLIP"
				END) AS oper,
				op.id_shop AS id_shop,
				ord.id_order AS id_sale,
				ord.ps_order AS id_order,
				ord.ps_slip AS id_slip,
				sa.customer_name AS cust,
				sad.item_name AS prod,
				sad.version_name AS version,
				sad.total_te/sad.quantity AS unit,
				sad.quantity AS qty,
				sad.total_te AS tax_excl,
				sad.total_ti AS tax_incl,
				sad.unit_te*sad.quantity*IFNULL(sad.measure,1) - sad.total_te AS total_disc_te,
				sad.unit_ti*sad.quantity*IFNULL(sad.measure,1) - sad.total_ti AS total_disc_ti,
				sad.purchase_te AS purchase_price,
				sad.purchase_te AS wholesale,
                sad.measure,
                sad.measure_unit,
                sad.measure_precision,
				o.reference AS ref,
				-- IF(cs.oper = "ORDER", 1, -1)*o.total_shipping AS shipping,
				-- IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_excl AS shipping_te,
				gl.name AS `cust_group`,
				od.product_id AS id_prod,
				od.product_attribute_id AS id_attr,
				p.id_category_default AS id_cat,
				col.name AS order_country,
				car.name AS carrier,
				/*cl.name AS cat,*/
				IF(ISNULL(sad.id_carrier), cl.name, sad.item_name) AS cat,
				IF(pa.reference != "", pa.reference, p.reference) AS ref,
                p.reference AS main_ref,
				IF(pa.ean13 != "", pa.ean13, p.ean13) AS ean,
				IF(pa.upc != "", pa.upc, p.upc) as upc,
				su.name AS supplier,
				su.id_supplier,
				ma.name AS manufacturer,
				ma.id_manufacturer,
				odk.note AS note,
				ok.delivery_mode AS delivery_mode,
                ' . $field_stock . '
                odk.id_tag
				-- (
				--	SELECT GROUP_CONCAT(op.payment_method) 
				--	FROM '._DB_PREFIX_.'order_payment op
				--	WHERE op.order_reference = o.reference AND op.amount > 0
				--	GROUP BY op.order_reference
				-- ) AS payment_method,
				-- ok.is_paid

			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order ord ON ord.id_order = sad.id_order
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_sale = sad.id_sale
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = sa.id_operation
			LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = ord.ps_order
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'address addr ON addr.id_address = o.id_address_delivery
			LEFT JOIN '._DB_PREFIX_.'country_lang col ON col.id_country = addr.id_country AND col.id_lang = '. (int) $this->id_lang .'
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = sa.id_customer
			LEFT JOIN '._DB_PREFIX_.'group_lang gl ON gl.id_group = c.id_default_group AND gl.id_lang = '. (int) $this->id_lang .'
			LEFT JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = sad.id_order_detail
			LEFT JOIN '._DB_PREFIX_.'order_detail_kerawen odk ON odk.id_order_detail = od.id_order_detail
			LEFT JOIN '._DB_PREFIX_.'product p ON p.id_product = od.product_id
			LEFT JOIN '._DB_PREFIX_.'product_shop ps ON ps.id_product = p.id_product AND ps.id_shop = od.id_shop
			LEFT JOIN '._DB_PREFIX_.'product_lang pl ON pl.id_product = p.id_product AND pl.id_shop = ps.id_shop AND pl.id_lang = '. (int) $this->id_lang .'
			LEFT JOIN '._DB_PREFIX_.'product_attribute pa ON pa.id_product_attribute = od.product_attribute_id
			LEFT JOIN '._DB_PREFIX_.'category_lang cl ON cl.id_category = p.id_category_default AND cl.id_shop = ps.id_shop AND cl.id_lang = '. (int) $this->id_lang .'
			LEFT JOIN '._DB_PREFIX_.'supplier su ON su.id_supplier = p.id_supplier
			LEFT JOIN '._DB_PREFIX_.'manufacturer ma ON ma.id_manufacturer = p.id_manufacturer
			LEFT JOIN '._DB_PREFIX_.'carrier car ON car.id_carrier = sad.id_carrier
            ' . $join_stock . '
			-- LEFT JOIN (
			-- 	SELECT
			-- 		sm.id_order, st.id_product, st.id_product_attribute,
			-- 		SUM(sm.physical_quantity*st.price_te)/SUM(sm.physical_quantity) AS price_te
			-- 	FROM '._DB_PREFIX_.'stock_mvt sm
			-- 	JOIN '._DB_PREFIX_.'stock st ON st.id_stock = sm.id_stock
			-- 	GROUP BY sm.id_order, st.id_product, st.id_product_attribute
			-- ) mvt ON mvt.id_order = o.id_order AND mvt.id_product = od.product_id AND mvt.id_product_attribute = od.product_attribute_id
			
			WHERE sad.gift_card = 0 AND sad.account_credit = 0
			AND '.$this->opfilter.' '.$this->safilter);

	}


	public function statshour() {
		
		$output = array();
		
		$from = $this->init_from;
		$to = $this->init_to;

		//start period from v 2.0
		$from = $this->db->getValue("
			SELECT IF(MIN(DATE) > '" . pSQL($this->init_from) . "', MIN(DATE), '" . pSQL($this->init_from) . "')
			FROM "._DB_PREFIX_. "kerawen_version
			WHERE STRCMP(VERSION, '2.0') >= 0 AND res = 1"
		);
		
		$refDate = $this->db->getRow("SELECT (DATEDIFF('" . pSQL($to) . "', '" . pSQL($from) . "') + 1 ) AS weeks");
		if ($refDate) {
			if ($refDate['weeks'] > 0) {
				$itemDiv = $refDate['weeks'];
				
				$q = '
					SELECT 
                        ' . $this->total_sale_te . ' AS total_te,
        				' . $this->total_sale_ti . ' AS total_ti,
                        HOUR(op.date) AS period
                    ' . $this->stat_join . '
					WHERE '.$this->init_filter.'
					GROUP BY period
					ORDER BY period ASC';
				
				$data = indexArray($this->db->executeS($q), 'period');
				
				for ($i=0 ; $i<24 ; $i++) {
					$output[] = isset($data[$i]) ?
					array(
						'period' => $data[$i]['period'],
						'total_ti' => $data[$i]['total_ti'] / $itemDiv,
						'total_te' => $data[$i]['total_te'] / $itemDiv
					) :
					array(
						'period' => $i,
						'total_ti' => "0",
						'total_te' => "0"
					);
				}

			}
		}

		return $output;
	}

	
	public function statsweek() {
		
		$output = array();

		$from = $this->init_from;
		$to = $this->init_to;
		
		//start period from v 2.0
		$from = $this->db->getValue("
			SELECT IF(MIN(DATE) > '" . pSQL($this->init_from) . "', MIN(DATE), '" . pSQL($this->init_from) . "')
			FROM "._DB_PREFIX_. "kerawen_version
			WHERE STRCMP(VERSION, '2.0') >= 0 AND res = 1"
		);
		
		$refDate = $this->db->getRow("SELECT (DATEDIFF('" . pSQL($to) . "', '" . pSQL($from) . "') + 1 ) / 7 AS weeks, WEEKDAY('" . pSQL($from) . "') AS wd");
		if ($refDate) {
			//date from < date_to
			if ($refDate['weeks'] > 0) {
				$arrayDiv = array();
				for ($j = 0; $j<7; $j++) {
					$arrayDiv[$j] = max(($j < $refDate['wd']) ? floor($refDate['weeks']) : ceil($refDate['weeks']), 1);
				}
	
				$q = '
					SELECT 
                        ' . $this->total_sale_te . ' AS total_te,
        				' . $this->total_sale_ti . ' AS total_ti,
                        WEEKDAY(op.date) AS period
                    ' . $this->stat_join . '
					WHERE '.$this->init_filter.'
					GROUP BY period
					ORDER BY period ASC';
				
				$data =  indexArray($this->db->executeS($q), 'period');
				
				for ($i = 0; $i<7; $i++) {
					$output[] = isset($data[$i]) ? 
						array(
							'period' => $data[$i]['period'], 
							'total_ti' => $data[$i]['total_ti'] / $arrayDiv[$i], 
							'total_te' => $data[$i]['total_te'] / $arrayDiv[$i]
						) :
						array(
							'period' => $i, 
							'total_ti' => "0",
							'total_te' => "0"
						);
				}
			}
		}	
			
		return $output;
	}
	
	
	public function statsmonth() {
		$q = '
			SELECT
                ' . $this->total_sale_te . ' AS total_te,
				' . $this->total_sale_ti . ' AS total_ti,
				UNIX_TIMESTAMP(DATE_FORMAT(op.date, "%Y-%m-%d")) AS ref,
				DATE_FORMAT(op.date, "%Y-%m-%d") AS period
            ' . $this->stat_join . '
			WHERE '.$this->init_filter.'
			GROUP BY period
			ORDER BY period ASC
		';

		$data =  indexArray($this->db->executeS($q), 'period');
		
		$delta = $this->db->getRow('SELECT UNIX_TIMESTAMP(DATE_FORMAT("' . pSQL($this->init_from) . '", "%Y-%m-%d")) AS ut_from, UNIX_TIMESTAMP(DATE_FORMAT("' . pSQL($this->init_to) . '", "%Y-%m-%d")) AS ut_to');
		$ut_from = $delta['ut_from'];
		$ut_to = $delta['ut_to'];
		
		$output = array();
		while ($ut_from <= $ut_to) {
			$period = date('Y-m-d', $ut_from);
			$output[] = isset($data[$period])
				? array( 'period' => strtotime($period), 'total_ti' => $data[$period]['total_ti'], 'total_te' => $data[$period]['total_te'] )
				: array( 'period' => strtotime($period), 'total_ti' => 0, 'total_te' => 0 );
			$ut_from += 86400;
		}
		return $output;
	}
	
	
	public function statsshops() {
	    
	    $shops = $this->db->executeS('
    		SELECT 
                s.name, 
                s.id_shop
    		FROM '._DB_PREFIX_.'shop s
            WHERE s.deleted = 0 AND s.active = 1
            ORDER BY s.id_shop ASC
       ');
	    
	    $data = array();
	    foreach ($shops as $s) {
	        $data[$s['id_shop']] = array('id_shop'=> (int) $s['id_shop']);
	        $data[$s['id_shop']]['ti_p2'] = 0;
	        $data[$s['id_shop']]['te_p2'] = 0;
	        if ($this->from2 && $this->to2) {
	            $data[$s['id_shop']]['ti_p1'] = 0;
	            $data[$s['id_shop']]['te_p1'] = 0;
	        }
	    }

	    $q = '
    	    SELECT
                ' . $this->total_sale_te . ' AS total_te,
				' . $this->total_sale_ti . ' AS total_ti,
                op.id_shop AS id_shop
            ' . $this->stat_join . '
    	    WHERE %s ' . $this->stats_filter . ' ' . $this->safilter . '
    	    GROUP BY op.id_shop
	    ';
	    
	    //N-1
	    if ($this->from2 && $this->to2) {
	        $where_p1 = ' op.date BETWEEN "'.pSQL($this->from2).'"  AND "'.pSQL($this->to2).'" ';
	        $res = $this->db->executeS(sprintf($q, $where_p1));
	        foreach ($res as $item) {
	            if (isset($data[$item['id_shop']])) {
	                $data[$item['id_shop']]['ti_p1'] = $item['total_ti'];
	                $data[$item['id_shop']]['te_p1'] = $item['total_te'];
	            }
	        }
	    }

	    //N
	    $where_p2 = ' op.date BETWEEN "'.pSQL($this->init_from).'" AND "'.pSQL($this->init_to).'" ';	    
	    $res = $this->db->executeS(sprintf($q, $where_p2));
	    foreach ($res as $item) {
	        if (isset($data[$item['id_shop']])) {
	            $data[$item['id_shop']]['ti_p2'] = $item['total_ti'];
	            $data[$item['id_shop']]['te_p2'] = $item['total_te'];
	        }
	    }

	    return $data;

	}
	

	public function statsshopsweb() {
	    
	    //0 => web
	    //1 => shop
	    $data = array();
	    $data[0] = array('item'=> "Shop");
	    $data[0]['ti_p2'] = 0;
	    $data[0]['te_p2'] = 0;
	    $data[1] = array('item'=> "Web");
	    $data[1]['ti_p2'] = 0;
	    $data[1]['te_p2'] = 0;
	    
	    if ($this->from2 && $this->to2) {
	        $data[0]['ti_p1'] = 0;
	        $data[0]['te_p1'] = 0;
	        $data[1]['ti_p1'] = 0;
	        $data[1]['te_p1'] = 0;
	    }
	    
	    $q = '
    	    SELECT
                ' . $this->total_sale_te . ' AS total_te,
				' . $this->total_sale_ti . ' AS total_ti,
                op.id_till AS id_till,
				ck.tillgroup
            ' . $this->stat_join . '
            LEFT JOIN '._DB_PREFIX_.'cash_drawer_kerawen ck ON ck.id_cash_drawer = op.id_till
    	    WHERE %s ' . $this->stats_filter . ' ' . $this->safilter . '
    	    GROUP BY op.id_till
	    ';
	    

	    //N-1
	    if ($this->from2 && $this->to2) {
	        $where_p1 = ' op.date BETWEEN "'.pSQL($this->from2).'"  AND "'.pSQL($this->to2).'" ';

	        
	        $res = $this->db->executeS(sprintf($q, $where_p1));
	        foreach ($res as $item) {
	            if ((int)$item['id_till'] > 0 && (int)$item['tillgroup'] == 0) {
	                $data[0]['ti_p1'] += $item['total_ti'];
	                $data[0]['te_p1'] += $item['total_te'];
	            } else {
	                $data[1]['ti_p1'] += $item['total_ti'];
	                $data[1]['te_p1'] += $item['total_te'];
	            }  
	        }
	    }
	    
	    //N
	    $where_p2 = ' op.date BETWEEN "'.pSQL($this->init_from).'" AND "'.pSQL($this->init_to).'" ';
	    $res = $this->db->executeS(sprintf($q, $where_p2));
	    
	    foreach ($res as $item) {
    	    if ((int)$item['id_till'] > 0 && (int)$item['tillgroup'] == 0) {
    	       $data[0]['ti_p2'] += $item['total_ti'];
    	       $data[0]['te_p2'] += $item['total_te'];
    	    } else {
    	       $data[1]['ti_p2'] += $item['total_ti'];
    	       $data[1]['te_p2'] += $item['total_te'];
    	    }
	    }
	    
	    return $data;
	    
	}
	

	public function statstotal() {

	    $count_cart = 'IFNULL(COUNT(DISTINCT(sa.id_sale)), 0)';
	    $data = array();

	    $q = '
            SELECT
                ' . $this->total_sale_te . ' AS total_te,
				' . $this->total_sale_ti . ' AS total_ti,
                ' . $this->total_sale_ti . '/' . $count_cart . ' AS avg_cart_ti,
                ' . $this->total_sale_te . '/' . $count_cart . ' AS avg_cart_te,
                IFNULL(SUM(IF(sad.quantity < 0, -sad.quantity, 0)), 0) AS count_ret,
                IFNULL(SUM(IF(sad.quantity > 0, sad.quantity, 0)), 0) AS count_prod,
                ' . $this->total_ret_te . '/' . $count_cart . ' AS avg_ret_te,
                ' . $this->total_ret_ti . '/' . $count_cart . ' AS avg_ret_ti
            ' . $this->stat_join . '

            WHERE  %s '.$this->stats_filter.' '.$this->safilter . '
        ';	    


	    $q2 = '
    	    SELECT
    	       COUNT(DISTINCT(ps_order)) AS carts
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
    	    JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON op.id_operation = sa.id_operation /*AND sa.nb_lines > 0*/
    	    JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order o ON o.id_sale = sa.id_sale AND o.ps_order > 0
            WHERE  %s '.$this->stats_filter.' '.$this->safilter . '
	    ';
	    	    
	    $where_p2 = ' op.date BETWEEN "'.pSQL($this->init_from).'" AND "'.pSQL($this->init_to).'" ';
	    
	    $res = $this->db->getRow(sprintf($q, $where_p2));
	    $carts = (int) $this->db->getvalue(sprintf($q2, $where_p2));

	    $data['total_cart']['ti_p2'] = $res ? $res['total_ti'] : 0;
	    $data['total_cart']['te_p2'] =  $res ? $res['total_te'] : 0;
	    $data['count_cart']['p2'] =  $carts;
	    $data['avg_cart']['ti_p2'] =  $res ? $res['avg_cart_ti'] : 0;
	    $data['avg_cart']['te_p2'] =  $res ? $res['avg_cart_te'] : 0;
	    $data['count_ret']['p2'] =  $res ? $res['count_ret'] : 0;
	    $data['count_prod']['p2'] =  $res ? $res['count_prod'] : 0;
	        
	    $data['avg_ret']['ti_p2'] =  $res ? $res['avg_ret_ti'] : 0;
	    $data['avg_ret']['te_p2'] =  $res ? $res['avg_ret_te'] : 0;
	    $data['avg_items']['p2'] = ($res && $carts > 0) ? $res['count_prod']/$carts : 0;


	    //N-1
	    if ($this->from2 && $this->to2) {
	        $where_p1 = ' op.date BETWEEN "'.pSQL($this->from2).'"  AND "'.pSQL($this->to2).'" ';
	        $res = $this->db->getRow(sprintf($q, $where_p1));
	        $carts = (int) $this->db->getvalue(sprintf($q2, $where_p1));

	        $data['total_cart']['ti_p1'] = $res ? $res['total_ti'] : 0;
	        $data['total_cart']['te_p1'] =  $res ? $res['total_te'] : 0;
	        $data['count_cart']['p1'] =  $carts;
	        $data['avg_cart']['ti_p1'] =  $res ? $res['avg_cart_ti'] : 0;
	        $data['avg_cart']['te_p1'] =  $res ? $res['avg_cart_te'] : 0;
	        $data['count_ret']['p1'] =  $res ? $res['count_ret'] : 0;
	        $data['count_prod']['p1'] =  $res ? $res['count_prod'] : 0;
	            
	        $data['avg_ret']['ti_p1'] =  $res ? $res['avg_ret_ti'] : 0;
	        $data['avg_ret']['te_p1'] =  $res ? $res['avg_ret_te'] : 0;
	        $data['avg_items']['p1'] = ($res && $carts > 0) ? $res['count_prod']/$carts : 0;

	    }
	    
	    return $data;
	}
	
	
	public function cats() {
		return sharedCats();
	}


	public function getAccountingData() {
		$delta = $this->db->getRow('SELECT UNIX_TIMESTAMP(DATE_FORMAT("' . pSQL($this->init_from) . '", "%Y-%m-%d")) AS ut_from, UNIX_TIMESTAMP(DATE_FORMAT("' . pSQL($this->init_to) . '", "%Y-%m-%d")) AS ut_to');
		$ut_from = $delta['ut_from'];
		$ut_to = $delta['ut_to'];
		
		$by = $this->group_by;
		
		$empty_row = array();
		$empty_row['id'] = '';
		
		$C = ' C';
		$D = ' D';
		
		$fieldsCols = array();
		if ($by == 'opcd') {
		    $C = $D = '';
		    $fieldsCols = array(
		        'date' => '',
		        'shop' => '',
		        'till' => '',
		        'employee' => '',
		        'customer' => '',
		        'company' => '',
		        'vat_num' => '',
		        'id_order' => ''
		    );
		    $empty_row = array_merge($empty_row, $fieldsCols);
		}

		// Only taxes used
		$taxes = indexArray($this->db->executeS('
			SELECT id_tax, MAX(IFNULL(tax_rate, 0)) AS tax_rate
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
			GROUP BY id_tax'), 'id_tax');
		foreach($taxes as $tax) {
		    $empty_row['HT '.round($tax['tax_rate'],2).$C] = '';
		    $empty_row['HT '.round($tax['tax_rate'],2).$D] = '';
			$empty_row['TVA '.round($tax['tax_rate'],2).$C] = '';
			$empty_row['TVA '.round($tax['tax_rate'],2).$D] = '';
		}

		$empty_row['CA'] = 0.0;
		$empty_row['BC'] = 0.0;
		$empty_row['Encaissement'] = 0.0;

		// Payment modes
		$modes_by_id = array();
		$all_modes = array();
		
		// Get standard active modes
		$payments = json_decode(Configuration::get('KERAWEN_PAYMENTS'), true);
		foreach($payments as $item) {
		    if ($item['id'] == _KERAWEN_PM_PAY_LATER_) continue;
		    if ($item['id'] == _KERAWEN_PM_REFUND_LATER_) continue;
		    if ($item['payment'] || $item['refund']) {
		        $mode = $item['label'];
		        $modes_by_id[$item['id']] = $mode;
		        $all_modes[] = $mode;
		    }
		}
		
		// Get additional payment modes registered within the period
		$buf = $this->db->executeS('
			SELECT p.id_mode, p.mode
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p
            JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = p.id_operation
            WHERE '.$this->opfilter);
		foreach($buf as $item) {
		    if ($item['id_mode']) {
		        $id = $item['id_mode'];
		        if (!isset($modes_by_id[$id])) {
		            $mode = $item['mode'];
		            $modes_by_id[$id] = $mode;
		            $all_modes[] = $mode;
		        }
		    }
		    else if (!array_search($item['mode'], $all_modes)) {
		        $all_modes[] = $item['mode'];
		    }
		}
		// Add special modes
		$buf = array('Carte cadeau', 'Compte client', 'Différé', 'Attente');
		foreach($buf as $item) {
		    if (!array_search($item, $all_modes)) $all_modes[] = $item;
		}
		
		// Make modes case & accent insensitive
		$map_modes = array();
		foreach($all_modes as $mode) {
		    $map_modes[strtoupper($mode)] = $mode;
		    $empty_row[$mode.$C] = '';
		    $empty_row[$mode.$D] = '';
		}
		
		$output = array();
		
		if ($by == 'op' || $by == 'opcd') {
		    $empty_row['invoice'] = '';
		}
		    
		$with_carrier = false;
		if ($by == 'opcd' && $with_carrier) {
		    $empty_row['carrier_te'] = 0;
		}
		    
		$periods = $this->report($by, 0, 0);

		if (isset($periods['details'])) {
			foreach($periods['details'] as $key => $item) {
			    
				$output[$key] = $empty_row;
				$output[$key]['id'] = $key;
				$empty = true;
				
				if ($by == 'op' || $by == 'opcd' )
				    $output[$key]['invoice'] = $item['invoice'];
				
				if ($by == 'opcd')
				    foreach ($fieldsCols as $k => $v)
				        $output[$key][$k] = $item[$k];
				
				// Products
				if (isset($item['products'])) {
					foreach($item['products'] as &$detail) {
						if ((int)$detail['accounted']) {
							if ((int)$detail['sign']) {
							    $sign = ($C == $D) ? 1 : (int)$detail['sign'];
							    $cat = round($taxes[$detail['id_tax']]['tax_rate'],2).((int)$detail['sign'] > 0 ? $C : $D);
							    $output[$key]['HT '.$cat] = (float)$output[$key]['HT '.$cat] + $sign*$detail['total_te'];
							    $output[$key]['TVA '.$cat] = (float)$output[$key]['TVA '.$cat] + $sign*($detail['total_ti'] - $detail['total_te']);
							    $output[$key]['CA'] = (float)$output[$key]['CA'] + $detail['total_ti'];
							}
						}
						else {
						    $output[$key]['BC'] = (float)$output[$key]['BC'] + $detail['total_ti'];
						}
						$empty = false;
					}
				}
				// Shipping
				if ($by == 'opcd' && $with_carrier) {
				    if (isset($item['carriers'])) {
				        $output[$key]['carrier_te'] = (float)$output[$key]['carrier_te'] + $item['carriers']['amount_te'];
				        $empty = false;
				    }
				}
					
				// Payments
				if (isset($item['payments']) && isset($item['payments']['details'])) {
					foreach($item['payments']['details'] as &$detail) {
						if ((int)$detail['sign']) {
						    $sign = ($C == $D) ? 1 : (int)$detail['sign'];
						    $mode = isset($modes_by_id[$detail['id_mode']]) ? $modes_by_id[$detail['id_mode']] : $detail['mode'];
						    $cat = $map_modes[strtoupper($mode)].((int)$detail['sign'] > 0 ? $D : $C);
							$output[$key][$cat] = (float)$output[$key][$cat] + $sign*(float)$detail['amount'];
							$output[$key]['Encaissement'] = (float)$output[$key]['Encaissement'] + $detail['amount'];
							
							if (isset($output[$key]['id_order'])) {
							    $output[$key]['id_order'] = $output[$key]['id_order'].','.$detail['id_order'];
							}
							$empty = false;
						}
					}
				}
				
				if (isset($item['deferred'])) {
				    foreach($item['deferred'] as &$detail) {
				        if ((int)$detail['sign']) {
				            $sign = ($C == $D) ? 1 : $detail['sign'];
				            $cat = 'Différé'.((int)$detail['sign'] > 0 ? $D : $C);
				            $output[$key][$cat] = (float)$output[$key][$cat] + $sign*$detail['amount'];
				            $output[$key]['Encaissement'] = (float)$output[$key]['Encaissement'] + $detail['amount'];
				            $empty = false;
				        }
				    }
				}
				
				if (isset($item['gcards'])) {
				    $sign = ($C == $D) ? -1 : +1;
				    $output[$key]['Carte cadeau'.$C] = (float)$output[$key]['Carte cadeau'.$C] + $sign*$item['gcards']['amount'];
				    $output[$key]['Encaissement'] = (float)$output[$key]['Encaissement'] - $item['gcards']['amount'];
				    $empty = false;
				}
				
				if (isset($item['acredits'])) {
				    $sign = ($C == $D) ? -1 : +1;
				    $output[$key]['Compte client'.$C] = (float)$output[$key]['Compte client'.$C] + $sign*$item['acredits']['amount'];
				    $output[$key]['Encaissement'] = (float)$output[$key]['Encaissement'] - $item['acredits']['amount'];
				    $empty = false;
				}
				
				if (isset($output[$key]['id_order'])) {
				    // Keep distinct order numbers
				    $output[$key]['id_order'] = implode(',', array_filter(array_unique(explode(',', $output[$key]['id_order']))));
				    $empty = false;
				}
				
				// Flows
				if (isset($item['flows']) && isset($item['flows']['details'])) {
				    foreach($item['flows']['details'] as &$detail) {
				        if ((int)$detail['sign']) {
				            $mode = isset($modes_by_id[$detail['id_mode']]) ? $modes_by_id[$detail['id_mode']] : $detail['mode'];
				            $sign = ($C == $D) ? 1 : (int)$detail['sign'];
				            
				            $cat = $map_modes[strtoupper($mode)].((int)$detail['sign'] > 0 ? $D : $C);
				            $output[$key][$cat] = (float)$output[$key][$cat] + $sign*(float)$detail['amount'];
				            
				            $cat = $map_modes[strtoupper('Attente')].((int)$detail['sign'] > 0 ? $C : $D);
				            $output[$key][$cat] = (float)$output[$key][$cat] + $sign*(float)$detail['amount'];

				            $empty = false;
				        }
				    }
				}
				
				// Remove entry if every thing is null
				if ($empty) unset($output[$key]);
			}
		}
        return array_values($output);
	}
	
	
	protected function report($by, $first, $size) {
		require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
		$limit = ($size == 0) ? '' : 'LIMIT '. (int) $first.', '. (int) $size;

		switch ($by) {
			case 'day':
				$group = 'DATE_FORMAT(op.date, "%Y-%m-%d")';
				$index = 'date';
				$info = '
					COUNT(op.id_operation) AS nb_op,
					DATE_FORMAT(op.date, "%Y-%m-%d") AS fdate';
				break;
				
			case 'op':
			default:
				$group = 'op.id_operation';
				$index = 'id_op';
				$info = '
					op.date AS date,
					op.operator_name,
					op.till_name,
					op.PS_SHOP_NAME';
				break;
		}
		
		$filter = $this->opfilter; //.' AND op.type = "'._KERAWEN_525_OP_SALE_.'"';

		$total = $this->db->getRow('
			SELECT COUNT(DISTINCT '.$group.') AS count
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
			WHERE '.$filter);
		
		$total = array_merge($total, $this->db->getRow('
			SELECT
				SUM(sa.total_te) AS total_te,
				SUM(sa.total_ti) AS total_ti,
				SUM(sa.total_ti - sa.total_te) AS total_tax
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
			WHERE '.$filter));
		
		$total['taxes'] = $this->db->getRow('
			SELECT
				SUM(sat.total_te) AS total_te,
				SUM(sat.total_ti) AS total_ti,
				SUM(sat.total_ti - sat.total_te) AS total_tax
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_tax sat ON sat.id_sale = sa.id_sale
			WHERE '.$filter);
		
		$total['taxes']['details'] = indexArray($this->db->executeS('
			SELECT
				sat.id_tax AS id_tax,
				sat.tax_rate AS tax_rate,
				SUM(sat.total_te) AS total_te,
				SUM(sat.total_ti) AS total_ti,
				SUM(sat.total_ti - sat.total_te) AS total_tax
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_tax sat ON sat.id_sale = sa.id_sale
			WHERE '.$filter.'
			GROUP BY sat.id_tax'), 'id_tax');
		
		$total['payments'] = $this->db->getRow('
			SELECT SUM(pa.amount) AS amount
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment pa ON pa.id_operation = op.id_operation
			WHERE '.$filter);
		
		$total['payments']['details'] = indexArray($this->db->executeS('
			SELECT
				pa.id_mode,
                pa.mode,
				SUM(pa.amount) AS amount
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment pa ON pa.id_operation = op.id_operation
			WHERE '.$filter.'
			GROUP BY pa.id_mode, pa.mode'), 'mode');
		
		$total['flows'] = $this->db->getRow('
			SELECT SUM(f.amount) AS amount
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow f ON f.id_operation = op.id_operation
			WHERE '.$filter);
		
		$total['payments']['details'] = indexArray($this->db->executeS('
			SELECT
				f.id_mode,
                f.mode,
				SUM(f.amount) AS amount
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow f ON f.id_operation = op.id_operation
			WHERE '.$filter.'
			GROUP BY f.id_mode, f.mode'), 'mode');
		
		$details = $this->db->executeS('
			SELECT '.$group.' AS '.$index.', '.$info.'
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
			WHERE '.$filter.'
			GROUP BY '.$group.' '.$limit);
		
		if (count($details)) {
			$min = $details[0][$index];
			$max = $details[count($details)-1][$index];
			$filter = $filter.'
					AND '.$group.' >= "'.$min.'" AND '.$group.' <= "'.$max.'"';
			
			$details = indexArray($details, $index);
			
			$sales = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					SUM(sa.total_te) AS total_te,
					SUM(sa.total_ti) AS total_ti,
					SUM(sa.total_ti - sa.total_te) AS total_tax
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
				WHERE '.$filter.'
				GROUP BY '.$group);
			foreach($sales as &$item) {
				$details[$item[$index]] = array_merge($details[$item[$index]], $item);
			}

			if ($by == 'op' || $by == 'opcd') {
			    
    			$invoices = $this->db->executeS('
    				SELECT 
    					op.id_operation AS id_op,
    					GROUP_CONCAT(o.number) AS invoice,
                        op.PS_SHOP_NAME AS shop,
                        op.till_name AS till,
                        op.operator_name AS employee,
                        GROUP_CONCAT(o.ps_order) AS id_order,
                        sa.customer_name AS customer,
                        c.company,
                        IF(o.ps_slip > 0, "", a.vat_number) AS vat_num
    				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
    				LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
    				LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order o ON o.id_sale = sa.id_sale
                    LEFT JOIN '._DB_PREFIX_.'customer c ON sa.id_customer = c.id_customer
			        LEFT JOIN '._DB_PREFIX_.'orders pso ON pso.id_order = o.ps_order
                    LEFT JOIN '._DB_PREFIX_.'address a ON pso.id_address_invoice = a.id_address
    				WHERE '.$filter.'
    				GROUP BY '.$group);

    			foreach($invoices as &$item) {
    			    $details[$item[$index]] = array_merge($details[$item[$index]], $item);
    			}
		    }

			$taxes = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					SUM(sat.total_te) AS total_te,
					SUM(sat.total_ti) AS total_ti,
					SUM(sat.total_ti - sat.total_te) AS total_tax
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_tax sat ON sat.id_sale = sa.id_sale
				WHERE '.$filter.'
				GROUP BY '.$group);
			foreach($taxes as &$item) {
				$details[$item[$index]]['taxes'] = $item;
				$details[$item[$index]]['taxes']['details'] = array();
			}
			
			$taxes = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					sat.id_tax AS id_tax,
					sat.tax_rate AS tax_rate,
					SUM(sat.total_te) AS total_te,
					SUM(sat.total_ti) AS total_ti,
					SUM(sat.total_ti - sat.total_te) AS total_tax
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_tax sat ON sat.id_sale = sa.id_sale
				WHERE '.$filter.'
				GROUP BY '.$group.', sat.id_tax');
			foreach($taxes as &$item) {
				$details[$item[$index]]['taxes']['details'][$item['id_tax']] = $item;
			}
			
			$products = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					sad.id_tax AS id_tax,
					sad.tax_rate AS tax_rate,
					SIGN(sad.total_ti) AS sign,
					SUM(sad.total_te) AS total_te,
					SUM(sad.total_ti) AS total_ti,
					SUM(sad.total_ti - sad.total_te) AS total_tax,
					sa.accounted AS accounted
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad ON sad.id_sale = sa.id_sale
				WHERE '.$filter.'
					AND sad.gift_card = 0 AND sad.account_credit = 0
				GROUP BY '.$group.', sa.accounted, sad.id_tax, SIGN(sad.total_ti)');
			foreach($products as &$item) {
				if (!isset($details[$item[$index]]['products'])) $details[$item[$index]]['products'] = array();
				$details[$item[$index]]['products'][] = $item;
			}
			
			$carriers = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					SUM(sad.total_ti) AS amount_ti,
                    SUM(sad.total_te) AS amount_te
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad ON sad.id_sale = sa.id_sale
				WHERE '.$filter.'
					AND sad.id_carrier IS NOT NULL
				GROUP BY '.$group);
			foreach($carriers as &$item) {
			    $details[$item[$index]]['carriers'] = $item;
			}
			
			$gcards = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					SUM(sad.total_ti) AS amount
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad ON sad.id_sale = sa.id_sale
				WHERE '.$filter.'
					AND sad.gift_card = 1
				GROUP BY '.$group);
			foreach($gcards as &$item) {
				$details[$item[$index]]['gcards'] = $item;
			}
			
			$acredits = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					SUM(sad.total_ti) AS amount
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa ON sa.id_operation = op.id_operation
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad ON sad.id_sale = sa.id_sale
				WHERE '.$filter.'
					AND sad.account_credit = 1
				GROUP BY '.$group);
			foreach($acredits as &$item) {
			    $details[$item[$index]]['acredits'] = $item;
			}
			
			$deferred = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					IF(IFNULL(due.due, 0) = 0, 0, 1) AS prod,
					SIGN(IFNULL(due.due, 0) - IFNULL(paid.paid, 0)) AS sign,
					SUM(IFNULL(due.due, 0) - IFNULL(paid.paid, 0)) AS amount
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				LEFT JOIN (
					SELECT
						sa.id_operation AS id_operation,
						SUM(sad.total_ti) AS due
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale sa
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sad ON sad.id_sale = sa.id_sale
					GROUP BY sa.id_operation) due ON due.id_operation = op.id_operation
				LEFT JOIN (
					SELECT
						pa.id_operation AS id_operation,
						SUM(pa.amount) AS paid
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment pa
					GROUP BY pa.id_operation) paid ON paid.id_operation = op.id_operation
				WHERE '.$filter.'
				GROUP BY '.$group.', prod, sign
				HAVING amount != 0');
			foreach($deferred as &$item) {
				if (!isset($details[$item[$index]]['deferred'])) $details[$item[$index]]['deferred'] = array();
				$details[$item[$index]]['deferred'][] = $item;
			}
			
			$payments = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					SUM(pa.amount) AS amount,
                    GROUP_CONCAT(pa.ps_orders) AS id_order
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment pa ON pa.id_operation = op.id_operation
				WHERE '.$filter.'
				GROUP BY '.$group);
			foreach($payments as &$item) {
				$details[$item[$index]]['payments'] = $item;
				$details[$item[$index]]['payments']['details'] = array();
			}
			
			$payments = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					pa.id_mode,
					pa.mode,
					SIGN(pa.amount) AS sign,
					SUM(pa.amount) AS amount,
                    GROUP_CONCAT(pa.ps_orders) AS id_order
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment pa ON pa.id_operation = op.id_operation
				WHERE '.$filter.'
				GROUP BY '.$group.', pa.id_mode, pa.mode, SIGN(pa.amount)');
			foreach($payments as &$item) {
				$details[$item[$index]]['payments']['details'][] = $item;
			}

			$flows = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					SUM(f.amount) AS amount
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow f ON f.id_operation = op.id_operation
				WHERE '.$filter.'
				GROUP BY '.$group);
			foreach($flows as &$item) {
			    $details[$item[$index]]['flows'] = $item;
			    $details[$item[$index]]['flows']['details'] = array();
			}
			
			$flows = $this->db->executeS('
				SELECT '.$group.' AS '.$index.',
					f.id_mode,
					f.mode,
					SIGN(f.amount) AS sign,
					SUM(f.amount) AS amount
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow f ON f.id_operation = op.id_operation
				WHERE '.$filter.'
				GROUP BY '.$group.', f.id_mode, f.mode, SIGN(f.amount)');
			foreach($flows as &$item) {
			    $details[$item[$index]]['flows']['details'][] = $item;
			}
		}
		
		return array(
			'total' => $total,
			'details' => $details,
		);
	}
	
	public function getAccountingLog() {
	    static $FEC_SEP = "\t";
	    
	    static $FEC_JOURNAL_CODE = 0;
	    static $FEC_JOURNAL_LIB = 1;
	    static $FEC_ECRITURE_NUM = 2;
	    static $FEC_ECRITURE_DATE = 3;
	    static $FEC_COMPTE_NUM = 4;
	    static $FEC_COMPTE_LIB = 5;
	    static $FEC_PIECE_REF = 8;
	    static $FEC_PIECE_DATE = 9;
	    static $FEC_ECRITURE_LIB = 10;
	    static $FEC_DEBIT = 11;
	    static $FEC_CREDIT = 12;
	    static $FEC_NB_COLS = 18;
	    
	    static $header = [
	        "JournalCode",      // Alphanumérique	1. Le code journal de l'écriture comptable
	        "JournalLib",       // Alphanumérique	2. Le libellé journal de l'écriture comptable
	        "EcritureNum",      // Alphanumérique	3. Le numéro sur une séquence continue de l'écriture comptable
	        "EcritureDate",     // Date	            4. La date de comptabilisation de l'écriture comptable
	        "CompteNum",        // Alphanumérique	5. Le numéro de compte, dont les 3 premiers caractères doivent correspondre à des chiffres respectant les normes du plan comptable français
	        "CompteLib",        // Alphanumérique	6. Le libellé de compte, conformément à la nomenclature du plan comptable français
	        "CompAuxNum",       // Alphanumérique	7. Le numéro de compte auxiliaire (à blanc si non utilisé)
	        "CompAuxLib",       // Alphanumérique	8. Le libellé de compteauxiliaire (à blanc si non utilisé)
	        "PieceRef",         // Alphanumérique	9. La référence de la pièce justificative
	        "PieceDate",        // Date	            10. La date de la pièce justificative
	        "EcritureLib",      // Alphanumérique	11. Le libellé de l'écriture comptable
	        "Debit",            // Numérique    	12. Le montant au débit
	        "Credit",           // Numérique    	13. Le montant au crédit
	        "EcritureLet",      // Alphanumérique	14. Le lettrage de l'écriture comptable (à blanc si non utilisé)
	        "DateLet",          // Date         	15. La date de lettrage (à blanc si non utilisé)
	        "ValidDate",        // Date         	16. La date de validation de l'écriture comptable
	        "Montantdevise",    // Numérique    	17. Le montant en devise (à blanc si non utilisé)
	        "Idevise",          // Alphanumérique	18. L'identifiant de la devise (à blanc si non utilisé)
	    ];
	    
	    $db = Db::getInstance();
	    
	    // Filter
	    $filter = $this->opfilter;
	    
	    // Grouping
	    $ref = 'op.id_operation';
	    if ($this->group_by == 'day') $ref = 'DATE_FORMAT(op.date, "%Y%m%d")';
	    
	    // Tax computation mode
	    $tax_address = Configuration::get('PS_TAX_ADDRESS_TYPE');
	    
	    // Not buffered parallel queries
	    $db1 = $db->connect();
	    $db1->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, false);
	    $op_query = $db1->query('
            SELECT
                '.$ref.' AS ref,
                DATE_FORMAT(op.date, "%Y%m%d") AS date
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
            WHERE '.$filter.'
            ORDER BY '.$ref.' ASC');
	    
	    $db2 = $db->connect();
	    $db2->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, false);
	    $prod_query = $db2->query('
            SELECT
                '.$ref.' AS ref,
                s.id_sale,
                i.id_order,
                i.ps_order,
                i.ps_slip,
                i.number,
                sd.id_sale_detail,
                sd.total_te AS te,
                sd.total_ti AS ti,
                sd.total_ti - sd.total_te AS tax,
                sd.tax_rate,
                p.id_category_default AS id_cat,
                sd.id_carrier,
                sd.wrapping,
                sd.gift_card,
                sd.account_credit,
                c.id_country,
                c.id_zone
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
            JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s ON s.id_operation = op.id_operation
            LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order i ON i.id_sale = s.id_sale
            LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sd ON sd.id_order = i.id_order
            LEFT JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = sd.id_order_detail
            LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = i.ps_order
            LEFT JOIN '._DB_PREFIX_.'address a ON a.id_address = o.'.$tax_address.'
            LEFT JOIN '._DB_PREFIX_.'country c ON c.id_country = a.id_country
            LEFT JOIN '._DB_PREFIX_.'product p ON p.id_product = od.product_id
            WHERE '.$filter.' AND s.accounted = 1
            ORDER BY '.$ref.' ASC, sd.id_sale_detail ASC');
	    
	    $db3 = $db->connect();
	    $db3->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, false);
	    $pay_query = $db3->query('
            SELECT
                '.$ref.' AS ref,
                p.id_payment,
                p.id_mode,
                p.mode,
                p.amount AS pay
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
            JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p ON p.id_operation = op.id_operation
            WHERE '.$filter.'
            ORDER BY '.$ref.' ASC, p.id_payment ASC');
	    
	    $db4 = $db->connect();
	    $db4->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, false);
	    $flow_query = $db4->query('
            SELECT
                '.$ref.' AS ref,
                f.id_tillflow,
                f.id_mode,
                f.mode,
                f.amount AS flow
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
            JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow f ON f.id_operation = op.id_operation
            WHERE '.$filter.'
            ORDER BY '.$ref.' ASC, f.id_tillflow ASC');
	    
	    $payments = json_decode(Configuration::get('KERAWEN_PAYMENTS'), true);
	    $modes_by_id = array();
	    foreach($payments as $item) {
	        $modes_by_id[$item['id']] = $item['label'];
	    }
	    $config = json_decode(Configuration::get('KERAWEN_FEC_ACCOUNTS'), true);
	    $id_fr = isset($config['FR']) ? $config['FR'] : 8;
	    $id_ue = isset($config['UE']) ? $config['UE'] : 1;
	    $accounts = isset($config['Accounts']) ? $config['Accounts'] : array();
	    
	    // Adapt
	    if (isset($accounts['PAIEMENTS'])) {
	        $buf = array();
	        foreach ($accounts['PAIEMENTS'] as $mode => $acc) {
	            $buf[strtoupper($mode)] = $acc;
	        }
	        $accounts['PAIEMENTS'] = $buf;
	    }
	    if (isset($accounts['ATTENTE'])) {
	        $buf = array();
	        foreach ($accounts['ATTENTE'] as $mode => $acc) {
	            $buf[strtoupper($mode)] = $acc;
	        }
	        $accounts['ATTENTE'] = $buf;
	    }
	    
	    $output = array();
	    
	    if ($op_query) {
	        $ec_index = 0;
	        
	        $prod = $prod_query ? $prod_query->fetch(PDO::FETCH_ASSOC) : false;
	        $pay = $pay_query ? $pay_query->fetch(PDO::FETCH_ASSOC) : false;
	        $flow = $flow_query ? $flow_query->fetch(PDO::FETCH_ASSOC) : false;
	        
	        while ($op = $op_query->fetch(PDO::FETCH_ASSOC)) {
	            $ref = $op['ref'];
	            $id_sale = false;
	            
	            $sales = array();
	            $payments = array();
	            $libs = array();
	            
	            while ($prod && $prod['ref'] == $ref) {
	                if (!$id_sale) $id_sale = $prod['id_sale'];
	                // else SOULD ENSURE id_sale is the same
	                
                    // Special products (backard compatibility)
                    if ($prod['gift_card']) {
                        $mode = isset($modes_by_id[_KERAWEN_PM_GIFT_CARD_]) ? $modes_by_id[_KERAWEN_PM_GIFT_CARD_] : 'Carte Cadeau';
                        
                        $acc = 'PAIEMENTS-'.strtoupper($mode);
                        if (!isset($payments[$acc])) $payments[$acc] = 0;
                        $payments[$acc] += round($prod['ti'],2);
                        $libs[$acc] = 'Paiements '.$mode;
                    }
                    else if ($prod['account_credit']) {
                        $mode = isset($modes_by_id[_KERAWEN_PM_ACCOUNT_]) ? $modes_by_id[_KERAWEN_PM_ACCOUNT_] : 'Compte client';
                        
                        $acc = 'PAIEMENTS-'.strtoupper($mode);
                        if (!isset($payments[$acc])) $payments[$acc] = 0;
                        $payments[$acc] += round($prod['ti'],2);
                        $libs[$acc] = 'Paiements '.$mode;
                    }
                    else if ($prod['id_sale_detail']) {
                        $cat = 'Ventes';
                        if ($prod['id_carrier']) $cat = 'Frais de port';
                        if ($prod['wrapping']) $cat = 'Emballage';
                        $dest = $prod['id_country'] == 8 ? 'FR' : ($prod['id_zone'] == 1 ? 'UE' : 'Export');
                        $tax = (float)$prod['tax_rate'];
                        
                        $acc = strtoupper($cat).'-'.strtoupper($dest).'-'.$tax;
                        if (!isset($sales[$acc])) $sales[$acc] = 0;
                        $sales[$acc] += round($prod['te'],2);
                        $libs[$acc] = $cat.' '.$dest.' '.$tax.'%';
                        
                        $acc = 'TAXES-'.$dest.'-'.$tax;
                        if (!isset($sales[$acc])) $sales[$acc] = 0;
                        $sales[$acc] += round($prod['tax'],2);
                        $libs[$acc] = 'TVA '.$dest.' '.$tax.'%';
                    }
	                $prod = $prod_query->fetch(PDO::FETCH_ASSOC);
                }
                
                while ($pay && $pay['ref'] == $ref) {
                    $mode = isset($modes_by_id[$pay['id_mode']]) ? $modes_by_id[$pay['id_mode']] : $pay['mode'];
                    
                    $acc = 'PAIEMENTS-'.strtoupper($mode);
                    if (!isset($payments[$acc])) $payments[$acc] = 0;
                    $payments[$acc] -= round($pay['pay'],2);
                    $libs[$acc] = 'Paiements '.$mode;
                    
                    $pay = $pay_query->fetch(PDO::FETCH_ASSOC);
                }
                
                while ($flow && $flow['ref'] == $ref) {
                    $mode = isset($modes_by_id[$flow['id_mode']]) ? $modes_by_id[$flow['id_mode']] : $flow['mode'];
                    
                    $acc = 'PAIEMENTS-'.strtoupper($mode);
                    if (!isset($payments[$acc])) $payments[$acc] = 0;
                    $payments[$acc] -= round($flow['flow'],2);
                    $libs[$acc] = 'Paiements '.$mode;
                    
                    $acc = 'ATTENTE-'.strtoupper($mode);
                    if (!isset($payments[$acc])) $payments[$acc] = 0;
                    $payments[$acc] += round($flow['flow'],2);
                    $libs[$acc] = 'Attente '.$mode;
                    
                    $flow = $flow_query->fetch(PDO::FETCH_ASSOC);
                }
                
                $balance = 0;
                foreach ($sales as $account => $amount) {
                    if (!$amount) unset($sales[$account]);
                    else $balance += $amount;
                }
                foreach ($payments as $account => $amount) {
                    if (!$amount) unset($payments[$account]);
                    else $balance += $amount;
                }
                $balance = round($balance, 2);
                
                $lines = array_merge($sales, $payments);
                if ($balance != 0) {
                    $lines['BALANCE'] = -$balance;
                    $libs['BALANCE'] = 'Balance';
                }
                
                if (count($lines)) {
                    $ec_index += 1;
                    foreach ($lines as $acc => $amount) {
                        $row = array();
                        foreach ($header as &$h) $row[$h] = '';
                        
                        $buf = $accounts;
                        foreach (explode('-', $acc) as $cat) {
                            if (!isset($buf[$cat])) {
                                $buf = false;
                                break;
                            }
                            $buf = &$buf[$cat];
                        }
                        $account = $buf ? $buf : $acc;
                        
                        $row[$header[$FEC_JOURNAL_CODE]] = 'CA';
                        $row[$header[$FEC_JOURNAL_LIB]] = 'Journal de caisse';
                        $row[$header[$FEC_ECRITURE_NUM]] = $ec_index;
                        $row[$header[$FEC_ECRITURE_DATE]] = $op['date'];
                        $row[$header[$FEC_ECRITURE_LIB]] = '';
                        $row[$header[$FEC_COMPTE_NUM]] = $account;
                        $row[$header[$FEC_COMPTE_LIB]] = $libs[$acc];
                        $row[$header[$FEC_DEBIT]] = $amount < 0 ? -$amount : 0;
                        $row[$header[$FEC_CREDIT]] = $amount > 0 ? $amount : 0;
                        $row[$header[$FEC_PIECE_REF]] = $this->group_by == 'op' ? 'Op #'.$ref.($id_sale ? ' Ticket #'.$id_sale : '') : '';
                        $row[$header[$FEC_PIECE_DATE]] = $this->group_by == 'op' ? $op['date'] : '';
                        
                        $output[] = $row;
                    }
                }
	        }
	    }
	    
	    $db1->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, true);
	    $db2->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, true);
	    $db3->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, true);
	    $db4->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, true);
	    return $output;
	}
	
	/*
	public function getAccountingLogCSV() {
	    static $FEC_SEP = "\t";
	    
	    static $FEC_JOURNAL_CODE = 1;
	    static $FEC_JOURNAL_LIB = 2;
	    static $FEC_ECRITURE_NUM = 3;
	    static $FEC_ECRITURE_DATE = 4;
	    static $FEC_COMPTE_NUM = 5;
	    static $FEC_COMPTE_LIB = 6;
	    static $FEC_PIECE_REF = 9;
	    static $FEC_PIECE_DATE = 10;
	    static $FEC_ECRITURE_LIB = 11;
	    static $FEC_DEBIT = 12;
	    static $FEC_CREDIT = 13;
	    static $FEC_NB_COLS = 18;
	    
	    static $header = [
	        "JournalCode",      // Alphanumérique	1. Le code journal de l'écriture comptable
	        "JournalLib",       // Alphanumérique	2. Le libellé journal de l'écriture comptable
	        "EcritureNum",      // Alphanumérique	3. Le numéro sur une séquence continue de l'écriture comptable
	        "EcritureDate",     // Date	            4. La date de comptabilisation de l'écriture comptable
	        "CompteNum",        // Alphanumérique	5. Le numéro de compte, dont les 3 premiers caractères doivent correspondre à des chiffres respectant les normes du plan comptable français
	        "CompteLib",        // Alphanumérique	6. Le libellé de compte, conformément à la nomenclature du plan comptable français
	        "CompAuxNum",       // Alphanumérique	7. Le numéro de compte auxiliaire (à blanc si non utilisé)
	        "CompAuxLib",       // Alphanumérique	8. Le libellé de compteauxiliaire (à blanc si non utilisé)
	        "PieceRef",         // Alphanumérique	9. La référence de la pièce justificative
	        "PieceDate",        // Date	            10. La date de la pièce justificative
	        "EcritureLib",      // Alphanumérique	11. Le libellé de l'écriture comptable
	        "Debit",            // Numérique    	12. Le montant au débit
	        "Credit",           // Numérique    	13. Le montant au crédit
	        "EcritureLet",      // Alphanumérique	14. Le lettrage de l'écriture comptable (à blanc si non utilisé)
	        "DateLet",          // Date         	15. La date de lettrage (à blanc si non utilisé)
	        "ValidDate",        // Date         	16. La date de validation de l'écriture comptable
	        "Montantdevise",    // Numérique    	17. Le montant en devise (à blanc si non utilisé)
	        "Idevise",          // Alphanumérique	18. L'identifiant de la devise (à blanc si non utilisé)
	    ];
	    
	    $db = Db::getInstance();
	    
	    // Get categories accounting chart
	    $id_country = $id_zone = 0;
	    $config = kerawenUnserializeConfig('KERAWEN_CHART_PRODUCT');
	    foreach ($config as $filter) {
	        if ($filter['id_country']) $id_country = $filter['id_country'];
	        if ($filter['id_zone']) $id_zone = $filter['id_zone'];
	    }
	    
	    $catnum = array();
	    $buf = $db->query('
            SELECT
                c.id_category,
                c.id_parent,
                ac.account_num AS country,
                az.account_num AS zone,
                aw.account_num AS world
            FROM (
                SELECT 0 AS id_category, 0 AS id_parent, 0 AS level_depth
                UNION SELECT id_category, id_parent, level_depth
                FROM '._DB_PREFIX_.'category
            ) c
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting ac ON ac.id_category = c.id_category AND ac.id_country = '.pSQL($id_country).'
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting az ON az.id_category = c.id_category AND az.id_zone = '.pSQL($id_zone).'
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting aw ON aw.id_category = c.id_category AND aw.id_country = 0 AND aw.id_zone = 0
            ORDER BY c.level_depth ASC');
	    foreach ($buf as $cat) {
	        if ($cat['id_category']) {
	            if (empty($cat['country'])) $cat['country'] = $catnum[$cat['id_parent']]['country'];
	            if (empty($cat['zone'])) $cat['zone'] = $catnum[$cat['id_parent']]['zone'];
	            if (empty($cat['world'])) $cat['world'] = $catnum[$cat['id_parent']]['world'];
	        }
	        $catnum[$cat['id_category']] = array(
	            'country' => $cat['country'],
	            'zone' => $cat['zone'],
	            'world' => $cat['world'],
	        );
	    }
	    
	    // Get sale operations on period
	    $filter = $this->opfilter.' AND op.type = "'._KERAWEN_525_OP_SALE_.'"';
	    
	    // Tax computation mode
	    $tax_address = Configuration::get('PS_TAX_ADDRESS_TYPE');
	    
	    // Not buffered parallel queries
	    $db1 = $db->connect();
	    $db1->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, false);
	    $op_query = $db1->query('
            SELECT
                op.id_operation,
                DATE_FORMAT(op.date, "%Y%m%d") AS date
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
            WHERE '.$filter.'
            ORDER BY op.id_operation ASC');
	    
	    $db2 = $db->connect();
	    $db2->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, false);
	    $prod_query = $db2->query('
            SELECT
                op.id_operation,
                s.id_sale,
                i.id_order,
                i.ps_order,
                i.ps_slip,
                i.number,
                sd.id_sale_detail,
                sd.total_te AS te,
                sd.total_ti AS ti,
                sd.total_ti - sd.total_te AS tax,
                p.id_category_default AS id_cat,
                sd.id_carrier,
                sd.wrapping,
                c.id_country,
                c.id_zone,
                IF(sd.id_carrier IS NULL,
                    IFNULL(apc.account_num, IFNULL(apz.account_num, apw.account_num)),
                    IFNULL(akk.account_num, akd.account_num)
                ) AS pnum,
                IFNULL(att.account_num, atd.account_num) AS tnum,
                IFNULL(acc.account_num, IFNULL(acg.account_num, acd.account_num)) AS cnum
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
            JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s ON s.id_operation = op.id_operation
            LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order i ON i.id_sale = s.id_sale
            LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sd ON sd.id_order = i.id_order
            LEFT JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = sd.id_order_detail
            LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = od.id_order
            LEFT JOIN '._DB_PREFIX_.'address a ON a.id_address = o.'.$tax_address.'
            LEFT JOIN '._DB_PREFIX_.'country c ON c.id_country = a.id_country
            LEFT JOIN '._DB_PREFIX_.'product p ON p.id_product = od.product_id
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting apc ON apc.id_product = p.id_product AND apc.id_country = c.id_country
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting apz ON apz.id_product = p.id_product AND apz.id_country = 0 AND apz.id_zone = c.id_zone
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting apw ON apw.id_product = p.id_product AND apw.id_country = 0 AND apw.id_zone = 0
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting akk ON akk.id_carrier = sd.id_carrier
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting akd ON akd.id_carrier = 0
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting att ON att.id_tax = sd.id_tax
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting atd ON atd.id_tax = 0
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting acc ON acc.id_customer = s.id_customer
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting acg ON acg.id_group = s.id_group
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting acd ON acd.id_group = 0
            WHERE '.$filter.' AND s.accounted = 1
            ORDER BY op.id_operation ASC, sd.id_sale_detail ASC');
	    
	    $db3 = $db->connect();
	    $db3->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, false);
	    $pay_query = $db3->query('
            SELECT
                op.id_operation,
                p.id_payment,
                p.amount AS pay,
                IFNULL(apc.account_num, IFNULL(apw.account_num, apd.account_num)) AS pnum
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
            JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p ON p.id_operation = op.id_operation
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting apc ON apc.id_mode = p.id_mode
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting apw ON apw.id_mode IS NULL AND apw.mode = p.mode
            LEFT JOIN '._DB_PREFIX_.'kerawen_accounting apd ON apd.id_mode = 0
            WHERE '.$filter.'
            ORDER BY op.id_operation ASC, p.id_payment ASC');
	    
	    header('Content-Type: application/csv');
	    echo(implode($FEC_SEP, $header).PHP_EOL);
	    
	    if ($op_query) {
	        $sale_index = 0;
	        $pay_index = 0;
	        
	        $prod = $prod_query ? $prod_query->fetch(PDO::FETCH_ASSOC) : false;
	        $pay = $pay_query ? $pay_query->fetch(PDO::FETCH_ASSOC) : false;
	        
	        while ($op = $op_query->fetch(PDO::FETCH_ASSOC)) {
	            $id_op = $op['id_operation'];
	            $cust_account = false;
	            $id_sale = false;
	            
	            while ($prod && $prod['id_operation'] == $id_op) {
	                if (!$cust_account) $cust_account = $prod['cnum'];
	                // else SOULD ENSURE cnum is the same
	                if (empty($cust_account)) $cust_account = 'CLIENT';
	                
	                if (!$id_sale) $id_sale = $prod['id_sale'];
	                // else SOULD ENSURE cnum is the same
	                
	                // Sales log: 1 EC per order/slip
	                $id_order = $prod['id_order'];
	                $invoice = $prod['number'];
	                $sales = array();
	                
	                while ($prod && $prod['id_order'] == $id_order) {
	                    if (!isset($sales[$cust_account])) $sales[$cust_account] = 0;
	                    $sales[$cust_account] -=  $prod['ti'];
	                    
	                    if ($prod['id_sale_detail']) {
	                        $prod_account = $prod['pnum'];
	                        if ($prod['id_carrier']) {
	                            if (empty($prod_account)) $prod_account = 'SHIP';
	                        }
	                        else if ($prod['wrapping']) {
	                            if (empty($prod_account)) $prod_account = 'WRAP';
	                        }
	                        else {
	                            if (empty($prod_account)) {
	                                $loc = 'world';
	                                if ($prod['id_zone'] == $id_zone) $loc = 'zone';
	                                if ($prod['id_country'] == $id_country) $loc = 'country';
	                                $prod_account = $catnum[$prod['id_cat']][$loc];
	                            }
	                            if (empty($prod_account)) $prod_account = 'PRODUIT';
	                        }
	                        if (!isset($sales[$prod_account])) $sales[$prod_account] = 0;
	                        $sales[$prod_account] += $prod['te'];
	                        
	                        $tax_account = $prod['tnum'];
	                        if (empty($tax_account)) $tax_account = 'TAXE';
	                        if (!isset($sales[$tax_account])) $sales[$tax_account] = 0;
	                        $sales[$tax_account] += $prod['tax'];
	                    }
	                    
	                    $prod = $prod_query->fetch(PDO::FETCH_ASSOC);
	                }
	                
	                foreach ($sales as $account => $amount) {
	                    if (!$amount) unset($sales[$account]);
	                }
	                if (count($sales)) {
	                    $sale_index += 1;
	                    krsort($sales, SORT_STRING); // products then taxes then customers
	                    foreach ($sales as $account => $amount) {
	                        $row = array_fill(1, $FEC_NB_COLS, '');
	                        $row[$FEC_JOURNAL_CODE] = 'VT';
	                        $row[$FEC_JOURNAL_LIB] = '"Journal des ventes"';
	                        $row[$FEC_ECRITURE_NUM] = $sale_index;
	                        $row[$FEC_ECRITURE_DATE] = $op['date'];
	                        $row[$FEC_ECRITURE_LIB] = '';
	                        $row[$FEC_COMPTE_NUM] = $account;
	                        $row[$FEC_COMPTE_LIB] = '';
	                        $row[$FEC_DEBIT] = $amount < 0 ? -$amount : 0;
	                        $row[$FEC_CREDIT] = $amount > 0 ? $amount : 0;
	                        $row[$FEC_PIECE_REF] = $invoice ? $invoice : '"Ticket #'.$id_sale.'"';
	                        $row[$FEC_PIECE_DATE] = $op['date'];
	                        echo(implode($FEC_SEP, $row).PHP_EOL);
	                    }
	                }
	            }
	            
	            // Payment log: 1 EC per operation
	            $payments = array();
	            
	            while ($pay && $pay['id_operation'] == $id_op) {
	                if (!isset($payments[$cust_account])) $payments[$cust_account] = 0;
	                $payments[$cust_account] += $pay['pay'];
	                
	                $pay_account = $pay['pnum'];
	                if (empty($pay_account)) $pay_account = 'CAISSE';
	                if (!isset($payments[$pay_account])) $payments[$pay_account] = 0;
	                $payments[$pay_account] -= $pay['pay'];
	                
	                $pay = $pay_query->fetch(PDO::FETCH_ASSOC);
	            }
	            
	            foreach ($payments as $account => $amount) {
	                if (!$amount) unset($payments[$account]);
	            }
	            if (count($payments)) {
	                $pay_index += 1;
	                ksort($payments, SORT_STRING); // customers then payments
	                foreach ($payments as $account => $amount) {
	                    $row = array_fill(1, $FEC_NB_COLS, '');
	                    $row[$FEC_JOURNAL_CODE] = 'CA';
	                    $row[$FEC_JOURNAL_LIB] = '"Journal de caisse"';
	                    $row[$FEC_ECRITURE_NUM] = $pay_index;
	                    $row[$FEC_ECRITURE_DATE] = $op['date'];
	                    $row[$FEC_ECRITURE_LIB] = '';
	                    $row[$FEC_COMPTE_NUM] = $account;
	                    $row[$FEC_COMPTE_LIB] = '';
	                    $row[$FEC_DEBIT] = $amount < 0 ? -$amount : 0;
	                    $row[$FEC_CREDIT] = $amount > 0 ? $amount : 0;
	                    $row[$FEC_PIECE_REF] = '"Ticket #'.$id_sale.'"';
	                    $row[$FEC_PIECE_DATE] = $op['date'];
	                    echo(implode($FEC_SEP, $row).PHP_EOL);
	                }
	            }
	        }
	    }
	    
	    $db1->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, true);
	    $db2->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, true);
	    $db3->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, true);
	    die();
	}
	*/
	
	public function bypage() {
	    //(int) Configuration::get('KERAWEN_REPORT_ITEMS_BY_PAGE');
	    return 10;
	}	
	
	
	public function getChartRootCategory() {
	    return (int) $this->db->getValue("SELECT id_parent FROM "._DB_PREFIX_."category WHERE is_root_category = 1");
	}
	
	public function getChartCategories()
	{
	    $cats = new kerawenCategories();
	    $res = $cats->getArrayLinearTree($this->getChartRootCategory());
	    array_unshift($res, $cats->getArrayLinearDefault());
	    return $res;
	}
	

	public function getChartProducts()
	{
	    $bypage = $this->bypage();
	    $page = (int) $this->page;

	    $id_lang = (int) Context::getContext()->language->id;
	    $id_shop = (int) Configuration::get('PS_SHOP_DEFAULT');
	    $id_category = (int) $this->vars->id_category;
	    
	    
	    $fields = array();
	    $joins = array();
	    
	    //dynamic fields
	    $arrayJoin = kerawenUnserializeConfig('KERAWEN_CHART_PRODUCT');
	    foreach($arrayJoin as $key => $value) {
	        $fields[] = "IFNULL(ka_" . $key . ".account_num, '') AS account_num_" . $key . ", ka_" . $key . ".id_account AS id_account_" . $key;
	        
	        $join_conditions = "";
	        
	        $join_conditions .= " AND ( ka_" . $key . ".id_country = " . (int) $value['id_country'] . " ";
	        if ((int) $value['id_country'] == 0) {
	            $join_conditions .= " OR ISNULL(ka_" . $key . ".id_country) ";
	        }
	        $join_conditions .= ") ";
	        
	        $join_conditions .= " AND ( ka_" . $key . ".id_zone = " . (int) $value['id_zone'] . " ";
	        if ((int) $value['id_zone'] == 0) {
	            $join_conditions .= " OR ISNULL(ka_" . $key . ".id_zone) ";
	        }
	        $join_conditions .= ") ";
	        
	        
	        $joins[] =  "LEFT JOIN "._DB_PREFIX_."kerawen_accounting ka_" . $key . " ON ka_" . $key . ".id_product = p.id_product" . $join_conditions;
	    }
	    
	    $q = "
    	    SELECT SQL_CALC_FOUND_ROWS
    	       p.id_product AS id,
    	       pl.name AS label,
               " . implode(",", $fields) . "
    	    FROM "._DB_PREFIX_."product p
    	    LEFT JOIN "._DB_PREFIX_."product_lang pl
                ON pl.id_product = p.id_product AND pl.id_shop = " . $id_shop . " AND pl.id_lang = "  . $id_lang .  "
            " . implode(" ", $joins) . "
    	    WHERE p.id_category_default = " . $id_category . "
            ORDER BY pl.name
        ";
	    
	    if ((int)$page > 0) {
	        $q .= " LIMIT " . ( ((int)$page - 1) * $bypage) . ", " . $bypage;
	    }
	    
	    $res = $this->db->executeS($q);
	    
	    $total = $this->db->getValue('SELECT FOUND_ROWS()');
	    
	    $cats = new kerawenCategories();
	    $breadCrumb = $cats->getArrayLinearBreadCrumb($id_category, $this->getChartRootCategory());
	    // back button
	    $backBtn = array(
	        "label" => "Catégories",
	        "id" => -1,
	        "depth" => -1,
	        "parent" => -1,
	    );
	    array_unshift($breadCrumb , $backBtn);
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage,
	        'breadCrumb' => $breadCrumb,
	    );
	    
	    return $data;
	}
	
	public function getChartDefault($field) {
	    $def =  $this->db->getValue("SELECT account_num FROM "._DB_PREFIX_."kerawen_accounting WHERE " . $field . " = 0");
	    $def = $def ? $def : "";
	    return array(
	        "id" => "0",
	        "label" => "Par défaut",
	        "account_num_Compte" => $def
	    );
	}
	
	public function getChartCarriers()
	{
	    $q = "
    		SELECT
    		   c.id_carrier AS id,
    		   c.name AS label,
               IFNULL(ka.account_num, '') AS account_num_Compte
	        
    		FROM "._DB_PREFIX_."carrier c
            LEFT JOIN "._DB_PREFIX_."kerawen_accounting ka
                ON ka.id_carrier = c.id_carrier
            ORDER BY c.id_carrier ASC
        ";
	    
	    $res = $this->db->executeS($q);
	    array_unshift($res, $this->getChartDefault("id_carrier"));
	    return $res;
	}
	
	public function getChartTaxes()
	{
	    
	    $fields = array();
	    $joins = array();
	    
	    //dynamic fields
	    $arrayJoin = kerawenUnserializeConfig('KERAWEN_CHART_TAX');
	    foreach($arrayJoin as $key => $value) {
	        $fields[] = "IFNULL(ka_" . $key . ".account_num, '') AS account_num_" . $key . ", ka_" . $key . ".id_account AS id_account_" . $key;
	        $join_conditions = " AND ( ka_" . $key . ".margin_tax = " . (int) $value['margin_tax'] . " ) ";
	        $joins[] =  "LEFT JOIN "._DB_PREFIX_."kerawen_accounting ka_" . $key . " ON ka_" . $key . ".id_tax = trg.id_tax_rules_group" . $join_conditions;
	    }
	    
	    $q = '
    	    SELECT
    	       trg.id_tax_rules_group AS id,
    	       trg.name AS label,
               '.implode(',', $fields).'
    	    FROM '._DB_PREFIX_.'tax_rules_group trg
            '.implode(' ', $joins).'
            WHERE trg.deleted= 0';
	    
	    $res = $this->db->executeS($q);
	    
	    
	    $default = array(
	        "id" => "0",
	        "label" => "Par défaut"
	    );
	    
	    //dynamic fields
	    foreach($arrayJoin as $key => $value) {
	        $def = $this->db->getRow('
                SELECT account_num, id_account
                FROM '._DB_PREFIX_.'kerawen_accounting
                WHERE id_tax = 0 AND margin_tax = '.(int)$value['margin_tax']);
	        $default["account_num_". $key] = $def ? $def['account_num'] : "";
	        $default["id_account_". $key] = $def ? $def['id_account'] : null;
	    }
	    
	    array_unshift($res, $default);
	    
	    return $res;
	}
	
	
	public function getChartGroups()
	{
	    $id_lang = (int) Context::getContext()->language->id;
	    
	    $q = "
    	    SELECT
        	    g.id_group AS id,
        	    gl.name AS label,
                IFNULL(c.qty, 0) AS qty,
        	    IFNULL(ka.account_num, '') AS account_num_Compte
	        
    	    FROM "._DB_PREFIX_."group g
    	    LEFT JOIN "._DB_PREFIX_."group_lang AS gl
    	       ON g.id_group = gl.id_group AND gl.id_lang = " . $id_lang . "
    	    LEFT JOIN "._DB_PREFIX_."kerawen_accounting ka
    	       ON ka.id_group = g.id_group
    	        
            LEFT JOIN (
                    SELECT
                        COUNT(*) AS qty,
                        id_default_group
                    FROM "._DB_PREFIX_."customer
                    GROUP BY id_default_group
                ) AS c
                ON g.id_group = c.id_default_group
                        
    	    ORDER BY g.id_group ASC
        ";
	    
	    $res = $this->db->executeS($q);
	    
	    $default = $this->getChartDefault("id_group");
	    $default["qty"] = " ";
	    array_unshift($res, $default);
	    
	    return $res;
	    
	}
	
	public function getChartCustomers()
	{
	    
	    $bypage = $this->bypage();
	    $page = (int) $this->page;

	    
	    $id_group = (int) $this->vars->id_group;
	    
	    $breadCrumb = array();
	    $breadCrumb[] = array(
	        'label' => 'Groupes',
	        "id" => -1,
	        "depth" => -1,
	        "parent" => -1,
	    );
	    $groups = indexArray($this->getChartGroups(), 'id');
	    if (isset($groups[$id_group])) {
	        $breadCrumb[] = $groups[$id_group];
	    }
	    
	    
	    $q = "
    	    SELECT SQL_CALC_FOUND_ROWS
        	    c.id_customer AS id,
        	    CONCAT(c.lastname, ' ', c.firstname) AS label,
        	    IFNULL(ka.account_num, '') AS account_num_Compte
	        
    	    FROM "._DB_PREFIX_."customer c
    	    LEFT JOIN "._DB_PREFIX_."kerawen_accounting ka
    	       ON ka.id_customer = c.id_customer
            WHERE c.id_default_group = " .  $id_group . "
    	    ORDER BY CONCAT(c.lastname, ' ', c.firstname) ASC
        ";
	    
	    if ((int)$page > 0) {
	        $q .= " LIMIT " . ( ((int)$page - 1) * $bypage) . ", " . $bypage;
	    }
	    
	    $res = $this->db->executeS($q);
	    
	    $total = $this->db->getValue('SELECT FOUND_ROWS()');
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage,
	        'breadCrumb' => $breadCrumb,
	    );
	    
	    return $data;
	    
	}
	
	
	public function getChartPayments()
	{
	    require_once(_KERAWEN_CLASS_DIR_.'/KerawenPayment.php');
	    
	    $q = "SELECT * FROM "._DB_PREFIX_."kerawen_accounting WHERE id_mode IS NOT NULL";
	    $res = indexArray($this->db->executeS($q), 'id_mode');
	    
	    $cfg = Tools::jsonDecode(Configuration::get('KERAWEN_PAYMENTS'), true);
	    
	    $kmp = KerawenPayment::getModes(Module::getInstanceByName('kerawen'));
	    foreach($kmp as $key => $value) {
	        $kmp[$key]['account_num_Compte'] = isset($res[$value['id']]) ? $res[$value['id']]['account_num'] : "";
	        $kmp[$key]['label'] = isset($cfg[$value['id']]['label']) ? $cfg[$value['id']]['label'] : $kmp[$key]['label'];
	        $kmp[$key]['type'] = "Caisse";
	    }
	    
	    //web
	    $ka = array();
	    $res = $this->db->executeS('
            SELECT
                ka.mode,
                ka.account_num
           FROM ' . _DB_PREFIX_ . 'kerawen_accounting ka
           WHERE ka.mode IS NOT NULL
        ');
	    foreach($res as $item) {
	        $ka[$item['mode']] = $item['account_num'];
	    }
	    
	    
	    $res = $this->db->executeS('
            SELECT
				p.mode AS label, p.id_mode AS id
			FROM ' . _DB_PREFIX_ . 'kerawen_525_payment p
            WHERE p.id_mode IS NULL
			GROUP BY p.mode
		');
	    
	    foreach($res as $item) {
	        $account_num = isset($ka[$item['label']]) ? $ka[$item['label']] : '';
	        $kmp[] = array(
	            'account_num_Compte' => $account_num,
	            'label' => $item['label'],
	            'type' => "Web"
	        );
	    }
	    
	    
	    $default = $this->getChartDefault("id_mode");
	    $default['type'] = " ";
	    $default['label'] = "Par défaut";
	    array_unshift($kmp, $default);
	    
	    return $kmp;
	}
	
	
	public function setChartSettings() {
	    if ($this->data) {

	        $arrayData = array();
	        $where = array();
	        
	        switch($this->data->id_tab) {
	            
	            case "getChartCategories":
	            case "getChartProducts":
	            case "getChartTaxes";
	            
	            if($this->data->id_tab == "getChartTaxes") {
	                $arrayJoin = kerawenUnserializeConfig('KERAWEN_CHART_TAX');
	            } else {
	                $arrayJoin = kerawenUnserializeConfig('KERAWEN_CHART_PRODUCT');
	            }
	            
	            if (isset($arrayJoin[$this->data->key])) {
	                
	                if (isset($arrayJoin[$this->data->key]['id_country'])) {
	                    $arrayData['id_country'] = $arrayJoin[$this->data->key]['id_country'];
	                }
	                if (isset($arrayJoin[$this->data->key]['id_zone'])) {
	                    $arrayData['id_zone'] = $arrayJoin[$this->data->key]['id_zone'];
	                }
	                if (isset($arrayJoin[$this->data->key]['margin_tax'])) {
	                    $arrayData['margin_tax'] = $arrayJoin[$this->data->key]['margin_tax'];
	                }
	                
	                $arrayField = array(
	                    "getChartCategories" => "id_category",
	                    "getChartProducts" => "id_product",
	                    "getChartTaxes"=> "id_tax",
	                );
	                
	                $field = $arrayField[$this->data->id_tab];
	                $arrayData[$field] =  (string) $this->data->id; //(string) important!!! for 0
	                
	                
	                foreach($arrayData as $key => $value) {
	                    if ($value == "" || $value == NULL || $value == "NULL") {
	                        $arrayData[$key] = NULL;
	                        $where[] = "ISNULL(" . $key . ")";
	                    } else {
	                        $where[] = $key . " = " . $value;
	                    }
	                }
	                
	                //new value
	                $arrayData['account_num'] = pSQL($this->data->val);
	                
	            }
	            
	            break;
	            
	            case "getChartPayments":
	                
	                if (gettype($this->data->id) == "boolean") {
	                    $field = "mode";
	                    $arrayData[$field] = pSQL($this->data->label);
	                    $where[] = $field . " = '" . pSQL($this->data->label) . "'";
	                } else {
	                    $field = "id_mode";
	                    $arrayData[$field] = (string) $this->data->id; //(string) important!!! for 0
	                    $where[] = $field . " = " . (string) $this->data->id;
	                }
	                
	                $arrayData['account_num'] = pSQL($this->data->val);
	                
	                break;
	                
	                
	            case "getChartGroups":
	            case "getChartCustomers":
	            case "getChartCarriers":
	                //Or default
	                
	                $id_key = array(
	                "getChartGroups" => "id_group",
	                "getChartCustomers" => "id_customer",
	                "getChartCarriers" => "id_carrier",
	                "getChartPayments" => "id_mode",
	                );
	                
	                $field = $id_key[$this->data->id_tab];
	                $arrayData[$field] = (string) $this->data->id; //(string) important!!! for 0
	                $where[] = $field . " = " . (int) $this->data->id;
	                $arrayData['account_num'] = pSQL($this->data->val);
	                
	                break;
	                
	        }
	        
	        if (count($arrayData)) {
	            $table = "kerawen_accounting";
	            $id_account = (int) $this->db->getValue("SELECT id_account FROM "._DB_PREFIX_.$table." WHERE " . implode(" AND ", $where));
	            if ($id_account) {
	                if ($arrayData['account_num'] == '') {
	                    // DELETE
	                    $this->db->delete($table, 'id_account = '. $id_account);
	                }
	                else {
	                    // UPDATE
	                    $this->db->update($table, $arrayData,  'id_account = '. $id_account, 1, true);
	                }
	            }
	            else {
	                if ($arrayData['account_num'] == '') {
	                    // IGNORE
	                }
	                else {
	                    // CREATE
	                    $this->db->insert($table, $arrayData, true);
	                }
	            }
	            return true;
	        }
	    }
	    return false;
	}
	
};

