<?php
/**
* @author    KerAwen <contact@kerawen.com>
* @copyright 2017-2018 KerAwen
*/

define('_KERAWEN_525_SIGN_URL_', 'https://www.kerawen.com/app/security/sign2.php');

define('_KERAWEN_525_PREFIX_', 'kerawen_525_');

define('_KERAWEN_525_OP_SALE_',			'SALE');
define('_KERAWEN_525_OP_DUPLICATE_',	'DUPLICATE');
define('_KERAWEN_525_OP_INVOICE_',		'INVOICE');
define('_KERAWEN_525_OP_GTOTAL_',		'GTOTAL');
define('_KERAWEN_525_OP_EVENT_',		'EVENT');
define('_KERAWEN_525_OP_ARCHIVE_',		'ARCHIVE');
define('_KERAWEN_525_OP_OPEN_',			'OPEN');
define('_KERAWEN_525_OP_FLOW_',			'FLOW');
define('_KERAWEN_525_OP_CLOSE_',		'CLOSE');

define('_KERAWEN_525_PER_PERPETUAL_',	'PERPETUAL');
define('_KERAWEN_525_PER_YEAR_',		'YEAR');
define('_KERAWEN_525_PER_MONTH_',		'MONTH');
define('_KERAWEN_525_PER_DAY_',			'DAY');
define('_KERAWEN_525_PER_SALE_',		'SALE');

define('_KERAWEN_525_EVT_ARCH_PERIOD_',		20);
define('_KERAWEN_525_EVT_ARCH_EXER_',		30);
define('_KERAWEN_525_EVT_STOP_',			40);
define('_KERAWEN_525_EVT_CLOSE_PERIOD_',	50);
define('_KERAWEN_525_EVT_CLOSE_EXER_',		60);
define('_KERAWEN_525_EVT_START_DEG_',		70);
define('_KERAWEN_525_EVT_START_',			80);
define('_KERAWEN_525_EVT_SIGN_DEFAULT_',	90);
define('_KERAWEN_525_EVT_EXEC_SPECIAL_',	100);
define('_KERAWEN_525_EVT_EXPORT_',			110);
define('_KERAWEN_525_EVT_STOP_DEG_',		120);
define('_KERAWEN_525_EVT_DECL_OPER_',		130);
define('_KERAWEN_525_EVT_IMPORT_',			140);
define('_KERAWEN_525_EVT_PRINT_FAIL_',		150);
define('_KERAWEN_525_EVT_ACCOUNT_GEN_',		160);
define('_KERAWEN_525_EVT_TILL_CHECK_',		170);
define('_KERAWEN_525_EVT_ACCOUNT_TRANSF_',	180);
define('_KERAWEN_525_EVT_CANCEL_OP_',		190);
define('_KERAWEN_525_EVT_PURGE_',			200);
define('_KERAWEN_525_EVT_TRANSF_DATA_',		210);
define('_KERAWEN_525_EVT_RESTORE_',			220);
define('_KERAWEN_525_EVT_BACKUP_',			230);
define('_KERAWEN_525_EVT_MAINTAIN_',		240);
define('_KERAWEN_525_EVT_INSTALL_',			250);
define('_KERAWEN_525_EVT_INIT_DATA_',		260);
define('_KERAWEN_525_EVT_EVOL_PARAM_',		270);
define('_KERAWEN_525_EVT_CONTROL_',			280);
define('_KERAWEN_525_EVT_ACCOUNT_EXCH_',	290);
define('_KERAWEN_525_EVT_ACTIV_PARAM_',		300);
define('_KERAWEN_525_EVT_DOC_CANCEL_',		310);
define('_KERAWEN_525_EVT_CHANGE_EXER_',		400);
define('_KERAWEN_525_EVT_FUNC_SPEC_',		999);
define('_KERAWEN_525_EVT_ERROR_',			998);


class Kerawen525 {
	
	/* **************************************************************
	 * SINGLETON
	*/
	
	static $instance = null;
	
	public static function getInstance() {
		if (!self::$instance) {
			self::$instance = new Kerawen525();
		}
		return self::$instance;
	}
	
	protected function __construct() {
		$this->changes = array(
		    'orders' => array(),
		    'details' => array(),
		    'slips' => array(),
		    'payments' => array(),
		);
		$this->exception = null;
	}
	
	
	/* **************************************************************
	 * SYSTEM CALLBACKS
	*/
	
	public static function OnException($exception) {
	    echo($exception); die();
		self::getInstance()->exception = $exception;
	}
	
	public static function OnShutdown() {
	    
	    self::getInstance()->logOperations(time());
		
		// Send order emails after certified logging
		require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
		Configuration::set('PS_MAIL_METHOD', getExtendedContext('PS_MAIL_METHOD', 3));
		$order_states = getExtendedContext('order_states', array());
		$db = Db::getInstance();
		foreach ($order_states as $os) {
			$id_order = $os['id_order'];
			$id_os = $os['id_os'];
			$id_history = $db->getValue('
				SELECT id_order_history
				FROM '._DB_PREFIX_.'order_history
				WHERE id_order = '.pSQL($id_order).'
					AND id_order_state = '.pSQL($id_os).'
				ORDER BY id_order_history DESC');
			$oh = new OrderHistory();
			$oh->id = $id_history;
			$oh->id_order = $id_order;
			$order = new Order($id_order);
			if (method_exists($oh, 'sendEmail')) {
				$oh->sendEmail($order);
			}
		}
	}
	
	
	/* **************************************************************
	 * DATABASE MODEL
	*/

	public function createDatabase() {
		$db = Db::getInstance();
		
		// Enumerations
		$op_type = 'ENUM("'.implode('","', array(
				_KERAWEN_525_OP_SALE_,
				_KERAWEN_525_OP_DUPLICATE_,
				_KERAWEN_525_OP_INVOICE_,
				_KERAWEN_525_OP_GTOTAL_,
				_KERAWEN_525_OP_EVENT_,
				_KERAWEN_525_OP_ARCHIVE_,
				_KERAWEN_525_OP_OPEN_,
				_KERAWEN_525_OP_FLOW_,
				_KERAWEN_525_OP_CLOSE_,
		)).'")';
		
		$period_type = 'ENUM("'.implode('","', array(
			_KERAWEN_525_PER_PERPETUAL_,
			_KERAWEN_525_PER_YEAR_,
			_KERAWEN_525_PER_MONTH_,
			_KERAWEN_525_PER_DAY_,
			_KERAWEN_525_PER_SALE_,
		)).'")';

		$prod_type = 'ENUM("'.implode('","', array(
		    _KERAWEN_TYPE_STANDARD_,
		    _KERAWEN_TYPE_DEALER_,
		    _KERAWEN_TYPE_DEPOSIT_,
		    _KERAWEN_TYPE_PREPAID_,
		    _KERAWEN_TYPE_GIFTCARD_,
		    _KERAWEN_TYPE_MEASURED_,
		    _KERAWEN_TYPE_GENERIC_,
		    _KERAWEN_TYPE_REFUND_,
		)).'")';
		

		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'signature (
				type '.$op_type.' NOT NULL,
				sign TEXT,
				short VARCHAR(32),
				PRIMARY KEY (type)
			)');
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation (
				id_operation INT UNSIGNED NOT NULL AUTO_INCREMENT,
				date DATETIME NOT NULL,
				type '.$op_type.' NOT NULL,
				id_till INT UNSIGNED,
				till_name TEXT,
				id_operator INT UNSIGNED,
				operator_name TEXT,
				PS_SHOP_NAME VARCHAR(255) NOT NULL DEFAULT "",
				PS_SHOP_EMAIL VARCHAR(255) NOT NULL DEFAULT "",
				PS_SHOP_ADDR1 VARCHAR(255) NOT NULL DEFAULT "",
				PS_SHOP_ADDR2 VARCHAR(255) NOT NULL DEFAULT "",
				PS_SHOP_CODE VARCHAR(255) NOT NULL DEFAULT "",
				PS_SHOP_CITY VARCHAR(255) NOT NULL DEFAULT "",
				PS_SHOP_COUNTRY VARCHAR(255) NOT NULL DEFAULT "",
				PS_SHOP_PHONE VARCHAR(255) NOT NULL DEFAULT "",
				KERAWEN_SHOP_SIRET VARCHAR(255) NOT NULL DEFAULT "",
				KERAWEN_SHOP_NAF VARCHAR(255) NOT NULL DEFAULT "",
				KERAWEN_SHOP_TVA_INTRA VARCHAR(255) NOT NULL DEFAULT "",
				KERAWEN_SHOP_URL VARCHAR(255) NOT NULL DEFAULT "",
				KERAWEN_MODULE_VERSION VARCHAR(255) NOT NULL DEFAULT "",
				PRIMARY KEY (id_operation)
			)');
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
				MODIFY COLUMN type '.$op_type.' NOT NULL');
		} catch (Exception $e) {}
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
				ADD COLUMN id_shop INT UNSIGNED AFTER operator_name');
		} catch (Exception $e) {}
		try {
			// Adapt for till operations logging
			$res = $db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
				ADD COLUMN id_ref INT UNSIGNED AFTER type,
				ADD COLUMN id_close INT UNSIGNED AFTER id_ref');
			// Mark previous ops as closed / to be closed
			if ($res) $db->update(_KERAWEN_525_PREFIX_.'operation', array(
				'id_ref' => 0,
				'id_close' => 0,
			));
		} catch (Exception $e) {}
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
				ADD COLUMN canceled TINYINT(1) UNSIGNED NULL DEFAULT NULL');
		} catch (Exception $e) {}
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
				MODIFY COLUMN canceled INT UNSIGNED NULL DEFAULT NULL');
		} catch (Exception $e) {}
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale (
				id_sale INT UNSIGNED NOT NULL AUTO_INCREMENT,
				id_operation INT UNSIGNED NOT NULL,
				id_customer INT UNSIGNED,
				customer_name TEXT,
				nb_lines INT,
				total_te DECIMAL(20,6),
				total_ti DECIMAL(20,6),
				description TEXT,
				signature TEXT,
				sign_short TEXT,
				nb_prints INT UNSIGNED,
				PRIMARY KEY (id_sale),
				FOREIGN KEY (id_operation)
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation(id_operation)
					ON DELETE CASCADE
			)');
		try {
			// TODO make a second table for pure orders ?
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale
				ADD COLUMN accounted TINYINT(1) DEFAULT 1 AFTER id_operation');
		} catch (Exception $e) {}
		try {
		    $db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale
                ADD COLUMN group_name TEXT AFTER customer_name,
				ADD COLUMN id_group INT UNSIGNED AFTER customer_name');
		} catch (Exception $e) {}
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_tax (
				id_sale INT UNSIGNED NOT NULL,
				id_tax INT UNSIGNED,
				tax_name TEXT,
				tax_rate DECIMAL(20,6),
				total_te DECIMAL(20,6),
				total_ti DECIMAL(20,6),
				tax_amount DECIMAL(20,6),
				PRIMARY KEY (id_sale, id_tax),
				FOREIGN KEY (id_sale)
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale(id_sale)
					ON DELETE CASCADE
			)');

		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail (
				id_sale_detail INT UNSIGNED NOT NULL AUTO_INCREMENT,
				id_sale INT UNSIGNED NOT NULL,
				id_order INT UNSIGNED,
				id_order_detail INT UNSIGNED,
				id_carrier INT UNSIGNED,
				wrapping TINYINT(1) UNSIGNED,
				item_name TEXT,
				id_tax INT UNSIGNED,
				tax_name TEXT,
				tax_rate DECIMAL(20,6),
				unit_te DECIMAL(20,6),
				unit_ti DECIMAL(20,6),
				ecotax_te DECIMAL(20,6),
				ecotax_ti DECIMAL(20,6),
				quantity INT,
				measure DECIMAL(20,6),
				measure_unit TEXT,
				measure_precision INT,
				discount_te DECIMAL(20,6),
				discount_ti DECIMAL(20,6),
				discount_rate DECIMAL(20,6),
				total_te DECIMAL(20,6),
				total_ti DECIMAL(20,6),
				PRIMARY KEY (id_sale_detail),
				FOREIGN KEY (id_sale)
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale(id_sale)
					ON DELETE CASCADE
			)');
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
				ADD FOREIGN KEY (id_order)
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order(id_order)
					ON DELETE CASCADE');
		} catch (Exception $e) {}
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
				ADD COLUMN margin_vat TINYINT(1) UNSIGNED AFTER tax_rate');
		} catch (Exception $e) {}
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
				ADD COLUMN version_name TEXT AFTER item_name,
				ADD COLUMN purchase_te DECIMAL(20,6) AFTER total_ti');
		} catch (Exception $e) {}
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
				ADD COLUMN discount_percent DECIMAL(20,6) AFTER measure_precision');
		} catch (Exception $e) {}
		try {
		    $db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
				ADD COLUMN gift_card TINYINT(1) UNSIGNED DEFAULT 0 NULL');
		} catch (Exception $e) {}
		try {
		    $db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
				ADD COLUMN account_credit TINYINT(1) UNSIGNED DEFAULT 0 NULL');
		} catch (Exception $e) {}
		try {
		    $db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
				ADD COLUMN rest_voucher TINYINT(1) UNSIGNED DEFAULT 0 NULL');
		} catch (Exception $e) {}
		try {
		    $db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
				ADD COLUMN product_type ' .$prod_type);
		} catch (Exception $e) {}
		try {
		    $db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail
				CHANGE `product_type` `product_type` ' .$prod_type);
		} catch (Exception $e) {}

		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'discount (
				id_sale INT UNSIGNED NOT NULL,
				id_order INT UNSIGNED NOT NULL,
				ps_order INT UNSIGNED NOT NULL,
				ps_cart_rule INT UNSIGNED NOT NULL,
				label VARCHAR(255),
				tax_incl DECIMAL(20,6) NOT NULL DEFAULT 0,
				tax_excl DECIMAL(20,6) NOT NULL DEFAULT 0,
				KEY (id_order),
				FOREIGN KEY (id_sale)
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale(id_sale)
					ON DELETE CASCADE
			)');
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'discount
				ADD COLUMN discount_percent DECIMAL(20,6) AFTER label');
		} catch (Exception $e) {}
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'discount
				ADD COLUMN free_shipping TINYINT(1) AFTER discount_percent');
		} catch (Exception $e) {}
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment (
				id_payment INT UNSIGNED NOT NULL AUTO_INCREMENT,
				id_operation INT UNSIGNED NOT NULL,
				id_order_payment INT UNSIGNED,
				id_currency INT UNSIGNED,
				currency TEXT,
				amount DECIMAL(20,6),
				mode TEXT,
				PRIMARY KEY (id_payment),
				FOREIGN KEY (id_operation)
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation(id_operation)
					ON DELETE CASCADE
			)');
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment
				ADD COLUMN ps_orders TEXT AFTER id_order_payment');
		} catch (Exception $e) {}
		try {
			// Adapt for till operations logging
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment
				ADD COLUMN id_mode INT UNSIGNED AFTER amount,
				ADD COLUMN deferred DATETIME,
				ADD COLUMN id_out INT UNSIGNED,
				ADD COLUMN corrected DECIMAL(20,6)');
			// Mark any payment from till closed/to be closed
			$db->execute('
				UPDATE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation o ON o.id_operation = p.id_operation
				SET p.id_out = 0
				WHERE o.id_till != 0');
			// Unmark cheques still in till
// 			$db->execute('
// 				UPDATE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p
// 				JOIN '._DB_PREFIX_.'cashdrawer_flow_kerawen f ON f.id_order_payment = p.id_order_payment
// 				SET p.id_mode = 2
// 				WHERE f.id_payment_mode = 2 AND f.op_out IS NULL');
			} catch (Exception $e) {}
			try {
			    $db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment
				ADD COLUMN `remain` TINYINT(1) UNSIGNED DEFAULT NULL AFTER mode');
			} catch (Exception $e) {}
			
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'duplicate (
				id_duplicate INT UNSIGNED NOT NULL AUTO_INCREMENT,
				id_sale INT UNSIGNED,
				nb_duplicate INT UNSIGNED,
				id_operator INT UNSIGNED,
				date DATETIME,
				description TEXT,
				signature TEXT,
				sign_short TEXT,
				PRIMARY KEY (id_duplicate),
				FOREIGN KEY (id_sale)
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale(id_sale)
					ON DELETE CASCADE
		)');
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order (
				id_order INT UNSIGNED NOT NULL AUTO_INCREMENT,
				id_sale INT UNSIGNED NOT NULL,
				ps_order INT UNSIGNED NOT NULL,
				ps_slip INT UNSIGNED NOT NULL,
				total_te DECIMAL(20,6),
				total_ti DECIMAL(20,6),
				date DATETIME,
				number VARCHAR(32),
				name TEXT,
				postcode TEXT,
				description TEXT,
				signature TEXT,
				sign_short TEXT,
				PRIMARY KEY (id_order),
				FOREIGN KEY (id_sale)
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale(id_sale)
					ON DELETE CASCADE
		)');

		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order_tax (
				id_order INT UNSIGNED NOT NULL,
				id_tax INT UNSIGNED,
				tax_name TEXT,
				tax_rate DECIMAL(20,6),
				total_te DECIMAL(20,6),
				total_ti DECIMAL(20,6),
				tax_amount DECIMAL(20,6),
				PRIMARY KEY (id_order, id_tax),
				FOREIGN KEY (id_order)
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order(id_order)
					ON DELETE CASCADE
			)');
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'invoice ( 
				id_order INT UNSIGNED NOT NULL, 
				invoice_date DATETIME,
 				invoice_label VARCHAR(20), 
				invoice_address TEXT, 
				delivery_address TEXT, 
				PRIMARY KEY (id_order),
				FOREIGN KEY (id_order) 
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order(id_order)
					ON DELETE CASCADE
			)');
		try {
			$db->execute('
				ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'invoice
				ADD COLUMN firstname TEXT AFTER invoice_label,
				ADD COLUMN lastname TEXT AFTER firstname,
				ADD COLUMN company TEXT AFTER lastname');
		} catch (Exception $e) {}
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal (
				id_gtotal INT UNSIGNED NOT NULL AUTO_INCREMENT,
				period_type '.$period_type.' NOT NULL,
				period_ref VARCHAR(16) NOT NULL,
				total_te DECIMAL(20,6) NOT NULL DEFAULT 0,
				total_ti DECIMAL(20,6) NOT NULL DEFAULT 0,
				perp_te DECIMAL(20,6) NOT NULL DEFAULT 0,
				perp_ti DECIMAL(20,6) NOT NULL DEFAULT 0,
				date DATETIME,
				description TEXT,
				signature TEXT,
				sign_short TEXT,
				PRIMARY KEY (id_gtotal)
			)');
		
		$db->execute('
			INSERT INTO '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal (id_gtotal, period_type, period_ref)
			VALUES (1, "'._KERAWEN_525_PER_PERPETUAL_.'", "")
			ON DUPLICATE KEY UPDATE period_type = VALUES(period_type)');
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal_tax (
				id_gtotal INT UNSIGNED NOT NULL,
				id_tax INT UNSIGNED,
				tax_name TEXT,
				tax_rate DECIMAL(20,6),
				total_te DECIMAL(20,6) NOT NULL DEFAULT 0,
				total_ti DECIMAL(20,6) NOT NULL DEFAULT 0,
				tax_amount DECIMAL(20,6) NOT NULL DEFAULT 0,
				PRIMARY KEY (id_gtotal, id_tax),
				FOREIGN KEY (id_gtotal)
					REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal(id_gtotal)
					ON DELETE CASCADE
			)');
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'event (
				id_event INT UNSIGNED NOT NULL AUTO_INCREMENT,
				date DATETIME NOT NULL,
				id_operator INT UNSIGNED,
				code INT UNSIGNED NOT NULL,
				label TEXT,
				info TEXT,
				description TEXT,
				signature TEXT,
				sign_short TEXT,
				PRIMARY KEY (id_event)
			)');
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'tax (
				id_tax INT UNSIGNED NOT NULL AUTO_INCREMENT,
				ps_tax INT UNSIGNED NOT NULL,
				PRIMARY KEY (id_tax)
			)');
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow (
				id_tillflow INT UNSIGNED NOT NULL AUTO_INCREMENT,
				id_operation INT UNSIGNED NOT NULL,
				id_currency INT UNSIGNED,
				currency TEXT,
				amount DECIMAL(20,6),
				id_mode INT UNSIGNED,
				mode TEXT,
				count INT UNSIGNED,
				comments TEXT,
				PRIMARY KEY (id_tillflow),
                INDEX(id_operation)
			)');

		try {
		    $db->execute('
                ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow 
                    ADD FOREIGN KEY (id_operation) 
                       REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation(id_operation)
					   ON DELETE CASCADE');
		} catch (Exception $e) {}
		
		
		
		$db->execute('
			CREATE TABLE IF NOT EXISTS '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_check (
				id_tillcheck INT UNSIGNED NOT NULL AUTO_INCREMENT,
				id_operation INT UNSIGNED NOT NULL,
				id_mode INT UNSIGNED,
				mode TEXT,
				id_currency INT UNSIGNED,
				currency TEXT,
				checked DECIMAL(20,6),
				count INT UNSIGNED,
				error DECIMAL(20,6),
				PRIMARY KEY (id_tillcheck),
                INDEX(id_operation)
			)');
		
		try {
		    $db->execute('
                ALTER TABLE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow
                    ADD FOREIGN KEY (id_operation) 
                       REFERENCES '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation(id_operation)
					   ON DELETE CASCADE');
		} catch (Exception $e) {}
		
		
		return true;
	}
	
	/* **************************************************************
	 * REGISTERING EVENTS
	*/
	
	public function registerOrder($order, $before = false) {
		if (!isset($this->changes['orders'][$order->id])) {
		    $this->changes['orders'][$order->id] = array(
			    'before' => $before ? new Order($order->id) : null,
				'after' => null,
			);
		}
	}
	
	public function registerOrderDetail($detail, $before = false) {
	    // Ignore pseudo details for returns without order
	    if ($detail->id_order == 0) return;
	    
	    if (!isset($this->changes['details'][$detail->id])) {
	        $this->changes['details'][$detail->id] = array(
	            'before' => $before ? new OrderDetail($detail->id) : null,
	            'after' => null,
	        );
	    }
	}
		
	public function registerOrderSlip($slip, $before = false) {
	    if (!isset($this->changes['slips'][$slip->id])) {
	        $this->changes['slips'][$slip->id] = array(
	            'before' => $before ? $this->getOrderSlip($slip->id) : null,
	            'after' => null,
	        );
	    }
	}
	
	public function registerOrderPayment($payment, $before = false) {
	    if (!isset($this->changes['payments'][$payment->id])) {
	        $this->changes['payments'][$payment->id] = array(
	            'before' => $before ? $this->getOrderPayment($payment->id) : null,
	            'after' => null,
	        );
	    }
	}
	
	
	/* **************************************************************
	 * LOGGING OPERATIONS
	*/
	
	public function logOperations($datetime) {
		if ((count($this->changes['orders'])
		    + count($this->changes['details'])
		    + count($this->changes['slips'])
		    + count($this->changes['payments'])) == 0) return;

		require_once(_KERAWEN_CLASS_DIR_.'/data.php');
		
		// Clean cache to prevent direct database changes to be omitted
		// In particular, marketplace modules modify orders this way
		Cache::clean('objectmodel_Order_*');
		$this->closeGrandTotal($datetime);
		
		// Start transaction
		$this->getDbLink()->beginTransaction();
		$db = Db::getInstance();
		
		// Prepare labels
		$module = Module::getInstanceByName('kerawen');
		$this->label_shipping = $module->l('Shipping', pathinfo(__FILE__, PATHINFO_FILENAME));
		$this->label_wrapping = $module->l('Wrapping', pathinfo(__FILE__, PATHINFO_FILENAME));
		
		// Set up common operations context
		$context = Context::getContext();
		$operator = $context->employee;
		$id_till = isset($context->kerawen) && isset($context->kerawen->id_cashdrawer) ? $context->kerawen->id_cashdrawer : 0;
		$id_shop = isset($context->kerawen) && isset($context->kerawen->id_shop) ? $context->kerawen->id_shop : Shop::getContextShopID();
		
		$ctx = array(
		    'id_shop' => $id_shop,
			'id_till' => $id_till,
			'till_name' => $db->getValue('SELECT name FROM '._DB_PREFIX_.'cash_drawer_kerawen WHERE id_cash_drawer = '.pSQL($id_till)),
			'id_operator' => $operator ? $operator->id : null,
			'operator_name' => $operator ? ($operator->firstname.' '.$operator->lastname) : null,
			'KERAWEN_MODULE_VERSION' => $module->version,
		);
		
		// Log errors into JET
		$errors = array();
		if ($error = error_get_last()) {
			$loggable = array(
				E_ERROR,
				E_CORE_ERROR,
				E_COMPILE_ERROR,
				E_USER_ERROR,
				E_RECOVERABLE_ERROR
			);
		
			if (in_array($error['type'], $loggable)) {
				$errors[] = 'ERROR '.$error['type'].': "'.$error['message'].'"'
					.' at '.$error['file'].', line '.$error['line'];
			}
		}
		if ($this->exception) {
			$errors[] = 'EXCEPTION '.$this->exception->getCode().': "'.$this->exception->getMessage().'"'
				.' at '.$this->exception->getFile().', line '.$this->exception->getLine();
		}
		foreach ($errors as &$error) {
			$this->logEvent(
				_KERAWEN_525_EVT_ERROR_,
				'Error',
				$error,
				$datetime,
				$ctx['id_operator']);
		}
		
		// Organize changes by order
		// TODO optimize: merge with following
		$buf = array(
		    'orders' => array(),
		    'slips' => array(),
		    'payments' => array(),
		);
		
		foreach($this->changes['orders'] as $id_order => $order) {
		    if (!isset($buf['orders'][$id_order])) {
		        $buf['orders'][$id_order] = array(
		            'before' => null,
		            'after' => null,
		            'items' => array(),
		        );
		    }
		    $o = &$buf['orders'][$id_order];
		    if (!$o['before']) $o['before'] = $order['before'];
		}
		
		foreach($this->changes['details'] as $id_detail => $detail) {
		    $od = new OrderDetail($id_detail);
		    $id_order = $od->id_order;
		    if (!isset($buf['orders'][$id_order])) {
		        $buf['orders'][$id_order] = array(
		            'before' => new Order($id_order),
		            'after' => null,
		            'items' => array(),
		            'slips' => array(),
		        );
		    }
		    $o = &$buf['orders'][$id_order];
		    if(!isset($o['items'][$id_detail])) {
		        $o['items'][$id_detail] = array(
		            'before' => null,
		            'after' => null,
		        );
		    }
		    $d = &$o['items'][$id_detail];
		    if (!$d['before']) $d['before'] = $detail['before'];
		}
		
		foreach($this->changes['slips'] as $id_slip => $slip) {
		    $sl = new OrderSlip($id_slip);
		    $id_order = $sl->id_order;
		    if (!isset($buf['orders'][$id_order])) {
		        $buf['orders'][$id_order] = array(
		            'before' => new Order($id_order),
		            'after' => null,
		            'items' => array(),
		            'slips' => array(),
		        );
		    }
		    $o = &$buf['orders'][$id_order];
		    if (!isset($o['slips'][$id_slip])) {
		        $o['slips'][$id_slip] = array(
		            'before' => null,
		            'after' => null,
		        );
		    }
		    $s = &$o['slips'][$id_slip];
		    if (!$s['before']) $s['before'] = $slip['before'];
		}

		foreach($this->changes['payments'] as $id_payment => $payment) {
		    if (!isset($buf['payments'][$id_payment])) {
		        $buf['payments'][$id_payment] = array(
		            'before' => null,
		            'after' => null,
		        );
		    }
		    $p = &$buf['payments'][$id_payment];
		    if (!$p['before']) $p['before'] = $payment['before'];
		    
		    $pay = $this->getOrderPayment($id_payment);
		    $ref = $pay->id ? $pay->order_reference : ($p['before'] ? $p['before']->order_reference : null);
		    if ($ref) {
		        $id_order = Db::getInstance()->getValue('
        			SELECT id_order
        			FROM '._DB_PREFIX_.'orders
        			WHERE reference = "'.pSQL($ref).'"
        			ORDER BY id_order');
		        if ($id_order) {
		            if (!isset($buf['orders'][$id_order])) {
    		            $buf['orders'][$id_order] = array(
    		                'before' => new Order($id_order),
    		                'after' => null,
    		                'items' => array(),
    		                'slips' => array(),
    		            );
		            }
		        }
		    }
		}
		
		$this->changes = array(
		    'REF' => $buf,
		);
		
		// Organize changes by customer
		$by_cust = array();
		
		// If operation from kerawen => one customer only
		$single_cust = false;
		if (isset($context->kerawen) && isset($context->kerawen->kerawen)) {
			$anonymous = Configuration::get('KERAWEN_ANONYMOUS_CUSTOMER');
			$single_cust = $anonymous;
			foreach($this->changes as &$change) {
				foreach($change['orders'] as $id_order => &$order) {
					$order['after'] = new Order($id_order);
					if (!$order['before'] && !$order['after']->id) continue;
					$id_cust = (int)($order['after']->id ? $order['after']->id_customer : $order['before']->id_customer);
					if ($id_cust && (!$single_cust || $single_cust == $anonymous)) {
						$single_cust = $id_cust;
					}
				}
			}
		}
		
		foreach($this->changes as &$change) {
			
			foreach($change['orders'] as $id_order => &$order) {
				$order['after'] = new Order($id_order);
				if (!$order['before'] && !$order['after']->id) continue;
				$id_cust = $single_cust ? $single_cust
				    : (int)($order['after']->id ? $order['after']->id_customer : $order['before']->id_customer);
				if (!isset($by_cust[$id_cust])) {
					$by_cust[$id_cust] = array(
						'orders' => array(),
						'slips' => array(),
						'payments' => array(),
					);
				}
				$by_cust[$id_cust]['orders'][$id_order] = $order;
				
				// Complete shop context if necessary
				if ($ctx['id_shop'] == 0) $ctx['id_shop'] = $order['after']->id_shop;
			}
			
			foreach ($change['payments'] as $id_payment => &$payment) {
				$payment['after'] = $this->getOrderPayment($id_payment);
				if (!$payment['before'] && !$payment['after']->id) continue;
				$ref = $payment['after']->id ? $payment['after']->order_reference : $payment['before']->order_reference;
				$id_cust = $single_cust ? $single_cust
				    : (int)$db->getValue('
        					SELECT id_customer FROM '._DB_PREFIX_.'orders
        					WHERE reference = "'.pSQL($ref).'"');
				if (!isset($by_cust[$id_cust])) {
					$by_cust[$id_cust] = array(
						'orders' => array(),
						'slips' => array(),
						'payments' => array(),
					);
				}
				$by_cust[$id_cust]['payments'][$id_payment] = $payment;
			}
		}

		// One operation by customer
		foreach($by_cust as $id_cust => &$change) {
			$this->logSaleOperation($datetime, $ctx, $change, $id_cust);
		}
		$this->getDbLink()->commit();
	}

	protected function logSaleOperation($datetime, $ctx, $change, $id_cust) {
	    $id_lang = Context::getContext()->language->id;
	    require_once(_KERAWEN_TOOLS_DIR_.'/compute.php');
	    $db = Db::getInstance();
		
		$operation = array(
			'data' => array_merge(array(
				'date' => date('Y-m-d H:i:s', $datetime),
				'type' => pSQL(_KERAWEN_525_OP_SALE_),
			), $ctx),
			'orders' => array(),
			'payments' => array(),
		);
		
		$customer = new Customer($id_cust);
		$group = new Group((int)$customer->id_default_group, $id_lang);
		
		// Constants
		require_once(_KERAWEN_CLASS_DIR_.'/KerawenTax.php');
		$id_vat_margin = KerawenTax::GetVatMarginRule();
		
		$product_to_payment = array(
		    _KERAWEN_TYPE_GIFTCARD_ => _KERAWEN_PM_GIFT_CARD_,
		    _KERAWEN_TYPE_PREPAID_ => _KERAWEN_PM_ACCOUNT_,
		    _KERAWEN_TYPE_REFUND_ => _KERAWEN_PM_DEPOSIT_,
		);
		
		$payment_modes = json_decode(Configuration::get('KERAWEN_PAYMENTS'), true);
		// TODO Add GIFTCARD to payments an propose them from till (like CREDIT)
		$payment_modes[_KERAWEN_PM_GIFT_CARD_] = array(
		    'label' => 'Carte cadeau',
		);
		
		$sales = array();
		foreach ($change['orders'] as $id_order => &$order) {
			$order['after'] = new Order($id_order);
			
			// Workaround for kerawen: update payment status now that everything is saved
			require_once(_KERAWEN_CLASS_DIR_.'/drawer.php');
			if ($order['after']->reference) {
			    updateOrderPaymentStatus($order['after']->reference);
			}
			
			// Valid status
			// When id_order = 0, it's a virtual order for returns
			$valid_before = $id_order ? ($order['before'] ? isOrderStateValid($order['before']->current_state) : false) : true;
			$valid_after = $id_order ? isOrderStateValid($order['after']->current_state) : true;
			
			// Accounted status
			require_once(_KERAWEN_CLASS_DIR_.'/order_state.php');
			$accounted_before = isOrderAccounted($id_order);
			
			$accountable = Configuration::get('KERAWEN_MANAGE_PO') && isset($order['after']->current_state)
			     ? isOrderStateAccountable($order['after']->current_state) : $valid_after;
			$accounted_after = $accounted_before || $accountable;
			$db->update('order_kerawen', array( 'accounted' => $accounted_after), 'id_order = '.pSQL($id_order));
			
			$info = $this->getOrderInfo($id_order);

			// Detect invoice
			$invoice_before = $order['before'] ? $order['before']->invoice_number : 0;
			$invoice_after = $order['after']->invoice_number;
			$invoice = ($invoice_after && !$invoice_before) ? $invoice_after : 0;

			if (!$valid_after) {
				if (!$valid_before) {
					// Nothing changed, log sale anyway for refunding
				    $sale = array(
				        'data' => array_merge(array(
				            'id_order' => $id_order,
				            'id_order_slip' => 0,
				        ), $info),
				        'accounted' => (int)$accounted_after,
				        'details' => array(),
				        'cartrules' => array(),
				        'global' => -1,
				    );
				    $sales[] = $sale;
				}
				else {
					// Order has been canceled
					// TODO check that accounted_before = accounted_after
					$sale = array(
						'data' => array_merge(array(
							'id_order' => $id_order,
							'id_order_slip' => 0,
						), $info),
						'accounted' => (int)$accounted_after,
						'details' => $this->getOrderDetails($order['before'], $info, true),
						'cartrules' => $info['cartrules'],
					    'global' => -1,
					);
					$sales[] = $sale;
				}
			}
			else {
				if (!$valid_before) {
					// Order has been validated
					// TODO test accounted_before vs valid_before
					$sale = array(
						'data' => array_merge(array(
							'id_order' => $id_order,
							'id_order_slip' => 0,
						), $info),
						'accounted' => (int)$accounted_after,
						'details' => $this->getOrderDetails($order['after'], $info, false),
						'cartrules' => $info['cartrules'],
					    'global' => +1,
					    'invoice' => $invoice,
					);
					$sales[] = $sale;
				}
				else if (!$accounted_before && $accounted_after) {
					// Order has been confirmed
					$sale = array(
						'data' => array_merge(array(
							'id_order' => $id_order,
							'id_order_slip' => 0,
						), $info),
						'accounted' => (int)$accounted_before,
						'details' => $this->getOrderDetails($order['after'], $info, true),
						'cartrules' => $info['cartrules'],
					    'global' => -1,
					);
					$sales[] = $sale;
					
					$sale = array(
						'data' => array_merge(array(
							'id_order' => $id_order,
							'id_order_slip' => 0,
						), $info),
						'accounted' => (int)$accounted_after,
						'details' => $this->getOrderDetails($order['after'], $info, false),
						'cartrules' => $info['cartrules'],
					    'global' => +1,
					);
					$sales[] = $sale;
				}
				else if (isset($order['slips']) && count($order['slips'])) {
				    // Part of order have been returned
					$id_address = $this->getOrderTaxAddress($order['after']);
					
					foreach($order['slips'] as $id_slip => &$s) {
						// TODO check accounted_before and after are both true
						$sale = array(
							'data' => array_merge(array(
								'id_order' => $id_order,
								'id_order_slip' => $id_slip,
							), $info),
							'accounted' => (int)$accounted_after,
							'details' => array(),
							'cartrules' => array(),
						    'global' => false,
						);
						
						$before = $s['before'];
						$after = new OrderSlip($id_slip);
						$after->details = OrderSlip::getOrdersSlipDetail($id_slip);
						// TODO handle change if before is not null ???
						
						// PS backward compatibility
						if (!isset($after->total_shipping_tax_excl)) {
							$o = new Order($after->id_order);
							$after->total_shipping_tax_incl = $after->shipping_cost_amount;
							$after->total_shipping_tax_excl = $after->shipping_cost_amount/(1 + $o->carrier_tax_rate/100);
						}
						
						foreach ($after->details as &$sd) {
							$id_order_detail = $sd['id_order_detail'];
							
							$order_detail = new OrderDetail($id_order_detail);
							$more = $db->getRow('
									SELECT odk.*,
                                        pk.is_gift_card,
                                        pk.is_account_credit,
                                        pk.product_type
                                    FROM '._DB_PREFIX_.'order_detail od
                                    LEFT JOIN '._DB_PREFIX_.'order_detail_kerawen odk ON odk.id_order_detail = od.id_order_detail
                                    LEFT JOIN '._DB_PREFIX_.'product_wm_kerawen pk ON pk.id_product = od.product_id
									WHERE od.id_order_detail = '.pSQL($id_order_detail));
							
							// PS backward compatibility
							if (!isset($sd['unit_price_tax_excl']))
								$sd['unit_price_tax_excl'] = $sd['amount_tax_excl']/$sd['product_quantity'];
							if (!isset($sd['unit_price_tax_incl']))
								$sd['unit_price_tax_incl'] = $sd['amount_tax_incl']/$sd['product_quantity'];
							if (!isset($order_detail->id_tax_rules_group))
								$order_detail->id_tax_rules_group = $db->getValue('
										SELECT id_tax_rules_group FROM '._DB_PREFIX_.'order_detail_kerawen
										WHERE id_order_detail = '.pSQL($order_detail->id));
							if (!isset($order_detail->original_wholesale_price))
								$order_detail->original_wholesale_price = 0.0;
							
							$prod = $this->getProductInfo($id_order_detail);
							
							$sale['details'][] = array(
								'id_order_detail' => $id_order_detail,
								'item_name' => $prod['item_name'],
								'version_name' => $prod['version_name'],
								'unit_te' => $sd['unit_price_tax_excl'],
								'unit_ti' => $sd['unit_price_tax_incl'],
								'id_tax' => $order_detail->id_tax_rules_group,
							    'margin_vat' => $order_detail->id_tax_rules_group == $id_vat_margin ? 1 : $more['margin_vat'],
							    'rest_voucher' => $more['rest_voucher'],
								'id_address' => $id_address,
							    'measure' => $more['measure'],
							    'measure_unit' => $more['unit'],
							    'measure_precision' => $more['precision'],
							    'quantity' => -$sd['product_quantity'],
							    'discount_te' => 0,
								'discount_ti' => 0,
								'discount_percent' => 0,
								'purchase_te' => $order_detail->purchase_supplier_price,
							    'gift_card' => $more['is_gift_card'],
							    'account_credit' => $more['is_account_credit'],
							    'product_type' => $more['product_type'],
							);
						}
						
						if ($after->total_shipping_tax_excl != 0) {
							$carrier = $this->getCarrierInfo($order['after']->id_carrier, $ctx['id_shop']);
							
							$sale['details'][] = array(
								'item_name' => $this->label_shipping,
								'version_name' => $carrier['name'],
								'id_carrier' => $carrier['id'],
								'unit_te' => $after->total_shipping_tax_excl,
								'unit_ti' => $after->total_shipping_tax_incl,
								'id_tax' => $carrier['id_tax'],
								'id_address' => $id_address,
								'quantity' => -1,
								'discount_te' => 0,
								'discount_ti' => 0,
								'discount_percent' => 0,
								'purchase_te' => $after->total_shipping_tax_excl,
							);
						}
						
						$sales[] = $sale;
					}
				}
				else {
					// Order has been modified
					// TODO check accounted_before and after are both false
					$sale = array(
						'data' => array_merge(array(
							'id_order' => $id_order,
							'id_order_slip' => 0,
						), $info),
						'accounted' => (int)$accounted_after,
						'details' => array(),
						'cartrules' => array(),
					    'global' => false,
					);
					
					$id_address = $this->getOrderTaxAddress($order['after']);
					
					// Check products
					foreach ($order['items'] as $id_order_detail => &$detail) {
						$before = $detail['before'];
						$after = new OrderDetail($id_order_detail);
						if (!$after->id_order_detail) {
							$after->qty = 0;
							$after->unit_price_tax_excl = $before->unit_price_tax_excl;
							// FIXME
							$prod = array(
								'item_name' => 'item deleted',
								'version_name' => 'version deleted',
							);
						}
						else {
							$after->qty = $after->product_quantity - $after->product_quantity_refunded - $after->product_quantity_return;
							$prod = $this->getProductInfo($id_order_detail);
						}
						
						if ($before) {
							$before->qty = $before->product_quantity - $before->product_quantity_refunded - $before->product_quantity_return;
							
							// Change in quantity
							if ($after->qty != $before->qty) {
								$sale['details'][] = array(
									'id_order_detail' => $id_order_detail,
									'item_name' => $prod['item_name'],
									'version_name' => $prod['version_name'],
									'unit_te' => $before->unit_price_tax_excl,
									'unit_ti' => $before->unit_price_tax_incl,
									'id_tax' => $before->id_tax_rules_group,
									'id_address' => $id_address,
									'quantity' => $after->qty - $before->qty,
									'discount_te' => 0,
									'discount_ti' => 0,
									'discount_percent' => 0,
									'purchase_te' => $before->unit_price_tax_excl,
								);
							}
							
							// Change in price
							if ($after->unit_price_tax_excl != $before->unit_price_tax_excl) {
								$sale['details'][] = array(
									'id_order_detail' => $id_order_detail,
									'item_name' => $prod['item_name'],
									'version_name' => $prod['version_name'],
									'unit_te' => $after->unit_price_tax_excl - $before->unit_price_tax_excl,
									'unit_ti' => $after->unit_price_tax_incl - $before->unit_price_tax_incl,
									'id_tax' => $after->id_tax_rules_group,
									'id_address' => $id_address,
									'quantity' => $after->qty,
									'discount_te' => 0,
									'discount_ti' => 0,
									'discount_percent' => 0,
									'purchase_te' => $after->unit_price_tax_excl - $before->unit_price_tax_excl,
								);
							}
						}
						else {
							// New product line in order
							$sale['details'][] = array(
								'id_order_detail' => $id_order_detail,
								'item_name' => $prod['item_name'],
								'version_name' => $prod['version_name'],
								'unit_te' => $after->unit_price_tax_excl,
								'unit_ti' => $after->unit_price_tax_incl,
								'id_tax' => $after->id_tax_rules_group,
								'id_address' => $id_address,
								'quantity' => $after->qty,
								'discount_te' => $after->reduction_amount_tax_excl,
								'discount_ti' => $after->reduction_amount_tax_incl,
								'discount_percent' => $after->reduction_percent,
								'purchase_te' => $after->unit_price_tax_excl,
							);
						}
					}
					
					// Check shipping & wrapping
					$after = $order['after'];
					$before = $order['before'];
					if (($after->total_shipping_tax_excl != $before->total_shipping_tax_excl) ||
					($after->total_shipping_tax_incl != $before->total_shipping_tax_incl)) {
						$carrier = $this->getCarrierInfo($order['after']->id_carrier, $ctx['id_shop']);
							
						$sale['details'][] = array(
							'item_name' => $this->label_shipping,
							'version_name' => $carrier['name'],
							'id_carrier' => $carrier['id'],
							'unit_te' => $after->total_shipping_tax_excl - $before->total_shipping_tax_excl,
							'unit_ti' => $after->total_shipping_tax_incl - $before->total_shipping_tax_incl,
							'id_tax' => $carrier['id_tax'],
							'id_address' => $id_address,
							'quantity' => 1,
							'discount_te' => 0,
							'discount_ti' => 0,
							'discount_percent' => 0,
							'purchase_te' => $after->total_shipping_tax_excl - $before->total_shipping_tax_excl,
						);
					}
					
					if (($after->total_wrapping_tax_excl != $before->total_wrapping_tax_excl) ||
					($after->total_wrapping_tax_incl != $before->total_wrapping_tax_incl)) {
						$sale['details'][] = array(
							'item_name' => $this->label_wrapping,
							'wrapping' => 1,
							'unit_te' => $after->total_wrapping_tax_excl - $before->total_wrapping_tax_excl,
							'unit_ti' => $after->total_wrapping_tax_incl - $before->total_wrapping_tax_incl,
							'id_tax' => Configuration::get('PS_GIFT_WRAPPING_TAX_RULES_GROUP'),
							'id_address' => $id_address,
							'quantity' => 1,
							'discount_te' => 0,
							'discount_ti' => 0,
							'discount_percent' => 0,
							'purchase_te' => $after->total_wrapping_tax_excl - $before->total_wrapping_tax_excl,
						);
					}
					
					$sales[] = $sale;
				}
			}
		}
		$operation['orders'] = $sales;
		
		// Gather payments
		$pcount = 0;
		$pmodule = false;
		foreach ($change['payments'] as $id_payment => &$p) {
			$pcount++;
			$before = $p['before'];
			$after = $this->getOrderPayment($id_payment);
			if (!$before && !$after->id) continue; // Payment has been created and deleted
			
			$more = $db->getRow('
					SELECT *
					FROM '._DB_PREFIX_.'order_payment_kerawen
					WHERE id_order_payment = '.pSQL($id_payment));
			if ($more && $more['id_mode'] == _KERAWEN_PM_RETURN_) continue; // Returned product used as payment
			
			// Get orders covered by payment
			$orders = $db->executeS('
				SELECT id_order, module
				FROM '._DB_PREFIX_.'orders
				WHERE reference = "'.pSQL($after->id ? $after->order_reference : $before->order_reference).'"');
			$buf = array();
			foreach ($orders as $o) { 
			    $buf[] = $o['id_order']; 
			    $pmodule = $o['module'];
			}
			$orders = implode(',', $buf);
			
			if ($before) {
			    if ($after->id && $after->payment_method != $before->payment_method) {
					// Get previous logged payment mode (the last one)
					$old_mode = $db->getValue('
						SELECT id_mode
						FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment
						WHERE id_order_payment = '.pSQL($id_payment).'
						ORDER BY id_payment DESC');
					
					$operation['payments'][] = array(
						'id_order_payment' => $id_payment,
						'ps_orders' => $orders,
						'id_currency' => $before->id_currency,
						'currency' => $this->getCurrencyInfo($before->id_currency)->name,
						'amount' => -$before->amount,
						'mode' => $before->payment_method,
						'id_mode' => $old_mode,
					    'deferred' => $more ? $more['deferred'] : null,
					    'remain' => $more ? $more['remain'] : null,
					);
					$operation['payments'][] = array(
						'id_order_payment' => $id_payment,
						'ps_orders' => $orders,
						'id_currency' => $after->id_currency,
						'currency' => $this->getCurrencyInfo($after->id_currency)->name,
						'amount' => $after->amount,
						'mode' => $after->payment_method,
						'id_mode' => $more ? $more['id_mode'] : null,
						'deferred' => $more ? $more['deferred'] : null,
					    'remain' => $more ? $more['remain'] : null,
					);
				}
				else if ($after->amount != $before->amount) {
				    // Keep before data as payment may have been deleted
					$operation['payments'][] = array(
						'id_order_payment' => $id_payment,
						'ps_orders' => $orders,
						'id_currency' => $before->id_currency,
					    'currency' => $this->getCurrencyInfo($before->id_currency)->name,
						'amount' => $after->amount - $before->amount,
					    'mode' => $before->payment_method,
					    'id_mode' => $more ? $more['id_mode'] : null,
					    'deferred' => $more ? $more['deferred'] : null,
					    'remain' => $more ? $more['remain'] : null,
					);
				}
			}
			else {
				$operation['payments'][] = array(
					'id_order_payment' => $id_payment,
					'ps_orders' => $orders,
					'id_currency' => $after->id_currency,
					'currency' => $this->getCurrencyInfo($after->id_currency)->name,
					'amount' => $after->amount,
					'mode' => $after->payment_method,
				    'id_mode' => $more ? $more['id_mode'] : null,
				    'deferred' => $more ? $more['deferred'] : null,
				    'remain' => $more ? $more['remain'] : null,
				);
			}
		}
		// TODO Update payment status in replacement of module kerawen ??
		
		// Summarize sale & order
		$sales = array();
		foreach (array(0, 1) as $accounted) {
			$sales[$accounted] = array(
				'data' => array(
					'accounted' => $accounted,
					'id_customer' => $customer ? $customer->id : null,
					'customer_name' => $customer ? ($customer->firstname.' '.$customer->lastname) : null,
				    'id_group' => $customer ? $customer->id_default_group : null,
				    'group_name' => $group ? $group->name : null,
					'nb_lines' => 0,
					'total_te' => 0,
					'total_ti' => 0,
				),
			    'count' => 0,
				'orders' => array(),
				'taxes' => array(),
			);
		}
		
		$ocount = 0;
		foreach ($operation['orders'] as &$order) {
		    $ocount++;

			// First loop on details:explode packs
			$new_details = array();
			foreach ($order['details'] as &$detail) {
				$unit_details = array();
				if ( isset($detail['packed']) && $detail['packed'] && (int) Configuration::get('KERAWEN_PACK_SPLIT') ) {
					$content = $this->getPackInfo($detail['id_order_detail']);
					$prices = array();
					foreach($content as &$p) {
						$info = $this->getTaxInfo($p['id_tax'], $detail['id_address']);
						$p['tax_name'] = $info['name'];
						$p['tax_rate'] = $info['rate'];
						$p['base_ti'] = round($p['base_te']*(1 + $p['tax_rate']/100), 6);
						$prices[] = $p['base_ti']*$p['quantity'];
					}

					// TODO take discount into account
					$discounts = proportionalize($detail['discount_ti'], $prices);
					$prices = proportionalize($detail['unit_ti'], $prices);
					
					for ($i = 0; $i < count($prices); $i++) {
						$p = &$content[$i];
						$d = array_merge(array(), $detail); // Copy pack detail
						$d['item_name'] = $detail['item_name'].' - '.$p['item_name'];
						$d['version_name'] = $p['version_name'];
						$d['quantity'] = $detail['quantity']*$p['quantity'];
						$d['unit_ti'] = $prices[$i]/$p['quantity'];
						$d['discount_ti'] = $discounts[$i]/$p['quantity'];
						$d['id_tax'] = $p['id_tax'];
						$d['purchase_te'] = $p['purchase_te'];
						
						$unit_details[] = $d;
					}
				}
				else {
					// Single product
					$unit_details[] = $detail;
				}
				$new_details = array_merge($new_details, $unit_details);
			}
			$order['details'] = $new_details;

			// Second loop: complete data & apply discounts
			$round_type = $order['data']['round_type'];
			if (!$round_type) $round_type = _KERAWEN_RM_LINE_;
			$nb_lines = 0;
			$global_discount_ti = 0;
			
			foreach ($order['details'] as $key => &$detail) {
			    // Ensure product compatibility
			    if (!isset($detail['product_type'])) $detail['product_type'] = '';
			    if (isset($detail['gift_card']) && $detail['gift_card']) $detail['product_type'] = _KERAWEN_TYPE_GIFTCARD_;
			    if (isset($detail['account_credit']) && $detail['account_credit']) $detail['product_type'] = _KERAWEN_TYPE_PREPAID_;
			    
			    // Some products are not "saled"
    		    switch($detail['product_type']) {
    		        case _KERAWEN_TYPE_GIFTCARD_:
    		        case _KERAWEN_TYPE_PREPAID_:
    		        case _KERAWEN_TYPE_REFUND_:
    		            $mode = $product_to_payment[$detail['product_type']];
    		            array_unshift($operation['payments'], array(
        		            'id_order_payment' => 0,
        		            'ps_orders' => $order['data']['id_order'],
        		            'id_currency' => null, //TODO
        		            'currency' => null, //TODO
        		            'amount' => -$detail['unit_ti']*$detail['quantity'],
    		                'mode' => pSQL(isset($payment_modes[$mode]) ? $payment_modes[$mode]['label'] : $mode),
    		                'id_mode' => $mode,
        		            'deferred' => null,
        		            'remain' => null,
    		            ));
    		            unset($order['details'][$key]);
    		            continue 2;
    		    }
			    
				$nb_lines++;
				
				if (!isset($detail['version_name']) || is_null($detail['version_name'])) $detail['version_name'] = '';
				if (!isset($detail['gift_card'])) $detail['gift_card'] = 0;
				if (!isset($detail['account_credit'])) $detail['account_credit'] = 0;
				if (!isset($detail['margin_vat'])) $detail['margin_vat'] = 0;
				if (!isset($detail['rest_voucher'])) $detail['rest_voucher'] = 0;
				if (!isset($detail['product_type'])) $detail['product_type'] = _KERAWEN_TYPE_STANDARD_;
				if (!isset($detail['measure'])) $detail['measure'] = 1;
				
				$tinfo = $this->getTaxInfo($detail['id_tax'], $detail['id_address']);
				$detail['tax_name'] = $tinfo['name'];
				$detail['tax_rate'] = $tinfo['rate'];
				unset($detail['id_address']);
				$tax_coef = 1 + $detail['tax_rate']/100;
				
				// Adjust purchase price from stock if any
				if (isset($detail['stock_te'])) $detail['purchase_te'] = $detail['stock_te'];
				unset($detail['stock_te']);
				
				// In the following, consider prices tax included
				// in order to comply to specific tax ru les (e.g. tax on margin)
				
				// Compute effective discount if percent
				if (isset($detail['discount_percent']) && $detail['discount_percent'] > 0) {
					$detail['discount_ti'] = $detail['unit_ti']*$detail['discount_percent']/(100 - $detail['discount_percent']);
				}
				
				// Avoid negative discounts
				if ($detail['discount_ti'] < 0) {
					$detail['discount_ti'] = 0;
				}
				$detail['discount_te'] = $detail['discount_ti']/$tax_coef;
				
				$detail['unit_ti'] = $detail['unit_ti'] + $detail['discount_ti'];
				if ($detail['margin_vat']) {
					$margin = $detail['unit_ti'] - $detail['purchase_te'];
					if ($margin < 0){
						$detail['unit_te'] = $detail['unit_ti'];
					} 
					else { 
						$detail['unit_te'] = $detail['purchase_te'] + $margin/$tax_coef;
					}
				}
				else {
					$detail['unit_te'] = $detail['unit_ti']/$tax_coef;
				}
				
				if (isset($detail['ecotax_te'])) {
				    $detail['ecotax_ti'] = $detail['ecotax_te']*$tax_coef;
				}
				
				// Packs are handled as a single item by PS
				if ($round_type == _KERAWEN_RM_ITEM_ && (!isset($detail['packed']) || !$detail['packed'])) {
					// Rounding is apply to final price
					$final_te = round($detail['unit_te'] - $detail['discount_te'], 2);
					$final_ti = round($detail['unit_ti'] - $detail['discount_ti'], 2);
					
					$detail['unit_te'] = round($detail['unit_te'], 2);
					$detail['unit_ti'] = round($detail['unit_ti'], 2);
					$detail['discount_te'] = $detail['unit_te'] - $final_te;
					$detail['discount_ti'] = $detail['unit_ti'] - $final_ti;
				}
				
				$detail['discount_rate'] = 0;
				if (isset($order['global']) && $order['global']) {
    				// Apply global discount, according to taxes
    				$detail['discount_rate'] = isset($detail['discount_rate_ti']) ? $detail['discount_rate_ti'] : 0;
				}
				unset($detail['discount_rate_te']);
				unset($detail['discount_rate_ti']);
				
				$detail['total_te'] = ($detail['unit_te'] - $detail['discount_te'])*(1 - $detail['discount_rate'])*$detail['quantity'];
				if ($round_type == _KERAWEN_RM_LINE_ || (isset($detail['packed']) && $detail['packed'])) {
				    $detail['total_te'] = round($detail['total_te'], 2);
				}
				$detail['total_ti'] = round(($detail['unit_ti'] - $detail['discount_ti'])*(1 - $detail['discount_rate'])*$detail['quantity'], 2);
				
				// Detect rounding error on global discount
				$global_discount_ti += round(($detail['unit_ti'] - $detail['discount_ti'])*$detail['quantity'], 2) - $detail['total_ti'];
				
				// Get actual unit prices
				$detail['unit_te'] = $detail['unit_te']/$detail['measure'];
				$detail['unit_ti'] = $detail['unit_ti']/$detail['measure'];
				$detail['discount_te'] = $detail['discount_te']/$detail['measure'];
				$detail['discount_ti'] = $detail['discount_ti']/$detail['measure'];
				
				unset($detail['packed']);
			}
            
			// Fix rounding error on global discount
			if (isset($order['global']) && $order['global'] && count($order['details'])) {
                $discount_error = $order['global']*$global_discount_ti - $order['data']['total_discounts_tax_incl'];
			    if ($discount_error != 0) {
			        // Sort by tax amount
			        $sort = function ($a, $b) {
			            return abs($b['total_ti'] - $b['total_te']) > abs($a['total_ti'] - $a['total_te']) ? 1 : -1;
			        };
			        $rows = &$order['details'];
			        uasort($rows, $sort);
			        
			        // Adjust items until no difference
			        $error = $order['global']*$discount_error;
			        $unit = 100;
			        $amount = (int) round($error*$unit);
			        $remainder = ($amount < 0 ? -1 : +1)*(abs($amount)%count($rows));
			        $adjust = ($amount - $remainder)/count($rows)/$unit;
			        $sign = $error >= 0 ? +1 : -1;
			        
			        $position = 0;
			        foreach ($rows as &$row) {
			            $row['total_ti'] += $adjust;
			            if ($position < abs($remainder)) {
			                $row['total_ti'] += $sign/$unit;
			            }
			            $position++;
			        }
    			}
			}
			
			// Third loop: summarize & compute taxes
			$total_te = $total_ti = 0;
			$taxes = array();
			
			foreach ($order['details'] as &$detail) {
			    $total_te += $detail['total_te'];
			    $total_ti += $detail['total_ti'];
			    
			    if (!isset($taxes[$detail['id_tax']])) {
			        $taxes[$detail['id_tax']] = array(
			            'id_tax' => $detail['id_tax'],
			            'tax_name' => $detail['tax_name'],
			            'tax_rate' => $detail['tax_rate'],
			            'total_te' => 0.0,
			            'total_ti' => 0.0,
			            'tax_amount' => 0.0,
			        );
			    }
			    $tax = &$taxes[$detail['id_tax']];
			    $tax['total_te'] += $detail['total_te'];
			    $tax['total_ti'] += $detail['total_ti'];
			    $tax['tax_amount'] = $tax['total_ti'] - $tax['total_te'];
			}
			
			$order['data']['total_te'] = $total_te;
			$order['data']['total_ti'] = $total_ti;
			$order['taxes'] = $taxes;
			
			$sale = &$sales[$order['accounted']];
			$sale['data']['nb_lines'] += $nb_lines;
			$sale['data']['total_te'] += $total_te;
			$sale['data']['total_ti'] += $total_ti;
			$sale['count']++;
			$sale['orders'][] = $order;
			
			foreach ($taxes as &$t) {
				if ($round_type == _KERAWEN_RM_TOTAL_) $t['total_te'] = round($t['total_te'], 2);
				$t['tax_amount'] = $t['total_ti'] - $t['total_te'];
					
				if (!isset($sale['taxes'][$t['id_tax']])) {
					$sale['taxes'][$t['id_tax']] = array(
						'id_tax' => $t['id_tax'],
						'tax_name' => $t['tax_name'],
						'tax_rate' => $t['tax_rate'],
						'total_te' => 0.0,
						'total_ti' => 0.0,
						'tax_amount' => 0.0,
					);
				}
				$tax = &$sale['taxes'][$t['id_tax']];
				$tax['total_te'] += $t['total_te'];
				$tax['total_ti'] += $t['total_ti'];
				$tax['tax_amount'] += $t['tax_amount'];
			}
		}
		
		// --------------------------------
		// 2. Sign & save (TO BE REDUNDED ON SECURE SERVER)
		
		if ($ocount || $pcount) {
		    if ($pcount && !$ocount) {
		        // At least one virtual accounted sale for payment change
		        $sales[1]['count']++;
		    }
		    
			// Store shop infos
			$operation['data']['id_shop'] = $ctx['id_shop'];
			$shop_fields = array(
				'PS_SHOP_NAME' => 'PS_SHOP_NAME',
				'PS_SHOP_EMAIL' => 'PS_SHOP_EMAIL',
				'PS_SHOP_ADDR1' => 'PS_SHOP_ADDR1',
				'PS_SHOP_ADDR2' => 'PS_SHOP_ADDR2',
				'PS_SHOP_CODE' => 'PS_SHOP_CODE',
				'PS_SHOP_CITY' => 'PS_SHOP_CITY',
				'PS_SHOP_COUNTRY' => 'PS_SHOP_COUNTRY',
				'PS_SHOP_PHONE' => 'PS_SHOP_PHONE',
				'KERAWEN_SHOP_SIRET' => 'KERAWEN_SHOP_SIRET',
				'KERAWEN_SHOP_NAF' => 'KERAWEN_SHOP_NAF',
				'KERAWEN_SHOP_TVA_INTRA' => 'KERAWEN_SHOP_TVA_INTRA',
				'KERAWEN_SHOP_URL' => 'KERAWEN_SHOP_URL',
			);
			foreach ($shop_fields as $name => $field) {
				$operation['data'][$field] = Configuration::get($name, null, null, $ctx['id_shop']);
			}
			
			$db->insert(_KERAWEN_525_PREFIX_.'operation', array_map('pSQL', $operation['data']));
			$id_operation = $db->Insert_ID();
			
			// Save sale
			foreach($sales as &$sale) {
			    if ($sale['count']) {
					$sale['data']['id_operation'] = $id_operation;
					$sale['data']['nb_prints'] = 1;
					
					$db->insert(_KERAWEN_525_PREFIX_.'sale', array_map('pSQL', $sale['data']));
					$id_sale = $db->Insert_ID();
					
					foreach ($sale['orders'] as &$order) {
						if (true /*count($order['details'])*/) {
							$db->insert(_KERAWEN_525_PREFIX_.'order', array(
								'id_sale' => $id_sale,
								'ps_order' => $order['data']['id_order'],
								'ps_slip' => $order['data']['id_order_slip'],
								'total_te' => $order['data']['total_te'],
								'total_ti' => $order['data']['total_ti'],
							), true);
							$id_order = $db->Insert_ID();
					
							foreach ($order['details'] as &$detail) {
								$detail['id_sale'] = $id_sale;
								$detail['id_order'] = $id_order;
								$db->insert(_KERAWEN_525_PREFIX_.'sale_detail', array_map('pSQL', $detail), false);
								$id_sale_detail = $db->Insert_ID();
							}
							
							if (count($order['taxes'])) {
								foreach ($order['taxes'] as &$t) {
									$t['id_order'] = $id_order;
									$t['tax_name'] = pSQL($t['tax_name']);
								}
								$db->insert(_KERAWEN_525_PREFIX_.'order_tax', $order['taxes'], true);
							}
						}
							
						if (count($order['cartrules'])) {
							foreach ($order['cartrules'] as &$cartrule) {
								$cartrule['id_sale'] = $id_sale;
								$cartrule['id_order'] = $id_order;
								$cartrule['label'] = pSQL($cartrule['label']);
							}
							$db->insert(_KERAWEN_525_PREFIX_.'discount', $order['cartrules'], true);
						}
						
						// Trigger invoice signature
						if (isset($order['invoice']) && $order['invoice']) {
						    $this->getInvoiceData($order['data']['id_order'], $order['data']['id_order_slip'], true, false);
						}
					}
						
					if (count($sale['taxes'])) {
						foreach ($sale['taxes'] as &$tax) {
							$tax['id_sale'] = $id_sale;
							$tax['tax_name'] = pSQL($tax['tax_name']);
						}
						$db->insert(_KERAWEN_525_PREFIX_.'sale_tax', $sale['taxes'], true);
					}
						
					// Signature
					$data = array();
					foreach ($sale['taxes'] as &$tax) {
						$data[$this->formatAmount($tax['tax_rate'])] = $this->formatAmount($tax['total_ti']);
					}
					$data = array(
						$data,
						$this->formatAmount($sale['data']['total_ti']),
						$this->formatDate($operation['data']['date']),
						$id_sale,
						_KERAWEN_525_OP_SALE_);
					$sign = $this->sign(_KERAWEN_525_OP_SALE_, $data);
					$db->update(_KERAWEN_525_PREFIX_.'sale', array(
						'description' => $sign['desc'],
						'signature' => $sign['sign'],
						'sign_short' => $sign['short'],
					), 'id_sale = '.pSQL($id_sale));
					
					// Grand totals
					// Only for sale, not for order
					if ($sale['data']['accounted']) {
						$this->updateGrandTotal($datetime, $id_sale, $sale['data'], $sale['taxes']);
					}
				}
			}
			
			// Payments
			if (count($operation['payments'])) {
			    //Ex: {"monetico1f":"CB Monetico"}
			    $mode_override = Configuration::get('KERAWEN_PAYMENT_MODE_OVERRIDE') ? (array) json_decode(Configuration::get('KERAWEN_PAYMENT_MODE_OVERRIDE'), true) : array();			    
				foreach ($operation['payments'] as &$p) {
				    $p['id_operation'] = $id_operation;
				    $p['mode'] = ($pmodule && isset($mode_override[$pmodule])) ? pSQL($mode_override[$pmodule]) : pSQL($p['mode']);
					$p['currency'] = pSQL($p['currency']);
				}
				$db->insert(_KERAWEN_525_PREFIX_.'payment', $operation['payments'], true);
			}
		}
	}
	
	public function cancelOperation($id_op, $reason) {
		$this->getDbLink()->beginTransaction();
		$db = Db::getInstance();
	
		$id_event = $this->logEvent(_KERAWEN_525_EVT_CANCEL_OP_,
			'Cancelation',
			'operation #'.$id_op.': '.$reason);
		
		$db->execute('
			UPDATE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
			SET canceled = '.pSQL($id_event).'
			WHERE id_operation = '.pSQL($id_op));
	
		// Update grand totals with opposite amounts
		$datetime = time();
		$sales = $db->executeS('
			SELECT * FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale
			WHERE id_operation = '.pSQL($id_op));
		
		foreach($sales as &$sale) {
			$id_sale = $sale['id_sale'];
			$sale['total_te'] = -$sale['total_te'];
			$sale['total_ti'] = -$sale['total_ti'];
			
			$taxes = $db->executeS('
				SELECT * FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_tax
				WHERE id_sale = '.pSQL($id_sale));
			
			foreach($taxes as &$tax) {
				$tax['total_te'] = -$tax['total_te'];
				$tax['total_ti'] = -$tax['total_ti'];
				$tax['tax_amount'] = -$tax['tax_amount'];
			}
			
			$this->updateGrandTotal($datetime, '-'.$id_sale, $sale, $taxes);
		}
		
		$this->getDbLink()->commit();
	}
	
	protected function logEvent($code, $label, $info,
		$datetime = false, $id_operator = false, $sign = true) {
		$db = Db::getInstance();
		
		if (!$datetime) {
			$datetime = time();
		}
		if (!$id_operator) {
			$context = Context::getContext();
			$operator = $context->employee;
			$id_operator = $operator ? $operator->id : null;
		}
		
		$event = array(
			'date' => date('Y-m-d H:i:s', $datetime),
			'id_operator' => $id_operator,
			'code' => $code,
			'label' => pSQL($label),
			'info' => pSQL($info),
		);
		
		$db->insert(_KERAWEN_525_PREFIX_.'event', $event, true);
		$id_event = $db->Insert_ID();
		
		if ($sign) {
			$data = array(
				$id_event,
				$event['code'],
				$event['label'],
				$this->formatDate($event['date']),
				$event['id_operator'],
				0, //$event['id_till'],
			);
			
			$sign = $this->sign(_KERAWEN_525_OP_EVENT_, $data);
			$db->update(_KERAWEN_525_PREFIX_.'event', array(
				'description' => $sign['desc'],
				'signature' => $sign['sign'],
				'sign_short' => $sign['short'],
			), 'id_event = '.pSQL($id_event));
		}
		
		return $id_event;
	}


	/* **************************************************************
	 * GRAND TOTAL
	*/
	
	protected function updateGrandTotal($datetime, $id_sale, $sale, $taxes) {
		$db = Db::getInstance();

		$period_ref = array(
			_KERAWEN_525_PER_DAY_ => date('Y-m-d', $datetime),
			_KERAWEN_525_PER_MONTH_ => date('Y-m', $datetime),
			_KERAWEN_525_PER_YEAR_ => date('Y', $datetime),
		);
		$date = date('Y-m-d H:i:s', $datetime);
		
		// Update perpetual total
		$id_gtotal = $db->getValue('
					SELECT id_gtotal FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal
					WHERE period_type = "'._KERAWEN_525_PER_PERPETUAL_.'"');
		if (!$id_gtotal) {
			$db->insert(_KERAWEN_525_PREFIX_.'gtotal', array(
				'period_type' => _KERAWEN_525_PER_PERPETUAL_,
				'period_ref' => '',
			));
			$id_gtotal = $db->Insert_ID();
		}
		$db->execute('
					UPDATE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal SET
						total_te = total_te + '.pSQL(abs($sale['total_te'])).',
						total_ti = total_ti + '.pSQL(abs($sale['total_ti'])).',
						date = "'.pSQL($date).'"
					WHERE id_gtotal = '.pSQL($id_gtotal));
		$perpetual = $db->getRow('
					SELECT * FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal
					WHERE id_gtotal = '.pSQL($id_gtotal));
			
		// Update grand totals for periods
		foreach ($period_ref as $type => $ref) {
			$id_gtotal = $db->getValue('
						SELECT id_gtotal FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal
						WHERE period_type = "'.pSQL($type).'"
						AND period_ref = "'.pSQL($ref).'"');
			if (!$id_gtotal) {
				$db->insert(_KERAWEN_525_PREFIX_.'gtotal', array(
					'period_type' => $type,
					'period_ref' => $ref,
				));
				$id_gtotal = $db->Insert_ID();
			}
			$db->execute('
						UPDATE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal SET
							total_te = total_te + '.pSQL($sale['total_te']).',
							total_ti = total_ti + '.pSQL($sale['total_ti']).',
							perp_te = '.pSQL($perpetual['total_te']).',
							perp_ti = '.pSQL($perpetual['total_ti']).',
							date = "'.pSQL($date).'"
						WHERE id_gtotal = '.pSQL($id_gtotal));
				
			foreach ($taxes as &$tax) {
				$db->execute('
							INSERT INTO '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal_tax
								(id_gtotal, id_tax, tax_name, tax_rate, total_te, total_ti, tax_amount)
							VALUES (
								'.pSQL($id_gtotal).',
								'.pSQL($tax['id_tax']).',
								"'.pSQL($tax['tax_name']).'",
								'.pSQL($tax['tax_rate']).',
								'.pSQL($tax['total_te']).',
								'.pSQL($tax['total_ti']).',
								'.pSQL($tax['tax_amount']).')
							ON DUPLICATE KEY UPDATE
								total_te = total_te + VALUES(total_te),
								total_ti = total_ti + VALUES(total_ti),
								tax_amount = tax_amount + VALUES(tax_amount)');
			}
		}
	
		// Save sale grand total
		$db->insert(_KERAWEN_525_PREFIX_.'gtotal', array(
			'period_type' => _KERAWEN_525_PER_SALE_,
			'period_ref' => $id_sale,
			'total_te' => $sale['total_te'],
			'total_ti' => $sale['total_ti'],
			'perp_te' => $perpetual['total_te'],
			'perp_ti' => $perpetual['total_ti'],
			'date' => pSQL($date),
		), true);
		$id_gtotal = $db->Insert_ID();
		$this->signGrandTotal($id_gtotal);
	}

	protected function closeGrandTotal($datetime) {
		$this->getDbLink()->beginTransaction();
		$db = Db::getInstance();
		
		$period_ref = array(
		    _KERAWEN_525_PER_YEAR_ => date('Y', $datetime),
			_KERAWEN_525_PER_MONTH_ => date('Y-m', $datetime),
		    _KERAWEN_525_PER_DAY_ => date('Y-m-d', $datetime),
		);
		
		$gtotals = $db->executeS('
			SELECT * FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal
			WHERE period_type IN (
                "'._KERAWEN_525_PER_YEAR_.'",
                "'._KERAWEN_525_PER_MONTH_.'",
                "'._KERAWEN_525_PER_DAY_.'")
			AND signature IS NULL
			ORDER BY id_gtotal DESC');
		foreach ($gtotals as $gt) {
			if ($gt['period_ref'] < $period_ref[$gt['period_type']]) {
				$this->signGrandTotal($gt['id_gtotal']);
				$this->logEvent(
					_KERAWEN_525_EVT_CLOSE_PERIOD_,
					'Close period',
					'#'.$gt['id_gtotal'].': '.$gt['period_ref'],
					$datetime);
			}
		}
		$this->getDbLink()->commit();
	}
	
	protected function signGrandTotal($id_gtotal) {
		$db = Db::getInstance();
		
		$gtotal = $db->getRow('
			SELECT * FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal
			WHERE id_gtotal = '.pSQL($id_gtotal));
		if ($gtotal['period_type'] == _KERAWEN_525_PER_SALE_) {
			$taxes = $db->executeS('
				SELECT * FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_tax
				WHERE id_sale = '.pSQL($gtotal['period_ref']));
		}
		else {
			$taxes = $db->executeS('
				SELECT * FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal_tax
				WHERE id_gtotal = '.pSQL($id_gtotal));
		}
		
		$data = array();
		foreach ($taxes as $id_tax => &$tax) {
			$data[$this->formatAmount($tax['tax_rate'])] = $this->formatAmount($tax['total_ti']);
		}
		$data = array(
			$data,
			$this->formatAmount($gtotal['total_ti']),
			$this->formatDate($gtotal['date']),
			$gtotal['period_ref'],
		);

		$sign = $this->sign(_KERAWEN_525_OP_GTOTAL_, $data);
		$db->update(_KERAWEN_525_PREFIX_.'gtotal', array(
			'description' => $sign['desc'],
			'signature' => $sign['sign'],
			'sign_short' => $sign['short'],
		), 'id_gtotal = '.pSQL($id_gtotal));
	}
	
	
	/* **************************************************************
	 * RECEIPTS
	*/
	
	public function getReceiptData($orders, $order_slips, $original, $prices, $isInvoice = false) {
		
		$this->getDbLink()->beginTransaction();
		$db = Db::getInstance();
		
		$datetime = time();
		$context = Context::getContext();
		$operator = $context->employee;
		
		$order_cond = count($orders) ? 'o.ps_order IN ('.implode(',', $orders).') AND o.ps_slip = 0' : 'FALSE';
		$return_cond = count($order_slips) ? 'o.ps_slip IN ('.implode(',', $order_slips).')' : 'FALSE';
		$res = $db->executeS('
			SELECT DISTINCT
				s.*,
				op.*,
				(s.total_ti - s.total_te) AS tax_amount,
				s.total_ti AS total_full
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = s.id_operation
            LEFT JOIN (
                SELECT sd.id_sale, SUM(ABS(sd.quantity)) AS nb_items
                FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sd
                GROUP BY sd.id_sale
            ) d ON d.id_sale = s.id_sale
			WHERE s.id_sale IN (
				SELECT o.id_sale FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order o
				WHERE '.$order_cond.' OR '.$return_cond.'
			)
				AND op.canceled IS NULL
				AND s.accounted = 1
				AND d.nb_items > 0');
		
		foreach($res as &$s) {
			$more = $db->getRow('
				SELECT
					ko.ps_order AS ps_order,
					ko.ps_slip AS ps_slip,
					o.reference AS reference,
					o.id_customer AS id_customer,
					IF (o.payment = "KerAwen", "", o.payment) as payment, 
					ok.invoice_note AS note
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s
				LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order ko ON ko.id_sale = s.id_sale AND ko.ps_slip = 0
				LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = ko.ps_order
				LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = ko.ps_order
				WHERE s.id_sale = '.pSQL($s['id_sale']));
			$s = array_merge($s, $more);
			
			// Check for duplicate
			if ($prices && (!$original || $s['nb_prints'] > 1)) {
				
				$duplicate = array(
					'id_sale' => $s['id_sale'],
					'nb_duplicate' => $s['nb_prints'],
					'id_operator' => $operator ? $operator->id : 0,
					'date' => date('Y-m-d H:i:s', $datetime),
				);
				$db->insert(_KERAWEN_525_PREFIX_.'duplicate', $duplicate, true);
				$id_duplicate = $db->Insert_ID();
				
				$sign = $this->sign(_KERAWEN_525_OP_DUPLICATE_, array(
					$id_duplicate,
					_KERAWEN_525_OP_DUPLICATE_,
					$duplicate['nb_duplicate'],
					$duplicate['id_operator'],
					$this->formatDate($duplicate['date']),
					$duplicate['id_sale'],
				));
				$db->update(_KERAWEN_525_PREFIX_.'duplicate', array(
					'description' => $sign['desc'],
					'signature' => $sign['sign'],
					'sign_short' => $sign['short'],
				), 'id_duplicate = '.pSQL($id_duplicate));
				
				$s['nb_prints']++;
				$db->update(_KERAWEN_525_PREFIX_.'sale', array(
					'nb_prints' => $s['nb_prints'],
				), 'id_sale = '.pSQL($s['id_sale']));

				$s['nb_duplicate'] = $duplicate['nb_duplicate'];
				// TODO Which signature on duplicata ???
				//$s['sign_short'] = $sign['short'];
			}
			
			$this->completeDetails($s, true);
			$this->completeCartRule($s);
			$this->completeMessages($s);
		}
		$this->getDbLink()->commit();
		
		return $res;
	}

	protected function completeMessages(&$s) {
		$s['messages'] = array();
		if ((int) $s['ps_order'] > 0 && (int) $s['ps_slip'] == 0) {
			$msgs = CustomerThread::getCustomerMessages($s['id_customer'], null, $s['ps_order']);
			foreach ($msgs as $msg) {
				$s['messages'][] = array(
					'priv' => (boolean) $msg['private'],
					'id_empl' => (int) $msg['id_employee'],
				    'text' => html_entity_decode($msg['message'], ENT_QUOTES),
					'id' => (int) $msg['id_customer_message'],
				);
			}
		}
	}

	protected function completeCartRule(&$s) {
		$db = Db::getInstance();
		$s['cart_rule'] = $db->executeS('
			SELECT 
				label AS item_name,
				discount_percent AS discount_percent, 
				-tax_incl AS unit_ti, 
				-tax_excl AS unit_te,
				-tax_incl AS total_ti, 
				-tax_excl AS total_te,
                ps_cart_rule AS id_cart_rule
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'discount
			WHERE id_sale = '.pSQL($s['id_sale'])
		);
	}
	
	protected function idTaxToId($id_tax) {
		$list = array('', 'A','B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S' , 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '1', '2', '3', '4', '5', '6', '7', '8', '9');
		return $list[(int) ($id_tax % count($list))];
	}
	
	protected function completeDetails(&$s, $isSale) {
		$db = Db::getInstance();
		
		$version = $db->getValue('
			SELECT op.KERAWEN_MODULE_VERSION as version
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = s.id_operation
			WHERE s.id_sale = '.pSQL($s['id_sale']));
		$cond = $isSale
			? 'sd.id_sale = '.pSQL($s['id_sale'])
			: 'sd.id_order = '.pSQL($s['id_order']);
		
		$sql = '
			SELECT
				sd.*,
				od.product_reference AS ref,
                od.product_ean13 AS ean13,
                od.product_upc AS upc,
				odk.note
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sd
			LEFT JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = sd.id_order_detail
			LEFT JOIN '._DB_PREFIX_.'order_detail_kerawen odk ON odk.id_order_detail = od.id_order_detail
			WHERE '.$cond;
		if (Tools::version_compare($version, '2.1', '<')) {
			$sql = '
				SELECT
					sd.*,
					sdd.*, 
				    od.product_reference AS ref,
                    od.product_ean13 AS ean13,
                    od.product_upc AS upc,
					odk.note
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sd
				LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail_discount sdd ON sdd.id_sale_detail = sd.id_sale_detail
				LEFT JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = sd.id_order_detail
				LEFT JOIN '._DB_PREFIX_.'order_detail_kerawen odk ON odk.id_order_detail = od.id_order_detail
				WHERE '.$cond;
		}
		$detail = $db->executeS($sql);
		
		$s['detail'] = $detail;
		
		$s['order'] = array();
		$s['slip'] = array();
		
		$s['carrier'] = array();
		$s['order_total_ti'] = 0;
		$s['order_total_te'] = 0;
		$s['slip_total_ti'] = 0;
		$s['slip_total_te'] = 0;
		$s['carrier_total_ti'] = 0;
		$s['carrier_total_te'] = 0;
		$s['ecotax_total_ti'] = 0;
		$s['ecotax_total_te'] = 0;
		
		$taxlist = $db->executeS('
			SELECT id_tax, ps_tax
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'tax'
		);
		$taxesArray = array();
		foreach($taxlist as $tax) {
			$taxesArray[(int) $tax['ps_tax']] = $tax['id_tax'];
		}
		
		// TODO VAT MARGIN ?
		$taxesDiffArray = array();
		$vat_margin_tax_symbol = '*';
		// TODO VAT MARGIN
		
		// Get customizations
		$customs = array();
		if ($s['ps_order']) {
			$order = new Order($s['ps_order']);
			$cf_cond = Tools::version_compare(_PS_VERSION_, '1.6.0.12', '>=')
				? 'AND cf.id_shop = '.pSQL($order->id_shop) : '';
			
			$buf = $db->executeS('
				SELECT od.*, c.*, cd.*, cf.*
				FROM '._DB_PREFIX_.'order_detail od
				JOIN '._DB_PREFIX_.'customization c
					ON c.id_product = od.product_id
					AND c.id_product_attribute = od.product_attribute_id
				JOIN '._DB_PREFIX_.'customized_data cd
					ON cd.id_customization = c.id_customization
				JOIN '._DB_PREFIX_.'customization_field_lang cf
					ON cf.id_customization_field = cd.index
					AND cf.id_lang = '.pSQL($order->id_lang).'
					'.$cf_cond.'
				WHERE od.id_order = '.pSQL($order->id).'
					AND c.id_cart = '.pSQL($order->id_cart));
			foreach($buf as $cd) {
				if (!isset($customs[$cd['id_order_detail']])) $customs[$cd['id_order_detail']] = array();
				$c = &$customs[$cd['id_order_detail']];
				$id_c = $cd['id_customization'];
				if (!isset($c[$id_c])) $c[$id_c] = array(
					'quantity' => $cd['quantity'],
					'data' => array(),
				);
				$c[$id_c]['data'][$cd['index']] = array(
					'name' => $cd['name'],
					'value' => $cd['value'],
				);
			}
		}
		
		foreach($detail as $k => $item) {
			if (!isset($taxesArray[(int) $item['id_tax']])) {
				$db->insert(_KERAWEN_525_PREFIX_.'tax', array('ps_tax' => $item['id_tax']) );
				$taxesArray[(int) $item['id_tax']] = $db->Insert_ID();
			}
			$item['id_tax'] = $this->idTaxToId($taxesArray[(int) $item['id_tax']]);
			
			$s['ecotax_total_ti'] += (float) $item['ecotax_ti'] * (int) $item['quantity'];
			$s['ecotax_total_te'] += (float) $item['ecotax_te'] * (int) $item['quantity'];
			
			$updatedItem = $item;
			
			if (isset($customs[$item['id_order_detail']])) {
				$updatedItem['customs'] = $customs[$item['id_order_detail']];
			}
			
			// Backward compatibility
			if (Tools::version_compare($version, '2.1', '<')) {
				$m = $updatedItem['measure'];
				$qty = $updatedItem['quantity'];
				
				if ($m) {
					$updatedItem['total_ti'] = $updatedItem['total_ti'] - $updatedItem['product_discount_ti'];
					$updatedItem['total_te'] = $updatedItem['total_te'] - $updatedItem['product_discount_te'];
				}
				
				$updatedItem['unit_ti'] = $item['product_unit_ti'];
				$updatedItem['unit_te'] = $item['product_unit_te'];
				$updatedItem['discount_ti'] = $updatedItem['product_discount_ti']/$qty;
				$updatedItem['discount_te'] = $updatedItem['product_discount_te']/$qty;
				
				if ($updatedItem['product_discount_type'] == "percent") {
					$updatedItem['unit_ti'] = $updatedItem['unit_ti'] + $updatedItem['discount_ti'];
					$updatedItem['unit_te'] = $updatedItem['unit_te'] + $updatedItem['discount_te'];
				}
				
				if ($m) {
					$updatedItem['unit_ti'] = $updatedItem['unit_ti']/$m;
					$updatedItem['unit_te'] = $updatedItem['unit_te']/$m;
					$updatedItem['discount_ti'] = $updatedItem['discount_ti']/$m;
					$updatedItem['discount_te'] = $updatedItem['discount_te']/$m;
				}
			}
				
			// TODO: VAT MARGIN
			if ((int) $item['margin_vat']) {
				//diff taxes
				if (empty($taxesDiffArray[$item['id_tax']])) {
					$taxesDiffArray[$item['id_tax']] = array(
						'total_te' => 0,
						'total_ti' => 0,
						'tax_amount' => 0,
					);
				}
				$taxesDiffArray[$item['id_tax']]['total_te'] += $updatedItem['total_te'];
				$taxesDiffArray[$item['id_tax']]['total_ti'] += $updatedItem['total_ti'];
				$taxesDiffArray[$item['id_tax']]['tax_amount'] += $updatedItem['total_ti'] - $updatedItem['total_te'];
				
				$updatedItem['total_te'] = $updatedItem['total_ti'];
				$updatedItem['unit_te'] = $updatedItem['unit_ti'];
				
				$updatedItem['id_tax'] = $vat_margin_tax_symbol;
			}
			// TODO: VAT MARGIN
			
			if (+$item['quantity'] > 0) {
				if (is_null($item['id_carrier'])) {
					$s['order'][] = $updatedItem;
					$s['order_total_ti'] += (float) $item['total_ti'];
					$s['order_total_te'] += (float) $item['total_te'];
				} else {
					$s['carrier'][] = $updatedItem;
					$s['carrier_total_ti'] += (float) $item['total_ti'];
					$s['carrier_total_te'] += (float) $item['total_te'];
				}
			} else {
				$s['slip'][] = $updatedItem;
				$s['slip_total_ti'] += (float) $item['total_ti'];
				$s['slip_total_te'] += (float) $item['total_te'];
			}
		}

		if ($isSale) {
			$s['taxes'] = $db->executeS('
				SELECT * FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_tax
				WHERE id_sale = '.pSQL($s['id_sale']).'
				ORDER BY id_tax ASC');
		} else {
			$s['taxes'] = $db->executeS('
				SELECT * FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order_tax
				WHERE id_order = '.pSQL($s['id_order']));
		}
		
		foreach($s['taxes'] as $k => $t) {
			//shoudn't be possible
			if (!isset($taxesArray[(int) $t['id_tax']])) {
				$db->insert(_KERAWEN_525_PREFIX_.'tax', array('ps_tax' => $t['id_tax']) );
				$taxesArray[(int) $t['id_tax']] = $db->Insert_ID();
			}
			
			$s['taxes'][$k]['id_tax'] = $this->idTaxToId($taxesArray[(int) $t['id_tax']]);
			
			//VAT margin reverse
			if (isset($taxesDiffArray[ $s['taxes'][$k]['id_tax'] ])) {
				$s['taxes'][$k]['total_te'] -= $taxesDiffArray[ $s['taxes'][$k]['id_tax'] ]['total_te'];
				$s['taxes'][$k]['total_ti'] -= $taxesDiffArray[ $s['taxes'][$k]['id_tax'] ]['total_ti'];
				$s['taxes'][$k]['tax_amount'] -= $taxesDiffArray[ $s['taxes'][$k]['id_tax'] ]['tax_amount'];	
				
				//Fully vat margin by rate
				if ($s['taxes'][$k]['total_te'] == 0 && $s['taxes'][$k]['total_ti'] == 0 && $s['taxes'][$k]['tax_amount'] == 0) {
					unset($s['taxes'][$k]);
				}
			}
		}
		
		$total_margin_vat_ti = 0;
		$total_margin_vat_te = 0;
		$total_margin_vat_tax = 0;
		
		//VAT margin total
		if (count($taxesDiffArray)) {
			foreach($taxesDiffArray as $row) {
				$total_margin_vat_ti += $row['total_ti'];
				$total_margin_vat_te += $row['total_te'];
				$total_margin_vat_tax += $row['tax_amount'];
			}
			$s['taxes'][] = array(
				'id' => '',
				'id_tax' => $vat_margin_tax_symbol,
				'tax_amount' => '-',
				'tax_name' => 'vat margin',
				'tax_rate' => '-',
				'total_te' => $total_margin_vat_ti,
				'total_ti' => $total_margin_vat_ti,
			);
			
			$s['tax_amount'] -= $total_margin_vat_tax;
			$s['total_te'] += $total_margin_vat_tax;
		}
		
		$s['payments'] = $db->executeS('
			SELECT
                op.*,
                op.payment_method AS mode,
                IFNULL(opk.deferred, op.date_add) AS date_add
			FROM '._DB_PREFIX_.'order_payment op
			LEFT JOIN '._DB_PREFIX_.'order_payment_kerawen opk ON opk.id_order_payment = op.id_order_payment
			WHERE op.order_reference = "' . $s['reference'] . '" AND "' . $s['reference'] . '" != ""
			ORDER BY op.id_order_payment ASC'
		);
		
		$s['total_paid'] = 0;
		if ($s['payments']) {
			foreach($s['payments'] as $k => $payment) {
				$s['total_paid'] += (float) $payment['amount'];
			}
		}

		$s['remain'] = (float) $s['total_full'] - (float) $s['total_paid'];
		unset($s['total_full']);

		if ($isSale) {
			$s['returns'] = array();
		} else {
			$s['returns'] = $db->executeS('
				SELECT -ksd2.total_ti AS amount
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail ksd
				LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail ksd2 ON ksd.id_sale = ksd2.id_sale AND ksd2.quantity < 0
				WHERE ksd2.total_ti IS NOT NULL AND ksd.id_order = ' .pSQL($s['id_order'])
			);
		}
	}
	
	
	/* **************************************************************
	 * INVOICES
	*/
	
	public function getInvoiceData($ps_order, $ps_slip, $check = true, $transaction = true) {
		$db = Db::getInstance();
		
		$i = $db->getRow('
			SELECT
				o.*,
				op.*,
				os.reference,
				o.date AS invoice_date,
				op.date AS receipt_date,
				os.id_cart,
				IF (os.payment = "KerAwen" ,"" ,os.payment ) as payment,
				(o.total_ti - o.total_te) AS tax_amount,
				ok.invoice_note AS note,
                ok.delivery_mode,
                ok.delivery_date
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order o
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s ON s.id_sale = o.id_sale
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = s.id_operation
			INNER JOIN '._DB_PREFIX_.'orders os ON os.id_order = o.ps_order
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = os.id_order
			WHERE op.canceled IS NULL
                AND s.accounted = 1
                AND o.ps_order = '.pSQL($ps_order).'
                AND o.ps_slip = '.pSQL($ps_slip)
		);
		
		// Old invoice, not certified
		if (!$i) return false;

		// Required to get left to paid
		// ??? Returns are counted twice in that case
		// $i['total_full'] = $db->getValue('SELECT SUM(total_ti) FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order WHERE id_sale = '.pSQL($i['id_sale']));
		$i['total_full'] = $i['total_ti'];
		
		$this->completeDetails($i, false);
		$this->completeCartRule($i);
		
		// Force PS invoice generation
		if ($check && !isset($i['number'])) {
			if ($ps_slip) {
				// Force initial invoice
				$this->getInvoiceData($ps_order, 0, true, $transaction);
			}
			$this->completeInvoice($i, $transaction);
			return $this->getInvoiceData($ps_order, $ps_slip, false, $transaction);
		}
		
		// Finalize
		$i['invoice'] = $db->getRow('
			SELECT * FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'invoice
			WHERE id_order = '.pSQL($i['id_order']));

		return $i;
	}
	
	protected function completeInvoice(&$i, $transaction) {
		if ($transaction) $this->getDbLink()->beginTransaction();
		$db = Db::getInstance();
	
		$data = $db->getRow('
			SELECT
				o.id_address_delivery,
				o.id_address_invoice,
				i.number AS invoice_number,
				o.id_shop,
				o.id_lang,
				o.id_customer
			FROM '._DB_PREFIX_.'orders o
			JOIN '._DB_PREFIX_.'order_invoice i ON i.id_order = o.id_order
			WHERE o.id_order = '.pSQL($i['ps_order']).'
			ORDER BY i.id_order_invoice DESC');

		$invoiceAddressPatternRules = json_decode(Configuration::get('PS_INVCE_INVOICE_ADDR_RULES'), true);
		$deliveryAddressPatternRules = json_decode(Configuration::get('PS_INVCE_DELIVERY_ADDR_RULES'), true);
		// Backward compatibility
		$invoiceAddressPatternRules = $invoiceAddressPatternRules ? $invoiceAddressPatternRules : array();
		$deliveryAddressPatternRules = $deliveryAddressPatternRules ? $deliveryAddressPatternRules : array();

		$formatted_invoice_address = '';
		$formatted_delivery_address = '';
	
		$invoice_address = new Address((int) $data['id_address_invoice']);
		if ($invoice_address) {
		    if (Configuration::get('KERAWEN_INVOICE_DISP_ONLY_COMP')) {
    		    if (isset($invoice_address->company) && $invoice_address->company != '' && $invoice_address->company != '-') {
    		        $invoice_address->firstname = null;
    		        $invoice_address->lastname = null;
    		    }
		    }
			$formatted_invoice_address = AddressFormat::generateAddress($invoice_address, $invoiceAddressPatternRules, '<br />', ' ');
		}
	
		if ($data['id_address_delivery'] != Configuration::get('KERAWEN_'))
		$delivery_address = new Address((int) $data['id_address_delivery']);
		if ($delivery_address) {
		    if (Configuration::get('KERAWEN_INVOICE_DISP_ONLY_COMP')) {
		        if (isset($delivery_address->company) && $delivery_address->company != '' && $delivery_address->company != '-') {
		            $delivery_address->firstname = null;
		            $delivery_address->lastname = null;
		        }
		    }
			$formatted_delivery_address = AddressFormat::generateAddress($delivery_address, $deliveryAddressPatternRules, '<br />', ' ');
		}
		
		if ($data['id_address_delivery'] == Configuration::get('KERAWEN_DEFAULT_ADDRESS')) {
// 			require_once(_KERAWEN_CLASS_DIR_.'/shop.php');
// 			$shops = selectShops($data['id_shop']);
// 			foreach($shops AS $shop) {
// 				$formatted_delivery_address = '';
// 				$formatted_delivery_address .= $shop['name'];
// 				if (!empty($shop['addr1'])) { $formatted_delivery_address .= '<br />' . $shop['addr1']; }
// 				if (!empty($shop['addr2'])) { $formatted_delivery_address .= '<br />' . $shop['addr2']; }
// 				if (!(empty($shop['postcode']) || empty($shop['city']))) { $formatted_delivery_address .= '<br />' . $shop['postcode'] . ' ' . $shop['city']; }
// 				if (!empty($shop['country'])) { $formatted_delivery_address .= '<br />' . $shop['country']; }
// 			}
			// Do not print adress in that case
			$formatted_delivery_address = '';
		}
		
		$format = '%1$s%2$06d';
		if (Configuration::get('PS_INVOICE_USE_YEAR')) {
			$format = Configuration::get('PS_INVOICE_YEAR_POS') ? '%1$s%3$s/%2$06d' : '%1$s%2$06d/%3$s';
		}
		$invoice_label = '';
		if ($i['ps_slip']) {
			$ps_slip_prefix = Configuration::get('PS_SLIP_PREFIX', (int)$data['id_lang'], null, (int)$data['id_shop']);
			$slip_prefix = $ps_slip_prefix ? $ps_slip_prefix : '#AV';
			$invoice_label = sprintf($format, $slip_prefix, $i['ps_slip'], date('Y'));
		}
		else {
			$invoice_label = sprintf($format, Configuration::get('PS_INVOICE_PREFIX', (int)$data['id_lang'], null, (int)$data['id_shop']), $data['invoice_number'], date('Y'));
		}
		
		$db->insert(_KERAWEN_525_PREFIX_.'invoice', array(
			'id_order' => $i['id_order'],
			'invoice_label' => pSQL($invoice_label),
			'firstname' => $invoice_address ? pSQL($invoice_address->firstname) :  null,
			'lastname' => $invoice_address ? pSQL($invoice_address->lastname) : null,
			'company' => $invoice_address ? pSQL($invoice_address->company) : null,
			'invoice_address' => pSQL($formatted_invoice_address, true),
			'delivery_address' => pSQL($formatted_delivery_address, true),
		), true);
		
		// Additional invoice info
		$info = array(
			'date' => date('Y-m-d H:i:s', time()),
			'number' => pSQL($invoice_label),
			'name' => $invoice_address ? pSQL($invoice_address->lastname) : '',
			'postcode' => $invoice_address ? pSQL($invoice_address->postcode) : '0000',
		);

		// Signature
		$data = array();
		foreach ($i['taxes'] as $id_tax => &$tax) {
			$data[$this->formatAmount($tax['tax_rate'])] = $this->formatAmount($tax['total_ti']);
		}
		$data = array(
			$data,
			$this->formatAmount($i['total_ti']),
			$this->formatDate($i['date']),
			$info['number'],
			$info['name'],
			$info['postcode'],
			'',
		);
		$sign = $this->sign(_KERAWEN_525_OP_INVOICE_, $data);

		$db->update(_KERAWEN_525_PREFIX_.'order', array_merge($info, array(
			'description' => $sign['desc'],
			'signature' => $sign['sign'],
			'sign_short' => $sign['short'],
		)), 'id_order = '.pSQL($i['id_order']));
		
		if ($transaction) $this->getDbLink()->commit();
	}

	
	/* **************************************************************
	 * TILL MANAGEMENT
	*/
	
	public function getTillState($id_till, $state = false) {
		$db = Db::getInstance();
		$op = $db->getRow('
			SELECT
				id_operation AS id_op, date, id_till, type AS state
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
			WHERE id_till = '.pSQL($id_till).'
			AND (type = "'._KERAWEN_525_OP_OPEN_.'" OR type = "'._KERAWEN_525_OP_CLOSE_.'")
			ORDER BY id_operation DESC');
		return (!$state || $op['state'] == $state) ? $op : false;
	}
	
	public function getTillContent($id_till, $state = false) {
		$data = false;
		$ref = $this->getTillState($id_till, $state);
		if ($ref) {
			$db = Db::getInstance();
			$content = array();
			$payments = array();
			
			$checks = $db->executeS('
				SELECT tc.*
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_check tc
				WHERE id_operation = '.pSQL($ref['id_op']));
			foreach ($checks as &$c) {
				$content[$c['id_mode']] = array(
					'count' => $c['count'], // Maybe null = not counted
					'amount' => (float)$c['checked'],
				);
			}
			
			$flows = $db->executeS('
				SELECT f.*
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow f
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = f.id_operation
				WHERE op.id_till = '.pSQL($ref['id_till']).'
                AND op.canceled IS NULL
				AND op.id_close IS NULL');
			
			foreach ($flows as &$f) {
				if (!isset($content[$f['id_mode']])) {
					$content[$f['id_mode']] = array(
						'count' => null,
						'amount' => 0.0,
					);
				}
				
				$sign = (float)$f['amount'] > 0 ? 1 : -1;
				$content[$f['id_mode']]['count'] += $sign*(int)$f['count'];
				$content[$f['id_mode']]['amount'] += (float)$f['amount'];
			}
			
			// Get not closed payments + remaining payments (cheques)
			// TODO Use payment configuration
			// ATM avoid cheques are counted twice
			unset($content['2']);
			
			$pay = $db->executeS('
				SELECT
					p.id_payment AS id_flow,
                    p.*,
					op.date,
                    op.id_close,
					(SELECT customer_name
						FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s
						WHERE s.id_operation = op.id_operation
						LIMIT 1) AS customer
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p
				JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = p.id_operation
				WHERE op.id_till = '.pSQL($ref['id_till']).'
                AND op.canceled IS NULL
				AND (op.id_close IS NULL AND p.id_mode != 2 OR p.id_mode = 2 AND p.id_out IS NULL)');
			foreach ($pay as &$p) {
				if (!isset($content[$p['id_mode']])) {
					$content[$p['id_mode']] = array(
						'count' => 0,
						'amount' => 0.0,
					);
				}
				if (!is_null($content[$p['id_mode']]['count'])) $content[$p['id_mode']]['count']++;
			 	$content[$p['id_mode']]['amount'] += (float)$p['amount'];
				
				if (is_null($p['id_out'])) {
					if (!isset($payments[$p['id_mode']])) $payments[$p['id_mode']] = array();
					$payments[$p['id_mode']][] = $p;
				}
			}
			
			$data = array(
				'ref' => $ref,
				'content' => $content,
				'payments' => $payments,
			);
		}
		return $data;
	}

	protected function logTillOperation($id_till, $type, $id_ref = null) {
		$datetime = time();
		$db = Db::getInstance();
		$context = Context::getContext();
		$id_shop = $context->shop ? $context->shop->id : null;
		$operator = $context->employee;
		$id_till = isset($context->kerawen) && isset($context->kerawen->id_cashdrawer) ? $context->kerawen->id_cashdrawer : 0;
		$till_name = $db->getValue('SELECT name FROM '._DB_PREFIX_.'cash_drawer_kerawen WHERE id_cash_drawer = '.pSQL($id_till));
		$module = Module::getInstanceByName('kerawen');
		
		$db->insert(_KERAWEN_525_PREFIX_.'operation', array(
			'date' => date('Y-m-d H:i:s', $datetime),
			'type' => pSQL($type),
			'id_ref' => $id_ref,
			'id_till' => $id_till,
			'till_name' => pSQL($till_name),
			'id_operator' => $operator ? $operator->id : null,
			'operator_name' => $operator ? pSQL($operator->firstname.' '.$operator->lastname) : null,
			'id_shop' => $id_shop,
			'KERAWEN_MODULE_VERSION' => $module->version,
		), true);
		$id_op= $db->Insert_ID();
		return $id_op;
	}
	
	public function openTill($id_till, $open_data) {
		$content = $this->getTillContent($id_till);
		if (!$content) {
			require_once(_KERAWEN_CLASS_DIR_.'/drawer.php');
			// Get content from last legacy closing...
			$content = getTillContent($id_till);
			// but do not reference legacy operation
			unset($content['ref']);
		}
		
		$this->getDbLink()->beginTransaction();
		$db = Db::getInstance();
		
		$id_close = isset($content['ref']['id_op']) ? $content['ref']['id_op'] : 0;
		
		$id_open = $this->logTillOperation($id_till, _KERAWEN_525_OP_OPEN_, $id_close);
		$payment_modes = json_decode(Configuration::get('KERAWEN_PAYMENTS'), true);
		
		$check = array();
		foreach ($open_data as $mode => $mode_data) {
			$expected = isset($content['content'][$mode]) ? (float)$content['content'][$mode]['amount'] : 0.0;
			$error = $mode_data->checked - $expected;
			$check[] = array(
				'id_operation' => $id_open,
				'id_mode' => $mode,
				'mode' => pSQL(isset($payment_modes[$mode]) ? $payment_modes[$mode]['label'] : $mode),
				'id_currency' => null, // TODO
				'currency' => null, // TODO
				'checked' => $mode_data->checked,
				'count' => null,
				'error' => $error,
			);
		}
		
		// TODO get cheques data from operator
		$mode = 2; // cheques
		$buf = false;
		if ($id_close) {
			$buf = $db->getRow('
				SELECT tc.*
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_check tc
				WHERE id_operation = '.pSQL($id_close).' AND id_mode = '.pSQL($mode));
		}
		else if ($content) {
			// Legacy closing: same as before
			$buf = array(
				'checked' => $content['content'][$mode]['amount'],
				'count' => $content['content'][$mode]['count'],
			);
		}
		$check[] = array(
			'id_operation' => $id_open,
			'id_mode' => $mode,
			'mode' => pSQL(isset($payment_modes[$mode]) ? $payment_modes[$mode]['label'] : $mode),
			'id_currency' => null, // TODO
			'currency' => null, // TODO
			'checked' => $buf ? $buf['checked'] : 0,
			'count' => $buf ? $buf['count'] : 0,
			'error' => 0.0,
		);
		
		$db->insert(_KERAWEN_525_PREFIX_.'till_check', $check, true);
		$this->getDbLink()->commit();
		return $id_open;
	}

	public function closeTill($id_till, $close_data) {
		$id_close = false;
		// Check if opened according to new mode
		$content = $this->getTillContent($id_till, _KERAWEN_525_OP_OPEN_);
		if ($content) {
			$this->getDbLink()->beginTransaction();
			$db = Db::getInstance();
			
			$payment_modes = json_decode(Configuration::get('KERAWEN_PAYMENTS'), true);
			$id_open = isset($content['ref']['id_op']) ? $content['ref']['id_op'] : null;
			
			// Prepare closing operation as FLOW at the moment
			$id_close = $this->logTillOperation($id_till, _KERAWEN_525_OP_FLOW_, $id_open);
			
			// Declare withdrawals
			foreach ($close_data as $mode => &$mode_data) {
				if ((isset($mode_data->withdrawn) && $mode_data->withdrawn)
					|| isset($mode_data->payments) && count($mode_data->payments)) {
					$this->flowTill($id_till, null, $mode,
						isset($mode_data->withdrawn) ? -$mode_data->withdrawn : 0,
						isset($mode_data->payments) ? $mode_data->payments : false,
						null, $id_close);
					
					if (isset($mode_data->amount) && isset($mode_data->withdrawn)) {
						$mode_data->amount -= $mode_data->withdrawn;
					}
				}
			}
			
			// Get remaining content
			$content = $this->getTillContent($id_till);
			
			// Set operation as CLOSING and mark pending operations as closed
			$db->execute('
				UPDATE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
				SET type = "'. _KERAWEN_525_OP_CLOSE_.'"
				WHERE id_operation = '.pSQL($id_close));
			$db->execute('
				UPDATE '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
				SET id_close = '.pSQL($id_close).'
				WHERE id_till = '.pSQL($id_till).'
					AND id_operation >= '.pSQL($content['ref']['id_op']).'
					AND id_close IS NULL');
			
			// Check content
			$check = array();
			foreach ($close_data as $mode => &$mode_data) {
				if (isset($mode_data->amount)) {
					// This mode has been re-counted by the operator
					$expected = isset($content['content'][$mode]) ? (float)$content['content'][$mode]['amount'] : 0.0;
					$error = $mode_data->amount - $expected;
					$remain = $mode_data->amount;
					
					$check[] = array(
						'id_operation' => $id_close,
						'id_mode' => $mode,
						'mode' => pSQL(isset($payment_modes[$mode]) ? $payment_modes[$mode]['label'] : $mode),
						'id_currency' => null, // TODO
						'currency' => null, // TODO
						'checked' => $remain,
						'count' => null,
						'error' => $error,
					);
				}
				else {
					// Count remaining payments
					$remain = 0;
					$count = 0;
					if (isset($content['payments'][$mode]) && $content['payments'][$mode]) {
						foreach($content['payments'][$mode] as &$p) {
							$remain += $p['amount'];
							$count++;
						}
					}
					
					// Summarize errors in withdrawals from last opening
					$error = 0;
					$withdrawals = $db->executeS('
						SELECT p.amount, p.corrected
						FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p
						JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op
							ON op.id_operation = p.id_out
						WHERE op.id_close = '.pSQL($id_close).'
							AND p.id_mode = '.pSQL($mode));
					
					foreach($withdrawals as $w) {
						if (isset($w['corrected'])) {
							$error += ($w['corrected'] - $w['amount']);
						}
					}
					
					$check[] = array(
						'id_operation' => $id_close,
						'id_mode' => $mode,
						'mode' => pSQL(isset($payment_modes[$mode]) ? $payment_modes[$mode]['label'] : $mode),
						'id_currency' => null, // TODO
						'currency' => null, // TODO
						'checked' => $remain,
						'count' => $count,
						'error' => $error,
					);
 				}
			}
			$db->insert(_KERAWEN_525_PREFIX_.'till_check', $check, true);
			
			$this->getDbLink()->commit();
		}
		return $id_close;
	}
	
	public function flowTill($id_till, $id_currency, $mode, $amount, $detail, $comments, $id_op = null) {
		// Check if opened/closed according to new mode
		$content = $this->getTillContent($id_till);
		if ($content) {
			$db = Db::getInstance();
			$payment_modes = json_decode(Configuration::get('KERAWEN_PAYMENTS'), true);
			
			$id_flow = $id_op;
			if (!$id_flow) $id_flow = $this->logTillOperation($id_till, _KERAWEN_525_OP_FLOW_);
			
			$count = null;
			if ($detail) {
				$count = 0;
				$amount = 0;
				$error = 0;
				
				foreach ($detail as $payment) {
					$count++;
					if (isset($payment->correct)) {
						$amount += $payment->correct;
						$error += $payment->correct - $payment->amount;
					}
					else {
						$amount += $payment->amount;
					}
					$db->update(_KERAWEN_525_PREFIX_.'payment', array(
						'id_out' => pSQL($id_flow),
						'corrected' => isset($payment->correct) ? $payment->correct : null,
					), 'id_payment = '.pSQL($payment->id), 0, true);
				}
				$amount = -$amount; // withdrawn
			}
			
			$db->insert(_KERAWEN_525_PREFIX_.'till_flow', array(
				'id_operation' => $id_flow,
				'id_currency' => $id_currency,
				'currency' => $this->getCurrencyInfo($id_currency)->name,
				'amount' => $amount,
				'id_mode' => $mode,
				'mode' => pSQL(isset($payment_modes[$mode]) ? $payment_modes[$mode]['label'] : $mode),
				'count' => $count,
				'comments' => pSQL($comments),
			), true);
			
			return array(
				'id_flow' => $id_flow,
			);
		}
		else {
			return false;
		}
	}
	
	function getOpeningData($id_op) {
		$db = Db::getInstance();
		$data = array();
		
		$data['open'] = $db->getRow('
			SELECT
				id_operation AS id_op, id_ref,
				id_till, date, operator_name AS empl
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
			WHERE id_operation = '.pSQL($id_op));
		if (!$data['open']) return false;
		
		$data['open']['modes'] = $db->executeS('
			SELECT id_mode AS mode, checked, count, error
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_check
			WHERE id_operation = '.pSQL($id_op));
			
		$id_till = $data['open']['id_till'];
		
		$data['till'] = $db->getRow('
			SELECT id_cash_drawer AS id_till, name
			FROM '._DB_PREFIX_.'cash_drawer_kerawen
			WHERE id_cash_drawer = '.pSQL($id_till));
		
		if ($data['open']['id_ref']) {
			$data['close'] = $db->getRow('
				SELECT id_operation AS id_op, date, operator_name AS empl
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
				WHERE id_operation = '.pSQL($data['open']['id_ref']));
		}

		if (isset($data['close']) && $data['close']) {
			$data['close']['modes'] = $db->executeS('
			SELECT id_mode AS mode, checked, count
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_check
			WHERE id_operation = '.pSQL($data['close']['id_op']));
		}
		else {
			// Get legacy closing data
			require_once(_KERAWEN_CLASS_DIR_.'/drawer.php');
			$op = lastOpenClose($id_till);
			if ($op && $op['oper'] == _KERAWEN_CDOP_CLOSE_) {
				$data['close'] = $db->getRow('
					SELECT
						co.id_cashdrawer_op AS id_op,
						co.date AS date,
						CONCAT(e.firstname, " ", e.lastname) AS empl
					FROM '._DB_PREFIX_.'cashdrawer_op_kerawen co
					LEFT JOIN '._DB_PREFIX_.'employee e ON e.id_employee = co.id_employee
					WHERE co.id_cashdrawer_op = '.pSQL($op['id_op']));
				if ($data['close']) {
					$data['close']['modes'] = $db->executeS('
						SELECT
							cc.id_payment_mode AS mode,
							cc.checked AS checked
						FROM '._DB_PREFIX_.'cashdrawer_close_kerawen cc
						WHERE cc.id_cashdrawer_op = '.pSQL($data['close']['id_op']));
					$data['close']['cheques'] = getRemaining($id_till, 2, $data['close']['date']);
				}
			}
		}
		
		return $data;
	}
	
	public function getClosingData($id_op, $id_till) {
		$db = Db::getInstance();
		$data = array();
		
		if ($id_op) {
			$data['close'] = $db->getRow('
				SELECT
					id_operation AS id_op, id_ref,
					id_till, date, operator_name AS empl
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
				WHERE id_operation = '.pSQL($id_op));
			if (!$data['close']) return false;
			
			$data['close']['modes'] = $db->executeS('
				SELECT id_mode AS mode, checked, count, error
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_check
				WHERE id_operation = '.pSQL($id_op));
			
			$id_till = $data['close']['id_till'];
		}
		elseif ($id_till) {
			$content = $this->getTillContent($id_till);
			// Check if till is opened according to new mode
			if (!$content) return false; 
			
			$modes = array();
			foreach($content['content'] as $mode => $mode_data)
				$modes[] = array(
					'mode' => $mode,
					'count' => $mode_data['count'],
					'checked' => $mode_data['amount'],
			);
			
			$data['state'] = array(
				'modes' => $modes,
			);
		}
		
		$data['till'] = $db->getRow('
			SELECT id_cash_drawer AS id_till, name
			FROM '._DB_PREFIX_.'cash_drawer_kerawen
			WHERE id_cash_drawer = '.pSQL($id_till));
		
		$data['open'] = $db->getRow('
			SELECT id_operation AS id_op, date, operator_name AS empl
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
			WHERE '.($id_op
				? 'id_operation = '.pSQL($data['close']['id_ref'])
				: 'id_till = '.pSQL($id_till).'
					AND type = "'.pSQL(_KERAWEN_525_OP_OPEN_).'"
					ORDER BY id_operation DESC'));
		
		if ($data['open']) {
			$data['open']['modes'] = $db->executeS('
			SELECT id_mode AS mode, checked, count
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_check
			WHERE id_operation = '.pSQL($data['open']['id_op']));
		}
		
		// Common filter for details (use op as alias)
		$op_filter = ($id_op
		    ? 'op.id_close = '.pSQL($id_op)
		    : 'op.id_till = '.pSQL($id_till).' AND op.id_close IS NULL')
            .' AND op.canceled IS NULL';
		
		$data['flows'] = $db->executeS('
			SELECT
				op.id_operation AS id_op,
				op.type AS oper,
				op.date,
				op.operator_name AS empl,
                op.id_operator AS id_empl,
				f.id_mode AS mode,
				f.amount,
				f.count
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow f
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = f.id_operation
			WHERE '.$op_filter);
		
		// WARNING: don't use UNION, it seems to imply DISTINCT !!!
		$data['flows'] = array_merge($data['flows'], $db->executeS('
			SELECT
				op.id_operation AS id_op,
				op.type AS oper,
				op.date,
				op.operator_name AS empl,
				p.id_mode AS mode,
				p.amount,
                p.remain
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p
			JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation op ON op.id_operation = p.id_operation
			WHERE '.$op_filter));
		
		require_once(dirname(__FILE__).'/KerawenLog.php');
		$log = new KerawenLog($op_filter);
		$data['more'] = array(
			'sales' => $log->sales(),
			'taxes' => $log->taxes(),
			//'prods' => $log->prods(), // TODO correct assignment products vs shipping/wrapping
		);
		
		return $data;
	}
	
	public function getFlowData($id_op) {
		$db = Db::getInstance();
		$data = $db->getRow('
			SELECT
				id_till AS id_till,
				till_name AS till,
				date AS date,
				operator_name AS empl
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation
			WHERE id_operation = '.pSQL($id_op));
		if (!$data) return false;
		
		$data['flows'] = $db->executeS('
			SELECT
				tf.*,
				tf.comments AS note
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'till_flow tf
			WHERE tf.id_operation = '.pSQL($id_op));
		return $data;
	}
	
	
	/* **************************************************************
	 * EXPORTING
	 */
	
	public function getArchivable() {
		$datetime = time();
		$this->closeGrandTotal($datetime);
		
		$db = Db::getInstance();
		$res = $db->executeS('
			SELECT
				id_gtotal AS id,
				period_type,
				period_ref
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal
			WHERE signature IS NOT NULL
			AND period_type != "'._KERAWEN_525_PER_SALE_.'"
			ORDER BY date DESC');
		
		$link = new Link();
		// TODO move to controller
		$url = $link->getAdminLink('AdminModules').'&configure=kerawen&archive&id=';
		foreach ($res as &$ar) {
			$ar['url'] = $url.$ar['id'];
		}
		return $res;
	}
	
	public function getArchive($id_gtotal) {
		$db = Db::getInstance();
		$archive = $db->getRow('
			SELECT *
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal
			WHERE id_gtotal = '.pSQL($id_gtotal));
		
		// Record event
		$this->logEvent(_KERAWEN_525_EVT_ARCH_PERIOD_, 'Archive', $archive['period_ref']);
		
		// Prepare empty directory
		$dir = tempnam(sys_get_temp_dir(), 'ARC');
		unlink($dir);
		mkdir($dir);
		$files = array();
		
		// Signature
		$taxes = $db->executeS('
			SELECT *
			FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal_tax
			WHERE id_gtotal = '.pSQL($id_gtotal));
		$data = array();
		foreach ($taxes as $id_tax => &$tax) {
			$data[$this->formatAmount($tax['tax_rate'])] = $this->formatAmount($tax['total_ti']);
		}
		$data = array(
			$data,
			$this->formatAmount($gtotal['total_ti']),
			$this->formatDate(date('Y-m-d H:i:s', time())),
			'',
			_KERAWEN_525_OP_ARCHIVE_,
		);
		$sign = $this->sign(_KERAWEN_525_OP_ARCHIVE_, $data);
		
		$filename = $dir.'/signature.txt';
		file_put_contents($filename,$sign['desc'].PHP_EOL.$sign['sign']);
		$files[] = $filename;
		
		// Content
		$outputs = array(
			'sales' => array(
				'sql' => '
					SELECT s.*, o.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation o ON o.id_operation = s.id_operation
					WHERE o.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY s.id_sale ASC',
			),
			'sale_details' => array(
				'sql' => '
					SELECT sd.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail sd
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s ON s.id_sale = sd.id_sale
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation o ON o.id_operation = s.id_operation
					WHERE o.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY sd.id_sale_detail ASC',
			),
			'sale_taxes' => array(
				'sql' => '
					SELECT st.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_tax st
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s ON s.id_sale = st.id_sale
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation o ON o.id_operation = s.id_operation
					WHERE o.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY st.id_sale, st.id_tax ASC',
			),
			'payments' => array(
				'sql' => '
					SELECT p.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation o ON o.id_operation = p.id_operation
					WHERE o.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY p.id_payment ASC',
			),
			'duplicates' => array(
				'sql' => '
					SELECT d.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'duplicate d
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s ON s.id_sale = d.id_sale
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation o ON o.id_operation = s.id_operation
					WHERE o.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY d.id_duplicate ASC',
			),
			'invoices' => array(
				'sql' => '
					SELECT i.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order i
					WHERE i.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY i.id_order ASC',
			),
			'invoice_details' => array(
				'sql' => '
					SELECT id.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale_detail id
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order i ON i.id_order = id.id_order
					WHERE i.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY id.id_sale_detail ASC',
			),
			'invoice_taxes' => array(
				'sql' => '
					SELECT it.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order_tax it
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order i ON i.id_order = it.id_order
					WHERE i.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY it.id_order, it.id_tax ASC',
			),
			'gtotal' => array(
				'sql' => '
					SELECT t.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal t
					WHERE t.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY t.id_gtotal ASC',
			),
			'gtotal_taxes' => array(
				'sql' => '
					SELECT tt.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal_tax tt
					JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'gtotal t ON t.id_gtotal = tt.id_gtotal
					WHERE t.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY t.id_gtotal ASC',
			),
			'jet' => array(
				'sql' => '
					SELECT e.*
					FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'event e
					WHERE e.date LIKE "'.$archive['period_ref'].'%"
					ORDER BY e.id_event ASC',
			),
		);
		
		foreach ($outputs as $name => $config) {
			$data = $db->executeS($config['sql']);
			$filename = $dir.'/'.$name.'.csv';
			$fd = fopen($filename, 'w');
			fputcsv($fd, array_keys(reset($data)));
			foreach ($data as $row) {
				fputcsv($fd, $row);
			}
			fclose($fd);
			$files[] = $filename;
		}
		
		// Zip
		$filename = $dir.'/'.$archive['period_ref'].'-archive.zip';
		$zip = new ZipArchive();
		$zip->open($filename, ZIPARCHIVE::CREATE);
		foreach ($files as $file) {
			$b = $zip->addFile($file, basename($file));
		}
		$zip->close();
		
		// Download
		header('Content-Type: application/zip');
		header('Cache-Control: no-store, no-cache');
		header('Content-Disposition: attachment; filename="'.basename($filename).'"');
		readfile($filename);
		
		// Clean-up
		foreach ($files as $file) {
			unlink($file);
		}
		unlink($filename);
		rmdir($dir);
		die();
	}
	
	
	/* **************************************************************
	 * RECOVERING DATA
	 */
	
	public function getLogs($types, $from, $to, $id_till, $id_operator, $id_shop, $accounted, $init_from, $init_to, $id_group, $from2, $to2, $id_order, $vars) {
		require_once(dirname(__FILE__).'/KerawenLog.php');
		$log = new KerawenLog(array(
			'from' => $from,
			'to' => $to,
			'id_till' => $id_till,
			'id_operator' => $id_operator,
			'id_shop' => $id_shop,
			'accounted' => $accounted,
			'init_from' => $init_from,
			'init_to' => $init_to,
		    'id_group' => $id_group,
		    'from2' => $from2,
		    'to2' => $to2,
		    'id_order' => $id_order,
		    'vars' => $vars,
		));
		//$types[] = "ops";
		$res = array();
		foreach ($types as $type) {
			$res[$type] = $log->$type();
		}
		return $res;
	}
	
	
	/* **************************************************************
	 * DATA MODEL UTILITIES
	*/

	protected function getOrderInfo($id_order) {
	    if ($id_order) {
	        $db = Db::getInstance();
	        
	        $info = $db->getRow('
    			SELECT
    				ok.round_type,
    				o.total_products,
    				o.total_products_wt,
    				o.total_shipping_tax_excl,
    				o.total_shipping_tax_incl,
    				o.total_discounts_tax_excl,
    				o.total_discounts_tax_incl
    			FROM '._DB_PREFIX_.'order_kerawen ok
    			JOIN '._DB_PREFIX_.'orders o ON o.id_order = ok.id_order
    			WHERE ok.id_order = '.pSQL($id_order));
    		
    		$info['cartrules'] = $db->executeS('
    			SELECT
    				ocr.id_order AS ps_order,
    				ocr.id_cart_rule AS ps_cart_rule,
    				ocr.name AS label,
    				ocr.value AS tax_incl,
    				ocr.value_tax_excl AS tax_excl,
    				cr.reduction_percent AS discount_percent,
    				cr.free_shipping AS free_shipping,
    				cr.gift_product AS gift_prod,
    				cr.gift_product_attribute AS gift_attr
    			FROM '._DB_PREFIX_.'order_cart_rule ocr
    			LEFT JOIN '._DB_PREFIX_.'cart_rule cr ON cr.id_cart_rule = ocr.id_cart_rule
    			WHERE ocr.id_order = '.pSQL($id_order));
	    }
	    else {
	        // Virtual order for returns
	        $info = array(
	            'round_type' => 0,
	            'total_products' => 0,
	            'total_products_wt' => 0,
	            'total_shipping_tax_excl' => 0,
	            'total_shipping_tax_incl' => 0,
	            'total_discounts_tax_excl' => 0,
	            'total_discounts_tax_incl' => 0,
	            'cartrules' => array(),
	        );
	    }
		
		// Manage cart rules
		$total_discounts_tax_excl = 0;
		$total_discounts_tax_incl = 0;
		$total_gifts_tax_excl = 0;
		$total_gifts_tax_incl = 0;
		
		$info['free_shipping'] = 0;
		$info['gifts'] = array();

		foreach($info['cartrules'] as $i => &$cr) {
		    if ($cr['free_shipping']) {
		        $info['free_shipping'] = 1;
		        $cr['tax_excl'] -= $info['total_shipping_tax_excl'];
		        $cr['tax_incl'] -= $info['total_shipping_tax_incl'];
		    }
		    
		    if ($cr['gift_prod']) {
		        // Get the gift price
		        $gift = $db->getRow('
        			SELECT
        				od.unit_price_tax_excl AS tax_excl,
        				od.unit_price_tax_incl AS tax_incl
        			FROM '._DB_PREFIX_.'order_detail od
        			WHERE od.id_order = '.pSQL($id_order).'
                        AND od.product_id = '.pSQL($cr['gift_prod']).'
                        AND od.product_attribute_id = '.pSQL($cr['gift_attr']));
		        if ($gift) {
    		        $info['gifts'][$cr['gift_prod'].'-'.$cr['gift_attr']] = array(
    		            'discount_te' => $gift['tax_excl'],
    		            'discount_ti' => $gift['tax_incl'],
    		        );
    		        $total_gifts_tax_excl += $gift['tax_excl'];
    		        $total_gifts_tax_incl += $gift['tax_incl'];
    		        
    		        $cr['tax_excl'] -= $gift['tax_excl'];
    		        $cr['tax_incl'] -= $gift['tax_incl'];
		        }
		    }
	        unset($cr['gift_prod']);
			unset($cr['gift_attr']);
		
			$total_discounts_tax_excl += $cr['tax_excl'];
			$total_discounts_tax_incl += $cr['tax_incl'];
		}
    	
		$info['total_discounts_tax_excl'] = $total_discounts_tax_excl;
		$info['total_discounts_tax_incl'] = $total_discounts_tax_incl;
		$info['total_products'] -= $total_gifts_tax_excl;
		$info['total_products_wt'] -= $total_gifts_tax_incl;
	
		if ($info['total_products'] != 0.0) {
			$info['global_discount_te'] = $info['total_discounts_tax_excl']/$info['total_products'];
			$info['global_discount_ti'] = $info['total_discounts_tax_incl']/$info['total_products_wt'];
		}
		else {
			$info['global_discount_te'] = $info['global_discount_ti'] = 0;
		}

		return $info;
	}
	
	protected function getOrderTaxAddress($order) {
		$id_address = $order->{Configuration::get('PS_TAX_ADDRESS_TYPE')};
		if (!$id_address) $id_address = $order->id_address_delivery;
		if (!$id_address) $id_address = Configuration::get('KERAWEN_DEFAULT_ADDRESS');
		return $id_address;
	}
	
	protected function getOrderDetails($order, $info, $cancel) {
	    require_once(_KERAWEN_CLASS_DIR_.'/data.php');
	    
	    $details = array();
		if (!$order->id) return $details;
		
		$db = Db::getInstance();
		$id_order = $order->id;
		$id_address = $this->getOrderTaxAddress($order);
		
		$tax_col = Tools::version_compare(_PS_VERSION_, '1.6.0.10', '>=') ? 'od.id_tax_rules_group' : 'odk.id_tax_rules_group';
		require_once(_KERAWEN_CLASS_DIR_.'/KerawenTax.php');
		$id_vat_margin = KerawenTax::GetVatMarginRule();
		
		$details = array_merge($details, $db->executeS('
			SELECT
				od.id_order_detail AS id_order_detail,
				od.product_id AS id_prod,
				od.product_attribute_id AS id_attr,
				pl.name AS item_name,
				(SELECT GROUP_CONCAT(al.name ORDER BY a.id_attribute_group SEPARATOR ",")
					FROM '._DB_PREFIX_.'product_attribute_combination pac
					LEFT JOIN '._DB_PREFIX_.'attribute a ON a.id_attribute = pac.id_attribute
					LEFT JOIN '._DB_PREFIX_.'attribute_lang al ON al.id_attribute = a.id_attribute
					WHERE pac.id_product_attribute = od.product_attribute_id
					AND pac.id_product_attribute > 0
					AND al.id_lang = o.id_lang
				) AS version_name,
				pk.packed AS packed, 
				od.unit_price_tax_excl AS unit_te,
				od.unit_price_tax_incl AS unit_ti,
				'.$tax_col.' AS id_tax,
				IF('.$tax_col.' = '.$id_vat_margin.', 1, odk.margin_vat) AS margin_vat,
                odk.rest_voucher AS rest_voucher,
				'.pSQL($id_address).' AS id_address,
				od.ecotax AS ecotax_te,
				odk.measure AS measure,
				odk.unit AS measure_unit,
				odk.precision AS measure_precision,
				'.($cancel ? '-' : '').'(od.product_quantity - od.product_quantity_return - od.product_quantity_refunded) AS quantity,
				od.reduction_amount_tax_excl AS discount_te,
				od.reduction_amount_tax_incl AS discount_ti,
				od.reduction_percent AS discount_percent,
                od.purchase_supplier_price AS purchase_te,
				(SELECT SUM(s.price_te*sm.physical_quantity)/SUM(sm.physical_quantity)
					FROM '._DB_PREFIX_.'stock s
					JOIN '._DB_PREFIX_.'stock_mvt sm ON sm.id_stock = s.id_stock
					WHERE s.id_product = od.product_id
						AND s.id_product_attribute = od.product_attribute_id
						AND sm.id_order = od.id_order
				) AS stock_te,
				pwk.is_gift_card AS gift_card,
                pwk.is_account_credit AS account_credit,
                pwk.product_type AS product_type
			FROM '._DB_PREFIX_.'order_detail od
			LEFT JOIN '._DB_PREFIX_.'order_detail_kerawen odk ON odk.id_order_detail = od.id_order_detail
			LEFT JOIN '._DB_PREFIX_.'order_detail_tax odt ON odt.id_order_detail = od.id_order_detail
			LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = od.id_order
			LEFT JOIN '._DB_PREFIX_.'product_lang pl ON pl.id_product = od.product_id AND pl.id_shop = o.id_shop AND pl.id_lang = o.id_lang
            LEFT JOIN '._DB_PREFIX_.'product_wm_kerawen pwk ON pwk.id_product = od.product_id
			LEFT JOIN (
				SELECT id_product_pack, COUNT(*) AS packed
				FROM '._DB_PREFIX_.'pack GROUP BY id_product_pack
				) pk ON pk.id_product_pack = od.product_id
			WHERE od.id_order = '.pSQL($id_order)));
		
		// Take into account previous refunding
		// TODO from 525 records instead of PS?
		if (Tools::version_compare(_PS_VERSION_, '1.6.0.11', '>=')) {
			$shipping_refund = $db->getRow('
				SELECT
					SUM(os.total_shipping_tax_excl) AS total_shipping_tax_excl,
					SUM(os.total_shipping_tax_incl) AS total_shipping_tax_incl
				FROM '._DB_PREFIX_.'order_slip os
				WHERE os.id_order = '.pSQL($id_order));
		}
		else {
			$shipping_refund = $db->getRow('
				SELECT
					SUM(os.shipping_cost_amount/(1 + o.carrier_tax_rate/100)) AS total_shipping_tax_excl,
					SUM(os.shipping_cost_amount) AS total_shipping_tax_incl
				FROM '._DB_PREFIX_.'order_slip os
				JOIN '._DB_PREFIX_.'orders o ON o.id_order = os.id_order
				WHERE os.id_order = '.pSQL($id_order));
		}
		$order->total_shipping_tax_excl -= $shipping_refund['total_shipping_tax_excl'];
		$order->total_shipping_tax_incl -= $shipping_refund['total_shipping_tax_incl'];
		
		if ($order->id_carrier && $order->id_carrier != getInShopCarrier()) {
			$carrier = $this->getCarrierInfo($order->id_carrier, $order->id_shop);
			$details[] = array(
				'item_name' => $this->label_shipping,
				'version_name' => $carrier['name'],
				'id_carrier' => $carrier['id'],
				'unit_te' => $info['free_shipping'] ? 0 : $order->total_shipping_tax_excl,
				'unit_ti' => $info['free_shipping'] ? 0 : $order->total_shipping_tax_incl,
				'id_tax' => $carrier['id_tax'],
				'id_address' => $id_address,
				'quantity' => $cancel ? -1 : +1,
				'discount_te' => $info['free_shipping'] ? $order->total_shipping_tax_excl : 0,
				'discount_ti' => $info['free_shipping'] ? $order->total_shipping_tax_incl : 0,
				'discount_percent' =>  0,
				'purchase_te' => $order->total_shipping_tax_excl,
			);
		}
		
		if ($order->total_wrapping_tax_incl != 0) {
			$details[] = array(
				'item_name' => $this->label_wrapping,
				'wrapping' => 1,
				'unit_te' => $order->total_wrapping_tax_excl,
				'unit_ti' => $order->total_wrapping_tax_incl,
				'id_tax' => Configuration::get('PS_GIFT_WRAPPING_TAX_RULES_GROUP'),
				'id_address' => $id_address,
				'quantity' => $cancel ? -1 : +1,
				'discount_te' => 0,
				'discount_ti' => 0,
				'discount_percent' => 0,
				'purchase_te' => $order->total_wrapping_tax_excl,
			);
		}
		
		foreach ($details as &$detail) {
			if (isset($detail['id_prod'])) {
				// Apply global discount to products
				$detail['discount_rate_te'] = $info['global_discount_te'];
				$detail['discount_rate_ti'] = $info['global_discount_ti'];
				
				// Apply gifts
				$buf = $detail['id_prod'].'-'.$detail['id_attr'];
				if (isset($info['gifts'][$buf])) {
					$diff = $info['gifts'][$buf]['discount_ti'] / $detail['quantity'];
					$detail['discount_ti'] += $diff;
					$detail['unit_ti'] -= $diff;
				}
				unset($detail['id_prod']);
				unset($detail['id_attr']);
			}
		}

		return $details;
	}
	
	protected function getProductInfo($id_order_detail) {
		$id_shop = Context::getContext()->shop->id;
		$id_lang = Context::getContext()->language->id;
		$db = Db::getInstance();

		return $db->getRow('
			SELECT
				pl.name AS item_name,
				(SELECT GROUP_CONCAT(al.name ORDER BY a.id_attribute_group SEPARATOR ",")
					FROM '._DB_PREFIX_.'product_attribute_combination pac
					LEFT JOIN '._DB_PREFIX_.'attribute a ON a.id_attribute = pac.id_attribute
					LEFT JOIN '._DB_PREFIX_.'attribute_lang al ON al.id_attribute = a.id_attribute
					WHERE pac.id_product_attribute = od.product_attribute_id
					AND pac.id_product_attribute > 0
					AND al.id_lang = IFNULL(o.id_lang, '.pSQL($id_lang).')
				) AS version_name,
				pk.packed AS packed
			FROM '._DB_PREFIX_.'order_detail od
			LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = od.id_order
			LEFT JOIN '._DB_PREFIX_.'product_lang pl
				ON pl.id_product = od.product_id
				AND pl.id_shop = IFNULL(o.id_shop, '.pSQL($id_shop).')
				AND pl.id_lang = IFNULL(o.id_lang, '.pSQL($id_lang).')
			LEFT JOIN (
				SELECT id_product_pack, COUNT(*) AS packed
				FROM '._DB_PREFIX_.'pack GROUP BY id_product_pack
				) pk ON pk.id_product_pack = od.product_id
			WHERE od.id_order_detail = '.pSQL($id_order_detail));
	}
	
	protected function getPackInfo($id_order_detail) {
		$db = Db::getInstance();
		return $db->executeS('
			SELECT
				pp.id_product_item AS id_prod,
				pp.id_product_attribute_item AS id_attr,
				pp.quantity AS quantity,
				pl.name AS item_name,
				(SELECT GROUP_CONCAT(al.name ORDER BY a.id_attribute_group SEPARATOR ",")
					FROM '._DB_PREFIX_.'product_attribute_combination pac
					LEFT JOIN '._DB_PREFIX_.'attribute a ON a.id_attribute = pac.id_attribute
					LEFT JOIN '._DB_PREFIX_.'attribute_lang al ON al.id_attribute = a.id_attribute
					WHERE pac.id_product_attribute = pa.id_product_attribute
					AND pac.id_product_attribute > 0
					AND al.id_lang = o.id_lang
				) AS version_name,
				p.price + IFNULL(pa.price, 0) AS base_te,
				p.id_tax_rules_group AS id_tax,
				(CASE
					WHEN pa.wholesale_price != 0 THEN pa.wholesale_price
					ELSE p.wholesale_price
				END) AS purchase_te
			FROM '._DB_PREFIX_.'order_detail od
			LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = od.id_order
			LEFT JOIN '._DB_PREFIX_.'pack pp ON pp.id_product_pack = od.product_id
			LEFT JOIN '._DB_PREFIX_.'product_shop p
				ON p.id_product = pp.id_product_item AND p.id_shop = o.id_shop
			LEFT JOIN '._DB_PREFIX_.'product_attribute_shop pa
				ON pa.id_product_attribute = pp.id_product_attribute_item AND pa.id_shop = o.id_shop
			LEFT JOIN '._DB_PREFIX_.'product_lang pl
				ON pl.id_product = p.id_product AND pl.id_shop = o.id_shop AND pl.id_lang = o.id_lang
			WHERE od.id_order_detail = '.pSQL($id_order_detail));
	}

	protected function getCarrierInfo($id_carrier, $id_shop) {
	    static $taxes = array();
	    $id_carrier = (int)$id_carrier;
	    $key = $id_carrier.'_'.$id_shop;
	    if (!isset($taxes[$key])) {
	        $ps_id_tax_rules_group = (Tools::version_compare(_PS_VERSION_, '8', '>=')) ? 'cts.id_tax_rules_group' : 'IFNULL(cts.id_tax_rules_group, c.id_tax_rules_group)';
	        $taxes[$key] = Db::getInstance()->getRow('
				SELECT
					c.id_carrier AS id,
					c.name AS name,
					IFNULL('.$ps_id_tax_rules_group.', 0) AS id_tax
				FROM '._DB_PREFIX_.'carrier c
				LEFT JOIN '._DB_PREFIX_.'carrier_tax_rules_group_shop cts
                    ON cts.id_carrier = c.id_carrier
                    AND cts.id_shop= '.pSQL($id_shop).'
				WHERE c.id_carrier = '.pSQL($id_carrier));
	    }
	    return $taxes[$key];
	}
	
	protected function getTaxInfo($id_taxrule, $id_address) {
		static $taxes = array();
		$id_taxrule = (int)$id_taxrule;
		$key = $id_taxrule.'_'.$id_address;
		if (!isset($taxes[$key])) {
		    if ($id_taxrule) {
    			$taxrule = new TaxRulesGroup($id_taxrule);
    			$tax_manager = TaxManagerFactory::getManager(new Address($id_address), $id_taxrule);
    			$taxes[$key] = array(
    			    'id_tax' => $id_taxrule,
    				'name' => $taxrule->name,
    				'rate' => $tax_manager->getTaxCalculator()->getTotalRate(),
    			);
		    }
		    else {
		        $taxes[$key] = array(
		            'id_tax' => $id_taxrule,
		            'name' => '-',
		            'rate' => 0,
		        );
		    }
		}
		return $taxes[$key];
	}
	
	protected function getCurrencyInfo($id_currency) {
		static $currs = array();
		$key = $id_currency;
		if (!isset($currs[$key])) {
			$currs[$key] = new Currency($id_currency);
		}
		return $currs[$key];
	}
	
	protected function getOrderSlip($id_slip) {
	    $slip = new OrderSlip($id_slip);
	    $slip->details = OrderSlip::getOrdersSlipDetail($id_slip);
	    return $slip;
	}
	
	protected function getOrderPayment($id_payment) {
		$payment = new OrderPayment($id_payment);
		if (!$payment->order_reference) {
			$payment->order_reference = Db::getInstance()->getValue('
				SELECT reference
				FROM '._DB_PREFIX_.'order_payment_kerawen
				WHERE id_order_payment = '.pSQL($id_payment));
		}
		return $payment;
	}

	
	/* **************************************************************
	 * SIGNING
	*/
	
	protected function formatAmount($amount) {
		return round($amount*100);
	}
	
	protected function formatDate($date) {
		return date('YmdHis', strtotime($date));
	}

	protected function sign($type, $data) {
	    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
	    $db = Db::getInstance();
		
		$prev = $db->getValue('
			SELECT sign FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'signature
			WHERE type = "'.pSQL($type).'"');
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, _KERAWEN_525_SIGN_URL_);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode(array(
		    'id' => getLicencedUrl(),
			'lic' => Configuration::get('KERAWEN_LICENCE_KEY'),
			'type' => $type,
			'data' => $data,
			'prev' => $prev,
		)));
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/plain'));
		$result = curl_exec($ch);
		curl_close($ch);
		$result = json_decode($result, true);
		
		$db->execute('
			INSERT INTO '._DB_PREFIX_._KERAWEN_525_PREFIX_.'signature (type, sign, short)
			VALUES (
				"'.pSQL($type).'",
				"'.pSQL($result['sign']).'",
				"'.pSQL($result['short']).'")
			ON DUPLICATE KEY UPDATE
				sign = VALUES(sign),
				short = VALUES(short)');
		
		if ($result['error']) {
			$this->logEvent(
				_KERAWEN_525_EVT_SIGN_DEFAULT_,
				'Signature default',
				$result['msg'].' for '.$type.': '.$result['desc'],
				false,
				false,
				$result['sign']); // Do not try to sign again if failed...
		}

		return $result;
	}
	
	
	/* **************************************************************
	 * DATABASE TRANSACTIONS
	*/
	
	protected function getDbLink() {
		static $db_link = null;
		if (!$db_link) {
			$db = Db::getInstance();
			$obj = new ReflectionObject($db);
			$prop = $obj->getProperty('link');
			$prop->setAccessible(true);
			$db_link = $prop->getValue($db);
		}
		return $db_link;
	}
}

set_exception_handler('Kerawen525::OnException');
register_shutdown_function('Kerawen525::OnShutdown');
