<?php
/**
 * 2014 KerAwen
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@kerawen.com so we can send you a copy immediately.
 *
 * @author    KerAwen <contact@kerawen.com>
 * @copyright 2014 KerAwen
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
 */

function getForLanguages($value /*, $module, $specific*/)
{
	static $id_langs = false;
	if (!$id_langs)
	{
		$buf = Db::getInstance()->executeS('SELECT id_lang FROM '._DB_PREFIX_.'lang');
		$id_langs = array();
		foreach ($buf as $lang) $id_langs[] = $lang['id_lang'];
	}
	return array_fill_keys($id_langs, $value);

	// The following doesn't work because of cache in Translate
// 	static $languages = false;
// 	if (!$languages)
// 	{
// 		$languages = array();
// 		$buf = Language::getLanguages(true);
// 		foreach ($buf as $l)
// 			$languages[] = new Language($l['id_lang']);
// 	}

// 	$context = Context::getContext();
// 	$current = $context->language;

// 	$translations = array();
// 	foreach ($languages as $lang)
// 	{
// 		$context->language = $lang;
// 		$translations[$lang->id] = $module->l($value, $specific);
// 	}

// 	$context->language = $current;
// 	return $translations;
}

function indexArray($array, $prop)
{
	$res = array();
	foreach($array as $item)
		$res[$item[$prop]] = $item;
	return $res;
}

function setExtendedContext($name, $value)
{
	$context = Context::getContext();
	if (!isset($context->kerawen)) $context->kerawen = new stdClass();
	$context->kerawen->$name =  $value;
}

function getExtendedContext($name, $default)
{
	$context = Context::getContext();
	return isset($context->kerawen->$name) ? $context->kerawen->$name : $default;
}

function backupShopContext()
{
	return array(
		'context' => Shop::getContext(),
		'id_shop' => Shop::getContextShopID(),
		'id_group' => Shop::getContextShopGroupID(),
	);
}

function restoreShopContext($backup)
{
	Shop::setContext(
	    $backup['context'],
        $backup['context'] == Shop::CONTEXT_ALL ? null
            : ($backup['context'] == Shop::CONTEXT_GROUP ? $backup['id_group'] : $backup['id_shop'])
	    );
}


function sharedCats() {

	$id_lang = Context::getContext()->language->id;
	
	$tree = array();
	$buf = Db::getInstance()->executeS('
			SELECT
				c.id_category AS id_cat,
				c.level_depth AS depth,
				c.id_parent AS id_parent,
				cl.id_shop AS id_shop,
				cl.name AS name
			FROM '._DB_PREFIX_.'category c
			JOIN '._DB_PREFIX_.'category_lang cl ON cl.id_category = c.id_category AND cl.id_lang = ' . (int) $id_lang);
	
	foreach($buf as $cat) {
		if (!isset($tree[$cat['id_shop']])) {
			$tree[$cat['id_shop']] = array();
		}
		
		$tree[$cat['id_shop']][$cat['id_cat']] = array(
				'name' => $cat['name'],
				'id_parent' => $cat['id_parent'],
				'depth' => $cat['depth']
		);
	}
	
	return $tree;
	
}


function getKerawenLink($controller, $params) {

    if (isset($params['id_shop'])) {
        $id_shop = (int) $params['id_shop'];
        unset($params['id_shop']);
    } else {
        $id_shop = (int) Configuration::get('PS_SHOP_DEFAULT');
    }

    $link = new Link;
    return $link->getModuleLink('kerawen', $controller, $params, null, null, $id_shop);
}


$shop_stack = array();

function pushShopContext($id_shop = false, $id_group = false) {
    global $shop_stack;
    $context = Context::getContext();
    
    $shop_stack[] = array(
        'shop' => $context->shop,
        'context' => Shop::getContext(),
        'id_shop' => Shop::getContextShopID(),
        'id_group' => Shop::getContextShopGroupID(),
    );
    
    if ($id_shop) {
        $context->shop = new Shop($id_shop);
        Shop::setContext(Shop::CONTEXT_SHOP, $id_shop);
    }
    else if ($id_group) {
        Shop::setContext(Shop::CONTEXT_GROUP, $id_group);
    }
    else {
        Shop::setContext(Shop::CONTEXT_ALL);
    }
}

function popShopContext() {
    global $shop_stack;
    
    if (count($shop_stack)) {
        $restore = array_pop($shop_stack);
        
        Shop::setContext($restore['context'],
            $restore['context'] == Shop::CONTEXT_SHOP ? $restore['id_shop']
            : ($restore['context'] == Shop::CONTEXT_GROUP ? $restore['id_group']
                : null));
        
        $context = Context::getContext();
        $context->shop = $restore['shop'];
    }
    else {
        // Exception ?
    }
}

function getLicencedUrl() {
    $id_shop_default = Configuration::get('PS_SHOP_DEFAULT');
    return Db::getInstance()->getValue('
		SELECT CONCAT(domain, physical_uri, virtual_uri)
		FROM '._DB_PREFIX_.'shop_url
		WHERE id_shop = '. (int) $id_shop_default .' AND main = 1 AND active = 1');
}

function dataListToIntList($list) {
    if (is_array($list)) { $list = implode(',', $list); }
    return implode(',', array_map('intval', explode(',', $list)));
}


function getKerawenManufacturers() {

    static $_manufacturers = null;
    
    if ($_manufacturers === null) {
    
        $_manufacturers = array();
        $mans =  Manufacturer::getManufacturers(false, 0, false);
        foreach ($mans as $man)
        {
            $_manufacturers[(int)$man['id_manufacturer']] = array(
                'id' => (int)$man['id_manufacturer'],
                'name' => $man['name'],
            );
        }
    
    }
    
    return $_manufacturers;
    
}


function getKerawenObjectToArray($data)
{
    $result = [];
    foreach ($data as $key => $value)
    {
        $result[$key] = (is_array($value) || is_object($value)) ? getKerawenObjectToArray($value) : $value;
    }
    return $result;
}


function getKerawenThreadStats() {
    static $_threadStats = null;
    
    if ($_threadStats === null) {
        $_threadStats = Db::getInstance()->executeS('
    		SELECT
                id_thread_state AS id,
                thread_state_label As name,
                thread_state_color AS color,
                thread_state_position AS position
    		FROM '._DB_PREFIX_.'customer_thread_state_kerawen
            ORDER BY thread_state_position ASC, id_thread_state ASC
        ');

    }
    
    return $_threadStats;
}


function getKerawenAttributes() {
    
    $db = Db::getInstance();
    $id_lang = Context::getContext()->language->id;
    $kasa = (String) Configuration::get('KERAWEN_ADV_SEARCH_ATTRIBUTES');
    
    $ags = $db->executeS('
        SELECT
            agl.id_attribute_group AS id,
            agl.name,
            ag.group_type
        FROM '._DB_PREFIX_.'attribute_group_lang agl
        LEFT JOIN '._DB_PREFIX_.'attribute_group ag ON ag.id_attribute_group = agl.id_attribute_group
        WHERE agl.id_lang = ' . (int) $id_lang . '
        ORDER BY ag.position ASC
    ');
    
    foreach($ags as &$ag) {
        $ag['selected'] = (strpos($kasa, ',' . $ag['id'] . ',') === false);
        $as = $db->executeS('
            SELECT
                a.id_attribute AS id,
                a.color,
                al.name
            FROM '._DB_PREFIX_.'attribute a
            LEFT JOIN '._DB_PREFIX_.'attribute_lang al ON a.id_attribute = al.id_attribute AND al.id_lang = ' . (int) $id_lang . '
            WHERE id_attribute_group = ' . (int) $ag['id'] . '
            ORDER BY a.position ASC
        ');
        
        $ag['vals'] = $as;
    }
    
    return $ags;
}


function getKerawenFeatures() {

    $id_lang = Context::getContext()->language->id;
    $kasf = (String) Configuration::get('KERAWEN_ADV_SEARCH_FEATURES');
    
    $fs = array();
    $features = Feature::getFeatures($id_lang, false);
    if ($features) {
        foreach ($features as $feature) {
            $fvs = FeatureValue::getFeatureValuesWithLang($id_lang, $feature['id_feature']);
            $vals = array();
            foreach ($fvs as $fv) {
                $vals[] = array(
                    'id' => $fv['id_feature_value'],
                    'name' => $fv['value']
                );
            }
            
            $fs[] = array(
                'id' => $feature['id_feature'],
                'name' => $feature['name'],
                'vals' => $vals,
                'selected' => (strpos($kasf, ',' . $feature['id_feature'] . ',') === false)
            );
        }
    }
    
    return $fs; 
}

function getKerawenStockRestriction() {
    
    static $_stockRestriction = null;
    
    if ($_stockRestriction === null) {
        
        $context = Context::getContext();
        $shop = new Shop($context->shop->id);
        $shop_group = $shop->getGroup();
        
        if ($shop_group->share_stock) {
            $_stockRestriction = 'sa.id_shop = 0 AND sa.id_shop_group = ' . (int) $shop_group->id;
        } else {
            $_stockRestriction = 'sa.id_shop = ' . (int) $shop->id;
        }
        
    }
    
    return $_stockRestriction;
}

