<?php
/**
 * 2014 KerAwen
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@kerawen.com so we can send you a copy immediately.
 *
 * @author    KerAwen <contact@kerawen.com>
 * @copyright 2014 KerAwen
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License (AFL 3.0)
 */

require_once(_KERAWEN_API_DIR_.'/constants.php');
require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
require_once(_KERAWEN_SECURE_DIR_.'/KerawenSecure.php');


function getLogFilter($column, $id)
{
	if ($id == _KERAWEN_CD_ALL_) return '';
	if ($id === _KERAWEN_CD_NONE_) return ' AND ('.$column.' IS NULL OR '.$column.' = 0)';
	//till selector : exclude Web
	if ($id == '-2') return ' AND ('.$column.' != 0)';

	$withNull = '';
	if ($column == 'co.id_employee') {		
		if (!(strpos(',' . $id . ',' , ',0,') === false)) {
			//$withNull = ' OR ' . $column . ' IS NULL ';
			return '';
		}
	}
	return ' AND ('.$column.' IN (' . implode(',', array_map('intval', explode(',', $id))) . ')' . $withNull . ')';
}



function getOpFilter($prefix, $ops)
{
	foreach ($ops as &$op) $op = pSQL($op);
	return $prefix.' IN ('.implode(', ', $ops).')';
}

function getSalesLog($from, $to, $till, $employee)
{
	$slip_amounts = '
		-(s.total_products_tax_excl + s.total_shipping_tax_excl) AS tax_excl,
		-(s.total_products_tax_incl + s.total_shipping_tax_incl) AS tax_incl';
	
	// Backward compatibility
	if (Tools::version_compare(_PS_VERSION_, '1.6.0.9', '<='))
			$slip_amounts = '
			-(SUM(sd.amount_tax_excl) + s.shipping_cost_amount/(1 + o.carrier_tax_rate/100)) AS tax_excl,
			-(SUM(sd.amount_tax_incl) + s.shipping_cost_amount) AS tax_incl';
	
	$query = '
		SELECT
			CONCAT("_", co.id_cashdrawer_op) AS id_op,
			co.`date` AS `date`,
			cd.name AS till,
			CONCAT(e.firstname, " ", e.lastname) AS employee,
			co.`oper` AS `oper`,
			cs.id_order AS `order`,
			NULL AS slip,
			o.reference AS reference,
			CONCAT(c.firstname, " ", c.lastname) AS customer,
			sh.name AS shop,
			IF(cs.oper = "ORDER", 1, -1)*o.total_paid_tax_excl AS tax_excl,
			IF(cs.oper = "ORDER", 1, -1)*o.total_paid_tax_incl AS tax_incl
		FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
		JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op
		LEFT JOIN '._DB_PREFIX_.'cash_drawer_kerawen cd ON cd.id_cash_drawer = co.id_cashdrawer
		LEFT JOIN '._DB_PREFIX_.'employee e ON e.id_employee = co.id_employee
		JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
		LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
		LEFT JOIN '._DB_PREFIX_.'shop sh ON sh.id_shop = o.id_shop
		WHERE
			cs.oper IN ("ORDER", "CANCEL")
			AND co.date BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'"
			'.getLogFilter('co.id_cash_drawer', $till).'
			'.getLogFilter('co.id_employee', $employee).'
		GROUP BY cs.id_cashdrawer_sale
		UNION
		SELECT
			CONCAT("_", co.id_cashdrawer_op) AS id_op,
			co.`date` AS `date`,
			cd.name AS till,
			CONCAT(e.firstname, " ", e.lastname) AS employee,
			co.`oper` AS `oper`,
			cs.id_order AS `order`,
			cs.id_order_slip AS slip,
			o.reference AS reference,
			CONCAT(c.firstname, " ", c.lastname) AS customer,
			sh.name AS shop,
			'.$slip_amounts.'
		FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
		JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op
		LEFT JOIN '._DB_PREFIX_.'cash_drawer_kerawen cd ON cd.id_cash_drawer = co.id_cashdrawer
		LEFT JOIN '._DB_PREFIX_.'employee e ON e.id_employee = co.id_employee
		JOIN '._DB_PREFIX_.'order_slip s ON s.id_order_slip = cs.id_order_slip
		JOIN '._DB_PREFIX_.'orders o ON o.id_order = s.id_order
		LEFT JOIN '._DB_PREFIX_.'order_slip_detail sd ON sd.id_order_slip = s.id_order_slip
		LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = s.id_customer
		LEFT JOIN '._DB_PREFIX_.'shop sh ON sh.id_shop = o.id_shop
		WHERE
			cs.oper = "SLIP"
			AND co.date BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'"
			'.getLogFilter('co.`id_cash_drawer`', $till).'
			'.getLogFilter('co.`id_employee`', $employee).'
		GROUP BY cs.id_cashdrawer_sale
		ORDER BY date';

	return Db::getInstance()->executeS($query);
}

function getTaxesLog($from, $to, $till, $employee)
{
	$query = '
		SELECT
			CONCAT("_", co.id_cashdrawer_op) AS id_op,
			co.`date` AS `date`,
			cd.name AS till,
			CONCAT(e.firstname, " ", e.lastname) AS employee,
			co.`oper` AS `oper`,
			cs.id_order AS `order`,
			NULL AS slip,
			o.reference AS reference,
			CONCAT(c.firstname, " ", c.lastname) AS customer,
			sh.name AS shop,
			odt.id_tax AS id,
			IF(cs.oper = "ORDER", 1, -1)*SUM(odt.unit_amount*od.product_quantity) AS tax,
			IF(cs.oper = "ORDER", 1, -1)*SUM(od.total_price_tax_excl)*(1 - IFNULL(ok.product_global_discount,0)) AS base
		FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
		JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op
		LEFT JOIN '._DB_PREFIX_.'cash_drawer_kerawen cd ON cd.id_cash_drawer = co.id_cashdrawer
		LEFT JOIN '._DB_PREFIX_.'employee e ON e.id_employee = co.id_employee
		JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
		JOIN '._DB_PREFIX_.'order_detail od ON od.id_order = o.id_order
		LEFT JOIN '._DB_PREFIX_.'order_detail_tax odt ON odt.id_order_detail = od.id_order_detail
		LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
		LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
		LEFT JOIN '._DB_PREFIX_.'shop sh ON sh.id_shop = o.id_shop
		WHERE
			cs.oper IN ("ORDER", "CANCEL")
			AND co.date BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'"
			'.getLogFilter('co.id_cash_drawer', $till).'
			'.getLogFilter('co.id_employee', $employee).'
		GROUP BY cs.id_cashdrawer_sale, odt.id_tax
		UNION
		SELECT
			CONCAT("_", co.id_cashdrawer_op) AS id_op,
			co.`date` AS `date`,
			cd.name AS till,
			CONCAT(e.firstname, " ", e.lastname) AS employee,
			co.`oper` AS `oper`,
			cs.id_order AS `order`,
			cs.id_order_slip AS slip,
			o.reference AS reference,
			CONCAT(c.firstname, " ", c.lastname) AS customer,
			sh.name AS shop,
			odt.id_tax AS id,
			-SUM(sd.amount_tax_incl - sd.amount_tax_excl) AS tax,
			-SUM(sd.amount_tax_excl) AS base
		FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
		JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op
		LEFT JOIN '._DB_PREFIX_.'cash_drawer_kerawen cd ON cd.id_cash_drawer = co.id_cashdrawer
		LEFT JOIN '._DB_PREFIX_.'employee e ON e.id_employee = co.id_employee
		JOIN '._DB_PREFIX_.'order_slip s ON s.id_order_slip = cs.id_order_slip
		JOIN '._DB_PREFIX_.'orders o ON o.id_order = s.id_order
		JOIN '._DB_PREFIX_.'order_slip_detail sd ON sd.id_order_slip = s.id_order_slip
		JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = sd.id_order_detail
		LEFT JOIN '._DB_PREFIX_.'order_detail_tax odt ON odt.id_order_detail = od.id_order_detail
		LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = s.id_customer
		LEFT JOIN '._DB_PREFIX_.'shop sh ON sh.id_shop = o.id_shop
		WHERE
			cs.oper = "SLIP"
			AND co.date BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'"
			'.getLogFilter('co.`id_cash_drawer`', $till).'
			'.getLogFilter('co.`id_employee`', $employee).'
		GROUP BY cs.id_cashdrawer_sale, odt.id_tax
		ORDER BY date';

	return Db::getInstance()->executeS($query);
}

function getProductsLog($from, $to, $till, $employee)
{
	$context = Context::getContext();
	$id_lang = $context->language->id;

	$db = Db::getInstance();
	$orders = $db->executeS('
		SELECT
			p.id_product AS id_prod,
			pa.id_product_attribute AS id_attr,
			p.`reference` AS `reference`,
			p.`ean13` AS `ean`,
			pl.`name` AS `product`,
			GROUP_CONCAT(al.`name` ORDER BY a.`id_attribute_group` SEPARATOR ",") AS `version`,
			su.`name` AS `supplier`,
			o.`id_order` AS `id_order`,
			o.`reference` AS `order`,
			NULL AS `id_slip`,
			NULL AS `slip`,
			o.`date_add` AS `date`,
			cd.`name` AS `till`,
			CONCAT(e.`firstname`, " ", e.`lastname`) AS `employee`,
			c.id_customer AS id_cust,
			CONCAT(c.`firstname`, " ", c.`lastname`) AS `customer`,
			c.id_default_group AS id_group,
			gl.name AS `group`,
			p.wholesale_price AS wholesale,
			od.`unit_price_tax_excl`*(1 - IFNULL(ok.product_global_discount, 0)) AS `unit`,
			od.`product_quantity` AS `quantity`,
			od.`total_price_tax_excl`*(1 - IFNULL(ok.product_global_discount, 0)) AS `total`,
			odk.note AS note
		FROM `'._DB_PREFIX_.'orders` o
		LEFT JOIN `'._DB_PREFIX_.'order_kerawen` ok
			ON ok.id_order = o.id_order
		LEFT JOIN `'._DB_PREFIX_.'cashdrawer_sale_kerawen` cs
			ON cs.`id_order` = o.`id_order`
		LEFT JOIN `'._DB_PREFIX_.'cashdrawer_op_kerawen` co
			ON co.`id_cashdrawer_op` = cs.`id_cashdrawer_op`
		LEFT JOIN `'._DB_PREFIX_.'cash_drawer_kerawen` cd
			ON cd.`id_cash_drawer` = co.`id_cashdrawer`
		LEFT JOIN `'._DB_PREFIX_.'employee` e
			ON e.`id_employee` = co.`id_employee`
		LEFT JOIN `'._DB_PREFIX_.'customer` c
			ON c.`id_customer` = o.`id_customer`
		LEFT JOIN `'._DB_PREFIX_.'group_lang` gl
			ON (gl.`id_group` = c.`id_default_group`
				AND gl.id_lang = '. (int) $id_lang .')
		LEFT JOIN `'._DB_PREFIX_.'order_detail` od
			ON od.`id_order` = o.`id_order`
		LEFT JOIN `'._DB_PREFIX_.'order_detail_kerawen` odk
			ON odk.`id_order_detail` = od.`id_order_detail`
		LEFT JOIN `'._DB_PREFIX_.'product` p
			ON p.`id_product` = od.`product_id`
		LEFT JOIN `'._DB_PREFIX_.'product_lang` pl
			ON (pl.`id_product` = p.`id_product`
				AND pl.`id_shop` = od.`id_shop`
				AND pl.id_lang = '. (int) $id_lang .')
		LEFT JOIN `'._DB_PREFIX_.'product_attribute` pa
			ON pa.id_product_attribute = od.product_attribute_id
		LEFT JOIN `'._DB_PREFIX_.'product_attribute_combination` pac
			ON pac.id_product_attribute = pa.id_product_attribute
		LEFT JOIN `'._DB_PREFIX_.'attribute` a
			ON a.id_attribute = pac.id_attribute
		LEFT JOIN `'._DB_PREFIX_.'attribute_lang` al
			ON (al.id_attribute = a.id_attribute
				AND al.id_lang = '. (int) $id_lang .')
		LEFT JOIN `'._DB_PREFIX_.'supplier` su
			ON su.id_supplier = p.id_supplier
		WHERE
			o.`date_add` BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'"
			AND o.`valid`= 1'
			.getLogFilter('co.`id_cashdrawer`', $till)
			.getLogFilter('co.`id_employee`', $employee).'
		GROUP BY od.`id_order_detail`');

	// TODO Check diff between amount_tax_excl and total_price_tax_excl (new)
	$slips = $db->executeS('
		SELECT
			p.id_product AS id_prod,
			pa.id_product_attribute AS id_attr,
			p.`reference` AS `reference`,
			p.`ean13` AS `ean`,
			pl.`name` AS `product`,
			GROUP_CONCAT(al.`name` ORDER BY a.`id_attribute_group` SEPARATOR ",") AS `version`,
			su.`name` AS `supplier`,
			o.`id_order` AS `id_order`,
			o.`reference` AS `order`,
			s.`id_order_slip` AS `id_slip`,
			s.`id_order_slip` AS `slip`,
			s.`date_add` AS `date`,
			cd.`name` AS `till`,
			CONCAT(e.`firstname`, " ", e.`lastname`) AS `employee`,
			c.id_customer AS id_cust,
			CONCAT(c.`firstname`, " ", c.`lastname`) AS `customer`,
			c.id_default_group AS id_group,
			gl.name AS `group`,
			p.wholesale_price AS wholesale,
			-sd.`amount_tax_excl`/sd.`product_quantity` AS `unit`,
			-sd.`product_quantity` AS `quantity`,
			-sd.`amount_tax_excl` AS `total`
		FROM '._DB_PREFIX_.'order_slip s
		LEFT JOIN '._DB_PREFIX_.'orders o
			ON o.id_order = s.id_order
		LEFT JOIN `'._DB_PREFIX_.'cashdrawer_sale_kerawen` cs
			ON cs.`id_order_slip` = s.`id_order_slip`
		LEFT JOIN `'._DB_PREFIX_.'cashdrawer_op_kerawen` co
			ON co.`id_cashdrawer_op` = cs.`id_cashdrawer_op`
		LEFT JOIN `'._DB_PREFIX_.'cash_drawer_kerawen` cd
			ON cd.`id_cash_drawer` = co.`id_cashdrawer`
		LEFT JOIN `'._DB_PREFIX_.'employee` e
			ON e.`id_employee` = co.`id_employee`
		LEFT JOIN `'._DB_PREFIX_.'customer` c
			ON c.`id_customer` = s.`id_customer`
		LEFT JOIN `'._DB_PREFIX_.'group_lang` gl
			ON (gl.`id_group` = c.`id_default_group`
				AND gl.id_lang = '. (int) $id_lang .')
		LEFT JOIN `'._DB_PREFIX_.'order_slip_detail` sd
			ON sd.`id_order_slip` = s.`id_order_slip`
		LEFT JOIN `'._DB_PREFIX_.'order_detail` od
			ON od.`id_order_detail` = sd.`id_order_detail`
		LEFT JOIN `'._DB_PREFIX_.'product` p
			ON p.`id_product` = od.`product_id`
		LEFT JOIN `'._DB_PREFIX_.'product_lang` pl
			ON (pl.`id_product` = p.`id_product`
				AND pl.`id_shop` = od.`id_shop`
				AND pl.id_lang = '. (int) $id_lang .')
		LEFT JOIN `'._DB_PREFIX_.'product_attribute` pa
			ON pa.id_product_attribute = od.product_attribute_id
		LEFT JOIN `'._DB_PREFIX_.'product_attribute_combination` pac
			ON pac.id_product_attribute = pa.id_product_attribute
		LEFT JOIN `'._DB_PREFIX_.'attribute` a
			ON a.id_attribute = pac.id_attribute
		LEFT JOIN `'._DB_PREFIX_.'attribute_lang` al
			ON (al.id_attribute = a.id_attribute
				AND al.id_lang = '. (int) $id_lang .')
		LEFT JOIN `'._DB_PREFIX_.'supplier` su
			ON su.id_supplier = p.id_supplier
		WHERE
			s.`date_add` BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'"'
			.getLogFilter('co.`id_cashdrawer`', $till)
			.getLogFilter('co.`id_employee`', $employee).'
		GROUP BY sd.id_order_slip, sd.id_order_detail');

	return array_merge($orders, $slips);
}

function getPaymentsLog($from, $to, $till, $employee, $deferred = false)
{
	$date_column = $deferred ? 'cf.date_deferred' : 'co.date';

	return Db::getInstance()->executeS('
		SELECT
			CONCAT("_", co.id_cashdrawer_op) AS id_op,
			co.`date` AS `date`,
			cd.`name` AS `till`,
			CONCAT(e.`firstname`, " ", e.`lastname`) AS `employee`,
			co.`oper` AS `oper`,
			cf.`id_payment_mode` AS `id_mode`,
			op.payment_method AS mode,
			cf.`amount` AS `amount`,
			cf.`date_deferred` AS `deferred`,
			cf.comments AS comments
		FROM `'._DB_PREFIX_.'cashdrawer_op_kerawen` co
		JOIN `'._DB_PREFIX_.'cashdrawer_flow_kerawen` cf
			ON cf.`id_cashdrawer_op` = co.`id_cashdrawer_op`
		LEFT JOIN `'._DB_PREFIX_.'cash_drawer_kerawen` cd
			ON cd.`id_cash_drawer` = co.`id_cashdrawer`
		LEFT JOIN `'._DB_PREFIX_.'employee` e
			ON e.`id_employee` = co.`id_employee`
		LEFT JOIN '._DB_PREFIX_.'order_payment op
			ON op.id_order_payment = cf.id_order_payment
		WHERE '.$date_column.' BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'"'
			.getLogFilter('co.`id_cashdrawer`', $till)
			.getLogFilter('co.`id_employee`', $employee).'
		ORDER BY '.$date_column);
}

function getClosingsLog($from, $to, $till, $employee)
{
	return Db::getInstance()->executeS('
		SELECT
			CONCAT("_", co.id_cashdrawer_op) AS id_op,
			co.`oper` AS `oper`,
			co.`date` AS `date`,
			cd.`name` AS `till`,
			CONCAT(e.`firstname`, " ", e.`lastname`) AS `employee`,
			co.`error` AS `error`
		FROM `'._DB_PREFIX_.'cashdrawer_op_kerawen` co
		LEFT JOIN `'._DB_PREFIX_.'cash_drawer_kerawen` cd
			ON cd.`id_cash_drawer` = co.`id_cashdrawer`
		LEFT JOIN `'._DB_PREFIX_.'employee` e
			ON e.`id_employee` = co.`id_employee`
		WHERE 
			co.oper IN (
				"'.pSQL(_KERAWEN_CDOP_OPEN_).'",
				"'.pSQL(_KERAWEN_CDOP_CLOSE_).'",
				"'.pSQL(_KERAWEN_CDOP_FLOW_).'"
			)
			AND co.date BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'"'
			.getLogFilter('co.`id_cashdrawer`', $till)
			.getLogFilter('co.`id_employee`', $employee));
}

// --------------------------
// FUTURE

class LogQuery
{

    public static function withOCS() {
        return Module::isInstalled('kerawen_ocs') && Module::isEnabled('kerawen_ocs');
    }
    
	public static function bypage() {
		//(int) Configuration::get('KERAWEN_REPORT_ITEMS_BY_PAGE');
		return 50;
	}	

	public static function setOrder($orderby, $sortby) {
	    
	    $array_sort = array(
	        'ASC',
	        'DESC'
	    );
	    
	    $array_order = array(
    	    'gender',
	        'id_customer',
    	    'firstname',
    	    'lastname',
    	    'email',
    	    'lang',
    	    'group',
    	    'amount',
    	    'limit',
    	    'shop',
    	    'newsletter',
    	    'date_add',
    	    'birthday',
    	    'account',
    	    'mobile',
    	    'phone',
    	    'cp',
    	    'last_order_date',
    	    'orders',
    	    'prod_name',
    	    'id',
    	    'declinaison',
    	    'ref',
    	    'buying_price',
    	    'ean',
    	    'cat',
    	    'supplier',
    	    'supplier_ref',
    	    'manufacturer',
    	    'saling_price',
    	    'stock_type',
    	    'stock',
    	    'projection',
    	    'qty_sale',
    	    'valorization',
    	    'low_stock',
    	    'to_order',
	        'id_row',
	        'dept',
	        'city',
	        'country',
	        'product_name',
	        'declinaison',
	        'category',
	        'ean13',
	        'main_reference',
	        'reference',
	        'price_vat_incl',
	        'mvtstock',
	        'id' => 'id',
	        'product' => 'product',
	        'unit_tax_incl' => 'unit_tax_incl',
	        'qty_sale_rate' => 'qty_sale_rate',
	        'qty_sale' => 'qty_sale',
	        'saleAmount' => 'saleAmount',
	        'buyQty' => 'buyQty',
	        'buyAmount' => 'buyAmount',
	        'tx_ca' => 'tx_ca'
	        
	   );

	    if (in_array($orderby, $array_order) && in_array($sortby, $array_sort)) {
	        return 	' ORDER BY `' . $orderby . '` ' . $sortby . ' ';
	    }
	    
	    return '';
	}
	
	public static function saleInfo() {
		return '
			CONCAT("_", cs.id_cashdrawer_op) AS id_op,
			cs.id_cashdrawer_sale AS id_sale,
			cs.oper AS oper';
	}
	

	public static function getDiscountParam($type) {
		
		switch($type) {
			case 'cart':
				$p = '0, odk.specific_price_cart';
			break;
			case 'catalog':
				$p = '1, (1 - odk.specific_price_cart)';
			break;
			default:
				$p = '1, 1';
			break;	
		}
		
		return $p;

	}
	
	public static function sales($db, $op_filter, $vars = false)
	{
		$discount_filter = (isset($vars->discount_filter)) ? $vars->discount_filter : 'all';

		$res = array_merge(
			self::orders($db, $op_filter, $discount_filter),
			self::slips($db, $op_filter, $discount_filter)
		);
		foreach($res as &$s) {
			$s['accounted'] = 1;
		}
		return $res;
	}
	
	public static function orders($db, $op_filter, $discount_filter = 'all')
	{
		$id_lang = Context::getContext()->language->id;
		
		$product_cost = 'od.original_wholesale_price';
		if (Tools::version_compare(_PS_VERSION_, '1.6.0.14', '<=')) {
			$product_cost = 'od.purchase_supplier_price';
		}
		
		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				IF(cs.oper = "ORDER", 1, -1)*o.total_paid_tax_excl AS tax_excl,
				IF(cs.oper = "ORDER", 1, -1)*o.total_paid_tax_incl AS tax_incl,
				IF(cs.oper = "ORDER",1,-1)*(o.total_discounts_tax_excl + psp2.reduc_tax_excl) AS disc_te,
				IF(cs.oper = "ORDER",1,-1)*(o.total_discounts_tax_incl + psp2.reduc_tax_incl) AS disc_ti,
				o.reference AS ref,
				/*o.invoice_number AS id_invoice,*/
				IF(o.invoice_number = 0, "", CONCAT("FA ", o.invoice_number)) AS id_invoice,
				o.id_shop AS id_shop,
				CONCAT(c.firstname, " ", c.lastname) AS cust,
				c.company,
				gl.name AS `group`,
				SUM(od.product_quantity) AS qty,
				IF(cs.oper = "ORDER",1,-1)*(o.total_paid_tax_excl - SUM(IF(ISNULL(mvt.id_order),'.$product_cost.',mvt.price_te)*od.product_quantity)) AS profit
					
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			LEFT JOIN '._DB_PREFIX_.'group_lang gl ON gl.id_group = c.id_default_group AND gl.id_lang = '. (int) $id_lang.'
			LEFT JOIN '._DB_PREFIX_.'order_detail od ON od.id_order = o.id_order
				
			LEFT JOIN (
    			SELECT
					sm.id_order, st.id_product, st.id_product_attribute,
					SUM(sm.physical_quantity*st.price_te)/SUM(sm.physical_quantity) AS price_te
    			FROM '._DB_PREFIX_.'stock_mvt sm
				JOIN '._DB_PREFIX_.'stock st ON st.id_stock = sm.id_stock
				GROUP BY sm.id_order, st.id_product, st.id_product_attribute
			) mvt ON mvt.id_order = o.id_order AND mvt.id_product = od.product_id AND mvt.id_product_attribute = od.product_attribute_id
				
			LEFT JOIN (
				SELECT 
					ok.id_order,
					SUM(IF(ISNULL(odk.specific_price_cart), ' . self::getDiscountParam($discount_filter) . ') * (IF ((od.reduction_percent > 0), od.total_price_tax_incl * ( 1 / ( 1 - od.reduction_percent/100 ) - 1 ), od.reduction_amount_tax_incl * od.product_quantity) ) ) AS reduc_tax_incl,
					SUM(IF(ISNULL(odk.specific_price_cart), ' . self::getDiscountParam($discount_filter) . ') * (IF ((od.reduction_percent > 0), od.total_price_tax_excl * ( 1 / ( 1 - od.reduction_percent/100 ) - 1 ), od.reduction_amount_tax_excl * od.product_quantity) ) ) AS reduc_tax_excl
				FROM ' . _DB_PREFIX_ . 'order_kerawen ok
				LEFT JOIN ' . _DB_PREFIX_ . 'order_detail od ON ok.id_order = od.id_order
				LEFT JOIN ' . _DB_PREFIX_ . 'order_detail_kerawen odk ON od.id_order_detail = odk.id_order_detail
				GROUP BY ok.id_order
			) psp2 ON o.id_order = psp2.id_order

			WHERE cs.oper IN ("ORDER", "CANCEL")
			GROUP BY cs.id_cashdrawer_sale, o.id_order');
	}

	public static function slips($db, $op_filter, $discount_filter = 'all')
	{
		$id_lang = Context::getContext()->language->id;
		
		$select_amounts = '
			-(s.total_products_tax_excl + s.total_shipping_tax_excl) AS tax_excl,
			-(s.total_products_tax_incl + s.total_shipping_tax_incl) AS tax_incl';

		$select_amounts_marge = ' (s.total_products_tax_excl + s.total_shipping_tax_excl) ';
		$select_profit_mvt = ' (mvt.price_te - sd.unit_price_tax_excl) * sd.product_quantity';


		// Backward compatibility
		if (Tools::version_compare(_PS_VERSION_, '1.6.0.9', '<=')) {
			$select_amounts = '
				-(SUM(sd.amount_tax_excl) + s.shipping_cost_amount/(1 + o.carrier_tax_rate/100)) AS tax_excl,
				-(SUM(sd.amount_tax_incl) + s.shipping_cost_amount) AS tax_incl';

			$select_amounts_marge = ' (SUM(sd.amount_tax_excl) + s.shipping_cost_amount/(1 + o.carrier_tax_rate/100)) ';
			//??? to check
			$select_profit_mvt = ' (mvt.price_te - SUM(sd.amount_tax_excl/sd.product_quantity)) * SUM(sd.product_quantity) ';
		}


		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				cs.id_order_slip AS id_slip,
				(-1) * IF(ISNULL(odk.specific_price_cart), ' . self::getDiscountParam($discount_filter) . ') * (IF ((od.reduction_percent > 0), od.total_price_tax_excl * ( 1 / ( 1 - od.reduction_percent/100 ) - 1 ), od.reduction_amount_tax_excl * sd.product_quantity) ) AS disc_te,
				(-1) * IF(ISNULL(odk.specific_price_cart), ' . self::getDiscountParam($discount_filter) . ') * (IF ((od.reduction_percent > 0), od.total_price_tax_incl * ( 1 / ( 1 - od.reduction_percent/100 ) - 1 ), od.reduction_amount_tax_incl * sd.product_quantity) ) AS disc_ti,
				'.$select_amounts.',
				o.reference AS ref,
				/*0 AS id_invoice,*/
				IF(o.invoice_number = 0, "", CONCAT("AV ", cs.id_order_slip)) AS id_invoice,
				o.id_shop AS id_shop,
				CONCAT(c.firstname, " ", c.lastname) AS cust,
				c.company,
				IF(ISNULL(mvt.id_stock_mvt), ((sd.product_quantity) * od.purchase_supplier_price) - ' . $select_amounts_marge . ', ' . $select_profit_mvt . ') AS profit,
				gl.name AS `group`
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'order_slip s ON s.id_order_slip = cs.id_order_slip
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = s.id_order
			LEFT JOIN '._DB_PREFIX_.'order_slip_detail sd ON sd.id_order_slip = s.id_order_slip
			LEFT JOIN ' . _DB_PREFIX_ . 'order_detail od ON sd.id_order_detail = od.id_order_detail
			LEFT JOIN ' . _DB_PREFIX_ . 'order_detail_kerawen odk ON sd.id_order_detail = odk.id_order_detail	
			LEFT JOIN ' . _DB_PREFIX_ . 'stock sto ON od.product_id = sto.id_product AND od.product_attribute_id = sto.id_product_attribute
			LEFT JOIN ' . _DB_PREFIX_ . 'stock_mvt mvt ON od.id_order = mvt.id_order AND sto.id_stock = mvt.id_stock
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			LEFT JOIN '._DB_PREFIX_.'group_lang gl ON gl.id_group = c.id_default_group AND gl.id_lang = '. (int) $id_lang .'
			WHERE cs.oper = "SLIP"
			GROUP BY cs.id_cashdrawer_sale');
	}

	public static function taxes($db, $op_filter)
	{
		$res = array_merge(
			self::orderProductTaxes($db, $op_filter),
			self::orderShippingTaxes($db, $op_filter),
			self::orderWrappingTaxes($db, $op_filter),
			self::slipProductTaxes($db, $op_filter),
			self::slipShippingTaxes($db, $op_filter)
			//temporary disable flow -> TODO LATER
			//self::orderDeferredTaxes($db, $op_filter)
		);
		foreach ($res as &$t) {
			$t['accounted'] = 1;
		}
		return $res;
	}

	/*
	//temporary disable flow -> TODO LATER
	public static function  orderDeferredTaxes($db, $op_filter) {

		return $db->executeS('
			SELECT 	
				'.self::saleInfo().',
				cs.id_order AS id_order,
				o.reference AS ref,
				o.id_shop AS id_shop,
				"DEFERRED" AS object,
				0 AS id_tax,
				cfk.amount AS gross_te,
				cfk.amount AS tax_excl,
				0 AS tax
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
			LEFT JOIN '._DB_PREFIX_.'cashdrawer_flow_kerawen cfk ON cfk.id_cashdrawer_op = cs.id_cashdrawer_op
			WHERE cfk.id_payment_mode IN (' . _KERAWEN_PM_GIFT_CARD_ . ')');
	}
	*/
	
	public static function orderProductTaxes($db, $op_filter)
	{
		$taxrule = 'od.id_tax_rules_group';
		$joinprod = '';
		if (Tools::version_compare(_PS_VERSION_, '1.6.0.9', '<='))
		{
			$taxrule = 'ps.id_tax_rules_group';
			$joinprod = 'LEFT JOIN '._DB_PREFIX_.'product_shop ps ON ps.id_product = od.product_id AND ps.id_shop = o.id_shop';
		}
		
		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				o.reference AS ref,
				IF(o.invoice_number = 0, "", CONCAT("FA ", o.invoice_number)) AS inv_num,
				o.id_shop AS id_shop,
				"PROD" AS object,
				IFNULL('.$taxrule.', -1) AS id_taxrule,
				IF(cs.oper = "ORDER", 1, -1)*SUM(ROUND(od.original_product_price,2)*od.product_quantity) AS gross_te,
				IF(cs.oper = "ORDER", 1, -1)*SUM(ROUND(od.total_price_tax_excl*(1 - IFNULL(ok.product_global_discount, 0)),2)) AS tax_excl,
				IF(cs.oper = "ORDER", 1, -1)*SUM(ROUND(IFNULL(odt.unit_amount, 0),2)*od.product_quantity) AS tax,
				CONCAT(c.firstname, " ", c.lastname) AS customer,
				c.company
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
			JOIN '._DB_PREFIX_.'order_detail od ON od.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'order_detail_tax odt ON odt.id_order_detail = od.id_order_detail
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			'.$joinprod.'
			WHERE
				cs.oper IN ("ORDER", "CANCEL")
			GROUP BY cs.id_cashdrawer_sale, '.$taxrule);
	}
	
	public static function orderShippingTaxes($db, $op_filter)
	{
		// TODO GET THE CORRECT TAX ID AND NOT TAX RULES ID
		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				o.reference AS ref,
				o.id_shop AS id_shop,
				IF(o.invoice_number = 0, "", CONCAT("FA ", o.invoice_number)) AS inv_num,
				"SHIP" AS object,
				IFNULL(ct.id_tax_rules_group, -1) AS id_taxrule,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_excl*(1 - IFNULL(ok.free_shipping, 0)) AS tax_excl,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_excl*(1 - IFNULL(ok.free_shipping, 0)) AS gross_te,
				IF(cs.oper = "ORDER", 1, -1)*(o.total_shipping_tax_incl - o.total_shipping_tax_excl)*(1 - IFNULL(ok.free_shipping, 0)) AS tax,
				o.carrier_tax_rate AS rate,
				car.name AS carrier,
				/*CONCAT(c.firstname, " ", c.lastname) AS cust,*/
				CONCAT(c.firstname, " ", c.lastname) AS customer,
				c.company
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'carrier_tax_rules_group_shop ct ON ct.id_carrier = o.id_carrier AND ct.id_shop = o.id_shop
			LEFT JOIN '._DB_PREFIX_.'carrier car ON o.id_carrier = car.id_carrier
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			WHERE
				cs.oper IN ("ORDER", "CANCEL")
				AND o.total_shipping_tax_excl > 0
			GROUP BY cs.id_cashdrawer_sale');
	}
	
	public static function orderWrappingTaxes($db, $op_filter)
	{
		// TODO GET THE CORRECT TAX ID AND NOT TAX RULES ID
		$id_taxrule = (int) Configuration::get('PS_GIFT_WRAPPING_TAX_RULES_GROUP');

		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				o.reference AS ref,
				o.id_shop AS id_shop,
				IF(o.invoice_number = 0, "", CONCAT("FA ", o.invoice_number)) AS inv_num,
				"WRAP" AS object,
				'.pSQL($id_taxrule).' as id_taxrule,
				IF(cs.oper = "ORDER", 1, -1)*o.total_wrapping_tax_excl AS tax_excl,
				IF(cs.oper = "ORDER", 1, -1)*o.total_wrapping_tax_excl AS gross_te,
				IF(cs.oper = "ORDER", 1, -1)*(o.total_wrapping_tax_incl - o.total_wrapping_tax_excl)*(1 - IFNULL(ok.free_shipping, 0)) AS tax,
				CONCAT(c.firstname, " ", c.lastname) AS customer,
				c.company
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			LEFT JOIN '._DB_PREFIX_.'carrier_tax_rules_group_shop ct ON ct.id_carrier = o.id_carrier AND ct.id_shop = o.id_shop
			WHERE cs.oper IN ("ORDER", "CANCEL") AND o.total_wrapping_tax_excl > 0
			GROUP BY cs.id_cashdrawer_sale');
	}
	
	public static function slipProductTaxes($db, $op_filter)
	{
		$taxrule = 'od.id_tax_rules_group';
		$joinprod = '';
		if (Tools::version_compare(_PS_VERSION_, '1.6.0.9', '<='))
		{
			$taxrule = 'ps.id_tax_rules_group';
			$joinprod = 'LEFT JOIN '._DB_PREFIX_.'product_shop ps ON ps.id_product = od.product_id AND ps.id_shop = o.id_shop';
		}
		
		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				cs.id_order_slip AS id_slip,
				o.reference AS ref,
				/*o.invoice_number AS inv_num,*/
				IF(o.invoice_number = 0, "", CONCAT("AV ", cs.id_order_slip)) AS inv_num,
				o.id_shop AS id_shop,
				"PROD" AS object,
				IFNULL('.$taxrule.', -1) AS id_taxrule,
				-SUM(ROUND(sd.amount_tax_excl,2)) AS tax_excl,
				-ROUND(od.original_product_price,2)*sd.product_quantity AS gross_te,
				-SUM(ROUND(sd.amount_tax_incl - sd.amount_tax_excl,2)) AS tax,
				CONCAT(c.firstname, " ", c.lastname) AS customer,
				c.company
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'order_slip s ON s.id_order_slip = cs.id_order_slip
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = s.id_order
			JOIN '._DB_PREFIX_.'order_slip_detail sd ON sd.id_order_slip = s.id_order_slip
			JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = sd.id_order_detail
			LEFT JOIN '._DB_PREFIX_.'order_detail_tax odt ON odt.id_order_detail = od.id_order_detail
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			'.$joinprod.'
			WHERE cs.oper = "SLIP"
			GROUP BY cs.id_cashdrawer_sale, '.$taxrule);
	}
	
	public static function slipShippingTaxes($db, $op_filter)
	{
		$select_amounts = '
			-s.total_shipping_tax_excl AS tax_excl,
			-s.total_shipping_tax_excl AS gross_te,
			-(s.total_shipping_tax_incl - s.total_shipping_tax_excl) AS tax';
		$condition = 's.total_shipping_tax_excl > 0';
		
		// Backward compatibility
		if (Tools::version_compare(_PS_VERSION_, '1.6.0.9', '<='))
		{
			$select_amounts = '
				-s.shipping_cost_amount/(1 + o.carrier_tax_rate/100) AS tax_excl,
				-s.shipping_cost_amount/(1 + o.carrier_tax_rate/100) AS gross_te,
				-s.shipping_cost_amount*o.carrier_tax_rate/(100 + o.carrier_tax_rate) AS tax';
			$condition = 's.shipping_cost_amount > 0';
		}
		
		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				cs.id_order_slip AS id_slip,
				o.reference AS ref,
				o.id_shop AS id_shop,
				/*o.invoice_number AS inv_num,*/
				IF(o.invoice_number = 0, "", CONCAT("AV ", cs.id_order_slip)) AS inv_num,
				"SHIP" AS object,
				IFNULL(ct.id_tax_rules_group, -1) AS id_taxrule,
				'.$select_amounts.',
				CONCAT(c.firstname, " ", c.lastname) AS customer,
				c.company
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'order_slip s ON s.id_order_slip = cs.id_order_slip
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = s.id_order
			LEFT JOIN '._DB_PREFIX_.'carrier_tax_rules_group_shop ct ON ct.id_carrier = o.id_carrier AND ct.id_shop = o.id_shop
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			WHERE cs.oper = "SLIP" AND '.$condition.'
			GROUP BY cs.id_cashdrawer_sale');
	}
	
	public static function productInfo($id_lang) {
		return '
			p.id_product AS id_prod,
			pa.id_product_attribute AS id_attr,
			p.id_category_default AS id_cat,
			pl.name AS prod,
			col.name AS order_country,
			car.name AS carrier,
			IF(ISNULL(mvt.id_order), od.purchase_supplier_price, mvt.price_te) AS purchase_price,
			(SELECT GROUP_CONCAT(al.name ORDER BY a.id_attribute_group SEPARATOR ",")
				FROM '._DB_PREFIX_.'product_attribute_combination pac
				LEFT JOIN '._DB_PREFIX_.'attribute a ON a.id_attribute = pac.id_attribute
				LEFT JOIN '._DB_PREFIX_.'attribute_lang al ON al.id_attribute = a.id_attribute AND al.id_lang = '. (int) $id_lang .'
				WHERE pac.id_product_attribute = pa.id_product_attribute
			) AS version,
			cl.name AS cat,
			IF(pa.reference != "", pa.reference, p.reference) AS ref,
            p.reference AS main_ref,
			IF(pa.ean13 != "", pa.ean13, p.ean13) AS ean,
			IF(pa.upc != "", pa.upc, p.upc) as upc,
			ps.wholesale_price AS wholesale,
			su.name AS supplier,
			su.id_supplier,
			ma.name AS manufacturer,
			ma.id_manufacturer,
			odk.note AS note,
            odk.id_tag AS id_tag';
	}
	
	public static function productJoin($id_lang) {
		return '
			LEFT JOIN '._DB_PREFIX_.'order_detail_kerawen odk ON odk.id_order_detail = od.id_order_detail
			LEFT JOIN '._DB_PREFIX_.'product p ON p.id_product = od.product_id
			LEFT JOIN '._DB_PREFIX_.'product_shop ps ON ps.id_product = p.id_product AND ps.id_shop = od.id_shop
			LEFT JOIN '._DB_PREFIX_.'product_lang pl ON pl.id_product = p.id_product AND pl.id_shop = ps.id_shop AND pl.id_lang = '. (int) $id_lang .'
			LEFT JOIN '._DB_PREFIX_.'product_attribute pa ON pa.id_product_attribute = od.product_attribute_id
			LEFT JOIN '._DB_PREFIX_.'category_lang cl ON cl.id_category = p.id_category_default AND cl.id_shop = ps.id_shop AND cl.id_lang = '. (int) $id_lang .'
			LEFT JOIN '._DB_PREFIX_.'supplier su ON su.id_supplier = p.id_supplier
			LEFT JOIN '._DB_PREFIX_.'manufacturer ma ON ma.id_manufacturer = p.id_manufacturer
			LEFT JOIN '._DB_PREFIX_.'address addr ON o.id_address_delivery = addr.id_address
			LEFT JOIN '._DB_PREFIX_.'country_lang col ON addr.id_country = col.id_country AND col.id_lang = '. (int) $id_lang .'
			LEFT JOIN '._DB_PREFIX_.'carrier car ON o.id_carrier = car.id_carrier
			LEFT JOIN (
				SELECT
					sm.id_order, st.id_product, st.id_product_attribute,
					SUM(sm.physical_quantity*st.price_te)/SUM(sm.physical_quantity) AS price_te
				FROM '._DB_PREFIX_.'stock_mvt sm
				JOIN '._DB_PREFIX_.'stock st ON st.id_stock = sm.id_stock
				GROUP BY sm.id_order, st.id_product, st.id_product_attribute
			) mvt ON mvt.id_order = o.id_order AND mvt.id_product = od.product_id AND mvt.id_product_attribute = od.product_attribute_id
		'; 
	}
	
	public static function completeProducts($prods, $id_shop = null) {
		$id_lang = Context::getContext()->language->id;
		foreach($prods as &$prod)
		{
			if ($id_shop) $prod['id_shop'] = $id_shop;
			$prod['root'] = self::getParentCategory($prod['id_cat'], $prod['id_shop'], $id_lang);
			$prod['accounted'] = 1;
		}
		return $prods;
	}
	
	public static function getParentCategory($id_cat, $id_shop, $id_lang) {
		static $parent = array();
		static $tree = null;
		
		$index = $id_cat.'_'.$id_shop.'_'.$id_lang;
		if (!isset($parent[$index]))
		{
			if (!$tree)
			{
				$buf = Db::getInstance()->executeS('
					SELECT
						c.id_category AS id_cat,
						c.level_depth AS depth,
						c.id_parent AS id_parent,
						cl.id_shop AS id_shop,
						cl.id_lang AS id_lang,
						cl.name AS name
					FROM '._DB_PREFIX_.'category c
					JOIN '._DB_PREFIX_.'category_lang cl ON cl.id_category = c.id_category');
				$tree = array();
				foreach($buf as $cat)
					$tree[$cat['id_cat'].'_'.$cat['id_shop'].'_'.$cat['id_lang']] = $cat;
			}
			if (isset($tree[$index]))
			{
				$cat = $tree[$index];
				while ($cat['depth'] > 2) {
					$cat = $tree[$cat['id_parent'].'_'.$id_shop.'_'.$id_lang];
				}
				$parent[$index] = $cat['name'];
			}
			else
				$parent[$index] = '-';
		}
		return $parent[$index];
	} 
	
	public static function prods($db, $op_filter)
	{
		return self::completeProducts(array_merge(
			self::orderProducts($db, $op_filter),
			self::orderShippings($db, $op_filter),
			self::orderWrappings($db, $op_filter),
			self::slipProducts($db, $op_filter),
			self::slipShippings($db, $op_filter)
		));
	}
	
	public static function orderProducts($db, $op_filter)
	{
		$id_lang = Context::getContext()->language->id;

		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				o.reference AS ref,
				o.id_shop AS id_shop,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping AS shipping,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_excl AS shipping_te,
				CONCAT(c.firstname, " ", c.lastname) AS cust,
				gl.name AS `cust_group`,
				'.self::productInfo($id_lang).',
				IF(cs.oper = "ORDER", 1, -1)*od.unit_price_tax_excl*(1 - IFNULL(ok.product_global_discount, 0)) AS unit,
				IF(cs.oper = "ORDER", od.product_quantity, -od.product_quantity) AS qty,
				IF(cs.oper = "ORDER", 1, -1)*od.total_price_tax_excl*(1 - IFNULL(ok.product_global_discount, 0)) AS tax_excl,
				IF(cs.oper = "ORDER", 1, -1)*od.total_price_tax_incl*(1 - IFNULL(ok.product_global_discount, 0)) AS tax_incl,
				IF(cs.oper = "ORDER", 1, -1) * ( (od.original_product_price * od.product_quantity) - od.total_price_tax_excl*(1 - IFNULL(ok.product_global_discount, 0)) ) AS total_disc_te,
				IF(cs.oper = "ORDER", 1, -1) * ( (od.original_product_price * od.product_quantity) - od.total_price_tax_excl*(1 - IFNULL(ok.product_global_discount, 0)) ) * od.total_price_tax_incl / od.total_price_tax_excl AS total_disc_ti,
				(
					SELECT GROUP_CONCAT(op.payment_method) 
					FROM '._DB_PREFIX_.'order_payment op
					WHERE op.order_reference = o.reference AND op.amount > 0
					GROUP BY op.order_reference
				) AS payment_method,
				ok.is_paid,
				ok.delivery_mode
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
			JOIN '._DB_PREFIX_.'order_detail od ON od.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			LEFT JOIN '._DB_PREFIX_.'group_lang gl ON gl.id_group = c.id_default_group AND gl.id_lang = '. (int) $id_lang .'
			'.self::productJoin($id_lang).'
			WHERE cs.oper IN ("ORDER", "CANCEL")
			GROUP BY cs.id_cashdrawer_sale, od.id_order_detail');
	}
	
	public static function orderShippings($db, $op_filter)
	{
		$context = Context::getContext();
		$id_lang = $context->language->id;
		$shipping = $context->module->l('Shipping', pathinfo(__FILE__, PATHINFO_FILENAME));
	
		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				o.reference AS ref,
				o.id_shop AS id_shop,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping AS shipping,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_excl AS shipping_te,
				CONCAT(c.firstname, " ", c.lastname) AS cust,
				gl.name AS `cust_group`,
				NULL AS id_prod,
				NULL AS id_attr,
				NULL AS id_cat,
				"'.$shipping.'" AS prod,
				col.name AS order_country,
				car.name AS carrier,
				0.0 AS purchase_price,
				car.name AS version,
				NULL AS cat,
				NULL AS ref,
                NULL AS main_ref,
				NULL AS ean,
				NULL as upc,
				0.0 AS wholesale,
				NULL AS supplier,
				NULL AS id_supplier,
				NULL AS manufacturer,
				NULL AS id_manufacturer,
				NULL AS note,
                NULL AS id_tag,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_excl AS unit,
				IF(cs.oper = "ORDER", 1, -1) AS qty,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_excl*(1 - IFNULL(ok.free_shipping, 0)) AS tax_excl,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_incl*(1 - IFNULL(ok.free_shipping, 0)) AS tax_incl,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_excl*IFNULL(ok.free_shipping, 0) AS total_disc_te,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_incl*IFNULL(ok.free_shipping, 0) AS total_disc_ti,
				(
					SELECT GROUP_CONCAT(op.payment_method)
					FROM '._DB_PREFIX_.'order_payment op
					WHERE op.order_reference = o.reference AND op.amount > 0
					GROUP BY op.order_reference
				) AS payment_method,
				ok.is_paid
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			LEFT JOIN '._DB_PREFIX_.'group_lang gl ON gl.id_group = c.id_default_group AND gl.id_lang = '. (int) $id_lang .'
			LEFT JOIN '._DB_PREFIX_.'address addr ON o.id_address_delivery = addr.id_address
			LEFT JOIN '._DB_PREFIX_.'country_lang col ON addr.id_country = col.id_country AND col.id_lang = '.(int) $id_lang .'
			LEFT JOIN '._DB_PREFIX_.'carrier car ON o.id_carrier = car.id_carrier
			WHERE cs.oper IN ("ORDER", "CANCEL") AND o.total_shipping_tax_excl > 0
			GROUP BY cs.id_cashdrawer_sale');
	}
	
	public static function orderWrappings($db, $op_filter)
	{
		$context = Context::getContext();
		$id_lang = $context->language->id;
		$wrapping = $context->module->l('Wrapping', pathinfo(__FILE__, PATHINFO_FILENAME));
	
		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				o.reference AS ref,
				o.id_shop AS id_shop,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping AS shipping,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_excl AS shipping_te,
				CONCAT(c.firstname, " ", c.lastname) AS cust,
				gl.name AS `cust_group`,
				NULL AS id_prod,
				NULL AS id_attr,
				NULL AS id_cat,
				"'.$wrapping.'" AS prod,
				col.name AS order_country,
				car.name AS carrier,
				0.0 AS purchase_price,
				NULL AS version,
				NULL AS cat,
				NULL AS ref,
                NULL AS main_ref,
				NULL AS ean,
				NULL as upc,
				0.0 AS wholesale,
				NULL AS supplier,
				NULL AS id_supplier,
				NULL AS manufacturer,
				NULL AS id_manufacturer,
				NULL AS note,
                NULL AS id_tag,
				IF(cs.oper = "ORDER", 1, -1)*o.total_wrapping_tax_excl AS unit,
				IF(cs.oper = "ORDER", 1, -1) AS qty,
				IF(cs.oper = "ORDER", 1, -1)*o.total_wrapping_tax_excl AS tax_excl,
				IF(cs.oper = "ORDER", 1, -1)*o.total_wrapping_tax_incl AS tax_incl,
				IF(cs.oper = "ORDER", 1, -1)*0.0 AS total_disc_te,
				IF(cs.oper = "ORDER", 1, -1)*0.0 AS total_disc_ti,
				(
					SELECT GROUP_CONCAT(op.payment_method)
					FROM '._DB_PREFIX_.'order_payment op
					WHERE op.order_reference = o.reference AND op.amount > 0
					GROUP BY op.order_reference
				) AS payment_method,
				ok.is_paid
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			LEFT JOIN '._DB_PREFIX_.'group_lang gl ON gl.id_group = c.id_default_group AND gl.id_lang = '. (int) $id_lang .'
			LEFT JOIN '._DB_PREFIX_.'address addr ON o.id_address_delivery = addr.id_address
			LEFT JOIN '._DB_PREFIX_.'country_lang col ON addr.id_country = col.id_country AND col.id_lang = '. (int) $id_lang .'
			LEFT JOIN '._DB_PREFIX_.'carrier car ON o.id_carrier = car.id_carrier
			WHERE cs.oper IN ("ORDER", "CANCEL") AND o.total_wrapping_tax_incl > 0
			GROUP BY cs.id_cashdrawer_sale');
	}
	
	public static function slipProducts($db, $op_filter)
	{
		$id_lang = Context::getContext()->language->id;
		
		$select_amounts = '
			-sd.unit_price_tax_excl AS unit,
			-sd.product_quantity AS qty,
			-sd.total_price_tax_excl AS tax_excl,
			-sd.total_price_tax_incl AS tax_incl,';
		
		// Backward compatibility
		if (Tools::version_compare(_PS_VERSION_, '1.6.0.9', '<='))
			$select_amounts = '
				-(sd.amount_tax_excl/sd.product_quantity) AS unit,
				-sd.product_quantity AS qty,
				-sd.amount_tax_excl AS tax_excl,
				-sd.amount_tax_incl AS tax_incl,';
		
		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				cs.id_order_slip AS id_slip,
				o.reference AS ref,
				o.id_shop AS id_shop,
				o.total_shipping AS shipping,
				CONCAT(c.firstname, " ", c.lastname) AS cust,
				gl.name AS `cust_group`,
				'.self::productInfo($id_lang).',
				'.$select_amounts.'
				0.0 AS total_disc_te,
				0.0 AS total_disc_ti,
				(
					SELECT GROUP_CONCAT(op.payment_method) 
					FROM '._DB_PREFIX_.'order_payment op
					WHERE op.order_reference = o.reference AND op.amount > 0
					GROUP BY op.order_reference
				) AS payment_method,
				ok.is_paid
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'order_slip s ON s.id_order_slip = cs.id_order_slip
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = s.id_order
			JOIN '._DB_PREFIX_.'order_slip_detail sd ON sd.id_order_slip = s.id_order_slip
			JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = sd.id_order_detail
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			LEFT JOIN '._DB_PREFIX_.'group_lang gl ON gl.id_group = c.id_default_group AND gl.id_lang = '. (int) $id_lang .'
			'.self::productJoin($id_lang).'
			WHERE cs.oper = "SLIP"
			GROUP BY sd.id_order_slip, sd.id_order_detail');
	}
	
	public static function slipShippings($db, $op_filter)
	{
		$context = Context::getContext();
		$id_lang = $context->language->id;
		$shipping = $context->module->l('Shipping', pathinfo(__FILE__, PATHINFO_FILENAME));

		$select_amounts = '
				-s.total_shipping_tax_excl AS unit,
				-1 AS qty,
				-s.total_shipping_tax_excl AS tax_excl,
				-s.total_shipping_tax_incl AS tax_incl';
		$condition = 's.total_shipping_tax_excl > 0';
		
		// Backward compatibility
		if (Tools::version_compare(_PS_VERSION_, '1.6.0.9', '<='))
		{
			$select_amounts = '
				-s.shipping_cost_amount/(1 + o.carrier_tax_rate/100) AS unit,
				-1 AS qty,
				-s.shipping_cost_amount/(1 + o.carrier_tax_rate/100) AS tax_excl,
				-s.shipping_cost_amount AS tax_incl';
			$condition = 's.shipping_cost_amount > 0';
		}
		
		return $db->executeS('
			SELECT
				'.self::saleInfo().',
				cs.id_order AS id_order,
				o.reference AS ref,
				o.id_shop AS id_shop,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping AS shipping,
				IF(cs.oper = "ORDER", 1, -1)*o.total_shipping_tax_excl AS shipping_te,
				CONCAT(c.firstname, " ", c.lastname) AS cust,
				gl.name AS `cust_group`,
				NULL AS id_prod,
				NULL AS id_attr,
				NULL AS id_cat,
				"'.$shipping.'" AS prod,
				col.name AS order_country,
				car.name AS carrier,
				0.0 AS purchase_price,
				car.name AS version,
				NULL AS cat,
				NULL AS ref,
                NULL AS main_ref,
				NULL AS ean,
				NULL as upc,
				0.0 AS wholesale,
				NULL AS supplier,
				NULL AS id_supplier,
				NULL AS manufacturer,
				NULL AS id_manufacturer,
				NULL AS note,
                NULL AS id_tag,
				'.$select_amounts.',
				-o.total_shipping_tax_excl*IFNULL(ok.free_shipping, 0) AS total_disc_te,
				-o.total_shipping_tax_incl*IFNULL(ok.free_shipping, 0) AS total_disc_ti,
				(
					SELECT GROUP_CONCAT(op.payment_method)
					FROM '._DB_PREFIX_.'order_payment op
					WHERE op.order_reference = o.reference AND op.amount > 0
					GROUP BY op.order_reference
				) AS payment_method,
				ok.is_paid
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'order_slip s ON s.id_order_slip = cs.id_order_slip
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = s.id_order
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
			LEFT JOIN '._DB_PREFIX_.'group_lang gl ON gl.id_group = c.id_default_group AND gl.id_lang = '. (int) $id_lang .'
			LEFT JOIN '._DB_PREFIX_.'address addr ON o.id_address_delivery = addr.id_address
			LEFT JOIN '._DB_PREFIX_.'country_lang col ON addr.id_country = col.id_country AND col.id_lang = '. (int) $id_lang.'
			LEFT JOIN '._DB_PREFIX_.'carrier car ON o.id_carrier = car.id_carrier
			WHERE cs.oper = "SLIP" AND '.$condition.'
			GROUP BY cs.id_cashdrawer_sale');
	}
	
	
	public static function prodStats($db, $op_filter)
	{
		// completeProducts applied on main shop
		return self::completeProducts(array_merge(
			self::orderProductsStats($db, $op_filter),
			self::slipProductsStats($db, $op_filter)
		), Configuration::get('PS_SHOP_DEFAULT'));
	}


	public static function orderProductsStats($db, $op_filter)
	{
		$id_lang = Context::getContext()->language->id;
		
		return $db->executeS('
			SELECT
				'.self::productInfo($id_lang).',
				SUM(IF(cs.oper = "ORDER", od.product_quantity, -od.product_quantity)) AS qty,
				SUM(IF(cs.oper = "ORDER", 1, -1)*od.total_price_tax_excl*(1 - IFNULL(ok.product_global_discount, 0))) AS tax_excl,
				SUM(IF(cs.oper = "ORDER", 1, -1)*od.total_price_tax_incl*(1 - IFNULL(ok.product_global_discount, 0))) AS tax_incl
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = cs.id_order
			JOIN '._DB_PREFIX_.'order_detail od ON od.id_order = o.id_order
			LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
			'.self::productJoin($id_lang).'
			WHERE
				cs.oper IN ("ORDER", "CANCEL")
			GROUP BY p.id_product, pa.id_product_attribute');
	}
	
	public static function slipProductsStats($db, $op_filter)
	{
		$id_lang = Context::getContext()->language->id;
		
		$select_amounts = '
			-SUM(sd.total_price_tax_excl) AS tax_excl,
			-SUM(sd.total_price_tax_incl) AS tax_incl';
		
		// Backward compatibility
		if (Tools::version_compare(_PS_VERSION_, '1.6.0.9', '<='))
			$select_amounts = '
				-SUM(sd.amount_tax_excl) AS tax_excl,
				-SUM(sd.amount_tax_incl) AS tax_incl';
		
		return $db->executeS('
			SELECT
				'.self::productInfo($id_lang).',
				-SUM(sd.product_quantity) AS qty,
				'.$select_amounts.'
			FROM '._DB_PREFIX_.'cashdrawer_sale_kerawen cs
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cs.id_cashdrawer_op AND '.$op_filter.'
			JOIN '._DB_PREFIX_.'order_slip s ON s.id_order_slip = cs.id_order_slip
			JOIN '._DB_PREFIX_.'orders o ON o.id_order = s.id_order
			JOIN '._DB_PREFIX_.'order_slip_detail sd ON sd.id_order_slip = s.id_order_slip
			JOIN '._DB_PREFIX_.'order_detail od ON od.id_order_detail = sd.id_order_detail
			'.self::productJoin($id_lang).'
			WHERE
				cs.oper = "SLIP"
			GROUP BY p.id_product, pa.id_product_attribute');
	}

	public static function flows($db, $op_filter)
	{
		return $db->executeS('
			SELECT
				CONCAT("_", cf.id_cashdrawer_op) AS id_op,
				cf.id_cashdrawer_flow AS id_flow,
				cf.id_order AS id_order,
				cf.id_order_slip AS id_slip,
				cf.id_order_payment AS id_payment,
				cf.id_payment_mode AS id_mode,
				IF(cf.id_payment_mode IS NULL, op.payment_method, NULL) AS mode,
				cf.amount AS amount,
				cf.count AS count,
				cf.date_deferred AS deferred,
				cf.comments AS note
			FROM '._DB_PREFIX_.'cashdrawer_flow_kerawen cf
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cf.id_cashdrawer_op AND '.$op_filter.'
			LEFT JOIN '._DB_PREFIX_.'order_payment op ON op.id_order_payment = cf.id_order_payment
			');
	}

	public static function checks($db, $op_filter)
	{
		return $db->executeS('
			SELECT
				CONCAT("_", cc.id_cashdrawer_op) AS id_op,
				cc.id_cashdrawer_close AS id_check,
				cc.id_payment_mode AS id_mode,
				cc.checked AS checked,
				cc.error AS error
			FROM '._DB_PREFIX_.'cashdrawer_close_kerawen cc
			JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen co ON co.id_cashdrawer_op = cc.id_cashdrawer_op AND '.$op_filter);
	}
	
	
	

	public static function ordermvts($db, $params)
	{

		$q = "
			SELECT				
				'order' AS act, order_detail.id_order, order_detail.id_order_detail, order_detail.product_id, order_detail.product_attribute_id, order_detail.product_name, order_detail.product_quantity AS qty_buy, order_detail.product_quantity_in_stock AS qty_in_stock,
				SUM(stock.physical_quantity) AS phys_total, GROUP_CONCAT(stock.physical_quantity SEPARATOR '+') AS phys_concat, SUM(stock.usable_quantity) AS usable_total, GROUP_CONCAT(stock.usable_quantity SEPARATOR '+') AS usable_concat,
				stock_available.real_total, stock_available.real_concat,
				stock_mvt.id_stock_mvt, stock_mvt.physical_quantity AS qty_ordered, stock_mvt.sign, orders.date_add AS date_added,
				stock_mvt_reason_lang.name AS reason
			
			FROM " . _DB_PREFIX_ . "order_detail order_detail
			LEFT JOIN " . _DB_PREFIX_ . "stock stock ON order_detail.product_id = stock.id_product AND order_detail.product_attribute_id = stock.id_product_attribute
			LEFT JOIN " . _DB_PREFIX_ . "stock_mvt stock_mvt ON order_detail.id_order = stock_mvt.id_order AND stock.id_stock = stock_mvt.id_stock
			LEFT JOIN " . _DB_PREFIX_ . "stock_mvt_reason stock_mvt_reason ON stock_mvt.id_stock_mvt_reason = stock_mvt_reason.id_stock_mvt_reason
			LEFT JOIN " . _DB_PREFIX_ . "stock_mvt_reason_lang stock_mvt_reason_lang ON stock_mvt_reason.id_stock_mvt_reason = stock_mvt_reason_lang.id_stock_mvt_reason
			LEFT JOIN " . _DB_PREFIX_ . "orders orders ON order_detail.id_order = orders.id_order
			
			LEFT JOIN (
			
				SELECT stock_available.id_product, stock_available.id_product_attribute, SUM(stock_available.quantity) AS real_total, GROUP_CONCAT(stock_available.quantity SEPARATOR '+') AS real_concat
				FROM " . _DB_PREFIX_ . "stock_available stock_available
				GROUP BY stock_available.id_product, stock_available.id_product_attribute
			
			) AS stock_available ON order_detail.product_id = stock_available.id_product AND order_detail.product_attribute_id = stock_available.id_product_attribute
			
			
			GROUP BY id_order_detail
			
			HAVING orders.date_add BETWEEN '" . pSQL($params->from) . "' AND '" . pSQL($params->to) . "'
			
			ORDER BY date_added DESC, id_order_detail DESC
		
		";


		$res['ordermvts'] = $db->executeS($q);
		if ($params->index) {
			$res['ordermvts'] = indexArray($res['ordermvts'], 'id_stock_mvt');
		}

		return $res['ordermvts'];

	}


	public static function mvts($db, $params)
	{

		$q = "

			SELECT
				stock_mvt.id_order AS id_order, order_detail.id_order_detail AS id_order_detail, stock.id_product_attribute AS product_attribute_id, stock.id_product AS product_id, order_detail.product_name AS product_name,
				stock.id_stock, stock.id_warehouse, stock.physical_quantity AS physical_quantity_2, stock.usable_quantity,
				stock_mvt.id_stock_mvt, stock_mvt.physical_quantity AS physical_quantity_1, stock_mvt.sign, stock_mvt.date_add AS date_added,
				stock_mvt_reason_lang.name AS reason
			FROM " . _DB_PREFIX_ . "stock_mvt stock_mvt
			LEFT JOIN " . _DB_PREFIX_ . "stock_mvt_reason stock_mvt_reason ON stock_mvt.id_stock_mvt_reason = stock_mvt_reason.id_stock_mvt_reason
			LEFT JOIN " . _DB_PREFIX_ . "stock_mvt_reason_lang stock_mvt_reason_lang ON stock_mvt_reason.id_stock_mvt_reason = stock_mvt_reason_lang.id_stock_mvt_reason
			LEFT JOIN " . _DB_PREFIX_ . "stock stock ON stock_mvt.id_stock = stock.id_stock
			LEFT JOIN " . _DB_PREFIX_ . "order_detail order_detail ON stock_mvt.id_order = order_detail.id_order AND stock.id_product = order_detail.product_id AND stock.id_product_attribute = order_detail.product_attribute_id
		
			WHERE stock_mvt.date_add BETWEEN '" . pSQL($params->from) . "' AND '" . pSQL($params->to) . "'
			ORDER BY stock_mvt.date_add DESC, id_order_detail DESC
		
		";
		

		$res['mvts'] = $db->executeS($q);
		if ($params->index) {
			$res['mvts'] = indexArray($res['mvts'], 'id_stock_mvt');
		}

		return $res['mvts'];

	}
	
	
	
	public static function ordersales_qty($filter, $from = false, $to = false) {
	    
	    $period = ($from  && $to) ? ' AND ord.date_add BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'" ' : '';
	    
	    return '
    	    SELECT
    	       od.product_id,
    	       od.product_attribute_id,
    	       SUM(IFNULL(od.product_quantity, 0)) - SUM(IFNULL(od.product_quantity_refunded, 0)) - SUM(IFNULL(od.product_quantity_return, 0)) AS qty,
    	       SUM((CAST(IFNULL(od.product_quantity, 0) AS SIGNED) - CAST(IFNULL(od.product_quantity_refunded, 0) AS SIGNED) - CAST(IFNULL(od.product_quantity_return, 0) AS SIGNED)) / od.product_quantity * od.total_price_tax_excl ) AS total_tax_excl
    	    FROM '._DB_PREFIX_.'order_detail od
    	    LEFT JOIN '._DB_PREFIX_.'orders `ord` ON ord.id_order = od.id_order
    	    WHERE ' . $filter . ' AND ord.current_state != 6 ' . $period . ' 
    	    GROUP BY od.product_id, od.product_attribute_id
	    ';
	}
	

	
	
	public static function ordersales($db, $params) {
	    
	    if (Tools::version_compare(_PS_VERSION_, '1.7', '<')) {
    	    $whs = Warehouse::getWarehouses(false, (int) $params->shop);
    	    if (count($whs) > 0) {
    	        return self::ordersales_legacy($db, $params);
    	    }
	    }
	    
	    $bypage = (int) self::bypage();
	    
	    $page = 1;
	    if (!empty($params->page)) {
	        $page = (int) $params->page;
	    }
	    
	    $from = new DateTime($params->from);
	    $to = new DateTime($params->to);
	    $diff = $to->diff($from);
	    $days = $diff->days + 1;

	    //single shop required
	    if (isset($params->shop)) {
	        if (strpos($params->shop, ',')) {
	            return array(
	                'data' => array(),
	                'total' => 0,
	                'bypage' => $bypage
	            );
	        }
	    }

	    $id_shop = (int) $params->shop;
	    $id_lang = Context::getContext()->language->id;
	    $id_country = Context::getContext()->country->id;
	    	   
	    $shop = new Shop($id_shop);
	    $shopGroup = new ShopGroup($shop->id_shop_group);
	    if ($shopGroup->share_stock) {
	        $stock_filter = 'sa.id_shop = 0 AND sa.id_shop_group = ' . (int) $shopGroup->id;
	    } else {
	        $stock_filter = 'sa.id_shop_group = 0 AND sa.id_shop = ' . (int) $id_shop;
	    }
	    
	    $where = array();
	    if (isset($params->supplier)) {
	        if ($params->supplier > 0) {
	            $where[] = 'p.id_supplier = ' . (int) $params->supplier;
	        }
	    }
	    
	    if (isset($params->manufacturer)) {
	        if ($params->manufacturer > 0) {
	            $where[] = 'p.id_manufacturer = ' . (int) $params->manufacturer;
	        }
	    }
	    
	    if (isset($params->pr_status)) {
	        if ((int) $params->pr_status === 0 || (int) $params->pr_status === 1) {
	            $where[] = 'ps.active = ' . (int) $params->pr_status;
	        }
	    }
	    
	    $buying_price = '(ROUND(IF(pas.wholesale_price IS NULL OR pas.wholesale_price = 0, IFNULL(ps.wholesale_price, 0), IFNULL(pas.wholesale_price,0)), 2))';
	    $stock = 'IFNULL(sa.quantity, 0)';
	    $valorization = $buying_price  . ' * GREATEST(' . $stock  . ', 0) AS valorization';
	    
	    $order_filter = 'ord.id_shop = ' . pSQL($id_shop);
	    
	    $join_o = '
            LEFT JOIN (
	           ' . self::ordersales_qty($order_filter, $params->from, $params->to) . '
	        ) AS o ON o.product_id = p.id_product AND o.product_attribute_id = IFNULL(pas.id_product_attribute, 0) 
        ';

	    $join_o2 = 'LEFT JOIN (
	           ' . self::ordersales_qty($order_filter, false, false) . '
	        ) AS o2 ON o2.product_id = p.id_product AND o2.product_attribute_id = IFNULL(pas.id_product_attribute, 0)
	    ';
	    
	    $fields = '';
	    $join = '';
	    
	    switch ($params->id_tab) {
	        case 'getProdSales':
	            $fields .= ', ' . 'IFNULL(o.qty, 0) AS qty_sale';
	            $fields .= ', ' . 'IF(COALESCE(o.qty, 0) > 0, GREATEST(CEIL(' . $days . ' * (' . $stock . ') / o.qty), 0), NULL) AS projection';
	            $join .= $join_o;
	            break;
	        case 'getProdValo':
	            $fields .= ', ' . $valorization;
	            break;
	        case 'getProdStock':
	            $low_stock = 'IFNULL(pas.low_stock_threshold, ps.low_stock_threshold)';
	            $to_order = '(' . $low_stock . ' - ' . $stock . ')';
	            
	            $fields .= ', ' . $low_stock . ' AS low_stock';
	            $fields .= ', ' . 'IF('. $to_order . ' > 0, ' . $to_order . ', NULL) AS to_order';
	            break;
	        case 'getProdSaleBuy':
	            //$fields .= ', ' . 'IFNULL(o2.qty, 0) AS qty_sale_total';
	            $fields .= ', ' . 'IF(o2.qty = 0 OR ISNULL(o2.qty) OR ISNULL(o.qty), 0, o.qty / o2.qty * 100) AS qty_sale_rate';
	            $fields .= ', ' . 'IFNULL(o.qty, 0) AS qty_sale';
	            $fields .= ', ' . 'IFNULL(o.total_tax_excl, 0) AS saleAmount';
	            $fields .= ', ' . 'm.buyQty AS buyQty';
	            $fields .= ', ' . $buying_price . ' * IFNULL(o.total_tax_excl, 0) AS buyAmount';
	            $fields .= ', ' . 'IF(m.buyQty = 0 OR ISNULL(m.buyQty) OR ISNULL(o2.qty), 0, o2.qty / m.buyQty * 100) AS tx_ca';
	            
	            $join .= '
    	            LEFT JOIN (
    	                SELECT
        	                id_stock,
        	                SUM(CAST(physical_quantity  AS SIGNED) * `sign`) AS buyQty
    	                FROM '._DB_PREFIX_.'stock_mvt FORCE INDEX (id_stock)
    	                WHERE id_stock_mvt_reason IN (' . Configuration::get('KERAWEN_MVT_BUY') . ')
    	                GROUP BY id_stock
    	            ) m ON m.id_stock = sa.id_stock_available
	            ';
	            $join .= $join_o;
	            $join .= $join_o2;

	            break;
	    }

	    $order = self::setOrder($params->orderby, $params->sortby);
	    if ($order == '') { $order = 'ORDER BY ps.id_product DESC, pas.id_product_attribute'; }
	    
	    $q = '
            SELECT
	        
                SQL_CALC_FOUND_ROWS
	        
            	ps.id_product AS id,
                pl.name AS product,
                ppac.decli AS declinaison,
	            IFNULL(p.reference, pa.reference) AS ref,
                cl.name AS cat,
            	p.id_supplier AS id_supplier,
            	s.name AS supplier,
            	psu.product_supplier_reference AS supplier_ref,
            	p.id_manufacturer AS id_manufacturer,
            	m.name AS manufacturer,
	        
            	IFNULL(pa.ean13, p.ean13) AS ean13,
            	IFNULL(pa.reference, p.reference) AS ref,
	        
                ROUND((IFNULL(ps.price, 0) + IFNULL(pas.price, 0)), 2) AS saling_price,
                ' . $buying_price . ' AS  buying_price,
                ' .  $stock . ' AS stock
                ' . $fields . '
                    
            FROM  '._DB_PREFIX_.'product_shop ps
            INNER JOIN '._DB_PREFIX_.'product p ON p.id_product = ps.id_product AND ps.id_shop = ' . (int) $id_shop . '
                
            LEFT JOIN '._DB_PREFIX_.'product_lang pl ON pl.id_product = ps.id_product AND pl.id_lang = ' . (int) $id_lang . ' AND pl.id_shop = '. (int) $id_shop . '
            LEFT JOIN '._DB_PREFIX_.'product_attribute_shop pas ON ps.id_product = pas.id_product AND pas.id_shop = ' . (int) $id_shop . '
            LEFT JOIN '._DB_PREFIX_.'product_attribute pa ON pas.id_product_attribute = pa.id_product_attribute
                
            LEFT JOIN (
                SELECT
            		GROUP_CONCAT(al.name ORDER BY a.id_attribute_group) AS decli,
                    pac.id_product_attribute
            	FROM '._DB_PREFIX_.'product_attribute_combination pac
            	LEFT JOIN '._DB_PREFIX_.'attribute a ON pac.id_attribute = a.id_attribute
            	LEFT JOIN '._DB_PREFIX_.'attribute_lang al ON al.id_attribute = a.id_attribute AND al.id_lang = ' . (int) $id_lang . '
            	GROUP BY pac.id_product_attribute
            ) AS ppac ON ppac.id_product_attribute = pas.id_product_attribute
            	    
            LEFT JOIN '._DB_PREFIX_.'stock_available sa 
                ON sa.id_product = ps.id_product 
                AND IFNULL(pas.id_product_attribute, 0) = sa.id_product_attribute 
                AND ' . $stock_filter . '
                
            LEFT JOIN '._DB_PREFIX_.'category c ON c.id_category = p.id_category_default
            LEFT JOIN '._DB_PREFIX_.'category_lang cl ON cl.id_category = c.id_category AND cl.id_lang = ' . (int) $id_lang . ' AND cl.id_shop = '. (int) $id_shop .'
                
            LEFT JOIN '._DB_PREFIX_.'supplier s ON s.id_supplier = p.id_supplier
            LEFT JOIN '._DB_PREFIX_.'product_supplier psu
            	ON psu.id_supplier = s.id_supplier
            	AND psu.id_product = p.id_product
            	AND (psu.id_product_attribute = ppac.id_product_attribute OR (ppac.id_product_attribute IS NULL AND psu.id_product_attribute = 0))
                
            LEFT JOIN '._DB_PREFIX_.'manufacturer m ON m.id_manufacturer = p.id_manufacturer
            ' . $join . '
            WHERE ps.id_shop = '. (int) $id_shop . '
            ' . (count($where) ? ' AND ' . implode(' AND ', $where) : '') . '
            ' . $order . '
            ' . (($bypage > 0) ? (' LIMIT ' . ( ((int) $page - 1) * (int) $bypage) . ', ' . (int) $bypage) : '');

	    /*
	     echo '<pre>';
	     echo $q;
	     echo '</pre>';
	     */
	    
	    $data = $db->executeS($q);
	    $total = $db->getValue('SELECT FOUND_ROWS()');
	    
	    $res = array(
	        'data' => $data,
	        'total' => $total,
	        'bypage' => $bypage
	    );
	    
	    return $res;
	}
	
	
	
	public static function ordersales_legacy($db, $params)
	{

	    $bypage = (int) self::bypage();
	    
	    $page = 1;
	    if (!empty($params->page)) {
	        $page = (int) $params->page;
	    }
	    
	    $from = new DateTime($params->from);
	    $to = new DateTime($params->to);
	    $diff = $to->diff($from);
	    $days = $diff->days + 1;
	    
	    
	    if (isset($params->shop)) {
	        if (strpos($params->shop, ',')) {
	            return array(
	                'data' => array(),
	                'total' => 0,
	                'bypage' => $bypage
	            );
	        }
	    }
	    
		
		$id_shop = $params->shop;
		
		$id_lang = Context::getContext()->language->id;
		// TODO remove: prices may be different from one shop to another
		$id_country = Context::getContext()->country->id;
		
		$order_filter = 'TRUE';
		$wh_filter = 'TRUE';
		$sa_filter = null;
		
		
		$virtual_shop = 0;
		if (Module::isInstalled('kerawen_ocs') && Module::isEnabled('kerawen_ocs')) {
		    $virtual_shop = (int) $db->getValue('SELECT id_shop FROM '._DB_PREFIX_.'kerawen_cnc_shop_type WHERE type = "virtual"');
		}

		if ($id_shop != -1) {
		    $order_filter = 'ord.id_shop = '.pSQL($id_shop);
			
		    $group = Shop::getGroupFromShop($id_shop, false);
			if ($group['share_stock']) {
				$sa_filter = 'sa.id_shop = 0 AND sa.id_shop_group = '. (int) $group['id'];
			}
			else {
			    $sa_filter = 'sa.id_shop_group = 0 AND sa.id_shop = '. (int) $id_shop;
			}
			
			//vistual stock => no stock
			if ($virtual_shop && (int) $id_shop == (int) $virtual_shop) {
			    $sa_filter = 'FALSE';
			    //$sa_filter = ' sa.id_shop NOT IN ('.$virtual_shop.') ';
			}
			
			
			$ids = array();
			$whs = Warehouse::getWarehouses(false, $id_shop);
			foreach($whs as $wh) $ids[] = (int) $wh['id_warehouse'];
			if (count($ids)) {
				$wh_filter = 'id_warehouse IN ('.implode(',', $ids).')';
			}
			else {
				$wh_filter = 'id_warehouse = 0';
			}
		}
		else {
		    $id_shop = Configuration::get('PS_SHOP_DEFAULT');
		    
			// Filter stock available from shop group configuration
		    $gids = array();
		    $sids = array();
			$groups = ShopGroup::getShopGroups();
			foreach ($groups as $g) {
				if ($g->share_stock) {
					$gids[] = (int) $g->id;
				}
				else {
					$shops = ShopGroup::getShopsFromGroup($g->id);
					foreach ($shops as $s) $sids[] = (int) $s['id_shop'];
				}
			}
			
			$g_filter = 'FALSE';
			$s_filter = 'FALSE';
			
			if ($virtual_shop) {
			     $gids = array_diff($gids, array($virtual_shop));
			     $sids = array_diff($sids, array($virtual_shop));
			     $g_filter = ' sa.id_shop NOT IN ('.$virtual_shop.') ';
			     $s_filter = ' sa.id_shop NOT IN ('.$virtual_shop.') ';
			}
			
			if (count($gids)) $g_filter = 'sa.id_shop = 0 AND sa.id_shop_group IN ('.implode(',', $gids).')';
			if (count($sids)) $s_filter = 'sa.id_shop_group = 0 AND sa.id_shop IN ('.implode(',', $sids).')';
			$sa_filter = '('.$g_filter.' OR '.$s_filter.')';
		}

		

		$pa_low_stock = ' low_stock_threshold AS low_stock, ';
		if ($params->shop == -1) {
		    $low_stock = 'IFNULL(d.low_stock, p.low_stock_threshold)';
		    $join_low_stock = '';
		} else {
		    $low_stock = 'IFNULL(product_attribute_shop.low_stock_threshold, product_shop.low_stock_threshold)';
		    $join_low_stock = '
                LEFT JOIN ' . _DB_PREFIX_ . 'product_shop product_shop
		        ON p.id_product = product_shop.id_product AND product_shop.id_shop = ' . (int) $params->shop . '
		        LEFT JOIN ' . _DB_PREFIX_ . 'product_attribute_shop product_attribute_shop
		        ON d.id_product_attribute = product_attribute_shop.id_product_attribute AND product_attribute_shop.id_shop = ' . (int) $params->shop . '
            ';
		}

		$to_order = $low_stock . ' - (IF(ss.stock_shop > 0, ss.qty, 0) + IF(ss.stock_warehouse > 0, sw.qty, 0))';
		$low_stock_fields = $low_stock . ' AS low_stock,
                    IF('. $to_order . ' > 0, ' . $to_order . ', NULL) AS to_order
        ';


		$where = array();
		if (isset($params->supplier)) {
		    if ($params->supplier > 0) {
		        $where[] = 'p.id_supplier = ' . (int) $params->supplier;
		    }
		}

		if (isset($params->manufacturer)) {
		    if ($params->manufacturer > 0) {
		        $where[] = 'p.id_manufacturer = ' . (int) $params->manufacturer;
		    }
		}

		if (isset($params->pr_status)) {
		    if ((int) $params->pr_status === 0 || (int) $params->pr_status === 1) {
		        $where[] = 'psh.active = ' . (int) $params->pr_status;
		    }
		}

		$order = 'ORDER BY qty_sale DESC, p.id_product, d.id_product_attribute';
		if ( isset($params->orderby) && isset($params->sortby) ) {
		    if (!($params->orderby == '' || $params->sortby == '')) {
		        switch ($params->orderby) {
		            case 'id':
		                $order = 'ORDER BY p.id_product ' . $params->sortby;
		                break;
		            case 'prod_name':
		                $order = 'ORDER BY pl.name ' . $params->sortby;
		                break;
		            case 'declinaison':
		                $order = 'ORDER BY d.name ' . $params->sortby;
		                break;
		            case 'ean':
		                $order = 'ORDER BY ean ' . $params->sortby;
		                break;
		            case 'ref':
		                $order = 'ORDER BY ref ' . $params->sortby;
		                break;
		            case 'cat':
		                $order = 'ORDER BY cl.name ' . $params->sortby;
		                break;
		            case 'supplier':
		                $order = 'ORDER BY s.name ' . $params->sortby;
		                break;
		            case 'supplier_ref':
		                $order = 'ORDER BY ps.product_supplier_reference ' . $params->sortby;
		                break;
		            case 'manufacturer':
		                $order = 'ORDER BY m.name ' . $params->sortby;
		                break;
		            case 'saling_price':
		                $order = 'ORDER BY saling_price ' . $params->sortby;
		                break;
		            case 'unit_tax_incl':
		                $order = 'ORDER BY unit_tax_incl ' . $params->sortby;
		                break;
		            case 'stock_type':
		                $order = 'ORDER BY stock_type ' . $params->sortby;
		                break;
		            case 'buying_price':
		                $order = 'ORDER BY buying_price ' . $params->sortby;
		                break;
		            case 'stock':
		                $order = 'ORDER BY stock ' . $params->sortby;
		                break;
		            case 'projection':
		                $order = 'ORDER BY projection ' . $params->sortby;
		                break;
		            case 'valorization':
		                $order = 'ORDER BY valorization ' . $params->sortby;
		                break;
		            case 'low_stock':
		                $order = 'ORDER BY low_stock ' . $params->sortby;
		                break;
		            case 'to_order':
		                $order = 'ORDER BY to_order ' . $params->sortby;
		                break;
		            case 'qty_sale_rate':
		                $order = 'ORDER BY qty_sale_rate ' . $params->sortby;
		                break;
		            case 'qty_sale':
		                $order = 'ORDER BY qty_sale ' . $params->sortby;
		                break;
		            case 'saleAmount':
		                $order = 'ORDER BY saleAmount ' . $params->sortby;
		                break;
		            case 'buyQty':
		                $order = 'ORDER BY buyQty ' . $params->sortby;
		                break;
		            case 'buyAmount':
		                $order = 'ORDER BY buyAmount ' . $params->sortby;
		                break;
		            case 'tx_ca':
		                $order = 'ORDER BY tx_ca ' . $params->sortby;
		                break;
		        }
		    }
		}
		
		
		$displayfields = array();
		$displayfields[] = 'p.id_product AS id';
		$displayfields[] = 'IF(pl.name = "" OR ISNULL(pl.name), pl2.name, pl.name) AS prod_name';
		$displayfields[] = 'd.name AS declinaison';
		$displayfields[] = 'IFNULL(d.reference, p.reference) AS ref';
		
		//PHP calcul later
		$buying_price = 'IFNULL(IF(d.wholesale_price > 0, d.wholesale_price, psh.wholesale_price), 0)';

		
		switch ($params->id_tab) {
		    case 'getProdSaleBuy':
		        $displayfields[] = 'cl.name AS cat';
		        
		        $displayfields[] = $buying_price . ' AS buying_price';
		        $displayfields[] = 'IF(ss.stock_shop > 0, ss.value, 0) + IF(ss.stock_warehouse > 0, sw.value, 0) AS valorization';

		        //$displayfields[] = 'IFNULL(p.price, 0) + IFNULL(d.price, 0) AS saling_price';
		        $displayfields[] = 'IF(ss.stock_shop > 0, ss.qty, 0) + IF(ss.stock_warehouse > 0, sw.qty, 0) AS stock';
		        //$displayfields[] = 'IFNULL(o2.qty, 0) AS qty_sale_total';
		        $displayfields[] = 'IF(o2.qty = 0 OR ISNULL(o2.qty) OR ISNULL(o.qty), 0, o.qty / o2.qty * 100) AS qty_sale_rate';
		        $displayfields[] = 'IFNULL(o.qty, 0) AS qty_sale';
		        $displayfields[] = 'IFNULL(o.total_tax_excl, 0) AS saleAmount';
		        $displayfields[] = 'ss.buyQty AS buyQty';
		        $displayfields[] = $buying_price . ' * IFNULL(o.total_tax_excl, 0) AS buyAmount';
		        $displayfields[] = 'IF(ss.buyQty = 0 OR ISNULL(ss.buyQty) OR ISNULL(o2.qty), 0, o2.qty / ss.buyQty * 100) AS tx_ca';
		        break; 
		    default:
		        
		        $displayfields[] = $buying_price . ' AS buying_price';
		        $displayfields[] = $buying_price . ' * IFNULL(o.total_tax_excl, 0) AS buyAmount';
		        $displayfields[] = 'IF(ss.stock_shop > 0, ss.value, 0) + IF(ss.stock_warehouse > 0, sw.value, 0) AS valorization';
		        
		        $displayfields[] = 'IFNULL(d.ean13, p.ean13) AS ean';
		        $displayfields[] = 'p.id_category_default AS id_cat';
		        $displayfields[] = 'cl.name AS cat';
		        $displayfields[] = 'p.id_supplier AS id_supplier';
		        $displayfields[] = 's.name AS supplier';
		        $displayfields[] = 'ps.product_supplier_reference AS supplier_ref';
		        $displayfields[] = 'p.id_manufacturer AS id_manufacturer';
		        $displayfields[] = 'm.name AS manufacturer';
		        $displayfields[] = 'IFNULL(psh.price, 0) + IFNULL(d.price, 0) AS saling_price';
		        $displayfields[] = '
                    CASE
            		    WHEN ss.stock_shop IS NULL THEN "manual"
            		    WHEN ss.stock_warehouse = 0 THEN "manual"
            		    WHEN ss.stock_shop = 0 THEN "warehouse"
            		    ELSE "manual/warehouse"
    		        END AS stock_type';
    		    $displayfields[] = 'IF(ss.stock_shop > 0, ss.qty, 0) + IF(ss.stock_warehouse > 0, sw.qty, 0) AS stock';

		        switch ($params->id_tab) {
		            case 'getProdSales':
		                $displayfields[] = 'IF(COALESCE(o.qty, 0) > 0, GREATEST(CEIL(' . $days . ' * (IF(ss.stock_shop > 0, ss.qty, 0) + IF(ss.stock_warehouse > 0, sw.qty, 0)) / o.qty), 0), NULL) AS projection';
		                break;
		            case 'getProdValo':
		                break;
		            case 'getProdStock':
		                $displayfields[] = $low_stock_fields;
		                break;
		        }
		        
		        $displayfields[] = 'IFNULL(o.qty, 0) AS qty_sale';

		        break;
		}
		

		$q = '
			SELECT
                SQL_CALC_FOUND_ROWS
                DISTINCT
                ' . implode(",", $displayfields) . '
			FROM '._DB_PREFIX_.'product p
			LEFT JOIN '._DB_PREFIX_.'product_shop psh ON psh.id_product = p.id_product AND psh.id_shop = '. (int) $id_shop.'
			LEFT JOIN '._DB_PREFIX_.'product_lang pl ON pl.id_product = p.id_product AND pl.id_lang = '. (int) $id_lang.' AND pl.id_shop = '. (int) $id_shop.'
            LEFT JOIN '._DB_PREFIX_.'product_lang pl2 ON pl2.id_product = p.id_product AND pl2.id_lang = '. (int) $id_lang.' AND pl2.id_shop = p.id_shop_default		

			LEFT JOIN (
				SELECT 
					pa.id_product, 
                    pa.id_product_attribute,
					pa.ean13,
					pa.reference,
					pa.price,
					pa.wholesale_price,
                    ' . $pa_low_stock . '
					(SELECT
						GROUP_CONCAT(al.name ORDER BY a.id_attribute_group)
						FROM '._DB_PREFIX_.'product_attribute_combination pac
						LEFT JOIN '._DB_PREFIX_.'attribute a ON pac.id_attribute = a.id_attribute
						LEFT JOIN '._DB_PREFIX_.'attribute_lang al ON al.id_attribute = a.id_attribute AND al.id_lang = '. (int) $id_lang .' 
						WHERE pac.id_product_attribute = pa.id_product_attribute
					) AS name
				FROM '._DB_PREFIX_.'product_attribute pa
				GROUP BY pa.id_product, pa.id_product_attribute
			) AS d ON d.id_product = p.id_product
			
			LEFT JOIN '._DB_PREFIX_.'category c ON c.id_category = p.id_category_default 
			LEFT JOIN '._DB_PREFIX_.'category_lang cl ON cl.id_category = c.id_category AND cl.id_lang = '. (int) $id_lang .' AND cl.id_shop = '. (int) $id_shop .' 
			LEFT JOIN '._DB_PREFIX_.'supplier s ON s.id_supplier = p.id_supplier
            LEFT JOIN '._DB_PREFIX_.'product_supplier ps
				ON ps.id_supplier = s.id_supplier
				AND ps.id_product = p.id_product
				AND (ps.id_product_attribute = d.id_product_attribute OR (d.id_product_attribute IS NULL AND ps.id_product_attribute = 0))
			LEFT JOIN '._DB_PREFIX_.'manufacturer m ON m.id_manufacturer = p.id_manufacturer
			LEFT JOIN '._DB_PREFIX_.'tax_rule tr ON tr.id_tax_rules_group = p.id_tax_rules_group AND tr.id_country = '. (int) $id_country.'
			LEFT JOIN '._DB_PREFIX_.'tax t ON t.id_tax = tr.id_tax
            ' . $join_low_stock . '

			LEFT JOIN (
                ' . self::ordersales_qty($order_filter, $params->from, $params->to) . '
			) AS o ON o.product_id = p.id_product AND o.product_attribute_id = IFNULL(d.id_product_attribute, 0)

            LEFT JOIN (
                ' . self::ordersales_qty($order_filter, false, false) . '
            ) AS o2 ON o2.product_id = p.id_product AND o2.product_attribute_id = IFNULL(d.id_product_attribute, 0)

			LEFT JOIN (
				SELECT
					sa.id_product,
					sa.id_product_attribute,
					SUM(1 - sm.depends_on_stock) AS stock_shop,
					SUM(sm.depends_on_stock) AS stock_warehouse,
					SUM(IF(sm.depends_on_stock = 0, sa.quantity, 0)) AS qty,
                    SUM(IF(sm.depends_on_stock = 0 AND sa.quantity > 0, sa.quantity*pp.wholesale_price, 0)) AS `value`,
                    SUM(m.buyQty) AS buyQty
				FROM '._DB_PREFIX_.'stock_available sa
            	LEFT JOIN (
            		SELECT 
            			id_stock, 
            			SUM(CAST(physical_quantity  AS SIGNED) * `sign`) AS buyQty
            			FROM '._DB_PREFIX_.'stock_mvt FORCE INDEX (id_stock)
                    WHERE id_stock_mvt_reason IN (' . Configuration::get('KERAWEN_MVT_BUY') . ') /*+ period/collection */
            		GROUP BY id_stock	
            	) m ON m.id_stock = sa.id_stock_available	
		
				LEFT JOIN (
				    SELECT sa2.id_product, sa2.id_shop, sa2.id_shop_group, sa2.depends_on_stock
				    FROM '._DB_PREFIX_.'stock_available sa2
				    WHERE sa2.id_product_attribute = 0
				) sm ON sm.id_product = sa.id_product AND sm.id_shop = sa.id_shop AND sm.id_shop_group = sa.id_shop_group
                LEFT JOIN (
                    SELECT
                        p2.id_product,
                        IFNULL(a2.id_product_attribute, 0) AS id_product_attribute,
                        CASE
                            WHEN as2.wholesale_price > 0 THEN as2.wholesale_price
                            WHEN as3.wholesale_price > 0 THEN as3.wholesale_price
                            WHEN ps2.wholesale_price > 0 THEN ps2.wholesale_price
                            WHEN ps3.wholesale_price > 0 THEN ps3.wholesale_price
                            ELSE 0
                        END AS wholesale_price
                    FROM '._DB_PREFIX_.'product p2
                    LEFT JOIN '._DB_PREFIX_.'product_attribute a2 ON a2.id_product = p2.id_product
                    LEFT JOIN '._DB_PREFIX_.'product_shop ps2 ON ps2.id_product = p2.id_product AND ps2.id_shop = '. (int) $id_shop .'
                    LEFT JOIN '._DB_PREFIX_.'product_attribute_shop as2 ON as2.id_product_attribute = a2.id_product_attribute AND as2.id_shop = ps2.id_shop
                    LEFT JOIN '._DB_PREFIX_.'product_shop ps3 ON ps3.id_product = p2.id_product AND ps3.id_shop = p2.id_shop_default
                    LEFT JOIN '._DB_PREFIX_.'product_attribute_shop as3 ON as3.id_product_attribute = a2.id_product_attribute AND as3.id_shop = ps3.id_shop
                ) pp ON pp.id_product = sa.id_product AND pp.id_product_attribute = sa.id_product_attribute
				WHERE '.$sa_filter.'
				GROUP BY sa.id_product, sa.id_product_attribute
			) ss ON ss.id_product = p.id_product
				AND ss.id_product_attribute = IFNULL(d.id_product_attribute, 0)
			
			LEFT JOIN (
				SELECT
					id_product,
					id_product_attribute,
					SUM(physical_quantity) AS qty,
					SUM(price_te*physical_quantity) AS value
				FROM '._DB_PREFIX_.'stock
				WHERE '.$wh_filter.'
				GROUP BY id_product, id_product_attribute
			) sw ON sw.id_product = p.id_product
				AND sw.id_product_attribute = IFNULL(d.id_product_attribute, 0)
            ' . (count($where) ? ' WHERE ' . implode(" AND ", $where) : "") . '
            ' . $order;

		if ($bypage > 0) {
		    $q .= " LIMIT " . ( ((int) $page - 1) * (int) $bypage) . ", " . (int) $bypage;
		}


		$data = $db->executeS($q);
		$total = $db->getValue('SELECT FOUND_ROWS()');

		// Fix buying pricesstock
		foreach ($data as &$row) {
		    if ((int)$row['stock'] > 0) { 
		        $row['buying_price'] = (float)$row['valorization'] / (int)$row['stock']; 
		        //$row['buyAmount'] = $row['buying_price'] * (int)$row['buyQty'];
		    }
		}
		
		$res = array(
		    'data' => $data,
		    'total' => $total,
		    'bypage' => $bypage
		);
		
		return $res;
	}
	

	public static function labelSettings($db, $params)
	{

	  $config_key = 'KERAWEN_LABEL';
	  
	  if (isset($params->data)) {
	    //optional parameters require -> context exist
	  	Configuration::updateValue($config_key, $params->data, false, 0, 0);
	  }

	  return json_decode(Configuration::get($config_key, 0, 0, 0));

	}


	public static function setBarCode($db, $params) {
	
		require_once(_KERAWEN_CLASS_DIR_.'/catalog.php');
		
		$p = self::labelSettings($db, $params);
		$mask = '20xxxxxxxxxx';
		if (isset( $p->barcode )) {
		    $mask = str_replace(' ', '', $p->barcode->mask) . $mask;
		}
	
		$q = "
			SELECT 
			  product.id_product, 
			  IFNULL(product_attribute.id_product_attribute, 0) AS id_product_attribute,
              IFNULL(pac.nb, 0) AS nb,
			  '' AS ean13,
              IFNULL(pwk.is_free_price, 0) AS is_free_price,
              product.ean13 AS default_ean13

			FROM " . _DB_PREFIX_ . "product product
			LEFT JOIN " . _DB_PREFIX_ . "product_attribute product_attribute ON product.id_product = product_attribute.id_product 
            LEFT JOIN " . _DB_PREFIX_ . "product_wm_kerawen pwk ON pwk.id_product = product.id_product

			LEFT JOIN (
				SELECT 
					COUNT(id_attribute) AS nb, 
					id_product_attribute
				FROM " . _DB_PREFIX_ . "product_attribute_combination
				GROUP BY id_product_attribute
			) AS pac ON pac.id_product_attribute = product_attribute.id_product_attribute

            WHERE 
              (
    			 (product_attribute.id_product_attribute IS NULL AND (product.ean13 = '0' OR product.ean13 = '' OR product.ean13 IS NULL))
    			 OR 
    			 (product_attribute.id_product_attribute IS NOT NULL AND (product_attribute.ean13 = '0' OR product_attribute.ean13 = '' OR product_attribute.ean13 IS NULL))
              )
		";
	
		$res = $db->executeS($q);
		

		$items = array();
		foreach ($res as $r) {

			$upd = false;
			
			if ( (int) $r['default_ean13'] == 0 && ( (int) $r['id_product_attribute'] == 0 || (int) $r['is_free_price'] == 1 )) {
			    $upd = array(
			        'table' => 'product',
			        'where' => 'id_product = ' . (int) $r['id_product'],
			    );
			} else {
			    if ($r['nb'] && (int) $r['is_free_price'] == 0) {
			        
			        $upd = array(
			            'table' => 'product_attribute',
			            'where' => 'id_product_attribute = ' . (int) $r['id_product_attribute'],
			        );
			    }
			}
			
			if ($upd) {

			    $barcode = 'xxxxxxxxxxxx';
			    for ($k = 0; $k < 12; $k++) {
			        if ($mask[$k] == 'x') {
			            $barcode[$k] = mt_rand(0, 9);
			        } else {
			            $barcode[$k] = $mask[$k];
			        }
			    }

			    $r['ean13'] = $barcode . ean_checkdigit($barcode);
			    //check if already exist ????
			    
			    $db->update($upd['table'], array('ean13' => pSQL($r['ean13'])), $upd['where']);
			    $items[] = $r;
			    
			}

		}

		return $items;
	
	}
	
	
	public static function productSupplier($db, $params)
	{

		//$id_lang = Context::getContext()->language->id;
  
		$q = "
			SELECT 
			  supplier.id_supplier, 
			  supplier.name, 
			  COUNT(supplier.id_supplier) AS nb 
			FROM " . _DB_PREFIX_ . "product product
			INNER JOIN " . _DB_PREFIX_ . "supplier supplier ON product.id_supplier = supplier.id_supplier
			GROUP BY supplier.id_supplier
			ORDER BY supplier.name
		";

		$res = $db->executeS($q);
				
		if ($params->index) {
			$res = indexArray($res, 'id_supplier');
		}
		
		return $res;

	}


	public static function productManufacturer($db, $params)
	{

		$q = "
			SELECT
			  manufacturer.id_manufacturer,
			  manufacturer.name,
			  COUNT(manufacturer.id_manufacturer) AS nb
			FROM " . _DB_PREFIX_ . "product product
			INNER JOIN " . _DB_PREFIX_ . "manufacturer manufacturer ON product.id_manufacturer = manufacturer.id_manufacturer
			GROUP BY manufacturer.id_manufacturer
			ORDER BY manufacturer.name
		";
	
		$res = $db->executeS($q);
	
		if ($params->index) {
			$res = indexArray($res, 'id_manufacturer');
		}
	
		return $res;
	
	}
	
	
	public static function productCategories($db, $params)
	{

		$id_lang = Context::getContext()->language->id;
  
		$q = "
		
			SELECT 
			  category_lang.id_category, 
			  category_lang.name, 
			  COUNT(category_lang.id_category) AS nb
			FROM " . _DB_PREFIX_ . "product product
			LEFT JOIN " . _DB_PREFIX_ . "category_lang category_lang
			INNER JOIN " . _DB_PREFIX_ . "category category ON category_lang.id_category = category.id_category
			ON product.id_category_default = category_lang.id_category AND product.id_shop_default = category_lang.id_shop AND category_lang.id_lang = " . (int) $id_lang . "
			GROUP BY category_lang.id_category
			ORDER BY category_lang.name
		
		";

		$res = $db->executeS($q);
				
		if ($params->index) {
			$res = indexArray($res, 'id_category');
		}
		
		return $res;

	}


	public static function productsListV2($db, $params) {
			
	    // Backup shop context
	    require_once(_KERAWEN_TOOLS_DIR_.'/utils.php');
	    require_once(_KERAWEN_CLASS_DIR_.'/KerawenTax.php');
	    $backup = backupShopContext();
	    
	    $taxrule_vatmargin = (int) KerawenTax::GetVATMarginRule();

		//define from params
		$bypage = (int) Configuration::get('KERAWEN_LABEL_ITEMS_BY_PAGE');
		$advanced_stock = (boolean) Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT');		

		$id_lang = Context::getContext()->language->id;
		$id_country = Context::getContext()->country->id;
		$id_currency = Context::getContext()->currency->id;

		$id_shop = 'product.id_shop_default';
		if ($params->shop != -1) {
		 	$id_shop = intval($params->shop);
		}

		$link = Context::getContext()->link;
		$id_shop_default = Configuration::get('PS_SHOP_DEFAULT');

		$search = '';
		if ($params->text != '') {
			$search = " AND (product_attribute.reference LIKE '%" . trim(pSQL($params->text)) . "%' OR product.reference LIKE '%" . trim(pSQL($params->text)) . "%' OR product.ean13 LIKE '%" . trim(pSQL($params->text)) . "%' OR product_attribute.ean13 LIKE '%" . trim(pSQL($params->text)) . "%' OR product_lang.name LIKE '%" . trim(pSQL($params->text)) .  "%') ";
		}
		
		
		$category = '';
		if ($params->id_cat != -1) {
		  $category = ' AND product_shop.id_category_default = ' . intval($params->id_cat) . ' ';
		}
		
		
		$supplier = '';
		if ($params->id_sup != -1) {
		  $supplier = ' AND product.id_supplier = ' . intval($params->id_sup) . ' ';
		}
		

		$manufacturer = '';
		if ($params->id_man != -1) {
		  $manufacturer = ' AND product.id_manufacturer = ' . intval($params->id_man) . ' ';
		}

		$product_active = '';
		$product_shop_active = '';
		if ($params->cr_status != -1) {
		    if ($params->shop == -1) {
		        $product_active = ' AND product.active = ' . intval($params->cr_status) . ' ';
		    } else {
		      $product_shop_active = ' AND product_shop.active = ' . intval($params->cr_status) . ' ';
		    }
		}
		
		$warehouse_condition = "";
		$q = "SELECT id_warehouse FROM " . _DB_PREFIX_ . "warehouse_shop WHERE id_shop = " . intval($params->shop);
		$warehouse = $db->executeS($q);
		
		if (count($warehouse)) {
			$w = array();
			foreach ($warehouse as $row) {
		  		$w[] = (int) $row['id_warehouse'];
			}
			$warehouse_condition = " AND id_warehouse IN(" . implode(",", $w) . ") ";
		}


		$id_group = null;
		if ($params->id_group != -1) {
			$id_group = $params->id_group;
		}
		

		$joinType = "INNER";
		// INNER: display row only if product exist on the specified shop
		// LEFT: display row even if product doesn't exist on the shop

		
		$adv_attribut = "'' AS mvtstock";
		$adv_join = "";
		$adv_group = "";
		$sortMvt = " ORDER BY mvtstock DESC ";
        $mvt_join = "";
        $sort = '';
        $flagMvt = false;
        
        
		if (!($params->from == "" || $params->to == "")) {
		
    		if ($advanced_stock) {
    			$adv_attribut = " IFNULL(SUM(stock_mvt.physical_quantity), 0) AS mvtstock ";
    			$adv_join = "
    			LEFT JOIN " . _DB_PREFIX_ . "stock stock 
    			ON product.id_product = stock.id_product AND product_attribute.id_product_attribute = stock.id_product_attribute " . $warehouse_condition . "
    			LEFT JOIN " . _DB_PREFIX_ . "stock_mvt stock_mvt
    			ON stock.id_stock = stock_mvt.id_stock AND stock_mvt.id_stock_mvt_reason IN (1,8) AND stock_mvt.date_add BETWEEN '" . pSQL($params->from) . "' AND '" . pSQL($params->to) . "' 
    			";
    			$adv_group = ", stock.id_product, stock.id_product_attribute ";
    			$sort = $sortMvt;
    			
    		} else {
    			
    			//TODO : change advanced stocks like below (subquery)
    			
    			if (Tools::version_compare(_PS_VERSION_, '1.7', '>=')) {
    				
    			    $flagMvt = true;
    				$withShop = "";
    				$_shop = "";
    				if ($params->shop != -1) {
    					$withShop = " AND sa.id_shop = " . (int) $params->shop . " ";
    					$_shop = "_shop";
    				}
    				
    				/*
    				$adv = "
    					FROM " . _DB_PREFIX_ . "stock_available sa
    					LEFT JOIN " . _DB_PREFIX_ . "stock_mvt sm ON sa.id_stock_available = sm.id_stock
    					WHERE sa.id_product = product.id_product AND sa.id_product_attribute = IFNULL(product_attribute.id_product_attribute, 0)
    					AND sm.id_stock_mvt_reason IN(11, 8)
    					AND sm.date_add BETWEEN '" . pSQL($params->from) . "' AND '" . pSQL($params->to) . "'" . $withShop . "
                        GROUP BY sa.id_product, sa.id_product_attribute
                    ";
    				
    				$adv_attribut = "
    				(
    					SELECT IFNULL(SUM(sm.physical_quantity * sm.sign), 0)
                        " . $adv . "
    				) AS mvtstock";
    				
    				
    				$adv_attribut_2 = "
    				(
                        SELECT IFNULL(sm.id_stock_mvt, 0)
                        " . $adv . "
    				) AS withstock";
    				
    				
    				$sort = $sortMvt2;
                    */


    				$mvt_join = "
        				INNER JOIN (
        				    SELECT
            				    sa.id_product,
            				    sa.id_product_attribute,
                                IFNULL(SUM(sm.physical_quantity * sm.sign), 0) AS mvtstock,
                                IFNULL(MAX(sm.id_stock_mvt), 0) AS id_stock_mvt
        				    FROM " . _DB_PREFIX_ . "stock_available sa
        				    INNER JOIN " . _DB_PREFIX_ . "stock_mvt sm ON sa.id_stock_available = sm.id_stock
        				    AND sm.id_stock_mvt_reason IN(11, 8) AND sm.sign = 1
        				    AND sm.date_add BETWEEN '" . pSQL($params->from) . "' AND '" . pSQL($params->to) . "'" . $withShop . "
        				    GROUP BY sa.id_product, sa.id_product_attribute
        				) AS stk ON stk.id_product = product" . $_shop . ".id_product  AND stk.id_product_attribute = IFNULL(product_attribute" . $_shop . ".id_product_attribute, 0)
    				";
    				
    				$adv_attribut = "stk.mvtstock AS mvtstock";
    				$sort = " ORDER BY stk.id_stock_mvt ASC ";

    			}
    		}
		}

		if (!($params->orderby == '' || $params->sortby == '')) {
		    if ($params->orderby == 'mvtstock' && $flagMvt) {
		        $sortby = ($params->sortby == 'DESC') ? "DESC" : "ASC";
		        $sort = " ORDER BY stk.id_stock_mvt " . $sortby;
		    } else {
		        $sort = self::setOrder($params->orderby, $params->sortby);
		    }
		}
		

		$q = "
			SELECT SQL_CALC_FOUND_ROWS

				CONCAT('" . pSQL($params->shop) . "', '-', product.id_product, '-', IFNULL(product_attribute.id_product_attribute, 0)) AS uniq,
				product_shop.id_shop AS id_shop,
				product.id_product AS id,
				IFNULL(product_attribute.id_product_attribute, 0) AS id_product_attribute,
				product_lang.name AS product_name,
                product_lang.description_short AS desc_short,
				GROUP_CONCAT(DISTINCT attribute_lang.name ORDER BY attribute.id_attribute_group) AS declinaison,
				GROUP_CONCAT(DISTINCT CONCAT_WS(':', agl.name, attribute_lang.name) ORDER BY attribute.id_attribute_group) AS declinaison_adv,
				 
				product.id_category_default AS id_category, 
				category.id_parent AS id_parent_category,
				category_lang.name AS category,

				IF(product_attribute.id_product_attribute IS NULL OR product_attribute.ean13 ='', product.ean13, product_attribute.ean13) AS ean13,
				IF(product_attribute.id_product_attribute IS NULL OR product_attribute.upc ='', product.upc, product_attribute.upc) AS upc,
                IF(product_attribute.weight IS NULL, product.weight, 
			        IF(product.weight = '', '', 
				        IF(product_attribute.weight = '', product.weight,  (product_attribute.weight + product.weight))
			        )
                ) AS weight,
                product.reference AS main_reference,		
				IF(product_attribute.id_product_attribute IS NULL, product.reference, product_attribute.reference) AS reference,				

                product_supplier.product_supplier_reference AS supplier_ref,

                IF(product_wm_kerawen.unit IS NULL, '', product_wm_kerawen.unit) AS unit,
                product_wm_kerawen.product_type AS product_type,

				ROUND(IF(product_attribute_shop.ecotax > 0, product_attribute_shop.ecotax, IFNULL(product_shop.ecotax, 0)),2)  AS ecotax,
                ROUND(IF(product_attribute_shop.ecotax > 0, product_attribute_shop.ecotax, IFNULL(product_shop.ecotax, 0)) + IFNULL(product_shop.price, 0) + IFNULL(product_attribute_shop.price, 0) + (IFNULL(product_shop.price, 0) + IFNULL(product_attribute_shop.price, 0))*(IFNULL(tax.rate,0) * (IF(product_shop.id_tax_rules_group = " . $taxrule_vatmargin . ", 0,1)) /100), 2) AS price_vat_incl,	
                ROUND(IFNULL(product_shop.price, 0) + IFNULL(product_attribute_shop.price, 0) - IF(product_attribute_shop.ecotax > 0, product_attribute_shop.ecotax, IFNULL(product_shop.ecotax, 0)) ,2) AS price_vat_excl,

                ROUND(IF(product_attribute_shop.wholesale_price IS NULL OR product_attribute_shop.wholesale_price = 0, product.wholesale_price, product_attribute_shop.wholesale_price), 2) AS wholesale_price,

				ROUND(IFNULL(tax.rate, 0), 2) AS tax,

				product_wm_kerawen.measured, product_wm_kerawen.precision,
				
				product.unit_price_ratio AS unit_ratio,
				product.unity AS unit_label,

				manufacturer.name AS manufacturer,

				features.feature,
				features.feature_adv,
                product_shop.id_tax_rules_group AS id_tax_rules_group, 

				" . $adv_attribut . "

			FROM " . _DB_PREFIX_ . "product product
			LEFT JOIN " . _DB_PREFIX_ . "product_lang product_lang
			ON product.id_product = product_lang.id_product AND product_lang.id_shop = " . $id_shop . " AND product_lang.id_lang = " . (int) $id_lang . "
			LEFT JOIN " . _DB_PREFIX_ . "category_lang category_lang 
			ON product.id_category_default = category_lang.id_category AND product.id_shop_default = category_lang.id_shop AND category_lang.id_lang = " . (int) $id_lang . "
			LEFT JOIN " . _DB_PREFIX_ . "category category 
			ON product.id_category_default = category.id_category
			" . $joinType . " JOIN " . _DB_PREFIX_ . "product_shop product_shop 
			ON product.id_product = product_shop.id_product AND product_shop.id_shop = " . $id_shop . " " . $product_shop_active . "
			
			LEFT JOIN " . _DB_PREFIX_ . "product_wm_kerawen product_wm_kerawen 
			ON product.id_product = product_wm_kerawen.id_product
		
			LEFT JOIN " . _DB_PREFIX_ . "product_attribute product_attribute 
			ON product.id_product = product_attribute.id_product 
			
			LEFT JOIN " . _DB_PREFIX_ . "product_attribute_shop product_attribute_shop 
			ON product_attribute.id_product_attribute = product_attribute_shop.id_product_attribute AND product_attribute_shop.id_shop = " . $id_shop . "

			LEFT JOIN " . _DB_PREFIX_ . "product_attribute_kerawen product_attribute_kerawen 
			ON product_attribute.id_product_attribute = product_attribute_kerawen.id_product_attribute

			LEFT JOIN " . _DB_PREFIX_ . "product_attribute_combination product_attribute_combination 
			ON product_attribute.id_product_attribute = product_attribute_combination.id_product_attribute

			LEFT JOIN " . _DB_PREFIX_ . "attribute_lang attribute_lang 
			ON product_attribute_combination.id_attribute = attribute_lang.id_attribute AND attribute_lang.id_lang = " . (int) $id_lang . "
			
			LEFT JOIN " . _DB_PREFIX_ . "attribute attribute 
			ON attribute_lang.id_attribute = attribute.id_attribute

			LEFT JOIN " . _DB_PREFIX_ . "attribute_group_lang agl
			ON attribute.id_attribute_group = agl.id_attribute_group AND agl.id_lang = " . (int) $id_lang . "

			LEFT JOIN " . _DB_PREFIX_ . "tax_rule tax_rule 
			ON product_shop.id_tax_rules_group = tax_rule.id_tax_rules_group AND tax_rule.id_country = " . (int) $id_country . "
			
			LEFT JOIN " . _DB_PREFIX_ . "tax tax 
			ON tax_rule.id_tax = tax.id_tax

			LEFT JOIN " . _DB_PREFIX_ . "manufacturer manufacturer
			ON product.id_manufacturer = manufacturer.id_manufacturer

			LEFT JOIN (
				SELECT
					fp.id_product,
					GROUP_CONCAT(DISTINCT fvl.value ORDER BY f.position ASC) AS feature,
					GROUP_CONCAT(DISTINCT CONCAT_WS(':', fl.name, fvl.value) ORDER BY f.position ASC) AS feature_adv
				FROM "._DB_PREFIX_."feature_product fp
				LEFT JOIN "._DB_PREFIX_."feature f ON f.id_feature = fp.id_feature
				LEFT JOIN "._DB_PREFIX_."feature_lang fl ON fl.id_feature = f.id_feature AND fl.id_lang = ". (int) $id_lang."
				LEFT JOIN "._DB_PREFIX_."feature_value_lang fvl ON fvl.id_feature_value = fp.id_feature_value AND fvl.id_lang = ". (int) $id_lang."
				GROUP BY fp.id_product
			) features ON features.id_product = product.id_product

			LEFT JOIN " . _DB_PREFIX_ . "supplier supplier ON supplier.id_supplier = product.id_supplier
			LEFT JOIN " . _DB_PREFIX_ . "product_supplier product_supplier
				ON product_supplier.id_supplier = supplier.id_supplier
				AND product_supplier.id_product = product.id_product
				AND (product_supplier.id_product_attribute = product_attribute.id_product_attribute OR (product_attribute.id_product_attribute IS NULL AND product_supplier.id_product_attribute = 0))

			" . $adv_join . "
            " . $mvt_join . "
			WHERE product_attribute_kerawen.id_product_attribute IS NULL " . $search . " " . $category . " " . $supplier . " " . $manufacturer .  " " . $product_active . " 
			
			GROUP BY product.id_product, product_attribute.id_product_attribute
			" . $adv_group . "
			" . $sort . "
		";

		
		if ($bypage > 0) {
			
			$q .= "
					LIMIT " . ( ((int)$params->page - 1) * (int) $bypage) . ", " . (int) $bypage;
			
		}

		/*
		echo '<pre>';
		echo $q;
		echo '</pre>';
		*/
		
		$data = $db->executeS($q);
		$total = $db->getValue('SELECT FOUND_ROWS()');
		
		
		if ($id_shop == 'product.id_shop_default') {
		    $id_shop_for_qty = (int) $id_shop_default;
		    Shop::setContext(Shop::CONTEXT_ALL);
		} else {
		    $id_shop_for_qty = (int) $id_shop;
		    Shop::setContext(Shop::CONTEXT_SHOP, $id_shop_for_qty);
		}
		

		foreach($data as $k=>$item) {

		    $tax_coef = (($data[$k]['id_tax_rules_group'] == $taxrule_vatmargin)) ? 1 : ($data[$k]['tax']/100 + 1);
		    
			//price/weight
			//Select first item added
			if ($item['measured']) {	
				$tmp_attr = (int) $item['id_product_attribute'] > 0 ? $item['id_product_attribute'] : -1;
				$precision = (int) $data[$k]['precision'] > 0 ? $data[$k]['precision'] : 2;
				if ($row = $db->getRow("
					SELECT unit_price
					FROM " . _DB_PREFIX_ . "product_wm_code_kerawen
					WHERE id_product = " . (int) $item['id'] . " AND (id_product_attribute = -1 OR id_product_attribute = " . (int) $tmp_attr . ")
					ORDER BY id_product_attribute DESC, id_code ASC
				")) {				
				$data[$k]['price_vat_incl'] = round($row["unit_price"] * $tax_coef, $precision);
				}
			}

			$specific = SpecificPrice::getSpecificPrice(
				$item['id'],
				$item['id_shop'],
				$id_currency,
				$id_country,
				$id_group,
				1,
				$item['id_product_attribute'],
				null,
				0,
				1);


			$data[$k]['discount'] = 0;
			$data[$k]['discount_te'] = 0;
			
			if ($specific) {

			    if ($specific['price'] != -1) {
			        $data[$k]['price_vat_excl'] = (float)$specific['price'];
			        $data[$k]['price_vat_incl'] = (float)$specific['price'] * $tax_coef;
			    }
			    
				if ($specific['reduction'] > 0) {
							
					if ($specific['reduction_type'] == 'percentage') {
						$data[$k]['discount'] = (float)$data[$k]['price_vat_incl'] * (float)$specific['reduction'];
						$data[$k]['discount_te'] = (float)$data[$k]['price_vat_excl'] * (float)$specific['reduction'];
					}

					if($specific['reduction_type'] == 'amount') {
					    
					    //PS 1.6.0.x
					    if (!isset($specific['reduction_tax'])) {
					        $specific['reduction_tax'] = 1;
					    }
					    
    					if ($specific['reduction_tax']) {
    					    $data[$k]['discount'] = (float)$specific['reduction'];
    					    $data[$k]['discount_te'] = (float)$specific['reduction'] / $tax_coef;
    					} else {
    					    $data[$k]['discount'] = (float)$specific['reduction'] * $tax_coef;
    					    $data[$k]['discount_te'] = (float)$specific['reduction'];
    					}
					}
				}
			}
			
			

			Cache::clean('StockAvailable::getQuantityAvailableByProduct_'.'*');
			$data[$k]['stock'] = StockAvailable::getQuantityAvailableByProduct($item['id'], $item['id_product_attribute'], $id_shop_for_qty);
			
			
			//ordering columns
			$mvt = $data[$k]['mvtstock'];
			unset($data[$k]['mvtstock']);			
			$data[$k]['mvtstock'] = $mvt;

			$data[$k]['vat'] = round(($data[$k]['price_vat_incl'] - $data[$k]['discount']) - ($data[$k]['price_vat_excl'] - $data[$k]['discount_te']), 2);

			$data[$k]['margin'] = $data[$k]['price_vat_excl'] - $data[$k]['discount_te'] - $data[$k]['wholesale_price'];
			$data[$k]['margin_rate'] = 100;
			if ($data[$k]['price_vat_incl'] != 0) {
			    $data[$k]['margin_rate'] = round( ($data[$k]['price_vat_excl'] - $data[$k]['discount_te'] - $data[$k]['wholesale_price']) / ($data[$k]['price_vat_incl'] / 100), 1);
			}

		    //round
			$data[$k]['margin'] = round($data[$k]['margin'], 2);
			$data[$k]['discount'] = round($data[$k]['discount'], 2);
			$data[$k]['discount_te'] = round($data[$k]['discount_te'], 2);
			
			$data[$k]['link'] = $link->getPageLink('product', null, $id_lang, array('id_product' => $item['id']), false, $id_shop_default, false);
			
			$anchor = '';
			if ((int) $item['id_product_attribute'] > 0) {
			    $product = new Product($item['id'], true, $id_lang, $item['id_shop']);
			    $anchor = $product->getAnchor($item['id_product_attribute'], true);
			    //fix duplicated items - PS 1.6
			    if ($anchor) {
			         $anchor = implode("/", array_unique(explode("/", $anchor)));
			    }
			}
			$data[$k]['link'] = $link->getPageLink('product', null, $id_lang, array('id_product' => $item['id']), false, $id_shop_default, false) . $anchor;
			
			$data[$k]['ext'] = '';
			if (Module::isInstalled('ppcmod') && Module::isEnabled('ppcmod')) {
			    if ($row = $db->getRow("
					SELECT ppc
					FROM " . _DB_PREFIX_ . "ppcmod
					WHERE ppc > 0 AND id_product = " . (int) $item['id'] . " AND (id_product_attribute = " . (int) $item['id_product_attribute'] . " OR id_product_attribute = 0)
                    ORDER BY id_product_attribute DESC
                ")) {
				    $data[$k]['ext'] = $row["ppc"];
			     }
			}
			
			$data[$k]['desc_short'] = strip_tags($data[$k]['desc_short']);

		}		

		restoreShopContext($backup);
		
		$res = array(
		  'data' => $data,
		  'total' => $total,
		  'bypage' => $bypage,
		  'sort' => $sort,
		);
		
		return $res;
	}

	public static function prepareSummaryPending($db, $params, $type, $now) {

	    $where = $now ? '(NOW() BETWEEN cr.date_from AND cr.date_to)' : '(NOW() > cr.date_to)';

	    $q = '
            SELECT
      		    SUM(cr.reduction_amount * cr.quantity) AS amount
        	FROM '._DB_PREFIX_.'cart_rule cr
        	INNER JOIN ' . _DB_PREFIX_ . 'cart_rule_kerawen crk ON crk.id_cart_rule = cr.id_cart_rule AND crk.type = "' . pSQL($type) . '"
        	WHERE cr.active = 1 AND cr.quantity > 0 AND ' . $where;
	    
	    return $db->getValue($q);
	}

	public static function prepareLoyaltyPending($db, $params, $now) {
	    $where = $now ? '(NOW() BETWEEN cr.date_from AND cr.date_to)' : '(NOW() > cr.date_to)';
	    $q = '
		  SELECT 
				  SUM(cr.reduction_amount) AS amount
		  FROM '._DB_PREFIX_.'cart_rule cr
		  INNER JOIN (
		      SELECT 
                  id_cart_rule
			  FROM '._DB_PREFIX_.'loyalty
			  GROUP BY id_cart_rule
		  ) AS l ON cr.id_cart_rule = l.id_cart_rule
		  WHERE cr.active = 1 AND cr.quantity > 0 AND ' . $where;
	    
	    return $db->getValue($q);
	}
	
	
	public static function getSummaryPending($db, $params) {
	    
	    $cr = array();
	    $cr[] =
	        array(
	           'label_type' => _KERAWEN_CR_CREDIT_, 
	            'cr_active' => self::prepareSummaryPending($db, $params, _KERAWEN_CR_CREDIT_, true), 
	            'cr_expired' => self::prepareSummaryPending($db, $params, _KERAWEN_CR_CREDIT_, false)
	        );
	     $cr[] = 
	        array(
	            'label_type' => _KERAWEN_CR_GIFT_CARD_,
	            'cr_active' => self::prepareSummaryPending($db, $params, _KERAWEN_CR_GIFT_CARD_, true),
	            'cr_expired' => self::prepareSummaryPending($db, $params, _KERAWEN_CR_GIFT_CARD_, false)
	        );
	     if (Module::isInstalled('loyalty') && Module::isEnabled('loyalty')) {

	         $cr[] = 
    	         array(
    	             'label_type' => _KERAWEN_CR_LOYALTY_,
    	             'cr_active' => self::prepareLoyaltyPending($db, $params, true),
    	             'cr_expired' => self::prepareLoyaltyPending($db, $params, false)
    	         );
	     }
	     
	     
	     $deffered = self::getDeffered($db, $params, '>');
	     $total_deffered = 0;
	     foreach ($deffered as $row) {
	         $total_deffered += (float) $row['amount'];
	     }
	     $cr[] =
    	     array(
    	         'label_type' => 'DEFFERED',
    	         'cr_active' => $total_deffered,
    	         'cr_expired' => ''
    	     );
	     
	     $refunded = self::getRefundDeffered($db, $params, '<');
	     $total_refunded = 0;
	     foreach ($refunded as $row) {
	         $total_refunded += (float) $row['amount'];
	     }
	     $cr[] =
	     array(
	         'label_type' => 'REFUND',
	         'cr_active' => $total_refunded,
	         'cr_expired' => ''
	     );
	     
	     if (Configuration::get('KERAWEN_ACCOUNT')) {
	         $accounted = $db->getValue('
        	     SELECT
        	       SUM(ck.amount) AS amount
        	     FROM '._DB_PREFIX_.'customer a
        	     LEFT JOIN '._DB_PREFIX_.'customer_kerawen ck ON a.id_customer = ck.id_customer
        	     WHERE a.deleted = 0
	         ');
	         
	         
    	     $cr[] =
        	     array(
        	         'label_type' => 'ACCOUNTED',
        	         'cr_active' => $accounted,
        	         'cr_expired' => ''
        	     );
	     }
        	     
	     return $cr;
	}
	

	public static function getCredit($db, $params)
	{
	    
	    $now = date('Y-m-d H:i:s');
	    
	    switch($params->cr_status) {
	        case "1":
	            $where = ' AND (("'.pSQL($now).'" BETWEEN cr.date_from AND cr.date_to) AND cr.active = 1 AND cr.quantity = 1) ';
	            break;
	        case "2";
    	        $where = ' AND cr.quantity = 0 AND cr.active = 1 ';
    	        break;
	        case "3";
    	        $where = ' AND NOT (("'.pSQL($now).'" BETWEEN cr.date_from AND cr.date_to)) AND cr.quantity = 1 ';
    	        break;
	        default:
	            $where = '';
	            break;
	    }
	    
	    $id_lang = Context::getContext()->language->id;
	    
	    if (Configuration::get("KERAWEN_CREDIT_CODE_DISPLAY")) {
	        $code = "cr.code";
	    } else {
	        $code = "CONCAT('#####', SUBSTRING(cr.code, -3))";
	    }
	    
	    $q = '
    		SELECT
    		  DISTINCT cr.id_cart_rule AS id,
    		  crl.name,
    		  ' . $code . ' AS code,
    		  cr.id_customer,
    		  IF(cr.id_customer = 0, "- -", CONCAT(c.firstname, " ", c.lastname)) AS customer,
    		  IF(e.id_employee IS NULL, "- -", CONCAT(e.firstname, " ", e.lastname)) AS employee,
    		  cr.date_from,
    		  cr.date_to,
              CASE
                  WHEN (cr.date_to < "'.pSQL($now).'" AND  cr.quantity = 1) THEN "Expired"
                  WHEN (cr.quantity = 0) THEN "Used"
                  WHEN (("'.pSQL($now).'" BETWEEN cr.date_from AND cr.date_to) AND cr.quantity = 1 AND cr.active = 1) THEN "Active"
                  ELSE "Inactive"
              END AS cr_status,
    		  cr.reduction_amount AS amount
    		FROM '._DB_PREFIX_.'cart_rule cr
    		JOIN '._DB_PREFIX_.'cart_rule_kerawen crk ON crk.id_cart_rule = cr.id_cart_rule AND crk.type = "'. pSQL(_KERAWEN_CR_CREDIT_) .'"
    		LEFT JOIN '._DB_PREFIX_.'cart_rule_lang crl ON crl.id_cart_rule = cr.id_cart_rule AND crl.id_lang = '. (int) $id_lang.'
    		LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = cr.id_customer
    		LEFT JOIN '._DB_PREFIX_.'cashdrawer_flow_kerawen cfk ON cr.id_cart_rule = cfk.id_credit
    		LEFT JOIN '._DB_PREFIX_.'cashdrawer_op_kerawen cok ON cfk.id_cashdrawer_op = cok.id_cashdrawer_op
    		LEFT JOIN '._DB_PREFIX_.'employee e ON cok.id_employee = e.id_employee
		    WHERE 1=1 ' . $where . '
    		ORDER BY cr.id_cart_rule DESC';
	    
	    $res = $db->executeS($q);
	    if ($params->index) {
	        $res = indexArray($res, 'id');
	    }
	    return $res;
	}
	
	public static function getGiftCard($db, $params)
	{
	    
	    $now = date('Y-m-d H:i:s');
	    
	    switch($params->cr_status) {
	        case "1":
	            $where = " AND (('".pSQL($now)."' BETWEEN cr.date_from AND cr.date_to) AND cr.active = 1 AND cr.quantity > 0) ";
	            break;
	        case "2";
    	        $where = " AND cr.quantity = 0 AND cr.active = 1 ";
    	        break;
	        case "3";
    	        $where = " AND ('".pSQL($now)."' > cr.date_to) AND cr.quantity > 0 ";
    	        break;
	        default:
	            $where = "";
	            break;
	    }
	    
	    
	    $id_lang = Context::getContext()->language->id;
	    
	    if (Configuration::get("KERAWEN_GIFT_CARD_CODE_DISPLAY")) {
	        $code = "cr.code";
	    } else {
	        $code = "CONCAT('#####', SUBSTRING(cr.code, -3))";
	    }
	    
	    $q = "
		SELECT
		  cr.id_cart_rule AS id,
		  crl.name,
		  " . $code . " AS code,
          IF(cr.id_customer = 0, '- -', CONCAT (c.firstname, ' ', c.lastname)) AS customer,
		  CONCAT (employee.firstname, ' ', employee.lastname) AS employee,
		  cr.date_from,
		  cr.date_to,
          CASE
              WHEN (cr.date_to < '".pSQL($now)."' AND  cr.quantity = 1) THEN 'Expired'
              WHEN (cr.quantity = 0) THEN 'Used'
              WHEN (('".pSQL($now)."' BETWEEN cr.date_from AND cr.date_to) AND cr.quantity = 1 AND cr.active = 1) THEN 'Active'
              ELSE 'Inactive'
          END AS cr_status,
		  cr.reduction_amount AS amount
		FROM " . _DB_PREFIX_ . "cart_rule cr
		JOIN " . _DB_PREFIX_ . "cart_rule_kerawen crk ON crk.id_cart_rule = cr.id_cart_rule AND crk.type = '" . _KERAWEN_CR_GIFT_CARD_ . "'
		LEFT JOIN " . _DB_PREFIX_ . "cart_rule_lang crl ON crl.id_cart_rule = cr.id_cart_rule AND crl.id_lang = " . (int) $id_lang . "
        LEFT JOIN " . _DB_PREFIX_ . "customer c ON c.id_customer = cr.id_customer
            
		LEFT JOIN " . _DB_PREFIX_ . "order_kerawen ok ON crk.id_order = ok.id_order
		LEFT JOIN " . _DB_PREFIX_ . "employee employee ON ok.id_employee = employee.id_employee
		LEFT JOIN " . _DB_PREFIX_ . "orders o ON crk.id_order = o.id_order
		    
		WHERE o.id_shop IN (" . pSQL($params->shop) . ") " . $where . "
		    
		ORDER BY cr.id_cart_rule DESC
		    
		";
	    
	    $res = $db->executeS($q);
	    if ($params->index) {
	        $res = indexArray($res, 'id');
	    }
	    
	    return $res;

	}
	
	
	public static function getLoyaly($db, $params)
	{

		$res = array();
		
		if (Module::isInstalled('loyalty') && Module::isEnabled('loyalty')) {
		
			$id_lang = Context::getContext()->language->id;
			$q = "

				SELECT
				  'pt' AS fid_type,
				  loyalty.id_order,
				  loyalty.id_customer,
				  CONCAT (customer.firstname, ' ', customer.lastname) AS customer, 
				  loyalty.date_add AS date_from,
				  '' AS date_to,
				  loyalty.points,
				  loyalty.points*" . Configuration::get('PS_LOYALTY_POINT_VALUE') . " AS amount
				FROM " . _DB_PREFIX_ . "loyalty loyalty
				LEFT JOIN " . _DB_PREFIX_ . "customer customer ON loyalty.id_customer = customer.id_customer
				LEFT JOIN " . _DB_PREFIX_ . "orders orders ON loyalty.id_order = orders.id_order
				WHERE loyalty.points > 0 
				AND loyalty.id_loyalty_state = 2
				AND CONCAT (customer.firstname, ' ', customer.lastname) != '- -'
				AND orders.id_shop IN (" . implode(',', array_map('intval', explode(',', $params->shop))) . ")
			
				UNION ALL
		
				SELECT 
				  'br' AS fid_type,
				  '' AS id_order,
				  customer.id_customer,
				  CONCAT (customer.firstname, ' ', customer.lastname) AS customer,
				  cart_rule.date_from,
				  cart_rule.date_to,
				  '' AS points,
				  cart_rule.reduction_amount
				FROM " . _DB_PREFIX_ . "cart_rule cart_rule
				INNER JOIN (
				  SELECT loyalty.id_cart_rule, loyalty.id_customer
				  FROM " . _DB_PREFIX_ . "loyalty loyalty
				  LEFT JOIN " . _DB_PREFIX_ . "orders orders ON loyalty.id_order = orders.id_order AND orders.id_shop IN (" . implode(',', array_map('intval', explode(',', $params->shop))) . ")
				  GROUP BY loyalty.id_cart_rule, loyalty.id_customer
				) AS loyalty ON cart_rule.id_cart_rule = loyalty.id_cart_rule
				LEFT JOIN " . _DB_PREFIX_ . "customer customer 
				ON loyalty.id_customer = customer.id_customer
				
				WHERE cart_rule.quantity > 0 AND cart_rule.active = 1 
				AND (NOW() BETWEEN cart_rule.date_from AND cart_rule.date_to)
			";

			$res = $db->executeS($q);
			if ($params->index) {
				$res = indexArray($res, 'id');
			}

		}

		return $res;

	}	
	

	public static function getDeffered($db, $params, $dir = '>')
	{
			$q = '
				SELECT
					  o.id_order AS id_order,
					  o.id_customer AS id_customer,
					  CONCAT(c.firstname, " ", c.lastname) AS customer,
					  o.id_shop AS id_shop,
					  ok.id_till AS id_cashdrawer,
					  ok.id_employee AS id_employee,
					  o.`date_add` AS `date`,
					  IFNULL(oo.total_order, 0) - IFNULL(s.total_slip, 0) - IFNULL(p.paid, 0) - IFNULL(pk.paid, 0) AS amount
				FROM '._DB_PREFIX_.'orders o
				LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON ok.id_order = o.id_order
				LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
				LEFT JOIN (SELECT
					o1.reference,
					-- COUNT(o1.id_order) AS nof_orders,
					SUM(IF(o1.current_state IN (6, 8), 0, 1)*o1.total_paid_tax_incl) AS total_order
					FROM '._DB_PREFIX_.'orders o1
					GROUP BY o1.reference) oo ON oo.reference = o.reference
				LEFT JOIN (SELECT
					o2.reference,
					-- COUNT(s2.id_order_slip) AS nof_slips,
					SUM(IF(o2.current_state IN (6, 8), 0, 1)*(s2.total_products_tax_incl + s2.total_shipping_tax_incl)) AS total_slip
					FROM '._DB_PREFIX_.'order_slip s2
					JOIN '._DB_PREFIX_.'orders o2 ON o2.id_order = s2.id_order
					GROUP BY o2.reference) s ON s.reference = o.reference
				LEFT JOIN (SELECT
					p3.order_reference,
					-- COUNT(p3.id_order_payment) AS nof_payments,
					SUM(p3.amount) AS paid
					FROM '._DB_PREFIX_.'order_payment p3
					GROUP BY p3.order_reference) p ON p.order_reference = o.reference
				LEFT JOIN (
					SELECT
					k4.reference,
					-- COUNT(k4.id_order_payment) AS nof_payments,
					SUM(p4.amount) AS paid
					FROM '._DB_PREFIX_.'order_payment_kerawen k4
					JOIN '._DB_PREFIX_.'order_payment p4 ON p4.id_order_payment = k4.id_order_payment
					GROUP BY k4.reference) pk ON pk.reference = o.reference
				WHERE o.id_shop IN ('.implode(',', array_map('intval', explode(',', $params->shop))).')
                    -- AND ok.tracked = 1
				GROUP BY o.id_order
				HAVING amount '.$dir.' 0.0
				ORDER BY o.date_add DESC';
							
			$res = $db->executeS($q);
			if ($params->index) {
				$res = indexArray($res, 'id_order');
			}
		return $res;
	}	


	public static function getRefundDeffered($db, $params, $dir = '<')
	{
		return self::getDeffered($db, $params, $dir);
	}	

	
	public static function setAftersalesSettings($db, $params) {
	    $list_id = array(0);
	    foreach($params->data as $key => $state) {	    
	        
	        if (((int) $state->id > -1) || ((int) $state->id == -1 && $state->name !== '')) {
    	        $state_item =  array(
    	            'thread_state_label' => pSQL($state->name),
    	            'thread_state_color' => pSQL($state->color),
    	            'thread_state_position' => (int) $key
    	        );
    	        if ((int) $state->id == -1) {
    	            $db->insert('customer_thread_state_kerawen', $state_item);
    	            $list_id[] = (int) $db->Insert_ID();
    	        } else {
    	            $db->update('customer_thread_state_kerawen', $state_item, 'id_thread_state = '. (int) $state->id);
    	            $list_id[] = (int) $state->id;
    	        }
	        }
	    }
	    
	    $db->delete('customer_thread_state_kerawen', 'id_thread_state NOT IN (' . implode(',', $list_id) . ')');
	    
	    return getKerawenThreadStats();
	}

	public static function getAftersalesSettings($db, $params) {
	    return array();
	}
	
	public static function getAftersales($db, $params)
	{
	    $bypage = self::bypage();
	    $page = (empty($params->page)) ? 1 : (int) $params->page;

	    $where = array();
	    if (isset($params->id_thread_state) && (int) $params->id_thread_state > -1) {
	        $where[] = 'IFNULL(ctk.id_thread_state, 0) = ' . $params->id_thread_state;  
	    }
	    if (isset($params->status) && $params->status != '' && $params->status != -1) {
	        $where[] = 'ct.status = "' . pSQL($params->status) . '"';
	    }
	    
	    $where = count($where) ? 'WHERE ' . implode(' AND ', $where) : '';
	    $limit = (int) $page > 0 ? " LIMIT " . ( ( (int) $page - 1) * (int) $bypage ) . ", " . (int)$bypage : '';
	    
	    $array_sort = array('ASC','DESC');
	    $array_order = array('id'=> 'ct.id_customer_thread', 'date_add' => 'ct.date_add', 'id_order' => 'ct.id_order', 'customer' => 'CONCAT(c.firstname, " ", c.lastname)', 'email' => 'c.email', 'as_state' => 'ctk.id_thread_state', 'as_status' => 'ct.status');
	    if (isset($array_order[$params->orderby]) && in_array($params->sortby, $array_sort)) {
	        $setOrder = ' ORDER BY ' . $array_order[$params->orderby] . ' ' . $params->sortby . ' ';
	    } else {
	        $setOrder = ' ORDER BY ct.id_customer_thread DESC ';
	    }
	    

        $q = '
            SELECT SQL_CALC_FOUND_ROWS
                ct.id_customer_thread AS id,
                ct.date_add AS date_add,
                ct.id_order AS id_order,
                CONCAT(c.firstname, " ", c.lastname) AS customer,
                IF(IFNULL(ck.fakemail, 0) > 0, "", c.email) AS email,
                ct.status AS as_status,
                ctsk.thread_state_label AS as_state,
                ctsk.thread_state_color AS color
        	FROM '._DB_PREFIX_.'customer_thread ct
            LEFT JOIN '._DB_PREFIX_.'customer_thread_kerawen ctk ON ct.id_customer_thread = ctk.id_customer_thread
        	LEFT JOIN '._DB_PREFIX_.'customer_thread_state_kerawen ctsk ON IFNULL(ctk.id_thread_state, 0) = ctsk.id_thread_state
            LEFT JOIN '._DB_PREFIX_.'orders o ON ct.id_order = o.id_order
            LEFT JOIN '._DB_PREFIX_.'customer c ON ct.id_customer = c.id_customer
            LEFT JOIN '._DB_PREFIX_.'customer_kerawen ck ON ck.id_customer = c.id_customer
            ' . $where . ' 
            ' . $setOrder . '
            ' . $limit
        ;	    

	    $res = $db->executeS($q);
	    $total = $db->getValue('SELECT FOUND_ROWS()');
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage
	    );
	    
	    return $data;
	    
	}
	
	public static function getAftersalesDetail($db, $params) {

	    $q = '
    	    SELECT
        	    cm.*,
        	    cm.id_customer_message AS id,
        	    IFNULL(cmk.id_thread_state, 0) AS id_thread_state,
        	    ctsk.thread_state_label AS state_label,
        	    ctsk.thread_state_color AS state_color,
                TRIM(CONCAT(c.firstname, " ", c.lastname)) AS customer,
                IF(IFNULL(ck.fakemail, 0) > 0, "", c.email) AS email,
                ct.id_customer,
                ct.status,
                ct.id_customer_thread,
                ct.id_order,
                ct.id_product

    	    FROM '._DB_PREFIX_.'customer_message cm
    	    LEFT JOIN '._DB_PREFIX_.'customer_message_kerawen cmk ON cm.id_customer_message = cmk.id_customer_message
    	    LEFT JOIN '._DB_PREFIX_.'customer_thread_state_kerawen ctsk ON IFNULL(cmk.id_thread_state, 0) = ctsk.id_thread_state
            LEFT JOIN '._DB_PREFIX_.'customer_thread ct ON ct.id_customer_thread = cm.id_customer_thread
            LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = ct.id_customer
            LEFT JOIN '._DB_PREFIX_.'customer_kerawen ck ON ck.id_customer = c.id_customer
            WHERE cm.id_customer_thread = ' . (int) $params->vars->id . '
    	    ORDER BY cm.`date_add` DESC
	   '; 
	    return $db->executeS($q);
	}
	

	public static function getOrderPickingFilters($params) {

	    $withOCS = self::withOCS();
	    
	    $shop_filter = ' AND TRUE ';
	    if (isset($params->shop)) {
	        $list_shop = array();
	        foreach(explode(',', $params->shop) as $id_shop) {
	            $list_shop[] = $withOCS ? '( (o.id_shop=' . (int) $id_shop . ' AND ocs.id_shop_pickupatstore IS NULL) OR ocs.id_shop_pickupatstore = ' . (int) $id_shop . ')' : 'o.id_shop=' . (int) $id_shop;
	        }
	        if (count($list_shop)) {
	            $shop_filter = ' AND (' . implode(' OR ', $list_shop) . ') ';
	        }
	    }

	    $till_filter = ' AND TRUE ';
	    if (isset($params->till)) {
	        $till_filter = ' AND ok.id_till IN ('.implode(',', array_map('intval', explode(',', $params->till))).') ';
	    }
	    
	    $mode_filter = ' AND TRUE ';
	    if (isset($params->mode)) {
	        $list_mode = array();
	        foreach(explode(',', $params->mode) as $mode) {
	            $mode_spec = '';
	            $mode_spec_ocs = '';
	            if ((int) $mode == 2) {
	                $mode_spec = ' OR ISNULL(ok.delivery_mode)';
	                if ($withOCS) {	                    
	                    $mode_spec_ocs = ' AND ocs.id_shop_pickupatstore IS NULL';
	                }
	            }
	            if ((int) $mode == 1) {
	                if ($withOCS) {
	                    $mode_spec_ocs = ' OR (ok.delivery_mode = 2 AND ocs.id_shop_pickupatstore IS NOT NULL)';
	                }
	            }
	            $list_mode[] = '((ok.delivery_mode = ' . (int) $mode . $mode_spec . ')' . $mode_spec_ocs . ')';

	        }
	        
	        if (count($list_mode)) {
	            $mode_filter = ' AND (' . implode(' OR ', $list_mode) . ') ';
	        }
	    }
	    

	    if (isset($params->vars) && isset($params->vars->app) && $params->vars->app == 'report') {
	        $stat_filter = ' TRUE ';
	        $selected_date = ' AND o.date_add BETWEEN "'.pSQL($params->from).'" AND "'.pSQL($params->to).'" ';
	        $timeslot = ' AND TRUE ';
	    } else {
	        $stat_filter = ' o.current_state != 6 ';
	        $selected_date = (boolean) Configuration::get('KERAWEN_DELIVERY_DATE_ORDER') ? ' AND ok.delivery_date BETWEEN "'. pSQL($params->from).'" AND "'.pSQL($params->to).'"' : ' AND o.date_add BETWEEN "'.pSQL($params->from).'" AND "'.pSQL($params->to).'"';
	        $timeslot = ($params->timeslot_start == '00:00:00' && $params->timeslot_start == '23:59:59') ? "" : " AND TIME(" . ((boolean) Configuration::get('KERAWEN_DELIVERY_DATE_ORDER') ? "ok.delivery_date": "o.date_add") .") BETWEEN '" . pSQL($params->timeslot_start) . "' AND '" . pSQL($params->timeslot_end) . "' ";
	    }

	    return  $stat_filter  . ' ' . $mode_filter . ' ' . $shop_filter . ' ' . $till_filter . ' ' . $selected_date . ' ' . $timeslot;

	}
	
	public static function joinOrderPickingWithOCS() {
	    return self::withOCS() ? ' LEFT JOIN '._DB_PREFIX_.'kerawen_cnc_pickupatstore_orders ocs ON ocs.id_order = o.id_order ' : '';
	}
	
	public static function getOrderPickingFields() {
	    if (self::withOCS()) {
	        $fields = '
                IFNULL(ocs.id_shop_pickupatstore, o.id_shop) AS cid_shop,
                IF (ocs.id_shop_pickupatstore IS NULL, IFNULL(ok.delivery_mode, 2), 1) AS del_mode,
            ';
	    } else {
	        $fields = '
                o.id_shop AS cid_shop,
                IFNULL(ok.delivery_mode, 2) AS del_mode,
            ';
	    }
	    return $fields;
	}
	

	public static function getOrderPicking($db, $params)
	{
	    
	    $id_lang = (int) Context::getContext()->language->id;
	    $bypage = self::bypage();
	    
	    $page = 1;
	    if (!empty($params->page)) {
	        $page = (int) $params->page;
	    }
	    
	    
	    $list_fields = array(
	        'id_row' => 'o.id_order',
	        'reference' => 'o.reference',
	        'order_date' => 'o.date_add',
	        'delivery_date' => 'ok.delivery_date',
	        'customer' => 'CONCAT(c.firstname, " ", c.lastname)',
	        'del_mode' => 'ok.delivery_mode',
	        'state' => 'osl.name'
	    );
	    
	    $orderby = 'id_row';
	    if (!empty($params->orderby)) {
	        if (array_key_exists($params->orderby, $list_fields)) {
	            $orderby = $list_fields[$params->orderby];
	        }
	    }
	    
	    $sortby = 'DESC';
	    if (!empty($params->sortby)) {
	        $sortby = $params->sortby;
	    }
	    
	    $join_carrier = '';
	    if ((int) $params->carrier > 0) {
	        $join_carrier = '
                INNER JOIN ' . _DB_PREFIX_.'order_carrier oca 
                    ON oca.id_order = o.id_order
                    AND oca.id_carrier IN (
                        SELECT id_carrier FROM '._DB_PREFIX_.'carrier WHERE id_reference = ' . (int) $params->carrier . '
                    )
            ';
	    }

	    $q = '
            SELECT SQL_CALC_FOUND_ROWS
                o.id_order AS id_row, 
                o.reference AS reference,
                o.current_state,
                o.date_add AS order_date,
                ok.delivery_date,
                ' . self::getOrderPickingFields() . '
                CONCAT(c.firstname, " ", c.lastname) AS customer,
                ocr.carrier,
                osl.name AS state,
                os.color,
                ok.is_paid
            FROM '._DB_PREFIX_.'orders o
            ' . self::joinOrderPickingWithOCS() . '
            LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON o.id_order = ok.id_order
            LEFT JOIN '._DB_PREFIX_.'order_state_lang osl ON o.current_state = osl.id_order_state AND osl.id_lang = '. (int) $id_lang.'
            LEFT JOIN '._DB_PREFIX_.'order_state os ON o.current_state = os.id_order_state
            LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = o.id_customer
            LEFT JOIN (
                SELECT 
                    oc.id_order,
                    GROUP_CONCAT(c.name) AS carrier
                FROM '._DB_PREFIX_.'order_carrier oc
                LEFT JOIN  '._DB_PREFIX_.'carrier c ON c.id_carrier = oc.id_carrier
                GROUP BY oc.id_order
            )  ocr ON ocr.id_order = o.id_order
            ' . $join_carrier . '
            WHERE ' . self::getOrderPickingFilters($params) . '
            ORDER BY ' . $orderby . ' ' . $sortby . ' 
        ';

	    if ((int)$page > 0) {
	        $q .= " LIMIT " . ( ((int)$page - 1) * (int) $bypage) . ", " . (int) $bypage;
	    }
	    
	    $res = $db->executeS($q);
	    $total = $db->getValue('SELECT FOUND_ROWS()');
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage,
	    );
	    
	    return $data;
	    
	}


	public static function getSalesByOrder($db, $params)
	{
	    $id_lang = (int) Context::getContext()->language->id;
	    $order_by = 'o.id_order DESC';
	    
	    $bypage = self::bypage();
	    
	    $page = 0;
	    if (!empty($params->page)) {
	        $page = (int) $params->page;
	    }
	    
	    
	    $q = '
            SELECT SQL_CALC_FOUND_ROWS
                o.id_order AS id_order,
                o.reference AS reference,
                o.current_state,
                o.date_add AS order_date,
                ok.delivery_date,
                CONCAT(cust.firstname, " ", cust.lastname) AS customer,
                ' . self::getOrderPickingFields() . '
                osl.name AS state,
                os.color,
                ok.is_paid,
                TRIM(CONCAT(IFNULL(ok.delivery_note, ""), " ", IFNULL(ok.invoice_note, ""))) AS public_note,
                IFNULL(ok.private_note, "") AS private_note
            FROM '._DB_PREFIX_.'orders o
            ' . self::joinOrderPickingWithOCS() . '
            LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON o.id_order = ok.id_order
            LEFT JOIN '._DB_PREFIX_.'order_state_lang osl ON o.current_state = osl.id_order_state AND osl.id_lang = '. (int) $id_lang.'
            LEFT JOIN '._DB_PREFIX_.'order_state os ON o.current_state = os.id_order_state
            LEFT JOIN '._DB_PREFIX_.'customer cust ON cust.id_customer = o.id_customer

            WHERE ' . self::getOrderPickingFilters($params) . '
            ORDER BY ' . $order_by . '
        ';

	    if ((int)$page > 0) {
	        $q .= " LIMIT " . ( ((int)$page - 1) * (int) $bypage) . ", " . (int) $bypage;
	    }
	    
	    $res = $db->executeS($q);
	    
	    $total = $db->getValue('SELECT FOUND_ROWS()');
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage
	    );
	    
	    return $data;
	}


	public static function getOrderPickingDetailByOrder($db, $params)
	{

	    $id_lang = (int) Context::getContext()->language->id;
	    $bypage = self::bypage();
	    
	    
	    $page = 0;
	    
	    $order_by = Configuration::get('KERAWEN_DELIVERY_DATE_ORDER') ? 'ok.delivery_date ASC' : 'o.date_add ASC';
	    
	    $q = '
            SELECT SQL_CALC_FOUND_ROWS
                o.id_order AS id_order,
                o.reference AS reference,
                o.current_state,
                o.date_add AS order_date,
                ok.delivery_date,
                IFNULL(CONCAT(ok.delivery_note, " ", ok.invoice_note), "") AS public_note,
                IFNULL(private_note, "") AS private_note,
                CONCAT(cust.firstname, " ", cust.lastname) AS customer,
                ' . self::getOrderPickingFields() . '
                osl.name AS state,
                os.color,
                ok.is_paid,
	            od.product_name AS product,
	            IFNULL(cd.note, "") AS note,
                od.product_quantity - od.product_quantity_refunded AS qty,
                IFNULL(odk.note, "") AS prodnote,
                IFNULL((SELECT message FROM '._DB_PREFIX_.'message m WHERE m.private = 0 AND m.id_customer > 0 AND m.id_order = o.id_order ORDER BY m.id_message DESC LIMIT 1), "") AS message

            FROM '._DB_PREFIX_.'orders o
            ' . self::joinOrderPickingWithOCS() . '
            LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON o.id_order = ok.id_order
            LEFT JOIN '._DB_PREFIX_.'order_state_lang osl ON o.current_state = osl.id_order_state AND osl.id_lang = '. (int) $id_lang.'
            LEFT JOIN '._DB_PREFIX_.'order_state os ON o.current_state = os.id_order_state
            LEFT JOIN '._DB_PREFIX_.'customer cust ON cust.id_customer = o.id_customer

	        LEFT JOIN '._DB_PREFIX_.'order_detail od ON o.id_order = od.id_order
	        LEFT JOIN '._DB_PREFIX_.'order_detail_kerawen odk ON odk.id_order_detail = od.id_order_detail

	        LEFT JOIN '._DB_PREFIX_.'customization c ON o.id_cart = c.id_cart AND od.product_id = c.id_product AND od.product_attribute_id = c.id_product_attribute
	        LEFT JOIN
	           (SELECT 
                    GROUP_CONCAT(`value`) AS note, 
                    id_customization 
                FROM '._DB_PREFIX_.'customized_data 
                GROUP BY id_customization
                ) AS cd ON c.id_customization = cd.id_customization

            WHERE ' . self::getOrderPickingFilters($params) . '
            ORDER BY ' . $order_by . '
        ';

	    if ((int)$page > 0) {
	        $q .= " LIMIT " . ( ((int)$page - 1) * (int)$bypage) . ", " . (int)$bypage;
	    }
	    
	    $res = $db->executeS($q);
	    
	    $total = $db->getValue('SELECT FOUND_ROWS()');
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage
	    );
	    
	    return $data;
	    
	}
	

	public static function getOrderPickingDetail($db, $params)
	{
	    
	    $bypage = self::bypage();
	    
	    $page = 1;
	    if (!empty($params->page)) {
	        $page = (int) $params->page;
	    }
	    
		if ( isset($params->orderby) && isset($params->sortby) ) {
		    if (!($params->orderby == '' || $params->sortby == '')) {
		        switch ($params->orderby) {
		            case 'location':
		                $order = 'ORDER BY location ' . $params->sortby;
		                break;
		            case 'qte':
		                $order = 'ORDER BY qte ' . $params->sortby;
		                break;
		            case 'product':
		                $order = 'ORDER BY product ' . $params->sortby;
		                break;
		            case 'order':
		                $order = 'ORDER BY `order` ' . $params->sortby;
		                break;
		            case 'note':
		                $order = 'ORDER BY note ' . $params->sortby;
		                break;
		        }
		    }
		}

		if (Tools::version_compare(_PS_VERSION_, '1.7.5', '>=')) {
		    $loc_field = 'IFNULL(sa.location, \'\')';
		    $loc_join = 'LEFT JOIN '._DB_PREFIX_.'stock_available sa ON od.product_id = sa.id_product AND IFNULL(od.product_attribute_id,0) = IFNULL(sa.id_product_attribute, 0) AND sa.id_shop = o.id_shop';
		} else {
		    $loc_field = '(CASE od.product_attribute_id WHEN \'0\' THEN pr.location ELSE pra.location END)';
		    $loc_join = 'LEFT JOIN '._DB_PREFIX_.'product_attribute pra ON IFNULL(pra.id_product_attribute, 0) = IFNULL(od.product_attribute_id, 0) AND pra.id_product = od.product_id';
		}

	    $q = '
	       SELECT SQL_CALC_FOUND_ROWS
	           SUM(od.product_quantity) - SUM(od.product_quantity_refunded) AS qte,
	           od.product_name AS product,
	           GROUP_CONCAT(od.id_order) AS `order`,
	           cd.note AS note,
			   ' . $loc_field . ' AS location
	       FROM '._DB_PREFIX_.'order_detail od
	       LEFT JOIN '._DB_PREFIX_.'order_detail_kerawen odk ON odk.id_order_detail = od.id_order_detail
	       LEFT JOIN '._DB_PREFIX_.'product pr ON pr.id_product = od.product_id
           LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = od.id_order
           ' . $loc_join . '
           ' . self::joinOrderPickingWithOCS() . '
	       LEFT JOIN '._DB_PREFIX_.'order_kerawen ok ON o.id_order = ok.id_order
	       LEFT JOIN '._DB_PREFIX_.'customization c ON o.id_cart = c.id_cart AND od.product_id = c.id_product AND od.product_attribute_id = c.id_product_attribute
	       LEFT JOIN
	           (SELECT 
                    GROUP_CONCAT(`value`) AS note, 
                    id_customization 
                FROM '._DB_PREFIX_.'customized_data 
                GROUP BY id_customization
                ) AS cd ON c.id_customization = cd.id_customization
	       	WHERE ' . self::getOrderPickingFilters($params) . '
	        GROUP BY od.product_id, od.product_attribute_id, cd.note
            '.(isset($order) ? $order : '').'
		';
	    
	    if ((int)$page > 0) {
	        $q .= " LIMIT " . ( ((int)$page - 1) * (int)$bypage) . ", " . (int)$bypage;
	    }
	    
	    $res = $db->executeS($q);
	    
	    $total = $db->getValue('SELECT FOUND_ROWS()');
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage
	    );
	    
	    return $data;
	    
	}
	
	public static function getOrderPickingSettings($db, $params) {	 	    
	    return json_decode(Configuration::get('KERAWEN_PICKING'));
	}
	
	public static function setPickingSettings($db, $params) {
	    
		if (isset($params->config)) {
		    Configuration::updateValue('KERAWEN_DELIVERY_DATE_ORDER', intval($params->config->delivery_date));
		    Configuration::updateValue('KERAWEN_ORDER_FILTER_TILL', intval($params->config->twf_till));
		    Configuration::updateValue('KERAWEN_ORDER_FILTER_WEB', intval($params->config->twf_web));
		    Configuration::updateValue('KERAWEN_ORDER_FILTER_CHECKOUT', intval($params->config->mf_is));
		    Configuration::updateValue('KERAWEN_ORDER_FILTER_TAKEAWAY', intval($params->config->mf_ta));
		    Configuration::updateValue('KERAWEN_ORDER_FILTER_DELIVERY', intval($params->config->mf_d));
		    Configuration::updateValue('KERAWEN_ORDER_FILTER_BOTH', intval($params->config->mf_tad));
	    }

	    if (isset($params->data)) {
	        Configuration::updateValue('KERAWEN_PICKING', $params->data);
	    }
	    return true;
	}


	public static function getDepositBySale($db, $params) {

	    $bypage = (int) self::bypage();
	    $id_lang = Context::getContext()->language->id;
	    $page = (!empty($params->page)) ? (int) $params->page : 1;
	    $and_cust = '';
	    
	    if (((int) $params->id_cust > 0)) {
	        $bypage = 0;
	        $page = 0;
	        $and_cust = ' AND c.id_customer = ' . (int) $params->id_cust. ' ';
	    }
	    
	    $q = '
            SELECT SQL_CALC_FOUND_ROWS
        	    o.reference,
        	    a.id_order,
        	    a.date_add AS `date`,
        	    od.product_id AS id,
        	    od.product_name AS prod_name,
                od.product_reference AS ref,
                if (a.amount >= 0, "stat_1", "stat_3") AS stat,
                a.amount AS total,
        	    c.firstname,
        	    c.lastname,
        	    c.email
    	    FROM '._DB_PREFIX_.'customer_kerawen_account a
    	    LEFT JOIN '._DB_PREFIX_.'order_detail od ON a.id_order_detail = od.id_order_detail
    	    LEFT JOIN '._DB_PREFIX_.'orders o ON od.id_order = o.id_order
    	    LEFT JOIN '._DB_PREFIX_.'customer c ON a.id_customer = c.id_customer
    	    WHERE a.type = "' . pSQL(_KERAWEN_TYPE_DEPOSIT_) . '"
    	       AND a.reason = "' . pSQL(_KERAWEN_CA_EARN_CREDIT_) . '"
    	       AND o.id_shop IN (' . dataListToIntList($params->shop) . ')
    	       AND (a.date_add BETWEEN "' . pSQL($params->from) . '" AND "' . pSQL($params->to) . '")
    	       ' . $and_cust . '
            ORDER BY a.date_add DESC';
	
    	if ((int)$page > 0) {
    	    $q .= ' LIMIT ' . ( ((int)$page - 1) * (int)$bypage) . ', ' . (int)$bypage . ' ';
    	}

    	//echo $q;
    	
    	$res = $db->executeS($q);
    	$total = $db->getValue('SELECT FOUND_ROWS()');
 
    	if (((int) $params->id_cust > 0)) {
    	    $bypage = $total;
    	}
    	
    	
    	$data = array(
    	    'data' => $res,
    	    'total' => $total,
    	    'bypage' => $bypage
    	);
    	
    	return $data;
	
	}

	public static function getDepositBySaler($db, $params) {
	    
	    $bypage = (int) self::bypage();
	    $id_lang = Context::getContext()->language->id;
	    $page = (!empty($params->page)) ? (int) $params->page : 1;
	    $and_cust = ((int) $params->id_cust > 0) ? ' AND pwk.id_customer = ' . (int) $params->id_cust. ' ' : '';
	    
	    $q = '
            SELECT SQL_CALC_FOUND_ROWS
	            pwk.id_customer AS id, 
                c.firstname,
                c.lastname,
            	c.email,
                IFNULL(cka.earn, 0) AS earn,
            	ck.deposit_amount AS balance
            FROM '._DB_PREFIX_.'product_wm_kerawen pwk
            LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = pwk.id_customer
            LEFT JOIN '._DB_PREFIX_.'customer_kerawen ck ON ck.id_customer = pwk.id_customer
    
            LEFT JOIN (
                SELECT 
                    scka.id_customer,
                    SUM(scka.amount) AS earn
                FROM '._DB_PREFIX_.'customer_kerawen_account scka
                WHERE scka.reason = "EARN" AND scka.type = "DEPOSIT" 
                    AND (scka.date_add BETWEEN "' . pSQL($params->from) . '" AND "' . pSQL($params->to) . '")
                GROUP BY scka.id_customer 
            ) cka ON cka.id_customer = pwk.id_customer

            WHERE pwk.id_customer > 0
            ' . $and_cust . '
            
            GROUP BY pwk.id_customer
            ORDER BY c.lastname DESC
        ';
	    
	    if ((int)$page > 0) {
	        $q .= ' LIMIT ' . ( ((int)$page - 1) * (int)$bypage) . ', ' . (int)$bypage . ' ';
	    }

	    $res = $db->executeS($q);
	    $total = $db->getValue('SELECT FOUND_ROWS()');
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage
	    );
	    
	    return $data;
	    
	}
	
	
	public static function getDepositBySalerDetail($db, $params) {
	    
	    
	    $bypage = (int) self::bypage();
	    $id_lang = Context::getContext()->language->id;
	    $page = (!empty($params->page)) ? (int) $params->page : 1;
	    $and_cust = '';
	    
	    if (((int) $params->id_cust > 0)) {
	        $bypage = 0;
	        $page = 0;
	        $and_cust = ' AND c.id_customer = ' . (int) $params->id_cust. ' ';
	    }
	    
	    $q = '
            SELECT SQL_CALC_FOUND_ROWS
        	    o.reference,
        	    a.id_order,
        	    a.date_add AS `date`,
        	    od.product_id AS id,
        	    od.product_name AS prod_name,
                od.product_reference AS ref,
                a.reason,
                a.amount AS total,
        	    c.firstname,
        	    c.lastname,
        	    c.email
    	    FROM '._DB_PREFIX_.'customer_kerawen_account a
    	    LEFT JOIN '._DB_PREFIX_.'order_detail od ON a.id_order_detail = od.id_order_detail
    	    LEFT JOIN '._DB_PREFIX_.'orders o ON od.id_order = o.id_order
    	    LEFT JOIN '._DB_PREFIX_.'customer c ON a.id_customer = c.id_customer
    	    WHERE a.type = "' . pSQL(_KERAWEN_TYPE_DEPOSIT_) . '"
    	       AND (a.date_add BETWEEN "' . pSQL($params->from) . '" AND "' . pSQL($params->to) . '")
    	       ' . $and_cust . '
            ORDER BY a.date_add DESC';
	    
	    if ((int)$page > 0) {
	        $q .= ' LIMIT ' . ( ((int)$page - 1) * (int)$bypage) . ', ' . (int)$bypage . ' ';
	    }
	    
	    //echo $q;
	    
	    $res = $db->executeS($q);
	    $total = $db->getValue('SELECT FOUND_ROWS()');
	    
	    if (((int) $params->id_cust > 0)) {
	        $bypage = $total;
	    }
	    
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage
	    );
	    
	    return $data;
	    
	}
	
	
	
/*
	public static function getDepositBySaler($db, $params) {
	    
	    $bypage = (int) self::bypage();
	    $id_lang = Context::getContext()->language->id;
	    $page = (!empty($params->page)) ? (int) $params->page : 1;
	    $and_cust = ((int) $params->id_cust > 0) ? ' AND cka.id_customer = ' . (int) $params->id_cust. ' ' : '';
	    
	    $q = '
            SELECT SQL_CALC_FOUND_ROWS
            	cka.id_customer AS id, 
                c.firstname,
                c.lastname,
            	c.email,
                SUM(cka.amount) AS earn,
            	ck.deposit_amount AS credit
            FROM '._DB_PREFIX_.'customer_kerawen_account cka
            LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = cka.id_customer
            LEFT JOIN '._DB_PREFIX_.'customer_kerawen ck ON ck.id_customer = cka.id_customer
            WHERE cka.reason = "EARN" AND cka.type = "DEPOSIT" 
            AND (cka.date_add BETWEEN "' . pSQL($params->from) . '" AND "' . pSQL($params->to) . '")
            ' . $and_cust . '
            GROUP BY cka.id_customer 
            ORDER BY c.lastname DESC
        ';
	    
	    if ((int)$page > 0) {
	        $q .= ' LIMIT ' . ( ((int)$page - 1) * (int)$bypage) . ', ' . (int)$bypage . ' ';
	    }
	    
	    
	    $res = $db->executeS($q);
	    $total = $db->getValue('SELECT FOUND_ROWS()');
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage
	    );
	    
	    return $data;
	    
	}
*/	

	public static function getDepositByProduct($db, $params) {
	    
	    $bypage = (int) self::bypage();
	    $id_lang = Context::getContext()->language->id;
	    $page = (!empty($params->page)) ? (int) $params->page : 1;
	    $and_cust = '';

	    if (((int) $params->id_cust > 0)) {
	        $bypage = 0;
	        $page = 0;
	        $and_cust = ' AND pwk.id_customer = ' . (int) $params->id_cust. ' ';
	    }
	    
	    
	    $q = '
            SELECT SQL_CALC_FOUND_ROWS
            	ps.id_product AS id,
                s.name AS id_shop,
                pl.name AS product,
                ppac.decli AS declinaison,       
            	IFNULL(pa.ean13, p.ean13) AS ean13,
            	IFNULL(pa.reference, p.reference) AS ref,

                ROUND(IF(pas.wholesale_price IS NULL OR pas.wholesale_price = 0, ps.wholesale_price, pas.wholesale_price), 2) AS amount,
                sa.quantity AS stock,
                ROUND(IF(pas.wholesale_price IS NULL OR pas.wholesale_price = 0, ps.wholesale_price, pas.wholesale_price), 2) * (IF(sa.quantity > 0, sa.quantity, 0)) AS total, 
                c.firstname,
                c.lastname,
                c.email

            FROM '._DB_PREFIX_.'product_shop ps
            INNER JOIN '._DB_PREFIX_.'product p ON p.id_product = ps.id_product AND ps.id_shop IN (' . dataListToIntList($params->shop) . ')
            INNER JOIN '._DB_PREFIX_.'product_wm_kerawen pwk ON p.id_product = pwk.id_product AND pwk.product_type = "DEPOSIT"
            LEFT JOIN '._DB_PREFIX_.'shop s ON s.id_shop = ps.id_shop
            LEFT JOIN '._DB_PREFIX_.'customer c ON c.id_customer = pwk.id_customer      
            
  
            LEFT JOIN '._DB_PREFIX_.'product_lang pl ON pl.id_product = ps.id_product AND pl.id_lang = ' . (int) $id_lang . ' AND pl.id_shop = ps.id_shop
            LEFT JOIN '._DB_PREFIX_.'product_attribute_shop pas ON ps.id_product = pas.id_product  AND pas.id_shop = ps.id_shop
            LEFT JOIN '._DB_PREFIX_.'product_attribute pa ON pas.id_product_attribute = pa.id_product_attribute
                
            LEFT JOIN (
                SELECT
            		GROUP_CONCAT(al.name ORDER BY a.id_attribute_group) AS decli,
                    pac.id_product_attribute
            	FROM '._DB_PREFIX_.'product_attribute_combination pac
            	LEFT JOIN '._DB_PREFIX_.'attribute a ON pac.id_attribute = a.id_attribute
            	LEFT JOIN '._DB_PREFIX_.'attribute_lang al ON al.id_attribute = a.id_attribute AND al.id_lang = ' . (int) $id_lang . '
            	GROUP BY pac.id_product_attribute
            ) AS ppac ON ppac.id_product_attribute = pas.id_product_attribute
            	    
            LEFT JOIN '._DB_PREFIX_.'stock_available sa 
                ON sa.id_product = ps.id_product 
                AND IFNULL(pas.id_product_attribute, 0) = sa.id_product_attribute AND sa.id_shop = ps.id_shop

        WHERE TRUE ' . $and_cust . '
        ORDER BY ps.id_product DESC
         ';
	    
	    if ((int)$page > 0) {
	        $q .= ' LIMIT ' . ( ((int)$page - 1) * (int)$bypage) . ', ' . (int)$bypage . ' ';
	    }
	    
        $res = $db->executeS($q);
        $total = $db->getValue('SELECT FOUND_ROWS()');

        if (((int) $params->id_cust > 0)) {
            $bypage = $total;
        }
        
        $data = array(
            'data' => $res,
            'total' => $total,
            'bypage' => $bypage
        );
        
        return $data;

	}

	public static function getCustomer($db, $params)
	{
	
		$bypage = self::bypage();
	
		$page = 1;
		if (!empty($params->page)) {
			$page = (int) $params->page;
		}
	
		$orderby = 'id_customer';
		if (!empty($params->orderby)) {
			$orderby = $params->orderby;
		}
	
		$sortby = 'DESC';
		if (!empty($params->sortby)) {
			$sortby = $params->sortby;
		}
	
	
		$id_shop = -1;
		if (!empty($params->shop)) {
			$id_shop = $params->shop;
		}
			
		$id_lang = (int) Context::getContext()->language->id;
	
		$q = "
			SELECT SQL_CALC_FOUND_ROWS
			a.`id_customer`,
			gl.`name` AS gender,
			a.`firstname`,
			a.`lastname`,		
			IF (ck.fakemail = 1 ,'',a.email ) as email, 
            l.iso_code AS lang,
            g.name AS `group`,
			shop.name AS shop,
			`newsletter`,
			a.date_add,
			a.birthday,
            ck.account,
            ck.limit,
            ck.amount,

			CASE 	 
				WHEN ck.mobile != '' THEN ck.mobile
				ELSE addr.phone_mobile	
			END AS mobile,
			CASE 
				WHEN ck.phone != '' THEN ck.phone
				ELSE addr.phone
			END AS phone,
			CASE
				WHEN ck.postalcode != '' THEN ck.postalcode 
				ELSE addr.postcode
			END AS postcode,

            addr.city,
			LEFT(
                CASE
				    WHEN ck.postalcode != '' THEN ck.postalcode 
				    ELSE addr.postcode
			    END
            , 2) AS dept,
            
            cl.name AS country,

			/*
			(
				SELECT c.date_add
				FROM " . _DB_PREFIX_ . "guest g
				LEFT JOIN " . _DB_PREFIX_ . "connections c ON c.id_guest = g.id_guest
				WHERE g.id_customer = a.id_customer ORDER BY c.date_add DESC LIMIT 1
			) AS connect,
			*/
		
			(
				SELECT o.date_upd
				FROM " . _DB_PREFIX_ . "orders o
				WHERE o.valid = 1 AND o.id_customer = a.id_customer
				ORDER BY o.date_upd DESC
				LIMIT 1
			)	AS last_order_date,
	
			/*
			(
				SELECT DATEDIFF(NOW(), o.date_upd)
				FROM " . _DB_PREFIX_ . "orders o
				WHERE o.valid = 1 AND o.id_customer = a.id_customer
				ORDER BY o.date_upd DESC
				LIMIT 1
			)	AS last_order_day,
			*/
	
			(
				SELECT COUNT(id_order)
				FROM " . _DB_PREFIX_ . "orders o
				WHERE o.id_customer = a.id_customer AND o.valid = 1
			) AS orders,					

			(
				SELECT SUM(total_paid_tax_incl / conversion_rate)
				FROM " . _DB_PREFIX_ . "orders o
				WHERE o.id_customer = a.id_customer AND o.valid = 1
			) AS amount						
						
			FROM `" . _DB_PREFIX_ . "customer` a
			LEFT JOIN `" . _DB_PREFIX_ . "gender_lang` gl ON (a.id_gender = gl.id_gender AND gl.id_lang = " . (int) $id_lang . ")
			LEFT JOIN `" . _DB_PREFIX_ . "shop` shop ON a.id_shop = shop.id_shop 
			LEFT JOIN `" . _DB_PREFIX_ . "customer_kerawen` ck ON a.id_customer = ck.id_customer 
            LEFT JOIN `" . _DB_PREFIX_ . "lang` l ON a.id_lang = l.id_lang
            LEFT JOIN `" . _DB_PREFIX_ . "group_lang` g ON (a.id_default_group = g.id_group AND g.id_lang = " . (int) $id_lang . ")

			/*
			JOIN (
 				SELECT
				o.id_customer,
				MAX(o.date_upd) AS date_upd,
				DATEDIFF(NOW(), MAX(o.date_upd)) AS last_order_day
				FROM " . _DB_PREFIX_ . "orders o
				WHERE o.valid = 1
 				GROUP BY o.id_customer
			) c ON (a.id_customer  = c.id_customer)
			*/

            LEFT JOIN `" . _DB_PREFIX_ . "address` addr ON addr.id_address = (
                SELECT 
                    ad.id_address
                FROM `" . _DB_PREFIX_ . "address` ad
                WHERE ad.active = 1 AND ad.deleted = 0 AND ad.id_customer = a.id_customer
                ORDER BY ad.date_upd DESC
                LIMIT 1
            )
            LEFT JOIN `" . _DB_PREFIX_ . "country_lang` cl ON cl.id_country = addr.id_country AND cl.id_lang = " . (int) $id_lang . "

			WHERE a.`deleted` = 0
						
		";
		
		if ($id_shop != -1) {
		    $q .= " AND a.id_shop IN (" . implode(',', array_map('intval', explode(',', $id_shop))) . ") ";
		}

		$q .= self::setOrder($orderby, $sortby);
		

		if ((int)$page > 0) {
			$q .= " LIMIT " . ( ((int)$page - 1) * (int)$bypage) . ", " . (int)$bypage;
		}
	
		$res = $db->executeS($q);
		if ($params->index) {
			$res = indexArray($res, 'id');
		}
	
		$total = $db->getValue('SELECT FOUND_ROWS()');
	
		$data = array(
				'data' => $res,
				'total' => $total,
				'bypage' => $bypage,
		);
	 
		return $data;
	
	}
		
	public static function getBestCustomer($db, $params)
	{
		$bypage = (int) self::bypage();
	
		$page = 1;
		if (!empty($params->page)) {
			$page = (int) $params->page;
		}
	
		$orderby = 'amount';
		$sortby = 'DESC';

		$id_shop = -1;
		if (!empty($params->shop)) {
			$id_shop = $params->shop;
		}		
		
		$id_lang = Context::getContext()->language->id;
	
		$q = "
			SELECT SQL_CALC_FOUND_ROWS
			a.`id_customer`,
			gl.`name` AS gender,
			a.`firstname`,
			a.`lastname`,
			IF (ck.fakemail = 1 ,'', a.email ) as email, 
            l.iso_code AS lang,
            g.name AS `group`,
			CASE 	 
				WHEN ck.mobile != '' THEN ck.mobile
				ELSE addr.phone_mobile	
			END AS mobile,
			CASE 
				WHEN ck.phone != '' THEN ck.phone
				ELSE addr.phone
			END AS phone,
			CASE
				WHEN ck.postalcode != '' THEN ck.postalcode 
				ELSE addr.postcode
			END AS postcode,

            addr.city,
			LEFT(
                CASE
				    WHEN ck.postalcode != '' THEN ck.postalcode 
				    ELSE addr.postcode
			    END
            , 2) AS dept,
            
            cl.name AS country,

			shop.name AS shop,
			(
				SELECT COUNT(id_order)
				FROM " . _DB_PREFIX_ . "orders o
				WHERE o.date_add BETWEEN '" . pSQL($params->from) . "' AND '" . pSQL($params->to) . "'
				AND o.id_customer = a.id_customer AND o.valid = 1
			) AS orders,

			(
				SELECT SUM(total_paid_real / conversion_rate)
				FROM " . _DB_PREFIX_ . "orders o
				WHERE o.date_add BETWEEN '" . pSQL($params->from) . "' AND '" . pSQL($params->to) . "'
				AND o.id_customer = a.id_customer AND o.valid = 1
			) AS amount
							
			FROM `" . _DB_PREFIX_ . "customer` a
			LEFT JOIN `" . _DB_PREFIX_ . "gender_lang` gl ON (a.id_gender = gl.id_gender AND gl.id_lang = " . (int) $id_lang . ")
			LEFT JOIN `" . _DB_PREFIX_ . "shop` shop ON a.id_shop = shop.id_shop 
			LEFT JOIN `" . _DB_PREFIX_ . "customer_kerawen` ck ON a.id_customer = ck.id_customer 
            LEFT JOIN `" . _DB_PREFIX_ . "lang` l ON a.id_lang = l.id_lang
            LEFT JOIN `" . _DB_PREFIX_ . "group_lang` g ON (a.id_default_group = g.id_group AND g.id_lang = " . (int) $id_lang . ")
            
            LEFT JOIN `" . _DB_PREFIX_ . "address` addr ON addr.id_address = (
                SELECT 
                    ad.id_address
                FROM `" . _DB_PREFIX_ . "address` ad
                WHERE ad.active = 1 AND ad.deleted = 0 AND ad.id_customer = a.id_customer
                ORDER BY ad.date_upd DESC
                LIMIT 1
            )
            LEFT JOIN `" . _DB_PREFIX_ . "country_lang` cl ON cl.id_country = addr.id_country AND cl.id_lang = " . (int) $id_lang . "

			WHERE a.`deleted` = 0 "
		;
		
		if ($id_shop != -1) {
		    $q .= " AND a.id_shop IN( " . implode(',', array_map('intval', explode(',', $id_shop))) . ") ";
		}

		
		if (!($orderby == '' || $sortby == '')) {
		    $q .= self::setOrder($orderby, $sortby);
		}
		

		if ((int)$page > 0) {
			$q .= " LIMIT " . ( ((int)$page - 1) * (int)$bypage) . ", " . (int)$bypage;
		}

		$res = $db->executeS($q);
		if ($params->index) {
			$res = indexArray($res, 'id');
		}
	
		$total = $db->getValue('SELECT FOUND_ROWS()');
	
		$data = array(
			'data' => $res,
			'total' => $total,
			'bypage' => $bypage
		);
	
		return $data;
	
	}


	public static function getCustAccount($db, $params)
	{
	    
	    $bypage = self::bypage();
	    
	    $page = 1;
	    if (!empty($params->page)) {
	        $page = (int) $params->page;
	    }
	    
	    $orderby = 'id_customer';
	    if (!empty($params->orderby)) {
	        $orderby = $params->orderby;
	    }
	    
	    $sortby = 'DESC';
	    if (!empty($params->sortby)) {
	        $sortby = $params->sortby;
	    }
	    
	    
	    $id_shop = -1;
	    if (!empty($params->shop)) {
	        $id_shop = $params->shop;
	    }
	    
	    $id_lang = (int) Context::getContext()->language->id;
	    
	    $q = "
			SELECT SQL_CALC_FOUND_ROWS
    			a.id_customer,
    			gl.`name` AS gender,
    			a.firstname,
    			a.lastname,
    			IF (ck.fakemail = 1 ,'',a.email ) as email,
                l.iso_code AS lang,
                g.name AS `group`,
                ck.amount,
                ck.limit
			FROM " . _DB_PREFIX_ . "customer a
			LEFT JOIN " . _DB_PREFIX_ . "gender_lang gl ON (a.id_gender = gl.id_gender AND gl.id_lang = " . (int) $id_lang . ")
			LEFT JOIN " . _DB_PREFIX_ . "shop shop ON a.id_shop = shop.id_shop
			LEFT JOIN " . _DB_PREFIX_ . "customer_kerawen ck ON a.id_customer = ck.id_customer
            LEFT JOIN " . _DB_PREFIX_ . "lang l ON a.id_lang = l.id_lang
            LEFT JOIN " . _DB_PREFIX_ . "group_lang g ON (a.id_default_group = g.id_group AND g.id_lang = " . (int) $id_lang . ")

			WHERE a.deleted = 0
				    
		";
	    
	    if ($id_shop != -1) {
	        $q .= " AND a.id_shop IN (" .  implode(',', array_map('intval', explode(',', $id_shop))) . ") ";
	    }
	   
	    $q .= self::setOrder($orderby, $sortby);
	    
	    if ((int)$page > 0) {
	        $q .= " LIMIT " . ( ((int)$page - 1) * (int)$bypage) . ", " . (int)$bypage;
	    }
	    
	    
	    //echo $q;
	    
	    $res = $db->executeS($q);
	    if ($params->index) {
	        $res = indexArray($res, 'id');
	    }
	    
	    $total = $db->getValue('SELECT FOUND_ROWS()');
	    
	    $data = array(
	        'data' => $res,
	        'total' => $total,
	        'bypage' => $bypage
	    );
	    
	    return $data;
	    
	}
	
	
	public static function getShops($db, $params) {
	    return $db->executeS('
		    SELECT
                s.name,
                s.id_shop
		    FROM '._DB_PREFIX_.'shop s
	        ORDER BY s.id_shop ASC
	    ');
	}
	
	public static function checkLoyalyModule() {
	    $module = Module::getInstanceByName('loyalty');
	    return Tools::version_compare($module->version, '2.1.0', '>=');
	}
	
	public static function getCartRuleByShop($db, $params) {
	    
	    $with_loyalty = SELF::checkLoyalyModule();
	    $shops = SELF::getShops($db, $params);
	    
	    $data = array();
	    foreach ($shops as $shop) {
	        $data[$shop['id_shop']] = array();
	        $data[$shop['id_shop']]['giftcard'] = array();
	        $data[$shop['id_shop']]['credit'] = array();
	        if ($with_loyalty) { $data[$shop['id_shop']]['loyalty'] = array(); }
	        $data[$shop['id_shop']]['cr_total'] = array();
	        
	        foreach ($shops as $subshop) {
	            $data[$shop['id_shop']]['giftcard'][$subshop['id_shop']] = 0;
	            $data[$shop['id_shop']]['credit'][$subshop['id_shop']] = 0;
	            if ($with_loyalty) { $data[$shop['id_shop']]['loyalty'][$subshop['id_shop']] = 0; }
	            $data[$shop['id_shop']]['cr_total'][$subshop['id_shop']] = 0;
	        }
	        
	    }
	    
	    $result = self::getOrderCartRule($db, $params);
	    foreach ($result as $row) {
	        if ((int) $row['shop_buy'] != (int) $row['shop_used'] && (int) $row['shop_buy'] > 0) {
	            if (isset($data[$row['shop_used']]) && isset($data[$row['shop_used']][$row['cr_type']])  && isset($data[$row['shop_used']][$row['cr_type']][$row['shop_buy']]) ) {
	                $data[$row['shop_used']][$row['cr_type']][$row['shop_buy']] += (float) $row['amount'];
	                $data[$row['shop_used']]['cr_total'][$row['shop_buy']] += (float) $row['amount'];
	            }
	        }
	    }
	    return $data;
	}
	
	
	public static function getOrderCartRule($db, $params) {
	    
	    $result = array_merge(
	        self::getOrderMode($db, $params, 'Carte cadeau'),
	        self::getOrderMode($db, $params, 'Avoir'),
	        self::getOrderLoyaltyByidLoyalty($db, $params)
	        );
	    return $result;
	}
	
	
	public static function getOrderLoyaltyByidLoyalty($db, $params) {
	    
	    if (!self::checkLoyalyModule()) {
	        return array();
	    }
	    
	    $q = '
    	    SELECT
	            "loyalty" AS `cr_type`,
	            o.date_add AS used_date,
		        l.id_shop AS shop_buy,
		        IF(l.id_order > 0, l.id_order, "") AS order_buy,
	            o.id_shop AS shop_used,
	            o.id_order AS order_used,
	            l.points AS points,
	        
		        IF(lg.total_point > 0, ocr.value * (l.points/lg.total_point), 0) AS amount,
	            0 AS remain,
	        
		        ocr.id_cart_rule AS id,
                "" AS id_parent,
	            cr.code
	        
    	    FROM '._DB_PREFIX_.'order_cart_rule ocr
    	    LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = ocr.id_order
            LEFT JOIN '._DB_PREFIX_.'cart_rule cr ON cr.id_cart_rule = ocr.id_cart_rule
                
            INNER JOIN '._DB_PREFIX_.'loyalty l ON l.id_cart_rule = ocr.id_cart_rule
            INNER JOIN
    	    (
    	        SELECT
    	           id_cart_rule,
    	           SUM(points) AS total_point
    	        FROM '._DB_PREFIX_.'loyalty
    	        GROUP BY id_cart_rule
    	    ) lg ON lg.id_cart_rule = ocr.id_cart_rule
    	            
            WHERE o.date_add BETWEEN "' . pSQL($params->from) . '" AND "' . pSQL($params->to) . '"
                
            ORDER BY o.id_order DESC, l.id_loyalty DESC
        ';
	    
	    return  $db->executeS($q);
	}
	
	
	public static function getOrderMode($db, $params, $mode) {
	    
	    $mode_mapping = array(
	        'Carte cadeau' => 'giftcard',
	        'Avoir' => 'credit',
	        'loyalty' => 'loyalty'
	    );
	    
	    $mode_label = isset($mode_mapping[$mode]) ? $mode_mapping[$mode] : 'undefined:' . $mode;
	    
	    
	    $q = '
            SELECT
               "' . pSQL($mode_label) . '" AS `cr_type`,
               pso.date_add AS used_date,
               IFNULL(psop.id_shop, pso.id_shop) AS shop_buy,
               crk.id_order AS order_buy,
                   
               o.id_shop AS shop_used,
               ko.ps_order AS order_used,
                   
               "" AS points,
               IFNULL(SUM(p.amount), 0) AS amount,
               IFNULL(cr.reduction_amount, 0) - IFNULL(SUM(p.amount), 0) AS remain,
                   
               cr.id_cart_rule AS id,
               crk.id_parent_cart_rule AS id_parent,
               cr.code AS `code`
                   
            FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'operation o
                
            LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'sale s ON s.id_operation = o.id_operation
            LEFT JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'order ko ON ko.id_sale = s.id_sale
                
            INNER JOIN '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p ON o.id_operation = p.id_operation AND p.`mode` = "' . pSQL($mode) . '"
            INNER JOIN '._DB_PREFIX_.'order_payment op ON op.id_order_payment = p.id_order_payment
                
            LEFT JOIN '._DB_PREFIX_.'cart_rule cr ON cr.id_cart_rule = op.transaction_id
            LEFT JOIN '._DB_PREFIX_.'cart_rule_kerawen crk ON crk.id_cart_rule = cr.id_cart_rule
                
            LEFT JOIN '._DB_PREFIX_.'orders pso ON pso.id_order = crk.id_order
                
            LEFT JOIN '._DB_PREFIX_.'cart_rule_kerawen crkp ON crkp.id_cart_rule = crk.id_parent_cart_rule
            LEFT JOIN '._DB_PREFIX_.'orders psop ON psop.id_order = crkp.id_order
                
            WHERE crk.id_order IS NOT NULL AND cr.date_add BETWEEN "' . pSQL($params->from) . '" AND "' . pSQL($params->to) . '"
                
            GROUP BY cr.date_add, pso.id_shop, crk.id_order, op.transaction_id, o.id_shop, ko.ps_order
                
            ORDER BY crk.id_order DESC
        ';
	    
	    return  $db->executeS($q);
	}
	
	
	public static function getOrderLoyalty($db, $params) {
	    
	    $shops = SELF::getShops($db, $params);
	    
	    $module = Module::getInstanceByName('loyalty');
	    
	    $fields_points = array();
	    $join = array();
	    foreach ($shops as $shop) {
	        $fields_points[] = '
                    IFNULL(s' . (int) $shop['id_shop'] . '.total_point, 0) AS points_shop' . (int) $shop['id_shop'];
	        
	        //Module compatibility
	        $where = array();
	        $where[] = ' o.id_order = ' . (int) $shop['id_shop'] . ' ';
	        if (Tools::version_compare($module->version, '2.1.0', '>=')) {
	            $insertData['id_shop'] = $context->shop->id;
	            $where[] = ' l.id_shop = ' . (int) $shop['id_shop'] . ' ';
	        }
	        
	        $join[] = '
            LEFT JOIN
            (
                SELECT
            	   l.id_cart_rule,
            	   SUM(l.points) AS total_point
            	FROM '._DB_PREFIX_.'loyalty l
            	LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = l.id_order
            	WHERE ' . implode('OR', $where) . '
            	GROUP BY id_cart_rule
            ) s' . $shop['id_shop'] . ' ON s' . $shop['id_shop'] . '.id_cart_rule = ocr.id_cart_rule
            ';
	    }
	    
	    
	    $q = '
	        
    	    SELECT
        	    ocr.id_order,
        	    ocr.value AS used_vat_incl,
        	    ocr.value_tax_excl AS used_vat_excl,
	        
        	    cr.reduction_amount,
        	    cr.reduction_tax,
	        
        	    ocr.id_cart_rule,
        	    o.id_shop,
	        
        	    IFNULL(s0.total_point, 0) AS points,
                ' . implode(',', $fields_points) . '
                    
    	    FROM '._DB_PREFIX_.'order_cart_rule ocr
    	    LEFT JOIN '._DB_PREFIX_.'orders o ON o.id_order = ocr.id_order
            LEFT JOIN '._DB_PREFIX_.'cart_rule cr ON cr.id_cart_rule = ocr.id_cart_rule
                
    	    INNER JOIN
    	    (
    	        SELECT
    	           id_cart_rule,
    	           SUM(points) AS total_point
    	        FROM '._DB_PREFIX_.'loyalty
    	        GROUP BY id_cart_rule
    	    ) s0 ON s0.id_cart_rule = ocr.id_cart_rule
    	            
            ' . implode(" ", $join) . '
            WHERE o.date_add BETWEEN "' . pSQL($params->from) . '" AND "' . pSQL($params->to) . '"
            ORDER BY ocr.id_order DESC
	    ';
	    
	    return  $db->executeS($q);
	}
	
	
	public static function getPaymentMethods($log = null) {
		
		$db = Db::getInstance();
		
		$methods = array();
		$excludeArray = array();
		
		//Active payment methods
		$cfg = json_decode(Configuration::get('KERAWEN_PAYMENTS'), true);
		foreach($cfg as $key=>$method) {
			if ($method['payment'] || $method['refund']) {
				$methods[(int)$method['id']] = $method['label'];
				$excludeArray[] = pSQL($method['label']);
			}
		}
		
		
		$exclude = 'NOT IN ("' . implode('", "', $excludeArray) . '", "Retour produits", "Carte cadeau")';
		
		//Others methods... (inactive now or web)
		if ($log == 'klog') {
			$res = $db->executeS('
				SELECT 
					p.mode AS payment_method 
				FROM '._DB_PREFIX_._KERAWEN_525_PREFIX_.'payment p
				WHERE p.mode ' . $exclude . '
				GROUP BY p.mode
				ORDER BY p.mode
			');
		} else {
			$res = $db->executeS('
				SELECT
					payment_method
				FROM ' . _DB_PREFIX_ . 'order_payment
				WHERE payment_method ' . $exclude . '
				GROUP BY payment_method
			');
		}
		
		
		foreach($res as $item) {
			$methods[] = $item['payment_method'];
		}
		
		
		return $methods;
	}


	public static function getOrders($db, $params)
	{
		
		$bypage = self::bypage();
		$paymentMethods = self::getPaymentMethods();
		
		$page = 1;
		if (!empty($params->page)) {
			$page = (int) $params->page;
		}
		
		$id_shop = -1;
		if (isset($params->shop)) {
			$id_shop = $params->shop;
		}
		
		$id_till = -1;
		if (isset($params->till)) {
			$id_till = $params->till;
		}
		
		$id_employee = -1;
		if (isset($params->employee)) {
			$id_employee = $params->employee;
		}
		
		$sum = "";
		foreach($paymentMethods AS $key=>$val) {
		    $sum .= " (SELECT SUM(op" . $key . ".amount) FROM " . _DB_PREFIX_ . "order_payment op" . $key . " WHERE op" . $key . ".order_reference = o.reference AND op" . $key . ".payment_method = '" . pSQL($val) . "') AS `" . pSQL($val) . "`, " . "\n";
		}
		
		$dateFormat = "'%d-%m-%Y'";
		
		$q = "
			SELECT SQL_CALC_FOUND_ROWS
				o.id_order AS `id_order`,
				o.reference AS `ref`,
				IF(o.invoice_number = 0, '', LPAD(o.invoice_number, 8, 'FA000000')) AS inv_num,
				DATE_FORMAT(o.date_add, " . $dateFormat . ") AS `date`,
				IF ((cdk.id_cash_drawer = 0 OR ISNULL(cdk.id_cash_drawer)), 'Web' , cdk.name) AS till,
				CONCAT(c.firstname, ' ', c.lastname) AS customer,
				o.total_paid AS `total`,
				" . $sum . "
				CASE
					WHEN NOT (ISNULL(SUM(op_99.amount)) OR ISNULL(SUM(op_98.amount))) THEN SUM(op_99.amount) + SUM(op_98.amount)
					WHEN NOT (ISNULL(SUM(op_99.amount))) THEN SUM(op_99.amount)
					WHEN NOT (ISNULL(SUM(op_98.amount))) THEN SUM(op_98.amount)
					ELSE NULL
				END AS `Retour produit`
			FROM " . _DB_PREFIX_ . "orders o
			LEFT JOIN " . _DB_PREFIX_ . "customer c ON o.id_customer = c.id_customer
			LEFT JOIN " . _DB_PREFIX_ . "order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN " . _DB_PREFIX_ . "cash_drawer_kerawen cdk ON ok.id_till = cdk.id_cash_drawer
			LEFT JOIN " . _DB_PREFIX_ . "order_payment_kerawen opk ON opk.reference = o.reference

			LEFT JOIN " . _DB_PREFIX_ . "order_payment op_99 ON op_99.id_order_payment = opk.id_order_payment AND op_99.payment_method = 'Retour produits'
			LEFT JOIN " . _DB_PREFIX_ . "order_payment op_98 ON op_98.order_reference = o.reference AND op_98.payment_method = 'Retour produits'
 
			WHERE o.date_add BETWEEN '" . pSQL($params->from) . "' AND '" . pSQL($params->to) . "'
		";
		
		if ($id_shop !== -1) {
		    $q .= " AND o.id_shop IN (" . implode(',', array_map('intval', explode(',', $id_shop))) . ") ";  
		}
		if ($id_till != -1) {
		    
		    //case NULL
		    $caseNull = "";
		    if (!(strpos("," . $id_till . ",", ",0,") === false)) {
		        $caseNull =  " OR ok.id_till IS NULL";
		    }
		    $q .= " AND (ok.id_till IN (" . implode(',', array_map('intval', explode(',', $id_till))) . ")" . $caseNull . ") ";
		}
		if ($id_employee != -1) {
		    $q .= " AND ok.id_employee = " . (int) $id_employee . " ";
		}
		
		$q .= "
			GROUP BY o.reference, op_99.payment_method, op_98.payment_method
			ORDER BY o.id_order DESC ";
		
		if ((int)$page > 0) {
		    $q .= " LIMIT " . ( ((int)$page - 1) * (int)$bypage) . ", " . (int)$bypage;
		}

		$res = $db->executeS($q);
		if ($params->index) {
			$res = indexArray($res, 'id_order');
		}
		
		$total = $db->getValue('SELECT FOUND_ROWS()');
		
		$data = array(
			'data' => $res,
			'total' => $total,
			'bypage' => $bypage,
		);
		
		return $data;
	}
	
	
	public static function getOperations($db, $params)
	{
		
		$bypage = self::bypage();
		$paymentMethods = self::getPaymentMethods();
		
		$page = 1;
		if (!empty($params->page)) {
			$page = (int) $params->page;
		}
		
		$id_shop = -1;
		if (isset($params->shop)) {
			$id_shop = $params->shop;
		}
		
		$id_till = -1;
		if (isset($params->till)) {
			$id_till = $params->till;
		}
		
		$id_employee = -1;
		if (isset($params->employee)) {
			$id_employee = $params->employee;
		}
		

		$q = "";
		$sum = "";
		$sum2 = "";

		foreach($paymentMethods AS $key=>$val) {
		    $sum .= "( SELECT SUM(op" . $key . ".amount) FROM " . _DB_PREFIX_ . "order_payment op" . $key . " WHERE op" . $key . ".order_reference = o.reference AND op" . $key . ".payment_method = '" . pSQL($val) . "' ) AS `" . pSQL($val) . "`," . "\n";
			$sum2 .= " (SELECT SUM(op" . $key . ".amount) FROM " . _DB_PREFIX_ . "kerawen_525_payment op" . $key . " WHERE ok.id_operation = op" . $key . ".id_operation AND op" . $key . ".mode = '" . pSQL($val) . "')  AS `" . pSQL($val) . "`, " . "\n";
		}
		

		$deltaDate = self::getDeltaDate($params->from, $params->to);
		$dateFormat = "'%d-%m-%Y'";
		
		$q .= "
			SELECT SQL_CALC_FOUND_ROWS
				1 AS weight,
				CONCAT('_', cok.id_cashdrawer_op) AS id_op, 
				DATE_FORMAT(cok.date, " . $dateFormat . ") AS `date`,
				GROUP_CONCAT(o.id_order) AS id_order, 
				GROUP_CONCAT(o.reference) AS ref, 
				IF ((cdk.id_cash_drawer = 0 OR ISNULL(cdk.id_cash_drawer)), 'Web' , cdk.name) AS till,
				GROUP_CONCAT(DISTINCT CONCAT(c.firstname, ' ', c.lastname)) AS customer,
				o.total_paid AS `total`,
				" . $sum . "
				CASE
					WHEN NOT (ISNULL(SUM(op_99.amount)) OR ISNULL(SUM(op_98.amount))) THEN SUM(op_98.amount)
					/*WHEN NOT (ISNULL(SUM(op_99.amount)) OR ISNULL(SUM(op_98.amount))) THEN SUM(op_99.amount) + SUM(op_98.amount)*/
					WHEN NOT (ISNULL(SUM(op_99.amount))) THEN -SUM(op_99.amount)
					WHEN NOT (ISNULL(SUM(op_98.amount))) THEN -SUM(op_98.amount)
					ELSE NULL
				END AS `Retour produit`

			FROM " . _DB_PREFIX_ . "cashdrawer_op_kerawen cok

			LEFT JOIN " . _DB_PREFIX_ . "cashdrawer_sale_kerawen csk ON cok.id_cashdrawer_op = csk.id_cashdrawer_op
			LEFT JOIN " . _DB_PREFIX_ . "orders o ON o.id_order = csk.id_order
			LEFT JOIN " . _DB_PREFIX_ . "order_kerawen ok ON ok.id_order = o.id_order
			LEFT JOIN " . _DB_PREFIX_ . "customer c ON o.id_customer = c.id_customer 
			LEFT JOIN " . _DB_PREFIX_ . "cash_drawer_kerawen cdk ON ok.id_till = cdk.id_cash_drawer
			LEFT JOIN " . _DB_PREFIX_ . "order_payment_kerawen opk ON opk.reference = o.reference

			LEFT JOIN " . _DB_PREFIX_ . "order_payment op_99 ON op_99.id_order_payment = opk.id_order_payment AND op_99.payment_method = 'Retour produits'
			LEFT JOIN " . _DB_PREFIX_ . "order_payment op_98 ON op_98.order_reference = o.reference AND op_98.payment_method = 'Retour produits'

			WHERE cok.oper = 'SALE' AND o.id_order IS NOT NULL	
			AND cok.date BETWEEN '" . pSQL($deltaDate["from1"]) . "' AND '" . pSQL($deltaDate["to1"]) . "'
		";

		if ($id_shop !== -1) {
		    $q .= " AND o.id_shop IN (" . implode(',', array_map('intval', explode(',', $id_shop))) . ") ";
		}
		if ($id_till != -1) {
			
			//case NULL
			$caseNull = "";
			if (!(strpos("," . $id_till . ",", ",0,") === false)) {
				$caseNull =  " OR ok.id_till IS NULL";
			}
			$q .= " AND (ok.id_till IN (" . implode(',', array_map('intval', explode(',', $id_till))) . ")" . $caseNull . ") ";
		}
		if ($id_employee != -1) {
			$q .= " AND ok.id_employee = " . (int) $id_employee . " ";
		}
		
		
		$q .= " GROUP BY cok.id_cashdrawer_op ";

		$q .= " UNION ALL ";
		
		$q .= " 
		SELECT
			2 AS weight,
			ok.id_operation AS id_op,
			DATE_FORMAT(ok.date, " . $dateFormat . ") AS `date`,
			GROUP_CONCAT(o.ps_order) AS id_order,
			GROUP_CONCAT(os.reference) AS ref,
			GROUP_CONCAT(DISTINCT IF (ok.id_till = 0, 'Web' ,  ok.till_name)) AS till,
			GROUP_CONCAT(DISTINCT CONCAT(c.firstname, ' ', c.lastname)) AS customer,
			os.total_paid AS total,
			" . $sum2 . "
			IF(o.ps_slip = 0, NULL, -o.total_ti) AS 'Retour produits'
		FROM " . _DB_PREFIX_._KERAWEN_525_PREFIX_ . "operation ok
		LEFT JOIN " . _DB_PREFIX_._KERAWEN_525_PREFIX_ . "sale ks ON ok.id_operation = ks.id_operation
		LEFT JOIN " . _DB_PREFIX_._KERAWEN_525_PREFIX_ . "order o ON o.id_sale = ks.id_sale
		LEFT JOIN " . _DB_PREFIX_ . "orders os ON os.id_order = o.ps_order
		LEFT JOIN " . _DB_PREFIX_ . "customer c ON os.id_customer = c.id_customer 
		WHERE ok.type = 'SALE'
		AND ok.date BETWEEN '" . pSQL($deltaDate["from2"]) . "' AND '" . pSQL($deltaDate["to2"]) . "'
		";

		if ($id_shop !== -1) {
		    $q .= " AND ok.id_shop IN (" . implode(',', array_map('intval', explode(',', $id_shop))) . ") ";
		}
		if ($id_till != -1) {
		    $q .= " AND (ok.id_till IN (" . implode(',', array_map('intval', explode(',', $id_till))) . ")) ";
		}
		if ($id_employee != -1) {
			$q .= " AND ok.id_operator = " . (int) $id_employee . " ";
		}

		$q .= " GROUP BY ok.id_operation 
				ORDER BY weight DESC, id_op DESC";
		

		if ((int)$page > 0) {
			$q .= " LIMIT " . ( ((int)$page - 1) * (int)$bypage) . ", " . (int)$bypage;
		}

		
		//echo '<pre>';
		//echo $q;
		
		$res = $db->executeS($q);
		if ($params->index) {
			$res = indexArray($res, 'id_op');
		}
		
		$total = $db->getValue('SELECT FOUND_ROWS()');
		
		$data = array(
			'data' => $res,
			'total' => $total,
			'bypage' => $bypage
		);
		
		return $data;
	}
	
	

	public static function getCertifiedDate($offset = 0) {
		return Db::getInstance()->getValue('
			SELECT DATE_FORMAT(MIN(date)  + INTERVAL ' . (int)$offset . ' SECOND, "%Y-%m-%d %H:%i:%s")
			FROM '._DB_PREFIX_.'kerawen_version
			WHERE STRCMP(version, "2.1") >= 0 AND res = 1');
	}


	public static function getDeltaDate($from, $to) {
		
		$refdate = self::getCertifiedDate();
		$delta = array();
		
		if ($refdate <= $from) {
			$delta['from1'] = 0;
			$delta['to1'] = 0;
			$delta['from2'] = $from;
			$delta['to2'] = $to;
		} elseif ($refdate > $to) {
			$delta['from1'] = $from;
			$delta['to1'] = $to;
			$delta['from2'] = 0;
			$delta['to2'] = 0;
		} else {
			$delta['from1'] = $from;
			$delta['to1'] = self::getCertifiedDate(-1);
			$delta['from2'] = $refdate;
			$delta['to2'] = $to;
		}
		return $delta;	
	}

	
	
	public static function statshour($db, $op_filter) {
		return array();
	}


	public static function statsweek($db, $op_filter) {
		return array();
	}
	
	public static function statsmonth($db, $op_filter) {
		return array();
	}

	public static function statsshops() {
	    return array();
	}

	public static function statsshopsWeb() {
	    return array();
	}

	public static function statsTotal() {
	    return array();
	}
	
	public static function cats() {
		return sharedCats();
	}
	
};


function getLog($types, $from, $to,
	$id_till, $id_empl, $id_shop,
	$init_from, $init_to, $id_group,
	$vars = array())
{
	$res = array();
	$db = Db::getInstance();
	
	$more_filter = '
		'.getLogFilter('co.id_cashdrawer', $id_till).'
		'.getLogFilter('co.id_employee', $id_empl).'
		'.getLogFilter('co.id_shop', $id_shop);

	$op_filter = '
		co.date BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'"
		'.$more_filter;

	$op_init_filter = '
		co.date BETWEEN "'.pSQL($init_from).'" AND "'.pSQL($init_to).'"
		'.$more_filter;

	$op_flow_filter = '
		co.date BETWEEN "'.pSQL($from).'" AND "'.pSQL($to).'"
		AND co.oper = "SALE"
		'.$more_filter;
	
	$op_close_filter = '
		co.date BETWEEN "'.pSQL($init_from).'" AND "'.pSQL($init_to).'"
		AND co.oper != "SALE"
		'.$more_filter;
	
	$res['ops'] = $db->executeS('
		SELECT
			CONCAT("_", co.id_cashdrawer_op) AS id_op,
			co.id_cashdrawer AS id_till,
			IFNULL(co.id_employee, 0) AS id_empl,
			co.id_shop AS id_shop,
			co.date AS date,
			co.oper AS oper
		FROM '._DB_PREFIX_.'cashdrawer_op_kerawen co
		WHERE '.$op_init_filter);

	foreach($types as $type) {
		if ($type == 'checks') {
			$res[$type] = LogQuery::$type($db, $op_close_filter, $vars);
		}
		else if ($type == 'flows') {
			$res[$type] = array_merge(
				LogQuery::$type($db, $op_flow_filter, $vars),
				LogQuery::$type($db, $op_close_filter, $vars));
		}
		else {
			$res[$type] = LogQuery::$type($db, $op_filter, $vars);
		}
	}

	return $res;
}

function getLogMvt($params) {
	$db = Db::getInstance();
	$res = array();
	foreach($params->require as $type)
		$res[$type] = LogQuery::$type($db, $params);
	return $res;
}

